define(['strftime', 'splunk.util'], function() {

    /*
     * monkey patches to the i18n library
     * see SPL-42469
     */

    // maintain a hash of locales where custom string replacements are needed to get correct translation
    var CUSTOM_LOCALE_FORMATS = {
        'ja_JP': [
            // match only the last 'd' of a block of 'd's
            [/d(?! d)/, 'd\u65e5'],
            // match only 'MM', not 'MMM'
            [/(?:^|[^M])(MM)(?:[^M]|$)/, 'MM\u6708'],
            [/YYYY/, 'YYYY\u5e74']
        ],
        'ko_KR': [
            [/d(?! d)/, 'd\uc77c'],
            [/(?:^|[^M])(MM)(?:[^M]|$)/, 'MM\uc6d4'],
            [/YYYY/, 'YYYY\ub144']
        ],
        'zh_CN': [
            [/d(?! d)/, 'd\u65e5'],
            [/(?:^|[^M])(MM)(?:[^M]|$)/, 'MM\u6708'],
            [/YYYY/, 'YYYY\u5e74']
        ],
        'zh_TW': [
            [/d(?! d)/, 'd\u65e5'],
            [/(?:^|[^M])(MM)(?:[^M]|$)/, 'MM\u6708'],
            [/YYYY/, 'YYYY\u5e74']
        ]
    };

    // maintain a list of replacements needed when a locale specifies that day comes before month
    var DAY_FIRST_FORMATS = [
        ['MMM d', 'd MMM']
    ];

    var oldFormatDate = window.format_date;

    window.format_date = function(date, format) {
        var i, replacements,
            locale = window.locale_name();

        // only modify format if format is not a preset
        if(!(format in { full: true, long: true, medium: true, short: true })) {
            if(format && window.locale_uses_day_before_month()) {
                replacements = DAY_FIRST_FORMATS;
                for(i = 0; i < replacements.length; i++) {
                    format = format.replace(replacements[i][0], replacements[i][1]);
                }
            }
            if(format && locale in CUSTOM_LOCALE_FORMATS) {
                replacements = CUSTOM_LOCALE_FORMATS[locale];
                for(i = 0; i < replacements.length; i++) {
                    // SPL-75443, match the regex to the format string, use the most specific match
                    // (if there is a capture group, use it) and replace the last occurrence
                    var replacement = replacements[i],
                        regex = replacement[0],
                        replace = replacement[1],
                        matches = format.match(regex);

                    if(matches && matches.length > 0) {
                        var find = matches[matches.length - 1],
                            lastIndex = format.lastIndexOf(find);

                        format = format.substr(0, lastIndex) + replace + format.substr(lastIndex + find.length);
                    }
                }
            }
        }
        return oldFormatDate(date, format);
    };
    
    return ({
        _: window.gettext,
        ungettext: window.ungettext,
        format_decimal: window.format_decimal,
        format_number: window.format_number,
        format_percent: window.format_percent,
        format_scientific: window.format_scientific,
        format_date: window.format_date,
        format_datetime: window.format_datetime,
        format_time: window.format_time,
        format_datetime_microseconds: window.format_datetime_microseconds,
        format_time_microseconds: window.format_time_microseconds,
        locale_uses_day_before_month: window.locale_uses_day_before_month,
        format_datetime_range: window.format_datetime_range,
        locale_name: window.locale_name,
        DateTime: window.DateTime,
        moment_install: window.moment_install,
        jQuery_ui_datepicker_install: window.jQuery_ui_datepicker_install
    });
});
