
// THIS FILE IS PURPOSEFULLY EMPTY FOR R.JS COMPILER
// IT IS A COMPILER TARGET FILE, AS WE CANNOT CREATE NEW FILES DYNAMICALLY FOR 
// THE COMPILER;
define("splunkjs/compiled/views", function(){});

define('views/Base',[
    'jquery',
    'backbone',
    'underscore',
    'mixins/viewlogging',
    'util/console'
],
function(
    $,
    Backbone,
    _,
    viewloggingmixin,
    console
){
    var BaseView = Backbone.View.extend({
        awake: true,
        touch: false,
        active: false,

        /**
         * A collection of Backbone.View utilities and conventions to assist with template compilation and cacheing, noop nested views and
         * className naming conventions. The general rule of thumb is to not override native Backbone.View methods but to complement the core
         * API with simple and useful patterns.
         *
         * @param {Object} options {
         *      @param {String} moduleId (Optional) An id (commonly the requirejs module.id) that is formatted and appended as a className to
         *                                          the root view el. Added as an instance member if passed in via the constructor.
         *
         *      @param {String} template (Optional) An _.template that is passed through the compileTemplate method. Added as an instance member
         *                                          if passed including an additional compiledTemplate instance member (see compileTemplate for
         *                                          specification).
         *  }
         */
        constructor: function(options) {
            this.options = options || {};
            this.children = {};
            
            if (this.options.moduleId) {
                this.moduleId = this.options.moduleId;
            }
            if (this.options.template) {
                this.template = this.options.template;
            }
            if (this.template) {
                this.compiledTemplate = this.compileTemplate(this.template);
            }
            Backbone.View.apply(this, arguments);
        },
        initialize: function() {
            var render = this.render;
            this.render = _.bind(function() {
                var start = (new Date()).getTime(), 
                    end;
                if (this.awake) {
                    render.apply(this);
                } else {
                    this.touch = true;
                }
                end = (new Date()).getTime();
                if(this.options.instrument !== false) {
                    this.$el.attr('data-render-time', (end - start)/1000);
                }
                return this;
            }, this);
            
            this.$el.attr('data-cid', this.cid);
            if (this.moduleId) {
                var className = this.cssNamespace();
                if (!this.options.dontAddModuleIdAsClass) {
                    this.$el.addClass(className);
                }
                this.$el.attr('data-view', this.moduleId);
            }
        },
        
        /**
         * Calls the method named by methodName on each child of the view.
         * Any extra arguments passed to invoke will be forwarded on to the method invocation.
         */
        invokeOnChildren: function(method) {
            if (!_.isString(method)) {
                throw new Error('Your method must be a string');
            }
            var argsForMethod = Array.prototype.slice.call(arguments, 1);
            
            _.each(this.children, function(child) {
                if (_.isArray(child)) {
                    _.each(child, function(view) {
                        if (_.isFunction(view[method])) {
                            view[method].apply(view, argsForMethod);
                        }
                    });
                } else {
                    if (_.isFunction(child[method])) {
                        child[method].apply(child, argsForMethod);
                    }
                }
            });
        },
        
        /**
         * Iterates over this children of this view, yielding each in turn to an iterator function.
         * The iterator is bound to the context object, if one is passed.
         * Each invocation of iterator is called with one argument: (child).
         */
        eachChild: function(iterator, context) {            
            if (!_.isFunction(iterator)) {
                throw new Error('Your iterator must be a function');
            }
            
            _.each(this.children, function(child){
                if (_.isArray(child)) {
                    _.each(child, function(view){
                        iterator.call(context, view);
                    });
                } else {
                    iterator.call(context, child);
                }
            });
        },
        
        /**
         * Replaces render noop proxy to original render routine. Additionally traverses
         * a special children attribute that consists of an object collection of views and
         * calls wake if that method exists.
         * 
         * @param {Object} options {
         *      @param {Boolean} syncRender (Optional) Call the standard (sync) render method over async (debouncedRender) variant.
         * }
         */
        wake: function(options) {
            options || (options = {});
            this.awake = true;
            if (this.touch) {
                this.touch = false;
                if (options.syncRender) {
                    this.render();
                } else {
                    this.debouncedRender();
                }
            }
            this.invokeOnChildren('wake', options);
            return this;
        },
        /**
         * Stops any subsequent calls to render and proxies them to a noop routine. Additionally traverses
         * a special children attribute that consists of an object collection of views and calls sleep if
         * that method exists.
         */
        sleep: function() {
            this.awake = false;
            this.invokeOnChildren('sleep');
            return this;
        },

        /**
         * Postpone the execution of render until after the input has stopped arriving.
         * Useful for assigning to model/collection change event listeners that should
         * only happen after the input has stopped arriving.
         */
        debouncedRender: function() {
            if (!this._debouncedRender) {
                this._debouncedRender = _.debounce(this.render, 0);
            }
            this._debouncedRender.apply(this, arguments);
        },
        /**
         * Similar to debouncedRender above, but for the reflow method
         */
        debouncedReflow: function() {
            if (!this._debouncedReflow) {
                this._debouncedReflow = _.debounce(this.reflow, 0);
            }
            this._debouncedReflow.apply(this, arguments);
        },
        /**
         * Compiles and memoizes an _.template string into a function that can be evaulated
         * for rendering.
         *
         * @param {String} templateStr An _.template string for compilation into a function.
         * @type {Function}
         * @return A function that can be evaluated for rendering, pass in a data object that
         * that has properties corresponding to the template's free variables.
         */
        compileTemplate: _.memoize(function(templateStr) {
            return _.template(templateStr);
        }),
        /**
         * Tailored for the requirejs module.id format converting into a safe and legal css 
         * class attribute. For example the following module id '/views/shared/SkidRow' 
         * would be converted to 'views-shared-skdrow'.
         */
        cssNamespace: function() {
            return (this.moduleId || '').toLowerCase().replace(/\//g, '-').replace(/\_/g, '').replace(/^views-/, '').replace(/-master$/, '');
        },
        /**
         * Generates a unique namespace that can be humanly cross-referenced to the view. 
         */
        uniqueNS: function() {
            return (this.moduleId || 'unidentified').toLowerCase().replace(/\//g, '-') + '-' + this.cid;
        },
        
        /**
         * Subscribe listeners.
         * implement as necessary
         */
        startListening: function() {
        },

        stopListening: function() {
            if(arguments.length === 0) {
                this.modelsOff(this.model);
                this.collectionsOff(this.collection);
            }
            Backbone.View.prototype.stopListening.apply(this, arguments);
        },
        
        remove: function() {
            this.removeChildren();
            if (this._isAddedToDocument) {
                this.onRemovedFromDocument();
                this._isAddedToDocument = false;
            }
            this.stopListening();
            this.$el.remove();
            return this;
        },
        
        debouncedRemove: function(options) {
            options || (options = {});
            var defaults = {
                detach: false
            };
            _.defaults(options, defaults);
            if (options.detach) {
                this.$el.detach();
            }
            if (!this._debouncedRemove) {
                this._debouncedRemove = _.debounce(this.remove, 0);
            }
            this._debouncedRemove.apply(this, arguments);
            return this;
        },
        
        removeChildren: function() {
            this.invokeOnChildren('remove');
        },
        
        ensureDeactivated: function(options) {
            if (this.active) {
                this.deactivate(options);
            }
        },
        
        activate: function(options) {
            options || (options = {});
            // the options for startListening and trigger are always shallow
            var clonedOptions = _.extend({startListening: true, trigger: true}, options);
            delete options.startListening;
            delete options.trigger;
            if (!this.active && clonedOptions.startListening) {
                this.startListening();
            }
            if (this.active) {
                console.debug("Calling activate again on view:" + this.moduleId, this.cid);
            } else {
                this.active = true;
                if (clonedOptions.trigger) {
                    this.trigger("activated");
                }                
            }
            //children get activated after the parent
            if (options.deep) {
                this.invokeOnChildren('activate', options);
            }
            return this;
        },
        
        deactivate: function(options) {
            options || (options = {});
            // the options for stopListening and trigger are always shallow
            var clonedOptions = _.extend({stopListening: true, trigger: true}, options);
            delete options.stopListening;
            delete options.trigger;
            //children get deactivated before the parent
            if (options.deep) {                
                this.invokeOnChildren('deactivate', options);
            }
            if (this.active && clonedOptions.stopListening) {
                this.stopListening();
            }
            if (!this.active) {
                console.debug("Calling deactivate again on view:" + this.moduleId, this.cid);
            } else {
                this.active = false;
                if (clonedOptions.trigger) {
                    this.trigger("deactivated");
                }                
            }
            return this;
        },
        
        /**
         *  Helper function for deepDestroy, recursively calls off() on its model(s)
         */
        modelsOff: function (model) {
            if (model instanceof Backbone.Model) {
                _.isFunction(model.associatedOff) && model.associatedOff(null, null, this);
                model.off(null, null, this);
            } else if (_.isArray(model) || $.isPlainObject(model)) { // SPL-90156
                _(model).each(function(mod) {
                    this.modelsOff(mod);
                }, this);
            } else if (model instanceof Backbone.Collection) { // TODO: revisit this logic after Dash release
                console.warn("Warning: trying to call modelsOff() on a Backbone.Collection object, please move it to this.collection instead.");
                this.collectionsOff(model);
            } else if (model != null) {
                console.warn('Warning: trying to call modelsOff() on non-Backbone.Model object! Maybe move it to this.options?');
            }
            return this;
        },
        /**
         *  Helper function for deepDestroy, recursively calls off() on its collections(s)
         */
         collectionsOff: function(collection) {
            if (collection instanceof Backbone.Collection) {
                collection.off(null, null, this);
            } else if (_.isArray(collection) || $.isPlainObject(collection)) {
                _(collection).each(function(coll) {
                    this.collectionsOff(coll);
                }, this);
            } else if (collection instanceof Backbone.Model) { // TODO: revisit this logic after Dash release
                console.warn("Warning: trying to call collectionsOff() on a Backbone.Model object, pleaes move it to this.model instead.");
                this.modelsOff(collection);
            } else if (collection != null) {
                console.warn('Warning: trying to call collectionsOff() on non-Backbone.Collection object! Maybe move it to this.options?');
            }
            return this;
        },

        /**
         * Helper function for removing the view from the DOM while a render occurs. If you call this on subviews in
         * a render function, jQuery will remove all of the subview's event listeners
         */
        detach: function() {
            if (this._isAddedToDocument) {
                this.onRemovedFromDocument();
                this._isAddedToDocument = false;
            }
            this.$el.detach();
        },

        // add instance methods for all commonly used jQuery attachment methods
        //replaceContentsOf is the analog of $.html($container)
        replaceContentsOf: function($container) {
            if(_.isString($container)) {
                throw new Error('replaceContentsOf does not support selectors or HTML strings');
            }
            
            // ensure the given $container parameter is a jQuery object
            $container = $($container);
            $container.empty();
            return this.attachToDocument($container, 'appendTo'); 
        },
        appendTo: function($container) { return this.attachToDocument($container, 'appendTo'); },
        prependTo: function($container) { return this.attachToDocument($container, 'prependTo'); },
        replaceAll: function($container) { return this.attachToDocument($container, 'replaceAll'); },
        insertAfter: function($container) { return this.attachToDocument($container, 'insertAfter'); },
        insertBefore: function($container) { return this.attachToDocument($container, 'insertBefore'); },

        /**
         * Attach the view to the given DOM element using the given method.
         * If afterward the view is attached to the document element, the onAddedToDocument method will be called.
         * If the view was already attached to the document element, and the given DOM element is different than
         * the view's current parent element, the onRemovedFromDocument method will be called first.
         *
         * Not meant to be called directly, only as a helper for the more specific attachment methods above.
         *
         * @param $container (DOM element or jQuery object)
         * @param attachmentMethod (a valid jQuery DOM attachment method name)
         */
        attachToDocument: function($container, attachmentMethod) {
            if(_.isString($container)) {
                throw new Error(attachmentMethod + ' does not support selectors or HTML strings');
            }

            // ensure the given $container parameter is a jQuery object
            $container = $($container);
            
            // if no container, abort
            if ($container.length === 0) {
                return this;
            }

            // get the raw DOM container
            var container = $container[0];

            // call onRemovedFromDocument if needed
            var oldParent = this.el.parentNode;
            if (oldParent && (oldParent !== container)) {
                if (this._isAddedToDocument) {
                    this.onRemovedFromDocument();
                    this._isAddedToDocument = false;
                }
            }

            // do the attachment
            this.$el[attachmentMethod]($container);

            // call onAddedToDocument if needed
            if (oldParent !== container) {
                if ($.contains(document.documentElement, this.el)) {
                    this._isAddedToDocument = true;
                    this.onAddedToDocument();
                }
            }

            return this;
        },

        /**
         * Called when this view is added to the document. Recursively calls
         * onAddedToDocument on all child views. Override this method with custom code
         * that should be run when this view is added to the document. Make sure to call
         * the base implementation when overriding.
         * 
         * This method is automatically called and should not be called directly.
         */
        onAddedToDocument: function() {
            this.eachChild(function(child) {
                if (_.isFunction(child.onAddedToDocument) && this.isAncestorOf(child) && !child._isAddedToDocument) {
                    child._isAddedToDocument = true;
                    child.onAddedToDocument();
                }
            }, this);
        },

        /**
         * Called when this view is removed from the document. Recursively calls
         * onRemovedFromDocument on all child views. Override this method with custom
         * code that should be run when this view is removed from the document. Make
         * sure to call the base implementation when overriding.
         * 
         * This method is automatically called and should not be called directly.
         */
        onRemovedFromDocument: function() {            
            this.eachChild(function(child) {
                if (_.isFunction(child.onRemovedFromDocument) && this.isAncestorOf(child) && child._isAddedToDocument) {
                    child.onRemovedFromDocument();
                    child._isAddedToDocument = false;
                }
            }, this);
        },

        /**
         * Returns a boolean indicating whether the view is currently attached to the document element
         */
        isAddedToDocument: function() {
            return this._isAddedToDocument;
        },

        /**
         * Returns true if this view is an ancestor of the given view; false otherwise;
         * 
         * @param {Backbone.View} descendant A Backbone.View instance to test against.
         */
        isAncestorOf: function(descendant) {
            return descendant.el ? $.contains(this.el, descendant.el) : false;
        },

        /**
         * Returns true if this view is a descendant of the given view; false otherwise;
         * 
         * @param {Backbone.View} ancestor A Backbone.View instance to test against.
         */
        isDescendantOf: function(ancestor) {
            return ancestor.el ? $.contains(ancestor.el, this.el) : false;
        },

        /**
         * Instruct the view to re-flow itself in its current container.
         * This method doesn't do any actual work beyond calling reflow on all children.
         *
         * Override this method to do any work that is required when the view needs to adjust to its container size.
         * Make sure to call the base implementation when overriding.
         */
        reflow: function() {
            this.invokeOnChildren('reflow');
        },

        /**
         * Helper function that returns the paths of ancestor Backbone views. The algorithm
         * traverses the DOM up the parent chain and the data-view attribute of each DOM element
         * with a data-view attribute. The data-view attribute is added to the DOM tree root of
         * each views/Base.
         *
         * The array is sorted starting with the top-most DOM element's data-view string
         *
         * This function will only return the full ancestor list once it has been added to a DOM
         *
         * @param depth {Number} specifies the maximum number of ancestors to return
         * @return {Array}
         */
        getAncestors: function(depth) {
            var results = [];
            var parent = this.$el;

            while (true) {
                parent = parent.parent();

                if (parent.length > 0) {
                    var dataView = parent.attr("data-view");
                    if (!_(dataView).isUndefined() && !_(dataView).isEmpty())
                        results.unshift(dataView);
                } else {
                    break;
                }

                if (!_(depth).isUndefined() && depth == results.length)
                    break;
            }

            return results;
        }

    });

    _.extend(BaseView.prototype, viewloggingmixin);

    return BaseView;

});

define('util/keyboard',['jquery', 'underscore'], function($, _) {
    var keyboard = {};

    keyboard.KEYS = {
        ENTER: 13,
        UP_ARROW: 38,
        DOWN_ARROW: 40,
        LEFT_ARROW: 37,
        RIGHT_ARROW: 39,
        PAGE_DOWN: 34,
        PAGE_UP: 33,
        SPACE_BAR: 32,
        TAB: 9,
        ESCAPE: 27
    };

    /**
     * A helper method that will enforce circular tabbing inside the given container (i.e. tabbing from the last element
     * will wrap around to the first element and reverse tabbing from the first element will wrap around to the last).
     *
     * Will only set focus on the next element if the event's target is inside the container, and if it does, will
     * prevent the default action of the event.
     *
     * @param $container (jQuery object) the container that should enforce the circular tabbing
     * @param event (DOM event object) the keydown event that initiated the tabbing action
     * @returns (jQuery object) the element that was focused, or null if no element was focused
     */
    keyboard.handleCircularTabbing = function($container, event, includeContainer) {
        var tabbableSelectors = 'a[href]:not(.synthetic-select), area[href], input:not([disabled]),' +
                                'select:not([disabled]), textarea:not([disabled]),' +
                                        'button:not([disabled]), iframe, object, embed, *[tabindex],' +
                                '*[contenteditable]',
            tabbableElements = $();


        // Support an array of containers instead of just one
        if (_.isArray($container)) {
            var containers = $container;
            _.each(containers, function($el) {
                tabbableElements = tabbableElements.add($el.find(tabbableSelectors));
            });
        } else {
            tabbableElements = $container.find(tabbableSelectors);
        }

        if (includeContainer) {
            tabbableElements = tabbableElements.add($container);
        }

        tabbableElements = _.chain(tabbableElements).uniq()
            .filter(function(el) {
                var $el = $(el);
                return $el.is(':visible') && $el.css('visibility') !== 'hidden';
            })
            .value();

        var firstElement = tabbableElements[0],
            lastElement = tabbableElements[tabbableElements.length - 1];

        if(_.contains(tabbableElements, event.target)) {
            event.preventDefault();
            if(event.target === lastElement && !event.shiftKey) {
                return $(firstElement).focus();
            }
            if(event.target === firstElement && event.shiftKey) {
                return $(lastElement).focus();
            }
            if(event.shiftKey) {
                return $(tabbableElements[_.indexOf(tabbableElements, event.target) - 1]).focus();
            }
            return $(tabbableElements[_.indexOf(tabbableElements, event.target) + 1]).focus();
        }
        return null;
    };

    return keyboard;
});
define('views/shared/Modal',
    [
        'jquery',
        'underscore',
        'views/Base',
        'util/keyboard',
        'bootstrap.transition',
        'bootstrap.modal'
    ],
    function($,
            _,
            Base,
            keyboard
            //bootstrap.transition
            //bootstrap.modal
            ) {
        var CLASS_NAME = 'modal fade',
            CLASS_MODAL_WIDE = 'modal-wide',
            HEADER_CLASS = 'modal-header',
            NON_SCROLLING_BODY_CLASS = 'modal-body',
            BODY_CLASS = 'modal-body modal-body-scrolling',
            FOOTER_CLASS = 'modal-footer',
            HEADER_SELECTOR = "." + HEADER_CLASS,
            HEADER_TITLE_SELECTOR = HEADER_SELECTOR + " > h3",
            LOADING_CLASS = "modal-loading",
            LOADING_SELECTOR = "." + LOADING_CLASS,
            LOADING_HORIZONTAL = '<div class="'+ LOADING_CLASS + '"></div>',
            BODY_SELECTOR = ".modal-body",
            BODY_FORM_SELECTOR = BODY_SELECTOR + " > div.form",
            FORM_HORIZONTAL = '<div class="form form-horizontal"></div>',
            FORM_HORIZONTAL_COMPLEX = '<div class="form form-horizontal form-complex"></div>',
            FORM_HORIZONTAL_JUSTIFIED = '<div class="form form-horizontal form-justified"></div>',
            FOOTER_SELECTOR = "." + FOOTER_CLASS,
            BUTTON_CANCEL = '<a href="#" class="btn cancel modal-btn-cancel pull-left" data-dismiss="modal">' + _('Cancel').t() + '</a>',
            BUTTON_BACK = '<a href="#" class="btn back modal-btn-back">' + _('Back').t() + '</a>',
            BUTTON_SAVE = '<a href="#" class="btn btn-primary modal-btn-primary pull-right">' + _('Save').t() + '</a>',
            BUTTON_DONE = '<a href="#" class="btn btn-primary modal-btn-primary pull-right" data-dismiss="modal">' + _('Done').t() + '</a>',
            BUTTON_DELETE = '<a href="#" class="btn btn-primary modal-btn-primary">' + _('Delete').t() + '</a>',
            BUTTON_NEXT = '<a href="#" class="btn btn-primary modal-btn-primary">' + _('Next').t() + '</a>',
            BUTTON_CONTINUE = '<a href="#" class="btn modal-btn-continue pull-left" data-dismiss="modal">' + _('Continue Editing').t() + '</a>',
            TEMPLATE = '\
                <div class="' + HEADER_CLASS + '">\
                    <button type="button" class="close" data-dismiss="modal" aria-hidden="true">&times;</button>\
                    <h3 class="modal-title">&nbsp;</h3>\
                </div>\
                <div class="' +  BODY_CLASS + '">\
                </div>\
                <div class="' + FOOTER_CLASS + '">\
                </div>\
            ';

        // non-exported constants
        var INPUT_SELECTOR = '.btn, input[type="text"], input[type="password"], textarea';

        return Base.extend({
                className: CLASS_NAME,
                attributes: {
                    style: 'display:none',
                    tabIndex: -1

                },
                initialize: function() {
                    Base.prototype.initialize.apply(this, arguments);

                    var defaults = {
                        show: false,
                        keyboard: true,
                        backdrop: true
                    };
                    this.keydownEventCode = null;
                    this.keydownEventTarget = null;
                    this.$el.modal(_.extend({}, defaults, _.pick(this.options, ['backdrop', 'keyboard', 'show', 'remote'])));
                    this.shown = false;

                    //when the hidden event is triggered the modal destroys itself
                    if (this.options.onHiddenRemove) {
                        this.on('hidden', this.remove, this);
                    }
                },
                //if you extend this class and need your own events object then you need to declare it like:
                // $.extend({}, Modal.prototype.events, {
                events: {
                    'show': function(e) {
                        if (e.target !== e.currentTarget) return;
                        this.trigger("show");
                    },
                    'shown': function(e) {
                        if (e.target !== e.currentTarget) return;
                        this.shown = true;
                        // check for any text inputs inside the dialog, focus the first visible one
                        var $textInputs = this.$(INPUT_SELECTOR),
                            textInputsLength = $textInputs.length;
                        if(textInputsLength > 0) {
                            for(var i = 0; i < textInputsLength; i++) {
                                var $textInput = $($textInputs[i]);
                                if ($textInput.is(':visible') && $textInput.css('visibility') !== 'hidden') {
                                    $textInput.focus();
                                    break;
                                }
                            }
                        }
                        this.trigger("shown");
                    },
                    'hide': function(e) {
                        if (e.target !== e.currentTarget) return;
                        this.trigger("hide");
                    },
                    'hidden': function(e) {
                        if (e.target !== e.currentTarget) return;
                        this.shown = false;
                        this.trigger("hidden");
                    },
                    'keydown': function(e) {
                        var keyCode = e.which;
                        this.keydownEventCode = e.which;
                        this.keydownEventTarget = e.target;

                        if (keyCode === keyboard.KEYS.TAB) {
                            keyboard.handleCircularTabbing(this.$el, e);
                        }
                    },
                    'keyup': function(e) {
                        var keyCode = e.which;
                        if (keyCode === keyboard.KEYS.ENTER && this.keydownEventCode === keyboard.KEYS.ENTER) {
                            var $target = $(e.target);

                            if(e.target !== this.keydownEventTarget && $(this.keydownEventTarget).is('input')  && $(this.keydownEventTarget).attr('type') === 'text')
                                $target = $(this.keydownEventTarget);

                            if ($target.is('input') && $target.attr('type') === 'text' && this.$el.find('.btn-primary:visible').length === 1) {
                                // if the currently focused element is any kind of text input,
                                // make sure to blur it so that any change listeners are notified
                                if ($target.is(INPUT_SELECTOR)) {
                                    $target.blur();
                                }
                                e.preventDefault();
                                this.keydownEventCode = null;
                                this.keydownEventTarget = null;
                                this.$el.find('.btn-primary:visible').click();
                            }
                        }
                    }
                },
                hide: function() {
                    this.$el.modal('hide');
                },
                show: function() {
                    this.$el.modal('show');
                },
                toggle: function() {
                    this.$el.modal('toggle');
                },
                remove: function() {
                    if (this.shown){
                        this.hide();
                    }
                    Base.prototype.remove.apply(this, arguments);
                }
            },
            {
                CLASS_NAME: CLASS_NAME,
                CLASS_MODAL_WIDE: CLASS_MODAL_WIDE,
                HEADER_CLASS: HEADER_CLASS,
                BODY_CLASS: BODY_CLASS,
                FOOTER_CLASS: FOOTER_CLASS,
                HEADER_SELECTOR: HEADER_SELECTOR,
                HEADER_TITLE_SELECTOR: HEADER_TITLE_SELECTOR,
                BODY_SELECTOR: BODY_SELECTOR,
                BODY_FORM_SELECTOR: BODY_FORM_SELECTOR,
                LOADING_CLASS: LOADING_CLASS,
                LOADING_SELECTOR: LOADING_SELECTOR,
                LOADING_HORIZONTAL: LOADING_HORIZONTAL,
                FOOTER_SELECTOR: FOOTER_SELECTOR,
                TEMPLATE: TEMPLATE,
                FORM_HORIZONTAL: FORM_HORIZONTAL,
                FORM_HORIZONTAL_COMPLEX: FORM_HORIZONTAL_COMPLEX,
                FORM_HORIZONTAL_JUSTIFIED: FORM_HORIZONTAL_JUSTIFIED,
                BUTTON_CANCEL: BUTTON_CANCEL,
                BUTTON_BACK: BUTTON_BACK,
                BUTTON_SAVE: BUTTON_SAVE,
                BUTTON_CONTINUE: BUTTON_CONTINUE,
                BUTTON_DELETE: BUTTON_DELETE,
                BUTTON_DONE: BUTTON_DONE,
                BUTTON_NEXT: BUTTON_NEXT
            }
        );
    }
);

define('views/shared/controls/Control',[
    'jquery',
    'underscore',
    'backbone',
    'module',
    'views/Base'
],
function(
    $,
    _,
    Backbone,
    module,
    Base
){

    /**
     * The base Control class for all controls.
     *
     * When the value has changed (either through a change to the model, calling setValue or a user action), the control
     * dispatches a "change" event with the arguments (newValue, oldValue).
     */
    return Base.extend({
        moduleId: module.id,
        className: 'control',
        initialize: function(){
            Base.prototype.initialize.apply(this, arguments);
            
            var defaults = {
                    defaultValue: '',
                    enabled: true,
                    validate: false,
                    forceUpdate: false,
                    updateModel: true
            };
            _.defaults(this.options, defaults);

            this._setValue(this.options.defaultValue, false);

            if (this.model) {
                this.activate({skipSetValueRender: true});
            }
            return this;
        },
        startListening: function() {
            if (this.model) {
                this.listenTo(this.model, 'change:'+this.options.modelAttribute, this.setValueFromModel);
            }
        },
        activate: function(options) {
            options = options || {};
            if (this.active) {
                return Base.prototype.activate.apply(this, arguments);
            }
            if (this.model) {
                this.setValueFromModel(options.skipSetValueRender ? false : true);
            }
            return Base.prototype.activate.apply(this, arguments);
        },
        /**
         * Public value accessor
         */
        getValue: function(){
            return this._value;
        },
        getModelAttribute: function() {
            return this.options.modelAttribute;
        },
        /**
         * Set Value and Update Model,
         * @param value {*} - the new value to assign to the control. If the updateModel flag was set to true in the constructor,
         * then this will also update the model's value
         * @param render {Boolean} - if true, then render the control to display the updated value
         * @param options {Object} - additional options to control behavior
         *      {
         *          silent {Boolean} - if true, then don't trigger a change event from the model or control. Default is false
         *      }
         */
        setValue: function(value, render, options){
            options = options || {};
            var returnValue = this._setValue(value, render, options.silent);
            if (this.options.updateModel) {
                this.updateModel(options);
            }
            return returnValue;
        },
        /**
         * Set Value. Does not update model.
         */
        _setValue: function(value, render, suppressEvent){
            value = this.normalizeValue(value);
            if (this._value !== value) {
                var oldValue = this._value;

                this._value = value;

                (render === false) || this.render(); // if render is undefined, render anyway.

                if (!suppressEvent) {
                    this.trigger('change', value, oldValue, this);
                }
            } else {
                if (render) {
                    this.render();
                }
            }
            return this;
        },
        /**
         * Override this method if the value needs to be normalized before being used.
         */
        normalizeValue: function(value) {
            return value;
        },
        /**
         * Unsets value from the model
         */
        unsetValue: function() {
            this.model.unset(this.getModelAttribute());
        },
        /**
         * Gets the value to use in the control from the model. The default behavior is to return the value
         * of the modelAttribute. Render is called except on initialize.
         */
        setValueFromModel: function(render) {
            this._setValue(this.model.get(this.options.modelAttribute), render);
            return this;
        },
        /**
         * Applies the value from the control to the model. Subclasses should call this in response to user
         * interaction
         */
        updateModel: function(options) {
            options = options || {};
            var updateOptions = {
                validate: this.options.validate,
                forceUpdate: this.options.forceUpdate
            };
            $.extend(true, updateOptions, options);

            if (this.model) {
                return (this.model[this.options.save ? 'save' : 'set']
                    (this.getUpdatedModelAttributes(), updateOptions));
            }
            return true;
        },
        /**
         * Returns a dictionary of attribute key-value pairs to apply when updating the model.
         * Can be overridden by subclasses as needed.
         */
        getUpdatedModelAttributes: function() {
            var updatedAttrs = {};
            updatedAttrs[this.options.modelAttribute] = this._value;
            return updatedAttrs;
        },
        enable: function() {
            this.$el.removeClass('disabled');
        },
        disable: function() {
            this.$el.addClass('disabled');
        }
    });

});

/**
 *   views/shared/delegates/Base
 *
 *   Desc:
 *     This a base class for other delegates.

 *   @param {Object} (Optional) options An optional object literal having one settings.
 *
 *    Usage:
 *       var p = new DelegateBase({el: {el}})
 *
 *    Options:
 *        el (required): The event delegate
 */


define('views/shared/delegates/Base',[ 'views/Base' ],function(BaseView){
    return BaseView.extend({
        
        initialize: function() {
            //no-op
        },

        wake: function() {
            //no-op
        },

        sleep: function() {
            //no-op
        },
        
        /**
         * Due to the nature of delegates acting as decorators (delegates do not own
         * their root el), we need to override on base class methods that operate on 
         * DOM elements that the view owns.
         */
        remove: function() {
            this.stopListening();
            return this;
        },

        detach: function() {
            throw new Error('detach is not supported by delegates.');
        },
        
        appendTo: function() {
            throw new Error('jQuery-like attachment methods are not supported by delegates.');
        },

        prependTo: function() {
            throw new Error('jQuery-like attachment methods are not supported by delegates.');
        },

        replaceAll: function() {
            throw new Error('jQuery-like attachment methods are not supported by delegates.');
        },

        insertBefore: function() {
            throw new Error('jQuery-like attachment methods are not supported by delegates.');
        },

        insertAfter: function() {
            throw new Error('jQuery-like attachment methods are not supported by delegates.');
        }

    });
});

define('util/string_utils',
    [],
    function() {

        var strEndsWith = function(str, suffix) {
            if (!str || !suffix) {
                return false;
            }
            return str.indexOf(suffix, str.length - suffix.length) !== -1;
        };

        /**
        * Changes the first letter of first word in a string to upper-case.
        * All other letters remain unchanged. 
        *
        * @param str {String} - string to capitalize.
        * @returns {String} - Returns a string with the first letter of first word upper-case. 
        */
        var capitalize = function(str) {
            return str.charAt(0).toUpperCase() + str.slice(1);
        };

        return {
            strEndsWith: strEndsWith,
            capitalize: capitalize
        };
    }
);
/**
 *   views/shared/delegates/Popdown
 *
 *   Desc:
 *     This class applies popdown menus and dialogs.

 *   @param {Object} (Optional) options An optional object literal having one settings.
 *
 *    Usage:
 *       var p = new Popdown({options})
 *
 *    Options:
 *        el (required): The dialog.
 *        arrow: jQuery selector or object for arrow that points up. Defaults to ".arrow".
 *        mode: menu (default) or dialog. Change to dialog to prevent clicks inside the dialog from closing the popdown.
 *        show: true or false (default). Open the menu on initialization.
 *        adjustPosition: true (default) or false. Adjust position will  keep the dialog within the screen margin.
 *        minMargin: Integer. The number of pixels between the edge of the screen and the dialog. Defaults to 20.
 *        ignoreClasses: [] array of HTML classes to ignore clicks inside of so that the popdown does not close
 *        ignoreToggleMouseDown: true or false (default). Ignore mousedown events on the toggle or toggle's children. 
 *                              If ignoreToggleMouseDown is enabled, the PopTart will take no action when
 *                              clicking on the PopTart's toggle activator element. This allows the creator of the 
 *                              PopTart to fully manage closing the opened PopTart.
 *
 *    Methods:
 *        show: show the dialog (directly calling show should be avoided and should not be necessary).
 *        hide: hide the dialog.
 *        scrollToVisible: scrolls the page if necessary to show the entire dialog.
 */


define('views/shared/delegates/PopdownDialog',[
    'jquery',
    'underscore',
    'views/shared/delegates/Base',
    'util/string_utils',
    'util/keyboard'
],function(
    $,
    _,
    DelegateBase,
    string_utils,
    keyboard
){
    return DelegateBase.extend({
        initialize: function() {
            var defaults = {
                arrow: ".arrow",
                mode: "dialog",  // "menu" or "dialog"
                adjustPosition: true,
                minMargin: 20,
                allowPopUp: true,
                direction:'auto',
                zIndex:'auto',
                scrollContainer: window,
                ignoreClasses: []
            };

            _.defaults(this.options, defaults);
            this.isShown = false;
            
            this.addEventHandlers = _.bind(this.addEventHandlers, this);
            this.dialogClick = _.bind(this.dialogClick, this);
            this.keepInBoundsDebounced =  _.debounce(this.keepInBounds, 100);

            // if this.$el doesn't already have an id, create a unique one (to be used in show())
            if (!this.$el.attr('id')) {
                this.$el.attr('id', 'dialog-' + this.cid);
            }
            
            if (this.options.show) {
                this.show(this.options.show);
            }
        },
        events: {
            'click': 'dialogClick',
            'keydown': function(e) {
                if (e.which === keyboard.KEYS.TAB) {
                    keyboard.handleCircularTabbing(this.$el, e);
                }
            },
            'keydown .close': function(e) {
                if (e.which === keyboard.KEYS.ENTER) {
                    //Prevent click event for anchor tags
                    e.preventDefault();
                    this.hide(e);
                }
            }
        },
        arrow: function() {
            return this.$el.find(this.options.arrow).first();
        },
        scrollContainer: function() {
            if (!this.$scrollContainer) {
                this.$scrollContainer = $(this.options.scrollContainer);
            }
            return this.$scrollContainer;
        },
        toggle: function($pointTo) {
            this.trigger('toggle', $pointTo);
            
            if (this.isShown) {
                this.hide();
            } else {
                this.show($pointTo);
            }
            
            this.trigger('toggled', $pointTo);

            return true;
        },
        show: function($pointTo, $toggle) {
            $toggle = $toggle || $pointTo;
            this.trigger('show', $pointTo);

            this.$pointTo = $pointTo;
            
            if (!this.isShown) {
                _.defer(this.addEventHandlers, $toggle);
            }
            if (this.options.zIndex != 'auto') {
                this.$el.css('zIndex', this.options.zIndex);
            }

            this.options.adjustPosition && $pointTo && this.adjustPosition($pointTo);
            this.isShown = true;
            this.$el.addClass('open').show();
            // add a data attribute to map the toggle to the open dialog, this is for automated testing
            $pointTo.attr('data-dialog-id', this.$el.attr('id'));

            this.trigger('shown', $pointTo);
        },
        hide: function(e) {
            if (!this.isShown) {
                return false;
            }

            this.trigger('hide');

            this.removeEventHandlers();
            this.isShown = false;
            this.$el.removeClass('open').hide();

            this.trigger('hidden', e);
        },
        addEventHandlers: function($toggle) {
            $('html').on('mousedown.popdown.' + this.cid, function(e) {
                this.bodyMouseDown(e, $toggle);
            }.bind(this));
            $(window).on('keydown.' + this.cid, function(e) {
                this.windowKeydown(e, $toggle);
            }.bind(this));
            
            
            if (this.options.scrollContainer !== window) {
                this.scrollPosition = {top: this.scrollContainer().scrollTop(), left: this.scrollContainer().scrollLeft()};
                this.scrollContainer().on('scroll.' + this.cid, function(e) {
                    if (this.$pointTo) {
                        this.adjustPosition(this.$pointTo);
                    } else {
                        this.containerScroll(e);
                    }
                }.bind(this));
            }
            
            return this;
        },
        removeEventHandlers: function() {
            $('html').off('.' + this.cid);
            $(window).off('.' + this.cid);
            
            if (this.options.scrollContainer !== window) {
                this.scrollContainer().off('.' + this.cid);
            }
            
            return this;
        },
        deactivate: function() {
            this.hide();
            return DelegateBase.prototype.deactivate.apply(this, arguments);
        },
        stopListening: function() {
            this.removeEventHandlers();
            return DelegateBase.prototype.stopListening.apply(this, arguments);
        },
        measure: function($el, offsets) {
            var measures = {};
            
            measures.width = $el.outerWidth();
            measures.halfWidth = measures.width / 2;
            measures.height = $el.outerHeight();
            measures.halfHeight = measures.height /2;
            if (offsets !== false) {
                measures.offset = $el.offset();
                measures.position = $el.position();
                measures.center = {
                    left: measures.offset.left + measures.halfWidth,
                    top: measures.offset.top + measures.halfHeight
                };
            }
            return measures;
        },
        adjustPosition: function($pointTo) {
            var dir = this.options.direction;
            this['adjustPosition' + string_utils.capitalize(dir)]($pointTo);
        },
        adjustPositionAuto: function($pointTo) {
            this.$el.css({top: -9999, left: -9999, bottom: 'auto'});
            this.$el.show();
        
            var m = {}, //measurements
                shift = {left: 0, top:0}; //necessary corrections to fit in view
            var positionFromTop = true;
            
            m.toggle = this.measure($pointTo);
            m.dialog = this.measure(this.$el);
            m.arrow = this.measure(this.arrow(), false);
            m.window = this.measure($(window), false);
            m.window.top =  $(window).scrollTop();
            m.window.bottom = m.window.height + $(window).scrollTop();
            
            // Compensate for different offset parents.
            m.dialogParent = {offset: this.$el.offsetParent().offset()};
            m.toggle.relativeOffset = {
                                left: m.toggle.offset.left - m.dialogParent.offset.left + m.toggle.halfWidth,
                                top: Math.floor(m.toggle.offset.top - m.dialogParent.offset.top)
                            };

            //Determine if the default centering need to be shifted left or right
            if (m.toggle.center.left < m.dialog.halfWidth + this.options.minMargin) { //Needs to be pushed right
                shift.left = (m.dialog.halfWidth - m.toggle.center.left) + this.options.minMargin;
            } else if (m.toggle.center.left + m.dialog.halfWidth + this.options.minMargin  > m.window.width) { //Needs to be pushed left
                shift.left = m.window.width - (m.toggle.center.left + m.dialog.halfWidth + this.options.minMargin);
            }
            shift.left=Math.round(shift.left);
            
            // Determine if there is enough room to pop down
            var popDownDialogBottom =  m.toggle.offset.top + m.toggle.height + m.dialog.height + this.options.minMargin;
            var popUpDialogTop =  m.toggle.offset.top - m.dialog.height -  this.options.minMargin;
            if (popDownDialogBottom > m.window.bottom && this.options.allowPopUp && popUpDialogTop > m.window.top ) {
                //Pop upward
                shift.top=-m.dialog.height - m.arrow.halfWidth;
                shift.bottom= m.toggle.height + m.arrow.halfWidth;
                this.$el.addClass('up');
                
                //If this hasn't been attached the body or some other element, set positionFromTop to false. It will be positioned from the bottom.
                //It's better to position from the bottom so dialogs that change height, like the timerangepicker, are correctly positioned.
                positionFromTop = this.options.attachDialogTo || this.$el.parent()[0] == $('body')[0] || this.$el.parent()[0] == $('.modal:visible')[0];
            } else {
                //Pop downward
                shift.top= m.toggle.height;
                this.$el.removeClass('up');
                if (popDownDialogBottom > m.window.bottom) {
                    //Scroll
                    this.scrollContainer().scrollTop(this.scrollContainer().scrollTop() + popDownDialogBottom - m.window.bottom);
                    
                    //reset the relative offsets
                    m.toggle.offset = $pointTo.offset();
                    m.toggle.relativeOffset.top = m.toggle.offset.top - m.dialogParent.offset.top;
                }
            }
            
            //Reset the position and center within the viewable area
            this.position = {
                 top:  positionFromTop ? shift.top + m.toggle.relativeOffset.top : 'auto',
                 left: m.toggle.relativeOffset.left,
                 marginLeft: - m.dialog.halfWidth + shift.left,
                 bottom: positionFromTop ? 'auto' : shift.bottom 
                };
            
            this.$el.css(this.position);
            this.arrow().css('marginLeft', 0 - m.arrow.halfWidth - shift.left);

            //Fix left corner rounding if necessary
            if (m.dialog.halfWidth - m.arrow.halfWidth - shift.left < 8) { //Falling off the left
                this.$el.css('borderTopLeftRadius', Math.max(m.dialog.halfWidth - m.arrow.halfWidth - shift.left, 0));
            } else { //Needs to be pushed left
                this.$el.css('borderTopLeftRadius', '');
            }

        },
        adjustPositionRight: function($pointTo) {
            this.$el.addClass('right');
            this.$el.css({top: -9999, left: -9999});
            this.$el.show();
        
            var m = {}, //measurements
                shift = {left: 0, top:0}, //necessary corrections
                calculateTop = function() {
                    return Math.round(shift.top) + m.toggle.relativeOffset.top - m.dialog.halfHeight + m.toggle.halfHeight;
                }.bind(this); 
            
            m.toggle = this.measure($pointTo);
            m.dialog = this.measure(this.$el);
            m.arrow = this.measure(this.arrow(), false);
            m.window = this.measure($(window), false);
            m.window.top =  $(window).scrollTop();
            m.window.bottom = m.window.height + $(window).scrollTop();
            m.arrow.minMargin = 10;
            m.arrow.maxShift = m.dialog.halfHeight - m.arrow.halfHeight - m.arrow.minMargin;
            
            // Compensate for different offset parents.
            m.dialogParent = {offset: this.$el.offsetParent().offset()};
            m.toggle.relativeOffset = {
                                left: m.toggle.offset.left - m.dialogParent.offset.left + m.toggle.width,
                                top: Math.floor(m.toggle.offset.top - m.dialogParent.offset.top)
                            };

            //Determine if the default centering need to be shifted up or down
            if (m.toggle.center.top - m.window.top < m.dialog.halfHeight + this.options.minMargin) { //Needs to be down
                shift.top = (m.dialog.halfHeight - (m.toggle.center.top - m.window.top)) + this.options.minMargin;
            } else if (m.toggle.center.top + m.dialog.halfHeight+ this.options.minMargin  > m.window.bottom ) { //Needs to be pushed up
                shift.top = m.window.bottom - (m.toggle.center.top + m.dialog.halfHeight + this.options.minMargin);
            }
            
            //Make sure it's not partially hidden over the top of page
            if (calculateTop() < this.options.minMargin) {
                shift.top = shift.top + (this.options.minMargin - calculateTop());
            }
            
            //Determine if there is sufficient room to include the point
            if (shift.top > m.arrow.maxShift) { //Needs to be down
                shift.top -= shift.top - m.arrow.maxShift;
            } else if (-shift.top > m.arrow.maxShift) { //Needs to be pushed up
                shift.top -= shift.top+m.arrow.maxShift;
            }
            
            //Reset the position and center within the viewable area
            this.position = {
                 top: calculateTop(),
                 left: m.toggle.relativeOffset.left, marginLeft: m.arrow.halfWidth
                };
                
            
            this.$el.css(this.position);
            this.arrow().css('marginTop', 0 - m.arrow.halfHeight - shift.top);

        },
        containerScroll: function(e)  {
            var newScrollTop = this.scrollContainer().scrollTop(),
                newScrollLeft = this.scrollContainer().scrollLeft();
                
            this.position.top = this.position.top + (this.scrollPosition.top - newScrollTop);
            this.position.left = this.position.left + (this.scrollPosition.left  - newScrollLeft);
            this.$el.css(this.position);
                            
            this.scrollPosition = {top: newScrollTop, left: newScrollLeft};
            this.keepInBoundsDebounced();
        },
        keepInBounds: function(e)  {
            // if it is no longer pointing at something shown in the view container, close;
            var containerTop = this.scrollContainer().offset().top,
                containerBottom  = containerTop + this.scrollContainer().outerHeight(),
                elEdge = this.$el.hasClass('up') ? this.$el.offset().top + this.$el.outerHeight() : this.$el.offset().top; // use the bottom edge when popping upward
                
            if (elEdge < containerTop || elEdge > containerBottom) {
                this.hide();
            }
            
        },
        bodyMouseDown: function(e, $toggle) {
            var $target = $(e.target);
            
            // If the menu is already closed, don't do anything.
            if (!this.isShown) {
                return;
            }

            // Ignore clicks on the toggle or toggle's children.
            if (this.options.ignoreToggleMouseDown && $toggle && (($target[0] === $toggle[0]) || $toggle.has($target).length)) {
                return;
            }

            // Ignore clicks inside of the dialog
            if ($.contains(this.$el[0], e.target) || e.target === this.$el[0]) {
                return;
            }
            
            //Ignore clicks on elements with classes specified in this.options.ignoreClasses
            for(var i = 0; i < this.options.ignoreClasses.length; i++){
                if ($target.closest("." + this.options.ignoreClasses[i]).length) {
                    return;
                }
            }

            this.hide(e);
        },
        dialogClick: function(e) {
            if (this.options.mode != "dialog") {
                this.hide(e);
            }
        },
        windowKeydown: function(e, $toggle) {
            var escapeKeyCode = 27,
                enterKeyCode = 13;
            
            if (e.keyCode == escapeKeyCode)  {
                this.hide();
                return true;
            }
            
            if (e.keyCode == enterKeyCode)  {
                this.bodyMouseDown(e, $toggle);
            }
            
            return true;
        },
        pointTo: function($pointTo) {
            if (!this.isShown) {
                this.show($pointTo);
            } else {
                this.adjustPosition($pointTo);
            }
        }
    });
});

/**
 *   views/shared/delegates/Popdown
 *
 *   Desc:
 *     This class applies popdown menus and dialogs.

 *   @param {Object} (Optional) options An optional object literal having one settings.
 *
 *    Usage:
 *       var p = new Popdown({options})
 *
 *    Options:
 *        el (required): The dialog and toggle container. Recommend that this is the offset container for the dialog.
 *        toggle: jQuery selector for the toggles defaults to "> .popdown-toggle, > dropdown-toggle".
 *        dialog: jQuery selector or object for the dialog defaults to "> .popdown-dialog,  > dropdown-menu".
 *        arrow: jQuery selector or object for arrow that points up. Defaults to ".arrow".
 *        mode: menu (default) or dialog. Change to dialog to prevent clicks inside the dialog from closing the popdown.
 *        show: true or false (default). Open the menu on initialization.
 *        adjustPosition: true (default) or false. Adjust position will  keep the dialog within the screen margin.
 *        minMargin: Integer. The number of pixels between the edge of the screen and the dialog. Defaults to 20.
 *        attachDialogTo: jQuery selector or object to attach the dialog element to, usually 'body'
 *                          this can be useful to avoid hidden overflow issues, but should be used with care since any
 *                          scoped CSS selectors or event delegate listeners will no longer work *
 *        ignoreClasses: [] array of HTML classes to ignore clicks inside of so that the popdown does not close
 *
 *    Methods:
 *        show: show the dialog (directly calling show should be avoided and should not be necessary).
 *        hide: hide the dialog.
 *        scrollToVisible: scrolls the page if necessary to show the entire dialog.
 */


define('views/shared/delegates/Popdown',[
    'jquery',
    'underscore',
    'views/shared/delegates/Base',
    'views/shared/delegates/PopdownDialog'
],function(
    $,
    _,
    DelegateBase,
    PopdownDialog
){
    return DelegateBase.extend({
        initialize: function() {
            var defaults = {
                toggle: "> .popdown-toggle, > .dropdown-toggle",
                dialog: "> .popdown-dialog, > .dropdown-menu",
                arrow: ".arrow",
                mode: "menu",  // "menu" or "dialog"
                adjustPosition: true,
                minMargin: 20,
                attachDialogTo: false,
                allowPopUp: true,
                scrollContainer: window,
                ignoreClasses: []
            };

            _.defaults(this.options, defaults);

            this.children = {};
            this.events = {};
            this.events["mousedown " + this.options.toggle] = "toggle";
            this.events["keydown " + this.options.toggle] = "keydownToggle";
            this.events["click " + this.options.toggle] = "clickToggle";
            this.delegateEvents(this.events);
            this.$activeToggle = this.$(this.options.toggle).first();

            if (this.options.show) {
                this.show();
            }
        },
        delegate: function() {
            if (!this.children.delegate) {
                var options = $.extend(true, {}, this.options);
                options.el = (options.dialog instanceof $) ? options.dialog[0] : this.$(options.dialog)[0];
                
                if (_.isString(options.scrollContainer)) {                
                    options.scrollContainer = this.$activeToggle.closest(options.scrollContainer);
                }
                               
                this.children.delegate = new PopdownDialog(options);
                
                this.listenTo(this.children.delegate, 'toggle', this.hearToggle);
                this.listenTo(this.children.delegate, 'toggled', this.hearToggled);
                this.listenTo(this.children.delegate, 'show', this.hearShow);
                this.listenTo(this.children.delegate, 'shown', this.hearShown);
                this.listenTo(this.children.delegate, 'hide', this.hearHide);
                this.listenTo(this.children.delegate, 'hidden', this.hearHidden);
                if (options.attachDialogTo) {
                    $(options.attachDialogTo).append(this.children.delegate.el);
                }
            }
            return this.children.delegate;
        },
        toggle: function(e) {
            e.preventDefault();

            if ($(e.currentTarget).is('.disabled, :disabled')) return;

            this.delegate().toggle($(e.currentTarget));

            return true;
        },
        hearToggle: function($toggle) {
            this.trigger('toggle', $toggle);
        },
        hearToggled: function($toggle) {
            this.trigger('toggled', $toggle);
        },
        show: function() {
            this.delegate().show(this.$activeToggle);
        },
        hearShow: function($toggle) {
            this.trigger('show', $toggle);

            $(window).on('resize.popdown.' + this.cid, this.windowResize.bind(this));
            this.$activeToggle.removeClass('active');
            this.$activeToggle = $toggle.addClass('active');
   
        },
        hearShown: function($toggle) {
            this.trigger('shown', $toggle);
        },
        hide: function(e) {
            if (!this.delegate().isShown) return false;

            $(window).off('.' + this.cid);

            this.$activeToggle.removeClass('active');

            if ($.contains(this.children.delegate.el, $(':focus')[0])) {
                this.$activeToggle.focus();
            }
            this.children.delegate.hide();
        },
        clickToggle: function(e) {
            e.preventDefault();
        },
        keydownToggle: function(e) {
            var enterKeyCode = 13;
            if (e.keyCode == enterKeyCode)  {
                this.toggle(e);
            }
        },
        hearHide: function($toggle) {
            $(window).off('.popdown.' + this.cid);
            this.trigger('hide', $toggle);

            this.$activeToggle.removeClass('active');
        },
        hearHidden: function($toggle) {
            this.trigger('hidden', $toggle);
        },
        stopListening: function() {
            this.children.delegate && this.children.delegate.remove();
            return DelegateBase.prototype.stopListening.apply(this, arguments);
        },
        adjustPosition: function() {
            this.delegate();
            this.children.delegate.adjustPosition(this.$activeToggle);
        },
        windowResize: function(e) {
            if (this.children.delegate && this.children.delegate.isShown) {
                this.adjustPosition(this.$activeToggle);
            }
        },
        pointTo: function($activeToggle) {
            this.$activeToggle = $activeToggle;
            this.delegate().adjustPosition(this.$activeToggle);
        },
        remove: function() {
            DelegateBase.prototype.remove.apply(this);
            this.children.delegate && this.children.delegate.remove();
            if (this.children.delegate && this.options.attachDialogTo) {
                this.children.delegate.$el.remove();
            }
            delete this.children.delegate;
            $(window).off('resize.popdown.' + this.cid);
            $(window).off('.' + this.cid);
            this.undelegateEvents();
            return this;
        }
    });
});

define('views/shared/controls/SyntheticSelectControl',[
    'jquery',
    'underscore',
    'backbone',
    'module',
    'views/shared/controls/Control',
    'views/shared/delegates/Popdown',
    'util/math_utils',
    'bootstrap.tooltip'
],
function(
    $,
    _,
    Backbone,
    module,
    Control,
    Popdown,
    math_utils,
    tooltip
){
    /**
     * Synthetic Select dropdown a-la Bootstrap
     *
     * @param {Object} options
     *     {Object} model The model to operate on
     *     {String} modelAttribute The attribute on the model to observe and update on selection
     *     {Object} items An array of elements and/or subarrays, where:
     *                    - An element is an object with keys described below (e.g. value & label)
     *                       ~ An element typically represent a menu item
     *                       ~ An element can be just a menu header, commonly used with menu item groupings (see below)
     *                       ~ An element can represent a submenu if it contains `children` array of elements (nested format)
     *                    - A subarray is a group of elements
     *                       ~ A subarray represents a grouping of menu items with an divider and an optional menu header
     *                    
     *                    label (textual display - if none specified, will default to value),
     *                    value (value to store in model - if none specified, element is considered a menu header instead of menu item)
     *                    icon (icon name to show in menu and button label)
     *                    iconURL (URL of icon to show in menu and button label)
     *                    enabled (optional boolean, defaults to true, whether to enable the selection)
     *                    description (additional text)
     *                    children (array of elements - use for submenus)
     *                    (ie, {label: 'Foo Bar', value: 'foo', icon: 'bar'}).
     *     {String} prompt (Optional) String to display if the value is undefined. If prompt is undefined,
     *                                then the control will just display the first item. Default value
     *                                is undefined
     *     {String} help (Optional) Html to display in the bottom help section
     *     {String} label (Optional) Html to display as the button label
     *     {String} toggleClassName (Optional) Class attribute to add to the parent element
     *     {String} menuClassName (Optional) Class attribute to add to the dialog element
     *     {String} menuWidth (Optional) narrow, normal, or wide
     *     {String} additionalClassNames (Optional) Class attribute(s) to add to control
     *     {String} iconClassName (Optional) Class attribute(s) to add to icon
     *     {String} iconURLClassName (Optional) Class attribute(s) to add to iconURL
     *     {Boolean} nearestValue (Optional) if true: try to select the nearest value from items for the value of the modelAttribute 
     *     {String} descriptionPosition (Optional) [right|bottom] Default value is "right". Position of the element's description text
     *     relative to the item's label
     *
     *
     *   Advanced example of items array with nesting & grouping of elements:
     *   var items = [
     *       [
     *           {value: "item1", label: "Menu item with desc", description: "Click here!"},
     *           {value: "item2", label: "Menu item"},
     *           {label: "Sub menu", children: [
     *               {value: "item3", label: "Action one"},
     *               {value: "item4", label: "Action two"}
     *           ]},
     *       ],
     *       [
     *           {label: "Menu group header"},
     *           {value: "item5", label: "Another menu item"},
     *           {value: "item6"}
     *       ],
     *       [
     *           {label: "Submenu in a group", children: [
     *               {label: "Cool submenu header"},
     *               {value: "item7", label: "More action", description: "foo"},
     *               {value: "item8", label: "Yet another action", description: "bar"},
     *           ]}
     *       ]
     *   ];
     */

    return Control.extend({
        moduleId: module.id,
        className: 'control btn-group',
        keys: {
            UP_ARROW: 38,
            DOWN_ARROW: 40,
            ESC: 27
        },
        items: undefined,
        selectedItem: undefined,
        renderList: true,
        initialize: function() {
            var defaults = {
                toggleClassName: '' ,
                menuClassName: '',
                iconClassName: 'icon-large',
                iconURLClassName: 'icon-large',
                descriptionPosition: 'right',
                label: '',
                popdownOptions: {el: this.el},
                html: '',
                nearestValue: false,
                prompt: undefined,
                // optional method to customize item label formatting
                formatLabel : function(item) {
                    // By default label displayed is item.label.
                    // If none specified this defaults to item.value. 
                    return item.label || item.value;
                }
            };

            _.defaults(this.options, defaults);
            _.defaults(this.options.popdownOptions, defaults.popdownOptions);
            // our own expando property
            this.options.expando = 'ssc-' + this.cid;
            // store structure in this.items
            this.setItems(this.options.items, {skipRender: true});

            // compile sub template (main template already compiled in constructor)
            this.compiledSubTemplate = this.compileTemplate(this.subTemplate);

            var modelAttribute = this.options.modelAttribute;
            if (modelAttribute) {
                this.$el.attr('data-name', modelAttribute);
            }

            Control.prototype.initialize.apply(this, arguments);
        },
        
        activate: function(options) {
            options = options || {};
            if (this.active) {
                return Control.prototype.activate.apply(this, arguments);
            }
            this.renderList = true;
            Control.prototype.activate.apply(this, arguments);
        },

        // override setValue to keep _value and selectedItem in sync by definition
        _setValue: function(value, render, suppressEvent) {
            var oldValue = this._value;
            value = this.normalizeValue(value);
            this._value = value;

            if (_.isUndefined(this._value) || !this.items || !this.items.length) {
                this.selectedItem = undefined;
            } else if (this.options.nearestValue) {
                var nearestObject = math_utils.nearestMatchAndIndexInArray(this._value, this.valuesAsInts);
                if (!_.isUndefined(nearestObject.index)) {
                    var nearestItem = this.itemsMap[nearestObject.index];
                    this.selectedItem = this.findItem(nearestItem.value);
                }
            } else {
                this.selectedItem = this.findItem(this._value);
            }

            // trigger change event
            if (!suppressEvent) {
                this.trigger('change', value, oldValue, this);
            }

            // if render requested or undefined, render anyway.
            if (render !== false) {
                this.render();
            }

            return this;
        },

        /**
         * This is a parametrized recursive algorithm to traverse
         * @items hierarchy and execute a base function @baseFn at
         * each leaf item. If baseFn returns a truthy value, this
         * signals that the required leaf operation is complete, and
         * traversal is terminated.
         */
        _recursiveTreeTraversal: function(items, baseFn) {
            var item, args, found, i, len;
            for (i = 0, len = items.length; i < len; i++) {
                item = items[i];
                // recursive case 1: array (i.e. grouped items)
                if (_.isArray(item)) {
                    // recursive call using current arguments with items=item
                    args = Array.prototype.slice.call(arguments, 1);
                    args.unshift(item);
                    found = this._recursiveTreeTraversal.apply(this, args);
                }
                // recursive case 2: children array (i.e. nested sub menu)
                else if (item.children) {
                    // recursive call using current arguments with items=item.children
                    args = Array.prototype.slice.call(arguments, 1);
                    args.unshift(item.children);
                    found = this._recursiveTreeTraversal.apply(this, args);
                }
                // base case for leaf items
                else if (_(item).has('value')) {
                    // call baseFn using current arguments without items or baseFn
                    args = Array.prototype.slice.call(arguments, 2);
                    args.unshift(item);
                    found = baseFn.apply(this, args);
                }
                if (found) { return found; }
            }
            return false;
        },

        getFirstItem: function() {
            return this._recursiveTreeTraversal(this.items, function(item){
                return item;
            });
        },

        // keeping this for backward compatibility but this is a not a proper naming
        // and not required, since item is now part of the view state.
        findItem: function(value) {
            // if no value set, default to selected item
            if (_.isUndefined(value)) {
                return this.selectedItem;
            }
            return this._recursiveTreeTraversal(this.items, function(item, value) {
                if (_.isEqual(item.value, value)) {
                    return item;
                }
            }, value);
        },

        _mapItemsToIndices: function() {
            // internal map of item index to item object
            this.itemsMap = [];
            this._recursiveTreeTraversal(this.items, function(item, memo) {
                // each item gets a unique index
                // note that expando is a random key to avoid collisions
                item[this.options.expando] = memo.length;
                memo.push(item);
            }.bind(this), this.itemsMap);
        },

        _mapItemsToValues: function() {
            this.valuesAsInts = [];
            this._recursiveTreeTraversal(this.items, function(item, memo) {
                var convertedToInt = parseInt(item.value, 10);
                if (_.isNaN(convertedToInt)) {
                    throw new Error('You cannot use the nearestValue option with a SyntheticSelect control that has values other than ints!');
                }
                memo.push(convertedToInt);
            }, this.valuesAsInts);
        },

        setItems: function(items, options) {
            this.items = items;
            // flag to re-render popdown
            this.renderList = true;
            // build local map
            if (this.items && this.items.length) {
                this._mapItemsToIndices();
                if (this.options.nearestValue) {
                    this._mapItemsToValues();
                }
            }
            if (options && options.skipRender) { return; }
            this.debouncedRender();
        },

        stopListening: function() {
            if (arguments.length === 0) {
                $(window).off('.' + this.cid);
                this.$menu && this.$menu.off('.' + this.cid);                
            }
            Control.prototype.stopListening.apply(this, arguments);
        },
        show: function(){
            this.$menu.find(".icon-check:visible").closest('a').focus();
            this.$menu.on('click.' + this.cid, 'a', this.click.bind(this));
            $(window).on('keydown.' + this.cid, this.keydown.bind(this));
            this.trigger('popdownShown');
        },
        hide: function(){
            $(window).off('.' + this.cid);
            this.$menu.off('.' + this.cid);
            this.trigger('popdownHidden');
        },
        disable: function(){
            this.options.enabled = false;
            this.$('a.dropdown-toggle').addClass('disabled');
        },
        enable: function(){
            this.options.enabled = true;
            this.$('a.dropdown-toggle').removeClass('disabled');
        },
        tooltip: function(options){
            this.$('a.dropdown-toggle').tooltip(options);
        },
        click: function(e) {
            var $currentTarget = $(e.currentTarget);
            if (!$currentTarget.hasClass('disabled') && !$currentTarget.is('.submenu-label')) {
                var itemIdx = $currentTarget.data('item-idx'),
                    item = this.itemsMap[itemIdx];
                this.setValue(item.value, true);
                this.$(".dropdown-toggle").focus();
            }
            e.preventDefault();
        },
        keydown: function (e) {
            if (e.shiftKey || e.ctrlKey || e.metaKey || e.altKey || e.which == 9) {
                return true;
            }

            if (e.keyCode == this.keys['ESC']) {
                this.$('.dropdown-toggle').focus();
                e.preventDefault();
                e.stopPropagation();
                return;
            }

            var $focused = $(':focus');
            // if dropdown toggle in focus, only handle DOWN key - leave rest for lovely Popdown
            if ($focused.is('.dropdown-toggle')) {
                if (e.keyCode == this.keys['DOWN_ARROW']) {
                    this.$menu.find('a:eq(0)').focus();
                }
                e.preventDefault();
                e.stopPropagation();
                return;
            }

            // if dropdown menu item in focus, respond to UP/DOWN/<letter/digit> keys 
            // TODO: support all dropdown-menu (i.e. submenu with LEFT/RIGHT keys)
            var $menu = $focused.closest('ul.dropdown-menu-main'),
                $items = $menu.find('> li a:visible'),
                index = -1, handled = false;

            if (!$items.length) return true;

            index = $items.index($focused);
            if (e.keyCode == this.keys['DOWN_ARROW']) {
                if (index < $items.length - 1) {
                    index++;
                    $items.eq(index).focus();
                }
                handled = true;
            } else if (e.keyCode == this.keys['UP_ARROW']) {
                if (index === 0) {
                    this.$(".dropdown-toggle").focus();
                } else {
                    index--;
                    $items.eq(index).focus();
                }
                handled = true;
            } else if (
                ((e.keyCode >= 48) && (e.keyCode <= 57))  || //number
                ((e.keyCode >= 65) && (e.keyCode <= 90))  || //uppercase letter
                ((e.keyCode >= 97) && (e.keyCode <= 122)) || //lowercase letter
                ((e.keyCode >= 128) && (e.keyCode <= 165))   //extended letter
            ) {
                var keyChar = String.fromCharCode(e.keyCode).toLowerCase();
                $items.each(function() {
                    var $option = $(this),
                        firstChar = $option.text().replace(/^\s\s*/, '').substring(0, 1).toLowerCase();

                    if (keyChar == firstChar) {
                        $option.focus();
                        handled = true;
                        return false; // break out of each() loops
                    }
                });
            }

            if (handled) {
                e.preventDefault();
                e.stopPropagation();
                return;
            }

            return true;
        },
        render: function() {
            var selectedItem = this.selectedItem;

            if (!this.items || !this.items.length) {
                return this;
            }

            // if no item selected
            if (!selectedItem) {
                // if we have a prompt then display prompt as label
                if (!_(this.options.prompt).isUndefined()) {
                    selectedItem = {
                        label: this.options.prompt,
                        value: undefined
                    };
                // otherwise default to first item of list
                } else {
                    selectedItem = this.getFirstItem();
                }
            }

            if (this.renderList) {
                var template = this.compiledTemplate({
                    items: this.items,
                    selectedItem: selectedItem,
                    options: this.options,
                    menuWidthClass: (this.options.menuWidth && this.options.menuWidth != "normal" ? "dropdown-menu-" + this.options.menuWidth : ''),
                    subTemplate: this.compiledSubTemplate
                });
                this.$el.html(template);
                
                this.$menu =  this.$('> .dropdown-menu');

                // menu has submenus if any of its items has children
                var hasSubMenus = !!(_.find(_.flatten(this.items, true), function(item) {
                    return _.has(item, 'children');
                }));
                if (hasSubMenus) {
                    // ensure sub menu flyouts are not hidden
                    this.$menu.find('> ul').css({
                        'overflow': 'visible',
                        'max-height': 'none'
                    });
                }

                if (this.children.popdown) {
                    this.children.popdown.remove();
                    this.stopListening(this.children.popdown);
                    delete this.children.popdown;
                }

                this.children.popdown = new Popdown($.extend(true, {}, this.options.popdownOptions));
                this.listenTo(this.children.popdown, "shown", this.show);
                this.listenTo(this.children.popdown, "hidden", this.hide);

                this.renderList = false;
            }

            var additionalClassNames = this.options.additionalClassNames;
            if(additionalClassNames) {
                this.$el.addClass(additionalClassNames); 
            }

            // TODO: this is not required if Popdown view supports successive rendering as it should
            // Hide or show the checkmarks
            this.$menu.find('a.synthetic-select').each(function(i, el) {
                // using attr() instead of data() since latter attempts to deduce the type
                // and automatically convert value from string to a JavaScript type value
                var $el = $(el),
                    itemIdx = $el.data('item-idx'),
                    item = this.itemsMap[itemIdx];
                $el.find('.icon-check')[(selectedItem && _.isEqual(item.value, selectedItem.value)) ? 'show' : 'hide']();
            }.bind(this));
                                    
            // TODO: this is not required if Popdown view supports successive rendering as it should
            //Update the toggle label
            if (selectedItem) {
                if (selectedItem.label || selectedItem.value) {
                    this.$(".dropdown-toggle > .link-label").text(_(this.options.label).t() + " " + _(selectedItem.label || selectedItem.value).t());
                }
                if (selectedItem.icon) {
                    this.$(".dropdown-toggle > i").attr('class',  "icon-" + selectedItem.icon);
                }
            }

            return this;
        },
        // NOTE: the "data-item-value" attribute of each list item is exposed for testing only
        subTemplate: '\
            <% if (_.isArray(item)) { %>\
                <% if (index > 0) { %> <li role="presentation" class="divider"></li> <% } %>\
                <% _.each(item, function(subItem, subIndex, list) { %>\
                    <%= subTemplate({item: subItem, index: subIndex, options: options, subTemplate: subTemplate}) %>\
                <% }); %>\
            <% } else if (item.children) { %>\
                <li class="dropdown-submenu">\
                    <a href="#" class="submenu-label"><%- options.formatLabel(item) %></a>\
                    <ul class="dropdown-menu">\
                        <% _.each(item.children, function(subItem, subIndex, list) { %>\
                            <%= subTemplate({item: subItem, index: subIndex, options: options, subTemplate: subTemplate}) %>\
                        <% }); %>\
                    </ul>\
                </li>\
            <% } else if (!_.has(item, "value")) { %>\
                <li role="presentation" class="dropdown-header"><%- options.formatLabel(item) %></li>\
            <% } else { %>\
                <li><a class="synthetic-select <%- item.enabled === false ? \"disabled\" : \"\"%>" href="#" data-item-idx="<%- item[options.expando] %>" data-item-value="<%- item.value %>">\
                    <i class="icon-check"></i>\
                    <% if (item.icon) { %> <i class="icon-<%-item.icon%> <%-options.iconClassName %>"></i><% } %>\
                    <% if (item.iconURL) { %> <img class="<%-options.iconURLClassName %>" src="<%-item.iconURL%>" alt="icon"><% } %>\
                    <span class="link-label"><%- options.formatLabel(item) %></span>\
                    <% if (item.description && (options.descriptionPosition == "right")) { %> <span class="link-description"><%- item.description %></span><% } %>\
                    <% if (item.description && (options.descriptionPosition == "bottom")) { %> <span class="link-description-below"><%- item.description %></span><% } %>\
                </a></li>\
            <% } %>\
        ',
        template: '\
            <a class="dropdown-toggle <%- options.toggleClassName %>" href="#">\
                <i class="<%- (selectedItem && selectedItem.icon) ? "icon-" + selectedItem.icon : ""%> icon-large"></i>\
                <span class="link-label"><%- options.label %> <%- selectedItem && options.formatLabel(selectedItem) %></span><span class="caret"></span>\
            </a>\
            <div class="dropdown-menu dropdown-menu-selectable <%- options.menuClassName %> <%- menuWidthClass %> ">\
                <div class="arrow"></div>\
                <ul class="dropdown-menu-main">\
                <% _.each(items, function(item, index, list) { %>\
                <%= subTemplate({item: item, index: index, options: options, subTemplate: subTemplate}) %>\
                <% }); %>\
                </ul>\
            </div>\
        '
    });
});

define('views/shared/controls/SyntheticRadioControl',
        [
            'jquery',
            'underscore',
            'module',
            'views/shared/controls/Control',
            'util/general_utils',
            'bootstrap.tooltip'
        ],
        function(
            $,
            _,
            module,
            Control,
            util
            // bootstrap tooltip
            ) {
    /**
     * Synthetic Radio Button Bar a-la iPhone
     *
     * @param {Object} options
     *                        {Object} model The model to operate on
     *                        {String} modelAttribute The attribute on the model to observe and update on selection
     *                        {Object} items An array of one-level deep data structures:
     *                                      label (textual display),
     *                                      value (value to store in model)
     *                                      icon (icon name to show in menu and button label)
     *                                      className (class attribute to be applied),
     *                                      tooltip  (Text to display in the tooltip)
     *                                 (ie, {label: 'Foo Bar', value: 'foo', icon: 'bar', className: 'foo'}).
     *                        {String} buttonClassName (Optional) Class attribute to each button element. Default is btn.
     *                        {Boolean} elastic Automatically assigns percentage width to children to completely fill the parent. Defaults to false.
     *                        {String} additionalClassNames (Optional) Class attribute(s) to add to control
     */

    return Control.extend({
        className: 'control btn-group',
        moduleId: module.id,
        initialize: function(){
            var defaults = {
                buttonClassName: 'btn',
                elastic: false
            };

            _.defaults(this.options, defaults);


            // dunno if this should be the default but it was hard coded before so defaulting for legacy
            var itemDefaults = {iconSize: 'icon-large'};
            // attempt to default iconSize for each item
            _.each(this.options.items, function(el, i, list){
                _.defaults(el, itemDefaults);
            });
            
            if (this.options.modelAttribute) {
                this.$el.attr('data-name', this.options.modelAttribute);
            }
            this.$el.addClass('btn-group-radio');
            Control.prototype.initialize.call(this, this.options);
            
        },
        events: {
            'click button': function(e) {
                if (this.options.enabled)
                    this.setValue( $(e.currentTarget).data('value'), true);
                e.preventDefault();
            }

        },
        disable: function(){
            this.options.enabled = false;
            this.$('button').addClass('disabled');
        },
        enable: function(){
            this.options.enabled = true;
            this.$('button').removeClass('disabled');
        },
        render: function(){
            if (!this.el.innerHTML) {
                var template = _.template(this.template, {
                                items: this.options.items,
                                buttonClassName: this.options.buttonClassName,
                                help: this.options.help,
                                elastic: this.options.elastic,
                                enabled: this.options.enabled,
                                modelAttribute: this.options.modelAttribute
                        });
                this.$el.html(template);
                this.$('[rel="tooltip"]').tooltip({animation:false, container: 'body'});

                //bind that values to the items
                var items = this.options.items;
                this.$el.find('button').each(function(i, el) {
                    $(el).attr('data-value', items[i].value).data('value', items[i].value);
                });
            }

            var value = this._value;

            this.$el.find('button').each(function(i, el) {
                var $el = $(el);
                $el[util.checkEquality($el.data('value'), value) ? 'addClass' : 'removeClass']('active');
            });

            var additionalClassNames = this.options.additionalClassNames;
            if(additionalClassNames) {
                this.$el.addClass(additionalClassNames);
            }

            return this;
        },
        remove: function(){
            this.$('[rel="tooltip"]').tooltip('destroy');
            Control.prototype.remove.call(this);
        },
        template: '\
            <% _.each(items, function(item, index){ %>\
                <button name="<%- modelAttribute || "" %>" \
                        <% if (elastic) { %> style="width:<%- Math.round(100*(1/items.length)) %>%" <% } %> \
                        <% if (item.tooltip) { %> rel="tooltip" title="<%=item.tooltip%>" <% } %>\
                        class="<%=buttonClassName%> <%- item.className || "" %> <%- enabled ? "" : "disabled" %>">\
                    <% if (item.icon) { %> <i class="icon-<%-item.icon%> <%-item.iconSize%>"></i><% } %>\
                    <% if(item.label){ %> <%= item.label%> <%}%>\
                </button>\
            <% }) %>\
        '
    });
});

define('views/shared/controls/SyntheticCheckboxControl', [
    'underscore',
    'module',
    'views/shared/controls/Control',
    'splunk.util'
], function(
    _,
    module,
    Control,
    splunk_util
    )
{
    /**
     * Synthetic Checkbox
     *
     * @param {Object} options
     *                        {Object} model The model to operate on
     *                        {String} modelAttribute The attribute on the model to observe and update on selection
     *                        {Object} items An array of one-level deep data structures:
     *                                 label (textual display),
     *                                      value (value to store in model)
     *                                      icon (icon name to show in menu and button label)
     *                                 (ie, {label: 'Foo Bar', value: 'foo', icon: 'bar'}).
     *                        {Boolean} invertValue (Optional) If true, then a checked checkBox has a value of false and
     *                                  an unchecked has a value of true. This is useful for model attributes that denote a negative
     *                                  (ex. disabled). Defaults to false.
     *                        {String} checkboxClassName (Optional) Class attribute to the button element. Default is btn.
     *                        {String} additionalClassNames (Optional) Class attribute(s) to add to control
     */
    var SyntheticCheckboxControl = Control.extend({
        className: 'control',
        moduleId: module.id,
        initialize: function(){
            var defaults = {
                checkboxClassName: 'btn',
                defaultValue: false,
                label: ''
            };

            _.defaults(this.options, defaults);
            
            if (this.options.modelAttribute) {
                this.$el.attr('data-name', this.options.modelAttribute);
            }
            
            Control.prototype.initialize.apply(this, arguments);
        },
        events: {
            'click label': function(e) {
                !this.options.enabled || this.setValue(!this._value);
                e.preventDefault();
            },
            'click .btn': function(e) {
                e.preventDefault();
            }

        },
        disable: function(){
            this.options.enabled = false;
            this.$('label').addClass('disabled');
            this.$('.btn').addClass('disabled');
        },
        enable: function(){
            this.options.enabled = true;
            this.$('label').removeClass('disabled');
            this.$('.btn').removeClass('disabled');
        },
        normalizeValue: function(value) {
            return splunk_util.normalizeBoolean(value) ? 1 : 0;
        },
        render: function(){
            var checked = this.options.invertValue ? !this.getValue() : this.getValue();

            if (!this.el.innerHTML) {
                var template = _.template(this.template, {
                                options: this.options,
                                checked: checked
                        });
                this.$el.html(template);
                
                if (!this.options.enabled) {
                    this.disable();
                }
            } else {
                this.$('.icon-check')[checked ? 'show' : 'hide']();
            }

            var additionalClassNames = this.options.additionalClassNames;
            if(additionalClassNames) {
                this.$el.addClass(additionalClassNames); 
            }

            return this;
        },
        template: '\
            <label class="checkbox">\
                  <a href="#" data-name="<%- options.modelAttribute || "" %>" class="<%- options.checkboxClassName %>"><i class="icon-check" <% if (!checked) {%>style="display:none"<% } %>></i></a>\
                  <%- options.label%>\
            </label>\
        '
    });
    
    return SyntheticCheckboxControl;
});

define('views/shared/controls/TextareaControl',['underscore', 'module', 'views/shared/controls/Control', 'util/dom_utils'], function(_, module, Control, dom_utils) {
    /**
     * Textarea with Bootstrap markup
     *
     * @param {Object} options
     *                        {String} modelAttribute The attribute on the model to observe and update on selection
     *                        {Object} model The model to operate on
     *                        {String} textareaClassName (Optional) Class attribute for the textarea
     *                        {String} additionalClassNames (Optional) Class attribute(s) to add to control
     */
    var ENTER_KEY = 13;

    return Control.extend({
        moduleId: module.id,
        initialize: function() {
            var defaults = {
                    textareaClassName: '',
                    placeholder: '',
                    useSyntheticPlaceholder: false,
                    trimLeadingSpace: true,
                    trimTrailingSpace: true,
                    spellcheck: true
            };
            _.defaults(this.options, defaults);
           
            if (this.options.placeholder && !dom_utils.supportsNativePlaceholder()) {
                this.options.useSyntheticPlaceholder = true;
            }
            
            if (this.options.modelAttribute) {
                this.$el.attr('data-name', this.options.modelAttribute);
            }
            
            Control.prototype.initialize.apply(this, arguments);
        },
        events: {
            'change textarea': function(e) {
                if (this.options.enabled) {
                    var textareaValue = this.$('textarea').val();
                    if(this.options.trimLeadingSpace) {
                        textareaValue = textareaValue.replace(/^\s+/g, '');
                    }
                    if(this.options.trimTrailingSpace) {
                        textareaValue = textareaValue.replace(/\s+$/g, '');
                    }
                    this.setValue(textareaValue, false);
                    this.updatePlaceholder();
                }
            },
            'click .placeholder': function(e) {
                if (this.options.enabled)
                    this.$textarea.focus();
            },
            'keyup textarea': function(e) {
                this.updatePlaceholder();
            },
            'mouseup textarea': function(e) { //could result in pasted text
                this.updatePlaceholder();
            },
            'keypress textarea': function(e) {
                // Eat the Enter event since the textarea input handles this event. Ideally we'd call preventDefault
                // and listen for defaultPrevented, but this isn't
                if (e.which == ENTER_KEY) {
                    e.stopPropagation();
                }
            }
        },
        updatePlaceholder: function() {
           if (this.options.useSyntheticPlaceholder) {
                this.$placeholder[this.$textarea.val() === '' ? 'show' : 'hide']();
           }
        },
        disable: function(){
            this.options.enabled = false;
            this.$textarea.hide();
            this.$disabledTextarea.show();
        },
        enable: function(){
            this.options.enabled = true;
            this.$textarea.show();
            this.$disabledTextarea.hide();
        },
        render: function() {
            if (!this.el.innerHTML) {
                var template = _.template(this.template, {
                        options: this.options,
                        value: this._value || ''
                    });
                this.$el.html(template);
                this.$textarea = this.$('textarea');
                this.$disabledTextarea = this.$('.uneditable-input');
                if (this.options.useSyntheticPlaceholder) {
                    this.$placeholder = this.$('.placeholder');
                }
            } else {
                this.$textarea.val(this._value);
                this.$disabledTextarea.text(this._value);
            }
            this.updatePlaceholder();
            
            var additionalClassNames = this.options.additionalClassNames;
            if(additionalClassNames) {
                this.$el.addClass(additionalClassNames);
            }

            return this;
        },
        template: '\
            <span class="uneditable-input uneditable-input-multiline \
                        <%= options.textareaClassName %>" \
                    <% if(options.enabled){ %>\
                        style="display:none"\
                    <%}%>>\
                <%- value %>\
            </span>\
            <textarea type="text" \
                        name="<%- options.modelAttribute || "" %>" \
                        class="<%= options.textareaClassName %>" \
                        <% if(options.placeholder && !options.useSyntheticPlaceholder){ %>\
                           placeholder="<%- options.placeholder %>"\
                        <%}%> \
                        <% if(!options.enabled){ %>\
                            style="display:none"\
                        <%}%>\
                        <% if(!options.spellcheck){ %>\
                            spellcheck="false"\
                        <%}%>\
            ><%- value %></textarea>\
            <% if (options.useSyntheticPlaceholder) { %> \
                <span class="placeholder"><%- options.placeholder %></span>\
            <% } %>\
        '
    });
});

define('views/shared/controls/LabelControl',['underscore', 'module', 'views/shared/controls/Control', 'bootstrap.tooltip'], function(_, module, Control /*tooltip*/) {
    /**
     * Text Input with Bootstrap markup
     *
     * @param {Object} options
     *                        {String} modelAttribute The attribute on the model to observe
     *                        {Object} model The model to observe
     *                        {String} inputClassName (Optional) Class attribute for the input
     *                        {Boolean} multiline (Optional) if enabled, behaves more like a textarea: scroll and pre-wrap
     *                        {Boolean} breakword (Optional) If true, words will break. Good for uris.
     *                        {Object} defaultValue (Optional) If the modelAttribute in the model is undefined, then
     *                                 use this value to populate the input
     *                        {String} additionalClassNames (Optional) Class attribute(s) to add to control
     *                        {String} tooltip (Optional) Text to display in the tooltip.
     */

    return Control.extend({
        moduleId: module.id,
        initialize: function() {
            var defaults = {
                inputClassName: 'input-label',
                defaultValue: 'label',
                iconSize: 'icon-large'
            };
         
            _.defaults(this.options, defaults);

            Control.prototype.initialize.apply(this, arguments);

        },
        render: function() {
            var value = this._value || '';
            
            if (!this.el.innerHTML) {
                var template = _.template(this.template, {
                        options: this.options,
                        value: value
                    });

                this.$el.html(template);
                this.$span = this.$('span');
            } else {
                this.$span.text(value);
            }

            var additionalClassNames = this.options.additionalClassNames;
            if(additionalClassNames) {
                this.$el.addClass(additionalClassNames); 
            }
            
            if (this.options.modelAttribute) {
                this.$el.attr('data-name', this.options.modelAttribute);
            }

            if (this.options.tooltip) {
                this.$('.tooltip-text').tooltip({animation:false, title: this.options.tooltip});
            }
            return this;
        },
        // Using concatenation for constructing the template so extra whitespace isn't introduced into the span.
        // This allows the use of preformatted text styling on the span.
        template: '' +
            '<span class="<%= options.inputClassName %><% if(options.tooltip) { %> tooltip-text<% } %>">' +
                '<% if (options.icon) { %> <i class="icon-<%-options.icon%> <%-options.iconSize%>"></i> <% } %>' +
                '<%- value %>' +
            '</span>'
    });
});

define('views/shared/controls/SyntheticSliderControl',
    [
        'jquery',
        'underscore',
        'module',
        'views/shared/controls/Control',
        'util/general_utils',
        'util/keyboard'
    ],
    function($, _, module, Control, util, keyboard) {
        /**
         * Synthetic Slider Control
         *
         * @param   {Object} options
         *            |
         *            |--> {Object} model               [Required] The model to operate on
         *            |--> {String} modelAttribute      [Required] The attribute on the model to observe and update on selection
         *            |
         *            |--> {Number} min                 [Optional: Defaults to 1] The minimum value selectable on the slider
         *            |--> {Number} max                 [Optional: Defaults to 5] The maximum value selectable on the slider
         *            |--> {Number} step                [Optional: Defaults to 1] The step interval amount between selectable values on the slider
         *            |
         *            |--> {Array} steps                [Optional: Default is unused] Allows every step of the slider to be explicitly defined. If
         *            |                                 this option is defined, it will override any previously defined min, max, and step settings.
         *            |
         *            |                                     Each array element should be one of the following:
         *            |
         *            |                                         (a) {String/Number} If the element is a string or number, the element will be used
         *            |                                         as both he slider step's label and value.
         *            |
         *            |                                             - OR -
         *            |
         *            |                                         (b) {Object: of format { label: String, value: String/Number } } If the element is
         *            |                                         of the above object format, the element's label property will be used
         *            |                                         as the slider step's label and the element's value property will be used as the
         *            |                                         slider's value.
         *            |                             
         *            |                                 Example: ['Low', 'Medium', { label: 'High', value: 10000 }]
         *            |--> {Boolean} enableStepLabels   [Optional: Defaults to true] If a steps array (as defined above) is provided, this
         *            |                                 determines whether or not to display step labels in the UI as the slider is moved.
         *            |
         *            |--> {Number} value               [Optional: Defaults to min] The default value selected on the slider
         *            |--> {Number} width               [Optional: Defaults to 256] The width of the slider bar in pixels
         *            |--> {Number} minLabel            [Optional] The text label displayed next to the minimum slider end
         *            |--> {Number} maxLabel            [Optional] The text label displayed next to the maximum slider end
         */
        return Control.extend({
            className: 'control slider-control',
            moduleId: module.id,
            initialize: function() {
                var defaults = {
                    min: 1,
                    max: 5,
                    step: 1,
                    steps: [],
                    enableStepLabels: true,
                    width: 256
                };
                _.defaults(this.options, defaults);
                this.setSyntheticSteps();
                this.syncFromModel();
                this.setStyles();
                Control.prototype.initialize.call(this, this.options);
            },
            syntheticStepsMode: false,
            value: 0,
            selected: false,
            notches: 0,
            styles: {
                slider: {
                    height: 0
                },
                sliderBar: {
                    width: 0,
                    height: 0,
                    top: 0,
                    borderRadius: 0
                },
                sliderHandle: {
                    width: 0,
                    height: 0,
                    top: 0,
                    left: 0,
                    borderRadius: 0
                },
                sliderNotch: {
                    height: 0,
                    marginLeftRight: 0
                }
            },
            setSyntheticSteps: function() {
                if (this.options.steps.length) {
                    this.syntheticStepsMode = true;
                    this.options.min = 0;
                    this.options.max = this.options.steps.length - 1;
                    this.options.step = 1;
                    if (this.options.value !== undefined) {
                        this.options.value = this.syntheticToInternalValue(this.options.value);
                    }
                }
            },
            syntheticValueAt: function(index) {
                var syntheticStep = this.options.steps[index];
                return syntheticStep.value || syntheticStep;
            },
            syntheticLabelAt: function(index) {
                var syntheticStep = this.options.steps[index];
                return syntheticStep.label || syntheticStep;
            },
            syntheticToInternalValue: function(syntheticValue) {
                var numSteps = this.options.steps.length;
                for (var i = 0; i < numSteps; i++) {
                    if (this.syntheticValueAt(i) == syntheticValue) {
                        return i;
                    }
                }
                return 0;
            },
            setStyles: function() {
                this.styles.slider.height = this.options.width / 5;        
                // Set bar styles
                this.styles.sliderBar.width = this.options.width;
                this.styles.sliderBar.height = this.styles.sliderBar.width / 50;
                this.styles.sliderBar.top = (this.styles.slider.height - this.styles.sliderBar.height) / 2;
                this.styles.sliderBar.borderRadius = this.styles.sliderBar.height / 2;
                // Set handle styles
                this.styles.sliderHandle.width = this.styles.sliderBar.width / 20;
                this.styles.sliderHandle.height = this.styles.sliderHandle.width;
                this.styles.sliderHandle.top = (this.styles.slider.height - this.styles.sliderHandle.height) / 2 - 1;
                this.styles.sliderHandle.borderRadius = this.styles.sliderHandle.width / 2 + 1;
                // Set notch styles
                this.notches = Math.round((this.options.max - this.options.min) / this.options.step) + 1;
                this.styles.sliderNotch.height = this.styles.sliderBar.height * 1.5;
                this.styles.sliderNotch.marginLeftRight = (this.styles.sliderBar.width - 2 * this.styles.sliderHandle.width) / (this.notches - 1);
            },
            activate: function() {
                Control.prototype.activate.apply(this, arguments);
                if (this.options.modelAttribute) {
                    this.syncFromModel();
                }
            },
            startListening: function() {
                Control.prototype.startListening.apply(this, arguments);
                if (this.options.modelAttribute) {
                    this.listenTo(this.options.model, 'change:' + this.options.modelAttribute, this.syncFromModel);
                }
            },
            events: {
                'mousedown .slider': function(e) {
                    this.select();
                    this.update(e);
                    $('body').addClass('text-highlight-disabled');

                    $(window).on('mousemove.slider', function(e) {
                        this.update(e);
                    }.bind(this));

                    $(window).on('mouseup.slider', function() {
                        this.deselect();
                        $('body').removeClass('text-highlight-disabled');
                        $(window).off('.slider');
                    }.bind(this));
                },
                'mousemove .slider': function(e) {
                    this.update(e);
                },
                'keydown .slider-handle': function(e) {
                    if (e.which == keyboard.KEYS.LEFT_ARROW || e.which == keyboard.KEYS.RIGHT_ARROW) {
                        e.preventDefault();
                        this.select();
                        var delta = e.which == keyboard.KEYS.LEFT_ARROW ? -this.options.step : this.options.step;
                        this.value = this.snapToValue(this.value + delta);
                        this.delayedDeselect();
                        this.render();
                        this.$el.find('.slider-handle').focus();
                    }
                }
            },
            delayedDeselect: _.debounce(function() {
                this.deselect();
            }, 750),
            syncFromModel: function() {
                var modelValue = this.options.model.get(this.options.modelAttribute);
                if (modelValue !== undefined) {
                    var oldValue = this.value;
                    this.value = this.syntheticStepsMode ? this.syntheticToInternalValue(modelValue) : this.snapToValue(parseFloat(modelValue));
                    if (oldValue != this.value) {
                        this.syncToModel();
                    }
                    this.render();
                } else {
                    this.value = this.options.value !== undefined ? this.snapToValue(this.options.value) : this.options.min;
                }
            },
            syncToModel: function() {
                var modelValue = this.options.model.get(this.options.modelAttribute),
                    newModelValue;
                if (this.syntheticStepsMode) {
                    newModelValue = this.syntheticValueAt(this.value);
                    if (modelValue === undefined || modelValue != newModelValue) {
                        this.setValue(newModelValue);
                    }
                } else {
                    newModelValue = this.value.toFixed(3);
                    if (modelValue === undefined || this.snapToValue(parseFloat(modelValue)).toFixed(3) != newModelValue) {
                        this.setValue(newModelValue);
                    }
                }
            },
            select: function() {
                this.selected = true;
                this.render();
            },
            deselect: function() {
                this.selected = false;
                this.syncToModel();
                this.render();
            },
            snapToValue: function(value) {
                var exactValue = Math.min(Math.max(value, this.options.min), this.options.max);
                return Math.round(exactValue / this.options.step) * this.options.step;
            },
            offsetToValue: function(offset) {
                var padding = this.styles.sliderHandle.width / 2;
                var range = this.styles.sliderBar.width - 4 * padding;
                var offsetInRange = offset - padding;
                var valueInRange = offsetInRange / range;
                var trueRange = this.options.max - this.options.min;
                var desiredValue = this.options.min + valueInRange * trueRange;
                return this.snapToValue(desiredValue);
            },
            valueToOffset: function(value) {
                var padding = this.styles.sliderHandle.width / 2;
                var range = this.styles.sliderBar.width - 4 * padding;
                var trueRange = this.options.max - this.options.min;
                var valueInRange = (value - this.options.min) / trueRange;
                return padding + valueInRange * range - 1;
            },
            update: function(e) {
                if (this.selected) {
                    var padding = this.styles.sliderHandle.width / 2;
                    var offset = e.clientX - this.$el.find('.slider-bar').offset().left - padding;
                    this.value = this.offsetToValue(offset);
                    this.render();
                }
            },
            render: function() {
                var currentLabel = this.syntheticStepsMode && this.options.enableStepLabels ? this.syntheticLabelAt(this.value) : undefined;
                this.styles.sliderHandle.left = this.valueToOffset(this.value);
                this.$el.html(_.template(this.template, {
                    currentLabel: currentLabel,
                    width: this.options.width,
                    notches: this.notches,
                    styles: this.styles,
                    sliderHandleClass: this.selected ? 'slider-handle-moving' : '',
                    sliderHandleTooltipClass: this.selected && currentLabel ? 'slider-handle-tooltip' : '',
                    minLabel: this.options.minLabel,
                    maxLabel: this.options.maxLabel
                }));
                return this;
            },
            template: '\
                <div class="slider-container">\
                    <% if (minLabel) { %>\
                        <div class="slider-min-label"><%= minLabel %></div>\
                    <% } %>\
                    <div class="slider" style="width:<%= styles.sliderBar.width %>px;height:<%= styles.slider.height %>px;">\
                        <div class="slider-bar" style="width:<%= styles.sliderBar.width %>px;height:<%= styles.sliderBar.height %>px;top:<%= styles.sliderBar.top %>px;border-radius:<%= styles.sliderBar.borderRadius %>px;">\
                            <% for (var i = 0; i < notches; i++) { %>\
                                <div class="slider-notch" style="height:<%= styles.sliderNotch.height %>px;left:<%= styles.sliderHandle.width + i * styles.sliderNotch.marginLeftRight %>px;"></div>\
                            <% } %>\
                        </div>\
                        <div class="slider-handle <%= sliderHandleTooltipClass %> <%= sliderHandleClass %>" tabindex="0" style="width:<%= styles.sliderHandle.width %>px;height:<%= styles.sliderHandle.height %>px;top:<%= styles.sliderHandle.top %>px;left:<%= styles.sliderHandle.left %>px;border-radius:<%= styles.sliderHandle.borderRadius %>px" data-label="<%- currentLabel %>"></div>\
                    </div>\
                    <% if (maxLabel) { %>\
                        <div class="slider-max-label"><%= maxLabel %></div>\
                    <% } %>\
                </div>\
            '
        });
    }
);

define('views/shared/controls/TextControl',['underscore', 'module', 'views/shared/controls/Control'], function(_, module, Control) {
    /**
     * Text Input with Bootstrap markup
     *
     * @param {Object} options
     *                        {String} modelAttribute The attribute on the model to observe and update on selection
     *                        {Object} model The model to operate on
     *                        {String} inputClassName (Optional) Class attribute for the input
     *                        {String} placeholder (Optional) Placeholder text to display in the input if browser supports
     *                        {String} useSyntheticPlaceholder (Optional) If true, use the placeholder value
     *                        {Object} defaultValue (Optional) If the modelAttribute in the model is undefined, then
     *                                 use this value to populate the text input
     *                        {String} additionalClassNames (Optional) Class attribute(s) to add to control
     *                        {Boolean} updateOnKeyUp (Optional) Update the model with the value on any key-up event
     *                        {Boolean} canClear (Optional) show a clear button
     *                        {Boolean} autocomplete (Optional) the input allows native autocomplete,
     *                        {String} elementId (Optional) Element id attribute(s) to add to control input
     *                        {Boolean} updateOnAutofill (Optional) Asyncronously updates the model based on the 
     *                                  value of the input element (autofill does not consistently emit an event). 
     *                                  Note this feature is dependant on the onAddedToDocument paradigm.
     *
     * TODO [JCS] Do we use prepend and append?
     */

    return Control.extend({
        moduleId: module.id,
        initialize: function() {
            var defaults = {
                inputClassName: '',
                placeholder: '',
                prepend: false,
                append: false,
                useSyntheticPlaceholder: false,
                trimLeadingSpace: true,
                trimTrailingSpace: true,
                password: false,
                updateOnKeyUp: false,
                canClear: false,
                autocomplete: false,
                elementId: '',
                updateOnAutofill: false
            };
            _.defaults(this.options, defaults);

            
            if (this.options.placeholder && !this.supportsNativePlaceholder()) {
                this.options.useSyntheticPlaceholder = true;
            }
            
            if (this.options.modelAttribute) {
                this.$el.attr('data-name', this.options.modelAttribute);
            }

            if (this.options.updateOnKeyUp) {
                this.listenTo(this, 'keyup', this.onInputChange);
            }

            Control.prototype.initialize.apply(this, arguments);
        },
        events: {
            'change input[type=password], input[type=text]': function(e) {
                this.updateClear();
                this.onInputChange();
            },
            'click .placeholder': function(e) {
                this.$input.focus();
            },
            'keyup input[type=password], input[type=text]': function(e) {
                this.updateClear();
                this.updatePlaceholder();
                this.trigger("keyup", e, this.$input.val());
            },

            'mouseup input[type=password], input[type=text]': function(e) { //could result in pasted text
                this.updateClear();
                this.updatePlaceholder();
            },
            'click a.control-clear': function(e) {
                e.preventDefault();
                this.clear();
                this.updateClear();
                this.updatePlaceholder();
            }
        },
        focus: function() {
            this.$("input").focus();
        },
        onInputChange: function() {
            var inputValue = this.$input.val();
            if(this.options.trimLeadingSpace) {
                inputValue = inputValue.replace(/^\s+/g, '');
            }
            if(this.options.trimTrailingSpace) {
                inputValue = inputValue.replace(/\s+$/g, '');
            }
            this.setValue(inputValue, false);
            this.updatePlaceholder();
        },

        updatePlaceholder: function() {
           if (this.options.useSyntheticPlaceholder)
               this.$placeholder[this.$input.val() === '' ? 'show' : 'hide']();
        },
        supportsNativePlaceholder: function() {
            return ('placeholder' in document.createElement('input'));
        },
        disable: function(){
            this.$input.hide();
            this.$disabledInput.css('display', 'inline-block');
        },
        enable: function(){
            this.$input.show();
            this.$disabledInput.hide();
        },
        updateClear: function(){
            this.$clear[this.$input.val() ? 'show' : 'hide']();
        },
        clear: function(){
            this.$input.val('');
            this.setValue('');
        },
        render: function() {
            if (!this.el.innerHTML) {
                var template = _.template(this.template, {
                        options: this.options,
                        value: this._value || ''
                    });

                this.$el.html(template);
                this.$input = this.$('input');
                this.$disabledInput = this.$('.uneditable-input');
                if (this.options.useSyntheticPlaceholder)
                    this.$placeholder = this.$('.placeholder');
                if (this.options.prepend)
                    this.$el.addClass('input-prepend').prepend(this.options.prepend);
                if (this.options.append)
                    this.$el.addClass('input-append').append(this.options.append);
            } else {
                this.$input.val(this._value);
                this.$disabledInput.text(this._value);
            }
            this.updatePlaceholder();

            var additionalClassNames = this.options.additionalClassNames;
            if(additionalClassNames) {
                this.$el.addClass(additionalClassNames);
            }

            this.$clear = this.$('.control-clear');

            return this;
        },
        onAddedToDocument: function() {
            Control.prototype.onAddedToDocument.apply(this, arguments);
            if (this.options.updateOnAutofill) {
                setTimeout(function() {//autofill does not emit events x-platform and when element is auto-filled by the browser it is added to the stack (not sync)
                    this.setValue(this.$input.val(), false);
                }.bind(this), 0);
            }
        },
        // TODO: the `for` control-controlCid needs to be hooked up to the input with same `id`
        template: '\
        <span class="uneditable-input <%= options.inputClassName %>" \
            <% if(options.enabled){ %>style="display:none"<%}%>><%- value %></span>\
        <input type="<% if (options.password){%>password<%}else{%>text<%}%>" \
               name="<%- options.modelAttribute || "" %>" \
               class="<%= options.inputClassName %>" \
               value="<%- value %>" \
               autocomplete="<% if (options.autocomplete){ %>on<% } else { %>off<% } %>" \
               <% if(options.elementId){ %>id="<%- options.elementId %>"<%}%> \
               <% if(options.placeholder && !options.useSyntheticPlaceholder){ %>placeholder="<%- options.placeholder %>"<%}%> \
               <% if(!options.enabled){ %>style="display:none"<%}%>>\
            <% if (options.useSyntheticPlaceholder) { %> <span class="placeholder"><%- options.placeholder %></span><% } %>\
            <% if (options.canClear) { %><a href="#" class="control-clear" style="display:none"><i class="icon-x-circle"></i></a><% } %>\
        '
    });
});

define('views/datapreview/sourceselector/Tree',
    [
        'underscore',
        'jquery',
        'module',
        'views/Base',
        'uri/route',
        'tree.jquery' //NO IMPORT
    ],
    function(
        _,
        $,
        module,
        Base,
        route
    ){
        return Base.extend({
            moduleId: module.id,
            className: 'treeview',
            initialize: function(options) {
                Base.prototype.initialize.apply(this, arguments);
                this.urlArgsOverride = this.options.urlArgsOverride || {};
            },
            events: {
                'tree.select': 'onTreeSelect'
            },
            onTreeSelect: function(event){
                if (event.node) {
                    this.trigger('select', event.node);
                }
            },
            render: function() {
                this.fileSystemUrl = this.options.explorerUrl;
                if (!this.fileSystemUrl){
                    return;
                }

                var tree = this.$el.tree({
                    dataUrl: function(node){
                        return this.treeUrl(node);
                    }.bind(this),
                    dataFilter: function(data){
                        var mapped = [];
                        _.each(data.entry, function(obj){
                            var loadOnDemand = false;
                            if (obj.content.hasSubNodes == true ||
                                !obj.content.hasOwnProperty('hasSubNodes') // some root nodes
                            ){
                                loadOnDemand = true;
                            }
                            mapped.push( {
                                id: obj.name,
                                label: obj.content.name || obj.name,
                                hasSubNodes: obj.content.hasSubNodes,
                                load_on_demand: loadOnDemand
                            });
                        });
                        return mapped;
                    },
                    onCanSelectNode: function(node) {
                        return true;
                    },
                    onLoadFailed: function(response) {
                        var msg = response.responseJSON.messages[0];
                        msg['html'] = msg['text'];
                        this.trigger('fetchError', msg);
                    }.bind(this)
                });

                tree.tree('loadDataFromUrl', this.treeUrl('/'), tree.children[0]);

                return this;
            },
            treeUrl: function(node) {
                node = node || {id:''};
                var urlArgs = {
                    output_mode: 'json',
                    count: 10000,
                    sort_key: ['hasSubNodes', 'name'],
                    sort_dir: 'desc',
                    sort_mode: 'num'
                };
                urlArgs = _.extend(urlArgs, this.urlArgsOverride);
                urlArgs = $.param(urlArgs).replace(/%5B%5D/g,'');
                return this.fileSystemUrl + '/' + encodeURIComponent(node.id || '') + '?' + urlArgs;
            }
        });
    }
);

define('contrib/text!views/datapreview/sourceselector/SourceSelector.html',[],function () { return '<div class="filepath selected-path">\n    <%=selectedPath%>\n</div>';});

define('views/datapreview/sourceselector/SourceSelector',
    [
        'underscore',
        'jquery',
        'module',
        'views/Base',
        'views/datapreview/sourceselector/Tree',
        'uri/route',
        'util/splunkd_utils',
        'contrib/text!views/datapreview/sourceselector/SourceSelector.html'
    ],
    function(
        _, 
        $,
        module, 
        Base,
        Tree,
        route,
        splunkd_utils,
        template
    ){
        return Base.extend({
            moduleId: module.id,
            template: template,
            initialize: function(options) {
                var self = this;
                this.explorerUrl = null;
                this.prompt = '';
                this.selectedNode = null;

                if (this.options.browserType == 'files') {
                    this.explorerUrl = route.fileExplorer(
                        this.model.application.get("root"),
                        this.model.application.get("locale"),
                        this.model.application.get("owner"),
                        this.model.application.get("app")
                    );
                    this.prompt = _('File path').t();
                } else if (this.options.browserType == 'registry') {
                    this.explorerUrl = route.regExplorer(
                        this.model.application.get("root"),
                        this.model.application.get("locale"),
                        this.model.application.get("owner"),
                        this.model.application.get("app")
                    );
                    this.prompt = _('Registry hive').t();
                } else if (this.options.browserType == 'ad') {
                    this.explorerUrl = route.adExplorer(
                        this.model.application.get("root"),
                        this.model.application.get("locale"),
                        this.model.application.get("owner"),
                        this.model.application.get("app")
                    );
                    this.prompt = _('Active Directory path').t();
                }

                Base.prototype.initialize.apply(this, arguments);

                if (this.model.classicUrl) {
                    this.model.classicUrl.on('change:file', function(){
                        this.render();
                    }.bind(this));
                }


                this.children.tree = new Tree({
                    collection: this.collection,
                    model: this.model,
                    deferreds: this.options.deferreds,
                    explorerUrl: this.explorerUrl,
                    urlArgsOverride: this.options.urlArgsOverride
                });

                this.children.tree.on('select', function(node){
                    self.selectedNode = node;
                    self.$('.selected-path').text(node.id);
                });

                this.children.tree.on('fetchError', function(message){
                    this.options.flashMsgHelper.addGeneralMessage('fetch-error', message);
                }, this);


            },

            getSelectedNode: function() {
                return this.selectedNode;
            },

            render: function() {
            	this.$el.html(this.compiledTemplate({
                      selectedPath : this.model.sourceModel.get('input'),
                      prompt: this.prompt
                  }));
                          
                this.$el.prepend(this.children.tree.render().el);
                return this;
            }

        });
    }
);

define('helpers/FlashMessagesHelper',
    [
        'jquery',
        'underscore',
        'models/shared/FlashMessage',
        'util/splunkd_utils'
    ],
    function($, _, FlashMessageModel, splunkDUtils) {

        /**
         * @param flashMessagesCollection - the collection to use as the target for messages
         * @param options {Object} {
         *     removeDuplicates {Boolean} - (optional) whether to dedup messages with the same content, defaults to false
         *     postProcess {Function} - (optional) a function to perform cleanup of the messages before they are placed in the collection
         *         the list of FlashMessage models will be passed as the only argument (duplicates will already be removed if enabled)
         *         the return value will become the contents of the FlashMessages collection
         * }
         * @constructor
         */

        var FlashMessagesHelper = function(flashMessagesCollection, options) {
            this.options = options || {};
            this.defaults = $.extend(true, {}, this.options);
            
            _.defaults(this.defaults, {
                removeDuplicates: true
            });
            
            if (_.isUndefined(flashMessagesCollection)) {
                throw "FlashMessagesHelper must be created with a valid FlashMessages collection";
            }
            this.flashMessagesCollection = flashMessagesCollection;
            // registered is a dictionary, indexed by model/collection cid
            // each entry is an object containing the model/collection itself,
            //      and (optionally) a whitelist of message types to broadcast
            this.registered = {};
            this.clientModelMessages = {};
            this.serverModelMessages = {};
            this.generalMessages = {};
        };

        FlashMessagesHelper.prototype = {
            // whitelist is optional, it should be a list of message types that should be broadcasted
            // if whitelist is omitted, all messages will be broadcasted
            register: function(obj, whitelist) {
                this.registered[obj.cid] = { instance: obj };
                if(whitelist) {
                    this.registered[obj.cid]['whitelist'] = _(whitelist).isArray() ? whitelist : [whitelist];
                }
                obj.on('serverValidated', _.debounce(this.serverValidationHandler, 0), this);
                obj.on("validated", _.debounce(this.clientValidationHandler, 0), this);
            },
            unregister: function(obj) {
                obj.off(null, null, this);
                delete this.clientModelMessages[obj.cid];
                delete this.serverModelMessages[obj.cid];
                delete this.registered[obj.cid];
            },
            destroy: function() {
                var registeredObjects = _(this.registered).pluck('instance');
                _(registeredObjects).each(function(obj) { this.unregister(obj); }, this);
            },
            clientValidationHandler: function(isValid, model, invalidAttrs) {
                this.clientModelMessages[model.cid] = invalidAttrs;
                this.updateFlashMessageCollection();
            },
            serverValidationHandler: function(isValid, model, messages) {
                this.serverModelMessages[model.cid] = messages;
                this.updateFlashMessageCollection();
            },
            /**
             * Manually add a message to the Flash Messages Collection. Use this if you want to display a message
             * that isn't associated with a model
             *
             * @param id - the caller must generate a unique ID for the message
             * @param message
             *              - type {string} The type of message (splunkDUtils.ERROR | splunkDUtils.WARNING | splunkDUtils.INFO)
             *              - html {string} The message text
             */
            addGeneralMessage: function(id, message) {
                this.generalMessages[id] = message;
                this.updateFlashMessageCollection();
            },
            /**
             * Remove a message from the Flash Messages Collection that was previously added via addMessage.
             *
             * @param id - the caller generated id used in the addMessage call
             */
            removeGeneralMessage: function(id) {
                if (_(this.generalMessages).has(id)) {
                    delete this.generalMessages[id];
                    this.updateFlashMessageCollection();
                }
            },

            /**
             * Returns the number of general messages
             *
             * @return {Number}
             */
            getGeneralMessagesSize: function() {
                return _(this.generalMessages).size();
            },

            /**
             * Returns the number of client validation messages
             *
             * @return {Number}
             */
            getClientModelMessagesSize: function() {
                return _(this.clientModelMessages).size();
            },

            /**
             * Returns the number of server messages
             *
             * @return {Number}
             */
            getServerModelMessagesSize: function() {
                return _(this.serverModelMessages).size();
            },

            /**
             * Update the flash message collection when the clientMessages, serverMessages or generalMessages change
             */
            updateFlashMessageCollection: function() {
                var clientMessages = [], serverMessages = [], generalMessages = [];
                _(this.clientModelMessages).each(function(clientMsgs) {
                    _(clientMsgs).each(function(msg) {
                        clientMessages.push(new FlashMessageModel({ type: splunkDUtils.ERROR, html: msg }));
                    }, this);
                }, this);
                _(this.serverModelMessages).each(function(serverMsgs, cid) {
                    var whitelist = this.registered[cid] ? this.registered[cid].whitelist : null;
                    _(serverMsgs).each(function(msg) {
                        if(!whitelist || _(whitelist).contains(msg.type)) {
                            serverMessages.push(new FlashMessageModel({
                                type: splunkDUtils.normalizeType(msg.type),
                                // make sure to HTML-escape here, since these messages are coming from the server
                                html: _.escape(msg.message)
                            }));
                        }
                    }, this);
                }, this);
                _(this.generalMessages).each(function(msg) {
                    generalMessages.push(new FlashMessageModel({
                        type: splunkDUtils.normalizeType(msg.type),
                        html: msg.html
                    }));
                }, this);
                var allMessages = _.union(clientMessages, serverMessages, generalMessages);
                if (this.defaults.removeDuplicates) {
                    allMessages = _(allMessages).uniq(function(msg) { return msg.get('html'); });
                }
                if (this.defaults.postProcess) {
                    allMessages = this.defaults.postProcess(allMessages);
                }
                this.flashMessagesCollection.reset(allMessages);
            }
        };
    return FlashMessagesHelper;
});

define('views/shared/FlashMessages',
    [
        'jquery',
        'underscore',
        'backbone',
        'views/Base', 
        'collections/shared/FlashMessages',
        'helpers/FlashMessagesHelper',
        'module'
    ], 
    function($, _, Backbone, Base, FlashMessagesCollection, FlashMessagesHelper,module) {
        return Base.extend({
            moduleId: module.id,
            className: 'alerts',
            collectionLength: 0,
           /**
            * @param {Object} options {
            *     model: {
            *         <name>: <model to be registered>
            *         ....
            *     },
            *     collection: {
            *         <name>: <collection to be registered>
            *         ....
            *     }
            * }
            */
            initialize: function(options){
                Base.prototype.initialize.call(this, options);
                this.flashMsgCollection = new FlashMessagesCollection();
                this.flashMsgHelper     = new FlashMessagesHelper(this.flashMsgCollection, this.options.helperOptions);
                
                this.activate({skipRender: true});

                // SPL-70327, put ourselves to sleep before the window unloads
                // this avoids rendering messages from the XHRs that are cancelled by the browser
                this.beforeUnloadHandler = _(function() { this.sleep(); }).bind(this);
                $(window).on('beforeunload', this.beforeUnloadHandler);
            },
            startListening: function() {
                this.listenTo(this.flashMsgCollection, 'add remove reset', this.onCollectionChange);
            },
            activate: function(options) {
                options = options || {};
                
                if (this.active) {
                    return Base.prototype.activate.apply(this, arguments);
                }
                
                if (this.model instanceof Backbone.Model){
                    this.flashMsgHelper.register(this.model, this.options.whitelist);
                    
                    //see if we already have errors in the model.error
                    if (this.model.error && this.model.error.get("messages")){
                        this.flashMsgHelper.serverValidationHandler(true, this.model, this.model.error.get("messages"));
                    }
                } else {
                    _(this.model).each(function(model, k) {
                        this.flashMsgHelper.register(model, this.options.whitelist);
                        
                        //see if we already have errors in the model.error
                        if (model.error && model.error.get("messages")){
                            this.flashMsgHelper.serverValidationHandler(true, model, model.error.get("messages"));
                        }
                    },this);
                }
                
                if (this.collection instanceof Backbone.Collection) {
                    this.flashMsgHelper.register(this.collection, this.options.whitelist);
                    this.collection.each(function(model){
                        this.flashMsgHelper.register(model, this.options.whitelist);
                    },this);
                } else {
                    _(this.collection).each(function(collection){
                        this.flashMsgHelper.register(collection, this.options.whitelist);
                        collection.each(function(model){
                            this.flashMsgHelper.register(model, this.options.whitelist);
                        },this);
                    },this);
                }
                
                if (!options.skipRender) {
                    this.render();
                }
                
                return Base.prototype.activate.apply(this, arguments);
            },
            deactivate: function(options) {
                if (!this.active) {
                    return Base.prototype.deactivate.apply(this, arguments);
                }
                this.flashMsgCollection.reset();
                Base.prototype.deactivate.apply(this, arguments);
                                
                if (this.model instanceof Backbone.Model){
                    this.flashMsgHelper.unregister(this.model);
                } else {
                    _(this.model).each(function(model, k) {
                        this.flashMsgHelper.unregister(model);
                    },this);
                }
                
                if (this.collection instanceof Backbone.Collection) {
                    this.flashMsgHelper.unregister(this.collection);
                    this.collection.each(function(model){
                        this.flashMsgHelper.unregister(model);
                    },this);
                } else {
                    _(this.collection).each(function(collection){
                        this.flashMsgHelper.unregister(collection);
                        collection.each(function(model){
                            this.flashMsgHelper.unregister(model);
                        },this);
                    },this);
                }
                return this;
            },
            /**
             * Listen to validation events from a given object
             *
             * @param obj - the object to listen on
             * @param whitelist (optional) - the array of event types to listen for. If ommitted, then we listen to all events
             */
            register: function(obj, whitelist) {
                this.flashMsgHelper.register(obj, whitelist);
            },
            /**
             * Stop listening to validation events from a given object
             *
             * @param obj - the object to stop listen to
             */
            unregister: function(obj) {
                this.flashMsgHelper.unregister(obj);
            },
            remove: function() {
                this.flashMsgHelper.destroy();
                $(window).off('beforeunload', this.beforeUnloadHandler);
                return Base.prototype.remove.apply(this, arguments);
            },
            onCollectionChange: function() {
                if (this.collectionLength != this.flashMsgCollection.length) {
                    // Trigger change if the number of messages has changed
                    this.trigger("change", this);
                    this.collectionLength = this.flashMsgCollection.length;
                }
                this.render();
            },
            render: function() {
                this.$el.empty();
                this.$el.append(this.compiledTemplate({ flashMessages: this.flashMsgCollection }));
                (!this.flashMsgCollection.length) ? this.$el.hide() : this.$el.show();
                return this;
            },
            template: '\
                <% flashMessages.each(function(flashMessage){ %> \
                    <div class="alert alert-<%- flashMessage.get("type") %>">\
                        <i class="icon-alert"></i>\
                        <%= flashMessage.get("html") %>\
                    </div>\
                <% }); %> \
            '
        });
    }
);

define('views/datapreview/sourceselector/Master',[
    'underscore',
    'jquery',
    'module',
    'views/Base',
    'views/datapreview/sourceselector/SourceSelector',
    'views/shared/FlashMessages'
],
function(
    _,
    $,
    module,
    Base,
    SourceSelector,
    FlashMessages
){	
    return Base.extend({
        moduleId: module.id,
        initialize: function(options) {
            Base.prototype.initialize.apply(this, arguments);

            this.children.flashMessages = new FlashMessages();

            this.children.selector = new SourceSelector({
                model: this.model,
                collection: this.collection,
                deferreds: this.options.deferreds,
                browserType: this.options.browserType,
                urlArgsOverride: this.options.urlArgsOverride,
                flashMsgHelper: this.children.flashMessages.flashMsgHelper
            });


        },
        getSelection: function(){
            return this.getSelectedNode().id;
        },
        getSelectedNode: function(){
            return this.children.selector.getSelectedNode();
        },
        render: function() {
            this.$el
                .append(this.children.selector.render().el)
                .append(this.children.flashMessages.render().el);
            
            return this;
        }
    });
});

define('views/datapreview/SourceSelectorDialog',[
    'jquery',
    'underscore',
    'backbone',
    'module',
    'views/datapreview/sourceselector/Master',
    'views/shared/Modal'
],
function(
    $,
    _,
    Backbone,
    module,
    SourceSelector,
    Modal
){
    return Modal.extend({
        moduleId: module.id,
        className: Modal.CLASS_NAME + " " + Modal.CLASS_MODAL_WIDE,
        initialize: function(options) {
            Modal.prototype.initialize.call(this, arguments);

            this.children.sourceSelector = new SourceSelector({
                model: this.model,
                collection: this.collection,
                deferreds: this.deferreds,
                browserType: this.options.browserType,
                urlArgsOverride: this.options.urlArgsOverride
            });
            
            this.$el.removeClass('fade');
        },
        events: $.extend({}, Modal.prototype.events, {
            'click .btn-primary': function(e) {
                this.hide();
                var selectedNode = this.children.sourceSelector.getSelectedNode();
                this.model.sourceModel.set({
                    input: selectedNode.id,
                    selectedNodeHasChildren: selectedNode.hasSubNodes
                });
            }
        }),
        render: function() {
            this.$el.html(Modal.TEMPLATE);
            this.$(Modal.BODY_SELECTOR).show();
            this.$(Modal.BODY_SELECTOR).removeClass('modal-body-scrolling').append(this.children.sourceSelector.render().el);
            this.$(Modal.HEADER_TITLE_SELECTOR).html(_("Select source").t());
            this.$(Modal.FOOTER_SELECTOR).append(Modal.BUTTON_CANCEL);
            this.$(Modal.FOOTER_SELECTOR).append('<a href="#" class="btn btn-primary modal-btn-primary">'+_('Select').t()+'</a>');
            return this;
        }
    });
});

/**
 * @author leo
 * @date 04/7/2014
 *
 * @description This component displays a textfield with a Browse button next to it, which brings
 * a popup with a treeview for selecting an item from some structure like filesystem.
 */

define('views/shared/controls/TextBrowseControl',
    [
        'underscore',
        'views/shared/controls/TextControl',
        'views/datapreview/SourceSelectorDialog',
        'module'
    ],
    function (
        _,
        TextControl,
        SourceSelectorDialog,
        module
        ) {

        return TextControl.extend({
            moduleId: module.id,
            className: 'control input-append',
            initialize: function () {
            
                 TextControl.prototype.initialize.apply(this, arguments);
                 
                 // showSourceSelectorDialog always sets input attribute, so need to remap to correct value;
                 this.options.model.on('change:input', function(val) {
                     this.options.model.set(this.options.modelAttribute, val.get('input') );
                 },this);
                 
            },
            events: $.extend({}, TextControl.prototype.events, {'click .btn' : 'showSourceSelectorDialog'}),
            showSourceSelectorDialog: function(){

                    if(!this.children.sourceSelectorDialog){
                        this.children.sourceSelectorDialog = new SourceSelectorDialog({
                            model: {
                                sourceModel: this.options.model,
                                application: this.options.applicationModel
                            },
                            browserType: this.options.browserType,
                            urlArgsOverride: this.options.urlArgsOverride
                        });
                        $('body').append(this.children.sourceSelectorDialog.render().el);
                    }
                    this.children.sourceSelectorDialog.show();

            },
            template: TextControl.prototype.template +
            '<button class="btn" id="browseBtn"><%= _("Browse").t() %></button>'
        });
    }
);
// footer nav
define('views/shared/controls/DateControl',[
    'jquery',
    'underscore',
    'backbone',
    'module',
    'views/shared/controls/Control',
    'jquery.ui.datepicker'
],
function(
    $,
    _,
    Backbone,
    module,
    Control,
    jqueryDatepicker
){
    /**
     * Synthetic Select drodpown a-la Bootstrap
     *
     * @param {Object} options
     *        {Object} model The DateTime model to operate on
     *        {String} inputClassName (Optional) Class attribute for the input
     *        {String} additionalClassNames (Optional) Class attribute(s) to add to control
     */

    return Control.extend({
        moduleId: module.id,
        tagName: 'span',
        className: 'pull-left',
        initialize: function() {
            var defaults = {
                    inputClassName: '',
                    help: 'MM/DD/YYYY'
            };
            _.defaults(this.options, defaults);
            Control.prototype.initialize.call(this, this.options);
        },
        startListening: function() {
            Control.prototype.startListening.apply(this, arguments);
            this.listenTo(this.model, 'change', this.render);
        },
        activate: function(options) {
            options = options || {};
            if (this.active) {
                return Control.prototype.activate.apply(this, arguments);
            }
            
            if (!options.skipSetValueRender) {
                this.render();
            }
            
            return Control.prototype.activate.apply(this, arguments);
        },
        events: {
            'change input[type=text]': function(e) {
                this.setValue(this.$('input').datepicker('getDate'), false);
            }
        },
        updateModel: function(options){
            options = options || {};
            var updateOptions = {
                validate: this.options.validate,
                forceUpdate: this.options.forceUpdate
            };

            $.extend(true, updateOptions, options);

            if (this.model) {
                return this.model.setMonDayYearFromJSDate(this._value, updateOptions);
            }
            return true;
        },

        render: function() {
            if (!this.el.innerHTML) {
                var template = _.template(this.template, {
                    options: this.options
                });

                this.$el.html(template);

                this.$('input').datepicker({
                    defaultDate: this.model.jsDate()
                });

                var additionalClassNames = this.options.additionalClassNames;
                if(additionalClassNames) {
                    this.$el.addClass(additionalClassNames);
                }
            }

            // SPL-70724, in IE setting the same date will cause the date picker dialog to open again
            // so we first check if they are equal
            var inputDate = this.$('input').datepicker('getDate'),
                modelDate = this.model.jsDate();

            if(!inputDate || inputDate.getTime() !== modelDate.getTime()) {
                this.$('input').datepicker('setDate',  modelDate);
            }
            this.$('input').blur();

            return this;
        },
        template: '\
            <input type="text" size="8" class="<%= options.inputClassName %>" value=""/>\
        '
    });
});

define('views/shared/controls/Accumulator',[
        'underscore',
        'module',
        'views/shared/controls/Control'
    ],
    function (
        _,
        module,
        Control
        ) {
        return Control.extend({
            moduleId: module.id,
            initialize: function() {
                var defaults = {
                    /*inputClassName: 'input-label',
                    defaultValue: 'label',
                    iconSize: 'icon-large'*/
                    itemName: _('item(s)').t()
                };

                _.defaults(this.options, defaults);

                Control.prototype.initialize.apply(this, arguments);
            },
            events: {
                'click a.addAllLink': function(e) {
                    this._addAll();
                    e.preventDefault();
                },
                'click a.removeAllLink': function(e) {
                    this._removeAll();
                    e.preventDefault();
                },
                'click .availableOptions li': function(e) {
                    var li = e.currentTarget;
                    this._addToSelected([li]);
                    e.preventDefault();
                },
                'click .selectedOptions li': function(e) {
                    var li = e.currentTarget;
                    this._removeFromSelected([li]);
                    e.preventDefault();
                }
            },

            render: function () {
                if (!this.el.innerHTML) {
                    this.$el.html(this.compiledTemplate({ options: this.options }));
                    this._populateAvailableOptions();
                    this._addPreSelected();
                    this.$('.availableOptionsHeader').text(_('Available ').t() + this.options.itemName);
                    this.$('.selectedOptionsHeader').text(_('Selected ').t() + this.options.itemName);
                }
                return this;
            },

            onInputChange: function() {
                // collect selected values and update the model
                var $selectedOptions = this.$('.selectedOptions li');
                var selectedValues = [];
                $.each($selectedOptions, function(ix, option) {
                    selectedValues.push($(option).data('id'));
                });
                this.setValue(selectedValues, false);

                // sort alphabetically
                $selectedOptions.sort(function(a,b){
                    var keyA = $(a).text();
                    var keyB = $(b).text();

                    if (keyA < keyB) return -1;
                    if (keyA > keyB) return 1;
                    return 0;
                });
                var selItemsUl = this.$('.selectedOptions');
                $.each($selectedOptions, function(i, li){
                    selItemsUl.append(li);
                });
            },

            _populateAvailableOptions: function(options) {
                var $availableOptions = this.$('.availableOptions');
                for (var i = 0, len = this.options.availableItems.length; i < len; i++) {
                    var item = this.options.availableItems[i],
                        iconClass = item['icon'],
                        $row = $('<div>').text(item['label']).append($('<span class="icon-class">').text(iconClass)),
                        opt = $('<li>').data({
                            id: item['value'],
                            label: item['label'],
                            'icon-class': iconClass
                        }).html($row);
                    $availableOptions.append(opt);
                }
            },

            _addToSelected: function(liArray) {
                var $selectedOptions = this.$('.selectedOptions');
                $.each(liArray, function(ix, li) {
                    if (!$(li).hasClass('selected')) {
                        var id = $(li).data('id'),
                            iconClass = $(li).data('icon-class'),
                            label = $(li).data('label'),
                            $row = $('<div>').text(label).append($('<span class="icon-class">').text(iconClass));
                        var newLi = $('<li>').data('id',id).html($row);
                        $selectedOptions.append(newLi);
                        $(li).addClass('selected');
                    } else {
                        this._removeFromSelected(liArray);
                    }
                }.bind(this));
                this.onInputChange();
            },
            _removeFromSelected: function(liArray) {
                $.each(liArray, function(ix, li) {
                    var id = $(li).data('id');
                    this.$('.availableOptions li').filter(function(ix,item) {
                        return $(item).data('id') == id;
                    }).removeClass("selected");
                    this.$('.selectedOptions li').filter(function(ix,item) {
                        return $(item).data('id') == id;
                    }).remove();
                }.bind(this));
                this.onInputChange();
            },
            _addAll: function() {
                var liArray = this.$('.availableOptions').find('li').not('.selected').get();
                this._addToSelected(liArray);
                this.onInputChange();
            },
            _removeAll: function() {
                this.$('.selectedOptions').find('li').remove();
                this.$('.availableOptions li.selected').removeClass("selected");
                this.onInputChange();
            },
            _addPreSelected: function() {
                if (this.options.selectedItems instanceof Array && this.options.selectedItems.length > 0) {
                    var liArray = this.$('.availableOptions li').filter(function(ix, item) {
                        return this.options.selectedItems.indexOf($(item).data('id')) > -1;
                    }.bind(this));
                    this._addToSelected(liArray);
                    this.onInputChange();
                }
            },
            remove: function () {
                return Control.prototype.remove.apply(this, arguments);
            },
            template: '\
                <style>\
                .accumulator {\
                    width: 700px;\
                }\
                .availableOptionsContainer, \
                .selectedOptionsContainer {\
                    float: left;\
                    width:300px;\
                    margin: 0px 20px 0px 0px;\
                }\
                .accumulator .addAllLink, \
                .accumulator .removeAllLink {\
                    float: right;\
                }\
                .accumulator .selectedOptionsContainer {\
                    margin-right:0px;\
                }\
                .accumulator h4 {\
                    font-weight:normal;\
                    font-size:12px;\
                    width:200px;\
                    margin-right:22px;\
                    padding-bottom:5px;\
                    line-height: 0px;\
                }\
                .accumulator h4.selectedOptionsHeader {\
                    width: 200px;\
                    margin-right:0px;\
                }\
                .accumulator ul.availableOptions,\
                .accumulator ul.selectedOptions {\
                    height:100px;\
                    overflow: auto;\
                    list-style: none none;\
                    margin: 0;\
                    padding:0;\
                    border: 1px solid #CCC;\
                    clear: left;\
                }\
                .accDisabled ul.availableOptions,\
                .accDisabled ul.selectedOptions {\
                    background-color: #f0f0f0;\
                }\
                .accumulator ul.availableOptions li,\
                .accumulator ul.selectedOptions li {\
                    clear:left;\
                    padding: 4px 5px;\
                    font-size:12px;\
                    cursor: pointer;\
                    line-height: 12px;\
                }\
                .accumulator ul.availableOptions li:hover,\
                .accumulator ul.selectedOptions li:hover {\
                    background-color: #e5f2f5;\
                }\
                .accumulator ul.availableOptions li span,\
                .accumulator ul.selectedOptions li span {\
                    margin:0px 8px 0 0;\
                    display: block;\
                    float: left;\
                }\
                .accumulator ul span.splIcon-arrow-e {\
                    background-color:  #73a550;\
                }\
                .accumulator ul.availableOptions li.selected,\
                .accDisabled ul li {\
                    color: #999;\
                }\
                .accumulator ul.availableOptions li.selected span.splIcon,\
                .accDisabled ul li span.splIcon {\
                    background-color:#CCC;\
                }\
                .accumulator .selected {\
                    background-color: transparent;\
                }\
                .accumulator .nofloat {\
                    clear: both;\
                }\
                .control-group.error ul {\
                    border-color: #d85d3c;\
                }\
                .accumulator .icon-class {\
                    color: #999999;\
                }\
                .accumulator div.wide {\
                    width: 340px;\
                }\
                </style>\
                \
                <div class="accumulator">\
                    <div class="availableOptionsContainer">\
                        <a class="addAllLink"><%= _("add all").t() %> &raquo;</a>\
                        <h4 class="availableOptionsHeader"></h4>\
                        <ul class="availableOptions"></ul>\
                    </div>\
                    \
                    <div class="selectedOptionsContainer">\
                        <a class="removeAllLink">&laquo; <%= _("remove all").t() %></a>\
                        <h4 class="selectedOptionsHeader"></h4>\
                        <ul class="selectedOptions"></ul> \
                    </div>\
                    <div class="nofloat"></div>\
                </div>'
        });
    });
// Control wrapper around a native input of type range
//
// NOTES:
// this has not been well tested


define('views/shared/controls/SliderControl',[
        'underscore',
        'module',
        'views/shared/controls/TextControl'
    ],
    function(
        _,
        module,
        TextControl
    ){
        return TextControl.extend({

            /**
            * @constructor
            * @param {Object} options {
            *     {String, required} modelAttribute The attribute on the model to observe and update on selection
            *     {Model, required} model The model to operate on
            *     {Number, optional} min The minimum allowed value. Default 0.
            *     {Number, optional} max The maximum allowed value. Default 100.
            *     {Number, optional} step The step to apply when buttons/arrow keys are used. Default 1.
            *     {String, optional} inputClassName A class name to apply to the input element
            * }
            */

            moduleId: module.id,
            initialize: function() {
                var defaults = {
                        min: '0',
                        max: '100',
                        step: '1',
                        inputClassName: '',
                        enabled: true
                };
                _.defaults(this.options, defaults);
                
                TextControl.prototype.initialize.apply(this, arguments);
            },
            events: _.extend({}, TextControl.prototype.events, {
                'change input[type=range]': function(e) {
                    this.setValue($(e.target).val(), false);
                }
            }),
            disable: function() {
                this.$('input').prop('disabled', true);
                TextControl.prototype.disable.call(this);
            },
            enable: function() {
                this.$('input').prop('disabled', false);
                TextControl.prototype.enable.call(this);
            },
            render: function() {
                if (!this.el.innerHTML) {
                    TextControl.prototype.render.call(this);
                    this.$el.prepend(_.template(this.sliderTemplate, {
                      options: this.options,
                      value: this._value
                    }));
                } else {
                    this.$('input').val(this._value);
                }
            },
            sliderTemplate: '\
                <input  class="slider <%= options.inputClassName %>" type="range" \
                min="<%- options.min %>" max="<%- options.max %>" \
                step="<%- options.step %>" value="<%- value %>" />\
            '
        });
    }
);
define('views/shared/controls/ControlGroup',
    [
        'jquery',
        'underscore',
        'module',
        'backbone',
        'views/Base',
        'views/shared/controls/Control',
        'views/shared/controls/SyntheticSelectControl',
        'views/shared/controls/SyntheticRadioControl',
        'views/shared/controls/SyntheticCheckboxControl',
        'views/shared/controls/SyntheticSliderControl',
        'views/shared/controls/TextControl',
        'views/shared/controls/TextareaControl',
        'views/shared/controls/TextBrowseControl',
        'views/shared/controls/DateControl',
        'views/shared/controls/LabelControl',
        'views/shared/controls/Accumulator',
        'views/shared/controls/SliderControl',
        'bootstrap.tooltip'
    ],
    function
    (
        $,
        _,
        module,
        Backbone,
        Base,
        Control,
        SyntheticSelectControl,
        SyntheticRadioControl,
        SyntheticCheckboxControl,
        SyntheticSliderControl,
        TextControl,
        TextareaControl,
        TextBrowseControl,
        DateControl,
        LabelControl,
        Accumulator,
        SliderControl
        //tooltip
    )
{
    /**
     * Wrapper around a Control that adds a label and an error state
     *
     * @param {Object} options
     *                        {String} controlType The attribute on the model to observe and update on selection
     *                        {Object} controlOptions dictionary passed to the control
     *                        {Object} controlOptions dictionary passed to the control
     *                        {Array or View} controls An array of dictionaries with types and options, and/or views, or a View
     *                        {String} label (Optional) the contents of the label tag
     *                        {Boolean} error (Optional) Whether or not the control group is in an error state
     *                        {Boolean} validate (Optional) Whether controls should use validation when setting model attributes,
     *                                                          defaults to false.
     *                        {Boolean} forceUpdate (Optional) Whether controls should force updates when setting model attributes,
     *                                                          defaults to false
     *                        {Boolean} enabled (Optional) Whether the control group should appear enabled, defaults to true
     *                        {Boolean} required (Optional) Whether the control group is required and should have a red asterisk,
     *                                                          defaults to false
     *                        {String} tooltip (Optional) Text to display in the tooltip.
     */

    return Base.extend({
        className: 'control-group',
        moduleId: module.id,
        initialize: function() {
            var defaults = {
                    label: '',
                    controls:[],
                    error: false,
                    _errorMsg : "",
                    controlClass: '',
                    enabled: true,
                    required: false
            };
            _.defaults(this.options, defaults);

            if (this.options.controls.length == 0) {
                this.options.controls = [{type: this.options.controlType, options: this.options.controlOptions}];
            }

            this.controlTypes= {
                    'SyntheticSelect': SyntheticSelectControl,
                    'SyntheticRadio': SyntheticRadioControl,
                    'SyntheticCheckbox': SyntheticCheckboxControl,
                    'SyntheticSlider': SyntheticSliderControl,
                    'Text': TextControl,
                    'Textarea': TextareaControl,
                    'TextBrowse': TextBrowseControl,
                    'Date': DateControl,
                    'Label': LabelControl,
                    'Accumulator': Accumulator,
                    'Slider': SliderControl
            };

            // Allow user to pass in there own control component.
            $.extend(true, this.controlTypes, this.options.controlTypes || {});

            Base.prototype.initialize.apply(this, arguments);

            if(!this.options.enabled) {
                this.$el.addClass('disabled');
            }

            // normalize controls to an array if it's a single item
            if(!_(this.options.controls).isArray()) {
                this.options.controls = [this.options.controls];
            }

            // create a list to hold the controls (in addition to the children dictionary) so that order can be preserved
            this.childList = [];

            _.each(this.options.controls, function(control, index) {
                // if the control is already a view, just add it to the children object
                if(control instanceof Backbone.View) {
                    this.children['child' + index] = control;
                    this.childList.push(control);
                }
                // otherwise construct a new view using the "type" and "options" fields
                else {
                    // allow a control to inherit the "validate", "forceUpdate" and "enabled" properties from the group options
                    var controlOptions = $.extend(
                        true,
                        {
                            validate: !!this.options.validate,
                            forceUpdate: !!this.options.forceUpdate,
                            enabled: !!this.options.enabled
                        },
                        control.options
                    );
                    var controlView = this.children['child' + index] = new this.controlTypes[control.type](controlOptions);
                    this.childList.push(controlView);
                }
            }, this);

            this.activate();
        },
        startListening: function() {
            var perAttrValidation = {};
            if(_.isArray(this.options.controls)){
                 _.each(this.options.controls, function(control, index) {
                    if(control.options && control.options.model){
                        perAttrValidation[control.options.modelAttribute] = true;
                        this.listenTo(control.options.model, 'attributeValidated:' + control.options.modelAttribute, function(isValid, key, error){
                            perAttrValidation[key] = isValid;
                            this.error(_.any(perAttrValidation, function(value) { return !value; }));
                        });
                    }
                 },this);
            }
            
            if(this.options.controlOptions && (this.options.controlOptions.model instanceof Backbone.Model)) {
                if(!_.isArray(this.options.controlOptions)) {
                    perAttrValidation[this.options.controlOptions.modelAttribute] = true;
                    this.listenTo(this.options.controlOptions.model, 'attributeValidated:' + this.options.controlOptions.modelAttribute, function(isValid, key, error){
                        perAttrValidation[key] = isValid;
                        this.error(_.any(perAttrValidation, function(value) { return !value; }));
                    });
                }
            }
        },
        events: {
            'click a.tooltip-link': function(e) {
                e.preventDefault();
            }
        },
        render: function() {
            if(!this.el.innerHTML) {
                var template = _.template(this.template, {
                    _: _,
                    controlCid: this.children.child0.cid,
                    label: this.options.label,
                    help: this.options.help,
                    controlClass: this.options.controlClass,
                    helpClass: this.options.helpClass,
                    tooltip: this.options.tooltip,
                    required: this.options.required
                });
                this.$el.html(template);

                if (this.options.tooltip) {
                    this.$('.tooltip-link').tooltip({animation:false, title: this.options.tooltip, container: 'body'});
                }

                _.each(this.childList, function(child, i) {
                    this.$('.controls').append(child.render().$el);
                    if(!i) { 
                        child.$el.attr('id', "control-" + this.children.child0.cid);
                    }
                }, this);
                if (this.options.help) {
                    this.$('.controls').append(this.$('.help-block')); //move the help block back to the end
                }
            }

            // TODO [JCS] How come this is coming from options and not from an internal variable set in error()?
            this.error(this.options.error);

            return this;
        },
        hide: function() {
            this.$el.hide();
        },
        show: function() {
            this.$el.show();
        },
        remove: function() {
            if(this.options.tooltip) {
                this.$('.tooltip-link').tooltip('destroy');
            }
            return Base.prototype.remove.apply(this, arguments);
        },
        enable: function() {
            this.$el.removeClass('disabled');
            _(this.getAllControls()).invoke('enable');
        },
        disable: function() {
            this.$el.addClass('disabled');
            _(this.getAllControls()).invoke('disable');
        },
        getModelAttributes: function() {
            var attrs = [];
            _.each(this.childList, function(child) {
                attrs.push(child.getModelAttribute());
            }, this);
            return attrs;
        },
        error: function(state, errorMsg) {
            state ? this.$el.addClass("error") : this.$el.removeClass("error");

            // Store the error message internally. For now, we aren't displaying it.
            this._errorMsg = errorMsg;
            return this;
        },
        getAllControls: function() {
            return _(this.children).filter(function(child) { return child instanceof Control; });
        },
        // TODO: the `for` control-controlCid needs to be hooked up to the input with same `id`
        template: '\
                <label class="control-label" for="control-<%- controlCid %>">\
                <%- label %>\
                <% if (tooltip) { %>\
                    <a href="#" class="tooltip-link"><%- _("?").t() %></a>\
                <% } %>\
                <% if (required) { %>\
                    <span class="required">*</span>\
                <% } %>\
                </label>\
                <div class="controls <%- controlClass %>">\
                <% if (help) { %> <span class="help-block <%- helpClass %>"><%= help %></span><% } %>\
                </div>\
                \
        '
    });
});

define('util/pdf_utils',['underscore', 'jquery', 'splunk.util', 'util/console', 'models/services/configs/AlertAction', 'models/search/Job', 'splunk.config'], function(_, $, splunkUtil, console, AlertAction, Job, splunkConfig) {

    var TYPE_PDFGEN = 'pdfgen', TYPE_DEPRECATED = 'deprecated';

    /**
     * Check the availibility of the PDF Generator
     * @return a jQuery Deferred object that is resolved when it has been determined what kind of PDF
     * rendering is available
     *       The deferred callbacks receive 2 arguments:
     *          - a boolean indicating the availability
     *          - the type (string) of the PDF server if available ("pdfgen" or "deprecated")
     */
    var isPdfServiceAvailable = _.once(function() {
        var dfd = $.Deferred();
        if (splunkConfig["PDFGEN_IS_AVAILABLE"]) {
            dfd.resolve(true, TYPE_PDFGEN);
        } else {
            $.getJSON(splunkUtil.make_url('report/is_enabled')).success(function(data) {
                if(data && data.status == 'enabled') {
                    dfd.resolve(true, TYPE_DEPRECATED);
                } else {
                    dfd.resolve(false, data && data.status);
                }
            }).error(function() {
                dfd.resolve(false);
            });
        }
        
        return dfd.promise();
    });

    /**
     * Get the email alert settings (models.AlertAction)
     * @returns a jQuery Deferred object that is resolved when the settings are loaded
     */
    var getEmailAlertSettings = _.once(function() {
        var dfd = $.Deferred();
        var emailAlertSettings = new AlertAction({ id: 'email' });

        emailAlertSettings.fetch().done(function() {
            dfd.resolve(emailAlertSettings);
        }).fail(function() {
                    dfd.reject();
                });

        return dfd.promise();
    });

    /**
     * Starts a search to send test email with the PDF version of a view via Email (using the sendemail command)
     * @param view (String) name of the dashboard
     * @param app (String) app of the dashboard
     * @param to (String) comma-separated list of recipients
     * @param options { paperSize: (String), paperOrientation: (String) }
     * @returns A jQuery Deferred object that is resolved once the search has successfully completed
     */
    function sendPDFEmail(view, app, to, options) {
        var dfd = $.Deferred();

        getEmailAlertSettings().done(function(emailSettings) {

            to = to.split(/[,\s]+/).join(',');

            var commandParams = {
                'server': emailSettings.getSetting('mailserver', 'localhost'),
                'use_ssl': emailSettings.getSetting('use_ssl', 'false'),
                'use_tls': emailSettings.getSetting('use_tls', 'false'),
                'to': to,
                'sendpdf': 'True',
                'from': emailSettings.getSetting('from', 'splunk@localhost'),
                'papersize': options.paperSize || 'a2',
                'paperorientation': options.paperOrientation || 'portrait',
                'pdfview': view
            };
            var searchString = '| sendemail ' + _(commandParams).map(function(v, k) {
                return [k, JSON.stringify(v)].join('=');
            }).join(' ');

            console.log('Starting search %o', searchString);

            var job = new Job();
            job.save({}, {
                data: {
                    search: searchString,
                    earliest_time: '0',
                    latest_time: 'now',
                    app: app,
                    namespace: app,
                    owner: splunkConfig.USERNAME,
                    ui_dispatch_app: app,
                    ui_dispatch_view: view,
                    preview: false
                }
            }).done(_.bind(function() {
                        job.startPolling();
                        job.entry.content.on('change:isDone', function(m, isDone) {
                            if(isDone) {
                                var messages = job.entry.content.get('messages');

                                _(messages).each(function(msg) {
                                    if(msg.type === 'ERROR') {
                                        console.error(msg.text);
                                    }
                                });

                                if(_(messages).any(function(msg) { return msg.type === 'ERROR'; })) {
                                    dfd.reject(_(messages).pluck('text')[0]);
                                } else {
                                    dfd.resolve();
                                }

                                _.defer(_.bind(job.destroy, job));
                            }
                        });
                        job.entry.content.on('change:isFailed', function(m, isFailed) {
                            if(isFailed) {
                                dfd.reject();
                            }
                            _.defer(_.bind(job.destroy, job));
                        });
                        job.on('error', function() {
                            dfd.reject('Error creating search job');
                            _.defer(_.bind(job.destroy, job));
                        });

                    }, this)).fail(_.bind(function() {
                        console.log('Search creation fail', arguments);
                    }, this));

        }).fail(function() {
                    dfd.reject();
                });

        return dfd.promise();
    }

    /** Workaround for SPL-67453 - double-encode certain XML characters */
    function encodeXMLForCustomEndpoint(xml) {
        return xml.replace(/&/g, '&amp;').replace(/%/g, '&#37;').replace(/</g, '&lt;').replace(/>/g, '&gt;');
    }

    var downloadReportFromXML = function(xml, app, params) {
        var baseURL = splunkUtil.make_full_url("splunkd/__raw/services/pdfgen/render");
        var form = $('<form method="POST" target="_blank"></form>').attr('action', baseURL);
        $('<input/>').attr({ type: 'hidden', name: 'input-dashboard-xml', value: encodeXMLForCustomEndpoint(xml) }).appendTo(form);
        $('<input/>').attr({ type: 'hidden', name: 'namespace', value: app }).appendTo(form);
        $('<input/>').attr({ type: 'hidden', name: 'splunk_form_key', value: splunkUtil.getFormKey() }).appendTo(form);
        if(params) {
            _.each(params, function(v, k) {
                $('<input/>').attr({ type: 'hidden', name: k, value: v }).appendTo(form);
            });
        }
        console.log('submitting form', form[0]);
        form.appendTo(document.body).submit();
        _.defer(function() {
            form.remove();
        });
    };

    /**
     * Generates the download URL for the PDF version of a dashboard|report
     * @param view the name of the dashboard|report
     * @param app the app the dashboard|report is defined in
     * @param params a object containing additional parameters for pdfgen (ignored for deprecated pdf server)
     * @param viewType dashboard|report. the type of the view defaults to dashboard
     * @returns a jQuery Deferred object that is resolved if the PDF Server is available. The first callback argument is
     * the download URL
     */
    var getRenderURL = function(view, app, params, viewType) {
        var dfd = $.Deferred();

        isPdfServiceAvailable().done(function(bool, type) {
            var inputType = viewType ? 'input-' + viewType : 'input-dashboard',
                    data = {
                        'namespace': app
                    };
            data[inputType] = view;

            if(type === TYPE_PDFGEN) {
                params = _.extend(
                        data,
                        params || {});
                dfd.resolve(splunkUtil.make_full_url("splunkd/__raw/services/pdfgen/render", params));
            } else if(type === TYPE_DEPRECATED) {
                dfd.resolve([splunkUtil.make_url('app', app, view), splunkUtil.propToQueryString({ output: 'pdf' })].join('?'));
            } else {
                dfd.reject();
            }
        }).fail(function() {
                    dfd.reject();
                });
        return dfd.promise();
    };

    return {
        isPdfServiceAvailable: isPdfServiceAvailable,
        getRenderURL: getRenderURL,
        downloadReportFromXML: downloadReportFromXML,
        getEmailAlertSettings: getEmailAlertSettings,
        sendTestEmail: sendPDFEmail
    };
});

define('views/shared/jobstatus/buttons/ExportResultsDialog',[
    'underscore',
    'models/Base',
    'module',
    'views/shared/Modal',
    'views/shared/controls/ControlGroup',
    'views/shared/FlashMessages',
    'uri/route',
    'util/pdf_utils',
    'util/math_utils'
    ],
    function(
        _,
        Base,
        module,
        Modal,
        ControlGroup,
        FlashMessagesV2,
        route,
        pdfUtils,
        mathUtils
    ) {
    return Modal.extend({
        moduleId: module.id,
         /**
         * @param {Object} options {
         *      model: {
         *          searchJob: <helpers.ModelProxy>,
         *          application: <models.Application>,
         *          report: <models.Report> (Only required for export to pdf. If passed in pdf will be a format option.)
         *      }
         * }
         */
        initialize: function(options) {
            Modal.prototype.initialize.apply(this, arguments);
            var Inmem = Base.extend({
                defaults: {
                    fileName: '',
                    format: 'csv',
                    limitResults: 'unlimited',
                    maxResults: 1000
                },
                validation: {
                    maxResults: {
                        fn: 'validateMaxResults'
                    },
                    format: {
                        fn: 'validateFormat'
                    }
                },
                validateMaxResults: function(value, attr, computedState){
                    if ((computedState.format === 'pdf' || computedState.limitResults === 'limit') &&
                        (!mathUtils.isInteger(value) || parseFloat(value) <= 0 )) {
                            return _('Max results must be an integer greater than 0').t();
                    }
                },
                validateFormat: function(value, attr, computedState){
                    if (computedState.format === 'pdf' && !(options.model.report && !options.model.report.isNew())) {
                        return _('Invalid format').t();
                    }
                }
            });

            this.model = {
                searchJob: this.model.searchJob,
                application: this.model.application,
                report: this.model.report,
                reportPristine: this.model.reportPristine,
                inmem: new Inmem()
            };

            this.children.flashMessage = new FlashMessagesV2({ model: this.model.inmem });

            this.deferredPdfAvailable = $.Deferred();
            this.deferredInitializeFormat = $.Deferred();

            this.usePanelType = options.usePanelType;
            this.resultTypeIsReport = this.usePanelType ? (this.model.report.entry.content.get('display.general.type') !== 'events') : this.model.searchJob.isReportSearch();
            
            if (this.model.report && !this.model.report.isNew()) {
                this.deferredPdfAvailable = pdfUtils.isPdfServiceAvailable();
            } else {
                this.deferredPdfAvailable.resolve(false);
            }

            $.when(this.deferredPdfAvailable).then(function(available) {
                var items = [
                    {
                        label: _('CSV').t(),
                        value: 'csv'
                    },
                    {
                        label: _('XML').t(),
                        value: 'xml'
                    },
                    {
                        label: _('JSON').t(),
                        value: 'json'
                    }
                ];

                if (available && (!this.model.reportPristine || !this.model.report.isDirty(this.model.reportPristine))) {
                    items.unshift({
                        label: _('PDF').t(),
                        value: 'pdf'
                    });
                }

                if (!this.resultTypeIsReport) {
                    items.unshift({
                        label: _('Raw Events').t(),
                        value: 'raw'
                    });
                }

                this.children.formatControl = new ControlGroup({
                    controlType: 'SyntheticSelect',
                    controlOptions: {
                        modelAttribute: 'format',
                        model: this.model.inmem,
                        items: items,
                        save: false,
                        toggleClassName: 'btn',
                        labelPosition: 'outside',
                        elastic: true,
                        popdownOptions: {
                            attachDialogTo: '.modal:visible',
                            scrollContainer: '.modal:visible .modal-body:visible'
                        }
                    },
                    label: _('Format').t()
                });

                this.deferredInitializeFormat.resolve();
            }.bind(this));

            this.children.filenameControl = new ControlGroup({
                controlType: 'Text',
                controlOptions: {
                    modelAttribute: 'fileName',
                    model: this.model.inmem,
                    placeholder: 'optional'
                },
                label: _('File Name').t()
            });

            this.children.limitResultsControl = new ControlGroup({
                controlType: 'SyntheticRadio',
                controlOptions: {
                    modelAttribute: 'limitResults',
                    model: this.model.inmem,
                    items: [
                        {
                            label: _('Unlimited').t(),
                            value: 'unlimited'
                        },
                        {
                            label: _('Limited').t(),
                            value: 'limit'
                        }
                    ],
                    save: false,
                    toggleClassName: 'btn',
                    labelPosition: 'outside',
                    elastic: true
                },
                label: _('Number of Results').t()
            });

            this.children.maxResultsControl = new ControlGroup({
                controlType: 'Text',
                controlOptions: {
                    modelAttribute: 'maxResults',
                    model: this.model.inmem
                },
                label: _('Max Results').t()
            });

            this.model.inmem.on('change:limitResults', this.toggleMaxResults, this);

            this.model.inmem.on('change:format', this.toggleByFormat, this);

            this.model.inmem.on('validated', function(isValid, model, invalidResults){
                if(isValid) {
                    var format = this.model.inmem.get('format'),
                        maxResults = this.model.inmem.get('maxResults'),
                        count = (this.model.inmem.get('limitResults') === 'limit') ? maxResults : 0;
                    if (format === 'pdf') {
                        var orientationSuffix = '',
                            orientation = this.model.report.entry.content.get('action.email.reportPaperOrientation'),
                            pageSize = this.model.report.entry.content.get('action.email.reportPaperSize') || 'a2';
                        if(orientation === 'landscape') {
                            orientationSuffix = '-landscape';
                        }
                        pdfUtils.getRenderURL(
                            this.model.report.entry.get('name'),
                            this.model.report.entry.acl.get('app'),
                            {
                                'sid': this.model.searchJob.id,
                                'paper-size': pageSize + orientationSuffix,
                                'max-rows-per-table': maxResults
                            },
                            'report'
                        ).done(function(url){
                            this.hide();
                            window.open(url);
                        }.bind(this));
                    } else {
                        window.location.href = route.exportUrl(this.model.application.get("root"), this.model.application.get("locale"), this.model.searchJob.get('id'),
                            this.model.inmem.get('fileName'), this.model.inmem.get('format'), this.model.inmem.get('limitResults'),
                            maxResults, count, this.resultTypeIsReport);
                        this.hide();
                    }
                }
            },this);
        },
        events: $.extend({}, Modal.prototype.events, {
            'click .btn-primary': function(e) {
                this.model.inmem.validate();
                e.preventDefault();
            }
        }),
        toggleByFormat: function() {
            if(this.model.inmem.get('format') === 'pdf') {
                this.children.filenameControl.$el.hide();
                this.children.limitResultsControl.$el.hide();
                this.children.maxResultsControl.$el.show();
            } else {
                this.children.filenameControl.$el.show();
                this.children.limitResultsControl.$el.show();
                this.toggleMaxResults();
            }
        },
        toggleMaxResults: function() {
            if(this.model.inmem.get('limitResults') === 'unlimited') {
                this.children.maxResultsControl.$el.hide();
            } else {
                this.children.maxResultsControl.$el.show();
            }
        },
        render : function() {
            $.when(this.deferredInitializeFormat).then(function() {
                this.$el.html(Modal.TEMPLATE);

                this.$(Modal.HEADER_TITLE_SELECTOR).html(_("Export Results").t());

                this.children.flashMessage.render().prependTo(this.$(Modal.BODY_SELECTOR));

                this.$(Modal.BODY_SELECTOR).append(Modal.FORM_HORIZONTAL_JUSTIFIED);

                this.children.formatControl.render().appendTo(this.$(Modal.BODY_FORM_SELECTOR));
                this.children.filenameControl.render().appendTo(this.$(Modal.BODY_FORM_SELECTOR));
                this.children.limitResultsControl.render().appendTo(this.$(Modal.BODY_FORM_SELECTOR));
                this.children.maxResultsControl.render().appendTo(this.$(Modal.BODY_FORM_SELECTOR));

                this.toggleByFormat();
                this.toggleMaxResults();

                this.$(Modal.FOOTER_SELECTOR).append(Modal.BUTTON_CANCEL);
                this.$(Modal.FOOTER_SELECTOR).append('<a href="#" class="btn btn-primary modal-btn-primary">' + _("Export").t() + '</a>');

                return this;
            }.bind(this));
        }
    });
});

/**
 *   views/shared/delegates/StopScrollPropagation
 *
 *   Desc:
 *     This class prevents the user from scrolling the page when scrolling a div.

 *   @param {Object} (Optional) options An optional object literal having one settings.
 *
 *    Usage:
 *       var p = new StopScrollPropagation({options})
 *
 *    Options:
 *        el (required): The event delegate.
 *        selector: jQuery selector for the scrolling elements. If not provided, all mousewheel events in the el will not propagate.
 *
 */


define('views/shared/delegates/StopScrollPropagation',['jquery', 'underscore', 'views/shared/delegates/Base'], function($, _, DelegateBase) {
    return DelegateBase.extend({
        initialize: function(){
            var defaults = {
                selector: ""
            };
            _.defaults(this.options, defaults);

            this.events = {};
            this.events["mousewheel " + this.options.selector] = "mousewheel";
            this.events["DOMMouseScroll " + this.options.selector] = "mousewheel";
            this.delegateEvents(this.events);
        },
        mousewheel: function (e) {
            var delta = -e.originalEvent.wheelDelta || e.originalEvent.detail* 20;
            e.currentTarget.scrollTop += delta;
            e.stopPropagation();
            e.preventDefault();
        }

    });
});
/**
 *   views/shared/delegates/TextareaResize
 *
 *   Desc:
 *     This class applies auto resizing to textareas.  Textareas will resize as input changes until it reaches the maximum
 *     number of lines (specified in options, defaults to 5), after which the textarea will scroll.

 *   @param {Object} (Optional) options An optional object literal having non-required settings.
 *
 *    Usage:
 *       var t = new Splunk.TextareaResize({ options array })
 *
 *    Options:
 *        maxLines: (int) number to indicate the maximum lines to expand to.  ex: 5 (limits expansion to 5 lines, scroll after)
 *
 *    Methods:
 *        resizeTextarea: If the view updates the content, it needs to call resizeTextarea().
 *
 *
 *    ***NOTE***: this is a port and modification of the Elastic plugin for jquery.  The original plugin's page can be found here:  http://www.unwrongest.com/projects/elastic/
 */
define('views/shared/delegates/TextareaResize',[
    'jquery',
    'views/shared/delegates/Base',
    'underscore',
    'splunk.logger'
],
function(
    $,
    DelegateBase,
    _,
    sLogger
){
    return DelegateBase.extend({
        maxHeightMultiple: 5,
        maxHeight: null,
        lineHeight: null,
        minHeight: null,
        initialize: function(){

            this.logger = sLogger.getLogger("textarea_resize.js");

            this.content = null;
            this.shadow = null;

            //defaults
            var defaults = {
                maxLines : this.maxHeightMultiple
            };

            _.defaults(this.options, defaults);

            this.lineHeight = parseInt(this.$el.css('lineHeight'),10) || parseInt(this.$el.css('fontSize'),10) + 1 || 20;
            this.minHeight = this.options.minHeight || this.options.lineHeight ;
            this.maxHeight = this.options['maxLines'] * this.lineHeight;

            // Create shadow div that will be identical to the textarea, used for measuring height changes
            this._createShadow();

            //fire off the first one to resize
            this.resizeTextarea();
            this.nameSpace = 'textarearesize-' + this.cid;

            //Non element event handlers
            $(window).on('resize.' +  this.nameSpace, this.resizeTextarea.bind(this));
        },
       /**
        *   function to set up event handles.
        */
       events: {
            "change": "resizeTextarea",
            "focus": "resizeTextarea",
            "blur": "resizeTextarea",
            "keyup": "resizeTextarea",
            "paste": "resizeTextarea",
            "cut": "resizeTextarea"
       },
       /**
        *   create a shadow div (or twin) of the textarea. we'll use this to measure how tall the textarea should be by copying the content from the
        *   textarea into the div and then measure it
        */
       _createShadow: function() {
            if (!this.shadow) {
                //style attributes to copy from the textarea into the shadow div
                var styleAttrs = new Array('paddingTop','paddingRight','paddingBottom','paddingLeft','fontSize','lineHeight','fontFamily','fontWeight', 'wordWrap','whiteSpace','borderLeftWidth','borderLeftColor','borderLeftStyle','borderRightWidth','borderRightColor','borderRightStyle','borderTopWidth','borderTopColor','borderTopStyle','borderBottomWidth','borderBottomColor','borderBottomStyle','boxSizing');

                //create the shadow div (twin)
               this.shadow = $('<div class="shadowTextarea"></div>').css({'position': 'absolute','left':'-9999px','top':'-9999px', 'marginRight' : "-3000px"}).appendTo(this.$el.parent());
                //    copy each of the attribute specified in styleAttributes from the textarea onto the shadow div
                var styles= {};
                _.each(styleAttrs, function(attr){
                    var value = this.$el.css(attr);
                    styles[attr] = (attr == "whiteSpace" && value == "normal") ? "pre-wrap" : value; // FF reports normal instead of pre-wrap.
                }, this);
                this.shadow.css(styles);

                this._adjustShadowWidth();
            }
       },
       /**
        *   function to set the shadow div's width to the textarea width
        */
       _adjustShadowWidth: function() {
          if (this.shadow) {
                this.shadow.width(this.$el.width());
          }
       },
       /**
        *   function to actually do the resizing
        */
       resizeTextarea: function() {           
           if (this.$el.val() != this.content) {
               this.content = this.$el.val();
               this.shadow.text(this.content);
           }

           //if the height of the two twins is different by more than 3 px.
           this._adjustShadowWidth();
           if (Math.abs(this.shadow.height() - this.$el.height()) > 3) {
                var newHeight = this.shadow.height();

                if (newHeight >= this.maxHeight) {
                    this._setHeight(this.maxHeight, 'auto');
                } else if ( newHeight <= this.minHeight ) {
                    this._setHeight(this.minHeight, 'hidden');
                } else {
                    this._setHeight(newHeight, 'hidden');
                }
           }
        },
        /**
         *  function to set the height of the textarea
         */
        _setHeight: function(height, overflow) {
            var curratedHeight = Math.floor(parseInt(height,10)) - 1; // this seems to be off by 1px for some reason
            if (this.$el.height() != curratedHeight) {
                this.$el.height(curratedHeight);
                this.$el.css('overflow', overflow);
            }
        },
         /**
        *  function to find out if there is more than one line of text.
        */
        isMultiline: function() {
            var lineCount = Math.round(this.$el.height() / this.lineHeight) ;
            return (lineCount > 1);
        },
        remove: function() {
            DelegateBase.prototype.remove.apply(this);
            $(window).off('resize.' + this.nameSpace);
            return this;
        }
    });
});

























/**
 *   views/shared/delegates/Accordion
 *
 *   Desc:
 *     This class applies accordion behaviors.
 *     Default markup is based on Twitter Bootstrap's Collapse
 *     http://twitter.github.com/bootstrap/
 *
 *     @param {Object} (Optional) options An optional object literal having one or more settings.
 *
 *    Usage:
 *       var p = new Popdown({options})
 *
 *    Options:
 *        el (required): The event delegate.
 *        group: jQuery selector for the toggle and body wrapper ".accordion-group".
 *        toggle: jQuery selector for the accordion group's toggle. Defaults to ".accordion-toggle".
 *        body: jQuery selector for the accordion group's body. Defaults to ".accordion-body".
 *        default: jQuery selector or object for the default group. Defaults to ".accordion-group:first-child".
 *        collapsible: Will allow for each to open/close on each's own w/o others toggling
 *
 *    Methods:
 *        show: show a panel. Parameters are the group, which can be a selector or jQuery object, and a boolean to enable or disable animation.
 */


define('views/shared/delegates/Accordion',[
    'jquery',
    'underscore',
    'views/shared/delegates/Base'
],function(
    $,
    _,
    DelegateBase
){
    return DelegateBase.extend({
        initialize: function(){
            var defaults = {
                group: ".accordion-group",
                toggle: ".accordion-toggle",
                body: ".accordion-body",
                defaultGroup: ".accordion-group:first-child",
                icon: ".icon-accordion-toggle",
                inactiveIconClass: "icon-triangle-right-small",
                activeIconClass: "icon-triangle-down-small",
                activeClass: "active",
                collapsible: false,
                speed: 300
            };

            _.defaults(this.options, defaults);
            
            this.events = {};
            this.events["click " + this.options.toggle] = "toggle";

            //setup
            this.$(this.options.body).hide();
            this.$(this.options.icon).addClass(this.options.inactiveIconClass);
            
            //show the default group all
            this.show(this.$(this.options.defaultGroup), false);
        },
        toggle: function (e) {
            e.preventDefault();      
            var $group = $(e.currentTarget).closest(this.options.group);
            
            if (this.options.collapsible) {
                this.singleToggle($group);
                return;
            }

            //if the group is already active, do nothing.
            if ($group.hasClass(this.options.activeClass)) {
                return;
            }
            
            this.trigger('toggle'); 
            this.show($group, true);
            this.trigger('toggled');
        },
        show: function (group, animate) {
            var that = this,
                $newGroup = $(group),
                $activeGroup = this.$(this.options.group + '.' + this.options.activeClass),
                showComplete = function () {
                  that.trigger('shown');
                },
                hideComplete = function () {
                  that.trigger('hidden');
                };

            //ignore if the item is already active.
            this.trigger('show');
            this.trigger('hide');
            
            //Swap the active classes.
            $activeGroup.removeClass(this.options.activeClass);
            $newGroup.addClass(this.options.activeClass);

            //Swap the icon classes
            $activeGroup.find(this.options.icon).addClass(this.options.inactiveIconClass).removeClass(this.options.activeIconClass);
            $newGroup.find(this.options.icon).addClass(this.options.activeIconClass).removeClass(this.options.inactiveIconClass);
            
            //Show hide the body
            if (animate){
                $activeGroup.find(this.options.body).slideUp({duration:this.options.speed, queue: false, complete:hideComplete});
                $newGroup.find(this.options.body).slideDown({duration:this.options.speed, queue: false, complete:showComplete});
            } else {
                $activeGroup.find(this.options.body).hide({duration:0, queue: false, complete:hideComplete});
                $newGroup.find(this.options.body).show({duration:0, queue: false, complete:showComplete});
            }
        },
        singleToggle: function(group) {
            if (group.hasClass(this.options.activeClass)) {
                group.find(this.options.icon).addClass(this.options.inactiveIconClass).removeClass(this.options.activeIconClass);
                group.find(this.options.body).slideUp({duration:this.options.speed, queue: false});
            } else {
                group.find(this.options.icon).addClass(this.options.activeIconClass).removeClass(this.options.inactiveIconClass);
                group.find(this.options.body).slideDown({duration:this.options.speed, queue: false});
            }
            group.toggleClass(this.options.activeClass);
            this.trigger('toggled');
        },
        initialToggleOn: function(group) {
            group.find(this.options.icon).addClass(this.options.activeIconClass).removeClass(this.options.inactiveIconClass);
            group.find(this.options.body).show();
            group.addClass(this.options.activeClass);
        }
    });
});
/**
 * @author sfishel
 *
 * A delegate view to handle the column sorting behavior for a grid-based view
 *
 * Events:
 *
 * sort (aka ColumnSort.SORT) triggered when the user clicks on a sortable column header
 *      @param sortKey {String} the key for the sorted column
 *      @param sortDirection {String} either 'asc' (aka ColumnSort.ASCENDING) or 'desc' (aka ColumnSort.DESCENDING)
 *                                    the direction of the sort
 */

define('views/shared/delegates/ColumnSort',['jquery', 'underscore', 'backbone', 'views/shared/delegates/Base'], function($, _, Backbone, DelegateBase) {

    var CONSTS = {

        // CSS class names
        SORTABLE_ROW: 'sorts',
        NOT_SORTABLE_CLASS: 'not-sortable',
        SORT_DISABLED_CLASS: 'sort-disabled',
        SORTED_CLASS: 'active',

        // Enum strings
        ASCENDING: 'asc',
        DESCENDING: 'desc',

        // Data attributes
        SORT_KEY_ATTR: 'data-sort-key',

        // Events
        SORT: 'sort'

    };

    var SORTABLE_CELL_SELECTOR = 'th.' + CONSTS.SORTABLE_ROW,
        SORTABLE_ICON_SELECTOR = 'i.icon-sorts';

    return DelegateBase.extend({

        events: (function() {
            var events = {};
            events['click ' + SORTABLE_CELL_SELECTOR] = 'onColumnHeaderClick';
            return events;
        }()),

        /**
         * @constructor
         * @param options {Object} {
         *     model {Model} optional, a model that the view will keep up-to-date with sort information
         *                   using the attribute names 'sortKey' and 'sortDirection'
         *                   the view will also respond to external changes to those attributes
         *                   if a model is not given, one will be created and managed internally
         *     autoUpdate {Boolean} optional, defaults to false
         *                          whether to automatically update the view's DOM elements when the sort configuration changes
         *    {String} sortKeyAttr: (Optional) attribute to set sort key on  default is 'sortKey',
         *    {String} sortDirAttr: (Optional) attribute to set sort direction on default is 'sortDirection'
         * }
         */

        initialize: function() {
            if(!this.model) {
                this.model = new Backbone.Model();
            }

            var defaults = {
                sortKeyAttr: 'sortKey',
                sortDirAttr: 'sortDirection'
            };
            this.options = $.extend(true, defaults, this.options);

            this.autoUpdate = !!this.options.autoUpdate;
            this.activate();
        },

        startListening: function() {
            if(this.autoUpdate) {
                this.listenTo(this.model, 'change:' + this.options.sortKeyAttr + ' change:' + this.options.sortDirAttr, _.debounce(function() {
                    this.update();
                }, 0));
            }
        },

        /**
         * Updates the given DOM elements to match the current sorting configuration
         *
         * @param $rootEl <jQuery object> optional, defaults to the view's $el property
         */

        update: function($rootEl) {
            $rootEl = $rootEl || this.$el;
            var $thList = $rootEl.find(SORTABLE_CELL_SELECTOR),
                sortKey = this.model.get(this.options.sortKeyAttr),
                $activeTh = $thList.filter('[' + CONSTS.SORT_KEY_ATTR + '="' + sortKey + '"]'),
                sortDirection = this.model.get(this.options.sortDirAttr);

            $thList.removeClass(CONSTS.SORTED_CLASS)
                .find(SORTABLE_ICON_SELECTOR).removeClass(CONSTS.ASCENDING + ' ' + CONSTS.DESCENDING);
            $activeTh.addClass(CONSTS.SORTED_CLASS)
                .find(SORTABLE_ICON_SELECTOR).addClass(sortDirection === 'asc' ? CONSTS.ASCENDING : CONSTS.DESCENDING);
        },

        // ----- private methods ----- //

        onColumnHeaderClick: function(e) {
            e.preventDefault();
            var $th = $(e.currentTarget);
            if($th.hasClass(CONSTS.NOT_SORTABLE_CLASS) || $th.hasClass(CONSTS.SORT_DISABLED_CLASS)) {
                return;
            }
            var sortKey = $th.attr(CONSTS.SORT_KEY_ATTR),
                sortDirection = $th.find(SORTABLE_ICON_SELECTOR).hasClass(CONSTS.DESCENDING) ? 'asc' : 'desc';

            var data = {};
            data[this.options.sortKeyAttr] = sortKey;
            data[this.options.sortDirAttr] = sortDirection;

            this.model.set(data);
            this.trigger(CONSTS.SORT, sortKey, sortDirection);
        }

    }, CONSTS);

});

/**
 * An abstract base view that defines some shared behaviors for managing a detached table header.
 *
 * BEWARE: this the initial step in what should be a larger refactor to avoid repeated code in TableDock and TableHeadStatic,
 * currently this is pretty fragile and depends on the options and variables defined in those views having the same names and meanings.
 */

define('views/shared/delegates/DetachedTableHeader',[
            'jquery',
            'underscore',
            'views/shared/delegates/Base'
        ],
        function(
            $,
            _,
            Base
        ) {

    return Base.extend({

        syncColumnWidths: function() {
            if(!this.$table) {
                return;
            }
            var $baseRow = this.$table.find('thead:first > tr'),
                $baseRowCells =  $baseRow.find('th'),
                $headerRow = this.$headerTable.find('thead:first > tr'),
                $headerCells = $headerRow.find('th'),
                $skipCell = false,
                widths = [];

            this.$table.css('table-layout', this.options.defaultLayout);
            if(this.options.flexWidthColumn !== false) {
                $skipCell = $baseRowCells.eq(this.options.flexWidthColumn);
            }
            $baseRowCells.not($skipCell[0]).css('width', '');

            _($baseRowCells).each(function(cell, i) {
                if ($skipCell && (cell === $skipCell[0])) {
                    return;
                }

                var $cell = $(cell),
                    cellWidth = {
                        // SPL-71945, the correct way to measure width is to use the clientWidth minus all horizontal padding
                        // this will correctly account for differences caused by border-collapse
                        width: $cell[0].clientWidth - parseFloat($cell.css('padding-right')) - parseFloat($cell.css('padding-left')),
                        index: i
                    };

                widths.push(cellWidth);
            }, this);

            this.$headerTable.width(this.$table.outerWidth());

            _.each(widths, function(cell) {
                var $cell = $baseRowCells.eq(cell.index),
                    $headerCell = $headerCells.eq(cell.index);

                $cell.width(cell.width);
                $headerCell.width(cell.width);
            });

            this.$table.css('table-layout', 'fixed');
        }
    });

});
/**
 * A delegate view to handle docking table header, footer, and scroll bar.
 */

define('views/shared/delegates/TableDock',['jquery', 'underscore', 'views/shared/delegates/DetachedTableHeader'], function($, _, DetachedTableHeader) {

    return DetachedTableHeader.extend({
        awake: true,
        touch: false,
        initialize: function(options) {

            var defaults = {
                    table: "> table",
                    offset: 0,
                    dockScrollBar: true,
                    flexWidthColumn: -1,
                    defaultLayout:'auto'
                };
    
            _.defaults(this.options, defaults);
            DetachedTableHeader.prototype.initialize.call(this, options);
            
            this.disabled = false;
            this.dockedScrollHidden = true;
            this.left = 0;
            this.scrollLeft = false;
            
            this.eventNS = 'table-dock-' + this.cid;
            this.activate();
        },
        activate: function(options) {
            options || (options = {});
            if(this.active) {
                return DetachedTableHeader.prototype.activate.call(this, options);
            }
            
            var debouncedResizeHandler = _.debounce(this.handleWindowResize, 50);
            $(window).on('resize.' + this.eventNS, _(debouncedResizeHandler).bind(this));
            $(window).on('scroll.' + this.eventNS, _(this.handleWindowScroll).bind(this));
            
            _.defer(this.update.bind(this));
            
            this.options.dockScrollBar && this.$el.on('scroll.' + this.eventNS, _(this.handleContainerScroll).bind(this));
            
            return DetachedTableHeader.prototype.activate.call(this, options);
        },
        deactivate: function(options) {
            options || (options = {});
            if(!this.active) {
                return DetachedTableHeader.prototype.deactivate.call(this, options);
            }
            DetachedTableHeader.prototype.deactivate.call(this, options);
            
            if (this.$el.html()){
                this.$el.scrollLeft(0);
            }
            this.left = 0;
            this.scrollLeft = false;
            
            $(window).off('resize.' + this.eventNS);
            $(window).off('scroll.' + this.eventNS);
            this.$el.off('.' + this.eventNS);
            this.$headerTable && this.$headerTable.off('.' + this.eventNS);
            this.$dockedScroll && this.$dockedScroll.off('.' + this.eventNS);

            return this;
        },
        update: function() {
            if (!this._updateDebounced) {
                this._updateDebounced = _.debounce(this._update);
            }
            this._updateDebounced();
        },
        _update: function() {
            this.$table = this.$(this.options.table ).first();
            
            this.updateHeaders();
            this.options.dockScrollBar && this.updateScroll();

            if(this.$table.is(':visible')) {
                this.syncMeasurements();
            } else {
                _.defer(this.syncMeasurements.bind(this));
            }
        },
        syncMeasurements: function() {
            this.scrollLeft = false;
            this.syncColumnWidths();
            this.options.dockScrollBar && this.syncScrollWidths();
            this.handleWindowScroll();
            this.options.dockScrollBar && this.handleContainerScroll();
            this.trigger('updated', this.$headerTable);
        },
        disable: function() {
            this.disabled = true;
            this.$disable && this.$disable.show();
        },
        enable: function() {
            this.disabled = false;
            this.$disable && this.$disable.hide();
        },
        updateHeaders: function() {
            if(this.$header) {
                this.$header.remove();
            }
            
            this.$header = $('<div class="header-table-docked" style="display:none"><table></table><div class="disable"></div></div>').css({top: this.options.offset, left:this.left, right: 0});
            this.$disable = this.$header.find('> .disable')[this.disabled ? 'show' : 'hide']();
            this.$headerTable = this.$header.find('> table');
            this.$headerTable.attr('class', this.$table.attr('class'));
            this.$table.find('> thead').clone().appendTo(this.$headerTable);
            this.$header.prependTo(this.el);
            this.$headerTable.on('click.' + this.eventNS, 'th', function(e) {
                e.preventDefault();
                e.stopPropagation();
                var colIndex = $(e.currentTarget).prevAll().length + 1;
                this.$table.find('> thead > tr > th:nth-child(' + colIndex + ')').click();
            }.bind(this));
            this.$headerTable.on('click.' + this.eventNS, 'th a', function(e) {
                e.preventDefault();
            });
        },
                
        updateScroll: function() {
            if(this.$dockedScroll) {
                this.$dockedScroll.remove();
            }
            
            this.$dockedScroll = $('<div />').addClass('table-scroll-bar-docked').hide().appendTo(this.$el).css('left', this.$el.position().left);
            this.$dockedScroll.on('scroll.' + this.eventNS, _(this.handleDockedScrollChange).bind(this));
        },

        // ------ private methods ------ //

        handleWindowResize: function() {
            // no-op if update has not been called yet
            if(!this.$table || !this.$table.is(':visible')) {
                return;
            }
            this.syncColumnWidths();
            
            if(this.options.dockScrollBar) {
                this.updateDockedScrollBar();
                this.syncScrollWidths();
            }
        },

        handleWindowScroll: function(e) {
            // no-op if update has not been called yet
            if(!this.$table || !this.$table.is(':visible')) {
                return;
            }
            var scrollTop = $(window).scrollTop(),
                tableOffsetTop = this.$table.offset().top;

            if(scrollTop >= tableOffsetTop - this.options.offset) {
                if(!this.$header.is(':visible')) {
                    this.$header.css('top', this.options.offset).show();
                }
            }
            else {
                this.$header.css('top', '-1000px').width(); //move off and force redraw
                this.$header.hide();
            }
            this.options.dockScrollBar && this.updateDockedScrollBar();
        },

        handleContainerScroll: function() {
            // no-op if update has not been called yet or the dock is hidden
            if( !this.$table) {
                return;
            }
            
            var scrollLeft = this.$el.scrollLeft();
            
            
            if(this.scrollLeft === scrollLeft) {
                return;
            }

            this.$headerTable.css('marginLeft', -scrollLeft + 'px');
            this.$dockedScroll.scrollLeft(scrollLeft);
            this.scrollLeft = scrollLeft;
        },

        handleDockedScrollChange: function() {
            var scrollLeft = this.$el.scrollLeft(),
                dockScrollLeft = this.$dockedScroll.scrollLeft();

            if(scrollLeft !== dockScrollLeft) {
                this.$headerTable.css('marginLeft', -dockScrollLeft + 'px');
                this.$el.scrollLeft(dockScrollLeft);
            }
        },
        syncColumnWidths: function() {
            DetachedTableHeader.prototype.syncColumnWidths.apply(this, arguments);
            this.left = this.$el.position().left;
            this.$header.css({left: this.left});
        },
        syncScrollWidths: function() {
            if(!this.$table || !this.$table[0]) {
                return;
            }

            var tableWidth = parseFloat(this.$table[0].scrollWidth),
                $fullWidthDiv = $('<div />').width(tableWidth).height(1);

            this.$dockedScroll.html($fullWidthDiv);
        },

        updateDockedScrollBar: function() {
            var scrollTop = $(window).scrollTop(),
                tableOffsetTop = this.$table.offset().top,
                windowHeight = $(window).height(),
                tableHeight = this.$el.outerHeight();

            // If the table's scroll bar is below the fold, show the docked scroll bar.
            // But not if the body itself has a scroll bar (SPL-89945, no double rainbows!).
            if((tableOffsetTop + tableHeight > scrollTop + windowHeight) && $(window).width() >= $('body').width()) {
                if (!this.dockedScrollHidden) {
                    return;
                }
                this.$dockedScroll.show();
                this.dockedScrollHidden = false;
                this.handleContainerScroll();
            }
            else if (!this.dockedScrollHidden){
                this.$dockedScroll.hide();
                this.dockedScrollHidden = true;
            }
        },
        remove: function() {
            DetachedTableHeader.prototype.remove.apply(this);
            $(window).off('resize.' + this.eventNS);
            $(window).off('scroll.' + this.eventNS);
            this.$el.off('.' + this.eventNS);
            this.$headerTable && this.$headerTable.off('.' + this.eventNS);
            this.$dockedScroll && this.$dockedScroll.off('.' + this.eventNS);
            return this;
        }
    });

});

/**
 * A delegate view to handle a statically positioned table header.
 */

define('views/shared/delegates/TableHeadStatic',['jquery', 'underscore', 'views/shared/delegates/DetachedTableHeader'], function($, _, DetachedTableHeader) {

    return DetachedTableHeader.extend({
        active: false,
        initialize: function(options) {
            var defaults = {
                    headerContainer: "> .header-table-static",
                    scrollContainer: "> .scroll-table-wrapper",
                    offset: 0,
                    // index of the column to "flex" to fit remaining width, can be set to false for no flex
                    flexWidthColumn: -1,
                    defaultLayout:'auto'
                };

            _.defaults(this.options, defaults);

            if (!this.options.table) {
                this.options.table = this.options.scrollContainer + ' > table';
            }
            
            // specified scroll container could be self or a downstream element
            this.$scrollContainer = (this.$el.is(this.options.scrollContainer)) ? this.$el : this.$(this.options.scrollContainer);
            DetachedTableHeader.prototype.initialize.call(this, options);
            
            this.activate({noUpdate: true});
        },
        wake: function() {
            return this.activate();
        },
        activate: function(options) {
            options || (options = {});
            if(this.active) {
                return DetachedTableHeader.prototype.activate.call(this, options);
            }
            
            var debouncedResizeHandler = _.debounce(this.handleWindowResize, 50);
            $(window).on('resize.' + this.cid, _(debouncedResizeHandler).bind(this));
            this.$scrollContainer = (this.$el.is(this.options.scrollContainer)) ? this.$el : this.$(this.options.scrollContainer);
            this.$scrollContainer.on('scroll.' + this.cid, _(this.handleContainerScroll).bind(this));
            
            if(!options.noUpdate) {
                _.defer(this.update.bind(this));
            }

            return DetachedTableHeader.prototype.activate.call(this, options);
        },
        sleep: function() {
            return this.deactivate();
        },
        deactivate: function(options) {
            options || (options = {});
            if(!this.active) {
                return DetachedTableHeader.prototype.deactivate.call(this, options);
            }
            DetachedTableHeader.prototype.deactivate.call(this, options);

            $(window).off('resize.' + this.cid);
            this.$scrollContainer.off('scroll.' + this.cid);
            this.$headerTable && this.$headerTable.off('.' + this.cid);
            
            return this;
        },
        remove: function() {
            DetachedTableHeader.prototype.remove.apply(this);
            $(window).off('resize.' + this.cid);
            this.$scrollContainer.off('scroll.' + this.cid);
            this.$headerTable && this.$headerTable.off('.' + this.cid);
            return this;
        },
        update: function() {
            if(!this.active) {
                return;
            }
            this.$scrollContainer = (this.$el.is(this.options.scrollContainer)) ? this.$el : this.$(this.options.scrollContainer);
            this.$table = this.$(this.options.table).first();
            this.$headerContainer = this.$(this.options.headerContainer);
            this.$headerContainer.empty();
            var $headerWrapper = $('<div class="header-table-wrapper"></div>').appendTo(this.$headerContainer);
            $headerWrapper.css({ 'margin-right': this.$scrollContainer[0].offsetWidth - this.$scrollContainer[0].clientWidth});
            this.$headerTable = $('<table>');
            this.$headerTable.attr('class', this.$table.attr('class')).width(this.$table.outerWidth()).css('table-layout', 'fixed');
            // SPL-77034, for in-place updates make sure we don't clone the visibility: hidden; that is added later
            this.$table.find('> thead').clone().css('visibility', 'visible').appendTo(this.$headerTable);
            $headerWrapper.prepend(this.$headerTable);

            // 508: make sure the browser doesn't tab to the original table header
            this.$table.find('thead').css('visibility', 'hidden');
            
            this.$headerTable.on('click.' + this.cid, 'th', function (e) {
                e.preventDefault();
                e.stopPropagation();
                var colIndex = $(e.currentTarget).prevAll().length + 1;
                this.$table.find('> thead > tr > th:nth-child(' + colIndex + ')').click();
            }.bind(this));
            
            this.$headerTable.on('click.' + this.cid, 'th a', function (e) {
                e.preventDefault();
            });

            // the newly rendered table head will be scrolled all the way to the left
            // reset this instance variable so that handleContainerScroll will do the right thing
            this.scrollLeft = 0;
            var updateDom = _(function() {
                this.syncColumnWidths();
                this.handleContainerScroll();
                this.trigger('updated', this.$headerTable);
            }).bind(this);

            if(this.$table.is(':visible')) {
                updateDom();
            } else {
                _.defer(updateDom);
            }
        },

        // ------ private methods ------ //
        handleContainerScroll: function() {
            // no-op if update has not been called yet
            if(!this.$table) {
                return;
            }
            var scrollLeft = this.$scrollContainer.scrollLeft();
            if(scrollLeft !== this.scrollLeft) {
                this.$headerTable.css('marginLeft', -scrollLeft + 'px');
                this.scrollLeft = scrollLeft;
            }
        },
        handleWindowResize: function() {
            // no-op if update has not been called yet
            if(!this.$table) {
                return;
            }
            this.syncColumnWidths();
        }

    });

});
