define([
            'jquery',
            'underscore',
            'controllers/Base',
            'collections/services/datamodel/DataModels',
            'views/data_models/DataModels',
            'models/config',
            'models/shared/Application',
            'models/classicurl'
        ],
        function(
            $,
            _,
            Base,
            DataModelCollection,
            DataModels,
            configModel,
            ApplicationModel,
            classicUrl
        ) {

    var DataModelsController = Base.extend({

        initialize: function(options) {
            Base.prototype.initialize.apply(this, arguments);
            this.application = options.application;

            this.dmCollection = new DataModelCollection();
            // set the collection to sort ascending by displayName
            this.dmCollection.fetchData.set({ sortKey: 'displayName', sortDirection: 'asc', count: 20 }, { silent: true });
            
            this.dataModelsView = new DataModels({
                collection: this.dmCollection,
                model: {
                    application: this.application
                }
            });

            var that = this,
                fetchData = {
                    concise: true,  
                    app: this.application.get('app'),
                    owner: this.application.get('owner')
                };

            this.dmCollection.fetch({ data: fetchData }).done(function() {
                // If there is only one non-empty data model, redirect immediately to the data model explorer, unless the "noredirect" flag is set
                if(that.dmCollection.length === 1 && !that.dmCollection.models[0].isEmpty() && !classicUrl.get('noredirect')) {
                    // set the "noredirect" flag before redirecting to avoid a back-button loop
                    var firstModel = that.dmCollection.models[0];
                    classicUrl.save({
                        noredirect: true,
                        model: firstModel.get('id')
                    }, { replaceState: true, trigger: true });
                }
            });

            _.defer(function() { that.trigger('ready'); });
        },

        remove: function() {
            Base.prototype.remove.apply(this, arguments);
            this.dataModelsView.remove();
        }
    });

    return DataModelsController;

});
