define(['../util/parsing_utils', '../util/color_utils'], function(parsingUtils, colorUtils) {

    var ColorPalette = function(colors, useInterpolation) {
        this.setColors(colors);
        this.useInterpolation = parsingUtils.normalizeBoolean(useInterpolation, false);
    };

    ColorPalette.prototype = {

        setColors: function(colors) {
            this.colors = colors || this.BASE_COLORS;
        },

        getColor: function(field, index, count) {
            var p, index1, index2,
                numColors = this.colors.length;

            if(numColors === 0) {
                return 0x000000;
            }
            if(index < 0) {
                index = 0;
            }
            if(!this.useInterpolation) {
                return this.colors[index % numColors];
            }
            if (count < 1) {
                count = 1;
            }
            if (index > count) {
                index = count;
            }
            p = (count === 1) ? 0 : (numColors - 1) * (index / (count - 1));
            index1 = Math.floor(p);
            index2 = Math.min(index1 + 1, numColors - 1);
            p -= index1;

            return this.interpolateColors(this.colors[index1], this.colors[index2], p);
        },

        getColorAsRgb: function(field, index, count) {
            var hexColor = this.getColor(field, index, count);
            return colorUtils.colorFromHex(hexColor);
        },

        interpolateColors: function(color1, color2, p) {
            var r1 = (color1 >> 16) & 0xFF,
                g1 = (color1 >> 8) & 0xFF,
                b1 = color1 & 0xFF,

                r2 = (color2 >> 16) & 0xFF,
                g2 = (color2 >> 8) & 0xFF,
                b2 = color2 & 0xFF,

                rInterp = r1 + Math.round((r2 - r1) * p),
                gInterp = g1 + Math.round((g2 - g1) * p),
                bInterp = b1 + Math.round((b2 - b1) * p);

            return ((rInterp << 16) | (gInterp << 8) | bInterp);
        },

        BASE_COLORS: [
            0x6BB7C8,
            0xFAC61D,
            0xD85E3D,
            0x956E96,
            0xF7912C,
            0x9AC23C,
            0x998C55,
            0xDD87B0,
            0x5479AF,
            0xE0A93B,
            0x6B8930,
            0xA04558,
            0xA7D4DF,
            0xFCDD77,
            0xE89E8B,
            0xBFA8C0,
            0xFABD80,
            0xC2DA8A,
            0xC2BA99,
            0xEBB7D0,
            0x98AFCF,
            0xECCB89,
            0xA6B883,
            0xC68F9B,
            0x416E79,
            0x967711,
            0x823825,
            0x59425A,
            0x94571A,
            0x5C7424,
            0x5C5433,
            0x85516A,
            0x324969,
            0x866523,
            0x40521D,
            0x602935
        ]

    };

    return ColorPalette;

});