define(['jquery', 'underscore'], function($, _) {

    var Tooltip = function(properties) {
        this.properties = properties || {};
    };

    Tooltip.prototype = {

        BASE_CONFIG: {
            enabled: true,
            backgroundColor: '#000000',
            borderColor: '#ffffff',
            hideDelay: 0,
            style: {
                color: '#cccccc'
            },
            /**
             * @author sfishel
             *
             * If the tooltip is too wide for the plot area, clip it left not right.
             *
             * unit test: js_charting/components/test_tooltip.html
             */
            positioner: function(boxWidth, boxHeight, point) {
                var position = this.getPosition(boxWidth, boxHeight, point),
                    plotWidth = this.chart.plotWidth, 
                    resetZoomButton = $('.btn-zoom-out'),
                    tooltipRight = position.x + boxWidth;

                // Prevent tooltip from blocking the reset zoom button
                if(resetZoomButton.length > 0){
                    var buttonPos = resetZoomButton.position();
                    if(buttonPos){
                        var buttonLeft = buttonPos.left,
                            buttonTop = buttonPos.top,
                            buttonHeight = resetZoomButton.height();
                        if(tooltipRight > buttonLeft){
                            if(position.y < buttonTop + buttonHeight){
                                // Shift tooltip to left by overlap distance between tooltip and button + 10 for padding
                                position.x -= (position.x + boxWidth - buttonLeft + 10);
                            }
                        }
                    }
                }
                if(boxWidth > plotWidth) {
                    var options = this.options;
                    position.x = options.borderRadius + options.borderWidth;
                }
                return position;
            },
            /**
             * @author sfishel
             *
             * Adjust the tooltip anchor position for column charts.
             * Use a position relative to the selected column instead of a shared one for the series group.
             *
             * unit test: js_charting/components/test_tooltip.html
             */
            getAnchorPostHook: function(points, mouseEvent, anchor) {
                if(points && !_.isArray(points) && points.series.options.type === 'column') {
                    anchor[0] = points.barX;
                }
                return anchor;
            }
        },

        getConfig: function() {
            return $.extend(true, {}, this.BASE_CONFIG, {
                borderColor: this.properties['borderColor']
            });
        },

        destroy: function() {}

    };

    return Tooltip;

});