define(['models/SplunkDBase'], 
function(SplunkDBase) {
    return SplunkDBase.extend({
        url: 'data/ui/ui-tour',
        initialize: function() {
            SplunkDBase.prototype.initialize.apply(this, arguments);
        },
        getName: function() {
            return this.entry.get("name");
        },
        getLinkedModel: function(linkName) {
            var link = undefined,
                linkedModel = undefined;

            if (!(linkName === "next" || linkName === "previous")) {
                return undefined;
            }

            if (this.entry.content.has(linkName) && this.collection) {
                var linkTourName = this.entry.content.get(linkName);
                linkedModel = this.collection.getTourModel(linkTourName);
            }
            
            return linkedModel;
        },
        getNextModel: function() {
            return this.getLinkedModel("next");
        },
        getPreviousModel: function() {
            return this.getLinkedModel("previous");
        },
        getUri: function(includeTourName) {
            var link = undefined;
            if (this.entry.content.has("uri")) {
                link = this.entry.content.get("uri");

                if (includeTourName) {
                    var name = this.getName();
                    if (name) {
                        //check to see if link already has query args
                        if (link.indexOf("?") == -1) {
                            link = link + "?tour=" + name;
                        } else {
                            link = link + "&tour=" + name;
                        }
                    }   
                }
            }    
            return link;
        },
        getLink: function(linkName) {
            var linkedModel = this.getLinkedModel(linkName);
            if (linkedModel) {
                return linkedModel.getUri(true);
            }
            return undefined;
        },
        getNextLink: function() {
            return this.getLink("next");
        },
        getPreviousLink: function() {
            return this.getLink("previous");
        },
        navigateToNext: function() {
            var nextModel = this.getLinkedModel("next");
            if (nextModel) {
                this.trigger("navigateRequest", nextModel);
            }
        },
        navigateToPrevious: function() {
            var previousModel = this.getLinkedModel("previous");
            if (previousModel) {
                this.trigger("navigateRequest", previousModel);
            }
        },
        endTour: function() {
            this.trigger("endTour");
        }
    });
});
