define(
    [
        'backbone',
        'underscore',
        'models/SplunkDBase',
        'util/general_utils'
    ],
    function(Backbone, _, SplunkDBaseModel, generalUtils) {
        var RAW_REX = /\$_raw\$/g;
        return SplunkDBaseModel.extend({
            url: "data/ui/workflow-actions",
            initialize: function() {
                SplunkDBaseModel.prototype.initialize.apply(this, arguments);
            },
            /**
             *  $field$ 
             */
            fieldSubstitute: function(key, text, data, fieldName, fieldValue) {
                var settings = {
                        interpolate: /\$([\s\S]+?)\$/g 
                    },
                    $matches$ = text.match(settings.interpolate),
                    templateVars = {};
                
                text = this.convertRaw(text, data);
                    

                if ($matches$) {
                    _($matches$).each(function($string$) {
                        var match = $string$.match(/\$([\s\S]+?)\$/)[1],
                            matchValue = match === fieldName ? fieldValue : data[match];
                        if (!matchValue) {
                            //underscore templates will raise if the value to be replaced
                            //is undefined.  Like 5.0, we will strip out $ delimited values
                            //if they do not exist in event. 
                            text = text.replace($string$, '');
                        } else {
                            if (_.isArray(matchValue)) {
                                matchValue = matchValue[0];
                            }
                            
                            //all of the field values come back as arrays
                            //we don't not support anything other than the first value
                            if (key === "link.uri") {
                                templateVars[match] = encodeURIComponent(matchValue);
                            } else {
                                templateVars[match] = matchValue;
                            }
                        }
                    }, this);
                }

                return _.template(text, templateVars, settings);
            },
            /**
             * $@sid$ $@offset$, $@namespace$, $@latest_time$, $@field_name$, $@field_value$ 
             */
            systemSubstitute: function(text, sid, offset, namespace, latest_time, fieldName, fieldValue) {
                var settings = {
                        interpolate: /\$@([\s\S]+?)\$/g
                    },
                    data = {
                        sid: sid,
                        offset: offset,
                        namespace: namespace,
                        latest_time: latest_time,
                        field_name: fieldName,
                        field_value: fieldValue
                    };

                return _.template(text, data, settings);
            },
            convertRaw: function(text, data) {
                /*
                 * SPL-77129, when we request results with segmentation the 
                 * _raw field is returned as an object.  Ensure that we 
                 * replace _raw with the _raw string before passing to the template.
                 */
                 if(RAW_REX.test(text) && !_(data._raw).isArray()) {
                    return text.replace(RAW_REX, data._raw.value);
                 }
                
                return text;
            },
            isInFieldMenu: function() {
                return (this.entry.content.get('display_location') != 'event_menu');
            },
            isInEventMenu: function() {
                return (this.entry.content.get('display_location') != 'field_menu');
            },
            isRestrictedByPresenceOfAllFields: function(event) {
                var requiredFields = _(this.entry.content.get('fields').split(',')).map(function(field) {
                    return $.trim(field);
                }, this);
                var existingFields = _(event.toJSON()).keys();
                return !generalUtils.isFuzzySubset(requiredFields, existingFields);
            }, 
            isRestrictedByEventtype: function(event) {
                var requiredEventTypes = this.entry.content.has('eventtypes') ?
                    _(this.entry.content.get('eventtypes').split(',')).map(function(eventType) {
                        return $.trim(eventType);
                    }, this) :
                    [];
                var existingEventTypes = event.has('eventtype') ? event.get('eventtype') : [];
                return !generalUtils.isFuzzySubset(requiredEventTypes, existingEventTypes);
            }
        });
    }
);
