define(
[
    'models/StaticIdSplunkDBase',
    'splunk.util'
],
function(
    SplunkDBaseModel,
    splunk_util
) {
    return SplunkDBaseModel.extend({
        urlRoot: 'server/info',
        initialize: function() {
            SplunkDBaseModel.prototype.initialize.apply(this, arguments);
        },
        hasAddOn: function(addOn) {
            var addOns = this.entry.content.get('addOns');
            if (!addOns) {
                return false;
            }
            return addOns.hasOwnProperty(addOn);
        },
        hasHadoopAddon: function() {
            return this.hasAddOn('hadoop');
        },
        isFreeLicense: function() {
            return splunk_util.normalizeBoolean(this.entry.content.get('isFree'));
        },
        isCloudTrial: function() {
        	return this.entry.content.get('instance_type') == 'cloudtrial';
        },
        getBranding: function() {
            return this.entry.content.get('product_type');
        },
        getLicenseGroup: function() {
            return this.entry.content.get('activeLicenseGroup');
        },
        getOsName: function() {
            return this.entry.content.get('os_name');
        },
        isCloud: function() {
            return this.entry.content.get('product_type') === 'cloud';
        },
        isHunk: function() {
            return this.entry.content.get('product_type') === 'hunk';
        },
        isEnterprise: function() {
            return this.entry.content.get('product_type') === 'enterprise';
        },
        isLitePaid: function() {
            return (this.entry.content.get('product_type') === 'lite');
        },
        isLiteFree: function() {
            return (this.entry.content.get('product_type') === 'lite_free');
        },
        isLite: function() {
            return (this.isLitePaid() || this.isLiteFree());
        },
        isLicenseStatePreviouslyKeyed: function() {
            return this.entry.content.get('licenseState') === 'PREVIOUS_KEYED_LICENSE';
        },
        isLicenseStateExpired: function() {
            return this.entry.content.get('licenseState') === 'EXPIRED';
        }
    },
    {
        id: 'server-info'
    });
});
