define(
    [
        'jquery',
        'underscore',
        'backbone',
        'uri/route',
        'models/Base'
    ],
    function($, _, Backbone, route, BaseModel){
        var Model = BaseModel.extend(
            {
                windowVariableName: 'spotParams',
                windowCallbackName: 'displaySpot',
                url: '',
                initialize: function() {
                    BaseModel.prototype.initialize.apply(this, arguments);
                    if (!window[this.windowCallbackName]) {
                        window[this.windowCallbackName] = function(){};
                    }
                },
                sync: function(method, model, options) {
                    if (method!=='read') {
                        throw new Error('invalid method: ' + method);
                    }
                    options = options || {};
                    var defaults = {
                            data: {},
                            dataType: 'script'
                        },
                        url = _.result(model, 'url') || model.id;
                    defaults.url = url;
                    $.extend(true, defaults, options);
                    return Backbone.sync.call(this, method, model, defaults);
                },
                parse: function(response, options) {
                    response = window[this.windowVariableName] || {};
                    return response;
                },
                fetchHelper: function(serverInfoModel, webConfModel, applicationModel, checkerLocation, uid, options) {
                    options = options || {};
                    var licenseLabels,
                        licenseDesc = 'pro',
                        defaults = {
                            url: route.updateChecker(
                                webConfModel.entry.content.get('updateCheckerBaseURL'), 
                                serverInfoModel.entry.content.get('isFree'), 
                                serverInfoModel.entry.content.get('version'),
                                checkerLocation,
                                serverInfoModel.entry.content.get('isTrial'),
                                serverInfoModel.entry.content.get('guid'),
                                serverInfoModel.entry.content.get('master_guid')
                            ),
                            data: {
                                locale: applicationModel.get('locale'),
                                cpu_arch: serverInfoModel.entry.content.get('cpu_arch'),
                                os_name: serverInfoModel.entry.content.get('os_name'),
                                guid: serverInfoModel.entry.content.get('guid'),
                                product: serverInfoModel.entry.content.get('product_type'),
                                uid: uid
                            }
                        };
                    if (serverInfoModel.entry.content.get('isFree')) {
                        licenseDesc = 'free';
                    } else if (!serverInfoModel.entry.content.get('isTrial') && serverInfoModel.entry.content.get('guid')===serverInfoModel.entry.content.get('master_guid')) {
                        licenseLabels = _.uniq(serverInfoModel.entry.content.get('license_labels') || []).slice(0,3);
                        if (licenseLabels.length) {
                            licenseDesc = licenseLabels.join(',');
                        }
                    }
                    defaults.data.license_desc = licenseDesc;
                    return this.fetch($.extend(true, defaults, options));
                }
            }
        );
        return Model;
    }
);
