define(
    [
        'underscore',
        'models/services/authentication/User',
        'models/services/server/ServerInfo'
    ],
    function(_, UserModel, ServerInfoModel) {
        return UserModel.extend({
            initialize: function(attributes, options) {
                UserModel.prototype.initialize.apply(this, arguments);
                options || (options={});
                var serverInfoModel = options.serverInfoModel;
                if (!options.serverInfoModel || !(serverInfoModel instanceof ServerInfoModel)) {
                    throw 'The following constructor arguments are required ({}, {serverInfoModel: <models/services/server/ServerInfo>}).';
                }
                this.associated.serverInfo = serverInfoModel;
                this.serverInfo = serverInfoModel;
            },
            canAddData: function() {
                var productType = this.serverInfo.entry.content.get("product_type");
                var isSplunk = productType != "hunk"; // Hunk uses Explore Data, not Add Data

                // SystemMenu uses this.canEditMonitor to decide whether to show this link
                return this.canEditMonitor() && isSplunk;
            },
            canExploreData: function() {
                var productType = this.serverInfo.entry.content.get("product_type");
                return !this.isFree() && (productType == "hunk" || productType == "splunk"); // Hunk uses Explore Data, not Add Data
            },
            canUseApps: function() {
                if (this.serverInfo.isLite()) {
                    return false;
                }
                return true;
            },
            canUseAlerts: function() {
                if (this.serverInfo.isLiteFree() || this.serverInfo.isFreeLicense()) {
                    return false;
                }
                return true;
            },
            canPivot: function() {
                if (this.serverInfo.isLite()) {
                    return false;
                }
                return true;
            },
            canSchedulePDF: function() {
                return (!this.serverInfo.isLiteFree() && this.canScheduleSearch());
            },
            canUseSidenav: function() {
                if (this.serverInfo.isLite()) {
                    return true;
                }
                return false;
            }
        });
    }
);