define(
	[
		'jquery',
		'underscore',
		'backbone',
		'models/services/server/ServerInfo',
		'mixins/managementconsole/DistributedSearchGroup'
	],
	function(
		$,
		_,
		Backbone,
		ServerInfoModel,
		DistributedSearchGroupMixin
	) {

		var STANDALONE_ROLES = ['indexer', 'search_head', 'license_master'];

		var LocalInstanceModel = ServerInfoModel.extend(
			{
			
				_distsearchGroups: null,

				initialize: function(attributes, options) {
					ServerInfoModel.prototype.initialize.call(this, attributes, options);

					this._distsearchGroups = options.distsearchGroups;

					if (!this._distsearchGroups) {
						throw Error("LocalInstanceModel requires a distsearchGroups argument");
					}

					this.on('sync', function() {
						this.entry.set('name', '');
						DistributedSearchGroupMixin.initializeDistributedSearchGroups.call(
							this,
							this._distsearchGroups,
							function() {
								return "localhost:localhost";
							}
						);
					}, this);
                    
                    // set flags by default
                    this.entry.content.set('type', 'localInstance');
                    this.entry.content.set('state', 'Configured');
                    this.entry.content.set('status-toggle', 'Enabled');

                    this.entry.content.set("errorMessages", []);
					this.entry.content.set("warningMessages", []);
				},

				save: function() {
					return $.when.apply($, _.map(
						this._distsearchGroups.models,
						function(model) { return model.save(); }
					));
				},

				hasOverrides: function() {
					return false;
				},

				isConfigured: function() {
					return true;
				},

				requiredRoles: STANDALONE_ROLES

			},
			DistributedSearchGroupMixin.staticMembers
		);

		_.extend(LocalInstanceModel.prototype, DistributedSearchGroupMixin);

		return LocalInstanceModel;
	}
);
