// Note: Ideally this file would exist in js/views/models/services/distributed/peers
// But I will leave it here until that endpoint is mature enough to support all necessary features

define(
	[
		'jquery',
		'underscore',
		'backbone',
		'models/SplunkDBase',
		'models/shared/managementconsole/Asset',
		'models/shared/managementconsole/DistsearchGroup',
		'collections/shared/FlashMessages',
		'mixins/managementconsole/DistributedSearchGroup'
	],
	function(
		$$$, // Mo' money mo' problems
		_, 
		Backbone,
		SplunkDBaseModel,
		AssetModel,
		DistsearchGroupModel,
		FlashMessagesCollection,
		DistributedSearchGroupMixin
	) {

		var PeerModel = SplunkDBaseModel.extend(
			{
				url: 'search/distributed/peers',
				_hasOverrides: false,

				initialize: function() {
					SplunkDBaseModel.prototype.initialize.apply(this, arguments);

					// check instance against configured list to determine whether instance is 'configured'
                    this.entry.content.set('configured', (function() {
                        var settingsAsset = this.collection.assets.find(function(asset) {
                            return asset.entry.get('name') === 'settings';
                        });
                        var configuredPeers = settingsAsset.entry.content.get('configuredPeers');
                        configuredPeers = $$$.trim(configuredPeers) ? configuredPeers.split(',') : [];
                        return _.contains(configuredPeers, this.entry.get('name')); 
                    }.bind(this))());

                    // check instance against blacklist to determine whether instance is 'blacklisted'
                    this.entry.content.set('blackListed', (function() {
                        var settingsAsset = this.collection.assets.find(function(asset) {
                            return asset.entry.get('name') === 'settings';
                        });
                        var blackList = settingsAsset.entry.content.get('blackList');
                        blackList = $$$.trim(blackList) ? blackList.split(',') : [];
                        return _.contains(blackList, this.entry.get('name'));
                    }.bind(this))());
				
                    if(!this.entry.content.get('configured')) {
                    	this.entry.content.set('state', 'New');
                    } else {
                    	this.entry.content.set('state', 'Configured');
                    }

                    if(!this.entry.content.get('blackListed')) {
                    	this.entry.content.set('status-toggle', 'Enabled');
                    } else {
                    	this.entry.content.set('status-toggle', 'Disabled');
                    }

                    // if instance is not configured, then it must be 'New' and is 'Enabled' by default
                    // if(!this.entry.content.get('configured')) {
                    //     this.entry.content.set('state', 'New');
                    //     this.entry.content.set('status-toggle', 'Enabled');
                    //     this.resetInitialInternalServerRoles();
                    // } else {
                    // 	// else instance has been configured
                    //     this.entry.content.set('state', 'Configured');

                    //     // in instance is 'blackListed' then it should be 'Disabled'
                    //     if(this.entry.content.get('blackListed')) {
                    //         this.entry.content.set('status-toggle', 'Disabled');

                    //     // else it should be 'Enabled'
                    //     } else {
                    //         this.entry.content.set('status-toggle', 'Enabled');
                    //     }
                    // }

                    // needs to check whether instance is 'Disabled' before initializing
                    DistributedSearchGroupMixin.initializeDistributedSearchGroups.call(
                        this,
                        this.collection.distsearches,
                        function() {
                            return this.entry.get('name');
                        }.bind(this)
                    );

					this.entry.content.on('change:host', this._updateHost, this);
					this.entry.content.on('change:host_fqdn', this._updateHostFqdn, this);

					var asset = this._getAsset();
					if (asset) {
						_.each(['host', 'host_fqdn'], function(attr) {
							if (asset.entry.content.get(attr)) {
								this.entry.content.set(attr, asset.entry.content.get(attr));
								this._hasOverrides = true;
							}
						}, this);
					}

					this.entry.content.set("errorMessages", []);
					this.entry.content.set("warningMessages", []);
                },
				save: function() {
					var deferred = [];
					var asset = this._getAsset();
					if (asset) {
						deferred.push(asset.save());
					}
					// TODO this really needs to be transactional.
					// How do I deal with this?
					var toDestroy = [];
					_.each(this.collection.distsearches.models, function(model) {
						if (model.isCustomGroup() && model.getServers().length === 0) {
							toDestroy.push(model);
						} else {
							deferred.push(model.save());
						}
					});

					_.each(toDestroy, function(model) {
						deferred.push(model.destroy());
					});

					return $$$.when.apply($$$, deferred);
				},
                
                hasOverrides: function() {
					return this._hasOverrides;
				},

				isConfigured: function() {
					var settingsAsset = this.collection.assets.find(function(asset) { return asset.entry.get('name') === 'settings'; });

					if (!settingsAsset) {
						return false;
					}

					var configuredPeers = settingsAsset.entry.content.get('configuredPeers');
					configuredPeers = $$$.trim(configuredPeers) ? configuredPeers.split(',') : [];

					return _.contains(configuredPeers, this.entry.get('name'));
				},

				// called when instance is disabled. strips the instance of all its configured and custom groups
                removeAllServerRoles: function() {
                    this.entry.content.set('active_server_roles', []);
                    this.entry.content.set('tags', []);
                },

				_updateHost: function() {
					var asset = this._getAsset(true);
					asset.entry.content.set('host', this.entry.content.get('host'));
				},

				_updateHostFqdn: function() {
					var asset = this._getAsset(true);
					asset.entry.content.set('host_fqdn', this.entry.content.get('host_fqdn'));
				},

				_getAsset: function(create) {
					create = _.isUndefined(create) ? false : true;

					var asset = this.collection.assets.find(function(asset) {
						return asset.entry.get('name') === this.entry.get('name');
					}, this);

					if (!asset && create) {
						asset = new AssetModel();
						asset.entry.set('name', this.entry.get('name'));
						asset.entry.content.set('host', this.entry.content.get('host'));
						asset.entry.content.set('host_fqdn', this.entry.content.get('host_fqdn'));
						this.collection.assets.add(asset);
					}

					return asset;
				}
			},

			DistributedSearchGroupMixin.staticMembers
		);
		_.extend(PeerModel.prototype, DistributedSearchGroupMixin);

		return PeerModel;
	}
);
