define(
    [
        'underscore',
        'jquery',
        'routers/Base',
        'models/Base',
        'models/search/Report',
        'models/classicurl',
        'collections/services/authorization/Roles',
        'collections/services/admin/Alerts',
        'views/alert/Master',
        'views/shared/documentcontrols/dialogs/permissions_dialog/Master',
        'views/shared/alertcontrols/dialogs/edit_dialog/Master',
        'views/shared/alertcontrols/dialogs/ActionsDialog',
        'util/splunkd_utils'
    ],
    function(
        _,
        $,
        BaseRouter,
        BaseModel,
        AlertModel,
        classicurlModel,
        RolesCollection,
        AlertsAdminCollection,
        AlertView,
        PermissionsDialog,
        EditTypeDialog,
        EditActionsDialog,
        splunkd_utils
    ){
        return BaseRouter.extend({
            initialize: function() {
                BaseRouter.prototype.initialize.apply(this, arguments);
                this.fetchUser = true;
                this.fetchAppLocals = true;
                
                // models
                this.alertModel = new AlertModel();
                this.stateModel = new BaseModel();

                // collections
                this.rolesCollection = new RolesCollection();
                this.alertsAdminCollection = new AlertsAdminCollection();
                //Set defaults for pagination
                this.alertsAdminCollection.fetchData.set({
                    count: 20,
                    offset: 0
                },{
                    silent:true
                });

                this.alertModel.entry.content.on('change:disabled', this.fetchAlertAdminCollection, this);

                //data init
                this.deferredRoles = this.rolesCollection.fetch();

                //refetch every minute (60000 milliseconds) if enabled
                //TODO: uncomment when fixed the fetching the alertsUrl problem
                /*setInterval(function(){
                    if(!this.alertModel.entry.content.get('disabled')) {
                        this.fetchAlertAdminCollection();
                    }
                }.bind(this), 60000);*/
            },
            initializeAndRenderAlertView: function() {
                this.alertView = new AlertView({
                    model: {
                        state: this.stateModel,
                        savedAlert: this.alertModel,
                        application: this.model.application,
                        appLocal: this.model.appLocal,
                        user: this.model.user,
                        serverInfo: this.model.serverInfo
                    },
                    collection: {
                        roles: this.rolesCollection,
                        alertsAdmin: this.alertsAdminCollection
                    }
                });

                this.pageView.$('.main-section-body').append(this.alertView.render().el);

                switch(classicurlModel.get('dialog')) {
                    case 'permissions':
                        this.showPermissionsDialog();
                        break;
                    case 'type':
                        this.showTypeDialog();
                        break;
                    case 'actions':
                        this.showActionsDialog();
                        break;
                }
            },
            showPermissionsDialog: function() {
                this.permissionsDialog = new PermissionsDialog({
                    model: {
                        document: this.alertModel,
                        nameModel: this.alertModel.entry,
                        user: this.model.user,
                        serverInfo: this.model.serverInfo
                    },
                    collection: this.rolesCollection,
                    onHiddenRemove: true,
                    nameLabel: _('Alert').t()
                });

                $("body").append(this.permissionsDialog.render().el);
                this.permissionsDialog.show();

                classicurlModel.save({ dialog: undefined }, { replaceState: true });

            },
            showTypeDialog: function() {
                this.editTypeDialog = new EditTypeDialog({
                    model:  {
                        report: this.alertModel,
                        application: this.model.application,
                        appLocal: this.model.appLocal,
                        user: this.model.user,
                        serverInfo: this.model.serverInfo
                    },
                    dialogType: 'edit',
                    onHiddenRemove: true
                });

                $("body").append(this.editTypeDialog.render().el);
                this.editTypeDialog.show();

                classicurlModel.save({ dialog: undefined }, { replaceState: true });

            },
            showActionsDialog: function() {
                this.editActionsDialog = new EditActionsDialog({
                    model:  {
                        report: this.alertModel,
                        application: this.model.application,
                        appLocal: this.model.appLocal,
                        user: this.model.user,
                        serverInfo: this.model.serverInfo
                    },
                    onHiddenRemove: true
                });

                $("body").append(this.editActionsDialog.render().el);
                this.editActionsDialog.show();

                classicurlModel.save({ dialog: undefined }, { replaceState: true });

            },
            page: function(locale, app, page) {
                BaseRouter.prototype.page.apply(this, arguments);

                classicurlModel.fetch({
                    success: function(model, response) {
                        var alertModelDeferred = $.Deferred();
                        this.alertModel.set('id', classicurlModel.get('s'));
                        if (this.alertModel.id) {
                            this.alertModel.fetch({
                                success: function(model, response) {
                                    alertModelDeferred.resolve();
                                }.bind(this),
                                error: function(model, response) {
                                    this.alertModel.trigger('error', this.alertModel, response);
                                    alertModelDeferred.resolve();
                                }.bind(this)
                            });
                        } else {
                            var noAlertIdError = splunkd_utils.createSplunkDMessage(
                                splunkd_utils.FATAL,
                                _("No alert was specified.").t());
                            this.alertModel.trigger("error", this.alertModel, noAlertIdError);
                            this.setPageTitle(_('Alert').t());
                            alertModelDeferred.resolve();
                        }
                        $.when(alertModelDeferred).then(function() {
                            var alertAdminFetchDeferred = this.fetchAlertAdminCollection();
                            this.setPageTitle(this.alertModel.entry.get('name')|| _('Alert').t());
                            $.when(alertAdminFetchDeferred, this.deferredRoles, this.deferreds.pageViewRendered).then(function() {
                                $('.preload').replaceWith(this.pageView.el);
                                this.initializeAndRenderAlertView();
                            }.bind(this));
                        }.bind(this));
                    }.bind(this)
                });
            },
            fetchAlertAdminCollection: function() {
                var alertsUrl = this.alertModel.entry.links.get('alerts');
                if (alertsUrl) {
                    this.alertsAdminCollection.url = this.alertModel.entry.links.get('alerts');
                    return this.alertsAdminCollection.fetch();
                } else {
                    return true;
                }
            }
        });
    }
);
