define(
[
    'jquery',
    'underscore',
    'backbone',
    'routers/BaseSearch',
    'views/authentication_users/Master',
    'models/datapreview/Preview',
    'models/knowledgeobjects/Sourcetype',
    'models/services/search/jobs/Result',
    'collections/knowledgeobjects/Sourcetypes',
    'collections/services/AppLocals',
    'collections/services/authentication/Users',
    'util/splunkd_utils',
    'splunk.util',
    'util/console'
],
function(
    $,
    _,
    Backbone,
    BaseSearch,
    MasterView,
    PreviewModel,
    SourcetypeModel,
    ResultModel,
    SourcetypesCollection,
    AppLocalCollection,
    UsersCollection,
    splunkdUtils,
    splunkUtil,
    console
){
    return BaseSearch.extend({
        initialize: function() {
            BaseSearch.prototype.initialize.apply(this, arguments);

            this.setPageTitle(_('Users').t());
            this.enableAppBar = false;
            //this.enableFooter = false;

            //this.fetchUser = false;
            this.fetchAppLocals = true;
            //this.fetchAppLocal = false; // TODO: uncomment when this is added in Base router

            // filter to capture all source model related attrs from url
            this.source_prefix = 'source.';
            this.source_regexes = [/^source\./];

            // models
            this.model.preview = new PreviewModel();
            this.model.sourceModel = new Backbone.Model();
            this.model.sourcetypeModel = new SourcetypeModel();
            // List of used models from superset of models instantiated by parent Base & BaseSearch routers:
            // this.model.appLocal
            // this.model.searchJob
            // this.model.report
            // this.model.classicUrl
            // this.model.result
            // this.model.summary
            // this.model.timeline
            // this.model.user
            // this.model.uiPrefs

            // collections
            this.collection.sourcetypesCollection = new SourcetypesCollection();
            this.collection.users = new UsersCollection();
            // List of used collections from superset of collections instantiated by parent Base & BaseSearch routers:
            // this.collection.appLocals
            // this.collection.eventRenderers
            // this.collection.selectedFields
            // this.collection.workflowActions

            // deferreds
            this.deferreds.sourcetypeModel = $.Deferred();
            this.deferreds.sourcetypesCollection = $.Deferred();
            this.deferreds.users = $.Deferred();
            // List of used deferred from superset of deferred instantiated by parent Base & BaseSearch routers:
            // this.deferreds.appLocals
            // this.deferreds.uiPrefs
            // this.deferreds.user
            // this.deferreds.eventRenderers
            // this.deferreds.workflowActions
            // this.deferreds.pageViewRendered

            // views
            this.masterView = new MasterView({
                model: this.model,
                collection: this.collection,
                deferreds: this.deferreds
            });

            // TODO remove - this is for debug only
            window.baseRouter = this;
        },
        page: function(locale, app, page) {
            BaseSearch.prototype.page.apply(this, arguments);

            this.baseActivateDeferred = $.Deferred();
            this.baseDeactivateDeferred = $.Deferred();
            
            if (!this.shouldRender) {
                this.deactivate();
            } else {
                this.baseDeactivateDeferred.resolve();
            }

            var appOwner = {
                app: this.model.application.get("app"),
                owner: this.model.application.get("owner")
            };

            // models
            if (this.deferreds.uiPrefs.state() !== 'resolved') {
                this.model.uiPrefs.bootstrap(this.deferreds.uiPrefs, this.model.application.get("page"), this.model.application.get("app"), this.model.application.get("owner"));
            }
            
            // collections
            // TODO: consider removing this collection fetch
            if (this.deferreds.eventRenderers.state() !== 'resolved') {
                this.collection.eventRenderers.fetch({
                    data: $.extend({}, appOwner, {
                        sort_key: 'priority',
                        sort_dir: 'desc',
                        count: -1
                    }),
                    success: function(model, response) {
                        this.deferreds.eventRenderers.resolve();
                    }.bind(this),
                    error: function(model, response) {
                        this.deferreds.eventRenderers.resolve();
                    }.bind(this)
                });
            }

            // TODO: consider removing this collection fetch
            if (this.deferreds.workflowActions.state() !== 'resolved') {
                this.collection.workflowActions.fetch({
                    data: $.extend({}, appOwner, {
                        sort_key: "name",
                        search: 'disabled=false',
                        count: -1
                    }),
                    success: function(model, response) {
                        this.deferreds.workflowActions.resolve();
                    }.bind(this),
                    error: function(model, response) {
                        this.deferreds.workflowActions.resolve();
                    }.bind(this)
                });
            }

            if (this.deferreds.sourcetypesCollection.state() !== 'resolved') {
                // grab list of sourcetypes to go in dropdown
                this.collection.sourcetypesCollection.fetch({
                    data: {
                        search: 'pulldown_type=1'
                    },
                    success: function(model, response) {
                        this.deferreds.sourcetypesCollection.resolve();
                    }.bind(this),
                    error: function(model, response) {
                        this.deferreds.sourcetypesCollection.resolve();
                    }.bind(this)
                });
            }

            if (this.deferreds.users.state() !== 'resolved') {
                this.collection.users.fetch({
                    data: {
                        count: -1
                    },
                    success: function(model, response) {
                        this.deferreds.users.resolve();
                    }.bind(this),
                    error: function(model, response) {
                        this.deferreds.users.resolve();
                    }.bind(this)
                });
            }

            // TODO: for debug only
            // wait until entire sourcetype collection is fetched
            $.when(this.deferreds.sourcetypesCollection).done(function(){
                var sourcetypeNames = this.collection.sourcetypesCollection.map(function(sourcetype) {
                    return sourcetype.entry.get('name');
                });
                console.log(sourcetypeNames);
            }.bind(this));

            $.when(this.baseDeactivateDeferred).then(function() {
                // data bootstrap
                this.bootstrapDeferred = $.Deferred();
                $.when(this.deferreds.uiPrefs).then(function() {
                    // grab query strings from url
                    this.model.classicUrl.fetch({
                        success: function(model, response) {
                            //remove values that should not be permalinked
                            this.model.classicUrl.set({
                                'display.events.timelineEarliestTime': undefined,
                                'display.events.timelineLatestTime': undefined,
                                'display.prefs.events.offset': undefined,
                                'display.prefs.statistics.offset': undefined,
                                'display.statistics.sortColumn': undefined,
                                'display.statistics.sortDirection': undefined,
                                'display.events.table.sortColumn': undefined,
                                'display.events.table.sortDirection': undefined
                            });

                            this.syncFromClassicURL();
                        }.bind(this)
                    });
                }.bind(this));

                $.when(
                    this.deferreds.user,
                    this.deferreds.eventRenderers,
                    this.deferreds.workflowActions,
                    this.deferreds.pageViewRendered,
                    this.bootstrapDeferred
                ).then(function(){
                    this.baseActivateDeferred.resolve();
                }.bind(this));

                $.when(this.deferreds.pageViewRendered).then(function(){
                    if (this.shouldRender) {
                        // insert the top bars
                        $('.preload').replaceWith(this.pageView.el);
                    }
                }.bind(this));
            }.bind(this));

            $.when(this.baseActivateDeferred).then(function() {
                // clear for takeoff: activate master view
                this.masterView.activate({deep: true});
                //window.setTimeout(function(){
                    this.activate();
                //}.bind(this), 2000);
                
                if (this.shouldRender) {
                    this.masterView.render().replaceContentsOf($('.main-section-body'));
                    //replace with application level event view:append
                    $(document).trigger("rendered");
                }
            }.bind(this));
        },
        activate: function() {
            // Events for source model
            this.model.sourceModel.on('change', function(model, options) {
                this.populateClassicUrlFromSource();
                this.startNewPreview(null, {
                    ignoreCurrentSourcetype: true,
                    applyDetectedSourcetype: true
                });
            }, this);

            // Events for sourcetype model
            this.model.sourcetypeModel.on('sync', function(model, resp, options) {
                this.populateReportFromSourcetype();
            }, this);

            this.model.sourcetypeModel.entry.content.on('change', function(model, options) {
                // skip handler if explicitly requested in passed-in arg or model is still parsing
                if (options && options.skipPreview) { return; }
                if (this.model.sourcetypeModel.isParsing()) { return; }
                console.log("[Datapreview:Sourcetype]");
                console.log(model.changedAttributes());
                // dispatch preview cycle using current props
                this.startNewPreview();
            }, this);

            this.model.sourcetypeModel.entry.on('change:name', function(model, value, options) {
                // skip handler if explicitly requested in passed-in arg or model is still parsing
                if (options && options.skipPreview) { return; }
                if (this.model.sourcetypeModel.isParsing()) { return; }
                // clear & fetch sourcetype model without dispatching new preview
                this.sourcetypeBootstrap(null, value);
                // dispatch preview cycle using new sourcetype id - ignores current sourcetype
                this.startNewPreview(null, {
                    applyPassedInSourcetype: true,
                    sourcetypeId: value
                });
            }, this);

            // Events for summary model
            // TODO: Consider listening to result.fields if summary does not get updated with every preview job
            this.model.summary.fields.on('reset', function() {
                this.populateSelectedFieldsFromSummary();
            }, this);

            // Events for result model
            this.model.result.on('forcefetch', function() {
                this.fetchResult();
            }, this);

            // Events for report model
            this.model.report.entry.content.on('change:display.prefs.events.offset change:display.events.maxLines change:display.prefs.events.count change:display.events.timelineEarliestTime change:display.events.timelineLatestTime change:display.general.type change:display.events.table.sortColumn change:display.events.table.sortDirection', _.debounce(function(model, value, options) {
                this.fetchResult();
            }, 0), this);

            // Events for searchJob model
            this.model.searchJob.on("prepared", function() {
                this.pollSearchJob();
            }, this);

            // Events for selectedFields collection
            this.collection.selectedFields.on('add remove reset', this.populateReportFromSelectedFields, this);
        },
        deactivate: function() {
            if (!this.shouldRender) {
                this.model.sourceModel.off(null, null, this);
                this.model.sourcetypeModel.off(null, null, this);
                this.model.sourcetypeModel.entry.content.off(null, null, this);
                this.model.summary.fields.off(null, null, this);
                this.model.result.off(null, null, this);
                this.model.report.entry.content.off(null, null, this);
                this.model.searchJob.off(null, null, this);
                this.collection.selectedFields.off(null, null, this);
            }
            this.masterView.deactivate({deep: true});
            BaseSearch.prototype.deactivate.apply(this, arguments);
        },
        syncFromClassicURL: function() {
            var sourcetypeIdFromUrl = this.model.classicUrl.get('sourcetype'),
                //TODO we are not filtering classicUrl as expected - catching sourcetype in here
                sourceAttrsFromUrl = this.model.classicUrl.filterByWildcards(this.source_regexes, {strip: this.source_prefix}),
                attrsFromUIPrefs = this.model.uiPrefs.entry.content.filterByWildcards(this.job_filter),
                sourcetypeFetchDeferred = $.Deferred(),
                reportFetchDeferred = $.Deferred(),
                previewCreateDeferred = $.Deferred();

            // set source attrs from url
            if (sourceAttrsFromUrl) {
                this.model.sourceModel.set(sourceAttrsFromUrl);
            }

            // merge attrs from the ui prefs into the report
            if (attrsFromUIPrefs) {
                // TODO: consider adding attrsFromUrl to report
                this.model.report.entry.content.set(attrsFromUIPrefs);
            }

            this.sourcetypeBootstrap(sourcetypeFetchDeferred, sourcetypeIdFromUrl);
            this.reportBootstrap(reportFetchDeferred);
            //this.jobBootstrap(jobFetchDeferred, jobIdFromUrl);

            // wait until url-specified sourcetype is fetched - if any
            $.when(sourcetypeFetchDeferred).done(function() {
                // preview using auto-detected sourcetype
                this.startNewPreview(previewCreateDeferred, {
                    applyDetectedSourcetype: true
                });
            }.bind(this));

            // wait until preview created and report model fetched
            $.when(previewCreateDeferred, reportFetchDeferred).done(function() {
                this.pollSearchJob();
                this.populateReportFromSourcetype();

                //this.populateClassicUrlFromSearchJob(); // TODO: populate url with new sid?
                this.bootstrapDeferred.resolve();

                // TODO remove - this is for debug only
                console.log(this.model.sourcetypeModel.entry.content.toJSON());
                console.log(this.model.sourcetypeModel.attributes);
            }.bind(this));
        },
        startNewPreview: function(deferredResponse, options) {
            console.log("[Datapreview:startNewPreview]");
            options || (options = {});
            var previewType = this.model.sourceModel.get('type'),
                previewInput = this.model.sourceModel.get('input'),
                previewXhr, sourcetypeId, props = {},
                defaults = {
                    ignoreCurrentSourcetype: false,
                    applyDetectedSourcetype: false,
                    applyPassedInSourcetype: false      // when set, overrides ignoreCurrentSourcetype setting
                };
            _.defaults(options, defaults);

            if (!previewInput) {
                // return & resolve immediately if no preview input found
                deferredResponse && deferredResponse.resolve();
                return;
            }

            // clear preview model with any remnant props
            this.model.preview.clear();

            // when just using passed-in sourcetype id
            if (options.applyPassedInSourcetype) {
                sourcetypeId = options.sourcetypeId;
            // when considering current sourcetype
            } else if (!options.ignoreCurrentSourcetype) {
                // apply current sourcetype explicit props in preview job
                //props = this.model.sourcetypeModel.entry.content.attributes;
                props = this.model.sourcetypeModel.getExplicitProps();
                sourcetypeId = this.model.sourcetypeModel.entry.get('name');
            }

            if (sourcetypeId) {
                // apply sourcetype id in preview job
                props.sourcetype = options.sourcetypeId;
            }

            // set preview model explicit props (if any)
            this.model.preview.setProps(props);

            // dispatch indexing preview job
            try {
                previewXhr = this.model.preview.preview(previewType, previewInput);
            } catch(e) {
                this.model.preview.trigger('error', this.model.preview, _(e.message).t());
                deferredResponse && deferredResponse.resolve();
                return;
            }

            $.when(previewXhr)
            .fail(function(){
                // create client error message if no server message
                if (! this.model.preview.error.get('messages')) {
                    this.model.preview.trigger('error', this.model.preview, _("Failed to load preview").t());
                }
                deferredResponse && deferredResponse.resolve();
            }.bind(this))
            .done(function() {
                // return immediately if no preview job created
                if (this.model.preview.isNew()) {
                    this.model.preview.trigger('error', this.model.preview, _("Failed to load preview").t());
                    deferredResponse && deferredResponse.resolve();
                    return;
                }

                // fetch new sourcetype if applying auto-detected sourcetype
                if (options.applyDetectedSourcetype) {
                    var sourcetypeIdNew = '';
                    // set to auto-detected sourcetype if different than current sourcetype,
                    // otherwise, if none detected, default to 'default' sourcetype
                    var preferredSourcetype = this.model.preview.getPreferredSourcetype();
                    if (preferredSourcetype && preferredSourcetype !== sourcetypeId) {
                        sourcetypeIdNew = preferredSourcetype;
                    } else {
                        sourcetypeIdNew = 'default';
                    }
                    // clear & fetch sourcetype model without dispatching new preview
                    this.sourcetypeBootstrap(null, sourcetypeIdNew);
                }

                // set search job based on preview job
                var jobIdFromPreview = this.model.preview.entry.get('name');
                this.model.searchJob.unregisterJobProgressLinksChild(null, this);
                this.model.searchJob.clear();
                this.model.searchJob.set("id", jobIdFromPreview);
                this.model.searchJob.fetch().always(function() {
                    deferredResponse && deferredResponse.resolve();
                });
            }.bind(this));
        },
        sourcetypeBootstrap: function(deferredResponse, sourcetypeId) {
            if (sourcetypeId) {
                // clear current sourcetype model without dispatching another preview
                if (! this.model.sourcetypeModel.isNew()) {
                    this.model.sourcetypeModel.clear({skipPreview: true});
                }
                // fetch specified sourcetype without dispatching another preview
                this.model.sourcetypeModel.set('id', sourcetypeId);
                this.model.sourcetypeModel.fetch({
                    data: {
                        app: this.model.application.get("app"),
                        owner: this.model.application.get("owner")
                    },
                    error: function(model, response) {
                        // TODO: remove if sourcetype permalink is not featured
                        // clear it from url if any
                        this.model.classicUrl.save({sourcetype: undefined}, {replaceState: true});
                    }.bind(this)
                }).always(function() {
                    deferredResponse && deferredResponse.resolve();
                });
            } else {
                deferredResponse && deferredResponse.resolve();
            }
        },
        jobBootstrap: function(deferredResponse, jobId) {
            if (jobId) {
                //only fetch the job if we have an id
                this.model.searchJob.set("id", jobId);
                this.model.searchJob.fetch({
                    error: function(model, response) {
                        // clear it from url if any
                        this.model.classicUrl.save({sid: undefined}, {replaceState: true});
                        this.model.searchJob.unset("id");
                        // save this error for when we make the job
                        this.jobBootstrapErrors = [
                             splunkdUtils.createMessageObject(
                                 splunkdUtils.WARNING,
                                 _("The job could not be loaded. A new job has started based on preview parameters.").t()
                             )
                        ];
                    }.bind(this)
                }).always(function() {
                    deferredResponse.resolve();
                });
            } else {
                deferredResponse.resolve();
            }
        },
        reportBootstrap: function(deferredResponse) {
            // fetch default report
            this.model.report.fetch({
                data: {
                    app: this.model.application.get("app"),
                    owner: this.model.application.get("owner")
                },
                success: function(model, response){
                    this.initializeReport();
                }.bind(this)
            }).always(function() {
                deferredResponse.resolve();
            });
        },
        populateClassicUrlFromSource: function() {
            var attrs = {
                'source.type': this.model.sourceModel.get('type'),
                'source.input': this.model.sourceModel.get('input')
            };
            this.model.classicUrl.save(attrs);
        },
        populateClassicUrlFromSourcetype: function() {
            var attrs = {
                'sourcetype': this.model.sourcetypeModel.entry.get('name')
            };
            this.model.classicUrl.save(attrs);
        },
        populateReportFromSourcetype: function() {
            // set tabular format for structured data
            this.model.report.entry.content.set({
                'display.events.type': this.model.sourcetypeModel.isStructuredDataFormat() ? 'table' : 'list'
            });
        },
        populateSelectedFieldsFromSummary: function() {
            var EventModel = ResultModel.Results.Result,
                fields = this.model.summary.fields,
                attrs = _.object(fields.pluck('name'), []),  // create hash of fields from array of field names
                result = new EventModel(attrs),              // dummy Event with all the interesting fields
                models = [];

            // collect non-system/time fields
            _.each(result.notSystemOrTime(), function(field) {
                models.push({name: field});
            }, this);

            this.collection.selectedFields.reset(models);
        },
        initializeReport: function() {
            // set default display type to be numbered list
            // set defaults for pagination and sorting, and disable drilldown
            this.model.report.entry.content.set({
                // numbered list of events
                'display.general.type': 'events',
                'display.events.type': 'list',
                'display.events.rowNumbers': true,
                // default pagination and sorting
                'display.prefs.events.offset': "0",
                'display.prefs.statistics.offset': "0",
                'display.events.table.sortDirection': 'asc',
                'display.statistics.sortDirection': 'asc',
                // turn off drilldown
                'display.events.list.drilldown': 'none',
                'display.events.raw.drilldown': 'none',
                'display.events.table.drilldown': '0',
                'display.statistics.drilldown': 'none',
                'display.visualizations.charting.drilldown': 'none'
            });
        },
        pollSearchJob: function() {
            if (!this.model.searchJob.isNew()) {
                if (!this.model.searchJob.isPreparing()) {
                    this.registerSearchJobFriends({
                        registerResultsJsonCols: false,
                        registerResultJsonRows: false,
                        registerTimeline: false
                    });
                }
                this.model.searchJob.startPolling();
            }
        },
        fetchResult: function(options) {
            console.log("[fetchResult]");
            options = options || {};
            $.extend(true, options, {
                data: {
                    field_list: ''   // fetch all fields for indexing preview
                }
            });
            return BaseSearch.prototype.fetchResult.call(this, options);
        }
    });
});
