define(
    [
        'underscore',
        'jquery',
        'backbone',
        'routers/BaseSearch',
        'models/search/Job',
        'util/splunkd_utils',
        'splunk.util'
    ],
    function(
        _,
        $,
        Backbone,
        BaseSearch,
        SearchJobModel,
        splunkd_utils,
        splunkUtil
    ) {
        return BaseSearch.extend({
            initialize: function(options) {
                options = options || {};
                BaseSearch.prototype.initialize.apply(this, arguments);
                this.loadingMessage = _('Loading...').t();
                this.requiresRoles = options.requiresRoles;
            },
            //Our only Action method
            page: function(locale, app, page) {
                BaseSearch.prototype.page.apply(this, arguments);
                
                this.baseActivateDeferred = $.Deferred();
                this.baseDeactivateDeferred = $.Deferred();
            
                if (!this.shouldRender) {
                    this.deactivate();
                } else {
                    this.baseDeactivateDeferred.resolve();
                }
                
                //Times collection first for the search bar
                if (this.deferreds.times.state() !== 'resolved') {
                    this.collection.times.fetch({
                        data: {
                            app: this.model.application.get("app"),
                            owner: this.model.application.get("owner"),
                            count: -1
                        },
                        success: function(model, response) {
                            this.deferreds.times.resolve();
                        }.bind(this),
                        error: function(model, response) {
                            this.deferreds.times.resolve();
                        }.bind(this)
                    });
                }
                
                //models
                if (this.deferreds.uiPrefs.state() !== 'resolved') {
                    this.model.uiPrefs.bootstrap(this.deferreds.uiPrefs, this.model.application.get("page"), this.model.application.get("app"), this.model.application.get("owner"));
                }
                
                if (this.deferreds.appLocal.state() !== 'resolved') {
                    this.model.appLocal.fetch({
                        url: splunkd_utils.fullpath(this.model.appLocal.url + "/" + encodeURIComponent(this.model.application.get("app"))),
                        data: {
                            app: this.model.application.get("app"),
                            owner: this.model.application.get("owner")
                        },
                        success: function(model, response) {
                            this.deferreds.appLocal.resolve();
                        }.bind(this),
                        error: function(model, response) {
                            this.deferreds.appLocal.resolve();
                        }.bind(this)
                    });
                }
                
                //other collections
                if (this.deferreds.eventRenderers.state() !== 'resolved') {
                    this.collection.eventRenderers.fetch({
                        data: {
                            app: this.model.application.get("app"),
                            owner: this.model.application.get("owner"),
                            sort_key: 'priority',
                            sort_dir: 'desc',
                            count: -1
                        },
                        success: function(model, response) {
                            this.deferreds.eventRenderers.resolve();
                        }.bind(this),
                        error: function(model, response) {
                            this.deferreds.eventRenderers.resolve();
                        }.bind(this)
                    });
                }

                if (this.deferreds.workflowActions.state() !== 'resolved') {
                    this.collection.workflowActions.fetch({
                        data: {
                            app: this.model.application.get("app"),
                            owner: this.model.application.get("owner"),
                            count: -1,
                            sort_key: "name",
                            search: 'disabled=false'
                        },
                        success: function(model, response) {
                            this.deferreds.workflowActions.resolve();
                        }.bind(this),
                        error: function(model, response) {
                            this.deferreds.workflowActions.resolve();
                        }.bind(this)
                    });
                }

                if (this.requiresRoles) {
                    if (this.deferreds.roles.state() !== 'resolved') {
                        this.collection.roles.fetch({
                            success: function(model, response) {
                                this.deferreds.roles.resolve();
                            }.bind(this),
                            error: function(model, response) {
                                this.deferreds.roles.resolve();
                            }.bind(this)
                        });
                    }
                } else {
                    this.deferreds.roles.resolve();
                }
                
                $.when(this.baseDeactivateDeferred).then(function(){
                    //set the defaults for the pagination and sorting
                    this.model.report.entry.content.set({
                        'display.prefs.events.offset': '0',
                        'display.prefs.statistics.offset': '0',
                        'display.events.table.sortDirection': 'asc',
                        'display.statistics.sortDirection': 'asc'
                    });
                    
                    //data bootstrap
                    this.bootstrapDeferred = $.Deferred();
                    
                    $.when(this.deferreds.uiPrefs).then(function() {
                        this.model.classicUrl.fetch({
                            success: function(model, response){
                                this.showAutoPause = this.model.classicUrl.get("auto_pause");
                                
                                //remove values that should not be permalinked
                                this.model.classicUrl.set({
                                    'display.events.timelineEarliestTime': undefined,
                                    'display.events.timelineLatestTime': undefined,
                                    'display.prefs.events.offset': undefined,
                                    'display.prefs.statistics.offset': undefined,
                                    'display.statistics.sortColumn': undefined,
                                    'display.statistics.sortDirection': undefined,
                                    'display.events.table.sortColumn': undefined,
                                    'display.events.table.sortDirection': undefined
                                });
                                
                                this.syncFromClassicURL();
                            }.bind(this)
                        });
                    }.bind(this));

                    $.when(
                        this.deferreds.roles,
                        this.deferreds.times,
                        this.deferreds.appLocal,
                        this.deferreds.user,
                        this.deferreds.eventRenderers,
                        this.deferreds.workflowActions,
                        this.deferreds.pageViewRendered,
                        this.deferreds.preloadReplaced,
                        this.deferreds.serverInfo,
                        this.bootstrapDeferred
                    ).then(function(){
                        this.baseActivateDeferred.resolve();
                    }.bind(this));                    
                }.bind(this));
            },
            syncFromClassicURL: function() {
                var jobFetchDeferred = $.Deferred(),
                    jobCreationDeferred = $.Deferred(),
                    reportFetchDeferred = $.Deferred(),
                    timeRangeDeferred = $.Deferred(),
                    reportIdFromUrl = this.model.classicUrl.get('s'),
                    searchFromUrl = splunkUtil.stripLeadingSearchCommand(this.model.classicUrl.get('q')),
                    jobIdFromUrl = this.model.classicUrl.get('sid'),
                    attrsFromUrl = this.model.classicUrl.filterByWildcards(this.url_filter, { allowEmpty: true }),
                    uiPrefsOffFromUrl = this.model.classicUrl.get('uiprefsoff'),
                    searchIsNew = true,
                    attrsFromJob = {},
                    attrsFromUIPrefs = {},
                    searchFromReport, searchFromJob, search, syncJobPreview;

                //for testing purposes we will not use UI prefs if the url tells us to
                if (!uiPrefsOffFromUrl) {
                    attrsFromUIPrefs = this.model.uiPrefs.entry.content.filterByWildcards(this.job_filter);
                } else {
                    attrsFromUIPrefs['display.events.type'] = 'list';
                }

                //we need to mediate the legacy attrs from the URL to the report
                this.mediateClassicUrlAttrsToReport(attrsFromUrl);

                //if we don't have a report, a job, or a search string to load then we should show initial help
                if (!reportIdFromUrl && !jobIdFromUrl && !searchFromUrl) {
                    //merge the attrs from the url into the report
                    this.model.report.entry.content.set($.extend({}, attrsFromUIPrefs, attrsFromUrl));

                    //need to bootstrap the timerange for the timerangepicker
                    this.timeRangeBootstrap(timeRangeDeferred);

                    //we need to show the initial help page with the metadata job
                    $.when(timeRangeDeferred).then(function() {
                        if (splunkUtil.normalizeBoolean(this.model.report.entry.content.get('display.prefs.enableMetaData'))) {
                            this.startMetaDataSearch(this.bootstrapDeferred);
                        } else {
                            this.bootstrapDeferred.resolve();
                        }
                    }.bind(this));
                    return;
                }

                this.reportBootstrap(reportFetchDeferred, jobCreationDeferred, reportIdFromUrl, searchFromUrl, jobIdFromUrl);
                this.jobBootstrap(jobFetchDeferred, jobCreationDeferred, reportIdFromUrl, searchFromUrl, jobIdFromUrl);

                $.when(reportFetchDeferred, jobFetchDeferred).then(function() {
                    searchFromReport = this.model.report.entry.content.get('search');
                    searchFromJob = splunkUtil.stripLeadingSearchCommand(this.model.searchJob.getSearch());
                    
                    //merge the attrs from the ui prefs into the report
                    if (this.model.report.isNew()) {
                        //the ui prefs should trump the report defaults    
                        this.model.report.entry.content.set($.extend({}, attrsFromUIPrefs));
                    } else {
                        //the values we get from the report should trump the ui pref values
                        var defaults = this.model.report.entry.content.toJSON();
                        _.defaults(defaults, attrsFromUIPrefs);
                        this.model.report.entry.content.set($.extend({}, defaults));
                    }

                    //merge the attrs from the job and the url into the report
                    if (this.model.searchJob.isNew()) {
                        if (searchFromUrl && ($.trim(searchFromUrl) !== $.trim(searchFromReport))) {
                            search = searchFromUrl;
                        } else {
                            search = searchFromReport;
                        }
                    } else {
                        if (searchFromUrl && ($.trim(searchFromUrl) !== $.trim(searchFromJob))) {
                            search = searchFromUrl;
                        } else {
                            searchIsNew = false;
                            search = searchFromJob;
                            
                            //get the attrs from the job for mediation
                            attrsFromJob = this.model.searchJob.entry.content.custom.filterByWildcards(this.job_filter);
                            attrsFromJob["display.page.search.mode"] = this.model.searchJob.getAdhocSearchMode();
                            attrsFromJob["dispatch.earliest_time"] = this.model.searchJob.getDispatchEarliestTimeOrAllTime();
                            attrsFromJob["dispatch.latest_time"] = this.model.searchJob.getDispatchLatestTimeOrAllTime();
                        }
                    }
                                    
                    if (searchIsNew) {
                        //merge the attrs from the url into the report
                        this.model.report.entry.content.set($.extend({}, attrsFromUrl, {search: search}));
                        this.startNewSearch(jobCreationDeferred, search, attrsFromUrl);
                    } else {
                        //make sure that the job has the same preview setting as the report
                        if (this.model.searchJob.entry.acl.canWrite()) {
                            syncJobPreview = this.model.searchJob.setPreview(splunkUtil.normalizeBoolean(this.model.report.entry.content.get('display.general.enablePreview')));
                        } else {
                            syncJobPreview = $.Deferred().resolve();
                        }
                        
                        //merge the attrs from the url and the job into the report
                        this.model.report.entry.content.set($.extend({}, attrsFromUrl, attrsFromJob, {search: search}));
                        
                        syncJobPreview.always(function(){
                            jobCreationDeferred.resolve();
                        }.bind(this));
                    }

                    this.populateSelectedFieldsFromReport();
                    this.timeRangeBootstrap(timeRangeDeferred);
                }.bind(this));

                $.when(jobCreationDeferred, timeRangeDeferred).then(function() {
                    if (!this.model.searchJob.isNew()){
                        if (!this.model.searchJob.isPreparing()) {
                            this.model.report.setSearchTab({ 
                                isTransforming: this.model.searchJob.isReportSearch(),
                                isUneventfulReportSearch: this.model.searchJob.isUneventfulReportSearch(),
                                canPatternDetect: this.model.user.canPatternDetect()
                            });
                            this.registerSearchJobFriends();
                        }
                        this.populateClassicUrlFromSearchJob();
                        this.model.searchJob.startPolling();
                    }
                    this.bootstrapDeferred.resolve();
                }.bind(this));
            },
            reportBootstrap: function(reportFetchDeferred, jobCreationDeferred, reportIdFromUrl, searchFromUrl, jobIdFromUrl) {
                if (reportIdFromUrl) {
                    this.model.report.set("id", this.applyModelFullPathToName(this.model.report, reportIdFromUrl));
                }
                
                //fetch the report no matter what
                //if we don't have an id then we need the defaults
                this.model.report.fetch({
                    data: {
                        app: this.model.application.get("app"),
                        owner: this.model.application.get("owner")
                    },
                    
                    success: function(model, response){
                        //set the report pristine before anything has been set on the report
                        this.model.reportPristine.setFromSplunkD(this.model.report.toSplunkD());
                        reportFetchDeferred.resolve();
                    }.bind(this),

                    error: function(model, response){
                        var errorResponse = response;

                        this.model.report.unset("id");
                        this.model.classicUrl.save({s: undefined}, {replaceState: true});

                        if (reportIdFromUrl && !searchFromUrl && !jobIdFromUrl){
                            //the user only provided a bad reportId
                            //we should initialize the UI with the bad report
                            this.model.report._onerror(this.model.report, response);
                            reportFetchDeferred.resolve();
                            jobCreationDeferred.resolve();
                        } else {
                            //fetch the report defaults and populate the report with them, and put the report into an error state
                            this.model.reportPristine.fetch({
                                success: function(model, response) {
                                    this.model.report.setFromSplunkD(this.model.reportPristine.toSplunkD());
                                    this.model.report.trigger("error", this.model.report, errorResponse);
                                    reportFetchDeferred.resolve();
                                }.bind(this),
                                error: function(model, response) {
                                    //this is extremely bad, it means we couldn't fetch _new
                                    reportFetchDeferred.resolve();
                                }.bind(this)
                            });
                        }
                    }.bind(this)
                });
            },
            jobBootstrap: function(jobFetchDeferred, jobCreationDeferred, reportIdFromUrl, searchFromUrl, jobIdFromUrl){
                if (jobIdFromUrl) {
                    //only fetch the job if we have an id
                    this.model.searchJob.set("id", jobIdFromUrl);
                    this.model.searchJob.fetch({
                        success: function(model, response){
                            jobFetchDeferred.resolve();
                        }.bind(this),

                        error: function(model, response){
                            this.model.classicUrl.save({sid: undefined}, {replaceState: true});
                            this.model.searchJob.unset("id");

                            if (jobIdFromUrl && !searchFromUrl && !reportIdFromUrl){

                                //the user only provided a bad jobId
                                //we should initialize the UI with the bad job
                                jobFetchDeferred.resolve();
                                jobCreationDeferred.resolve();
                            } else {
                                //save this error for when we make the job
                                this.jobBootstrapErrors = [
                                     splunkd_utils.createMessageObject(
                                         splunkd_utils.WARNING,
                                         _("The job could not be loaded. A new job has been started with the same search.").t()
                                     )
                                ];

                                //we have other searches to use
                                //don't broadcast an error
                                jobFetchDeferred.resolve();
                            }
                        }.bind(this)
                    });
                } else {
                    jobFetchDeferred.resolve();
                }
            },
            timeRangeBootstrap: function(timeRangeDeferred) {
                this.model.timeRange.save(
                    {
                        'earliest': this.model.report.entry.content.get('dispatch.earliest_time'),
                        'latest': this.model.report.entry.content.get('dispatch.latest_time')
                    },
                    {
                        validate: false,
                        success: function(model, response) {
                            timeRangeDeferred.resolve();
                        }.bind(this),
                        error: function(model, response) {
                            timeRangeDeferred.resolve();
                        }.bind(this)
                    }
                );
            }
        });
    }
);
