define([
            'underscore',
            'jquery',
            'routers/Base',
            'collections/services/data/props/Extractions',
            'collections/knowledgeobjects/Sourcetypes',
            'collections/services/authorization/Roles',
            'models/classicurl',
            'models/services/data/props/Extraction',
            'models/search/Job',
            'models/services/search/jobs/Summary',
            'models/services/search/jobs/Result',
            'views/field_extractor/FieldExtractorMaster',
            'views/shared/FlashMessages',
            'util/splunkd_utils',
            'util/field_extractor_utils',
            'splunk.util'
        ],
        function(
            _,
            $,
            BaseRouter,
            Extractions,
            Sourcetypes,
            Roles,
            classicurl,
            Extraction,
            Job,
            Summary,
            Result,
            FieldExtractorMaster,
            FlashMessages,
            splunkdUtils,
            fieldExtractorUtils,
            splunkUtils
        ) {

    var JOB_FETCH_ERROR = 'job-fetch-error',
        EXTRACTION_FETCH_ERROR = 'extraction-fetch-error',
        JOB_SOURCETYPE_ERROR = 'job-sourcetype-error';

    return BaseRouter.extend({

        initialize: function() {
            BaseRouter.prototype.initialize.apply(this, arguments);
            this.fetchUser = true;
            this.setPageTitle(_('Field Extractor').t());
            this.model.classicurl = classicurl;
            this.collection.extractions = new Extractions();
            this.collection.sourcetypes = new Sourcetypes();
            this.collection.roles = new Roles();
            this.collection.sourcetypes.fetchData.set({
                count: 200
            }, { silent: true });
            this.model.extraction = new Extraction();
        },

        page: function(locale, app, page) {
            BaseRouter.prototype.page.apply(this, arguments);
            this.model.classicurl.fetch().done(_(function() {
                // No matter what, we'll need the roles and apps.
                var bootstrapDfd = $.when(this.bootstrapRoles(), this.deferreds.pageViewRendered, this.deferreds.serverInfo, this.deferreds.user);
                // NOTE: all branches must ensure that this.sourcetype is set correctly (or correctly not set)
                //       when bootstrapDfd is resolved.
                if(this.model.classicurl.has('sid')) {
                    // When operating from an sid, we need to determine which sourcetypes should be available.
                    this.sid = this.model.classicurl.get('sid');
                    bootstrapDfd = $.when(
                        bootstrapDfd, 
                        // If we have a reference to a specific event, we will use its sourcetype so no need to fetch the list.
                        this.model.classicurl.has('offset') || this.fetchAllowedSourcetypes().then(_(function(allowedSourcetypes) {
                            // If there is only one sourcetype, just use it, no need to fetch the list (SPL-88198).
                            if(allowedSourcetypes.length === 1) {
                                this.sourcetype = allowedSourcetypes[0];
                                return $.Deferred().resolve();
                            }
                            return this.bootstrapSourcetypes(allowedSourcetypes);
                        }).bind(this))
                    );
                    if(this.model.classicurl.has('offset')) {
                        bootstrapDfd = $.when(
                            bootstrapDfd,
                            this.bootstrapEventByOffset().then(_(function(event) {
                                this.masterEvent = event.get('_raw')[0];
                                this.sourcetype = event.get('sourcetype')[0];
                            }).bind(this))
                        );
                    }
                }
                else {
                    // When loading an existing extraction, use its sourcetype.
                    if(this.model.classicurl.has('extractionId')) {
                        var extractionDfd = this.bootstrapExtractionById(this.model.classicurl.get('extractionId'));
                        bootstrapDfd = $.when(bootstrapDfd, extractionDfd.then(_(function() {
                            this.sourcetype = this.model.extraction.entry.content.get('stanza');
                        }).bind(this)));
                    }
                    // When not loading from an sid or an existing extraction, either use the sourcetype from the URL,
                    // or if there is not one, load all available sourcetypes.
                    else {
                        if(this.model.classicurl.has('sourcetype')) {
                            this.sourcetype = this.model.classicurl.get('sourcetype');
                        }
                        else {
                            bootstrapDfd = $.when(bootstrapDfd, this.bootstrapSourcetypes());
                        }
                    }
                }
                bootstrapDfd
                    .done(_(function() {
                        if(this.shouldRender) {
                            $('.preload').replaceWith(this.pageView.el);
                        }
                        if(this.masterView) {
                            this.stopListening(this.masterView);
                            this.masterView.remove();
                        }
                        this.masterView = new FieldExtractorMaster({
                            sourcetype: this.sourcetype,
                            sid: this.sid,
                            masterEvent: this.masterEvent,
                            model: {
                                application: this.model.application,
                                user: this.model.user,
                                extraction: this.model.extraction,
                                serverInfo: this.model.serverInfo
                            },
                            collection: {
                                extractions: this.collection.extractions,
                                sourcetypes: this.collection.sourcetypes,
                                roles: this.collection.roles
                            }
                        });
                        this.masterView.render().appendTo(this.pageView.$('.main-section-body'));
                        this.onAfterPageRender();
                    }).bind(this))
                    .fail(_(this.displayLoadError).bind(this));
            }).bind(this));
        },

        bootstrapSourcetypes: function(allowedSourcetypes) {
            var fetchData = this.model.application.pick('app', 'owner');
            if(allowedSourcetypes) {
                fetchData.search = _(allowedSourcetypes)
                    .map(function(sourcetype) { return 'name=' + splunkUtils.searchEscape(sourcetype); })
                    .join(' OR ');
            }
            return this.collection.sourcetypes.fetch({ data: fetchData }).then(
                _(function() {
                    if(this.collection.sourcetypes.length === 0) {
                        return $.Deferred().reject(JOB_SOURCETYPE_ERROR);
                    }
                }).bind(this),
                function() {
                    return JOB_SOURCETYPE_ERROR;
                }
            );
        },

        bootstrapRoles: function() {
            return this.collection.roles.fetch({ data: this.model.application.pick('app', 'owner') });
        },

        bootstrapExtractionById: function(id) {
            this.model.extraction.clear();
            this.model.extraction.set(this.model.extraction.idAttribute, id);
            return this.model.extraction.fetch().then(
                null,
                function() {
                    return EXTRACTION_FETCH_ERROR;
                }
            );
        },

        fetchAllowedSourcetypes: function() {
            var summary = new Summary();
            summary.set(
                summary.idAttribute,
                splunkdUtils.fullpath(
                    Job.prototype.url + '/' + this.sid + '/summary',
                    this.model.application.pick('app', 'owner')
                )
            );
            var summaryFetch = summary.fetch({ data: { top_count: 100, f: 'sourcetype' } });
            return summaryFetch.then(
                _(function() {
                    var sourcetypeSummary = summary.fields.get('sourcetype');
                    if(!sourcetypeSummary) {
                        return $.Deferred().reject(JOB_SOURCETYPE_ERROR);
                    }
                    return _(sourcetypeSummary.get('modes')).pluck('value');
                }).bind(this),
                function() {
                    return JOB_FETCH_ERROR;
                }
            );
        },

        bootstrapEventByOffset: function() {
            var result = new Result();
            result.set(
                result.idAttribute,
                splunkdUtils.fullpath(
                    Job.prototype.url + '/' + this.sid + '/events',
                    this.model.application.pick('app', 'owner')
                )
            );
            var fetchData = {
                offset: this.model.classicurl.get('offset'),
                count: 1,
                field_list: '_raw,sourcetype',
                max_lines: fieldExtractorUtils.MAX_EVENT_LINES
            };
            return result.fetch({ data: fetchData }).then(
                _(function() {
                    var eventInfo = result.results.at(0);
                    if(!eventInfo.has('sourcetype')) {
                        return $.Deferred().reject(JOB_SOURCETYPE_ERROR);
                    }
                    return eventInfo;
                }).bind(this),
                function() {
                    return JOB_FETCH_ERROR;
                }
            );
        },

        displayLoadError: function(errorType) {
            var messages = [],
                flashMessages = new FlashMessages();

            if(errorType === JOB_FETCH_ERROR) {
                messages.push(splunkUtils.sprintf(_('Unknown sid: %s').t(), this.sid));
            }
            else if(errorType === EXTRACTION_FETCH_ERROR) {
                messages.push(splunkUtils.sprintf(_('Extraction not found: %s').t(), this.model.extraction.id));
            }
            else if(errorType === JOB_SOURCETYPE_ERROR) {
                messages.push(splunkUtils.sprintf(_('The events associated with this job have no sourcetype information: %s').t(), this.sid));
            }
            _(messages).each(function(message) {
                flashMessages.flashMsgHelper.addGeneralMessage(_.uniqueId('field-extractor-error'), {
                    type: splunkdUtils.ERROR,
                    html: message
                });
            });
            $.when(this.deferreds.pageViewRendered).done(_(function() {
                this.pageView.replaceAll($('.preload'));
                flashMessages.render().appendTo(this.pageView.$('.main-section-body'));
            }).bind(this));
        },

        onAfterPageRender: function() {
            // If we're doing things based on an existing job, make sure to keep that job alive.
            if(this.sid) {
                var job = new Job({}, { delay: Job.DEFAULT_KEEP_ALIVE_INTERVAL, processKeepAlive: true });
                job.set(job.idAttribute, this.sid);
                job.fetch({ data: this.model.application.pick('app', 'owner') }).done(_(function() {
                    // In the success handler, poll the job if it's not already done.
                    if(!job.isDone()) {
                        job.startPolling();
                        job.stopKeepAlive();
                    }
                }).bind(this));
            }
        }

    });

});