define(
    [
        'underscore',
        'jquery',
        'backbone',
        'routers/Base',
        'collections/services/data/ui/Navs',
        'models/services/data/ui/Manager',
        'models/search/Dashboard',
        'models/services/data/ui/Pref',
        'models/classicurl',
        'views/home/Master',
        'views/shared/interstitialdialog/Master',
        'util/splunkd_utils'
    ], 
    function(
        _,
        $,
        Backbone,
        BaseRouter,
        NavsCollection,
        ManagerModel,
        DashboardModel,
        UIPrefModel,
        ClassicUrlModel,
        HomeView,
        InterstitialDialogView,
        splunkdUtils 
    ){
        return BaseRouter.extend({
            initialize: function() {
                BaseRouter.prototype.initialize.apply(this, arguments);
                var owner = this.model.application.get('owner');
                this.enableAppBar = false;
                this.enableFooter = false;
                this.showAppsList = false;
                this.fetchAppLocals = true;
                this.fetchUser = true;
                this.fetchUserPref = true; 
                this.model.dashboard = new DashboardModel();
                this.model.uiPref = new UIPrefModel();
                this.model.classicUrl = ClassicUrlModel;
                this.collection.navs = new NavsCollection();
                this.deferreds.appsSorted = $.Deferred();
                this.deferreds.uiPref = $.Deferred();
                this.deferreds.dashboard =  $.Deferred();
                this.deferreds.classicUrl = $.Deferred();
                this.setPageTitle(_('Home').t());
                this.canManageRemoteAppsDfd = $.Deferred();
                
                this.deferreds.navsCollection = this.collection.navs.fetch({
                     data: {
                        app: '-',
                        owner: owner,
                        count: -1
                    }
                });

				// this.model.classicUrl.fetch().done(function() {
					// this.deferreds.classicUrl.resolve();
				// }.bind(this));
				
                $.when(
                    this.deferreds.appLocals,
                    this.deferreds.userPref,
                    this.deferreds.navsCollection,
                    this.deferreds.uiPref,
                    this.deferreds.user,
                    this.deferreds.dashboard,
                    this.deferreds.managers,
                    this.deferreds.classicUrl
                ).then(function() {
                    this.collection.appLocals.sortWithString(this.model.userPref.entry.content.get('appOrder'));
                    if (this.model.user.canManageRemoteApps()) {
                        this.canManageRemoteAppsDfd.resolve();
                    } else {
                        this.canManageRemoteAppsDfd.reject();
                    }
                    this.views.home = new HomeView({
                        collection: {
                            apps: this.collection.appLocals,
                            appNavs: this.collection.navs,
                            managers: this.collection.managers
                        },
                        model: {
                            userPref: this.model.userPref,
                            uiPref: this.model.uiPref,
                            application: this.model.application,
                            user: this.model.user,
                            dashboard: this.model.dashboard
                        },
                        canManageRemoteAppsDfd: this.canManageRemoteAppsDfd
                    });
                    this.deferreds.appsSorted.resolve();
                    
                	// Should interstitial dialog be displayed? Only display it if a) it's cloud trial and b)it's admin user and 
                	// c) add_data flag is set or  d) coming from login page (and 'hideInterstitialDialog' is not set in user preferences)
                    var showInterstitialDialog = this.model.serverInfo.isCloudTrial() &&
                    							 this.model.user.canEditMonitor() && 
                    							 (this.model.classicUrl.get('add_data') ||
                    							 (this.model.userPref.entry.content.get('hideInterstitialDialog') != "1" &&
                								 (document.referrer && (document.referrer.indexOf("account/login") >= 0 || document.referrer.indexOf("account/logout") >= 0 || document.referrer.indexOf("/account/passwordchange") >= 0)
                								 )));
                    
                    if (showInterstitialDialog)
            			var dlg = new InterstitialDialogView({
            				owner: this.model.application.get('owner'),
            				onHiddenRemove: true
            			});                      
                }.bind(this));
                
                this.model.userPref.on('sync', function(){ //TODO: move within .then in constructor
                    this.collection.appLocals.sortWithString(this.model.userPref.entry.content.get('appOrder'));
                    this.collection.appLocals.trigger('ready'); // TODO check if this 'ready' trigger is still needed.
                }, this);
                
                $.when(this.deferreds.userPref).then(function() {
                    var dashboardID = this.model.userPref.entry.content.get('display.page.home.dashboardId');
                    if (dashboardID) {
                        this.model.dashboard.set('id', dashboardID);
                        this.model.dashboard.fetch({
                            success: function() {
                                this.deferreds.dashboard.resolve();
                            }.bind(this),
                            error: function() {
                                this.deferreds.dashboard.resolve();
                            }.bind(this)
                        });
                    } else {
                        this.deferreds.dashboard.resolve();
                    }
                }.bind(this));
            },
            page: function(locale, app, page) {
                BaseRouter.prototype.page.apply(this, arguments);
                this.model.uiPref.bootstrap(this.deferreds.uiPref, this.model.application.get('page'), this.model.application.get('app'), this.model.application.get('owner'));
                $.when(this.deferreds.pageViewRendered, this.deferreds.appsSorted).then(function() {
                    $('.preload').replaceWith(this.pageView.el);
                    this.pageView.$('.main-section-body').append(this.views.home.render().el);
                }.bind(this));
                
                // IB: Classic url fetch doesn't work inside initialize for some reason, but works in page()
				this.model.classicUrl.fetch().done(function() {
					this.deferreds.classicUrl.resolve();
				}.bind(this));                
            }
        });
    }
);
