define([
    'jquery',
    'underscore',
    'backbone',
    'routers/Base',

    //COLLECTIONS
    'collections/services/AppLocals',
    'collections/services/data/ui/Times',
    'collections/services/data/ui/Views',
    'collections/services/saved/Searches',
    'collections/services/data/ui/Navs',
    

    //MODELS
    'models/Base',
    'models/services/AppLocal',
    'models/services/authentication/User',
    'models/services/data/UserPref',
    'models/services/data/ui/Pref',
    'models/services/server/ServerInfo',
    'models/search/Report',
    'models/shared/TimeRange',
    'models/services/search/jobs/Result',
    'models/search/Job',

    //VIEWS
    'views/homeclassic/Home',
    'views/shared/Page',

    //TEMPLATES
    'contrib/text!views/homeclassic/Home.html',

    //OTHER
    'helpers/AppNav',
    'uri/route',
    'util/splunkd_utils'

],
function(
    $,
    _,
    Backbone,
    BaseRouter,

    AppsCollection,
    TimesCollection,
    ViewsCollection,
    SavedSearchesCollection,
    NavsCollection,
    
    BaseModel,
    AppLocalModel,
    UserModel,
    UserPrefModel,
    UIPrefsModel,
    ServerInfoModel,
    ReportModel,
    TimeRangeModel,
    ResultModel,
    SearchJobModel,

    HomeView,
    PageView,

    HomeTemplate,

    appNavParser,
    route,
    splunkd_utils
){
    return BaseRouter.extend({
        initialize: function() {
            this.SEARCH_APP = 'search';
            this.METADATA_POLLING_INTERVAL = 3000; //msec
            this.METADATA_AUTO_CANCEL = 30; //sec

            BaseRouter.prototype.initialize.apply(this, arguments);
            this.setPageTitle(_('Home').t());
            this.enablePageView = false;//home does it's own instantiation of page view due to it's apps fetch
            
            this.model.uiPrefs = new UIPrefsModel();
            this.model.user = new UserModel();
            this.model.report = new ReportModel();
            this.model.appLocal = new AppLocalModel();
            this.model.searchBarState = new BaseModel();
            this.model.metaDataJob = new SearchJobModel({}, {delay: this.METADATA_POLLING_INTERVAL});
            this.model.metaDataResult = new ResultModel();
            this.model.searchJob = new SearchJobModel();

            this.uiPrefsDfd = $.Deferred();
            this.userDfd = $.Deferred();
            this.reportDfd = $.Deferred();
            this.canEditMonitorDfd = $.Deferred();
            this.canManageRemoteAppsDfd = $.Deferred();

            var that = this,
                appNavsCollection = new Backbone.Collection(),
                appsCollection = new AppsCollection(),
                userPrefModel = new UserPrefModel();

            // SearchBar settings
            this.uiPrefsfilter = [
                "^display\..*"
            ];
            this.searchBarDefaults = {
                'display.prefs.searchContext': this.SEARCH_APP, // Search app is the default search context
                'dispatch.earliest_time': '', // default timerangePicker to All time
                'dispatch.latest_time': '', // default timerangePicker to All time
                'display.prefs.autoOpenSearchAssistant': true,
                'disable_timerange_trigger': true // don't trigger search on timerange change

            };

            this.model.application.on('change', function(applicationModel){
                var root = applicationModel.get('root');
                var owner = applicationModel.get('owner');
                var locale = applicationModel.get('locale');

                /*
                Setup search bar
                 */
                that.model.uiPrefs.bootstrap(
                    that.uiPrefsDfd,
                    that.model.application.get("page"),
                    that.model.application.get("app"),
                    that.model.application.get("owner")
                );
                that.uiPrefsDfd.done(function() {
                    _.defaults(that.model.uiPrefs.entry.content.attributes, that.searchBarDefaults);
                    that.model.uiPrefs.entry.content.set({
                        name: that.model.application.get('page')
                    });
                    that.saveUiPrefs();
                    that.model.searchBarState.set($.extend({}, that.model.uiPrefs.entry.content.toJSON()));
                    that.uiPrefsDfd.resolve();
                });

                that.model.report.fetch({
                    data: {
                        app: that.model.application.get("app"),
                        owner: that.model.application.get("owner")
                    }
                }).done(function() {
                    that.reportDfd.resolve();
                });

                that.model.searchBarState.on('change', function() {
                    var changed = $.extend(true, {}, that.model.searchBarState.changedAttributes());
                    that.saveUiPrefs(changed);
                });

                that.model.searchBarState.on('change:search', function() {
                    if (!that.model.searchBarState.get('search')) {
                        return;
                    }
                    var uriArgs = {};
                    $.each({
                        'search': 'q',
                        'dispatch.earliest_time': 'earliest',
                        'dispatch.latest_time': 'latest'
                    }, function(k,v) {
                        uriArgs[v] = that.model.searchBarState.get(k) || '';
                    });

                    var searchInApp = that.model.searchBarState.get('display.prefs.searchContext'),
                        nav = appNavsCollection.get(searchInApp),
                        searchView = nav ? nav.get('navData').searchView : false;
                    if (searchView){
                        //IE has an uri limit.
                        //So, to maintain a consistent behavior in all the browsers, we will dispatch the search first and then redirect to the target app
                        that.handleRedirect(root, locale, searchInApp, searchView, uriArgs);
                    }
                }, this);

                // Check user capabilities
                that.model.user.fetch({
                    url: splunkd_utils.fullpath(that.model.user.url + "/" + encodeURIComponent(that.model.application.get("owner"))),
                    data: {
                        app: that.model.application.get("app"),
                        owner: that.model.application.get("owner")
                    }
                }).done(function() {
                    that.userDfd.resolve();
                    if (that.model.user.canEditMonitor()) {
                        that.canEditMonitorDfd.resolve();
                    } else {
                        that.canEditMonitorDfd.reject();
                    }

                    if (that.model.user.canManageRemoteApps()) {
                        that.canManageRemoteAppsDfd.resolve();
                    } else {
                        that.canManageRemoteAppsDfd.reject();
                    }
                });

                that.model.appLocal.fetch({
                    url: splunkd_utils.fullpath(that.model.appLocal.url + "/" + encodeURIComponent(that.model.application.get("app"))),
                    data: {
                        app: that.model.application.get("app"),
                        owner: that.model.application.get("owner")
                    }
                });

                var viewsCollection = new ViewsCollection();
                var parseViewsDfd = $.Deferred();
                viewsCollection.fetch({data: {
                    app:'-',
                    owner: owner,
                    count:-1,
                    digest: 1
                }});
                viewsCollection.on('sync', function() {
                    parseViewsDfd.resolve(viewsCollection);
                });

                var savedSearchesCollection = new SavedSearchesCollection();
                var searchesDfd = $.Deferred();
                savedSearchesCollection.fetch({data:{app:'-', owner: owner, search:'is_visible=1 AND disabled=0', count:-1}}); //TODO: _with_new=1?
                savedSearchesCollection.on('sync', searchesDfd.resolve);

                var appsDfd = $.Deferred();
                appsCollection.fetch({
                    data: {
                        sort_key: 'name',
                        sort_dir: 'desc',
                        app: applicationModel.get('app'),
                        owner: owner,
                        search: 'visible=true AND disabled=0 AND name!=launcher',
                        count:-1
                    }
                });
                appsCollection.on('sync', appsDfd.resolve);

                var appSortDfd = $.Deferred();
                userPrefModel.fetch({data: {app:'user-prefs', owner: owner, count:-1}});
                userPrefModel.on('sync', function(){
                    appsDfd.done(function(){
                        appsCollection.sortWithString(userPrefModel.entry.content.get('appOrder'));
                        appSortDfd.resolve();
                        appsCollection.trigger('ready');
                    });
                });

                var navsDfd = $.Deferred();
                var navsCollection = new NavsCollection();
                navsCollection.on('sync', navsDfd.resolve);
                navsCollection.fetch({ data: { app: '-', owner: owner, count: -1} });

                $.when(
                    appsDfd,
                    parseViewsDfd,
                    searchesDfd,
                    navsDfd,
                    appSortDfd
                ).done(function(apps, parsedViewLabels, savedSearches, navsCollection){
                    savedSearches = savedSearches[0];
                    navsCollection = navsCollection[0];
                    apps = apps[0];
                    apps.each(function(app){
                        var appName = app.entry.get('name');
                        var appNavModel = new Backbone.Model({ id: appName, appName: appName, appLabel: app.entry.content.get('label') });
                        var navModel = navsCollection.get('/servicesNS/nobody/'+ appName +'/data/ui/nav/default');
                        appNavsCollection.add(appNavModel);
                        if(navModel){
                            var navData = appNavParser.parseNavModel(navModel, parsedViewLabels, savedSearches, applicationModel.get('root'));
                            appNavModel.set('navData', navData);
                        }
                    });
                });

                $.when(that.uiPrefsDfd, that.reportDfd).done(function(uiPrefs, report) {
                    that.model.report.entry.content.set($.extend({}, that.model.uiPrefs.entry.content.toJSON()));
                });
            });

            that.canEditMonitorDfd.done(function() {
                // Dispatch a new job for Stats
                that.metadataSearchDfd = $.Deferred();
                that.startMetaDataSearch(that.metadataSearchDfd);
            });

            var serverInfo = new ServerInfoModel();
            serverInfo.fetch();

            this.pageView = new PageView({
                splunkBar: true,
                footer: true,
                model: {
                    application: this.model.application,
                    config: this.model.config,
                    user: this.model.user,
                    serverInfo: this.model.serverInfo
                },
                collection: {
                    apps: appsCollection
                }
            });

            var timesCollection = new TimesCollection(),
                timeRangeWorking = new BaseModel(),
                timeRangeModel = new TimeRangeModel();

            // update time collection when context changes
            this.model.searchBarState.on('change:display.prefs.searchContext', function() {
                timesCollection.fetch({
                    data: {
                        app: this.model.searchBarState.get('display.prefs.searchContext'),
                        owner: this.model.application.get("owner"),
                        count: -1
                    }
                });
                timeRangeModel.save(
                    {
                        'earliest': this.model.searchBarState.get('dispatch.earliest_time'),
                        'latest': this.model.searchBarState.get('dispatch.latest_time')
                    },
                    {
                    }
                );
            }, this);


            this.homeView = new HomeView({
                collection: {
                    apps: appsCollection,
                    times: timesCollection,
                    navs: appNavsCollection
                },
                model: {
                    userPref: userPrefModel,
                    setting: this.model.searchBarState,
                    timeRangeWorking: timeRangeWorking,
                    timeRange: timeRangeModel,
                    serverInfo: serverInfo,
                    application: this.model.application,
                    appLocal: this.model.appLocal,
                    user: this.model.user,
                    metaDataResult: this.model.metaDataResult,
                    searchJob: this.model.searchJob,
                    report: this.model.report
                },
                canEditMonitorDfd: this.canEditMonitorDfd,
                canManageRemoteAppsDfd: this.canManageRemoteAppsDfd,
                uiPrefsDfd: this.uiPrefsDfd,
                userDfd: this.userDfd
            });


        },
        handleRedirect: function(root, locale, searchInApp, searchView, uriArgs) {
            var jobDispatchData = {
                    app: searchInApp,
                    search: uriArgs.q,
                    earliest_time: uriArgs.earliest,
                    ui_dispatch_app: searchInApp,
                    preview: this.model.report.entry.content.get('display.general.enablePreview'),
                    adhoc_search_level: this.model.report.entry.content.get('display.page.search.mode'),
                    owner: this.model.application.get('owner'),
                    indexedRealtime: this.model.report.entry.content.get('dispatch.indexedRealtime')
                };

            if(uriArgs.latest) {
                jobDispatchData.latest_time = uriArgs.latest;
            }

            this.model.searchJob.save({}, { data: jobDispatchData }).done(function() {
                document.location.href = route.page(root, locale, searchInApp, searchView, {data: { sid: this.model.searchJob.id }});
            }.bind(this));
        },
        saveUiPrefs: function(changedAttrs) {
            var data = {},
                saveUiPrefsDeferred = $.Deferred(),
                filteredAttrs,
                changedAttrsModel;
            if (changedAttrs && !_.isEmpty(changedAttrs)) {
                changedAttrs = $.extend(true, {}, changedAttrs);
                changedAttrsModel = new BaseModel(changedAttrs);
                filteredAttrs = changedAttrsModel.filterByWildcards(this.uiPrefsfilter, {allowEmpty: true});

                if (!_.isEmpty(filteredAttrs)){
                    if (this.model.uiPrefs.isNew()) {
                        data = {
                            app: this.model.application.get("app"),
                            owner: this.model.application.get("owner")
                        };
                    }
                    this.model.uiPrefs.entry.content.set(filteredAttrs);
                    this.model.uiPrefs.save({}, {
                        data: data,
                        success: function(model, response) {
                            saveUiPrefsDeferred.resolve();
                        }.bind(this),
                        error: function(model, response) {
                            saveUiPrefsDeferred.resolve();
                        }.bind(this)
                    });
                }
            }
            saveUiPrefsDeferred.resolve();
            return saveUiPrefsDeferred.promise();
        },
        startMetaDataSearch: function(deferred) {
            var search = '| metadata type=sourcetypes | search totalCount > 0';

            this.model.metaDataJob.save({}, {
                data: {
                    app: this.SEARCH_APP,
                    owner: this.model.application.get("owner"),
                    search: search,
                    preview: "true",
                    earliest_time: "rt",
                    latest_time: "rt",
                    auto_cancel: this.METADATA_AUTO_CANCEL,
                    max_count: 100000
                },
                success: function(model, response) {
                    deferred.resolve();
                    this.registerMetaDataJobFriends();
                    this.model.metaDataJob.startPolling();
                }.bind(this),
                error: function(model, response) {
                    deferred.resolve();
                }.bind(this)
            });
        },
        registerMetaDataJobFriends: function() {
            var resultsPreviewId = this.model.metaDataJob.entry.links.get('results_preview');

            if (resultsPreviewId) {
                this.model.metaDataResult.set("id", resultsPreviewId);
                this.fetchMetaDataResult();
                this.model.metaDataJob.on("jobProgress", this.fetchMetaDataResult, this);
            }
        },
        fetchMetaDataResult: function() {
            this.model.metaDataResult.safeFetch({
                data: {
                    count: 1,
                    search: "| stats sum(totalCount) as cnt, min(firstTime) as min, max(lastTime) as max"
                }
            });
        },
        page: function(locale, app, page) {
            BaseRouter.prototype.page.apply(this, arguments);
            $('.preload').replaceWith(this.pageView.render().el);
            this.pageView.$('.main-section-body').append(this.homeView.render().el);
        }
    });
});
