define(
	[
		'underscore',
		'jquery',
		'backbone',
		'routers/BaseListings',
		'collections/shared/managementconsole/Peers',
		'models/shared/managementconsole/LocalInstance',
		'views/managementconsole/Master'
	],
	function(
		_,
		$,
		Backbone,
		BaseListingsRouter,
		PeersCollection,
		LocalInstanceModel,
		ManagementConsoleConfigureView
	) {
		return BaseListingsRouter.extend({
			initialize: function() {
				BaseListingsRouter.prototype.initialize.apply(this, arguments);
				this.setPageTitle(_('Setup').t());
				this.loadingMessage = _('Loading...').t();

				this.stateModel.set({
					sortKey: 'peerName',
					sortDirection: 'asc',
					count: 30,
					offset: 0,
					search: '',
					changesMade: false
				});
				this.collection.peers = new PeersCollection();
				this.model.localInstance = new LocalInstanceModel({}, {
					distsearchGroups: this.collection.peers.distsearches
				});

				// TODO I don't know why I have to do this
				$.when(this.deferreds.appLocals).done(function() { 
					this.model.appLocal = this.collection.appLocals.find(function(appLocal) {
						return appLocal.entry.get('name') === this.model.application.get('app');
					}, this);
				}.bind(this));

				$(window).on('beforeunload', function() {
					if (this.stateModel.get('changesMade')) {
						return _("Please apply your changes!").t();
					}
				}.bind(this));

			},

			initializeAndRenderViews: function() {

				this.managementConsoleConfigureView = new ManagementConsoleConfigureView({
					model: {
						state: this.stateModel,
						appLocal: this.model.appLocal,
						application: this.model.application,
						localInstance: this.model.localInstance
					},
					collection: {
						peers: this.collection.peers
					}
				});
				this.pageView.$('.main-section-body').html(this.managementConsoleConfigureView.render().el);
			},

			fetchListCollection: function() {
				var doneDfd = $.Deferred();

				// Grab everything for the client side
				var data = {
					sort_dir: 'asc',
					sort_key: 'peerName',
					count: 0,
					offset: 0,
					search: 'disabled=0'
				};
				
				this.collection.peers.fetch({ data: data }).done(function() {
					this.stateModel.set(
						'changesMade', 
						this.collection.peers.contains(function(peer) { return !peer.isConfigured(); }) ||
						this.stateModel.get('changesMade')
					);
					this.model.localInstance.fetch().done(function() {
						doneDfd.resolve();
					}.bind(this)).fail(function() {
						doneDfd.reject();
					});
				}.bind(this));

				return doneDfd;
			}


		});
	}

);