define(
    [
        'jquery',
        'underscore',
        'collections/services/datamodel/DataModels',
        'models/classicurl',
        'models/services/datamodel/DataModel',
        'views/data_models/DataModels',
        'routers/Base',
        'controllers/DataModelsController',
        'controllers/DataObjectsController',
        'controllers/PivotController',
        'splunk.util',
        'models/pivot/PivotReport',
        'models/services/AppLocal',
        'models/services/authentication/User',
        'models/services/server/ServerInfo',
        'collections/services/data/ui/Times',    
        'views/shared/ResponsiveHeaderPage',
        'views/shared/FlashMessages',
        'helpers/pivot/ReportHistoryManager',
        'uri/route',
        'util/splunkd_utils'
    ],
    function(
        $,
        _,
        DataModelCollection,
        classicUrl,
        DataModel,
        DataModels,
        BaseRouter,
        DataModelsController,
        DataObjectsController,
        PivotController,
        splunkUtils,
        PivotReport,
        AppLocal,
        User,
        ServerInfo,
        Times,
        ResponsiveHeaderPage,
        FlashMessages,
        ReportHistoryManager,
        route,
        splunkdUtils
    ) {
        return BaseRouter.extend({

            initialize: function() {
                BaseRouter.prototype.initialize.apply(this, arguments);
                this.fetchUser = true;
                this.classicurl = classicUrl;
                this.timePresets = new Times();
                this.dataModels = new DataModelCollection();
                this.reportDefaults = new PivotReport();
                this.reportHistoryManager = new ReportHistoryManager();
            },

            page: function(locale, app, page) {
                // We'll be using a ResponsiveHeaderPage for Pivot
                if (this.model.serverInfo.isLite()) {
                    return;
                }
                var that = this;
                // We'll be using a ResponsiveHeaderPage for Pivot
                this.enablePageView = false;
                BaseRouter.prototype.page.apply(this, arguments);
                if(!this.pageView) {
                    this.$whenPageViewDependencies().done(function() {
                        that.pageView = new ResponsiveHeaderPage({
                            splunkBar: true,
                            footer: false,
                            section: 'pivot',
                            model: {
                                application: that.model.application,
                                appNav: that.model.appNav,
                                config: that.model.config,
                                user: that.model.user,
                                serverInfo: that.model.serverInfo,
                                tour: that.model.tour
                            },
                            modelRef: that.model,
                            collection: {
                                tours: that.collection.tours
                            },
                            deferreds: {
                                user: that.deferreds.user,
                                serverInfo: that.deferreds.serverInfo,
                                tour: that.deferreds.tour,
                                managers: that.collection.managers
                            }
                        });
                        that.pageView.render();
                        that.deferreds.pageViewRendered.resolve();
                    });
                }

                this.classicurl.fetch().done(function() {
                    // The strange order is necessary to have step 1 be the catch-all
                    // Step 3 of Pivot - have a model and an object (main Pivot page)
                    if(that.classicurl.get('q') || that.classicurl.get('s') || that.classicurl.get('seedSid')
                            || (that.classicurl.get('model') && that.classicurl.get('object'))) {
                        that.setPageTitle(_('Pivot').t());
                        // We're going to remove any controller that isn't an instance of PivotController here.
                        // This way, the user gets immediate feedback when transitioning between steps in the pivot process,
                        // but the page isn't blank when transitioning between states in step 3.
                        if(that.controller && !(that.controller instanceof PivotController)) {
                            that.controller.off(null, null, that);
                            that.controller.remove();
                        }

                        // create a "ready" deferred that will be passed down to the controller
                        var appOwner = { app: that.model.application.get('app'), owner: that.model.application.get('owner') };
                        that.readyDfd = $.when(
                            that.timePresets.fetch({
                                data: {
                                    app: that.model.application.get('app'),
                                    owner: that.model.application.get('owner'),
                                    count: -1
                                }
                            }),
                            that.dataModels.fetch({ data: $.extend({ concise: true }, appOwner) }),
                            that.deferreds.appLocal,
                            that.deferreds.user
                        );

                        if(that.flashMessages) {
                            that.flashMessages.remove();
                        }
                        that.flashMessages = new FlashMessages({
                            model: {
                                appLocal: that.model.appLocal,
                                user: that.model.user
                            },
                            collection: {
                                timePresets: that.timePresets
                            },
                            helperOptions: {
                                postProcess: function(messages) {
                                    // Remove the "Could not find object id" messages.
                                    // They are confusing because the word object has special meaning in the data model context.
                                    // Custom error messages are rendered below in `handleLoadFailure`
                                    return _(messages).filter(function(message) {
                                        return !/Could not find object id=/.test(message.get('html'));
                                    });
                                }
                            }
                        });
                        var flashMessagesHelper = that.flashMessages.flashMsgHelper;
                        that.dataModel = that.dataModel ? that.dataModel.clone() : new DataModel();
                        flashMessagesHelper.register(that.dataModel);

                        if(that.report) {
                            that.reportHistoryManager.unregister(that.report);
                        }
                        that.report = that.report ? that.report.clone() : new PivotReport();
                        flashMessagesHelper.register(that.report);
                        // we leave it up to the controller to register the report with the reportHistoryManager at the correct time
                        var controller = new PivotController({
                            model: {
                                application: that.model.application,
                                classicurl: that.classicurl,
                                dataModel: that.dataModel,
                                report: that.report,
                                reportDefaults: that.reportDefaults,
                                appLocal: that.model.appLocal,
                                user: that.model.user
                            },
                            collection: {
                                timePresets: that.timePresets,
                                dataModels: that.dataModels
                            },
                            reportHistoryManager: that.reportHistoryManager,
                            routerReadyDfd: that.readyDfd,
                            flashMessages: that.flashMessages
                        });
                        // expose these two here for unit testing
                        that.collectJob = controller.model.collectJob;
                        that.timeRange = controller.model.timeRange;

                        $('.main-section-body .load-failure-messages').remove();
                        controller.on('ready', function() {
                            if (that.controller) {
                                // TODO [sff] this is pretty hacky, but it prevents the viz config menu from losing its scroll position
                                if(that.controller.masterView) {
                                    controller.masterView.vizConfigMenuOffset = that.controller.masterView.vizConfigMenuOffset;
                                }
                                that.controller.off(null, null, that);
                                that.controller.remove();
                            }
                            that.controller = controller;
                            $.when(that.deferreds.pageViewRendered).done(function() {
                                that.pageView.replaceAll($('.preload'));
                                that.controller.masterView.render().appendTo($('.main-section-body'));
                            });
                        }, that);
                        controller.on('failed', function(errorType, redirectUrl) {
                            // if we have already rendered the page, update the errors in place
                            // if the current controller supports it
                            if(that.controller && that.controller instanceof PivotController) {
                                controller.off(null, null, that);
                                controller.remove();
                                that.controller.masterView.renderErrors(that.flashMessages);
                            }
                            // otherwise handle the page load failure
                            else {
                                that.handleLoadFailure(errorType, redirectUrl);
                            }
                        }, that);
                    }
                    // Step 2 of Pivot - model selected, but not object (data object selector)
                    else if(that.classicurl.get('model')) {
                        that.setPageTitle(_('Select a Data Object').t());
                        if(that.controller) {
                            that.controller.off(null, null, that);
                            that.controller.remove();
                        }
                        $('.main-section-body .load-failure-messages').remove();

                        that.controller = new DataObjectsController({
                            application: that.model.application
                        });

                        that.controller.on('ready', function() {
                            var controllerId = that.controller.cid;
                            $.when(that.deferreds.pageViewRendered).done(function() {
                                if(that.controller.cid === controllerId) {
                                    that.pageView.replaceAll($('.preload'));
                                    that.controller.dmExplorerView.render().appendTo($('.main-section-body'));
                                }
                            });
                        }, that);
                    }
                    // Step 1 of Pivot - no model or object selected (data model selector)
                    else {
                        that.setPageTitle(_('Select a Data Model').t());
                        if(that.controller) {
                            that.controller.off(null, null, that);
                            that.controller.remove();
                        }
                        $('.main-section-body .load-failure-messages').remove();

                        that.controller = new DataModelsController({
                            application: that.model.application
                        });

                        that.controller.on('ready', function() {
                            // It's possible for a new route to be fired between here and when the pageViewRendered promise
                            // is fulfilled.  In that case, the controller will have been swapped out with a new one, so if
                            // the ids don't match, bail out (SPL-90024).
                            var controllerId = that.controller.cid;
                            $.when(that.deferreds.pageViewRendered).done(function() {
                                if(that.controller.cid === controllerId) {
                                    that.pageView.replaceAll($('.preload'));
                                    that.controller.dataModelsView.render().appendTo($('.main-section-body'));
                                }
                            });
                        }, that);
                    }
                    
                    
                });
            },

            handleLoadFailure: function(errorType, redirectUrl) {
                if(errorType === PivotController.URL_ERROR) {
                    window.location.href = redirectUrl;
                    return;
                }
                var that = this,
                    $preloadPlaceholder = $('.preload');

                if($preloadPlaceholder.length > 0) {
                    $.when(this.deferreds.pageViewRendered).done(function() {
                        that.pageView.replaceAll($preloadPlaceholder);
                    });
                }

                var $failMessagesHolder = $('<div class="load-failure-messages"></div>').appendTo($('.main-section-body')),
                    $linksHolder = $('<div class="links-holder"></div>').appendTo($failMessagesHolder),
                    pageRouter = route.getContextualPageRouter(this.model.application);

                var addLink = function(text, href, klass, callback) {
                    var $anchor = $('<a></a>').addClass(klass).text(text).attr('href', href);
                    $linksHolder.append($anchor);

                    if(callback) {
                        callback.apply(null, arguments);
                    }
                };

                this.flashMessages.prependTo($failMessagesHolder);
                if(errorType === PivotController.DATA_MODEL_ERROR) {
                    addLink(_('Select New Data Model').t(), pageRouter.pivot(), "select-new-data-model", function() {
                        $("a.select-new-data-model").click(function(e) {
                            e.preventDefault();
                            classicUrl.clear().save({}, { trigger: true });
                        });
                    });
                    addLink(_('Manage Data Models').t(), pageRouter.data_model_manager());
                    this.flashMessages.flashMsgHelper.addGeneralMessage(_.uniqueId('pivot-search-error'), {
                        type: splunkdUtils.ERROR,
                        html: splunkUtils.sprintf(
                            _('Your pivot cannot complete because the following supporting data model does not exist or is not shared globally to all apps: %s').t(),
                            this.dataModel.id
                        )
                    });
                }
                else if(errorType === PivotController.OBJECT_ERROR) {
                    var dataModelId = this.dataModel.id;
                    addLink(_('Select New Object').t(), pageRouter.pivot({ data: { model: dataModelId } }), "select-new-object", function() {
                        $("a.select-new-object").click(function(e) {
                            e.preventDefault();
                            classicUrl.clear().save({ model: dataModelId }, { trigger: true });
                        });
                    });
                    addLink(_('Edit Objects').t(), pageRouter.data_model_editor({ data: { model: dataModelId } }));
                }
                else if(errorType === PivotController.REPORT_ERROR) {
                    addLink(_('Manage Reports').t(), pageRouter.reports());
                    this.flashMessages.flashMsgHelper.addGeneralMessage(_.uniqueId('pivot-search-error'), {
                        type: splunkdUtils.ERROR,
                        html: splunkUtils.sprintf(
                            _('The report %s either does not exist or does not have the correct permissions.').t(),
                            this.report.id
                        )
                    });
                }
                else if(errorType === PivotController.PIVOT_SEARCH_ERROR) {
                    this.flashMessages.flashMsgHelper.addGeneralMessage(_.uniqueId('pivot-search-error'), {
                        type: splunkdUtils.ERROR,
                        html: _('The search string could not be parsed into a valid pivot configuration.').t()
                    });
                    var reportContent = this.report.entry.content,
                        latestTime = classicUrl.get('latest') || reportContent.get('dispatch.latest_time'),
                        searchParams = {
                            q: classicUrl.get('q') || reportContent.get('s'),
                            earliest: this.classicurl.get('earliest') || reportContent.get('dispatch.earliest_time') || 0
                        };

                    if(latestTime) {
                        searchParams.latest = latestTime;
                    }
                    addLink(_('Open In Search').t(), pageRouter.search({ data: searchParams }));
                    if(this.dataModel.id) {
                        addLink(_('Select New Object').t(), pageRouter.pivot({ data: { model: dataModelId } }), "select-new-object", function() {
                            $("a.select-new-object").click(function(e) {
                                e.preventDefault();
                                classicUrl.clear().save({ model: dataModelId }, { trigger: true });
                            });
                        });
                    }
                    else {
                        addLink(_('Select New Data Model').t(), pageRouter.pivot(), "select-new-data-model", function() {
                            $("a.select-new-data-model").click(function(e) {
                                e.preventDefault();
                                classicUrl.clear().save({}, { trigger: true });
                            });
                        });
                    }
                }
            }
        });
    }
);
