define(
    [
        'underscore',
        'jquery',
        'backbone',
        'routers/BootstrapSearch',
        'views/report/Master',
        'views/shared/documentcontrols/dialogs/permissions_dialog/Master',
        'views/shared/reportcontrols/dialogs/schedule_dialog/Master',
        'views/shared/reportcontrols/dialogs/AccelerationDialog',
        'views/shared/reportcontrols/dialogs/embed_dialog/Master',
        'models/search/Report',
        'util/splunkd_utils',
        'splunk.util'
    ],
    function(
        _,
        $,
        Backbone,
        BootstrapSearch,
        ReportView,
        PermissionsDialog,
        ScheduleDialog,
        AccelerationDialog,
        EmbedDialog,
        ReportModel,
        splunkd_utils,
        splunkUtils
    ) {
        return BootstrapSearch.extend({
            initialize: function(options) {
                options = options || {};
                options.requiresRoles = true;
                BootstrapSearch.prototype.initialize.call(this, options);
                this.setPageTitle(_('Report').t());
                
                this.model.eventsViewerReport = new ReportModel();
            },
            //Our only Action method
            page: function(locale, app, page) {
                BootstrapSearch.prototype.page.apply(this, arguments);
                
                $.when(this.baseDeactivateDeferred, this.deferreds.times, this.deferreds.pageViewRendered).then(function(){
                    if (this.shouldRender) {
                        //insert the top bars
                        //this.pageView.$('.section-padded').remove();//remove once all pages migrated to Page view class correctly
                        this.initializeReportView();
                        $('.preload').replaceWith(this.pageView.el);
                        this.deferreds.preloadReplaced.resolve();
                    }
                }.bind(this));
                
                $.when(this.baseActivateDeferred).then(function() {
                    this.reportView.activate();
                    this.activate();
                    
                    if (this.shouldRender) {
                        this.setPageTitle(this.model.report.entry.get('name') || _('Report').t());
                        this.reportView.render().replaceContentsOf($('.main-section-body'));

                        switch(this.model.classicUrl.get('dialog')) {
                            case 'permissions':
                                this.showPermissionsDialog();
                                break;
                            case 'schedule':
                                this.showScheduleDialog();
                                break;
                            case 'acceleration':
                                this.showAccelerationDialog();
                                break;
                            case 'embed':
                                this.showEmbedDialog();
                                break;
                        }
                        $(document).trigger("rendered");
                    }
                }.bind(this));  
            },
            initializeReportView: function () {
                if (!this.reportView) {
                    this.reportView = new ReportView({
                        model: {
                            result: this.model.result,
                            summary: this.model.summary,
                            searchJob: this.model.searchJob,
                            resultJsonCols: this.model.resultJsonCols,
                            resultJsonRows: this.model.resultJsonRows,
                            report: this.model.report,
                            reportPristine: this.model.reportPristine,
                            eventsViewerReport: this.model.eventsViewerReport,
                            application: this.model.application,
                            timeRange: this.model.timeRange,
                            appLocal: this.model.appLocal,
                            user: this.model.user,
                            serverInfo: this.model.serverInfo
                        },
                        collection: {
                            times: this.collection.times,
                            selectedFields: this.collection.selectedFields,
                            eventRenderers: this.collection.eventRenderers,
                            roles: this.collection.roles,
                            workflowActions: this.collection.workflowActions
                        },
                        deferreds: {
                            user: this.deferreds.user,
                            appLocal: this.deferreds.appLocal,
                            times: this.deferreds.times,
                            uiPrefs: this.deferreds.uiPrefs,
                            roles: this.deferreds.roles,          
                            eventRenderers: this.deferreds.eventRenderers,
                            workflowActions: this.deferreds.workflowActions
                        }
                    });
                }
            },
            activate: function() {
                // populate eventViewerReport
                this.model.eventsViewerReport.setFromSplunkD(this.model.report.toSplunkD());

                //Events for the report model
                this.model.eventsViewerReport.entry.content.on('change:display.events.table.sortColumn change:display.events.table.sortDirection',_.debounce(function() {
                    this.model.report.entry.content.set({
                        'display.events.table.sortDirection': this.model.eventsViewerReport.entry.content.get('display.events.table.sortDirection'),
                        'display.events.table.sortColumn': this.model.eventsViewerReport.entry.content.get('display.events.table.sortColumn')
                    });
                }, 0), this);

                this.model.report.entry.content.on('change:display.prefs.events.offset change:display.events.maxLines change:display.events.raw.drilldown change:display.events.list.drilldown change:display.events.fields change:display.prefs.events.count change:display.events.timelineEarliestTime change:display.events.timelineLatestTime change:display.events.table.sortColumn change:display.events.table.sortDirection', _.debounce(function() {
                    this.fetchResult();
                }, 0), this);

                this.model.report.entry.content.on('change:display.event.fields', this.populateSelectedFieldsFromReport, this);
                this.collection.selectedFields.on('add remove reset', this.populateReportFromSelectedFields, this);

                this.model.report.entry.content.on('change:display.events.timelineEarliestTime change:display.events.timelineLatestTime change:display.events.fieldsFilter change:display.events.fieldCoverage', _.debounce(function() {
                    this.fetchSummary();
                }, 0), this);

                this.model.report.entry.content.on('change:display.events.timelineEarliestTime change:display.events.timelineLatestTime', _.debounce(function() {
                    this.fetchTimeline();
                }, 0), this);

                this.model.report.entry.content.on('change:display.prefs.events.count', function(){
                    if (this.model.report.entry.content.get('display.prefs.events.count')){
                        this.model.report.entry.content.set({'display.prefs.events.offset': '0'});
                    }
                }, this);

                this.model.report.entry.content.on('change:display.prefs.statistics.count', function(){
                    if (this.model.report.entry.content.get('display.prefs.statistics.count')){
                        this.model.report.entry.content.set({'display.prefs.statistics.offset': '0'});
                    }
                }, this);

                this.model.report.entry.content.on('change:display.general.enablePreview', function(){
                    if (splunkUtils.normalizeBoolean(this.model.report.entry.content.get('display.general.enablePreview'))) {
                        this.model.searchJob.enablePreview();
                    } else {
                        this.model.searchJob.disablePreview();
                    }
                }, this);

                this.model.report.entry.content.on('change:display.statistics.sortColumn change:display.statistics.sortDirection change:display.prefs.statistics.count change:display.prefs.statistics.offset', _.debounce(function() {
                    this.fetchResultJSONRows();
                }, 0), this);

                //arbitrators to determine when to push on the history stack
                this.model.report.entry.content.on('change', function(){
                    var changed = this.model.report.entry.content.changedAttributes(),
                        jobPopulationDeferred = $.Deferred(),
                        uiPrefsPopulationDeferred = $.Deferred();

                    //we have to delete the change to search so we don't override the search of the report
                    delete changed.search;
                    
                    this.populateJob(jobPopulationDeferred, changed);
                    this.populateUIPrefs(uiPrefsPopulationDeferred, changed);
                    
                    $.when(jobPopulationDeferred, uiPrefsPopulationDeferred).always(function() {
                        this.populateClassicUrl(changed);
                    }.bind(this));
                }, this);

                //Events for the reportPristine model
                this.model.reportPristine.on('change:id', function(){
                    this.model.classicUrl.save({ s:this.model.reportPristine.id }, { replaceState: true });
                    var payload = this.model.reportPristine.toSplunkD();
                    payload.entry[0].content = {};
                    this.model.report.setFromSplunkD(payload);
                    this.model.eventsViewerReport.setFromSplunkD(payload);
                }, this);

                this.listenTo(this.model.reportPristine.entry.content, 'change:dispatch.earliest_time change:dispatch.latest_time', _.debounce(function() {
                        this.model.report.entry.content.set({
                            'dispatch.earliest_time': this.model.reportPristine.entry.content.get('dispatch.earliest_time'),
                            'dispatch.latest_time': this.model.reportPristine.entry.content.get('dispatch.latest_time')
                        });
                }, 0));

                this.listenTo(this.model.reportPristine.entry.content, 'change:display.page.search.mode', _.debounce(function() {
                        this.model.report.entry.content.set({
                            'display.page.search.mode': this.model.reportPristine.entry.content.get('display.page.search.mode')
                        },
                        {silent:true});
                }, 0));

                //Events on the searchJob model
                this.model.searchJob.on("prepared", function(){
                    this.registerSearchJobFriends();
                    this.model.report.setDisplayType(this.model.searchJob.isReportSearch());
                }, this);
                
                this.model.searchJob.on('destroy', function() {
                    this.model.classicUrl.save(
                        {
                            sid: "",
                            "display.prefs.events.offset": undefined,
                            "display.prefs.statistics.offset": undefined,
                            "display.statistics.sortColumn": undefined,
                            "display.statistics.sortDirection": undefined,
                            "display.events.table.sortColumn": undefined,
                            "display.events.table.sortDirection": undefined,
                            auto_pause: ""
                        },
                        {
                            replaceState: true
                        }
                    );
                }, this);

                this.model.searchJob.on('reload', function() {
                    this.model.classicUrl.save({sid: undefined}, {trigger: true});
                }, this);

            },
            deactivate: function() {
                if (!this.shouldRender) {
                    this.model.report.off(null, null, this);
                    this.model.report.entry.content.off(null, null, this);
                    this.model.reportPristine.off(null, null, this);
                    this.model.reportPristine.entry.content.off(null, null, this);
                    this.model.eventsViewerReport.entry.content.off(null, null, this);
                    this.model.searchJob.off(null, null, this);
                }
                this.reportView.deactivate({deep: true});
                BootstrapSearch.prototype.deactivate.apply(this, arguments);
                this.model.eventsViewerReport.clear();
            },
            fetchResultJSONColumn: function() {
                if (this.model.resultJsonCols.id && splunkUtils.normalizeBoolean(this.model.report.entry.content.get('display.visualizations.show'))) {
                    if(this.model.report.entry.content.get('display.visualizations.type') === 'mapping') {
                        if(this.model.resultJsonCols.fetchData.get('search')) {
                            this.model.resultJsonCols.safeFetch();
                        }
                    }
                    else {
                        BootstrapSearch.prototype.fetchResultJSONColumn.apply(this);
                    }
                }
            },
            registerSearchJobFriends: function() {
                if(this.model.report.entry.content.get('display.visualizations.type') === 'mapping') {
                    // TODO [sff] should set the count from a report attribute here
                    this.model.resultJsonCols.fetchData.set({ show_metadata: false }, { silent: true });
                }
                BootstrapSearch.prototype.registerSearchJobFriends.apply(this, arguments);
            },
            showPermissionsDialog: function() {
                this.permissionsDialog = new PermissionsDialog({
                    model: {
                        document: this.model.reportPristine,
                        nameModel: this.model.reportPristine.entry,
                        user: this.model.user,
                        serverInfo: this.model.serverInfo
                    },
                    collection: this.collection.roles,
                    onHiddenRemove: true,
                    nameLabel: _('Report').t()
                });

                $("body").append(this.permissionsDialog.render().el);
                this.permissionsDialog.show();

                this.model.classicUrl.save({ dialog: undefined }, { replaceState: true });

            },
            showScheduleDialog: function() {
                this.scheduleDialog = new ScheduleDialog({
                    model: {
                        report: this.model.reportPristine,
                        application: this.model.application,
                        appLocal: this.model.appLocal,
                        user: this.model.user,
                        serverInfo: this.model.serverInfo
                    },
                    onHiddenRemove: true
                });

                $("body").append(this.scheduleDialog.render().el);
                this.scheduleDialog.show();

                this.model.classicUrl.save({ dialog: undefined }, { replaceState: true });

            },
            showAccelerationDialog: function() {
                this.accelerationDialog = new AccelerationDialog({
                    model: {
                        report: this.model.reportPristine,
                        application: this.model.application
                    },
                    onHiddenRemove: true
                });

                $("body").append(this.accelerationDialog.render().el);
                this.accelerationDialog.show();

                this.model.classicUrl.save({ dialog: undefined }, { replaceState: true });

            },
            showEmbedDialog: function() {
                this.embedDialog = new EmbedDialog({
                    model: {
                        report: this.model.reportPristine,
                        application: this.model.application,
                        appLocal: this.model.appLocal,
                        user: this.model.user
                    },
                    onHiddenRemove: true                    
                });
                
                $("body").append(this.embedDialog.render().el);
                this.embedDialog.show();

                this.model.classicUrl.save({ dialog: undefined }, { replaceState: true });
            }
        });
    }
);
