define(function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var SolidStrokeBrush = require("jgatt").graphics.brushes.SolidStrokeBrush;
	var ObservableProperty = require("jgatt").properties.ObservableProperty;
	var NumericAxisLabels = require("splunk/charting/NumericAxisLabels");
	var GraphicsVizBase = require("splunk/viz/GraphicsVizBase");

	return jg_extend(GraphicsVizBase, function(GridLines, base)
	{

		// Public Properties

		this.foregroundColor = new ObservableProperty("foregroundColor", Number, 0x000000)
			.writeFilter(function(value)
			{
				return !isNaN(value) ? Math.min(Math.max(Math.floor(value), 0x000000), 0xFFFFFF) : 0x000000;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderGraphicsPass");
			});

		this.axisLabels = new ObservableProperty("axisLabels", NumericAxisLabels, null)
			.onChanged(function(e)
			{
				var target = e.target;
				if ((target === this) || ((target instanceof NumericAxisLabels) && (e.event === target.labelsChanged)))
					this.invalidate("renderGraphicsPass");
			});

		// Private Properties

		this._lineBrush = null;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.addStyleClass("splunk-charting-GridLines");

			this._lineBrush = new SolidStrokeBrush(this.getInternal("foregroundColor"), 0.1, 1);
		};

		// Protected Methods

		this.renderGraphicsOverride = function(graphics, width, height)
		{
			graphics.clear();

			var axisLabels = this.getInternal("axisLabels");
			if (!axisLabels)
				return;

			var lineBrush = this._lineBrush;
			lineBrush.set("color", this.getInternal("foregroundColor"));

			var positions = axisLabels.get("positions");
			var numPositions = positions.length;
			var position;
			var y;
			for (var i = 0; i < numPositions; i++)
			{
				position = positions[i];
				y = Math.round(height * (1 - position));
				lineBrush.beginBrush(graphics);
				lineBrush.moveTo(0, y);
				lineBrush.lineTo(width, y);
				lineBrush.endBrush();
			}
		};

	});

});
