define(function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var NumberUtils = require("jgatt").utils.NumberUtils;

	return jg_extend(Object, function(LogScale, base)
	{

		// Private Properties

		this._base = 0;
		this._baseMultiplier = 0;

		// Constructor

		this.constructor = function(base)
		{
			if ((base != null) && (typeof base !== "number"))
				throw new Error("Parameter base must be a number.");

			this._base = ((base > 0) && (base < Infinity)) ? base : 10;
			this._baseMultiplier = Math.log(this._base);
		};

		// Public Getters/Setters

		this.base = function()
		{
			return this._base;
		};

		// Public Methods

		this.valueToScale = function(value)
		{
			if (this._base <= 1)
				return 0;

			var scale = 0;

			var isNegative = (value < 0);

			if (isNegative)
				value = -value;

			if (value < this._base)
				value += (this._base - value) / this._base;
			scale = Math.log(value) / this._baseMultiplier;

			scale = NumberUtils.toPrecision(scale, -1);

			if (isNegative)
				scale = -scale;

			return scale;
		};

		this.scaleToValue = function(scale)
		{
			if (this._base <= 1)
				return 0;

			var value = 0;

			var isNegative = (scale < 0);

			if (isNegative)
				scale = -scale;

			value = Math.exp(scale * this._baseMultiplier);
			if (value < this._base)
				value = this._base * (value - 1) / (this._base - 1);

			value = NumberUtils.toPrecision(value, -1);

			if (isNegative)
				value = -value;

			return value;
		};

	});

});
