define(function(require)
{

	var Leaflet = require("leaflet");
	var jg_extend = require("jg_global").jg_extend;
	var LatLon = require("splunk/mapping/LatLon");

	return jg_extend(Object, function(LatLonBounds, base)
	{

		// Public Static Methods

		LatLonBounds.fromLeaflet = function(latLngBounds)
		{
			var sw = latLngBounds.getSouthWest();
			var ne = latLngBounds.getNorthEast();
			return new LatLonBounds(sw.lat, sw.lng, ne.lat, ne.lng);
		};

		// Public Properties

		this.s = 0;
		this.w = 0;
		this.n = 0;
		this.e = 0;

		// Constructor

		this.constructor = function(s, w, n, e)
		{
			this.s = (s !== undefined) ? s : 0;
			this.w = (w !== undefined) ? w : 0;
			this.n = (n !== undefined) ? n : 0;
			this.e = (e !== undefined) ? e : 0;
		};

		// Public Methods

		this.getSW = function()
		{
			return new LatLon(this.s, this.w);
		};

		this.getSE = function()
		{
			return new LatLon(this.s, this.e);
		};

		this.getNW = function()
		{
			return new LatLon(this.n, this.w);
		};

		this.getNE = function()
		{
			return new LatLon(this.n, this.e);
		};

		this.getCenter = function()
		{
			return new LatLon((this.s + this.n) / 2, (this.w + this.e) / 2);
		};

		this.expand = function(latLon)
		{
			if (latLon.lat < this.s)
				this.s = latLon.lat;
			if (latLon.lat > this.n)
				this.n = latLon.lat;
			if (latLon.lon < this.w)
				this.w = latLon.lon;
			if (latLon.lon > this.e)
				this.e = latLon.lon;
		};

		this.contains = function(latLon)
		{
			return ((latLon.lat >= this.s) &&
			        (latLon.lat <= this.n) &&
			        (latLon.lon >= this.w) &&
			        (latLon.lon <= this.e));
		};

		this.normalize = function(center)
		{
			var s = this.s;
			if (s < -90)
				s = -90;
			else if (s > 90)
				s = 90;

			var n = this.n;
			if (n < s)
				n = s;
			else if (n > 90)
				n = 90;

			var centerLon = center ? center.lon : 0;
			var w = (this.w - centerLon);
			var e = (this.e - centerLon);
			if ((e - w) >= 360)
			{
				w = -180;
				e = 180;
			}
			else
			{
				w %= 360;
				if (w < -180)
					w += 360;
				else if (w > 180)
					w -= 360;

				e %= 360;
				if (e < -180)
					e += 360;
				else if (e > 180)
					e -= 360;

				if (e < w)
				{
					if (e > -w)
						w -= 360;
					else
						e += 360;
				}
			}
			w += centerLon;
			e += centerLon;

			return new LatLonBounds(s, w, n, e);
		};

		this.isFinite = function()
		{
			return (((this.s - this.s) === 0) &&
			        ((this.w - this.w) === 0) &&
			        ((this.n - this.n) === 0) &&
			        ((this.e - this.e) === 0));
		};

		this.equals = function(bounds)
		{
			return ((this.s == bounds.s) &&
			        (this.w == bounds.w) &&
			        (this.n == bounds.n) &&
			        (this.e == bounds.e));
		};

		this.clone = function()
		{
			return new LatLonBounds(this.s, this.w, this.n, this.e);
		};

		this.toString = function()
		{
			return "(" + this.s + ", " + this.w + ", " + this.n + ", " + this.e + ")";
		};

		this.toLeaflet = function()
		{
			// Leaflet.LatLng wraps the coordinates passed to the constructor
			// we must assign the values manually to avoid this

			var sw = new Leaflet.LatLng(0, 0);
			sw.lat = this.s;
			sw.lng = this.w;

			var ne = new Leaflet.LatLng(0, 0);
			ne.lat = this.n;
			ne.lng = this.e;

			return new Leaflet.LatLngBounds(sw, ne);
		};

	});

});
