define(function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var jg_mixin = require("jg_global").jg_mixin;
	var Property = require("jgatt").properties.Property;
	var FunctionUtils = require("jgatt").utils.FunctionUtils;
	var MRenderTarget = require("splunk/viz/MRenderTarget");

	return jg_extend(Object, function(LayerBase, base)
	{

		base = jg_mixin(this, MRenderTarget, base);

		// Public Static Constants

		LayerBase.METADATA_KEY = "__splunk_mapping_layers_LayerBase_metadata";

		// Public Properties

		this.map = new Property("map", Object, null, true);

		this.leafletLayer = null;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this._map_boundsChanged = FunctionUtils.bind(this._map_boundsChanged, this);

			this.leafletLayer = this.createLeafletLayer();
			if (!this.leafletLayer)
				throw new Error("Value returned from createLeafletLayer() must be non-null.");
		};

		// Public Methods

		this.render = function()
		{
			this.validatePreceding("renderPass");

			if (this.isValid("renderPass"))
				return;

			var map = this.getInternal("map");
			if (map)
				this.renderOverride(map);

			this.setValid("renderPass");
		};

		// Protected Methods

		this.createLeafletLayer = function()
		{
			throw new Error("Must implement method createLeafletLayer.");
		};

		this.renderOverride = function(map)
		{
		};

		this.onAddedToMap = function(map)
		{
			this.setInternal("map", map);

			map.addEventListener("boundsChanged", this._map_boundsChanged);

			this.invalidate("renderPass");
		};

		this.onRemovedFromMap = function(map)
		{
			map.removeEventListener("boundsChanged", this._map_boundsChanged);

			this.setInternal("map", null);
		};

		// Private Methods

		this._map_boundsChanged = function(e)
		{
			this.invalidate("renderPass");
		};

	});

});
