define(function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var Property = require("jgatt").properties.Property;
	var FunctionUtils = require("jgatt").utils.FunctionUtils;
	var MRenderTarget = require("splunk/viz/MRenderTarget");
	var VizBase = require("splunk/viz/VizBase");

	return jg_extend(VizBase, function(ControlBase, base)
	{

		base = jg_mixin(this, MRenderTarget, base);

		// Public Properties

		this.map = new Property("map", Object, null, true);

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.addStyleClass("splunk-mapping2-controls-ControlBase");

			this.setStyle({ position: "absolute" });

			this._map_boundsChanged = FunctionUtils.bind(this._map_boundsChanged, this);
		};

		// Public Methods

		this.render = function()
		{
			this.validatePreceding("renderPass");

			if (this.isValid("renderPass"))
				return;

			var map = this.getInternal("map");
			if (map)
				this.renderOverride(map);

			this.setValid("renderPass");
		};

		// Protected Methods

		this.renderOverride = function(map)
		{
		};

		this.onAddedToMap = function(map)
		{
			this.setInternal("map", map);

			map.addEventListener("boundsChanged", this._map_boundsChanged);

			this.invalidate("renderPass");
		};

		this.onRemovedFromMap = function(map)
		{
			map.removeEventListener("boundsChanged", this._map_boundsChanged);

			this.setInternal("map", null);
		};

		// Private Methods

		this._map_boundsChanged = function(e)
		{
			this.invalidate("renderPass");
		};

	});

});
