define(function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var Rectangle = require("jgatt").geom.Rectangle;
	var FunctionUtils = require("jgatt").utils.FunctionUtils;
	var ControlBase = require("splunk/mapping2/controls/ControlBase");
	var MAttributionTarget = require("splunk/mapping2/controls/MAttributionTarget");
	var Rect = require("splunk/vectors/Rect");
	var Text = require("splunk/vectors/Text");
	var Viewport = require("splunk/vectors/Viewport");

	return jg_extend(ControlBase, function(SVGAttributionControl, base)
	{

		// Private Properties

		this._layers = null;
		this._attributionViewport = null;
		this._attributionText = null;
		this._attributionRect = null;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.addStyleClass("splunk-mapping2-controls-SVGAttributionControl");

			this._map_layersChanged = FunctionUtils.bind(this._map_layersChanged, this);
			this._layer_attribution_changed = FunctionUtils.bind(this._layer_attribution_changed, this);

			this._layers = [];

			this._attributionViewport = new Viewport()
				.appendTo(this.element);

			this._attributionRect = new Rect()
				.fillColor(0xFFFFFF)
				.fillOpacity(0.7)
				.appendTo(this._attributionViewport);

			this._attributionText = new Text()
				.fillColor(0x333333)
				.fontFamily("Helvetica, Arial, sans-serif")
				.fontSize("11px")
				.dominantBaseline("text-before-edge")
				.appendTo(this._attributionViewport);
		};

		// Public Methods

		this.toSVGString = function()
		{
			this.validate();

			return this._attributionViewport.toSVGString();
		};

		// Protected Methods

		this.renderOverride = function(map)
		{
			var width = map.get("width");
			var height = map.get("height");

			var str = "";
			var layers = this._layers;
			var attribution;
			for (var i = 0, l = layers.length; i < l; i++)
			{
				attribution = layers[i].get(MAttributionTarget.attribution);
				if (attribution)
				{
					if (str)
						str += " ";
					str += attribution;
				}
			}

			var text = this._attributionText;
			text.text(str);

			var padding = str ? { left: 5, top: 1, right: 0, bottom: 1 } : { left: 0, top: 0, right: 0, bottom: 0 };

			var textBounds = text.getBounds();

			var rectWidth = Math.round(textBounds.width + padding.left + padding.right);
			var rectHeight = Math.round(textBounds.height + padding.top + padding.bottom);
			var rectX = Math.round(width - rectWidth);
			var rectY = Math.round(height - rectHeight);

			var textX = Math.round(rectX + padding.left);
			var textY = Math.round(rectY + padding.top);

			text.x(textX);
			text.y(textY);
			text.display(str ? null : "none");

			var rect = this._attributionRect;
			rect.x(rectX);
			rect.y(rectY);
			rect.width(rectWidth);
			rect.height(rectHeight);
			rect.display(str ? null : "none");

			var viewport = this._attributionViewport;
			viewport.width(width);
			viewport.height(height);
			viewport.viewBox(new Rectangle(0, 0, width, height));
		};

		this.onAddedToMap = function(map)
		{
			base.onAddedToMap.call(this, map);

			map.addEventListener("layersChanged", this._map_layersChanged);

			this._updateLayers();
		};

		this.onRemovedFromMap = function(map)
		{
			map.removeEventListener("layersChanged", this._map_layersChanged);

			base.onRemovedFromMap.call(this, map);

			this._updateLayers();
		};

		// Private Methods

		this._updateLayers = function()
		{
			var map = this.getInternal("map");
			var oldLayers = this._layers;
			var newLayers = this._layers = map ? map.getLayers() : [];
			var layer;
			var i, l;

			for (i = 0, l = oldLayers.length; i < l; i++)
				oldLayers[i].removeEventListener("attribution.changed", this._layer_attribution_changed);

			for (i = 0, l = newLayers.length; i < l; i++)
				newLayers[i].addEventListener("attribution.changed", this._layer_attribution_changed);

			this.invalidate("renderPass");
		};

		this._map_layersChanged = function(e)
		{
			this._updateLayers();
		};

		this._layer_attribution_changed = function(e)
		{
			this.invalidate("renderPass");
		};

	});

});
