define(function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var ObservableProperty = require("jgatt").properties.ObservableProperty;
	var MarkerBase = require("splunk/mapping2/layers/MarkerBase");
	var Circle = require("splunk/vectors/Circle");

	return jg_extend(MarkerBase, function(CircleMarker, base)
	{

		// Public Properties

		this.color = new ObservableProperty("color", Number, 0x000000)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value <= Infinity)) ? Math.min(Math.floor(value), 0xFFFFFF) : 0x000000;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		this.opacity = new ObservableProperty("opacity", Number, 1)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value <= Infinity)) ? Math.min(value, 1) : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		// Constructor

		this.constructor = function(latLon, size, color, opacity)
		{
			base.constructor.call(this, new Circle(), latLon, size);

			if (color != null)
				this.set("color", color);
			if (opacity != null)
				this.set("opacity", opacity);
		};

		// Protected Methods

		this.renderOverride = function(element, x, y, size)
		{
			element.x(x);
			element.y(y);
			element.radius(size / 2);
			element.fillColor(this.getInternal("color"));
			element.fillOpacity(this.getInternal("opacity"));
		};

	});

});
