define(function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var jg_mixin = require("jg_global").jg_mixin;
	var Point = require("jgatt").geom.Point;
	var ObservableProperty = require("jgatt").properties.ObservableProperty;
	var Property = require("jgatt").properties.Property;
	var LatLon = require("splunk/mapping2/LatLon");
	var VectorElement = require("splunk/vectors/VectorElement");
	var MRenderTarget = require("splunk/viz/MRenderTarget");

	return jg_extend(Object, function(MarkerBase, base)
	{

		base = jg_mixin(this, MRenderTarget, base);

		// Public Properties

		this.latLon = new ObservableProperty("latLon", LatLon, new LatLon())
			.readFilter(function(value)
			{
				return value ? value.clone() : new LatLon();
			})
			.writeFilter(function(value)
			{
				return (value && value.isFinite()) ? value.clone() : new LatLon();
			})
			.changedComparator(function(oldValue, newValue)
			{
				return !oldValue.equals(newValue);
			});

		this.size = new ObservableProperty("size", Number, 50)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value < Infinity)) ? value : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		this.markerLayer = new Property("markerLayer", Object, null, true);

		this.renderPriority = 1;

		this.element = null;

		this.metadata = null;

		// Private Properties

		this._position = null;

		// Constructor

		this.constructor = function(element, latLon, size)
		{
			if (element == null)
				throw new Error("Parameter element must be non-null.");
			if (!(element instanceof VectorElement))
				throw new Error("Parameter element must be an instance of splunk.vectors.VectorElement.");

			base.constructor.call(this);

			this.element = element;

			this.metadata = {};

			if (latLon != null)
				this.set("latLon", latLon);
			if (size != null)
				this.set("size", size);

			this._position = new Point();
		};

		// Public Methods

		this.render = function(position)
		{
			this.validatePreceding("renderPass");

			if (position == null)
				position = this._position;
			else if (!(position instanceof Point))
				throw new Error("Parameter position must be an instance of jgatt.geom.Point.");

			if (this.isValid("renderPass"))
			{
				if (this._position.equals(position))
					return;
				this.invalidate("renderPass");
			}

			if (this._position !== position)
				this._position = position.clone();

			this.renderOverride(this.element, position.x, position.y, this.getInternal("size"));

			this.setValid("renderPass");
		};

		// Protected Methods

		this.renderOverride = function(element, x, y, size)
		{
		};

		this.onAddedToLayer = function(markerLayer)
		{
			this.setInternal("markerLayer", markerLayer);
		};

		this.onRemovedFromLayer = function(markerLayer)
		{
			this.setInternal("markerLayer", null);
		};

	});

});
