define(function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var Dictionary = require("jgatt").utils.Dictionary;
	var Parser = require("splunk/parsers/Parser");
	var ParseUtils = require("splunk/parsers/ParseUtils");
	var StringParser = require("splunk/parsers/StringParser");

	return jg_extend(Parser, function(ArrayParser, base)
	{

		// Private Static Properties

		var _instances = new Dictionary();

		// Public Static Methods

		ArrayParser.getInstance = function(elementParser)
		{
			var instance = _instances.get(elementParser);
			if (!instance)
				instance = _instances.set(elementParser, new ArrayParser(elementParser));
			return instance;
		};

		// Protected Properties

		this.elementParser = null;

		// Constructor

		this.constructor = function(elementParser)
		{
			if (elementParser == null)
				throw new Error("Parameter elementParser must be non-null.");
			if (!(elementParser instanceof Parser))
				throw new Error("Parameter elementParser must be an instance of splunk.parsers.Parser.");

			this.elementParser = elementParser;
		};

		// Public Methods

		this.stringToValue = function(str)
		{
			var array = ParseUtils.prepareArray(str);
			if (!array)
				return null;

			var elementParser = this.elementParser;
			for (var i = 0, l = array.length; i < l; i++)
				array[i] = elementParser.stringToValue(array[i]);

			return array;
		};

		this.valueToString = function(value)
		{
			var array = (value instanceof Array) ? value : null;
			if (!array)
				return null;

			var str = "";

			var elementParser = this.elementParser;
			var elementValue;
			for (var i = 0, l = array.length; i < l; i++)
			{
				elementValue = array[i];
				if (str)
					str += ",";
				if (elementParser instanceof StringParser)
					str += ParseUtils.escapeString(elementParser.valueToString(elementValue));
				else
					str += elementParser.valueToString(elementValue);
			}

			return "[" + str + "]";
		};

	});

});
