define(function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var Dictionary = require("jgatt").utils.Dictionary;
	var Parser = require("splunk/parsers/Parser");
	var ParseUtils = require("splunk/parsers/ParseUtils");
	var StringParser = require("splunk/parsers/StringParser");

	return jg_extend(Parser, function(ObjectParser, base)
	{

		// Private Static Properties

		var _instances = new Dictionary();

		// Public Static Methods

		ObjectParser.getInstance = function(elementParser)
		{
			var instance = _instances.get(elementParser);
			if (!instance)
				instance = _instances.set(elementParser, new ObjectParser(elementParser));
			return instance;
		};

		// Protected Properties

		this.elementParser = null;

		// Constructor

		this.constructor = function(elementParser)
		{
			if (elementParser == null)
				throw new Error("Parameter elementParser must be non-null.");
			if (!(elementParser instanceof Parser))
				throw new Error("Parameter elementParser must be an instance of splunk.parsers.Parser.");

			this.elementParser = elementParser;
		};

		// Public Methods

		this.stringToValue = function(str)
		{
			var map = ParseUtils.prepareObject(str);
			if (!map)
				return null;

			var elementParser = this.elementParser;
			for (var key in map)
			{
				if (map.hasOwnProperty(key))
					map[key] = elementParser.stringToValue(map[key]);
			}

			return map;
		};

		this.valueToString = function(value)
		{
			var map = (value instanceof Object) ? value : null;
			if (!map)
				return null;

			var str = "";

			var elementParser = this.elementParser;
			for (var key in map)
			{
				if (map.hasOwnProperty(key))
				{
					if (str)
						str += ",";
					str += ParseUtils.escapeString(key) + ":";
					if (elementParser instanceof StringParser)
						str += ParseUtils.escapeString(elementParser.valueToString(map[key]));
					else
						str += elementParser.valueToString(map[key]);
				}
			}

			return "{" + str + "}";
		};

	});

});
