define(function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var Point = require("jgatt").geom.Point;
	var Graphics = require("jgatt").graphics.Graphics;
	var ObservableProperty = require("jgatt").properties.ObservableProperty;
	var PropertyComparator = require("jgatt").utils.PropertyComparator;
	var ValidatePass = require("jgatt").validation.ValidatePass;
	var VizBase = require("splunk/viz/VizBase");

	return jg_extend(VizBase, function(GraphicsVizBase, base)
	{

		// Public Passes

		this.renderGraphicsPass = new ValidatePass("renderGraphics", 2, new PropertyComparator("renderGraphicsPriority"));

		// Public Properties

		this.x = new ObservableProperty("x", Number, 0)
			.writeFilter(function(value)
			{
				return ((value > -Infinity) && (value < Infinity)) ? value : 0;
			})
			.setter(function(value)
			{
				this.setStyle({ left: value + "px" });
			});

		this.y = new ObservableProperty("y", Number, 0)
			.writeFilter(function(value)
			{
				return ((value > -Infinity) && (value < Infinity)) ? value : 0;
			})
			.setter(function(value)
			{
				this.setStyle({ top: value + "px" });
			});

		this.width = new ObservableProperty("width", Number, 0)
			.writeFilter(function(value)
			{
				return ((value > -Infinity) && (value < Infinity)) ? value : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderGraphicsPass");
			});

		this.height = new ObservableProperty("height", Number, 0)
			.writeFilter(function(value)
			{
				return ((value > -Infinity) && (value < Infinity)) ? value : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderGraphicsPass");
			});

		this.renderGraphicsPriority = 0;
		this.graphics = null;

		// Constructor

		this.constructor = function(html)
		{
			base.constructor.call(this, html);

			this.addStyleClass("splunk-viz-GraphicsVizBase");

			this.setStyle({ position: "absolute", left: "0px", top: "0px" });

			this.graphics = new Graphics();
			this.graphics.appendTo(this.element);

			this.invalidate("renderGraphicsPass");
		};

		// Public Methods

		this.renderGraphics = function()
		{
			this.validatePreceding("renderGraphicsPass");

			if (this.isValid("renderGraphicsPass"))
				return;

			var width = this.getInternal("width");
			var height = this.getInternal("height");

			var graphics = this.graphics;
			graphics.setSize(width, height);

			this.renderGraphicsOverride(graphics, width, height);

			this.setValid("renderGraphicsPass");
		};

		this.localToGlobal = function(point)
		{
			if (point == null)
				throw new Error("Parameter point must be non-null.");
			if (!(point instanceof Point))
				throw new Error("Parameter point must be of type splunk.geom.Point.");

			var offset = this.$element.offset();
			return new Point(point.x + offset.left, point.y + offset.top);
		};

		this.globalToLocal = function(point)
		{
			if (point == null)
				throw new Error("Parameter point must be non-null.");
			if (!(point instanceof Point))
				throw new Error("Parameter point must be of type splunk.geom.Point.");

			var offset = this.$element.offset();
			return new Point(point.x - offset.left, point.y - offset.top);
		};

		// Protected Methods

		this.renderGraphicsOverride = function(graphics, width, height)
		{
		};

	});

});
