define(function(require)
{

	var jg_static = require("jg_global").jg_static;
	var jg_mixin = require("jg_global").jg_mixin;
	var MObservable = require("jgatt").events.MObservable;
	var MPropertyTarget = require("jgatt").properties.MPropertyTarget;
	var ObservableArrayProperty = require("jgatt").properties.ObservableArrayProperty;
	var ObservableProperty = require("jgatt").properties.ObservableProperty;
	var FunctionUtils = require("jgatt").utils.FunctionUtils;
	var MValidateTarget = require("jgatt").validation.MValidateTarget;
	var ValidatePass = require("jgatt").validation.ValidatePass;
	var Legend = require("splunk/charting/Legend");

	return jg_static(function(MDataTarget)
	{

		// Mixin

		this.mixin = function(base)
		{
			base = jg_mixin(this, MObservable, base);
			base = jg_mixin(this, MPropertyTarget, base);
			base = jg_mixin(this, MValidateTarget, base);
		};

		// Public Passes

		this.processDataPass = new ValidatePass("processData", 0.1);
		this.updateLegendLabelsPass = new ValidatePass("updateLegendLabels", 0.2);
		this.renderDataPass = new ValidatePass("renderData", 0.3);

		// Public Properties

		this.data = new ObservableProperty("data", Array, null)
			.onChanged(function(e)
			{
				this.invalidate("processDataPass");
			});

		this.fields = new ObservableArrayProperty("fields", String, null)
			.onChanged(function(e)
			{
				this.invalidate("processDataPass");
			});

		this.legend = new ObservableProperty("legend", Legend, null)
			.onChanged(function(e)
			{
				if (e.target === this)
				{
					var oldLegend = e.oldValue;
					var newLegend = e.newValue;

					if (oldLegend)
					{
						oldLegend.removeEventListener("settingLabels", this._legend_settingLabels);
						oldLegend.unregister(this);
					}

					if (newLegend)
					{
						newLegend.register(this);
						newLegend.addEventListener("settingLabels", this._legend_settingLabels);
					}

					this.invalidate("updateLegendLabelsPass");
					return;
				}

				if (e.event === e.target.labelIndexMapChanged)
				{
					this.invalidate("renderDataPass");
					return;
				}
			});

		// Private Properties

		this._cachedData = null;
		this._cachedFields = null;
		this._cachedLegend = null;

		// Constructor

		this.constructor = function()
		{
			this._legend_settingLabels = FunctionUtils.bind(this._legend_settingLabels, this);
		};

		// Public Methods

		this.processData = function()
		{
			this.validatePreceding("processDataPass");

			if (this.isValid("processDataPass"))
				return;

			this.invalidate("updateLegendLabelsPass");

			var data = this._cachedData = this.getInternal("data") || [];
			var fields = this._cachedFields = this.getInternal("fields") || [];

			this.processDataOverride(data, fields);

			this.setValid("processDataPass");
		};

		this.updateLegendLabels = function()
		{
			this.validatePreceding("updateLegendLabelsPass");

			if (this.isValid("updateLegendLabelsPass"))
				return;

			this.invalidate("renderDataPass");

			var legend = this._cachedLegend = this.getInternal("legend");
			var labels = null;

			if (legend)
				labels = this.updateLegendLabelsOverride(this._cachedData, this._cachedFields);

			this.setValid("updateLegendLabelsPass");

			// this must run last to avoid recursion
			if (legend)
				legend.setLabels(this, labels);
		};

		this.renderData = function()
		{
			this.validatePreceding("renderDataPass");

			if (this.isValid("renderDataPass"))
				return;

			this.renderDataOverride(this._cachedData, this._cachedFields, this._cachedLegend);

			this.setValid("renderDataPass");
		};

		// Protected Methods

		this.processDataOverride = function(data, fields)
		{
		};

		this.updateLegendLabelsOverride = function(data, fields)
		{
			return null;
		};

		this.renderDataOverride = function(data, fields, legend)
		{
		};

		// Private Methods

		this._legend_settingLabels = function(e)
		{
			this.validate("updateLegendLabelsPass");
		};

	});

});
