/**
 * Requires:
 * jquery
 * jg_global
 * jg_library
 */

jg_import.define("splunk.palettes.ColorPalette", function()
{
jg_namespace("splunk.palettes", function()
{

	var MObservable = jg_import("jgatt.events.MObservable");
	var MPropertyTarget = jg_import("jgatt.properties.MPropertyTarget");
	var Property = jg_import("jgatt.properties.Property");
	var PropertyEventData = jg_import("jgatt.properties.PropertyEventData");

	this.ColorPalette = jg_extend(Object, function(ColorPalette, base)
	{

		base = jg_mixin(this, MObservable, base);
		base = jg_mixin(this, MPropertyTarget, base);

		// Private Properties

		this._properties = null;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.addEventListener(this.changed, this._self_changed, Infinity);
		};

		// Public Methods

		this.getColor = function(field, index, count)
		{
			if ((field != null) && (typeof field !== "string"))
				throw new Error("Parameter field must be a string.");
			if (index == null)
				throw new Error("Parameter index must be non-null.");
			if (typeof index !== "number")
				throw new Error("Parameter index must be a number.");
			if (count == null)
				throw new Error("Parameter count must be non-null.");
			if (typeof count !== "number")
				throw new Error("Parameter count must be a number.");

			if (!this._properties)
				this._properties = this._getProperties();

			return this.getColorOverride(this._properties, field, Math.floor(index), Math.floor(count));
		};

		// Protected Methods

		this.getColorOverride = function(properties, field, index, count)
		{
			return 0x000000;
		};

		// Private Methods

		this._getProperties = function()
		{
			var properties = {};
			var property;
			for (var p in this)
			{
				property = this[p];
				if (property instanceof Property)
					properties[p] = this.getInternal(property);
			}
			return properties;
		};

		this._self_changed = function(e)
		{
			if ((e.target === this) && (e instanceof PropertyEventData))
				this._properties = null;
		};

	});

});
});

jg_import.define("splunk.palettes.FieldColorPalette", function()
{
jg_namespace("splunk.palettes", function()
{

	var ObservableProperty = jg_import("jgatt.properties.ObservableProperty");
	var ColorPalette = jg_import("splunk.palettes.ColorPalette");

	this.FieldColorPalette = jg_extend(ColorPalette, function(FieldColorPalette, base)
	{

		// Private Static Methods

		var _cloneFieldColors = function(fieldColors)
		{
			var fieldColors2 = {};
			for (var field in fieldColors)
			{
				if (fieldColors[field] != null)
					fieldColors2[field] = Number(fieldColors[field]);
			}
			return fieldColors2;
		};

		// Public Properties

		this.fieldColors = new ObservableProperty("fieldColors", Object, {})
			.readFilter(function(value)
			{
				return _cloneFieldColors(value);
			})
			.writeFilter(function(value)
			{
				return _cloneFieldColors(value);
			});

		this.defaultColorPalette = new ObservableProperty("defaultColorPalette", ColorPalette, null);

		// Constructor

		this.constructor = function(fieldColors, defaultColorPalette)
		{
			base.constructor.call(this);

			if (fieldColors != null)
				this.set(this.fieldColors, fieldColors);
			if (defaultColorPalette != null)
				this.set(this.defaultColorPalette, defaultColorPalette);
		};

		// Protected Methods

		this.getColorOverride = function(properties, field, index, count)
		{
			if (field)
			{
				var color = properties.fieldColors[field];
				if ((color != null) && !isNaN(color))
					return color;
			}

			if (properties.defaultColorPalette)
				return properties.defaultColorPalette.getColor(field, index, count);

			return 0x000000;
		};

	});

});
});

jg_import.define("splunk.palettes.ListColorPalette", function()
{
jg_namespace("splunk.palettes", function()
{

	var ColorUtils = jg_import("jgatt.graphics.ColorUtils");
	var ObservableArrayProperty = jg_import("jgatt.properties.ObservableArrayProperty");
	var ObservableProperty = jg_import("jgatt.properties.ObservableProperty");
	var ColorPalette = jg_import("splunk.palettes.ColorPalette");

	this.ListColorPalette = jg_extend(ColorPalette, function(ListColorPalette, base)
	{

		// Public Properties

		this.colors = new ObservableArrayProperty("colors", Number, [])
			.readFilter(function(value)
			{
				return value.concat();
			})
			.writeFilter(function(value)
			{
				return value ? value.concat() : [];
			});

		this.interpolate = new ObservableProperty("interpolate", Boolean, false);

		// Constructor

		this.constructor = function(colors, interpolate)
		{
			base.constructor.call(this);

			if (colors != null)
				this.set(this.colors, colors);
			if (interpolate != null)
				this.set(this.interpolate, interpolate);
		};

		// Protected Methods

		this.getColorOverride = function(properties, field, index, count)
		{
			var colors = properties.colors;
			var numColors = colors.length;

			if (numColors == 0)
				return 0x000000;

			if (index < 0)
				index = 0;

			if (properties.interpolate)
			{
				if (count < 1)
					count = 1;
				if (index > count)
					index = count;

				var p = (count == 1) ? 0 : (numColors - 1) * (index / (count - 1));
				var index1 = Math.floor(p);
				var index2 = Math.min(index1 + 1, numColors - 1);
				p -= index1;

				return ColorUtils.interpolate(colors[index1], colors[index2], p);
			}

			return colors[index % numColors];
		};

	});

});
});

jg_import.define("splunk.parsers.Parser", function()
{
jg_namespace("splunk.parsers", function()
{

	this.Parser = jg_extend(Object, function(Parser, base)
	{

		// Public Methods

		this.stringToValue = function(str)
		{
			return null;
		};

		this.valueToString = function(value)
		{
			return null;
		};

	});

});
});

jg_import.define("splunk.parsers.ParseUtils", function()
{
jg_namespace("splunk.parsers", function()
{

	this.ParseUtils = jg_static(function(ParseUtils)
	{

		// Private Static Constants

		var _UNESCAPE_PATTERN = /\\([.\n\r]?)/g;
		var _ESCAPE_SLASH_PATTERN = /\\/g;
		var _ESCAPE_QUOTE_PATTERN = /"/g;

		// Public Static Methods

		ParseUtils.prepareArray = function(str)
		{
			if (!str || (typeof str !== "string"))
				return null;

			str = ParseUtils.trimWhiteSpace(str);
			if (!str)
				return null;

			var length = str.length;
			if (length < 2)
				return null;

			if (str.charAt(0) != "[")
				return null;

			if (str.charAt(length - 1) != "]")
				return null;

			str = str.substring(1, length - 1);
			length = str.length;

			var arr = [];
			var index = -1;
			var value;

			while (index < length)
			{
				index++;
				value = _readUntil(str, index, ",");
				index += value.length;

				value = ParseUtils.trimWhiteSpace(value);
				if (value || (index < length) || (arr.length > 0))
					arr.push(ParseUtils.unescapeString(value));
			}

			return arr;
		};

		ParseUtils.prepareObject = function(str)
		{
			if (!str || (typeof str !== "string"))
				return null;

			str = ParseUtils.trimWhiteSpace(str);
			if (!str)
				return null;

			var length = str.length;
			if (length < 2)
				return null;

			if (str.charAt(0) != "{")
				return null;

			if (str.charAt(length - 1) != "}")
				return null;

			str = str.substring(1, length - 1);
			length = str.length;

			var obj = {};
			var index = 0;
			var key;
			var value;

			while (index < length)
			{
				key = _readUntil(str, index, ":");
				index += key.length + 1;

				if (index > length)
					break;

				value = _readUntil(str, index, ",");
				index += value.length + 1;

				key = ParseUtils.unescapeString(key);
				if (key)
					obj[key] = ParseUtils.unescapeString(value);
			}

			return obj;
		};

		ParseUtils.prepareTuple = function(str)
		{
			if (!str || (typeof str !== "string"))
				return null;

			str = ParseUtils.trimWhiteSpace(str);
			if (!str)
				return null;

			var length = str.length;
			if (length < 2)
				return null;

			if (str.charAt(0) != "(")
				return null;

			if (str.charAt(length - 1) != ")")
				return null;

			str = str.substring(1, length - 1);
			length = str.length;

			var arr = [];
			var index = -1;
			var value;

			while (index < length)
			{
				index++;
				value = _readUntil(str, index, ",");
				index += value.length;

				value = ParseUtils.trimWhiteSpace(value);
				if (value || (index < length) || (arr.length > 0))
					arr.push(ParseUtils.unescapeString(value));
			}

			return arr;
		};

		ParseUtils.unescapeString = function(str)
		{
			if ((str == null) || (typeof str !== "string"))
				return null;

			if (!str)
				return str;

			str = ParseUtils.trimWhiteSpace(str);
			if (!str)
				return str;

			var length = str.length;
			if (length < 2)
				return str;

			if (str.charAt(0) != "\"")
				return str;

			if (str.charAt(length - 1) != "\"")
				return str;

			str = str.substring(1, length - 1);
			if (!str)
				return str;

			str = str.replace(_UNESCAPE_PATTERN, "$1");

			return str;
		};

		ParseUtils.escapeString = function(str)
		{
			if ((str == null) || (typeof str !== "string"))
				return null;

			// two simple replace calls are faster than str.replace(/([\\"])/g, "\\$1")
			str = str.replace(_ESCAPE_SLASH_PATTERN, "\\\\");
			str = str.replace(_ESCAPE_QUOTE_PATTERN, "\\\"");

			return "\"" + str + "\"";
		};

		ParseUtils.trimWhiteSpace = function(str)
		{
			if ((str == null) || (typeof str !== "string"))
				return null;

			if (!str)
				return str;

			var startIndex = 0;
			var endIndex = str.length - 1;

			for (startIndex; startIndex <= endIndex; startIndex++)
			{
				if (!ParseUtils.isWhiteSpace(str.charAt(startIndex)))
					break;
			}

			for (endIndex; endIndex >= startIndex; endIndex--)
			{
				if (!ParseUtils.isWhiteSpace(str.charAt(endIndex)))
					break;
			}

			return str.substring(startIndex, endIndex + 1);
		};

		ParseUtils.isWhiteSpace = function(ch)
		{
			return ((ch === " ") || (ch === "\t") || (ch === "\n") || (ch === "\r"));
		};

		// Private Static Methods

		var _readUntil = function(str, startIndex, endChar)
		{
			var substr = "";

			var index = startIndex;
			var length = str.length;
			var ch;
			var isQuote = false;
			var nestLevel = 0;
			var nestBeginChar;
			var nestEndChar;

			while (index < length)
			{
				ch = str.charAt(index);
				if (isQuote)
				{
					if (ch == "\"")
					{
						isQuote = false;
					}
					else if (ch == "\\")
					{
						substr += ch;
						index++;
						ch = str.charAt(index);
					}
				}
				else if (nestLevel > 0)
				{
					if (ch == nestEndChar)
						nestLevel--;
					else if (ch == nestBeginChar)
						nestLevel++;
					else if (ch == "\"")
						isQuote = true;
				}
				else if (ch != endChar)
				{
					if (ch == "[")
					{
						nestLevel = 1;
						nestBeginChar = "[";
						nestEndChar = "]";
					}
					else if (ch == "{")
					{
						nestLevel = 1;
						nestBeginChar = "{";
						nestEndChar = "}";
					}
					else if (ch == "(")
					{
						nestLevel = 1;
						nestBeginChar = "(";
						nestEndChar = ")";
					}
					else if (ch == "\"")
					{
						isQuote = true;
					}
				}
				else
				{
					break;
				}

				substr += ch;
				index++;
			}

			return substr;
		};

	});

});
});

jg_import.define("splunk.parsers.StringParser", function()
{
jg_namespace("splunk.parsers", function()
{

	var Parser = jg_import("splunk.parsers.Parser");

	this.StringParser = jg_extend(Parser, function(StringParser, base)
	{

		// Private Static Properties

		var _instance = null;

		// Public Static Methods

		StringParser.getInstance = function()
		{
			if (!_instance)
				_instance = new StringParser();
			return _instance;
		};

		// Public Methods

		this.stringToValue = function(str)
		{
			return ((str == null) || (typeof str !== "string")) ? null : str;
		};

		this.valueToString = function(value)
		{
			return (value == null) ? null : String(value);
		};

	});

});
});

jg_import.define("splunk.parsers.ArrayParser", function()
{
jg_namespace("splunk.parsers", function()
{

	var Dictionary = jg_import("jgatt.utils.Dictionary");
	var Parser = jg_import("splunk.parsers.Parser");
	var ParseUtils = jg_import("splunk.parsers.ParseUtils");
	var StringParser = jg_import("splunk.parsers.StringParser");

	this.ArrayParser = jg_extend(Parser, function(ArrayParser, base)
	{

		// Private Static Properties

		var _instances = new Dictionary();

		// Public Static Methods

		ArrayParser.getInstance = function(elementParser)
		{
			var instance = _instances.get(elementParser);
			if (!instance)
				instance = _instances.set(elementParser, new ArrayParser(elementParser));
			return instance;
		};

		// Protected Properties

		this.elementParser = null;

		// Constructor

		this.constructor = function(elementParser)
		{
			if (elementParser == null)
				throw new Error("Parameter elementParser must be non-null.");
			if (!(elementParser instanceof Parser))
				throw new Error("Parameter elementParser must be an instance of splunk.parsers.Parser.");

			this.elementParser = elementParser;
		};

		// Public Methods

		this.stringToValue = function(str)
		{
			var array = ParseUtils.prepareArray(str);
			if (!array)
				return null;

			var elementParser = this.elementParser;
			for (var i = 0, l = array.length; i < l; i++)
				array[i] = elementParser.stringToValue(array[i]);

			return array;
		};

		this.valueToString = function(value)
		{
			var array = (value instanceof Array) ? value : null;
			if (!array)
				return null;

			var str = "";

			var elementParser = this.elementParser;
			var elementValue;
			for (var i = 0, l = array.length; i < l; i++)
			{
				elementValue = array[i];
				if (str)
					str += ",";
				if (elementParser instanceof StringParser)
					str += ParseUtils.escapeString(elementParser.valueToString(elementValue));
				else
					str += elementParser.valueToString(elementValue);
			}

			return "[" + str + "]";
		};

	});

});
});

jg_import.define("splunk.parsers.BooleanParser", function()
{
jg_namespace("splunk.parsers", function()
{

	var Parser = jg_import("splunk.parsers.Parser");
	var ParseUtils = jg_import("splunk.parsers.ParseUtils");

	this.BooleanParser = jg_extend(Parser, function(BooleanParser, base)
	{

		// Private Static Properties

		var _instance = null;

		// Public Static Methods

		BooleanParser.getInstance = function()
		{
			if (!_instance)
				_instance = new BooleanParser();
			return _instance;
		};

		// Public Methods

		this.stringToValue = function(str)
		{
			str = ParseUtils.trimWhiteSpace(str);
			if (str)
				str = str.toLowerCase();
			return ((str === "true") || (str === "t") || (str === "1"));
		};

		this.valueToString = function(value)
		{
			return value ? "true" : "false";
		};

	});

});
});

jg_import.define("splunk.parsers.NumberParser", function()
{
jg_namespace("splunk.parsers", function()
{

	var Parser = jg_import("splunk.parsers.Parser");
	var ParseUtils = jg_import("splunk.parsers.ParseUtils");

	this.NumberParser = jg_extend(Parser, function(NumberParser, base)
	{

		// Private Static Properties

		var _instance = null;

		// Public Static Methods

		NumberParser.getInstance = function()
		{
			if (!_instance)
				_instance = new NumberParser();
			return _instance;
		};

		// Public Methods

		this.stringToValue = function(str)
		{
			str = ParseUtils.trimWhiteSpace(str);
			return str ? Number(str) : NaN;
		};

		this.valueToString = function(value)
		{
			return (typeof value === "number") ? String(value) : String(NaN);
		};

	});

});
});

jg_import.define("splunk.parsers.ObjectParser", function()
{
jg_namespace("splunk.parsers", function()
{

	var Dictionary = jg_import("jgatt.utils.Dictionary");
	var Parser = jg_import("splunk.parsers.Parser");
	var ParseUtils = jg_import("splunk.parsers.ParseUtils");
	var StringParser = jg_import("splunk.parsers.StringParser");

	this.ObjectParser = jg_extend(Parser, function(ObjectParser, base)
	{

		// Private Static Properties

		var _instances = new Dictionary();

		// Public Static Methods

		ObjectParser.getInstance = function(elementParser)
		{
			var instance = _instances.get(elementParser);
			if (!instance)
				instance = _instances.set(elementParser, new ObjectParser(elementParser));
			return instance;
		};

		// Protected Properties

		this.elementParser = null;

		// Constructor

		this.constructor = function(elementParser)
		{
			if (elementParser == null)
				throw new Error("Parameter elementParser must be non-null.");
			if (!(elementParser instanceof Parser))
				throw new Error("Parameter elementParser must be an instance of splunk.parsers.Parser.");

			this.elementParser = elementParser;
		};

		// Public Methods

		this.stringToValue = function(str)
		{
			var map = ParseUtils.prepareObject(str);
			if (!map)
				return null;

			var elementParser = this.elementParser;
			for (var key in map)
			{
				if (map.hasOwnProperty(key))
					map[key] = elementParser.stringToValue(map[key]);
			}

			return map;
		};

		this.valueToString = function(value)
		{
			var map = (value instanceof Object) ? value : null;
			if (!map)
				return null;

			var str = "";

			var elementParser = this.elementParser;
			for (var key in map)
			{
				if (map.hasOwnProperty(key))
				{
					if (str)
						str += ",";
					str += ParseUtils.escapeString(key) + ":";
					if (elementParser instanceof StringParser)
						str += ParseUtils.escapeString(elementParser.valueToString(map[key]));
					else
						str += elementParser.valueToString(map[key]);
				}
			}

			return "{" + str + "}";
		};

	});

});
});

jg_import.define("splunk.mapping2.LatLon", function()
{
jg_namespace("splunk.mapping2", function()
{

	this.LatLon = jg_extend(Object, function(LatLon, base)
	{

		// Public Properties

		this.lat = 0;
		this.lon = 0;

		// Constructor

		this.constructor = function(lat, lon)
		{
			this.lat = (lat !== undefined) ? lat : 0;
			this.lon = (lon !== undefined) ? lon : 0;
		};

		// Public Methods

		this.normalize = function(center)
		{
			var lat = this.lat;
			if (lat < -90)
				lat = -90;
			else if (lat > 90)
				lat = 90;

			var centerLon = center ? center.lon : 0;
			var lon = (this.lon - centerLon) % 360;
			if (lon < -180)
				lon += 360;
			else if (lon > 180)
				lon -= 360;
			lon += centerLon;

			return new LatLon(lat, lon);
		};

		this.isFinite = function()
		{
			return (((this.lat - this.lat) === 0) &&
			        ((this.lon - this.lon) === 0));
		};

		this.equals = function(latLon)
		{
			return ((this.lat == latLon.lat) &&
			        (this.lon == latLon.lon));
		};

		this.clone = function()
		{
			return new LatLon(this.lat, this.lon);
		};

		this.toString = function()
		{
			return "(" + this.lat + ", " + this.lon + ")";
		};

	});

});
});

jg_import.define("splunk.mapping2.LatLonBounds", function()
{
jg_namespace("splunk.mapping2", function()
{

	var LatLon = jg_import("splunk.mapping2.LatLon");

	this.LatLonBounds = jg_extend(Object, function(LatLonBounds, base)
	{

		// Public Properties

		this.s = 0;
		this.w = 0;
		this.n = 0;
		this.e = 0;

		// Constructor

		this.constructor = function(s, w, n, e)
		{
			this.s = (s !== undefined) ? s : 0;
			this.w = (w !== undefined) ? w : 0;
			this.n = (n !== undefined) ? n : 0;
			this.e = (e !== undefined) ? e : 0;
		};

		// Public Methods

		this.getSW = function()
		{
			return new LatLon(this.s, this.w);
		};

		this.getSE = function()
		{
			return new LatLon(this.s, this.e);
		};

		this.getNW = function()
		{
			return new LatLon(this.n, this.w);
		};

		this.getNE = function()
		{
			return new LatLon(this.n, this.e);
		};

		this.getCenter = function()
		{
			return new LatLon((this.s + this.n) / 2, (this.w + this.e) / 2);
		};

		this.expand = function(latLon)
		{
			if (latLon.lat < this.s)
				this.s = latLon.lat;
			if (latLon.lat > this.n)
				this.n = latLon.lat;
			if (latLon.lon < this.w)
				this.w = latLon.lon;
			if (latLon.lon > this.e)
				this.e = latLon.lon;
		};

		this.contains = function(latLon)
		{
			return ((latLon.lat >= this.s) &&
			        (latLon.lat <= this.n) &&
			        (latLon.lon >= this.w) &&
			        (latLon.lon <= this.e));
		};

		this.normalize = function(center)
		{
			var s = this.s;
			if (s < -90)
				s = -90;
			else if (s > 90)
				s = 90;

			var n = this.n;
			if (n < s)
				n = s;
			else if (n > 90)
				n = 90;

			var centerLon = center ? center.lon : 0;
			var w = (this.w - centerLon);
			var e = (this.e - centerLon);
			if ((e - w) >= 360)
			{
				w = -180;
				e = 180;
			}
			else
			{
				w %= 360;
				if (w < -180)
					w += 360;
				else if (w > 180)
					w -= 360;

				e %= 360;
				if (e < -180)
					e += 360;
				else if (e > 180)
					e -= 360;

				if (e < w)
				{
					if (e > -w)
						w -= 360;
					else
						e += 360;
				}
			}
			w += centerLon;
			e += centerLon;

			return new LatLonBounds(s, w, n, e);
		};

		this.isFinite = function()
		{
			return (((this.s - this.s) === 0) &&
			        ((this.w - this.w) === 0) &&
			        ((this.n - this.n) === 0) &&
			        ((this.e - this.e) === 0));
		};

		this.equals = function(bounds)
		{
			return ((this.s == bounds.s) &&
			        (this.w == bounds.w) &&
			        (this.n == bounds.n) &&
			        (this.e == bounds.e));
		};

		this.clone = function()
		{
			return new LatLonBounds(this.s, this.w, this.n, this.e);
		};

		this.toString = function()
		{
			return "(" + this.s + ", " + this.w + ", " + this.n + ", " + this.e + ")";
		};

	});

});
});

jg_import.define("splunk.viz.MRenderTarget", function()
{
jg_namespace("splunk.viz", function()
{

	var MObservable = jg_import("jgatt.events.MObservable");
	var MPropertyTarget = jg_import("jgatt.properties.MPropertyTarget");
	var PropertyComparator = jg_import("jgatt.utils.PropertyComparator");
	var MValidateTarget = jg_import("jgatt.validation.MValidateTarget");
	var ValidatePass = jg_import("jgatt.validation.ValidatePass");

	this.MRenderTarget = jg_static(function(MRenderTarget)
	{

		// Mixin

		this.mixin = function(base)
		{
			base = jg_mixin(this, MObservable, base);
			base = jg_mixin(this, MPropertyTarget, base);
			base = jg_mixin(this, MValidateTarget, base);
		};

		// Public Passes

		this.renderPass = new ValidatePass("render", 3, new PropertyComparator("renderPriority"));

		// Public Properties

		this.renderPriority = 0;

		// Public Methods

		this.render = function()
		{
		};

	});

});
});

jg_import.define("splunk.viz.VizBase", function()
{
jg_namespace("splunk.viz", function()
{

	var $ = jg_import("jQuery");
	var MObservable = jg_import("jgatt.events.MObservable");
	var MPropertyTarget = jg_import("jgatt.properties.MPropertyTarget");
	var Property = jg_import("jgatt.properties.Property");
	var MValidateTarget = jg_import("jgatt.validation.MValidateTarget");

	this.VizBase = jg_extend(Object, function(VizBase, base)
	{

		base = jg_mixin(this, MObservable, base);
		base = jg_mixin(this, MPropertyTarget, base);
		base = jg_mixin(this, MValidateTarget, base);

		// Private Static Constants

		var _INSTANCE_KEY = "__splunk_viz_VizBase_instance";

		// Private Static Properties

		var _instanceCount = 0;

		// Public Static Methods

		VizBase.getInstance = function(element)
		{
			if (element == null)
				return null;

			element = $(element);
			if (element.length == 0)
				return null;

			element = element[0];

			var instance = element[_INSTANCE_KEY];
			return (instance instanceof VizBase) ? instance : null;
		};

		// Public Properties

		this.id = new Property("id", String, null, true);

		this.element = null;
		this.$element = null;

		// Constructor

		this.constructor = function(html)
		{
			if ((html != null) && (typeof html !== "string"))
				throw new Error("Parameter html must be a string.");

			var query = $(html ? html : "<div></div>");
			if (query.length == 0)
				throw new Error("Parameter html must be valid markup.");

			base.constructor.call(this);

			var id = "splunk-viz-VizBase-" + (++_instanceCount);

			this.element = query[0];
			//this.element[_INSTANCE_KEY] = this;
			//this.element.id = id;

			this.$element = $(this.element);

			this.setInternal("id", id);

			this.addStyleClass("splunk-viz-VizBase");
		};

		// Public Methods

		this.addStyleClass = function(styleClass)
		{
			this.$element.addClass(styleClass);
		};

		this.removeStyleClass = function(styleClass)
		{
			this.$element.removeClass(styleClass);
		};

		this.setStyle = function(style)
		{
			this.$element.css(style);
		};

		this.appendTo = function(parentElement)
		{
			if (parentElement == null)
				throw new Error("Parameter parentElement must be non-null.");

			if (parentElement instanceof VizBase)
				parentElement = parentElement.element;

			parentElement = $(parentElement);
			if (parentElement.length == 0)
				return;

			parentElement = parentElement[0];

			var oldParent = this.element.parentNode;
			if (oldParent && (oldParent !== parentElement))
				this.onRemove();

			parentElement.appendChild(this.element);

			if (oldParent !== parentElement)
				this.onAppend();
		};

		this.replace = function(element)
		{
			if (element == null)
				throw new Error("Parameter element must be non-null.");

			if (element instanceof VizBase)
				element = element.element;

			element = $(element);
			if (element.length == 0)
				return;

			element = element[0];

			var parentElement = element.parentNode;
			if (parentElement == null)
				return;

			var oldParent = this.element.parentNode;
			if (oldParent && (oldParent !== parentElement))
				this.onRemove();

			parentElement.replaceChild(this.element, element);

			if (oldParent !== parentElement)
				this.onAppend();
		};

		this.remove = function()
		{
			var element = this.element;
			var parentElement = element.parentNode;
			if (!parentElement)
				return;

			this.onRemove();

			parentElement.removeChild(element);
		};

		this.dispose = function()
		{
			this.remove();

			// ensure all jquery data and events are removed
			this.$element.remove();
		};

		// Protected Methods

		this.onAppend = function()
		{
		};

		this.onRemove = function()
		{
		};

	});

});
});

jg_import.define("splunk.mapping2.controls.ControlBase", function()
{
jg_namespace("splunk.mapping2.controls", function()
{

	var Property = jg_import("jgatt.properties.Property");
	var FunctionUtils = jg_import("jgatt.utils.FunctionUtils");
	var MRenderTarget = jg_import("splunk.viz.MRenderTarget");
	var VizBase = jg_import("splunk.viz.VizBase");

	this.ControlBase = jg_extend(VizBase, function(ControlBase, base)
	{

		base = jg_mixin(this, MRenderTarget, base);

		// Public Properties

		this.map = new Property("map", Object, null, true);

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.addStyleClass("splunk-mapping2-controls-ControlBase");

			this.setStyle({ position: "absolute" });

			this._map_boundsChanged = FunctionUtils.bind(this._map_boundsChanged, this);
		};

		// Public Methods

		this.render = function()
		{
			this.validatePreceding("renderPass");

			if (this.isValid("renderPass"))
				return;

			var map = this.getInternal("map");
			if (map)
				this.renderOverride(map);

			this.setValid("renderPass");
		};

		// Protected Methods

		this.renderOverride = function(map)
		{
		};

		this.onAddedToMap = function(map)
		{
			this.setInternal("map", map);

			map.addEventListener("boundsChanged", this._map_boundsChanged);

			this.invalidate("renderPass");
		};

		this.onRemovedFromMap = function(map)
		{
			map.removeEventListener("boundsChanged", this._map_boundsChanged);

			this.setInternal("map", null);
		};

		// Private Methods

		this._map_boundsChanged = function(e)
		{
			this.invalidate("renderPass");
		};

	});

});
});

jg_import.define("splunk.mapping2.controls.MAttributionTarget", function()
{
jg_namespace("splunk.mapping2.controls", function()
{

	var MObservable = jg_import("jgatt.events.MObservable");
	var MPropertyTarget = jg_import("jgatt.properties.MPropertyTarget");
	var ObservableProperty = jg_import("jgatt.properties.ObservableProperty");

	this.MAttributionTarget = jg_static(function(MAttributionTarget)
	{

		// Mixin

		this.mixin = function(base)
		{
			base = jg_mixin(this, MObservable, base);
			base = jg_mixin(this, MPropertyTarget, base);
		};

		// Public Properties

		this.attribution = new ObservableProperty("attribution", String, null);

	});

});
});

jg_import.define("splunk.mapping2.layers.LayerBase", function()
{
jg_namespace("splunk.mapping2.layers", function()
{

	var Property = jg_import("jgatt.properties.Property");
	var FunctionUtils = jg_import("jgatt.utils.FunctionUtils");
	var MAttributionTarget = jg_import("splunk.mapping2.controls.MAttributionTarget");
	var MRenderTarget = jg_import("splunk.viz.MRenderTarget");
	var VizBase = jg_import("splunk.viz.VizBase");

	this.LayerBase = jg_extend(VizBase, function(LayerBase, base)
	{

		base = jg_mixin(this, MRenderTarget, base);
		base = jg_mixin(this, MAttributionTarget, base);

		// Public Properties

		this.map = new Property("map", Object, null, true);

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.addStyleClass("splunk-mapping2-layers-LayerBase");

			this.setStyle({ position: "absolute" });

			this._map_boundsChanged = FunctionUtils.bind(this._map_boundsChanged, this);
		};

		// Public Methods

		this.render = function()
		{
			this.validatePreceding("renderPass");

			if (this.isValid("renderPass"))
				return;

			var map = this.getInternal("map");
			if (map)
				this.renderOverride(map);

			this.setValid("renderPass");
		};

		// Protected Methods

		this.renderOverride = function(map)
		{
		};

		this.onAddedToMap = function(map)
		{
			this.setInternal("map", map);

			map.addEventListener("boundsChanged", this._map_boundsChanged);

			this.invalidate("renderPass");
		};

		this.onRemovedFromMap = function(map)
		{
			map.removeEventListener("boundsChanged", this._map_boundsChanged);

			this.setInternal("map", null);
		};

		// Private Methods

		this._map_boundsChanged = function(e)
		{
			this.invalidate("renderPass");
		};

	});

});
});

jg_import.define("splunk.mapping2.layers.TileLayerBase", function()
{
jg_namespace("splunk.mapping2.layers", function()
{

	var Point = jg_import("jgatt.geom.Point");
	var ObservableArrayProperty = jg_import("jgatt.properties.ObservableArrayProperty");
	var ObservableProperty = jg_import("jgatt.properties.ObservableProperty");
	var FunctionUtils = jg_import("jgatt.utils.FunctionUtils");
	var NumberUtils = jg_import("jgatt.utils.NumberUtils");
	var LayerBase = jg_import("splunk.mapping2.layers.LayerBase");

	this.TileLayerBase = jg_extend(LayerBase, function(TileLayerBase, base)
	{

		// Public Properties

		this.tileSize = new ObservableProperty("tileSize", Number, 256)
			.writeFilter(function(value)
			{
				return ((value > 0) && (value < Infinity)) ? value : 256;
			});

		this.bufferSize = new ObservableProperty("bufferSize", Number, 0)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value < Infinity)) ? Math.min(value, 1) : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		this.url = new ObservableProperty("url", String, null)
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		this.subdomains = new ObservableArrayProperty("subdomains", String, [ "a", "b", "c" ])
			.readFilter(function(value)
			{
				return value.concat();
			})
			.writeFilter(function(value)
			{
				return value ? value.concat() : [];
			})
			.changedComparator(function(oldValue, newValue)
			{
				if (oldValue.length !== newValue.length)
					return true;

				for (var i = 0, l = oldValue.length; i < l; i++)
				{
					if (oldValue[i] !== newValue[i])
						return true;
				}

				return false;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		this.minZoom = new ObservableProperty("minZoom", Number, 0)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value < Infinity)) ? Math.floor(value) : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		this.maxZoom = new ObservableProperty("maxZoom", Number, Infinity)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value < Infinity)) ? Math.floor(value) : Infinity;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		this.wrapX = new ObservableProperty("wrapX", Boolean, true)
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		this.wrapY = new ObservableProperty("wrapY", Boolean, false)
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		this.invertY = new ObservableProperty("invertY", Boolean, false)
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		// Private Properties

		this._tileContainers = null;
		this._activeContainer = null;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.addStyleClass("splunk-mapping2-layers-TileLayerBase");

			this._tileContainers = {};
		};

		// Protected Methods

		this.renderOverride = function(map)
		{
			var zoom = map.get("zoom");

			var bufferSize = this.getInternal("bufferSize");
			var minZoom = this.getInternal("minZoom");
			var maxZoom = this.getInternal("maxZoom");

			// get viewport bounds in relative coordinates
			var relativeBounds = map.getRelativeBounds();
			var relativeLeft = relativeBounds.x;
			var relativeTop = relativeBounds.y;
			var relativeRight = relativeLeft + relativeBounds.width;
			var relativeBottom = relativeTop + relativeBounds.height;

			// buffer viewport bounds
			relativeLeft -= relativeBounds.width * bufferSize;
			relativeTop -= relativeBounds.height * bufferSize;
			relativeRight += relativeBounds.width * bufferSize;
			relativeBottom += relativeBounds.height * bufferSize;

			// get map bounds in pixels relative to viewport
			var mapTL = map.relativeToViewport(new Point(0, 0));
			var mapBR = map.relativeToViewport(new Point(1, 1));
			var mapLeft = mapTL.x;
			var mapTop = mapTL.y;
			var mapWidth = mapBR.x - mapLeft;
			var mapHeight = mapBR.y - mapTop;

			// snap and clamp tile zoom
			var tileZoom = Math.floor(zoom);
			if ((zoom - tileZoom) > 0.9)
				tileZoom++;
			tileZoom = NumberUtils.maxMin(tileZoom, maxZoom, minZoom);

			// compute tile scale
			var tileScale = Math.pow(2, tileZoom);

			var tileRequests = this._getTileRequests(relativeLeft, relativeTop, relativeRight, relativeBottom, tileZoom);
			var tileRequest;
			var tileContainers = this._tileContainers;
			var tileContainer;
			var tiles;
			var tile;
			var url;
			var cid;
			var id;
			var x1;
			var x2;
			var y1;
			var y2;
			var i;
			var l;

			// mark all active tiles as inactive
			var activeContainer = this._activeContainer;
			if (activeContainer)
			{
				tiles = activeContainer.tiles;
				for (id in tiles)
				{
					if (tiles.hasOwnProperty(id))
					{
						tile = tiles[id];
						tile.isActive = false;
					}
				}
			}

			// update/create active container
			cid = this._getContainerID(tileZoom);
			activeContainer = tileContainers[cid];
			if (!activeContainer)
				activeContainer = tileContainers[cid] = new TileContainer(this, cid);
			else if (activeContainer !== this._activeContainer)
				activeContainer.display(true);
			this._activeContainer = activeContainer;

			// update/create active tiles
			tiles = activeContainer.tiles;
			for (i = 0, l = tileRequests.length; i < l; i++)
			{
				tileRequest = tileRequests[i];
				url = this._getTileURL(tileRequest.x, tileRequest.y, tileZoom);
				if (url)
				{
					id = this._getTileID(tileRequest.x, tileRequest.y);
					tile = tiles[id];
					if (!tile)
					{
						tile = tiles[id] = new Tile(activeContainer, id, url);
						tile.x1 = tileRequest.x / tileScale;
						tile.x2 = (tileRequest.x + 1) / tileScale;
						tile.y1 = tileRequest.y / tileScale;
						tile.y2 = (tileRequest.y + 1) / tileScale;
					}
					tile.isActive = true;
				}
			}

			// destroy unloaded tiles
			var unloadedTiles = this._getUnloadedTiles();
			if (unloadedTiles.length > 0)
				this._destroyTiles(unloadedTiles);

			// position tiles
			for (cid in tileContainers)
			{
				if (tileContainers.hasOwnProperty(cid))
				{
					tileContainer = tileContainers[cid];
					tiles = tileContainer.tiles;
					for (id in tiles)
					{
						if (tiles.hasOwnProperty(id))
						{
							tile = tiles[id];
							x1 = Math.round(mapLeft + mapWidth * tile.x1);
							x2 = Math.round(mapLeft + mapWidth * tile.x2);
							y1 = Math.round(mapTop + mapHeight * tile.y1);
							y2 = Math.round(mapTop + mapHeight * tile.y2);
							tile.position(x1, y1, x2 - x1, y2 - y1);
						}
					}
				}
			}

			this._checkTilesLoaded();
		};

		this.createTileContainer = function()
		{
			throw new Error("Must implement method createTileContainer.");
		};

		this.destroyTileContainer = function(container)
		{
			throw new Error("Must implement method destroyTileContainer.");
		};

		this.displayTileContainer = function(container, display)
		{
			throw new Error("Must implement method displayTileContainer.");
		};

		this.createTile = function(container, url, onLoad, onError)
		{
			throw new Error("Must implement method createTile.");
		};

		this.destroyTile = function(container, tile)
		{
			throw new Error("Must implement method destroyTile.");
		};

		this.positionTile = function(tile, x, y, width, height)
		{
			throw new Error("Must implement method positionTile.");
		};

		// Private Methods

		this._checkTilesLoaded = function()
		{
			var activeContainer = this._activeContainer;
			if (activeContainer)
			{
				var tiles = activeContainer.tiles;
				var tile;
				for (var id in tiles)
				{
					if (tiles.hasOwnProperty(id))
					{
						tile = tiles[id];
						if (tile.isActive && !tile.isLoaded)
							return;
					}
				}

				var inactiveTiles = this._getInactiveTiles(activeContainer);
				if (inactiveTiles.length > 0)
					this._destroyTiles(inactiveTiles);
			}

			var inactiveContainers = this._getInactiveContainers();
			if (inactiveContainers.length > 0)
				this._destroyContainers(inactiveContainers);
		};

		this._getUnloadedTiles = function()
		{
			var unloadedTiles = [];

			var containers = this._tileContainers;
			var container;
			var cid;
			var tiles;
			var tile;
			var id;
			for (cid in containers)
			{
				if (containers.hasOwnProperty(cid))
				{
					container = containers[cid];
					tiles = container.tiles;
					for (id in tiles)
					{
						if (tiles.hasOwnProperty(id))
						{
							tile = tiles[id];
							if (!tile.isActive && !tile.isLoaded)
								unloadedTiles.push(tile);
						}
					}
				}
			}

			return unloadedTiles;
		};

		this._getInactiveContainers = function()
		{
			var inactiveContainers = [];

			var activeContainer = this._activeContainer;
			var containers = this._tileContainers;
			var container;
			for (var id in containers)
			{
				if (containers.hasOwnProperty(id))
				{
					container = containers[id];
					if (container !== activeContainer)
						inactiveContainers.push(container);
				}
			}

			return inactiveContainers;
		};

		this._getInactiveTiles = function(container)
		{
			var inactiveTiles = [];

			var tiles = container.tiles;
			var tile;
			for (var id in tiles)
			{
				if (tiles.hasOwnProperty(id))
				{
					tile = tiles[id];
					if (!tile.isActive)
						inactiveTiles.push(tile);
				}
			}

			return inactiveTiles;
		};

		this._destroyContainers = function(containersList)
		{
			var containers = this._tileContainers;
			var container;
			for (var i = 0, l = containersList.length; i < l; i++)
			{
				container = containersList[i];
				delete containers[container.id];
				container.destroy();
			}
		};

		this._destroyTiles = function(tilesList)
		{
			var tile;
			for (var i = 0, l = tilesList.length; i < l; i++)
			{
				tile = tilesList[i];
				delete tile.container.tiles[tile.id];
				tile.destroy();
			}
		};

		this._getTileRequests = function(left, top, right, bottom, zoom)
		{
			// left, top, right, and bottom are in relative coordinates (0 to 1)
			// zoom is an integer

			var requests = [];

			var scale = Math.pow(2, zoom);

			var xCenter = ((left + right) / 2) * scale - 0.5;
			var yCenter = ((top + bottom) / 2) * scale - 0.5;

			var x1 = Math.floor(left * scale);
			var x2 = Math.ceil(right * scale);
			var y1 = Math.floor(top * scale);
			var y2 = Math.ceil(bottom * scale);

			var dx;
			var dy;

			for (var y = y1; y < y2; y++)
			{
				dy = y - yCenter;
				dy *= dy;
				for (var x = x1; x < x2; x++)
				{
					dx = x - xCenter;
					dx *= dx;
					requests.push({ x: x, y: y, dist: Math.sqrt(dx + dy) });
				}
			}

			requests.sort(function(r1, r2)
			{
				if (r1.dist < r2.dist)
					return -1;
				if (r1.dist > r2.dist)
					return 1;
				if (r1.y < r2.y)
					return -1;
				if (r1.y > r2.y)
					return 1;
				if (r1.x < r2.x)
					return -1;
				if (r1.x > r2.x)
					return 1;
				return 0;
			});

			return requests;
		};

		this._getTileURL = function(x, y, z)
		{
			// x, y, and z are integers, z is >= 0

			var url = this.getInternal("url");
			if (!url)
				return null;

			var scale = Math.pow(2, z);
			if (this.getInternal("invertY"))
				y = scale - y - 1;

			if (!this.getInternal("wrapX") && ((x < 0) || (x >= scale)))
				return null;

			if (!this.getInternal("wrapY") && ((y < 0) || (y >= scale)))
				return null;

			x %= scale;
			if (x < 0)
				x += scale;

			y %= scale;
			if (y < 0)
				y += scale;

			url = url.replace("{x}", x).replace("{y}", y).replace("{z}", z);

			var subdomain = this._getSubdomain(x, y);
			if (subdomain)
				url = url.replace("{s}", subdomain);

			return url;
		};

		this._getSubdomain = function(x, y)
		{
			var subdomains = this.getInternal("subdomains");
			var count = subdomains.length;
			if (count == 0)
				return null;

			var index = (x + y) % count;
			if (index < 0)
				index += count;

			return subdomains[index];
		};

		this._getContainerID = function(z)
		{
			var url = this.getInternal("url");
			var subdomains = this.getInternal("subdomains");
			return "z:" + z + " s:" + subdomains.join(",") + " u:" + (url || "");
		};

		this._getTileID = function(x, y)
		{
			return x + "," + y;
		};

	});

	var TileContainer = jg_extend(Object, function(TileContainer, base)
	{

		// Public Properties

		this.tileLayer = null;
		this.container = null;
		this.tiles = null;
		this.id = null;

		// Constructor

		this.constructor = function(tileLayer, id)
		{
			this.tileLayer = tileLayer;
			this.tiles = {};
			this.id = id;

			this.container = tileLayer.createTileContainer();
		};

		// Public Methods

		this.display = function(display)
		{
			this.tileLayer.displayTileContainer(this.container, display);
		};

		this.destroy = function()
		{
			var tiles = this.tiles;
			for (var id in tiles)
			{
				if (tiles.hasOwnProperty(id))
					tiles[id].destroy();
			}

			this.tileLayer.destroyTileContainer(this.container);

			this.tileLayer = null;
			this.container = null;
			this.tiles = null;
		};

	});

	var Tile = jg_extend(Object, function(Tile, base)
	{

		// Public Properties

		this.tileLayer = null;
		this.container = null;
		this.tile = null;
		this.id = null;
		this.url = null;
		this.x1 = 0;
		this.x2 = 0;
		this.y1 = 0;
		this.y2 = 0;
		this.isActive = false;
		this.isLoaded = false;

		// Constructor

		this.constructor = function(container, id, url)
		{
			this.tileLayer = container.tileLayer;
			this.container = container;
			this.id = id;
			this.url = url;

			this._tile_loaded = FunctionUtils.bind(this._tile_loaded, this);
			this._tile_error = FunctionUtils.bind(this._tile_error, this);

			this.tile = this.tileLayer.createTile(container.container, url, this._tile_loaded, this._tile_error);
		};

		// Public Methods

		this.position = function(x, y, width, height)
		{
			this.tileLayer.positionTile(this.tile, x, y, width, height);
		};

		this.destroy = function()
		{
			this.tileLayer.destroyTile(this.container.container, this.tile);

			this.tileLayer = null;
			this.container = null;
			this.tile = null;
		};

		// Private Methods

		this._tile_loaded = function()
		{
			if (!this.tile)
				return;

			this.isLoaded = true;

			this.tileLayer._checkTilesLoaded();
		};

		this._tile_error = function()
		{
			if (!this.tile)
				return;

			this.isLoaded = true;

			this.tileLayer._checkTilesLoaded();
		};

	});

});
});

jg_import.define("splunk.mapping2.projections.Projection", function()
{
jg_namespace("splunk.mapping2.projections", function()
{

	this.Projection = jg_extend(Object, function(Projection, base)
	{

		// Public Methods

		this.latLonToRelative = function(latLon)
		{
			throw new Error("Must implement method latLonToRelative.");
		};

		this.relativeToLatLon = function(relativePoint)
		{
			throw new Error("Must implement method relativeToLatLon.");
		};

	});

});
});

jg_import.define("splunk.mapping2.projections.SphericalMercatorProjection", function()
{
jg_namespace("splunk.mapping2.projections", function()
{

	var Point = jg_import("jgatt.geom.Point");
	var NumberUtils = jg_import("jgatt.utils.NumberUtils");
	var LatLon = jg_import("splunk.mapping2.LatLon");
	var Projection = jg_import("splunk.mapping2.projections.Projection");

	this.SphericalMercatorProjection = jg_extend(Projection, function(SphericalMercatorProjection, base)
	{

		// Private Static Constants

		var _MAX_LAT = 85.051128779806;
		var _PI = Math.PI;
		var _PI2 = _PI * 2;
		var _DEG_TO_RAD = _PI / 180;
		var _RAD_TO_DEG = 180 / _PI;

		// Private Static Properties

		var _instance = null;

		// Public Static Methods

		SphericalMercatorProjection.getInstance = function()
		{
			if (!_instance)
				_instance = new SphericalMercatorProjection();
			return _instance;
		};

		// Public Methods

		this.latLonToRelative = function(latLon)
		{
			var x = (latLon.lon + 180) / 360;
			var y = NumberUtils.minMax(latLon.lat, -_MAX_LAT, _MAX_LAT) * _DEG_TO_RAD;
			y = Math.log(Math.tan((_PI / 4) + (y / 2)));
			y = (_PI - y) / _PI2;
			return new Point(x, y);
		};

		this.relativeToLatLon = function(relativePoint)
		{
			var lat = _PI - relativePoint.y * _PI2;
			lat = 2 * Math.atan(Math.exp(lat)) - (_PI / 2);
			lat = NumberUtils.minMax(lat * _RAD_TO_DEG, -_MAX_LAT, _MAX_LAT);
			var lon = relativePoint.x * 360 - 180;
			return new LatLon(lat, lon);
		};

	});

});
});

jg_import.define("splunk.mapping2.Map", function()
{
jg_namespace("splunk.mapping2", function()
{

	var ChainedEvent = jg_import("jgatt.events.ChainedEvent");
	var EventData = jg_import("jgatt.events.EventData");
	var Point = jg_import("jgatt.geom.Point");
	var Rectangle = jg_import("jgatt.geom.Rectangle");
	var ObservableProperty = jg_import("jgatt.properties.ObservableProperty");
	var ArrayUtils = jg_import("jgatt.utils.ArrayUtils");
	var FunctionUtils = jg_import("jgatt.utils.FunctionUtils");
	var NumberUtils = jg_import("jgatt.utils.NumberUtils");
	var LatLon = jg_import("splunk.mapping2.LatLon");
	var LatLonBounds = jg_import("splunk.mapping2.LatLonBounds");
	var ControlBase = jg_import("splunk.mapping2.controls.ControlBase");
	var LayerBase = jg_import("splunk.mapping2.layers.LayerBase");
	var TileLayerBase = jg_import("splunk.mapping2.layers.TileLayerBase");
	var Projection = jg_import("splunk.mapping2.projections.Projection");
	var SphericalMercatorProjection = jg_import("splunk.mapping2.projections.SphericalMercatorProjection");
	var VizBase = jg_import("splunk.viz.VizBase");

	this.Map = jg_extend(VizBase, function(Map, base)
	{

		// Public Events

		this.boundsChanged = new ChainedEvent("boundsChanged", this.changed, EventData);
		this.layersChanged = new ChainedEvent("layersChanged", this.changed, EventData);
		this.controlsChanged = new ChainedEvent("controlsChanged", this.changed, EventData);

		// Public Properties

		this.projection = new ObservableProperty("projection", Projection, SphericalMercatorProjection.getInstance())
			.writeFilter(function(value)
			{
				return value ? value : SphericalMercatorProjection.getInstance();
			})
			.onChanged(function(e)
			{
				if (this._updatePropertiesCache())
					this.notifyBoundsChanged();
			});

		this.center = new ObservableProperty("center", LatLon, new LatLon())
			.readFilter(function(value)
			{
				return value ? value.clone() : new LatLon();
			})
			.writeFilter(function(value)
			{
				return (value && value.isFinite()) ? value.clone() : new LatLon();
			})
			.changedComparator(function(oldValue, newValue)
			{
				return !oldValue.equals(newValue);
			})
			.onChanged(function(e)
			{
				if (this._updatePropertiesCache())
					this.notifyBoundsChanged();
			});

		this.zoom = new ObservableProperty("zoom", Number, 0)
			.writeFilter(function(value)
			{
				value = ((value >= 0) && (value < Infinity)) ? value : 0;
				return NumberUtils.maxMin(value, this._cachedMaxZoom, this._cachedMinZoom);
			})
			.onChanged(function(e)
			{
				if (this._updatePropertiesCache())
					this.notifyBoundsChanged();
			});

		this.minZoom = new ObservableProperty("minZoom", Number, 0)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value < Infinity)) ? Math.floor(value) : 0;
			})
			.onChanged(function(e)
			{
				var zoom = this._updateZoomRangeCache();
				this.set("zoom", zoom);
			});

		this.maxZoom = new ObservableProperty("maxZoom", Number, Infinity)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value < Infinity)) ? Math.floor(value) : Infinity;
			})
			.onChanged(function(e)
			{
				var zoom = this._updateZoomRangeCache();
				this.set("zoom", zoom);
			});

		this.width = new ObservableProperty("width", Number, 0)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value < Infinity)) ? value : 0;
			})
			.onChanged(function(e)
			{
				if (this._updatePropertiesCache())
					this.notifyBoundsChanged();
			});

		this.height = new ObservableProperty("height", Number, 0)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value < Infinity)) ? value : 0;
			})
			.onChanged(function(e)
			{
				if (this._updatePropertiesCache())
					this.notifyBoundsChanged();
			});

		// Private Properties

		this._layers = null;
		this._layersContainer = null;
		this._controls = null;
		this._controlsContainer = null;
		this._updateSizeInterval = 0;

		this._cachedProjection = null;
		this._cachedCenter = null;
		this._cachedZoom = 0;
		this._cachedMinZoom = 0;
		this._cachedMaxZoom = 0;
		this._cachedWidth = 0;
		this._cachedHeight = 0;
		this._cachedTileSize = 0;
		this._cachedMapSize = 0;
		this._cachedViewportOffsetX = 0;
		this._cachedViewportOffsetY = 0;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.addStyleClass("splunk-mapping2-Map");

			this.setStyle({ position: "relative", width: "100%", height: "100%", overflow: "hidden" });

			this.updateSize = FunctionUtils.bind(this.updateSize, this);
			this._layer_tileSize_changed = FunctionUtils.bind(this._layer_tileSize_changed, this);
			this._layer_minZoom_changed = FunctionUtils.bind(this._layer_minZoom_changed, this);
			this._layer_maxZoom_changed = FunctionUtils.bind(this._layer_maxZoom_changed, this);

			this._layers = [];

			this._layersContainer = document.createElement("div");
			this._layersContainer.className = "splunk-mapping2-Map-layers";
			this._layersContainer.style.position = "absolute";
			this._layersContainer.style.left = "0px";
			this._layersContainer.style.top = "0px";

			this._controls = [];

			this._controlsContainer = document.createElement("div");
			this._controlsContainer.className = "splunk-mapping2-Map-controls";
			this._controlsContainer.style.position = "absolute";
			this._controlsContainer.style.left = "0px";
			this._controlsContainer.style.top = "0px";

			this.element.appendChild(this._layersContainer);
			this.element.appendChild(this._controlsContainer);

			this._updatePropertiesCache();
			this._updateTileSizeCache();
			this._updateZoomRangeCache();
		};

		// Public Methods

		this.addLayer = function(layer)
		{
			if (layer == null)
				throw new Error("Parameter layer must be non-null.");
			if (!(layer instanceof LayerBase))
				throw new Error("Parameter layer must be an instance of splunk.mapping2.layers.LayerBase.");

			var map = layer.get("map");
			if (map && (map !== this))
				map.removeLayer(layer);

			var layers = this._layers;
			var index = ArrayUtils.indexOf(layers, layer);
			if (index >= 0)
				layers.splice(index, 1);

			layers.push(layer);
			layer.appendTo(this._layersContainer);
			if (map !== this)
			{
				layer.onAddedToMap(this);
				if (layer instanceof TileLayerBase)
				{
					layer.addEventListener("tileSize.changed", this._layer_tileSize_changed);
					layer.addEventListener("minZoom.changed", this._layer_minZoom_changed);
					layer.addEventListener("maxZoom.changed", this._layer_maxZoom_changed);
				}
			}

			this.dispatchEvent("layersChanged", new EventData());

			if (this._updateTileSizeCache())
				this.notifyBoundsChanged();
			var zoom = this._updateZoomRangeCache();
			this.set("zoom", zoom);
		};

		this.removeLayer = function(layer)
		{
			if (layer == null)
				throw new Error("Parameter layer must be non-null.");
			if (!(layer instanceof LayerBase))
				throw new Error("Parameter layer must be an instance of splunk.mapping2.layers.LayerBase.");

			var layers = this._layers;
			var index = ArrayUtils.indexOf(layers, layer);
			if (index < 0)
				return;

			if (layer instanceof TileLayerBase)
			{
				layer.removeEventListener("tileSize.changed", this._layer_tileSize_changed);
				layer.removeEventListener("minZoom.changed", this._layer_minZoom_changed);
				layer.removeEventListener("maxZoom.changed", this._layer_maxZoom_changed);
			}
			layer.onRemovedFromMap(this);
			layer.remove();
			layers.splice(index, 1);

			this.dispatchEvent("layersChanged", new EventData());

			if (this._updateTileSizeCache())
				this.notifyBoundsChanged();
			var zoom = this._updateZoomRangeCache();
			this.set("zoom", zoom);
		};

		this.getLayers = function()
		{
			return this._layers.concat();
		};

		this.addControl = function(control)
		{
			if (control == null)
				throw new Error("Parameter control must be non-null.");
			if (!(control instanceof ControlBase))
				throw new Error("Parameter control must be an instance of splunk.mapping2.controls.ControlBase.");

			var map = control.get("map");
			if (map && (map !== this))
				map.removeControl(control);

			var controls = this._controls;
			var index = ArrayUtils.indexOf(controls, control);
			if (index >= 0)
				controls.splice(index, 1);

			controls.push(control);
			control.appendTo(this._controlsContainer);
			if (map !== this)
				control.onAddedToMap(this);

			this.dispatchEvent("controlsChanged", new EventData());

			if (this._updateTileSizeCache())
				this.notifyBoundsChanged();
			var zoom = this._updateZoomRangeCache();
			this.set("zoom", zoom);
		};

		this.removeControl = function(control)
		{
			if (control == null)
				throw new Error("Parameter control must be non-null.");
			if (!(control instanceof ControlBase))
				throw new Error("Parameter control must be an instance of splunk.mapping2.controls.ControlBase.");

			var controls = this._controls;
			var index = ArrayUtils.indexOf(controls, control);
			if (index < 0)
				return;

			control.onRemovedFromMap(this);
			control.remove();
			controls.splice(index, 1);

			this.dispatchEvent("controlsChanged", new EventData());

			if (this._updateTileSizeCache())
				this.notifyBoundsChanged();
			var zoom = this._updateZoomRangeCache();
			this.set("zoom", zoom);
		};

		this.getControls = function()
		{
			return this._controls.concat();
		};

		this.fitWorld = function(viewportInside)
		{
			if ((viewportInside != null) && (typeof viewportInside !== "boolean"))
				throw new Error("Parameter viewportInside must be a boolean.");

			this.fitBounds(new LatLonBounds(-60, -180, 85, 180), viewportInside);
		};

		this.fitBounds = function(latLonBounds, viewportInside)
		{
			if (latLonBounds == null)
				throw new Error("Parameter latLonBounds must be non-null.");
			if (!(latLonBounds instanceof LatLonBounds))
				throw new Error("Parameter latLonBounds must be an instance of splunk.mapping2.LatLonBounds.");
			if ((viewportInside != null) && (typeof viewportInside !== "boolean"))
				throw new Error("Parameter viewportInside must be a boolean.");

			latLonBounds = latLonBounds.isFinite() ? latLonBounds : new LatLonBounds(-60, -180, 85, 180);
			viewportInside = (viewportInside === true);

			var projection = this._cachedProjection;
			var minZoom = this._cachedMinZoom;
			var maxZoom = this._cachedMaxZoom;
			var width = this._cachedWidth;
			var height = this._cachedHeight;
			var tileSize = this._cachedTileSize;

			var relativeTL = projection.latLonToRelative(latLonBounds.getNW());
			var relativeBR = projection.latLonToRelative(latLonBounds.getSE());
			var relativeWidth = Math.abs(relativeBR.x - relativeTL.x);
			var relativeHeight = Math.abs(relativeBR.y - relativeTL.y);

			var mapSize1 = (relativeWidth > 0) ? (width / relativeWidth) : Infinity;
			var mapSize2 = (relativeHeight > 0) ? (height / relativeHeight) : Infinity;
			var mapSize = viewportInside ? Math.max(mapSize1, mapSize2) : Math.min(mapSize1, mapSize2);
			var zoom = Infinity;
			if (mapSize < Infinity)
			{
				var mapScale = Math.max(mapSize / tileSize, 1);
				var mapZoom = Math.log(mapScale) / Math.log(2);
				zoom = viewportInside ? Math.ceil(mapZoom) : Math.floor(mapZoom);
			}

			zoom = NumberUtils.maxMin(zoom, maxZoom, minZoom);
			if (zoom < Infinity)
				this.set("zoom", zoom);

			var centerRelative = new Point((relativeTL.x + relativeBR.x) / 2, (relativeTL.y + relativeBR.y) / 2);
			var center = projection.relativeToLatLon(centerRelative);
			this.set("center", center);
		};

		this.latLonToRelative = function(latLon)
		{
			if (latLon == null)
				throw new Error("Parameter latLon must be non-null.");
			if (!(latLon instanceof LatLon))
				throw new Error("Parameter latLon must be an instance of splunk.mapping2.LatLon.");

			return this._cachedProjection.latLonToRelative(latLon);
		};

		this.latLonToAbsolute = function(latLon)
		{
			if (latLon == null)
				throw new Error("Parameter latLon must be non-null.");
			if (!(latLon instanceof LatLon))
				throw new Error("Parameter latLon must be an instance of splunk.mapping2.LatLon.");

			return this.relativeToAbsolute(this.latLonToRelative(latLon));
		};

		this.latLonToViewport = function(latLon)
		{
			if (latLon == null)
				throw new Error("Parameter latLon must be non-null.");
			if (!(latLon instanceof LatLon))
				throw new Error("Parameter latLon must be an instance of splunk.mapping2.LatLon.");

			return this.absoluteToViewport(this.relativeToAbsolute(this.latLonToRelative(latLon)));
		};

		this.relativeToLatLon = function(relativePoint)
		{
			if (relativePoint == null)
				throw new Error("Parameter relativePoint must be non-null.");
			if (!(relativePoint instanceof Point))
				throw new Error("Parameter relativePoint must be an instance of jgatt.geom.Point.");

			return this._cachedProjection.relativeToLatLon(relativePoint);
		};

		this.relativeToAbsolute = function(relativePoint)
		{
			if (relativePoint == null)
				throw new Error("Parameter relativePoint must be non-null.");
			if (!(relativePoint instanceof Point))
				throw new Error("Parameter relativePoint must be an instance of jgatt.geom.Point.");

			return new Point(this._cachedMapSize * relativePoint.x, this._cachedMapSize * relativePoint.y);
		};

		this.relativeToViewport = function(relativePoint)
		{
			if (relativePoint == null)
				throw new Error("Parameter relativePoint must be non-null.");
			if (!(relativePoint instanceof Point))
				throw new Error("Parameter relativePoint must be an instance of jgatt.geom.Point.");

			return this.absoluteToViewport(this.relativeToAbsolute(relativePoint));
		};

		this.absoluteToLatLon = function(absolutePoint)
		{
			if (absolutePoint == null)
				throw new Error("Parameter absolutePoint must be non-null.");
			if (!(absolutePoint instanceof Point))
				throw new Error("Parameter absolutePoint must be an instance of jgatt.geom.Point.");

			return this.relativeToLatLon(this.absoluteToRelative(absolutePoint));
		};

		this.absoluteToRelative = function(absolutePoint)
		{
			if (absolutePoint == null)
				throw new Error("Parameter absolutePoint must be non-null.");
			if (!(absolutePoint instanceof Point))
				throw new Error("Parameter absolutePoint must be an instance of jgatt.geom.Point.");

			return new Point(absolutePoint.x / this._cachedMapSize, absolutePoint.y / this._cachedMapSize);
		};

		this.absoluteToViewport = function(absolutePoint)
		{
			if (absolutePoint == null)
				throw new Error("Parameter absolutePoint must be non-null.");
			if (!(absolutePoint instanceof Point))
				throw new Error("Parameter absolutePoint must be an instance of jgatt.geom.Point.");

			return new Point(absolutePoint.x - this._cachedViewportOffsetX, absolutePoint.y - this._cachedViewportOffsetY);
		};

		this.viewportToLatLon = function(viewportPoint)
		{
			if (viewportPoint == null)
				throw new Error("Parameter viewportPoint must be non-null.");
			if (!(viewportPoint instanceof Point))
				throw new Error("Parameter viewportPoint must be an instance of jgatt.geom.Point.");

			return this.relativeToLatLon(this.absoluteToRelative(this.viewportToAbsolute(viewportPoint)));
		};

		this.viewportToRelative = function(viewportPoint)
		{
			if (viewportPoint == null)
				throw new Error("Parameter viewportPoint must be non-null.");
			if (!(viewportPoint instanceof Point))
				throw new Error("Parameter viewportPoint must be an instance of jgatt.geom.Point.");

			return this.absoluteToRelative(this.viewportToAbsolute(viewportPoint));
		};

		this.viewportToAbsolute = function(viewportPoint)
		{
			if (viewportPoint == null)
				throw new Error("Parameter viewportPoint must be non-null.");
			if (!(viewportPoint instanceof Point))
				throw new Error("Parameter viewportPoint must be an instance of jgatt.geom.Point.");

			return new Point(viewportPoint.x + this._cachedViewportOffsetX, viewportPoint.y + this._cachedViewportOffsetY);
		};

		this.getLatLonBounds = function()
		{
			var tl = this.viewportToLatLon(new Point(0, 0));
			var br = this.viewportToLatLon(new Point(this._cachedWidth, this._cachedHeight));
			return new LatLonBounds(br.lat, tl.lon, tl.lat, br.lon);
		};

		this.getRelativeBounds = function()
		{
			var tl = this.viewportToRelative(new Point(0, 0));
			var br = this.viewportToRelative(new Point(this._cachedWidth, this._cachedHeight));
			return new Rectangle(tl.x, tl.y, br.x - tl.x, br.y - tl.y);
		};

		this.getAbsoluteBounds = function()
		{
			var tl = this.viewportToAbsolute(new Point(0, 0));
			var br = this.viewportToAbsolute(new Point(this._cachedWidth, this._cachedHeight));
			return new Rectangle(tl.x, tl.y, br.x - tl.x, br.y - tl.y);
		};

		this.updateSize = function()
		{
			this.set("width", this.$element.width());
			this.set("height", this.$element.height());
		};

		this.dispose = function()
		{
			var controls = this._controls.concat();
			var control;
			var layers = this._layers.concat();
			var layer;
			var i;

			for (i = controls.length - 1; i >= 0; i--)
			{
				control = controls[i];
				this.removeControl(control);
				control.dispose();
			}

			for (i = layers.length - 1; i >= 0; i--)
			{
				layer = layers[i];
				this.removeLayer(layer);
				layer.dispose();
			}

			base.dispose.call(this);
		};

		// Protected Methods

		this.onAppend = function()
		{
			this._updateSizeInterval = setInterval(this.updateSize, 50);

			this.updateSize();
		};

		this.onRemove = function()
		{
			clearInterval(this._updateSizeInterval);
		};

		this.notifyBoundsChanged = function()
		{
			this.dispatchEvent("boundsChanged", new EventData());
		};

		// Private Methods

		this._updatePropertiesCache = function()
		{
			var projection = this.getInternal("projection");
			var center = this.getInternal("center");
			var zoom = this.getInternal("zoom");
			var width = this.getInternal("width");
			var height = this.getInternal("height");

			if ((this._cachedProjection === projection) &&
			    (this._cachedCenter && this._cachedCenter.equals(center)) &&
			    (this._cachedZoom === zoom) &&
			    (this._cachedWidth === width) &&
			    (this._cachedHeight === height))
				return false;

			this._cachedProjection = projection;
			this._cachedCenter = center;
			this._cachedZoom = zoom;
			this._cachedWidth = width;
			this._cachedHeight = height;

			this._updateComputedCache();

			return true;
		};

		this._updateTileSizeCache = function()
		{
			var tileSize = 256;

			var layers = this._layers;
			var layer;
			for (var i = 0, l = layers.length; i < l; i++)
			{
				layer = layers[i];
				if (layer instanceof TileLayerBase)
				{
					tileSize = layer.get("tileSize");
					break;
				}
			}

			if (this._cachedTileSize === tileSize)
				return false;

			this._cachedTileSize = tileSize;

			this._updateComputedCache();

			return true;
		};

		this._updateZoomRangeCache = function()
		{
			var zoom = this.getInternal("zoom");
			var minZoom = this.getInternal("minZoom");
			var maxZoom = this.getInternal("maxZoom");

			var layers = this._layers;
			var layer;
			for (var i = 0, l = layers.length; i < l; i++)
			{
				layer = layers[i];
				if (layer instanceof TileLayerBase)
				{
					minZoom = Math.max(minZoom, layer.get("minZoom"));
					maxZoom = Math.min(maxZoom, layer.get("maxZoom"));
				}
			}

			this._cachedMinZoom = minZoom;
			this._cachedMaxZoom = maxZoom;

			return NumberUtils.maxMin(zoom, maxZoom, minZoom);
		};

		this._updateComputedCache = function()
		{
			var projection = this._cachedProjection;
			var center = this._cachedCenter;
			var zoom = this._cachedZoom;
			var width = this._cachedWidth;
			var height = this._cachedHeight;
			var tileSize = this._cachedTileSize;

			var mapSize = tileSize * Math.pow(2, zoom);
			var centerRelative = projection.latLonToRelative(center);

			this._cachedMapSize = mapSize;
			this._cachedViewportOffsetX = (mapSize * centerRelative.x) - (width / 2);
			this._cachedViewportOffsetY = (mapSize * centerRelative.y) - (height / 2);
		};

		this._layer_tileSize_changed = function(e)
		{
			if (this._updateTileSizeCache())
				this.notifyBoundsChanged();
		};

		this._layer_minZoom_changed = function(e)
		{
			var zoom = this._updateZoomRangeCache();
			this.set("zoom", zoom);
		};

		this._layer_maxZoom_changed = function(e)
		{
			var zoom = this._updateZoomRangeCache();
			this.set("zoom", zoom);
		};

	});

});
});

jg_import.define("splunk.vectors.VectorElement", function()
{
jg_namespace("splunk.vectors", function()
{

	this.VectorElement = jg_extend(Object, function(VectorElement, base)
	{

		// Private Static Constants

		var _HAS_SVG = (typeof document.createElementNS === "function");
		var _HAS_VML = (!_HAS_SVG && (function()
		{
			try
			{
				document.namespaces.add("splvml", "urn:schemas-microsoft-com:vml");

				var styleText = ".splvml { behavior: url(#default#VML); display: inline-block; position: absolute; }";

				var styleNode = document.createElement("style");
				styleNode.setAttribute("type", "text/css");

				var headNode = document.getElementsByTagName("head")[0];
				headNode.appendChild(styleNode);

				if (styleNode.styleSheet)
					styleNode.styleSheet.cssText = styleText;
				else
					styleNode.appendChild(document.createTextNode(styleText));

				return true;
			}
			catch (e)
			{
				return false;
			}
		})());

		// Public Static Methods

		VectorElement.mixin = function(target, sourceSVG, sourceVML)
		{
			if (_HAS_SVG)
			{
				jg_mixin(target, sourceSVG);
				// jg_mixin doesn't copy constructor, so do it manually
				if ((sourceSVG.constructor !== Object) && (typeof sourceSVG.constructor === "function"))
					target.constructor = sourceSVG.constructor;
			}
			else if (_HAS_VML)
			{
				jg_mixin(target, sourceVML);
				// jg_mixin doesn't copy constructor, so do it manually
				if ((sourceVML.constructor !== Object) && (typeof sourceVML.constructor === "function"))
					target.constructor = sourceVML.constructor;
			}
		};

		// Public Properties

		this.hasSVG = _HAS_SVG;
		this.hasVML = _HAS_VML;
		this.element = null;

		// Constructor

		this.constructor = function(tagName)
		{
			if ((tagName != null) && (typeof tagName !== "string"))
				throw new Error("Parameter tagName must be a string.");

			this.element = this.createElement(tagName || null);
		};

		// Public Methods

		this.appendTo = function(parentElement)
		{
			if (parentElement == null)
				throw new Error("Parameter parentElement must be non-null.");
			if (!(parentElement instanceof VectorElement))
				throw new Error("Parameter parentElement must be an instance of splunk.vectors.VectorElement.");

			parentElement.element.appendChild(this.element);

			return this;
		};

		this.remove = function()
		{
			if (this.element.parentNode)
				this.element.parentNode.removeChild(this.element);

			return this;
		};

		this.dispose = function()
		{
			this.remove();

			this.element = null;
		};

		this.display = function(value)
		{
			this.element.style.display = value ? value : "";

			return this;
		};

		this.visibility = function(value)
		{
			this.element.style.visibility = value ? value : "";

			return this;
		};

		this.translate = function(x, y)
		{
			x = ((x != null) && (x > -Infinity) && (x < Infinity)) ? x : 0;
			y = ((y != null) && (y > -Infinity) && (y < Infinity)) ? y : 0;

			this.element.style.left = (x != 0) ? x + "px" : "";
			this.element.style.top = (y != 0) ? y + "px" : "";

			return this;
		};

		// Protected Methods

		this.createElement = function(tagName)
		{
			var dummy = document.createElement("div");
			dummy.style.position = "absolute";
			return dummy;
		};

		// Inner Mixin Classes

		var SVGVectorElement = jg_static(function(SVGVectorElement)
		{

			// Private Static Constants

			var _NS_SVG = "http://www.w3.org/2000/svg";

			// Public Methods

			this.display = function(value)
			{
				if (value)
					this.element.setAttribute("display", value);
				else
					this.element.removeAttribute("display");

				return this;
			};

			this.visibility = function(value)
			{
				if (value)
					this.element.setAttribute("visibility", value);
				else
					this.element.removeAttribute("visibility");

				return this;
			};

			this.translate = function(x, y)
			{
				x = ((x != null) && (x > -Infinity) && (x < Infinity)) ? x : 0;
				y = ((y != null) && (y > -Infinity) && (y < Infinity)) ? y : 0;

				if ((x != 0) || (y != 0))
					this.element.setAttribute("transform", "translate(" + x + "," + y + ")");
				else
					this.element.removeAttribute("transform");

				return this;
			};

			// Protected Methods

			this.createElement = function(tagName)
			{
				return document.createElementNS(_NS_SVG, tagName || "g");
			};

		});

		var VMLVectorElement = jg_static(function(VMLVectorElement)
		{

			// Protected Methods

			this.createElement = function(tagName)
			{
				return document.createElement("<splvml:" + (tagName || "group") + " class=\"splvml\">");
			};

		});

		VectorElement.mixin(this, SVGVectorElement, VMLVectorElement);

	});

});
});

jg_import.define("splunk.vectors.Shape", function()
{
jg_namespace("splunk.vectors", function()
{

	var NumberUtils = jg_import("jgatt.utils.NumberUtils");
	var VectorElement = jg_import("splunk.vectors.VectorElement");

	this.Shape = jg_extend(VectorElement, function(Shape, base)
	{

		// Constructor

		this.constructor = function(tagName)
		{
			base.constructor.call(this, tagName);
		};

		// Public Methods

		this.fillColor = function(value)
		{
			return this;
		};

		this.fillOpacity = function(value)
		{
			return this;
		};

		this.strokeColor = function(value)
		{
			return this;
		};

		this.strokeOpacity = function(value)
		{
			return this;
		};

		this.strokeWidth = function(value)
		{
			return this;
		};

		this.strokeLineCap = function(value)
		{
			return this;
		};

		this.strokeLineJoin = function(value)
		{
			return this;
		};

		this.strokeMiterLimit = function(value)
		{
			return this;
		};

		// Inner Mixin Classes

		var SVGShape = jg_static(function(SVGShape)
		{

			// Constructor

			this.constructor = function(tagName)
			{
				base.constructor.call(this, tagName);

				this.fillColor(NaN);
				this.strokeColor(NaN);
				this.strokeLineCap("none");
				this.strokeLineJoin("miter");
			};

			// Public Methods

			this.fillColor = function(value)
			{
				if ((value != null) && !isNaN(value))
				{
					value = NumberUtils.minMax(Math.floor(value), 0x000000, 0xFFFFFF);
					this.element.setAttribute("fill", "#" + (value | 0x1000000).toString(16).substring(1));
				}
				else
				{
					this.element.setAttribute("fill", "none");
				}

				return this;
			};

			this.fillOpacity = function(value)
			{
				if ((value != null) && !isNaN(value))
					this.element.setAttribute("fill-opacity", NumberUtils.minMax(value, 0, 1));
				else
					this.element.removeAttribute("fill-opacity");

				return this;
			};

			this.strokeColor = function(value)
			{
				if ((value != null) && !isNaN(value))
				{
					value = NumberUtils.minMax(Math.floor(value), 0x000000, 0xFFFFFF);
					this.element.setAttribute("stroke", "#" + (value | 0x1000000).toString(16).substring(1));
				}
				else
				{
					this.element.removeAttribute("stroke");
				}

				return this;
			};

			this.strokeOpacity = function(value)
			{
				if ((value != null) && !isNaN(value))
					this.element.setAttribute("stroke-opacity", NumberUtils.minMax(value, 0, 1));
				else
					this.element.removeAttribute("stroke-opacity");

				return this;
			};

			this.strokeWidth = function(value)
			{
				if ((value != null) && (value < Infinity))
					this.element.setAttribute("stroke-width", Math.max(value, 1));
				else
					this.element.removeAttribute("stroke-width");

				return this;
			};

			this.strokeLineCap = function(value)
			{
				if (value === "round")
					this.element.setAttribute("stroke-linecap", "round");
				else if (value === "square")
					this.element.setAttribute("stroke-linecap", "square");
				else  // none
					this.element.removeAttribute("stroke-linecap");

				return this;
			};

			this.strokeLineJoin = function(value)
			{
				if (value === "round")
					this.element.setAttribute("stroke-linejoin", "round");
				else if (value === "bevel")
					this.element.setAttribute("stroke-linejoin", "bevel");
				else  // miter
					this.element.removeAttribute("stroke-linejoin");

				return this;
			};

			this.strokeMiterLimit = function(value)
			{
				if ((value != null) && (value < Infinity))
					this.element.setAttribute("stroke-miterlimit", Math.max(value, 1));
				else
					this.element.removeAttribute("stroke-miterlimit");

				return this;
			};

		});

		var VMLShape = jg_static(function(VMLShape)
		{

			// Private Properties

			this._fillElement = null;
			this._strokeElement = null;

			// Constructor

			this.constructor = function(tagName)
			{
				base.constructor.call(this, tagName);

				this._fillElement = this.createElement("fill");
				this._strokeElement = this.createElement("stroke");

				this.element.appendChild(this._fillElement);
				this.element.appendChild(this._strokeElement);

				this.fillColor(NaN);
				this.strokeColor(NaN);
				this.strokeLineCap("none");
				this.strokeLineJoin("miter");
			};

			// Public Methods

			this.dispose = function()
			{
				base.dispose.call(this);

				this._fillElement = null;
				this._strokeElement = null;
			};

			this.fillColor = function(value)
			{
				if ((value != null) && !isNaN(value))
				{
					value = NumberUtils.minMax(Math.floor(value), 0x000000, 0xFFFFFF);
					this._fillElement.on = true;
					this._fillElement.color = "#" + (value | 0x1000000).toString(16).substring(1);
				}
				else
				{
					this._fillElement.on = false;
					this._fillElement.color = "#000000";
				}

				return this;
			};

			this.fillOpacity = function(value)
			{
				if ((value != null) && !isNaN(value))
					this._fillElement.opacity = NumberUtils.minMax(value, 0, 1);
				else
					this._fillElement.opacity = 1;

				return this;
			};

			this.strokeColor = function(value)
			{
				if ((value != null) && !isNaN(value))
				{
					value = NumberUtils.minMax(Math.floor(value), 0x000000, 0xFFFFFF);
					this._strokeElement.on = true;
					this._strokeElement.color = "#" + (value | 0x1000000).toString(16).substring(1);
				}
				else
				{
					this._strokeElement.on = false;
					this._strokeElement.color = "#000000";
				}

				return this;
			};

			this.strokeOpacity = function(value)
			{
				if ((value != null) && !isNaN(value))
					this._strokeElement.opacity = NumberUtils.minMax(value, 0, 1);
				else
					this._strokeElement.opacity = 1;

				return this;
			};

			this.strokeWidth = function(value)
			{
				if ((value != null) && (value < Infinity))
					this._strokeElement.weight = Math.max(value, 1) + "px";
				else
					this._strokeElement.weight = "1px";

				return this;
			};

			this.strokeLineCap = function(value)
			{
				if (value === "round")
					this._strokeElement.endcap = "round";
				else if (value === "square")
					this._strokeElement.endcap = "square";
				else // none
					this._strokeElement.endcap = "flat";

				return this;
			};

			this.strokeLineJoin = function(value)
			{
				if (value === "round")
					this._strokeElement.joinstyle = "round";
				else if (value === "bevel")
					this._strokeElement.joinstyle = "bevel";
				else // miter
					this._strokeElement.joinstyle = "miter";

				return this;
			};

			this.strokeMiterLimit = function(value)
			{
				if ((value != null) && (value < Infinity))
					this._strokeElement.miterlimit = Math.max(value, 1);
				else
					this._strokeElement.miterlimit = 4;

				return this;
			};

		});

		VectorElement.mixin(this, SVGShape, VMLShape);

	});

});
});

jg_import.define("splunk.vectors.Rect", function()
{
jg_namespace("splunk.vectors", function()
{

	var Shape = jg_import("splunk.vectors.Shape");
	var VectorElement = jg_import("splunk.vectors.VectorElement");

	this.Rect = jg_extend(Shape, function(Rect, base)
	{

		// Constructor

		this.constructor = function(x, y, width, height)
		{
			base.constructor.call(this);
		};

		// Public Methods

		this.x = function(value)
		{
			return this;
		};

		this.y = function(value)
		{
			return this;
		};

		this.width = function(value)
		{
			return this;
		};

		this.height = function(value)
		{
			return this;
		};

		// Inner Mixin Classes

		var SVGRect = jg_static(function(SVGRect)
		{

			// Constructor

			this.constructor = function(x, y, width, height)
			{
				base.constructor.call(this, "rect");

				if (x != null)
					this.x(x);
				if (y != null)
					this.y(y);
				if (width != null)
					this.width(width);
				if (height != null)
					this.height(height);
			};

			// Public Methods

			this.x = function(value)
			{
				if ((value != null) && (value > -Infinity) && (value < Infinity))
					this.element.setAttribute("x", value);
				else
					this.element.removeAttribute("x");

				return this;
			};

			this.y = function(value)
			{
				if ((value != null) && (value > -Infinity) && (value < Infinity))
					this.element.setAttribute("y", value);
				else
					this.element.removeAttribute("y");

				return this;
			};

			this.width = function(value)
			{
				if ((value != null) && (value < Infinity))
					this.element.setAttribute("width", Math.max(value, 0));
				else
					this.element.removeAttribute("width");

				return this;
			};

			this.height = function(value)
			{
				if ((value != null) && (value < Infinity))
					this.element.setAttribute("height", Math.max(value, 0));
				else
					this.element.removeAttribute("height");

				return this;
			};

		});

		var VMLRect = jg_static(function(VMLRect)
		{

			// Constructor

			this.constructor = function(x, y, width, height)
			{
				base.constructor.call(this, "rect");

				if (x != null)
					this.x(x);
				if (y != null)
					this.y(y);
				if (width != null)
					this.width(width);
				if (height != null)
					this.height(height);
			};

			// Public Methods

			this.x = function(value)
			{
				if ((value != null) && (value > -Infinity) && (value < Infinity))
					this.element.style.left = Math.round(value) + "px";
				else
					this.element.style.left = "";

				return this;
			};

			this.y = function(value)
			{
				if ((value != null) && (value > -Infinity) && (value < Infinity))
					this.element.style.top = Math.round(value) + "px";
				else
					this.element.style.top = "";

				return this;
			};

			this.width = function(value)
			{
				if ((value != null) && (value < Infinity))
					this.element.style.width = Math.round(Math.max(value, 0)) + "px";
				else
					this.element.style.width = "";

				return this;
			};

			this.height = function(value)
			{
				if ((value != null) && (value < Infinity))
					this.element.style.height = Math.round(Math.max(value, 0)) + "px";
				else
					this.element.style.height = "";

				return this;
			};

		});

		VectorElement.mixin(this, SVGRect, VMLRect);

	});

});
});

jg_import.define("splunk.vectors.Text", function()
{
jg_namespace("splunk.vectors", function()
{

	var Rectangle = jg_import("jgatt.geom.Rectangle");
	var Shape = jg_import("splunk.vectors.Shape");
	var VectorElement = jg_import("splunk.vectors.VectorElement");

	this.Text = jg_extend(Shape, function(Text, base)
	{

		// Constructor

		this.constructor = function(x, y, text)
		{
			base.constructor.call(this);
		};

		// Public Methods

		this.x = function(value)
		{
			return this;
		};

		this.y = function(value)
		{
			return this;
		};

		this.text = function(value)
		{
			return this;
		};

		this.alignmentBaseline = function(value)
		{
			return this;
		};

		this.baselineShift = function(value)
		{
			return this;
		};

		this.direction = function(value)
		{
			return this;
		};

		this.dominantBaseline = function(value)
		{
			return this;
		};

		this.fontFamily = function(value)
		{
			return this;
		};

		this.fontSize = function(value)
		{
			return this;
		};

		this.fontSizeAdjust = function(value)
		{
			return this;
		};

		this.fontStretch = function(value)
		{
			return this;
		};

		this.fontStyle = function(value)
		{
			return this;
		};

		this.fontVariant = function(value)
		{
			return this;
		};

		this.fontWeight = function(value)
		{
			return this;
		};

		this.glyphOrientationHorizontal = function(value)
		{
			return this;
		};

		this.glyphOrientationVertical = function(value)
		{
			return this;
		};

		this.kerning = function(value)
		{
			return this;
		};

		this.letterSpacing = function(value)
		{
			return this;
		};

		this.textAnchor = function(value)
		{
			return this;
		};

		this.textDecoration = function(value)
		{
			return this;
		};

		this.textRendering = function(value)
		{
			return this;
		};

		this.unicodeBidi = function(value)
		{
			return this;
		};

		this.wordSpacing = function(value)
		{
			return this;
		};

		this.writingMode = function(value)
		{
			return this;
		};

		this.getBounds = function()
		{
			return new Rectangle();
		};

		// Inner Mixin Classes

		var SVGText = jg_static(function(SVGText)
		{

			// Constructor

			this.constructor = function(x, y, text)
			{
				base.constructor.call(this, "text");

				this.fillColor(0x000000);
				if (x != null)
					this.x(x);
				if (y != null)
					this.y(y);
				if (text != null)
					this.text(text);
			};

			// Public Methods

			this.x = function(value)
			{
				if ((value != null) && (value > -Infinity) && (value < Infinity))
					this.element.setAttribute("x", value);
				else
					this.element.removeAttribute("x");

				return this;
			};

			this.y = function(value)
			{
				if ((value != null) && (value > -Infinity) && (value < Infinity))
					this.element.setAttribute("y", value);
				else
					this.element.removeAttribute("y");

				return this;
			};

			this.text = function(value)
			{
				this.element.textContent = value ? value : "";

				return this;
			};

			this.alignmentBaseline = function(value)
			{
				if (value)
					this.element.setAttribute("alignment-baseline", value);
				else
					this.element.removeAttribute("alignment-baseline");

				return this;
			};

			this.baselineShift = function(value)
			{
				if (value)
					this.element.setAttribute("baseline-shift", value);
				else
					this.element.removeAttribute("baseline-shift");

				return this;
			};

			this.direction = function(value)
			{
				if (value)
					this.element.setAttribute("direction", value);
				else
					this.element.removeAttribute("direction");

				return this;
			};

			this.dominantBaseline = function(value)
			{
				if (value)
					this.element.setAttribute("dominant-baseline", value);
				else
					this.element.removeAttribute("dominant-baseline");

				return this;
			};

			this.fontFamily = function(value)
			{
				if (value)
					this.element.setAttribute("font-family", value);
				else
					this.element.removeAttribute("font-family");

				return this;
			};

			this.fontSize = function(value)
			{
				if (value)
					this.element.setAttribute("font-size", value);
				else
					this.element.removeAttribute("font-size");

				return this;
			};

			this.fontSizeAdjust = function(value)
			{
				if (value)
					this.element.setAttribute("font-size-adjust", value);
				else
					this.element.removeAttribute("font-size-adjust");

				return this;
			};

			this.fontStretch = function(value)
			{
				if (value)
					this.element.setAttribute("font-stretch", value);
				else
					this.element.removeAttribute("font-stretch");

				return this;
			};

			this.fontStyle = function(value)
			{
				if (value)
					this.element.setAttribute("font-style", value);
				else
					this.element.removeAttribute("font-style");

				return this;
			};

			this.fontVariant = function(value)
			{
				if (value)
					this.element.setAttribute("font-variant", value);
				else
					this.element.removeAttribute("font-variant");

				return this;
			};

			this.fontWeight = function(value)
			{
				if (value)
					this.element.setAttribute("font-weight", value);
				else
					this.element.removeAttribute("font-weight");

				return this;
			};

			this.glyphOrientationHorizontal = function(value)
			{
				if (value)
					this.element.setAttribute("glyph-orientation-horizontal", value);
				else
					this.element.removeAttribute("glyph-orientation-horizontal");

				return this;
			};

			this.glyphOrientationVertical = function(value)
			{
				if (value)
					this.element.setAttribute("glyph-orientation-vertical", value);
				else
					this.element.removeAttribute("glyph-orientation-vertical");

				return this;
			};

			this.kerning = function(value)
			{
				if (value)
					this.element.setAttribute("kerning", value);
				else
					this.element.removeAttribute("kerning");

				return this;
			};

			this.letterSpacing = function(value)
			{
				if (value)
					this.element.setAttribute("letter-spacing", value);
				else
					this.element.removeAttribute("letter-spacing");

				return this;
			};

			this.textAnchor = function(value)
			{
				if (value)
					this.element.setAttribute("text-anchor", value);
				else
					this.element.removeAttribute("text-anchor");

				return this;
			};

			this.textDecoration = function(value)
			{
				if (value)
					this.element.setAttribute("text-decoration", value);
				else
					this.element.removeAttribute("text-decoration");

				return this;
			};

			this.textRendering = function(value)
			{
				if (value)
					this.element.setAttribute("text-rendering", value);
				else
					this.element.removeAttribute("text-rendering");

				return this;
			};

			this.unicodeBidi = function(value)
			{
				if (value)
					this.element.setAttribute("unicode-bidi", value);
				else
					this.element.removeAttribute("unicode-bidi");

				return this;
			};

			this.wordSpacing = function(value)
			{
				if (value)
					this.element.setAttribute("word-spacing", value);
				else
					this.element.removeAttribute("word-spacing");

				return this;
			};

			this.writingMode = function(value)
			{
				if (value)
					this.element.setAttribute("writing-mode", value);
				else
					this.element.removeAttribute("writing-mode");

				return this;
			};

			this.getBounds = function()
			{
				try
				{
					var box = this.element.getBBox();
					return new Rectangle(box.x, box.y, box.width, box.height);
				}
				catch (e)
				{
					return new Rectangle();
				}
			};

		});

		var VMLText = jg_static(function(VMLText)
		{
		});

		VectorElement.mixin(this, SVGText, VMLText);

	});

});
});

jg_import.define("splunk.vectors.VectorUtils", function()
{
jg_namespace("splunk.vectors", function()
{

	this.VectorUtils = jg_static(function(VectorUtils)
	{

		// Public Static Methods

		VectorUtils.toSVGString = function(element)
		{
			// svg elements don't have innerHTML attribute...
			// clone svg element and place in container div so we can use innerHTML of the container
			var clonedElement = element.cloneNode(true);
			var containerElement = document.createElement("div");
			containerElement.appendChild(clonedElement);

			// get svg string using innerHTML
			var svgString = containerElement.innerHTML;

			// fix or add xlink namespace on href attributes
			svgString = svgString.replace(/xlink:href=|href=/g, "x:href=");

			// properly close image tags
			svgString = svgString.replace(/<image([\S\s]*?)\s*\/?>\s*(<\/image>)?/g, "<image$1></image>");

			// add xmlns attributes to root svg tag
			svgString = svgString.replace(/^<svg/, "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:x=\"http://www.w3.org/1999/xlink\"");

			// clear element references
			clonedElement = null;
			containerElement = null;

			return svgString;
		};

		VectorUtils.concatSVGStrings = function(/*...*/)
		{
			var concatString = "";
			var svgString;
			var viewBoxMatch;
			var viewBox;
			var width = 0;
			var height = 0;

			for (var i = 0, l = arguments.length; i < l; i++)
			{
				svgString = arguments[i];

				// read and parse viewBox attribute from root svg tag
				viewBoxMatch = svgString.match(/^<svg[^>]*viewBox=\"([^ ]+) ([^ ]+) ([^ ]+) ([^\"]+)\"[^>]*>/);
				if (viewBoxMatch && (viewBoxMatch.length == 5))
				{
					viewBox = {
						x: Number(viewBoxMatch[1]),
						y: Number(viewBoxMatch[2]),
						width: Number(viewBoxMatch[3]),
						height: Number(viewBoxMatch[4])
					};

					// expand width and height to include viewBox
					width = Math.max(width, viewBox.width);
					height = Math.max(height, viewBox.height);
				}
				else
				{
					viewBox = null;
				}

				// replace root svg tag with g tag, including translate transform if needed
				if (viewBox && ((viewBox.x != 0) || (viewBox.y != 0)))
					svgString = svgString.replace(/^<svg[^>]*>/, "<g transform=\"translate(" + (-viewBox.x) + ", " + (-viewBox.y) + ")\">");
				else
					svgString = svgString.replace(/^<svg[^>]*>/, "<g>");
				svgString = svgString.replace(/<\/svg>$/, "</g>");

				concatString += svgString;
			}

			// generate new root svg tag around concatString
			svgString = "<svg";
			svgString += " xmlns=\"http://www.w3.org/2000/svg\"";
			svgString += " xmlns:x=\"http://www.w3.org/1999/xlink\"";
			svgString += " width=\"" + width + "\"";
			svgString += " height=\"" + height + "\"";
			svgString += " viewBox=\"0 0 " + width + " " + height + "\"";
			svgString += ">";
			svgString += concatString;
			svgString += "</svg>";

			return svgString;
		};

	});

});
});

jg_import.define("splunk.vectors.Viewport", function()
{
jg_namespace("splunk.vectors", function()
{

	var Rectangle = jg_import("jgatt.geom.Rectangle");
	var VectorElement = jg_import("splunk.vectors.VectorElement");
	var VectorUtils = jg_import("splunk.vectors.VectorUtils");

	this.Viewport = jg_extend(VectorElement, function(Viewport, base)
	{

		// Constructor

		this.constructor = function(width, height, viewBox, preserveAspectRatio)
		{
			base.constructor.call(this);
		};

		// Public Methods

		this.width = function(value)
		{
			return this;
		};

		this.height = function(value)
		{
			return this;
		};

		this.viewBox = function(value)
		{
			return this;
		};

		this.preserveAspectRatio = function(value)
		{
			return this;
		};

		this.toSVGString = function()
		{
			return "";
		};

		// Inner Mixin Classes

		var SVGViewport = jg_static(function(SVGViewport)
		{

			// Constructor

			this.constructor = function(width, height, viewBox, preserveAspectRatio)
			{
				base.constructor.call(this, "svg");

				this.width((width != null) ? width : 0);
				this.height((height != null) ? height : 0);
				if (viewBox != null)
					this.viewBox(viewBox);
				if (preserveAspectRatio != null)
					this.preserveAspectRatio(preserveAspectRatio);
			};

			// Public Methods

			this.appendTo = function(parentElement)
			{
				if (parentElement == null)
					throw new Error("Parameter parentElement must be non-null.");
				if (parentElement.appendChild == null)
					throw new Error("Parameter parentElement must be a DOM node.");

				parentElement.appendChild(this.element);

				return this;
			};

			this.width = function(value)
			{
				if ((value != null) && (value < Infinity))
					this.element.setAttribute("width", Math.max(value, 0));
				else
					this.element.setAttribute("width", 0);

				return this;
			};

			this.height = function(value)
			{
				if ((value != null) && (value < Infinity))
					this.element.setAttribute("height", Math.max(value, 0));
				else
					this.element.setAttribute("height", 0);

				return this;
			};

			this.viewBox = function(value)
			{
				if (value && (value instanceof Rectangle) && value.isFinite())
					this.element.setAttribute("viewBox", value.x + " " + value.y + " " + value.width + " " + value.height);
				else
					this.element.removeAttribute("viewBox");

				return this;
			};

			this.preserveAspectRatio = function(value)
			{
				if (value)
					this.element.setAttribute("preserveAspectRatio", value);
				else
					this.element.removeAttribute("preserveAspectRatio");

				return this;
			};

			this.toSVGString = function()
			{
				return VectorUtils.toSVGString(this.element);
			};

		});

		var VMLViewport = jg_static(function(VMLViewport)
		{

			// Private Properties

			this._containerElement = null;
			this._width = 0;
			this._height = 0;
			this._viewBox = null;

			// Constructor

			this.constructor = function(width, height, viewBox, preserveAspectRatio)
			{
				base.constructor.call(this, "group");

				this._containerElement = document.createElement("div");
				this._containerElement.style.position = "relative";
				this._containerElement.style.overflow = "hidden";
				this._containerElement.appendChild(this.element);

				this.width((width != null) ? width : 0);
				this.height((height != null) ? height : 0);
				if (viewBox != null)
					this.viewBox(viewBox);
				if (preserveAspectRatio != null)
					this.preserveAspectRatio(preserveAspectRatio);
			};

			// Public Methods

			this.appendTo = function(parentElement)
			{
				if (parentElement == null)
					throw new Error("Parameter parentElement must be non-null.");
				if (parentElement.appendChild == null)
					throw new Error("Parameter parentElement must be a DOM node.");

				parentElement.appendChild(this._containerElement);

				return this;
			};

			this.remove = function()
			{
				if (this._containerElement.parentNode)
					this._containerElement.parentNode.removeChild(this._containerElement);

				return this;
			};

			this.dispose = function()
			{
				base.dispose.call(this);

				this._containerElement = null;
			};

			this.display = function(value)
			{
				this._containerElement.style.display = value ? value : "";

				return this;
			};

			this.visibility = function(value)
			{
				this._containerElement.style.visibility = value ? value : "";

				return this;
			};

			this.translate = function(x, y)
			{
				x = ((x != null) && (x > -Infinity) && (x < Infinity)) ? x : 0;
				y = ((y != null) && (y > -Infinity) && (y < Infinity)) ? y : 0;

				this._containerElement.style.left = (x != 0) ? x + "px" : "";
				this._containerElement.style.top = (y != 0) ? y + "px" : "";

				return this;
			};

			this.width = function(value)
			{
				this._width = ((value != null) && (value < Infinity)) ? Math.max(value, 0) : 0;
				this._updateView();

				return this;
			};

			this.height = function(value)
			{
				this._height = ((value != null) && (value < Infinity)) ? Math.max(value, 0) : 0;
				this._updateView();

				return this;
			};

			this.viewBox = function(value)
			{
				this._viewBox = (value && (value instanceof Rectangle) && value.isFinite()) ? value.clone() : null;
				this._updateView();

				return this;
			};

			this.preserveAspectRatio = function(value)
			{
				return this;
			};

			// Private Methods

			this._updateView = function()
			{
				var width = Math.round(this._width);
				var height = Math.round(this._height);
				var viewBox = this._viewBox;
				var viewX = viewBox ? Math.round(viewBox.x) : 0;
				var viewY = viewBox ? Math.round(viewBox.y) : 0;
				var viewWidth = viewBox ? Math.round(Math.max(viewBox.width, 1)) : width;
				var viewHeight = viewBox ? Math.round(Math.max(viewBox.height, 1)) : height;

				var element = this.element;
				var style = element.style;
				var containerStyle = this._containerElement.style;

				style.display = "none";  // prevent premature rendering

				element.coordorigin = viewX + "," + viewY;
				element.coordsize = viewWidth + "," + viewHeight;

				style.width = width + "px";
				style.height = height + "px";

				containerStyle.width = width + "px";
				containerStyle.height = height + "px";

				style.display = "";  // enable rendering
			};

		});

		VectorElement.mixin(this, SVGViewport, VMLViewport);

	});

});
});

jg_import.define("splunk.mapping2.controls.SVGAttributionControl", function()
{
jg_namespace("splunk.mapping2.controls", function()
{

	var Rectangle = jg_import("jgatt.geom.Rectangle");
	var FunctionUtils = jg_import("jgatt.utils.FunctionUtils");
	var ControlBase = jg_import("splunk.mapping2.controls.ControlBase");
	var MAttributionTarget = jg_import("splunk.mapping2.controls.MAttributionTarget");
	var Rect = jg_import("splunk.vectors.Rect");
	var Text = jg_import("splunk.vectors.Text");
	var Viewport = jg_import("splunk.vectors.Viewport");

	this.SVGAttributionControl = jg_extend(ControlBase, function(SVGAttributionControl, base)
	{

		// Private Properties

		this._layers = null;
		this._attributionViewport = null;
		this._attributionText = null;
		this._attributionRect = null;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.addStyleClass("splunk-mapping2-controls-SVGAttributionControl");

			this._map_layersChanged = FunctionUtils.bind(this._map_layersChanged, this);
			this._layer_attribution_changed = FunctionUtils.bind(this._layer_attribution_changed, this);

			this._layers = [];

			this._attributionViewport = new Viewport()
				.appendTo(this.element);

			this._attributionRect = new Rect()
				.fillColor(0xFFFFFF)
				.fillOpacity(0.7)
				.appendTo(this._attributionViewport);

			this._attributionText = new Text()
				.fillColor(0x333333)
				.fontFamily("Helvetica, Arial, sans-serif")
				.fontSize("11px")
				.dominantBaseline("text-before-edge")
				.appendTo(this._attributionViewport);
		};

		// Public Methods

		this.toSVGString = function()
		{
			this.validate();

			return this._attributionViewport.toSVGString();
		};

		// Protected Methods

		this.renderOverride = function(map)
		{
			var width = map.get("width");
			var height = map.get("height");

			var str = "";
			var layers = this._layers;
			var attribution;
			for (var i = 0, l = layers.length; i < l; i++)
			{
				attribution = layers[i].get(MAttributionTarget.attribution);
				if (attribution)
				{
					if (str)
						str += " ";
					str += attribution;
				}
			}

			var text = this._attributionText;
			text.text(str);

			var padding = str ? { left: 5, top: 1, right: 0, bottom: 1 } : { left: 0, top: 0, right: 0, bottom: 0 };

			var textBounds = text.getBounds();

			var rectWidth = Math.round(textBounds.width + padding.left + padding.right);
			var rectHeight = Math.round(textBounds.height + padding.top + padding.bottom);
			var rectX = Math.round(width - rectWidth);
			var rectY = Math.round(height - rectHeight);

			var textX = Math.round(rectX + padding.left);
			var textY = Math.round(rectY + padding.top);

			text.x(textX);
			text.y(textY);
			text.display(str ? null : "none");

			var rect = this._attributionRect;
			rect.x(rectX);
			rect.y(rectY);
			rect.width(rectWidth);
			rect.height(rectHeight);
			rect.display(str ? null : "none");

			var viewport = this._attributionViewport;
			viewport.width(width);
			viewport.height(height);
			viewport.viewBox(new Rectangle(0, 0, width, height));
		};

		this.onAddedToMap = function(map)
		{
			base.onAddedToMap.call(this, map);

			map.addEventListener("layersChanged", this._map_layersChanged);

			this._updateLayers();
		};

		this.onRemovedFromMap = function(map)
		{
			map.removeEventListener("layersChanged", this._map_layersChanged);

			base.onRemovedFromMap.call(this, map);

			this._updateLayers();
		};

		// Private Methods

		this._updateLayers = function()
		{
			var map = this.getInternal("map");
			var oldLayers = this._layers;
			var newLayers = this._layers = map ? map.getLayers() : [];
			var layer;
			var i, l;

			for (i = 0, l = oldLayers.length; i < l; i++)
				oldLayers[i].removeEventListener("attribution.changed", this._layer_attribution_changed);

			for (i = 0, l = newLayers.length; i < l; i++)
				newLayers[i].addEventListener("attribution.changed", this._layer_attribution_changed);

			this.invalidate("renderPass");
		};

		this._map_layersChanged = function(e)
		{
			this._updateLayers();
		};

		this._layer_attribution_changed = function(e)
		{
			this.invalidate("renderPass");
		};

	});

});
});

jg_import.define("splunk.mapping2.layers.MarkerBase", function()
{
jg_namespace("splunk.mapping2.layers", function()
{

	var Point = jg_import("jgatt.geom.Point");
	var ObservableProperty = jg_import("jgatt.properties.ObservableProperty");
	var Property = jg_import("jgatt.properties.Property");
	var LatLon = jg_import("splunk.mapping2.LatLon");
	var VectorElement = jg_import("splunk.vectors.VectorElement");
	var MRenderTarget = jg_import("splunk.viz.MRenderTarget");

	this.MarkerBase = jg_extend(Object, function(MarkerBase, base)
	{

		base = jg_mixin(this, MRenderTarget, base);

		// Public Properties

		this.latLon = new ObservableProperty("latLon", LatLon, new LatLon())
			.readFilter(function(value)
			{
				return value ? value.clone() : new LatLon();
			})
			.writeFilter(function(value)
			{
				return (value && value.isFinite()) ? value.clone() : new LatLon();
			})
			.changedComparator(function(oldValue, newValue)
			{
				return !oldValue.equals(newValue);
			});

		this.size = new ObservableProperty("size", Number, 50)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value < Infinity)) ? value : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		this.markerLayer = new Property("markerLayer", Object, null, true);

		this.renderPriority = 1;

		this.element = null;

		this.metadata = null;

		// Private Properties

		this._position = null;

		// Constructor

		this.constructor = function(element, latLon, size)
		{
			if (element == null)
				throw new Error("Parameter element must be non-null.");
			if (!(element instanceof VectorElement))
				throw new Error("Parameter element must be an instance of splunk.vectors.VectorElement.");

			base.constructor.call(this);

			this.element = element;

			this.metadata = {};

			if (latLon != null)
				this.set("latLon", latLon);
			if (size != null)
				this.set("size", size);

			this._position = new Point();
		};

		// Public Methods

		this.render = function(position)
		{
			this.validatePreceding("renderPass");

			if (position == null)
				position = this._position;
			else if (!(position instanceof Point))
				throw new Error("Parameter position must be an instance of jgatt.geom.Point.");

			if (this.isValid("renderPass"))
			{
				if (this._position.equals(position))
					return;
				this.invalidate("renderPass");
			}

			if (this._position !== position)
				this._position = position.clone();

			this.renderOverride(this.element, position.x, position.y, this.getInternal("size"));

			this.setValid("renderPass");
		};

		// Protected Methods

		this.renderOverride = function(element, x, y, size)
		{
		};

		this.onAddedToLayer = function(markerLayer)
		{
			this.setInternal("markerLayer", markerLayer);
		};

		this.onRemovedFromLayer = function(markerLayer)
		{
			this.setInternal("markerLayer", null);
		};

	});

});
});

jg_import.define("splunk.vectors.Circle", function()
{
jg_namespace("splunk.vectors", function()
{

	var Shape = jg_import("splunk.vectors.Shape");
	var VectorElement = jg_import("splunk.vectors.VectorElement");

	this.Circle = jg_extend(Shape, function(Circle, base)
	{

		// Constructor

		this.constructor = function(x, y, radius)
		{
			base.constructor.call(this);
		};

		// Public Methods

		this.x = function(value)
		{
			return this;
		};

		this.y = function(value)
		{
			return this;
		};

		this.radius = function(value)
		{
			return this;
		};

		// Inner Mixin Classes

		var SVGCircle = jg_static(function(SVGCircle)
		{

			// Constructor

			this.constructor = function(x, y, radius)
			{
				base.constructor.call(this, "circle");

				if (x != null)
					this.x(x);
				if (y != null)
					this.y(y);
				if (radius != null)
					this.radius(radius);
			};

			// Public Methods

			this.x = function(value)
			{
				if ((value != null) && (value > -Infinity) && (value < Infinity))
					this.element.setAttribute("cx", value);
				else
					this.element.removeAttribute("cx");

				return this;
			};

			this.y = function(value)
			{
				if ((value != null) && (value > -Infinity) && (value < Infinity))
					this.element.setAttribute("cy", value);
				else
					this.element.removeAttribute("cy");

				return this;
			};

			this.radius = function(value)
			{
				if ((value != null) && (value < Infinity))
					this.element.setAttribute("r", Math.max(value, 0));
				else
					this.element.removeAttribute("r");

				return this;
			};

		});

		var VMLCircle = jg_static(function(VMLCircle)
		{

			// Constructor

			this.constructor = function(x, y, radius)
			{
				base.constructor.call(this, "oval");

				if (x != null)
					this.x(x);
				if (y != null)
					this.y(y);
				if (radius != null)
					this.radius(radius);
			};

			// Public Methods

			this.x = function(value)
			{
				if ((value != null) && (value > -Infinity) && (value < Infinity))
					this.element.style.left = Math.round(value) + "px";
				else
					this.element.style.left = "";

				return this;
			};

			this.y = function(value)
			{
				if ((value != null) && (value > -Infinity) && (value < Infinity))
					this.element.style.top = Math.round(value) + "px";
				else
					this.element.style.top = "";

				return this;
			};

			this.radius = function(value)
			{
				var style = this.element.style;
				if ((value != null) && (value < Infinity))
				{
					value = Math.round(Math.max(value, 0));
					style.width = value * 2 + "px";
					style.height = value * 2 + "px";
					style.marginLeft = -value + "px";
					style.marginTop = -value + "px";
				}
				else
				{
					style.width = "";
					style.height = "";
					style.marginLeft = "";
					style.marginTop = "";
				}

				return this;
			};

		});

		VectorElement.mixin(this, SVGCircle, VMLCircle);

	});

});
});

jg_import.define("splunk.mapping2.layers.CircleMarker", function()
{
jg_namespace("splunk.mapping2.layers", function()
{

	var ObservableProperty = jg_import("jgatt.properties.ObservableProperty");
	var MarkerBase = jg_import("splunk.mapping2.layers.MarkerBase");
	var Circle = jg_import("splunk.vectors.Circle");

	this.CircleMarker = jg_extend(MarkerBase, function(CircleMarker, base)
	{

		// Public Properties

		this.color = new ObservableProperty("color", Number, 0x000000)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value <= Infinity)) ? Math.min(Math.floor(value), 0xFFFFFF) : 0x000000;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		this.opacity = new ObservableProperty("opacity", Number, 1)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value <= Infinity)) ? Math.min(value, 1) : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		// Constructor

		this.constructor = function(latLon, size, color, opacity)
		{
			base.constructor.call(this, new Circle(), latLon, size);

			if (color != null)
				this.set("color", color);
			if (opacity != null)
				this.set("opacity", opacity);
		};

		// Protected Methods

		this.renderOverride = function(element, x, y, size)
		{
			element.x(x);
			element.y(y);
			element.radius(size / 2);
			element.fillColor(this.getInternal("color"));
			element.fillOpacity(this.getInternal("opacity"));
		};

	});

});
});

jg_import.define("splunk.mapping2.layers.MarkerLayer", function()
{
jg_namespace("splunk.mapping2.layers", function()
{

	var Point = jg_import("jgatt.geom.Point");
	var Rectangle = jg_import("jgatt.geom.Rectangle");
	var ObservableProperty = jg_import("jgatt.properties.ObservableProperty");
	var Dictionary = jg_import("jgatt.utils.Dictionary");
	var FunctionUtils = jg_import("jgatt.utils.FunctionUtils");
	var LatLonBounds = jg_import("splunk.mapping2.LatLonBounds");
	var LayerBase = jg_import("splunk.mapping2.layers.LayerBase");
	var MarkerBase = jg_import("splunk.mapping2.layers.MarkerBase");
	var Viewport = jg_import("splunk.vectors.Viewport");

	this.MarkerLayer = jg_extend(LayerBase, function(MarkerLayer, base)
	{

		// Public Properties

		this.wrapX = new ObservableProperty("wrapX", Boolean, true)
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		this.wrapY = new ObservableProperty("wrapY", Boolean, false)
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		// Private Properties

		this._markerMap = null;
		this._markerViewport = null;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.addStyleClass("splunk-mapping2-layers-MarkerLayer");

			this._marker_latLon_changed = FunctionUtils.bind(this._marker_latLon_changed, this);

			this._markerMap = new Dictionary();
			this._markerViewport = new Viewport()
				.appendTo(this.element);
		};

		// Public Methods

		this.addMarker = function(marker)
		{
			if (marker == null)
				throw new Error("Parameter marker must be non-null.");
			if (!(marker instanceof MarkerBase))
				throw new Error("Parameter marker must be an instance of splunk.mapping2.layers.MarkerBase.");

			var markerLayer = marker.get("markerLayer");
			if (markerLayer && (markerLayer !== this))
				markerLayer.removeMarker(marker);

			this._markerMap.set(marker, marker);
			marker.element.appendTo(this._markerViewport);
			if (markerLayer !== this)
			{
				marker.onAddedToLayer(this);
				marker.addEventListener("latLon.changed", this._marker_latLon_changed);

				this.invalidate("renderPass");
			}
		};

		this.removeMarker = function(marker)
		{
			if (marker == null)
				throw new Error("Parameter marker must be non-null.");
			if (!(marker instanceof MarkerBase))
				throw new Error("Parameter marker must be an instance of splunk.mapping2.layers.MarkerBase.");

			if (!this._markerMap.has(marker))
				return;

			marker.removeEventListener("latLon.changed", this._marker_latLon_changed);
			marker.onRemovedFromLayer(this);
			marker.element.remove();
			this._markerMap.del(marker);
		};

		this.getLatLonBounds = function(center)
		{
			if ((center != null) && !(center instanceof LatLon))
				throw new Error("Parameter center must be an instance of splunk.mapping2.LatLon.");

			this.validate();

			var bounds = new LatLonBounds(Infinity, Infinity, -Infinity, -Infinity);

			var markerList = this._markerMap.values();
			var marker;
			var latLon;
			for (var i = 0, l = markerList.length; i < l; i++)
			{
				marker = markerList[i];
				latLon = marker.getInternal("latLon");
				bounds.expand(latLon.normalize(center));
			}

			return bounds.isFinite() ? bounds : null;
		};

		this.toSVGString = function()
		{
			this.validate();

			return this._markerViewport.toSVGString();
		};

		// Protected Methods

		this.renderOverride = function(map)
		{
			var width = map.get("width");
			var height = map.get("height");
			var center = map.get("center");
			var centerRelative = map.latLonToRelative(center);

			var wrapX = this.getInternal("wrapX");
			var wrapY = this.getInternal("wrapY");

			// get map bounds in pixels relative to viewport
			var mapTL = map.relativeToViewport(new Point(0, 0));
			var mapBR = map.relativeToViewport(new Point(1, 1));
			var mapLeft = mapTL.x;
			var mapTop = mapTL.y;
			var mapWidth = mapBR.x - mapLeft;
			var mapHeight = mapBR.y - mapTop;

			var markerList = this._markerMap.values();
			var marker;
			var markerPosition;
			for (var i = 0, l = markerList.length; i < l; i++)
			{
				marker = markerList[i];
				markerPosition = map.latLonToRelative(marker.getInternal("latLon"));

				if (wrapX)
				{
					markerPosition.x -= centerRelative.x;
					markerPosition.x %= 1;
					if (markerPosition.x > 0.5)
						markerPosition.x -= 1;
					else if (markerPosition.x < -0.5)
						markerPosition.x += 1;
					markerPosition.x += centerRelative.x;
				}

				if (wrapY)
				{
					markerPosition.y -= centerRelative.y;
					markerPosition.y %= 1;
					if (markerPosition.y > 0.5)
						markerPosition.y -= 1;
					else if (markerPosition.y < -0.5)
						markerPosition.y += 1;
					markerPosition.y += centerRelative.y;
				}

				markerPosition.x = Math.round(mapLeft + mapWidth * markerPosition.x);
				markerPosition.y = Math.round(mapTop + mapHeight * markerPosition.y);

				marker.render(markerPosition);
			}

			var viewport = this._markerViewport;
			viewport.width(width);
			viewport.height(height);
			viewport.viewBox(new Rectangle(0, 0, width, height));
		};

		// Private Methods

		this._marker_latLon_changed = function(e)
		{
			this.invalidate("renderPass");
		};

	});

});
});

jg_import.define("splunk.charting.Legend", function()
{
jg_namespace("splunk.charting", function()
{

	var ChainedEvent = jg_import("jgatt.events.ChainedEvent");
	var Event = jg_import("jgatt.events.Event");
	var EventData = jg_import("jgatt.events.EventData");
	var MObservable = jg_import("jgatt.events.MObservable");
	var MPropertyTarget = jg_import("jgatt.properties.MPropertyTarget");
	var ObservableProperty = jg_import("jgatt.properties.ObservableProperty");
	var ArrayUtils = jg_import("jgatt.utils.ArrayUtils");
	var Dictionary = jg_import("jgatt.utils.Dictionary");

	this.Legend = jg_extend(Object, function(Legend, base)
	{

		base = jg_mixin(this, MObservable, base);
		base = jg_mixin(this, MPropertyTarget, base);

		// Public Events

		this.settingLabels = new Event("settingLabels", EventData);
		this.labelIndexMapChanged = new ChainedEvent("labelIndexMapChanged", this.changed);

		// Public Properties

		this.labels = new ObservableProperty("labels", Array, [])
			.readFilter(function(value)
			{
				return value.concat();
			})
			.writeFilter(function(value)
			{
				return value ? value.concat() : [];
			})
			.onChanged(function(e)
			{
				this._updateLabelMap();
			});

		// Private Properties

		this._targetMap = null;
		this._targetList = null;
		this._labelMap = null;
		this._labelList = null;
		this._isSettingLabels = false;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this._targetMap = new Dictionary();
			this._targetList = [];
			this._labelMap = {};
			this._labelList = [];
		};

		// Public Methods

		this.register = function(target)
		{
			if (target == null)
				throw new Error("Parameter target must be non-null.");

			var targetData = this._targetMap.get(target);
			if (targetData)
				return;

			targetData = { labels: null };
			this._targetMap.set(target, targetData);
			this._targetList.push(targetData);
		};

		this.unregister = function(target)
		{
			if (target == null)
				throw new Error("Parameter target must be non-null.");

			var targetData = this._targetMap.get(target);
			if (!targetData)
				return;

			var targetIndex = ArrayUtils.indexOf(this._targetList, targetData);
			if (targetIndex >= 0)
				this._targetList.splice(targetIndex, 1);
			this._targetMap.del(target);

			this._updateLabelMap();
		};

		this.setLabels = function(target, labels)
		{
			if (target == null)
				throw new Error("Parameter target must be non-null.");
			if ((labels != null) && !(labels instanceof Array))
				throw new Error("Parameter labels must be an array.");

			var targetData = this._targetMap.get(target);
			if (!targetData)
				return;

			targetData.labels = labels ? labels.concat() : null;

			this.notifySettingLabels();
		};

		this.getLabelIndex = function(label)
		{
			if (label == null)
				throw new Error("Parameter label must be non-null.");
			if (typeof label !== "string")
				throw new Error("Parameter label must be a string.");

			var index = this.getLabelIndexOverride(label);
			if (index < 0)
			{
				var labelIndex = this._labelMap[label];
				index = (labelIndex != null) ? labelIndex : -1;
			}
			return index;
		};

		this.getNumLabels = function()
		{
			var value = this.getNumLabelsOverride();
			if (value < 0)
				value = this._labelList.length;
			return value;
		};

		this.notifySettingLabels = function()
		{
			if (this._isSettingLabels)
				return;

			try
			{
				this._isSettingLabels = true;
				this.dispatchEvent(this.settingLabels, new EventData());
				this._updateLabelMap();
			}
			finally
			{
				this._isSettingLabels = false;
			}
		};

		this.notifyLabelIndexMapChanged = function()
		{
			this.dispatchEvent(this.labelIndexMapChanged, new EventData());
		};

		// Protected Methods

		this.getNumLabelsOverride = function()
		{
			return -1;
		};

		this.getLabelIndexOverride = function(label)
		{
			return -1;
		};

		this.updateLabelsOverride = function(labels)
		{
			return false;
		};

		// Private Methods

		this._updateLabelMap = function()
		{
			var currentLabelList = this._labelList;
			var changed = false;

			var labelMap = {};
			var labelList = [];

			var targetList = this._targetList;
			var targetData;
			var targetLabels;
			var targetLabel;

			var i;
			var j;
			var l;
			var m;

			targetLabels = this.getInternal(this.labels);
			for (i = 0, l = targetLabels.length; i < l; i++)
			{
				targetLabel = String(targetLabels[i]);
				if (labelMap[targetLabel] == null)
				{
					labelMap[targetLabel] = labelList.length;
					labelList.push(targetLabel);
				}
			}

			for (i = 0, l = targetList.length; i < l; i++)
			{
				targetData = targetList[i];
				targetLabels = targetData.labels;
				if (targetLabels)
				{
					for (j = 0, m = targetLabels.length; j < m; j++)
					{
						targetLabel = String(targetLabels[j]);
						if (labelMap[targetLabel] == null)
						{
							labelMap[targetLabel] = labelList.length;
							labelList.push(targetLabel);
						}
					}
				}
			}

			if (labelList.length != currentLabelList.length)
			{
				changed = true;
			}
			else
			{
				for (i = 0, l = labelList.length; i < l; i++)
				{
					if (labelList[i] !== currentLabelList[i])
					{
						changed = true;
						break;
					}
				}
			}

			if (changed)
			{
				this._labelMap = labelMap;
				this._labelList = labelList;

				if (!this.updateLabelsOverride(labelList.concat()))
					this.notifyLabelIndexMapChanged();
			}
		};

	});

});
});

jg_import.define("splunk.viz.MDataTarget", function()
{
jg_namespace("splunk.viz", function()
{

	var MObservable = jg_import("jgatt.events.MObservable");
	var MPropertyTarget = jg_import("jgatt.properties.MPropertyTarget");
	var ObservableArrayProperty = jg_import("jgatt.properties.ObservableArrayProperty");
	var ObservableProperty = jg_import("jgatt.properties.ObservableProperty");
	var FunctionUtils = jg_import("jgatt.utils.FunctionUtils");
	var MValidateTarget = jg_import("jgatt.validation.MValidateTarget");
	var ValidatePass = jg_import("jgatt.validation.ValidatePass");
	var Legend = jg_import("splunk.charting.Legend");

	this.MDataTarget = jg_static(function(MDataTarget)
	{

		// Mixin

		this.mixin = function(base)
		{
			base = jg_mixin(this, MObservable, base);
			base = jg_mixin(this, MPropertyTarget, base);
			base = jg_mixin(this, MValidateTarget, base);
		};

		// Public Passes

		this.processDataPass = new ValidatePass("processData", 0.1);
		this.updateLegendLabelsPass = new ValidatePass("updateLegendLabels", 0.2);
		this.renderDataPass = new ValidatePass("renderData", 0.3);

		// Public Properties

		this.data = new ObservableProperty("data", Array, null)
			.onChanged(function(e)
			{
				this.invalidate("processDataPass");
			});

		this.fields = new ObservableArrayProperty("fields", String, null)
			.onChanged(function(e)
			{
				this.invalidate("processDataPass");
			});

		this.legend = new ObservableProperty("legend", Legend, null)
			.onChanged(function(e)
			{
				if (e.target === this)
				{
					var oldLegend = e.oldValue;
					var newLegend = e.newValue;

					if (oldLegend)
					{
						oldLegend.removeEventListener("settingLabels", this._legend_settingLabels);
						oldLegend.unregister(this);
					}

					if (newLegend)
					{
						newLegend.register(this);
						newLegend.addEventListener("settingLabels", this._legend_settingLabels);
					}

					this.invalidate("updateLegendLabelsPass");
					return;
				}

				if (e.event === e.target.labelIndexMapChanged)
				{
					this.invalidate("renderDataPass");
					return;
				}
			});

		// Private Properties

		this._cachedData = null;
		this._cachedFields = null;
		this._cachedLegend = null;

		// Constructor

		this.constructor = function()
		{
			this._legend_settingLabels = FunctionUtils.bind(this._legend_settingLabels, this);
		};

		// Public Methods

		this.processData = function()
		{
			this.validatePreceding("processDataPass");

			if (this.isValid("processDataPass"))
				return;

			this.invalidate("updateLegendLabelsPass");

			var data = this._cachedData = this.getInternal("data") || [];
			var fields = this._cachedFields = this.getInternal("fields") || [];

			this.processDataOverride(data, fields);

			this.setValid("processDataPass");
		};

		this.updateLegendLabels = function()
		{
			this.validatePreceding("updateLegendLabelsPass");

			if (this.isValid("updateLegendLabelsPass"))
				return;

			this.invalidate("renderDataPass");

			var legend = this._cachedLegend = this.getInternal("legend");
			var labels = null;

			if (legend)
				labels = this.updateLegendLabelsOverride(this._cachedData, this._cachedFields);

			this.setValid("updateLegendLabelsPass");

			// this must run last to avoid recursion
			if (legend)
				legend.setLabels(this, labels);
		};

		this.renderData = function()
		{
			this.validatePreceding("renderDataPass");

			if (this.isValid("renderDataPass"))
				return;

			this.renderDataOverride(this._cachedData, this._cachedFields, this._cachedLegend);

			this.setValid("renderDataPass");
		};

		// Protected Methods

		this.processDataOverride = function(data, fields)
		{
		};

		this.updateLegendLabelsOverride = function(data, fields)
		{
			return null;
		};

		this.renderDataOverride = function(data, fields, legend)
		{
		};

		// Private Methods

		this._legend_settingLabels = function(e)
		{
			this.validate("updateLegendLabelsPass");
		};

	});

});
});

jg_import.define("splunk.mapping2.layers.CircleMarkerLayer", function()
{
jg_namespace("splunk.mapping2.layers", function()
{

	var ObservableProperty = jg_import("jgatt.properties.ObservableProperty");
	var NumberUtils = jg_import("jgatt.utils.NumberUtils");
	var LatLon = jg_import("splunk.mapping2.LatLon");
	var CircleMarker = jg_import("splunk.mapping2.layers.CircleMarker");
	var MarkerLayer = jg_import("splunk.mapping2.layers.MarkerLayer");
	var ColorPalette = jg_import("splunk.palettes.ColorPalette");
	var ListColorPalette = jg_import("splunk.palettes.ListColorPalette");
	var MDataTarget = jg_import("splunk.viz.MDataTarget");

	this.CircleMarkerLayer = jg_extend(MarkerLayer, function(CircleMarkerLayer, base)
	{

		base = jg_mixin(this, MDataTarget, base);

		// Public Properties

		this.markerColorPalette = new ObservableProperty("markerColorPalette", ColorPalette, new ListColorPalette([ 0x00CC00, 0xCCCC00, 0xCC0000 ], true))
			.onChanged(function(e)
			{
				this.invalidate("renderDataPass");
			});

		this.markerOpacity = new ObservableProperty("markerOpacity", Number, 1)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value <= Infinity)) ? Math.min(value, 1) : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderDataPass");
			});

		this.markerMinSize = new ObservableProperty("markerMinSize", Number, 10)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value < Infinity)) ? value : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderDataPass");
			});

		this.markerMaxSize = new ObservableProperty("markerMaxSize", Number, 50)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value < Infinity)) ? value : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderDataPass");
			});

		this.defaultSeriesName = new ObservableProperty("defaultSeriesName", String, "marker")
			.onChanged(function(e)
			{
				this.invalidate("processDataPass");
			});

		// Private Properties

		this._seriesList = null;
		this._markerList = null;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.addStyleClass("splunk-mapping2-layers-CircleMarkerLayer");

			this._seriesList = [];
			this._markerList = [];
		};

		// Protected Methods

		this.processDataOverride = function(data, fields)
		{
			var seriesList = this._seriesList = [];

			var markerList = this._markerList;
			var numMarkers = 0;
			var marker;
			var i;

			var numRows = data.length;
			var numFields = fields.length;
			if ((numRows > 0) && (numFields > 1))
			{
				var fieldLat = fields[0];
				var fieldLon = fields[1];
				var fieldMag = (numFields > 2) ? fields[2] : null;
				var fieldSeries = (numFields > 3) ? fields[3] : null;

				var valueLat;
				var valueLon;
				var valueMag;
				var valueSeries;

				var magMin = Infinity;
				var magMax = -Infinity;
				var magSpan = 0;

				var obj;
				var scale;
				var latLon;

				var defaultSeriesName = this.getInternal("defaultSeriesName");
				var seriesName;
				var seriesMap = {};
				var series;

				// modify fields list to include only plotted fields
				fields = fields.slice(0, Math.min(numFields, 4));

				// compute magnitude range
				if (numFields > 2)
				{
					for (i = 0; i < numRows; i++)
					{
						obj = data[i];
						if (obj == null)
							continue;

						valueMag = NumberUtils.parseNumber(obj[fieldMag]);
						if (isNaN(valueMag))
							continue;

						if (valueMag < magMin)
							magMin = valueMag;
						if (valueMag > magMax)
							magMax = valueMag;
					}
				}
				magMin = ((magMin > -Infinity) && (magMin < Infinity)) ? magMin : 0;
				magMax = ((magMax > -Infinity) && (magMax < Infinity)) ? magMax : 0;
				magSpan = magMax - magMin;

				// interpret data
				for (i = 0; i < numRows; i++)
				{
					obj = data[i];
					if (obj == null)
						continue;

					valueLat = NumberUtils.parseNumber(obj[fieldLat]);
					valueLon = NumberUtils.parseNumber(obj[fieldLon]);
					valueMag = (numFields > 2) ? NumberUtils.parseNumber(obj[fieldMag]) : NaN;
					valueSeries = (numFields > 3) ? obj[fieldSeries] : defaultSeriesName;

					if (isNaN(valueLat) || isNaN(valueLon) || (valueSeries == null))
						continue;

					seriesName = String(valueSeries);
					if (!seriesName)
						continue;

					if (isNaN(valueMag))
						valueMag = magMin;

					latLon = new LatLon(valueLat, valueLon);
					scale = (magSpan > 0) ? NumberUtils.minMax((valueMag - magMin) / magSpan, 0, 1) : 0;

					// create or reuse series
					series = seriesMap[seriesName];
					if (!series)
					{
						series = seriesMap[seriesName] = {};
						series.seriesName = seriesName;
						series.markerList = [];
						seriesList.push(series);
					}

					// create or reuse marker
					if (numMarkers < markerList.length)
					{
						marker = markerList[numMarkers];
					}
					else
					{
						marker = new CircleMarker();
						markerList.push(marker);
						this.addMarker(marker);
					}

					series.markerList.push(marker);

					marker.set("latLon", latLon);
					marker.metadata.seriesName = seriesName;
					marker.metadata.data = obj;
					marker.metadata.fields = fields;
					marker.metadata.scale = scale;

					numMarkers++;
				}
			}

			// remove unused markers
			for (i = markerList.length - 1; i >= numMarkers; i--)
			{
				marker = markerList.pop();
				this.removeMarker(marker);
			}
		};

		this.updateLegendLabelsOverride = function(data, fields)
		{
			var seriesList = this._seriesList;
			var numSeries = seriesList.length;
			var labels = (numSeries > 0) ? new Array(numSeries) : null;
			for (var i = 0; i < numSeries; i++)
				labels[i] = seriesList[i].seriesName;
			return labels;
		};

		this.renderDataOverride = function(data, fields, legend)
		{
			var seriesList = this._seriesList;
			var numSeries = seriesList.length;
			var series;
			var seriesIndex;
			var seriesCount;

			var markerColorPalette = this.getInternal("markerColorPalette");
			var markerOpacity = this.getInternal("markerOpacity");
			var markerMinSize = this.getInternal("markerMinSize");
			var markerMaxSize = this.getInternal("markerMaxSize");
			var markerList;
			var numMarkers;
			var marker;
			var markerColor;
			var markerSize;

			var i;
			var j;

			seriesCount = legend ? legend.getNumLabels() : numSeries;

			for (i = 0; i < numSeries; i++)
			{
				series = seriesList[i];
				markerList = series.markerList;
				numMarkers = markerList.length;

				seriesIndex = legend ? legend.getLabelIndex(series.seriesName) : i;

				markerColor = markerColorPalette ? markerColorPalette.getColor(series.seriesName, seriesIndex, seriesCount) : 0x000000;

				for (j = 0; j < numMarkers; j++)
				{
					marker = markerList[j];

					markerSize = Math.round(NumberUtils.interpolate(markerMinSize, markerMaxSize, marker.metadata.scale));

					marker.set("size", markerSize);
					marker.set("color", markerColor);
					marker.set("opacity", markerOpacity);
				}
			}
		};

	});

});
});

jg_import.define("splunk.mapping2.layers.IMGTileLayer", function()
{
jg_namespace("splunk.mapping2.layers", function()
{

	var TileLayerBase = jg_import("splunk.mapping2.layers.TileLayerBase");

	this.IMGTileLayer = jg_extend(TileLayerBase, function(IMGTileLayer, base)
	{

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.addStyleClass("splunk-mapping2-layers-IMGTileLayer");
		};

		// Protected Methods

		this.createTileContainer = function()
		{
			var container = document.createElement("div");
			container.style.position = "absolute";
			container.style.left = "0px";
			container.style.top = "0px";

			this.element.appendChild(container);

			return container;
		};

		this.destroyTileContainer = function(container)
		{
			this.element.removeChild(container);
		};

		this.displayTileContainer = function(container, display)
		{
			if (display)
			{
				container.style.display = "";
				this.element.appendChild(container);
			}
			else
			{
				container.style.display = "none";
			}
		};

		this.createTile = function(container, url, onLoad, onError)
		{
			var tile = document.createElement("img");
			tile.style.position = "absolute";
			tile.style.margin = "0px";
			tile.style.padding = "0px";
			tile.style.border = "0px none";
			tile.style.visibility = "hidden";
			tile.style.opacity = "0";
			tile.style.transition = "opacity 0.2s linear";
			tile.style.OTransition = "opacity 0.2s linear";
			tile.style.MozTransition = "opacity 0.2s linear";
			tile.style.WebkitTransition = "opacity 0.2s linear";

			tile.onload = function()
			{
				tile.style.visibility = "";
				tile.style.opacity = "1";

				tile.onload.transitionTimeout = setTimeout(onLoad, 200);
			};

			tile.onerror = function()
			{
				onError();
			};

			tile.src = url;

			container.appendChild(tile);

			return tile;
		};

		this.destroyTile = function(container, tile)
		{
			clearTimeout(tile.onload.transitionTimeout);

			tile.onload = null;
			tile.onerror = null;

			container.removeChild(tile);

			tile.src = null;
		};

		this.positionTile = function(tile, x, y, width, height)
		{
			tile.style.left = x + "px";
			tile.style.top = y + "px";
			tile.style.width = width + "px";
			tile.style.height = height + "px";
		};

	});

});
});

jg_import.define("splunk.vectors.Group", function()
{
jg_namespace("splunk.vectors", function()
{

	var VectorElement = jg_import("splunk.vectors.VectorElement");

	this.Group = jg_extend(VectorElement, function(Group, base)
	{

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);
		};

		// Inner Mixin Classes

		var SVGGroup = jg_static(function(SVGGroup)
		{

			// Constructor

			this.constructor = function()
			{
				base.constructor.call(this, "g");
			};

		});

		var VMLGroup = jg_static(function(VMLGroup)
		{

			// Constructor

			this.constructor = function()
			{
				base.constructor.call(this, "group");

				this.element.style.width = "1px";
				this.element.style.height = "1px";
				this.element.coordsize = "1,1";
			};

		});

		VectorElement.mixin(this, SVGGroup, VMLGroup);

	});

});
});

jg_import.define("splunk.vectors.Wedge", function()
{
jg_namespace("splunk.vectors", function()
{

	var NumberUtils = jg_import("jgatt.utils.NumberUtils");
	var Shape = jg_import("splunk.vectors.Shape");
	var VectorElement = jg_import("splunk.vectors.VectorElement");

	this.Wedge = jg_extend(Shape, function(Wedge, base)
	{

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);
		};

		// Public Methods

		this.draw = function(x, y, radiusX, radiusY, startAngle, arcAngle)
		{
			return this;
		};

		// Inner Mixin Classes

		var SVGWedge = jg_static(function(SVGWedge)
		{

			// Constructor

			this.constructor = function()
			{
				base.constructor.call(this, "path");
			};

			// Public Methods

			this.draw = function(x, y, radiusX, radiusY, startAngle, arcAngle)
			{
				x = ((x != null) && (x > -Infinity) && (x < Infinity)) ? x : 0;
				y = ((y != null) && (y > -Infinity) && (y < Infinity)) ? y : 0;
				radiusX = ((radiusX != null) && (radiusX < Infinity)) ? Math.max(radiusX, 0) : 0;
				radiusY = ((radiusY != null) && (radiusY < Infinity)) ? Math.max(radiusY, 0) : 0;
				startAngle = ((startAngle != null) && (startAngle > -Infinity) && (startAngle < Infinity)) ? startAngle : 0;
				arcAngle = ((arcAngle != null) && (arcAngle != null) && !isNaN(arcAngle)) ? NumberUtils.minMax(arcAngle, -360, 360) : 0;

				if ((radiusX == 0) || (radiusY == 0) || (arcAngle == 0))
				{
					this.element.removeAttribute("d");
					return this;
				}

				var a1 = (startAngle / 180) * Math.PI;
				var x1 = x + Math.cos(a1) * radiusX;
				var y1 = y + Math.sin(a1) * radiusY;
				var a2 = ((startAngle + arcAngle / 2) / 180) * Math.PI;
				var x2 = x + Math.cos(a2) * radiusX;
				var y2 = y + Math.sin(a2) * radiusY;
				var a3 = ((startAngle + arcAngle) / 180) * Math.PI;
				var x3 = x + Math.cos(a3) * radiusX;
				var y3 = y + Math.sin(a3) * radiusY;

				var sweepFlag = (arcAngle < 0) ? 0 : 1;

				var pathData = "";
				if ((arcAngle > -360) && (arcAngle < 360))
				{
					pathData += "M" + x + "," + y;
					pathData += " L" + x1 + "," + y1;
				}
				else
				{
					pathData += "M" + x1 + "," + y1;
				}
				pathData += " A" + radiusX + "," + radiusY + " 0 0 " + sweepFlag + " " + x2 + "," + y2;
				pathData += " " + radiusX + "," + radiusY + " 0 0 " + sweepFlag + " " + x3 + "," + y3;
				pathData += " Z";

				this.element.setAttribute("d", pathData);

				return this;
			};

		});

		var VMLWedge = jg_static(function(VMLWedge)
		{

			// Private Static Constants

			var _RES = 64;

			// Private Properties

			this._pathElement = null;

			// Constructor

			this.constructor = function()
			{
				base.constructor.call(this, "shape");

				this._pathElement = this.createElement("path");

				this.element.style.width = "1px";
				this.element.style.height = "1px";
				this.element.coordsize = _RES + "," + _RES;
				this.element.appendChild(this._pathElement);
			};

			// Public Methods

			this.dispose = function()
			{
				base.dispose.call(this);

				this._pathElement = null;
			};

			this.draw = function(x, y, radiusX, radiusY, startAngle, arcAngle)
			{
				x = ((x != null) && (x > -Infinity) && (x < Infinity)) ? x : 0;
				y = ((y != null) && (y > -Infinity) && (y < Infinity)) ? y : 0;
				radiusX = ((radiusX != null) && (radiusX < Infinity)) ? Math.max(radiusX, 0) : 0;
				radiusY = ((radiusY != null) && (radiusY < Infinity)) ? Math.max(radiusY, 0) : 0;
				startAngle = ((startAngle != null) && (startAngle > -Infinity) && (startAngle < Infinity)) ? startAngle : 0;
				arcAngle = ((arcAngle != null) && (arcAngle != null) && !isNaN(arcAngle)) ? NumberUtils.minMax(arcAngle, -360, 360) : 0;

				if ((radiusX == 0) || (radiusY == 0) || (arcAngle == 0))
				{
					this._pathElement.v = " ";
					return this;
				}

				var a1 = (startAngle / 180) * Math.PI;
				var x1 = x + Math.cos(a1) * radiusX;
				var y1 = y + Math.sin(a1) * radiusY;
				var a2 = ((startAngle + arcAngle / 2) / 180) * Math.PI;
				var x2 = x + Math.cos(a2) * radiusX;
				var y2 = y + Math.sin(a2) * radiusY;
				var a3 = ((startAngle + arcAngle) / 180) * Math.PI;
				var x3 = x + Math.cos(a3) * radiusX;
				var y3 = y + Math.sin(a3) * radiusY;

				var left = Math.round((x - radiusX) * _RES);
				var top = Math.round((y - radiusY) * _RES);
				var right = Math.round((x + radiusX) * _RES);
				var bottom = Math.round((y + radiusY) * _RES);

				x = Math.round(x * _RES);
				y = Math.round(y * _RES);
				x1 = Math.round(x1 * _RES);
				y1 = Math.round(y1 * _RES);
				x2 = Math.round(x2 * _RES);
				y2 = Math.round(y2 * _RES);
				x3 = Math.round(x3 * _RES);
				y3 = Math.round(y3 * _RES);

				var pathData = "";
				if ((arcAngle > -360) && (arcAngle < 360))
				{
					pathData += "m " + x + "," + y;
					pathData += " l " + x1 + "," + y1;
				}
				else
				{
					pathData += "m " + x1 + "," + y1;
				}
				pathData += (arcAngle < 0) ? " at" : " wa";
				pathData += " " + left + "," + top + "," + right + "," + bottom + ", " + x1 + "," + y1 + ", " + x2 + "," + y2;
				pathData += ", " + left + "," + top + "," + right + "," + bottom + ", " + x2 + "," + y2 + ", " + x3 + "," + y3;
				pathData += " x";

				this._pathElement.v = pathData;

				return this;
			};

		});

		VectorElement.mixin(this, SVGWedge, VMLWedge);

	});

});
});

jg_import.define("splunk.mapping2.layers.PieMarkerLayer", function()
{
jg_namespace("splunk.mapping2.layers", function()
{

	var Point = jg_import("jgatt.geom.Point");
	var Rectangle = jg_import("jgatt.geom.Rectangle");
	var ObservableProperty = jg_import("jgatt.properties.ObservableProperty");
	var NumberUtils = jg_import("jgatt.utils.NumberUtils");
	var LatLon = jg_import("splunk.mapping2.LatLon");
	var LatLonBounds = jg_import("splunk.mapping2.LatLonBounds");
	var LayerBase = jg_import("splunk.mapping2.layers.LayerBase");
	var ColorPalette = jg_import("splunk.palettes.ColorPalette");
	var ListColorPalette = jg_import("splunk.palettes.ListColorPalette");
	var Group = jg_import("splunk.vectors.Group");
	var Viewport = jg_import("splunk.vectors.Viewport");
	var Wedge = jg_import("splunk.vectors.Wedge");
	var MDataTarget = jg_import("splunk.viz.MDataTarget");

	this.PieMarkerLayer = jg_extend(LayerBase, function(PieMarkerLayer, base)
	{

		base = jg_mixin(this, MDataTarget, base);

		// Public Properties

		this.markerColorPalette = new ObservableProperty("markerColorPalette", ColorPalette, new ListColorPalette([ 0x00CC00, 0xCCCC00, 0xCC0000 ], true))
			.onChanged(function(e)
			{
				this.invalidate("renderDataPass");
			});

		this.markerOpacity = new ObservableProperty("markerOpacity", Number, 1)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value <= Infinity)) ? Math.min(value, 1) : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderDataPass");
			});

		this.markerMinSize = new ObservableProperty("markerMinSize", Number, 10)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value < Infinity)) ? value : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderDataPass");
			});

		this.markerMaxSize = new ObservableProperty("markerMaxSize", Number, 50)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value < Infinity)) ? value : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderDataPass");
			});

		this.wrapX = new ObservableProperty("wrapX", Boolean, true)
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		this.wrapY = new ObservableProperty("wrapY", Boolean, false)
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		// Private Properties

		this._seriesList = null;
		this._markerList = null;
		this._markerViewport = null;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.addStyleClass("splunk-mapping2-layers-PieMarkerLayer");

			this._seriesList = [];
			this._markerList = [];
			this._markerViewport = new Viewport()
				.appendTo(this.element);
		};

		// Public Methods

		this.getLatLonBounds = function(center)
		{
			if ((center != null) && !(center instanceof LatLon))
				throw new Error("Parameter center must be an instance of splunk.mapping2.LatLon.");

			this.validate();

			var bounds = new LatLonBounds(Infinity, Infinity, -Infinity, -Infinity);

			var markerList = this._markerList;
			for (var i = 0, l = markerList.length; i < l; i++)
				bounds.expand(markerList[i].latLon.normalize(center));

			return bounds.isFinite() ? bounds : null;
		};

		this.toSVGString = function()
		{
			this.validate();

			return this._markerViewport.toSVGString();
		};

		// Protected Methods

		this.processDataOverride = function(data, fields)
		{
			var seriesList = this._seriesList;
			var numSeries = 0;
			var series;

			var markerList = this._markerList;
			var numMarkers = 0;
			var marker;

			var sliceList;
			var numSlices;
			var slice;

			var i;
			var j;

			var numRows = data.length;
			var numFields = fields.length;
			if ((numRows > 0) && (numFields > 2))
			{
				var viewport = this._markerViewport;

				var fieldLat = fields[0];
				var fieldLon = fields[1];
				var fieldSeries;

				var valueLat;
				var valueLon;
				var valueSeries;

				var magMin = Infinity;
				var magMax = -Infinity;
				var magSpan = 0;
				var mag;

				var sum;
				var angle1;
				var angle2;

				// create or reuse series
				for (i = 2; i < numFields; i++)
				{
					fieldSeries = fields[i];
					if (numSeries < seriesList.length)
					{
						series = seriesList[numSeries];
					}
					else
					{
						series = new Series();
						seriesList.push(series);
					}

					series.name = fieldSeries;

					numSeries++;
				}

				// create or reuse markers
				for (i = 0; i < numRows; i++)
				{
					obj = data[i];
					if (obj == null)
						continue;

					valueLat = NumberUtils.parseNumber(obj[fieldLat]);
					valueLon = NumberUtils.parseNumber(obj[fieldLon]);
					if (isNaN(valueLat) || isNaN(valueLon))
						continue;

					if (numMarkers < markerList.length)
					{
						marker = markerList[numMarkers];
					}
					else
					{
						marker = new PieMarker();
						marker.appendTo(viewport);
						markerList.push(marker);
					}

					// create or reuse slices and compute marker magnitude
					sliceList = marker.sliceList;
					numSlices = 0;
					mag = 0;
					for (j = 0; j < numSeries; j++)
					{
						series = seriesList[j];

						valueSeries = NumberUtils.parseNumber(obj[series.name]);
						if (isNaN(valueSeries) || (valueSeries <= 0))
							continue;

						if (numSlices < sliceList.length)
						{
							slice = sliceList[numSlices];
						}
						else
						{
							slice = new PieSlice();
							slice.appendTo(marker);
							sliceList.push(slice);
						}

						slice.series = series;
						slice.value = valueSeries;

						mag += valueSeries;

						numSlices++;
					}

					if (numSlices === 0)
						continue;

					// record marker attributes
					marker.latLon = new LatLon(valueLat, valueLon);
					marker.data = obj;
					marker.fields = fields;
					marker.magnitude = mag;

					// update magnitude min and max
					if (mag < magMin)
						magMin = mag;
					if (mag > magMax)
						magMax = mag;

					// compute slice angles
					sum = 0;
					angle1 = 0;
					angle2 = 0;
					for (j = 0; j < numSlices; j++)
					{
						slice = sliceList[j];

						sum += slice.value;
						angle1 = angle2;
						angle2 = 360 * (sum / mag);

						slice.startAngle = angle1 - 90;
						slice.arcAngle = angle2 - angle1;
					}

					// remove unused slices
					for (j = sliceList.length - 1; j >= numSlices; j--)
					{
						slice = sliceList.pop();
						slice.dispose();
					}

					numMarkers++;
				}

				// compute marker scales
				magSpan = magMax - magMin;
				for (i = 0; i < numMarkers; i++)
				{
					marker = markerList[i];
					marker.scale = (magSpan > 0) ? NumberUtils.minMax((marker.magnitude - magMin) / magSpan, 0, 1) : (1 / numMarkers);
				}
			}

			// remove unused markers
			for (i = markerList.length - 1; i >= numMarkers; i--)
			{
				marker = markerList.pop();
				marker.dispose();
			}

			// remove unused series
			for (i = seriesList.length - 1; i >= numSeries; i--)
				seriesList.pop();
		};

		this.updateLegendLabelsOverride = function(data, fields)
		{
			var seriesList = this._seriesList;
			var numSeries = seriesList.length;
			var labels = (numSeries > 0) ? new Array(numSeries) : null;
			for (var i = 0; i < numSeries; i++)
				labels[i] = seriesList[i].name;
			return labels;
		};

		this.renderDataOverride = function(data, fields, legend)
		{
			this.invalidate("renderPass");

			var seriesList = this._seriesList;
			var numSeries = seriesList.length;
			var series;
			var seriesIndex;
			var seriesCount;

			var markerColorPalette = this.getInternal("markerColorPalette");
			var markerOpacity = this.getInternal("markerOpacity");
			var markerMinSize = this.getInternal("markerMinSize");
			var markerMaxSize = this.getInternal("markerMaxSize");
			var markerList = this._markerList;
			var numMarkers = markerList.length;
			var marker;

			var sliceList;
			var numSlices;
			var slice;

			var i;
			var j;

			// assign series colors
			seriesCount = legend ? legend.getNumLabels() : numSeries;
			for (i = 0; i < numSeries; i++)
			{
				series = seriesList[i];
				seriesIndex = legend ? legend.getLabelIndex(series.name) : i;
				series.color = markerColorPalette ? markerColorPalette.getColor(series.name, seriesIndex, seriesCount) : 0x000000;
			}

			// render pie slices
			for (i = 0; i < numMarkers; i++)
			{
				marker = markerList[i];
				sliceList = marker.sliceList;
				numSlices = sliceList.length;

				marker.radius = Math.round(NumberUtils.interpolate(markerMinSize, markerMaxSize, marker.scale)) / 2;
				marker.display("none");  // fixes vml flicker

				for (j = 0; j < numSlices; j++)
				{
					slice = sliceList[j];
					slice.fillColor(slice.series.color);
					slice.fillOpacity(markerOpacity);
					slice.draw(0, 0, marker.radius, marker.radius, slice.startAngle, slice.arcAngle);
				}
			}
		};

		this.renderOverride = function(map)
		{
			var width = map.get("width");
			var height = map.get("height");
			var center = map.get("center");
			var centerRelative = map.latLonToRelative(center);

			var wrapX = this.getInternal("wrapX");
			var wrapY = this.getInternal("wrapY");

			// get map bounds in pixels relative to viewport
			var mapTL = map.relativeToViewport(new Point(0, 0));
			var mapBR = map.relativeToViewport(new Point(1, 1));
			var mapLeft = mapTL.x;
			var mapTop = mapTL.y;
			var mapWidth = mapBR.x - mapLeft;
			var mapHeight = mapBR.y - mapTop;

			var markerList = this._markerList;
			var marker;
			var markerPosition;

			for (var i = 0, l = markerList.length; i < l; i++)
			{
				marker = markerList[i];
				markerPosition = map.latLonToRelative(marker.latLon);

				if (wrapX)
				{
					markerPosition.x -= centerRelative.x;
					markerPosition.x %= 1;
					if (markerPosition.x > 0.5)
						markerPosition.x -= 1;
					else if (markerPosition.x < -0.5)
						markerPosition.x += 1;
					markerPosition.x += centerRelative.x;
				}

				if (wrapY)
				{
					markerPosition.y -= centerRelative.y;
					markerPosition.y %= 1;
					if (markerPosition.y > 0.5)
						markerPosition.y -= 1;
					else if (markerPosition.y < -0.5)
						markerPosition.y += 1;
					markerPosition.y += centerRelative.y;
				}

				markerPosition.x = Math.round(mapLeft + mapWidth * markerPosition.x);
				markerPosition.y = Math.round(mapTop + mapHeight * markerPosition.y);

				marker.translate(markerPosition.x, markerPosition.y);
				if (((markerPosition.x + marker.radius) < 0) || ((markerPosition.x - marker.radius) > width) ||
				    ((markerPosition.y + marker.radius) < 0) || ((markerPosition.y - marker.radius) > height))
					marker.display("none");
				else
					marker.display(null);
			}

			var viewport = this._markerViewport;
			viewport.width(width);
			viewport.height(height);
			viewport.viewBox(new Rectangle(0, 0, width, height));
		};

	});

	var Series = jg_extend(Object, function(Series, base)
	{

		// Public Properties

		this.name = null;
		this.color = 0x000000;

		// Constructor

		this.constructor = function()
		{
		};

	});

	var PieMarker = jg_extend(Group, function(PieMarker, base)
	{

		// Public Properties

		this.sliceList = null;
		this.latLon = null;
		this.data = null;
		this.fields = null;
		this.magnitude = 0;
		this.scale = 0;
		this.radius = 0;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.sliceList = [];
		};

		// Public Methods

		this.dispose = function()
		{
			var sliceList = this.sliceList;
			for (var i = sliceList.length - 1; i >= 0; i--)
				sliceList[i].dispose();

			base.dispose.call(this);
		};

	});

	var PieSlice = jg_extend(Wedge, function(PieSlice, base)
	{

		// Public Properties

		this.series = null;
		this.value = 0;
		this.startAngle = 0;
		this.arcAngle = 0;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);
		};

	});

});
});

jg_import.define("splunk.vectors.Image", function()
{
jg_namespace("splunk.vectors", function()
{

	var VectorElement = jg_import("splunk.vectors.VectorElement");

	this.Image = jg_extend(VectorElement, function(Image, base)
	{

		// Constructor

		this.constructor = function(x, y, width, height, href, preserveAspectRatio)
		{
			base.constructor.call(this);
		};

		// Public Methods

		this.x = function(value)
		{
			return this;
		};

		this.y = function(value)
		{
			return this;
		};

		this.width = function(value)
		{
			return this;
		};

		this.height = function(value)
		{
			return this;
		};

		this.href = function(value)
		{
			return this;
		};

		this.preserveAspectRatio = function(value)
		{
			return this;
		};

		this.onLoad = function(value)
		{
			return this;
		};

		this.onError = function(value)
		{
			return this;
		};

		// Inner Mixin Classes

		var SVGImage = jg_static(function(SVGImage)
		{

			// Private Static Constants

			var _NS_XLINK = "http://www.w3.org/1999/xlink";

			// Constructor

			this.constructor = function(x, y, width, height, href, preserveAspectRatio)
			{
				base.constructor.call(this, "image");

				if (x != null)
					this.x(x);
				if (y != null)
					this.y(y);
				if (width != null)
					this.width(width);
				if (height != null)
					this.height(height);
				if (href != null)
					this.href(href);
				if (preserveAspectRatio != null)
					this.preserveAspectRatio(preserveAspectRatio);
			};

			// Public Methods

			this.x = function(value)
			{
				if ((value != null) && (value > -Infinity) && (value < Infinity))
					this.element.setAttribute("x", value);
				else
					this.element.removeAttribute("x");

				return this;
			};

			this.y = function(value)
			{
				if ((value != null) && (value > -Infinity) && (value < Infinity))
					this.element.setAttribute("y", value);
				else
					this.element.removeAttribute("y");

				return this;
			};

			this.width = function(value)
			{
				if ((value != null) && (value < Infinity))
					this.element.setAttribute("width", Math.max(value, 0));
				else
					this.element.removeAttribute("width");

				return this;
			};

			this.height = function(value)
			{
				if ((value != null) && (value < Infinity))
					this.element.setAttribute("height", Math.max(value, 0));
				else
					this.element.removeAttribute("height");

				return this;
			};

			this.href = function(value)
			{
				if (value)
					this.element.setAttributeNS(_NS_XLINK, "href", value);
				else
					this.element.removeAttributeNS(_NS_XLINK, "href");

				return this;
			};

			this.preserveAspectRatio = function(value)
			{
				if (value)
					this.element.setAttribute("preserveAspectRatio", value);
				else
					this.element.removeAttribute("preserveAspectRatio");

				return this;
			};

			this.onLoad = function(value)
			{
				if (typeof value === "function")
					this.element.onload = value;
				else
					this.element.onload = null;

				return this;
			};

			this.onError = function(value)
			{
				if (typeof value === "function")
					this.element.onerror = value;
				else
					this.element.onerror = null;

				return this;
			};

		});

		var VMLImage = jg_static(function(VMLImage)
		{
		});

		VectorElement.mixin(this, SVGImage, VMLImage);

	});

});
});

jg_import.define("splunk.mapping2.layers.SVGTileLayer", function()
{
jg_namespace("splunk.mapping2.layers", function()
{

	var Rectangle = jg_import("jgatt.geom.Rectangle");
	var TileLayerBase = jg_import("splunk.mapping2.layers.TileLayerBase");
	var Group = jg_import("splunk.vectors.Group");
	var Image = jg_import("splunk.vectors.Image");
	var Viewport = jg_import("splunk.vectors.Viewport");

	this.SVGTileLayer = jg_extend(TileLayerBase, function(SVGTileLayer, base)
	{

		// Private Properties

		this._tileViewport = null;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.addStyleClass("splunk-mapping2-layers-SVGTileLayer");

			this._tileViewport = new Viewport()
				.appendTo(this.element);
		};

		// Public Methods

		this.toSVGString = function()
		{
			this.validate();

			return this._tileViewport.toSVGString();
		};

		// Protected Methods

		this.renderOverride = function(map)
		{
			var width = map.get("width");
			var height = map.get("height");

			var tileViewport = this._tileViewport;
			tileViewport.width(width);
			tileViewport.height(height);
			tileViewport.viewBox(new Rectangle(0, 0, width, height));

			base.renderOverride.call(this, map);
		};

		this.createTileContainer = function()
		{
			var container = new Group();
			container.appendTo(this._tileViewport);
			return container;
		};

		this.destroyTileContainer = function(container)
		{
			container.remove();
		};

		this.displayTileContainer = function(container, display)
		{
			if (display)
			{
				container.display(null);
				container.appendTo(this._tileViewport);
			}
			else
			{
				container.display("none");
			}
		};

		this.createTile = function(container, url, onLoad, onError)
		{
			var tile = new Image();
			tile.onLoad(onLoad);
			tile.onError(onError);
			tile.preserveAspectRatio("none");
			tile.href(url);
			tile.appendTo(container);
			return tile;
		};

		this.destroyTile = function(container, tile)
		{
			tile.onLoad(null);
			tile.onError(null);
			tile.href(null);
			tile.remove();
		};

		this.positionTile = function(tile, x, y, width, height)
		{
			tile.x(x);
			tile.y(y);
			tile.width(width);
			tile.height(height);
		};

	});

});
});

jg_import.define("splunk.mapping2.parsers.LatLonBoundsParser", function()
{
jg_namespace("splunk.mapping2.parsers", function()
{

	var LatLonBounds = jg_import("splunk.mapping2.LatLonBounds");
	var NumberParser = jg_import("splunk.parsers.NumberParser");
	var Parser = jg_import("splunk.parsers.Parser");
	var ParseUtils = jg_import("splunk.parsers.ParseUtils");

	this.LatLonBoundsParser = jg_extend(Parser, function(LatLonBoundsParser, base)
	{

		// Private Static Properties

		var _instance = null;

		// Public Static Methods

		LatLonBoundsParser.getInstance = function()
		{
			if (!_instance)
				_instance = new LatLonBoundsParser();
			return _instance;
		};

		// Protected Properties

		this.numberParser = null;

		// Constructor

		this.constructor = function()
		{
			this.numberParser = NumberParser.getInstance();
		};

		// Public Methods

		this.stringToValue = function(str)
		{
			var values = ParseUtils.prepareTuple(str);
			if (!values)
				return null;

			var latLonBounds = new LatLonBounds();

			var numValues = values.length;
			if (numValues > 0)
				latLonBounds.s = this.numberParser.stringToValue(values[0]);
			if (numValues > 1)
				latLonBounds.w = this.numberParser.stringToValue(values[1]);
			if (numValues > 2)
				latLonBounds.n = this.numberParser.stringToValue(values[2]);
			if (numValues > 3)
				latLonBounds.e = this.numberParser.stringToValue(values[3]);

			return latLonBounds;
		};

		this.valueToString = function(value)
		{
			var latLonBounds = (value instanceof LatLonBounds) ? value : null;
			if (!latLonBounds)
				return null;

			var str = "";

			str += this.numberParser.valueToString(latLonBounds.s) + ",";
			str += this.numberParser.valueToString(latLonBounds.w) + ",";
			str += this.numberParser.valueToString(latLonBounds.n) + ",";
			str += this.numberParser.valueToString(latLonBounds.e);

			return "(" + str + ")";
		};

	});

});
});

jg_import.define("splunk.mapping2.parsers.LatLonParser", function()
{
jg_namespace("splunk.mapping2.parsers", function()
{

	var LatLon = jg_import("splunk.mapping2.LatLon");
	var NumberParser = jg_import("splunk.parsers.NumberParser");
	var Parser = jg_import("splunk.parsers.Parser");
	var ParseUtils = jg_import("splunk.parsers.ParseUtils");

	this.LatLonParser = jg_extend(Parser, function(LatLonParser, base)
	{

		// Private Static Properties

		var _instance = null;

		// Public Static Methods

		LatLonParser.getInstance = function()
		{
			if (!_instance)
				_instance = new LatLonParser();
			return _instance;
		};

		// Protected Properties

		this.numberParser = null;

		// Constructor

		this.constructor = function()
		{
			this.numberParser = NumberParser.getInstance();
		};

		// Public Methods

		this.stringToValue = function(str)
		{
			var values = ParseUtils.prepareTuple(str);
			if (!values)
				return null;

			var latLon = new LatLon();

			var numValues = values.length;
			if (numValues > 0)
				latLon.lat = this.numberParser.stringToValue(values[0]);
			if (numValues > 1)
				latLon.lon = this.numberParser.stringToValue(values[1]);

			return latLon;
		};

		this.valueToString = function(value)
		{
			var latLon = (value instanceof LatLon) ? value : null;
			if (!latLon)
				return null;

			var str = "";

			str += this.numberParser.valueToString(latLon.lat) + ",";
			str += this.numberParser.valueToString(latLon.lon);

			return "(" + str + ")";
		};

	});

});
});

jg_import.define("splunk.mapping2.projections.EquirectangularProjection", function()
{
jg_namespace("splunk.mapping2.projections", function()
{

	var Point = jg_import("jgatt.geom.Point");
	var LatLon = jg_import("splunk.mapping2.LatLon");
	var Projection = jg_import("splunk.mapping2.projections.Projection");

	this.EquirectangularProjection = jg_extend(Projection, function(EquirectangularProjection, base)
	{

		// Private Static Properties

		var _instance = null;

		// Public Static Methods

		EquirectangularProjection.getInstance = function()
		{
			if (!_instance)
				_instance = new EquirectangularProjection();
			return _instance;
		};

		// Public Methods

		this.latLonToRelative = function(latLon)
		{
			var x = (latLon.lon + 180) / 360;
			var y = (90 - latLon.lat) / 180;
			return new Point(x, y);
		};

		this.relativeToLatLon = function(relativePoint)
		{
			var lat = 90 - relativePoint.y * 180;
			var lon = relativePoint.x * 360 - 180;
			return new LatLon(lat, lon);
		};

	});

});
});
