define(function(require, exports, module) {
    var _ = require('underscore');
    var Backbone = require('backbone');
    var console = require('util/console');
    var DomTrackerMixin = require('mixins/domtracker');
    var mvc = require('./mvc');
    
    var BaseManager = Backbone.Model.extend({
        constructor: function(attributes, options) {
            attributes = attributes || {};
            options = options || {};
            
            // Get or generate a name
            var id = options.id || attributes.id;
 
            if (id === undefined) {
                id = options.name || attributes.name;
                if (id !== undefined) {
                    console.log("Use of 'name' to specify the ID of a Splunk model is deprecated.");
                }
            }
            
            if (id === undefined) {
                id = _.uniqueId('manager_');
            }
            
            // Store it on the instance/options
            this.id = this.name = options.name = options.id = id;
            var returned = Backbone.Model.prototype.constructor.apply(this, arguments);
            
            // Register it on the global registry
            mvc.Components.registerInstance(this.id, this, { replace: options.replace });
            
            return returned;
        },
        
        _start: function() {},

        dispose: function() {
            this.stopListeningDOM();
            this.stopListening();

            if (mvc.Components.getInstance(this.id) === this) {
                mvc.Components.revokeInstance(this.id);
            }
        }
    });
    
    _.extend(BaseManager.prototype, DomTrackerMixin);
    
    return BaseManager;
});
