define(function(require, exports, module) {

    var _ = require('underscore');
    var $ = require('jquery');
    var Backbone = require('backbone');
    var BaseMultiChoiceView = require("./basemultichoiceview");
    var SyntheticCheckboxControl = require("views/shared/controls/SyntheticCheckboxControl");

    require("css!../css/checkbox.css");

    var asArray = function(obj) {
        if (obj === undefined) {
            return [];
        }
        return _.isArray(obj) ? obj : [obj];
    };

    /**
     * Displays a list of boolean checkboxes.  This control's value is
     * always the array of selected checkbox values.
     */
    var CheckboxGroupView = BaseMultiChoiceView.extend({
        moduleId: module.id,
        
        className: "splunk-checkboxgroup splunk-choice-input",
        
        options: {
            valueField: "",
            labelField: "",
            "default": undefined,
            choices: [],
            value: undefined,
            disabled: false
        },
        
        initialize: function() {
            this.options = _.extend({}, BaseMultiChoiceView.prototype.options, this.options);
            BaseMultiChoiceView.prototype.initialize.apply(this, arguments);
            this._selections = new Backbone.Model();
            this.listenTo(this._selections, "change", this._updateValue, this);
            this.updateDomVal();
        },

        _updateValue: function(model, change, options) {
            // Must copy array so we always get a change event
            var val = this.val().slice(0);
            _(model.changed).each(function(v, k) {
               if (!v) {
                   val = _(val).without(k);
               } else if (val.indexOf(k) < 0) {
                   val.push(k);
               }
            });
            this.val(val);
        },

        _disable: function(state) {
            _.each(this._checkboxes, function(checkbox) {
                if (state){
                    checkbox.disable();
                } else {
                    checkbox.enable();
                }
            });
        },
        
        // Used by unit tests
        _domVal: function() {
            var value = [];
            _.each($('.checkbox', this.el), function(item) {
                var iStyle = $('i', item).attr('style') || '';
                var isDisabled = 
                    (iStyle.indexOf('display:none') !== -1) ||
                    (iStyle.indexOf('display: none') !== -1);
                var subvalue = $('a', item).data('name');
                
                if (!isDisabled) {
                    value.push(subvalue);
                }
            });
            return value;
        },

        updateDomVal: function() {
            var oldSelections = this._selections.toJSON();
            var newSelections = {};
            _(oldSelections).each(function(value, key) {
                newSelections[key] = 0;
            });
            _.each(this.val(), function(val) {
                newSelections[val] = 1;
            },this);
            this._selections.set(newSelections);
        },

        createView: function() {
            this.$el.empty();
            return $("<div class='splunk-checkboxgroup-choices'/>").appendTo(this.el);
        },

        updateView: function(viz, data) {
            viz.empty();

            if (this._checkboxes){
                _.each(this._checkboxes, function(checkbox){
                    checkbox.remove();
                });
            }
            this._checkboxes = [];
            // If there is no data, we don't want to just render a message,
            // because that will look odd. Instead, we render a single checkbox
            // that will subsequently get disabled (in BaseChoiceView), plus
            // the message. Finally, we also set the label to " " to make sure it
            // gets picked up.
            if (!data || data.length === 0) {
                data = [{value: "", label: " "}];
            }

            var $choices = this.$(".splunk-checkboxgroup-choices");
            _.each(data, function(value){
                var syntheticCheckboxControl = new SyntheticCheckboxControl({
                    model: this._selections,
                    modelAttribute: value.value,
                    label: _($.trim(value.label) || value.value || '').t()
                });

                syntheticCheckboxControl.render().appendTo($choices);
                this._checkboxes.unshift(syntheticCheckboxControl);
            }, this );

            return this;
        }
    });
    
    return CheckboxGroupView;
});
