define(function(require, exports, module) {

    var _ = require('underscore');
    var $ = require('jquery');
    var BaseInputView = require("./baseinputview");

    require("css!../css/checkbox.css");

    /**
     * Displays a boolean checkbox.
     */
    var CheckboxView = BaseInputView.extend({
        moduleId: module.id,
        
        className: "splunk-checkbox",
        inputType: "checkbox", 

        options: {
            "default": undefined,
            value: undefined,
            disabled: false
        },

        events: {
            "change input:checkbox": "_onChange"
        },
        
        initialize: function() {
            this.options = _.extend({}, BaseInputView.prototype.options, this.options);
            BaseInputView.prototype.initialize.apply(this, arguments);
            
            // Update view if model changes.
            this.settings.on("change", this.render, this);
            
            // Update model if view changes.
            var that = this;
            this.on("change", function() {
                that.settings.set("value", this.val());
            });
        },

        createView: function() {
            var viz = $("<input type='" + this.inputType + "'>");
            this.$el.html(viz);
            return viz;
        },

        updateView: function(viz, data) {
            this.val(this.settings.get("value") || false);
            return this;
        },

        // Get or set the current input's value.
        val: function(value) {
            var input = $("input", this.$el);
            if (value === undefined) {
                return input.prop('checked');
            }

            if (value !== this.val()) {
                input.prop('checked', Boolean(value));
                this.settings.set("value", Boolean(value));
                this._onChange();
            }
            return this.val();
        }
    });
    
    return CheckboxView;
});
