define(function (require, exports, module) {
    var mvc = require('./mvc'),
        BaseSplunkView = require("./basesplunkview"),
        GlobalNav = require('views/shared/splunkbar/Master'),
        AppNav = require('views/shared/appbar/Master'),
        SideNav = require('views/shared/litebar/Master'),
        sharedModels = require('./sharedmodels'),
        userAgent = require('helpers/user_agent'),
        TourBar = require('views/shared/tourbar/TourBar'),
        splunkUtils = require('splunk.util'),
        config = require('models/config'),
        route = require('uri/route');


    var HeaderView = BaseSplunkView.extend({
        moduleId: module.id,
        
        className: 'splunk-header',

        options: {
            appbar: true,
            splunkbar: true,
            acceleratedAppNav: false,
            useSessionStorageCache: false
        },

        initialize: function() {
            this.configure();
            this.model = this.model || {};
            this.model.application= sharedModels.get("app");
            this.model.user= sharedModels.get("user");
            this.model.appLocal= sharedModels.get("appLocal");
            this.model.serverInfo= sharedModels.get("serverInfo");
            
            var acceleratedAppNav = this.settings.get('acceleratedAppNav') && userAgent.isIE7();
            this.collection = this.collection || {};
            this.collection.appLocals = sharedModels.get("appLocals");
            this.deferreds = this.deferreds || {};
            this.initTour();
            this.useSideNav = this.settings.get('litebar');

            if (this.useSideNav) {
                this.sideNav = SideNav.create({
                    model: {
                        application: this.model.application,
                        appLocal: this.model.appLocal,
                        user: this.model.user,
                        serverInfo: this.model.serverInfo,
                        config: config
                    },
                    collection: {
                        apps: this.collection.appLocals
                    }
                });
            } else {
                if(this.settings.get('splunkbar')) {
                    this.globalNav = GlobalNav.create({
                        model: {
                            application: this.model.application,
                            appLocal: this.model.appLocal,
                            user: this.model.user,
                            serverInfo: this.model.serverInfo,
                            config: config
                        },
                        collection: {
                            apps: this.collection.appLocals
                        }
                    });
                }

                if(this.settings.get('appbar')){
                    this.appNav = AppNav.create({
                        model: {
                            application: this.model.application,
                            app: this.model.appLocal,
                            user: this.model.user,
                            serverInfo: this.model.serverInfo
                        },
                        appServerUrl: acceleratedAppNav,
                        useSessionStorageCache: this.settings.get('useSessionStorageCache'),
                        autoRender: false
                    });
                }
            }
        },
        initTour: function() {
            this.deferreds.tour = $.Deferred();

            // get the tour name from the query string
            this.tourName = undefined; 
            var queryProps = splunkUtils.queryStringToProp(window.location.search);
            if (queryProps.hasOwnProperty("tour")) {
                var tours = sharedModels.get("uiTours");
                this.tourName = queryProps["tour"];
                if (tours && this.tourName && this.tourName.length > 0) {

                    // when the collection has been fetched, grab the current tour stanza
                    tours.dfd.done(function() {
                        this.model.tour = tours.getTourModel(this.tourName);
                        if (this.model.tour) {
                            this.tourbar = new TourBar({
                                model: this.model.tour
                            });
                            this.model.tour.on("navigateRequest", function(destTourModel) {
                                var link = destTourModel.getUri(true);  
                                var path = "app/" + this.model.application.get("app") + "/" + link;
                                splunkUtils.redirect_to(path);
                            }, this);
                            this.model.tour.on("endTour", function() {
                                // route to home page
                                var path = route.home(this.model.application.get('root'),
                                                      this.model.application.get('locale'));
                                window.location.href = path;
                            }, this);
                            this.deferreds.tour.resolve();
                        }
                    }.bind(this));
                }
            }
        },
        render: function() {
            if (this.useSideNav) {
                this.$el.append(this.sideNav.render().el);
            } else {
                this.$el.empty();
                if (this.settings.get('splunkbar')) {
                    this.$el.append(this.globalNav.render().el);
                }
                if(this.settings.get('appbar')){
                    this.$el.append(this.appNav.render().el);
                }
            }
            if (this.tourName) {
                this.deferreds.tour.then(function(){
                    if (this.model.tour && this.tourbar) {
                        this.$el.append(this.tourbar.render().el);
                    }
                }.bind(this));
            }
            return this;
        }
    });

    return HeaderView;
});
