define(function(require, exports, module) {

    var _ = require('underscore');
    var $ = require('jquery');
    var BaseChoiceView = require("./basechoiceview");

    require("css!../css/radio.css");

    /**
     * Displays a list of radio buttons.
     *  
     * The value of this control is the value of the single button
     * selected, or undefined if no button is selected.
     */
    var RadioGroupView = BaseChoiceView.extend({
        moduleId: module.id,
        className: "splunk-radiogroup splunk-choice-input",
        
        options: {
            valueField: "",
            labelField: "",
            "default": undefined,
            "choices": [],
            selectFirstChoice: false,
            disabled: false,
            value: undefined
        },

        events: {
            "change input:radio": "onDomChange"
        },

        initialize: function() {
            this.options = _.extend({}, BaseChoiceView.prototype.options, this.options);
            BaseChoiceView.prototype.initialize.apply(this, arguments);
        },

        _domVal: function() {
            return this.$("input:radio:checked").val();
        },

        onDomChange: function() {
            this.val(this._domVal());
        },

        updateDomVal: function(value) {
            this.$('input:radio').prop('checked', false);
            this.$('input:radio[value="' + value + '"]').prop('checked', 'checked');
        },
        
        createView: function() {
            this.$el.empty();
            return $("<fieldset class='splunk-radiogroup-choices'/>").appendTo(this.el);
        },

        updateView: function(viz, data) {
            viz.empty();

            // If there is no data, we don't want to just render a message,
            // because that will look odd. Instead, we render a single radio
            // that will subsequently get disabled (in BaseChoiceView), plus
            // the message.
            if (!data || data.length === 0) {
                data = [{value: "", label: " "}];
            }

            var that = this;
            var controlValue = this.settings.get("value");
            var id = this.id;

            _.each(data || [], function(entry, idx) {
                var itemId = id + String(idx);
                // As this control is represented by a group of HTML
                // objects, each object must have its own unique ID.
                var input = $('<input type="radio" />')
                    .attr({name: id, value: entry.value, id: itemId});

                if (entry.value == controlValue) {
                    input.prop({'checked': 'checked'});
                }

                var choice = $('<div class="choice" />')
                    .append(input)
                    .append($('<label />')
                            .attr("for", itemId)
                            .text(_($.trim(entry.label) || entry.value || '').t()));

                viz.append(choice);
            });

            return this;
        }
    });
    
    return RadioGroupView;
});
