define(function(require, exports, module) {
    var $ = require('jquery');
    var _ = require('underscore');
    var BaseView = require('views/Base');
    var splunkUtil = require('splunk.util');
    require('bootstrap.tooltip');

    var ReportItem = BaseView.extend({
    	moduleId: module.id,
    	tagName: 'li',
        className: 'panel-content',
    	initialize: function() {
            BaseView.prototype.initialize.apply(this, arguments);
            this.model.highlightSelectedModel.on('change:currentSelectedView', this.checkSelectedItem, this);
    	},
    	render: function() {
    		var report = this.model.report;
			var general = report.entry.content.get('display.general.type');
            var qualifiedType = general;
            var vsid = report.entry.content.get("vsid");
            var displayview = report.entry.content.get("displayview");
            var hasBeenMigrated = splunkUtil.normalizeBoolean(report.entry.content.get("display.general.migratedFromViewState"));

			if (vsid && displayview && !hasBeenMigrated) {
                qualifiedType = null;
            } else {
                switch (general) {
                    case 'visualizations':
                        var subtype = report.entry.content.get('display.visualizations.charting.chart');
                        if (report.entry.content.get('display.visualizations.type') === "charting") {
                            switch (subtype) {
                                case 'radialGauge':
                                    qualifiedType = 'gauge-radial';
                                    break;
                                case 'fillerGauge':
                                    qualifiedType = 'gauge-filler';
                                    break;
                                case 'markerGauge':
                                    qualifiedType = 'gauge-marker';
                                    break;
                                default:
                                    qualifiedType = 'chart-' + subtype;
                                    break;
                            }
                        } else if (report.entry.content.get('display.visualizations.type') === "mapping") {
                            qualifiedType = 'location';
                        } else {
                            qualifiedType = 'single-value';
                        }
                        break;
                    case 'events':
                        qualifiedType = 'list';
                        break;
                    case 'statistics':
                        qualifiedType = 'table';
                        break;
                }
            }
    		this.$el.html(this.panelReportTemplate({name: _(this.model.report.entry.get('name')).t(), qualifiedType: qualifiedType}));
    		this.$el.tooltip({
    			template: '<div class="tooltip add-report-name"><div class="tooltip-arrow"></div><div class="tooltip-inner"></div></div>',
    			container: 'body',
    			delay: {show: 500, hide: 0}
    		});
    		return this;
    	},
    	events: {
    		'click a': 'select'
    	},
    	select: function(evt) {
    		evt.preventDefault();
    		this.trigger('previewPanel', this.model.report, evt);
            this.model.highlightSelectedModel.set('currentSelectedView', this.cid);
    	},
        checkSelectedItem: function(changedModel) {
            if(changedModel.get('currentSelectedView') === this.cid) {
                this.$el.addClass('selected');
            } else {
                this.$el.removeClass('selected');
            }
        },
		panelReportTemplate: _.template('<a href="#" class="panel-content-report" title="<%- name %>"><div class="icons"><i class="icon-<%- qualifiedType %>"></i></div><%- name %></a>')
    });

    return ReportItem;
});