define(function(require, exports, module){
    var _ = require('underscore'); 
    var mvc = require('../../mvc');
    var TokenUtils = require('../../tokenutils');
    
    var registry = mvc.Components;

    function isTokenDefined(token) {
        return registry.get(token.namespace).has(token.name);
    }

    var TokenDependenciesMixin = {
        setupTokenDependencies: function(){
            var deps = this.settings.get('tokenDependencies', { tokens: true });
            if (deps) {
                var element = this;
                var requiredTokens = deps.depends ? TokenUtils.getTokens(deps.depends, { tokenNamespace: 'submitted' }) : [];
                var rejectedTokens = deps.rejects ? TokenUtils.getTokens(deps.rejects, { tokenNamespace: 'submitted' }) : [];
                var allTokens = requiredTokens.concat(rejectedTokens);

                function handleTokenChange() {
                    if (_(requiredTokens).all(isTokenDefined) && !_(rejectedTokens).any(isTokenDefined)) {
                        element._tokenDepsMet = true;
                        element.show();
                    } else {
                        element._tokenDepsMet = false;
                        element.hide();
                    }
                }

                _(allTokens).each(function(token) {
                    var ns = registry.get(token.namespace);
                    element.listenTo(ns, 'change:' + token.name, handleTokenChange, element);
                });
                handleTokenChange();
            } else {
                this._tokenDepsMet = true;
            }
        },
        tokenDependenciesMet: function(){
            return this._tokenDepsMet;
        },
        show: function() {
            this.$el.removeClass('hidden').trigger('elementVisibilityChanged');
        },
        hide: function() {
            this.$el.addClass('hidden').trigger('elementVisibilityChanged');
        }
    };
    
    return TokenDependenciesMixin;
});