define(['underscore', 'models/shared/ClassicURL', 'util/console'], function(_, ClassicURLModel, console) {

    function _encodePrimitiveValue(value) {
        if (_.isObject(value)) {
            console.error('Encountered non-primitive value %o to be encoded as URL param.', value);
            throw new Error('Non-primitive values are not allowed in the query string');
        }
        return encodeURIComponent(value);
    }

    /**
     * Subclass of the ClassicURL model which adds support for encoding and decoding arrays of strings as a repeated
     * list of URL params.
     */
    var DashboardURLModel = ClassicURLModel.extend({
        encode: function(options) {
            var object = this.toJSON();
            var queryArray = [], encodedKey;
            options = _.extend({ preserveEmptyStrings: true, preserveNull: true }, options);
            _.each(object, function(value, key) {
                if (_.isUndefined(value) || (value === null && !options.preserveNull)
                    || (value === "" && !options.preserveEmptyStrings)) {
                    return;
                }
                encodedKey = encodeURIComponent(key);
                if (value === null) {
                    queryArray.push(encodedKey);
                } else if (_.isArray(value)) {
                    _.each(value, function(v) {
                        queryArray.push(encodedKey + '=' + _encodePrimitiveValue(v));
                    });
                } else {
                    queryArray.push(encodedKey + '=' + _encodePrimitiveValue(value));
                }
            });
            return queryArray.join('&');
        },
        decode: function(queryString) {
            queryString = (queryString || '').replace(/^[&\?#]|[&#]$/g, '');
            if (!queryString) {
                return {};
            }
            var output = {};
            _(queryString.split('&')).each(function(param) {
                var parts = param.split('=', 2);
                var key = decodeURIComponent(parts[0]);
                var value = parts.length > 1 ? decodeURIComponent(parts[1]) : null;
                if (output.hasOwnProperty(key)) {
                    var cur = output[key];
                    if (!_.isArray(cur)) {
                        cur = output[key] = [cur];
                    }
                    cur.push(value);
                } else {
                    output[key] = value;
                }
            });
            return output;
        }
    });

    // Return a singleton instance of the dashboard URL model
    return new DashboardURLModel();
});