define(['underscore', 'jquery'], function (_, $) {

    var HTML_COMMENTS_PATTERN = /<!--.+?-->/gmi;
    var BAD_NODE_SELECTOR = 'script,link,meta,head,*[type="text/javascript"]';
    //var ALLOWED_URLS = /^(?:\/[^\/]|https?:|#)/g;
    var BAD_URL_SCHEMES = /^(?:javascript|jscript|livescript|vbscript|data|about|mocha):/i;
    var EVENT_HANDLER_ATTRIBUTE_PREFIX = "on";
    var CSS_NODE_SELECTOR = 'style';
    var CSS_EXPRESSION_PATTERN = /(^|[\s\W])expression(\s*\()/gmi;
    var CSS_EXPRESSION_REPLACE = '$1no-xpr$2';
    var URL_ATTRIBUTES = {
        link: ['href'],
        applet: ['code', 'object'],
        iframe: ['src'],
        img: ['src'],
        embed: ['src'],
        layer: ['src'],
        a: ['href']
    };

    function removeBadNodes($root) {
        if($root.is(BAD_NODE_SELECTOR)) {
            return $([]);
        }
        $root.find(BAD_NODE_SELECTOR).remove();
        return $root;
    }

    function cleanupUrl(url) {
        return decodeURIComponent(_.unescape($.trim(url || ''))).replace(/\s/gmi, '');
    }

    function isBadUrl(url) {
        return BAD_URL_SCHEMES.test(cleanupUrl(url));
    }

    function cleanAttributes($root) {
        _($root).each(function (node) {
            var $node = $(node);
            var nodeName = (node.tagName||'').toLowerCase();
            _(node.attributes).each(function (attr) {
                var attrName = attr.name.toLowerCase();
                if (attrName.indexOf(EVENT_HANDLER_ATTRIBUTE_PREFIX) === 0) {
                    $node.removeAttr(attr.name);
                } else {
                    var urlAttrs = URL_ATTRIBUTES[nodeName];
                    if (urlAttrs && _(urlAttrs).contains(attrName)) {
                        if (isBadUrl(attr.value)) {
                            $node.attr(attr.name, '#');
                        }
                    }
                }
            });
            _($node.children()).chain().map($).each(cleanAttributes);
        });
    }

    function stripComments(txt) {
        return txt.replace(HTML_COMMENTS_PATTERN, '');
    }

    function cleanStylesheets($root) {
        _($('<div />').append($root).find(CSS_NODE_SELECTOR)).each(cleanStylesheet);
    }
    
    function cleanStylesheet(styleNode) {
        var $style = $(styleNode);
        var cssText = $style.html();
        var newText = cleanCssText(cssText);
        if (cssText != newText) {
            $style.text(newText);
        }
    }

    function cleanCssText(cssText) {
        CSS_EXPRESSION_PATTERN.lastIndex = 0;
        return cssText.replace(CSS_EXPRESSION_PATTERN, CSS_EXPRESSION_REPLACE);
    }

    function cleanHtml(htmlText) {
        var $html = $(stripComments(htmlText));
        $html = removeBadNodes($html);
        cleanAttributes($html);
        cleanStylesheets($html);
        return $('<div />').append($html).html();
    }

    return {
        clean: cleanHtml,
        _stripComments: stripComments,
        _isBadUrl: isBadUrl,
        _cleanAttributes: cleanAttributes,
        _removeScripts: removeBadNodes,
        _cleanStylesheets: cleanStylesheets,
        _cleanCssText: cleanCssText
    };

});