/**
 * Util package for formatting number.
 */

define(['underscore', 'splunk.i18n', 'splunk.util', 'util/math_utils'], function(_, i18n, splunkUtil, mathUtils) {

    /**
     * method to convert number to file size representation
     * @param bytes (file size in bytes)
     */

    var bytesToFileSize = function(bytes) {
            var fileSizeInt = bytes;
            var fileString = "";
            var fileSize = Math.floor((fileSizeInt || 0)/1024);
            if(fileSize === 0) {
                fileString = i18n.format_decimal(fileSizeInt) + " " +  _("B").t();
            } else {
                var newFileSize = Math.floor((fileSize || 0)/1024);
                if(newFileSize === 0) {
                    fileString = i18n.format_decimal((fileSizeInt/(1024)).toFixed(2)) + " " +  _("KB").t();
                } else {
                    var theFileSize = Math.floor((newFileSize || 0)/1024);
                    if(theFileSize === 0){
                        fileString = i18n.format_decimal((fileSizeInt/(1024*1024)).toFixed(2)) + " " + _("MB").t();
                    } else {
                        fileString = i18n.format_decimal((fileSizeInt/(1024*1024*1024)).toFixed(2)) + " " + _("GB").t();
                    }
                }
            }
          //  console.log("here is the file size in bytes ", fileSizeInt);
            return fileString;
        };




    /**
     * A method to abbreviate number to a maximum of 4 digits 
     *
     * @param number {integer} number to format
     */

    var abbreviateNumber = function(number) {
        if (number <= 1000) {
            //less than 1,000
            return i18n.format_decimal(number);
        } else if (number < 10000) {
            //less than 10,000
            return splunkUtil.sprintf(_('%sK').t(), i18n.format_decimal(mathUtils.roundToDecimal(number/1000, -2)));
        } else if (number < 100000) {
            //less than 100,000
            return splunkUtil.sprintf(_('%sK').t(), i18n.format_decimal(mathUtils.roundToDecimal(number/1000, -1)));
        } else if (number < 999500) {
            //less than 999,500
            return splunkUtil.sprintf(_('%sK').t(), i18n.format_decimal(mathUtils.roundToDecimal(number/1000, 0)));
        } else if (number < 10000000) {
            //less than 10,000,000
            return splunkUtil.sprintf(_('%sM').t(), i18n.format_decimal(mathUtils.roundToDecimal(number/1000000, -2)));
        } else if (number < 100000000) {
            //less than 100,000,000
            return splunkUtil.sprintf(_('%sM').t(), i18n.format_decimal(mathUtils.roundToDecimal(number/1000000, -1)));
        } else if (number < 999500000) {
            //less than 999,500,000
            return splunkUtil.sprintf(_('%sM').t(), i18n.format_decimal(mathUtils.roundToDecimal(number/1000000, 0)));
        } else if (number < 10000000000) {
            //less than 10,000,000,000
            return splunkUtil.sprintf(_('%sB').t(), i18n.format_decimal(mathUtils.roundToDecimal(number/1000000000, -2)));
        } else if (number < 100000000000) {
            //less than 100,000,000,000
            return splunkUtil.sprintf(_('%sB').t(), i18n.format_decimal(mathUtils.roundToDecimal(number/1000000000, -1)));
        } else {
            //greater than or equal to 100,000,000,000
            return splunkUtil.sprintf(_('%sB').t(), i18n.format_decimal(mathUtils.roundToDecimal(number/1000000000, 0)));
        }
    };
    return ({
        abbreviateNumber: abbreviateNumber,
        bytesToFileSize: bytesToFileSize
    });
});