define(
    [
        'module',
        'contrib/text!./Master.html',
        'underscore',
        'splunk.util',
        'uri/route',
        'views/Base',
        'views/account/login/Master',
        'views/account/passwordchange/Master'
    ],
    function(module, template, _, splunkutil, route, BaseView, LoginView, PasswordChangeView) {
        return BaseView.extend({
            moduleId: module.id,
            template: template,
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                this.children.passwordChange = new PasswordChangeView({
                    model: {
                        application: this.model.application,
                        session: this.model.session,
                        login: this.model.login,
                        user: this.model.user
                    }
                });
                this.children.login = new LoginView({
                    model: {
                        application: this.model.application,
                        serverInfo: this.model.serverInfo,
                        session: this.model.session,
                        web: this.model.web,
                        updateChecker: this.model.updateChecker,
                        login: this.model.login
                    }
                });
                this.listenTo(this.model.application, 'change:page', this.visibility);
            },
            visibility: function() {
                if (this.model.application.get('page') === 'passwordchange' || this.model.session.entry.content.get('forcePasswordChange')) {
                    this.children.login.$el.hide();
                    this.children.passwordChange.show();
                } else {
                    this.children.passwordChange.$el.hide();
                    this.children.login.show();
                }
            },
            onAddedToDocument: function() {
                BaseView.prototype.onAddedToDocument.apply(this, arguments);
                this.visibility();
            },
            render: function() {
                var html = this.compiledTemplate({
                    year: new Date().getFullYear(),
                    _: _,
                    model: {
                        serverInfo: this.model.serverInfo
                    },
                    splunkutil: splunkutil
                });
                this.$el.html(html);
                this.visibility();
                this.children.login.render().appendTo(this.$('.content'));
                this.children.passwordChange.render().appendTo(this.$('.content'));
                return this;
            }
        });
    }
);
