define(
    [
        'module',
        'contrib/text!./Master.html',
        'underscore',
        'splunk.util',
        'uri/route',
        'views/Base',
        'views/shared/controls/TextControl',
        'views/account/login/FirstTime',
        'views/account/login/updatechecker/Master',
        'views/account/login/Error'
    ],
    function(module, template, _, splunkutil, route, BaseView, TextControlView, FirstTimeModalView, UpdateCheckerView, ErrorView) {
        return BaseView.extend({
            moduleId: module.id,
            template: template,
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                this.children.username = new TextControlView({
                    model: this.model.login,
                    modelAttribute: 'username',
                    elementId: 'username',
                    autocomplete: this.model.web.entry.content.get('enable_autocomplete_login'),
                    placeholder: _('Username').t(),
                    updateOnKeyUp: true,
                    updateOnAutofill: true,
                    disabled: this.model.serverInfo.isLicenseStatePreviouslyKeyed()
                });
                this.children.password = new TextControlView({
                    model: this.model.login,
                    modelAttribute: 'password',
                    elementId: 'password',
                    autocomplete: this.model.web.entry.content.get('enable_autocomplete_login'),
                    placeholder: _('Password').t(),
                    updateOnKeyUp: true,
                    updateOnAutofill: true,
                    password: true,
                    disabled: this.model.serverInfo.isLicenseStatePreviouslyKeyed()
                });
                if (this.model.application.get('page')==='login') {
                    this.children.updateChecker = new UpdateCheckerView({
                        model: {
                            updateChecker: this.model.updateChecker,
                            serverInfo: this.model.serverInfo,
                            login: this.model.login,
                            application: this.model.application,
                            session: this.model.session,
                            web: this.model.web
                        }
                    });
                }
                this.children.firstTimeModal = new FirstTimeModalView();
                this.children.error = new ErrorView({
                    model: {
                        login: this.model.login,
                        serverInfo: this.model.serverInfo,
                        application: this.model.application,
                        session: this.model.session
                    }
                });
                this.listenTo(this.model.login, 'error', function() {
                    if (this.$el.is(':visible') && this.$('input[type=submit]').is(':focus')) {
                        this.children.username.focus();
                    }
                });
            },
            events: {
                'click a.hint': function(e) {
                    this.children.firstTimeModal.show();
                    e.preventDefault();
                },
                'submit form': function(e) {
                    this.model.login.save({
                        cval: this.model.session.entry.content.get('cval'),
                        return_to: this.model.application.get('return_to'),
                        set_has_logged_in: !this.model.session.entry.content.get('hasLoggedIn')
                    });
                    e.preventDefault();
                }
            },
            show: function() {
                this.$el.show();
                this.children.username.focus();
            },
            render: function() {
                var html = this.compiledTemplate({
                    _: _,
                    model: {
                        application: this.model.application,
                        serverInfo: this.model.serverInfo,
                        session: this.model.session,
                        web: this.model.web,
                        login: this.model.login
                    },
                    route: route,
                    splunkutil: splunkutil
                });
                this.$el.html(html);
                this.children.username.render().insertBefore(this.$('input[type=submit]'));
                this.children.password.render().insertBefore(this.$('input[type=submit]'));
                this.children.firstTimeModal.render().insertBefore(this.$('fieldset'));
                this.children.error.render().appendTo(this.el);
                if (!this.model.session.entry.content.get('hasLoggedIn') && this.model.application.get('page')==='login') {
                   this.children.firstTimeModal.show();
                }
                if (this.children.updateChecker) {
                    this.children.updateChecker.render().appendTo(this.el);
                }
                return this;
            }
        });
    }
);
