define(
    [
        'module',
        'underscore',
        'splunk.util',
        'views/Base',
        'views/account/login/updatechecker/IFrame',
        'views/account/login/updatechecker/Error',
        'views/account/login/updatechecker/Loading',
        'views/account/login/updatechecker/Download'
    ],
    function(
        module,
        _,
        util,
        BaseView,
        IFrameView,
        ErrorView,
        LoadingView,
        DownloadView
    ) {
        return BaseView.extend({
            moduleId: module.id,
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                if (this.model.serverInfo.isFreeLicense()) {
                    this.children.loading = new LoadingView();
                    this.children.download = new DownloadView({
                        model: {
                            application: this.model.application,
                            session: this.model.session,
                            login: this.model.login
                        }
                    });
                }
                this.children.iframe = new IFrameView({
                    model: {
                        updateChecker: this.model.updateChecker,
                        serverInfo: this.model.serverInfo
                    }
                });
                this.children.error = new ErrorView({
                    model: {
                        serverInfo: this.model.serverInfo,
                        application: this.model.application,
                        session: this.model.session,
                        login: this.model.login
                    }
                });
                this.listenTo(this.model.updateChecker, 'error', function() {
                    this.children.iframe.$el.hide();
                    if (this.model.serverInfo.isFreeLicense()) {
                        this.children.loading.$el.hide();
                        this.children.download.$el.hide();
                    }
                    this.children.error.$el.show();
                });
                this.listenTo(this.model.updateChecker, 'sync', function() {
                    this.children.iframe.$el.show();
                    if (this.model.serverInfo.isFreeLicense()) {
                        this.children.loading.$el.hide();
                        if (this.model.updateChecker.get('hasNewVersion')) {
                            this.children.download.$el.show();
                        }
                    }
                    this.children.error.$el.hide();
                });
            },
            render: function() {
                if (this.model.serverInfo.isFreeLicense() && util.normalizeBoolean(this.model.web.entry.content.get('updateCheckerBaseURL'))) {
                    this.children.loading.render().appendTo(this.el);
                }
                this.children.iframe.render().appendTo(this.el);
                this.children.error.render().appendTo(this.el).$el.hide();
                if (this.model.serverInfo.isFreeLicense()) {
                    this.children.download.render().appendTo(this.el).$el.hide();
                }
                return this;
            }
        });
    }
);
