/**
 * Last step in the workflow: Success.
 * Shows links where to go next.
 *
 */
define(
    [
        'underscore',
        'module',
        'views/Base',
        'uri/route',
        'contrib/text!views/add_data/SuccessScreen.html'
    ],
    function (
        _,
        module,
        BaseView,
        route,
        template
    ) {
        /**
         */
        return BaseView.extend({
            template: template,
            moduleId: module.id,

            initialize: function (options) {
                BaseView.prototype.initialize.apply(this, arguments);

                // unsetting inputmode here, so that clicking browser's Back would takes us to the initial step
                this.model.wizard.unset('inputMode');
                this.model.wizard.trigger('disableWizardSteps');

                var searchString = this.generateSearchString();

                this.dataInputsLink = route.manager(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    this.model.application.get('app'),
                    'datainputstats'
                );

                this.searchLink = route.search(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    this.model.application.get('app'),
                    {
                        data: {
                              q: searchString
                        }
                    }
                );

                this.searchHelpLink = route.docHelp(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    'learnmore.adddata.success.search'
                );

                this.addDataLink = route.manager(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    this.model.application.get('app'),
                    'adddata'
                );

                this.addDataHelpLink = route.docHelp(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    'learnmore.adddata.datatutorial'
                );

                this.extractFieldsLink = route.field_extractor(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    this.model.input.get('appContext'),
                    {
                        data: {
                            sourcetype: this.model.input.get('ui.sourcetype')
                        }
                    }
                );

                this.fieldsHelpLink = route.docHelp(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    'manager.fields'
                );

                this.downloadAppsLink = route.manager(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    this.model.application.get('app'),
                    ['apps', 'remote']
                );

                this.downloadAppsHelpLink = route.docHelp(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    'learnmore.recipes.apps'
                );

                this.buildDashboardsLink = route.page(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    this.model.application.get('app'),
                    'dashboards'
                );

                this.buildDashboardsHelpLink = route.docHelp(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    'learnmore.dashboards'
                );

            },

            getOsSep: function() {
                return (this.model.serverInfo.getOsName().toLowerCase() === 'windows') ? '\\\\' : '/';
            },

            generateSearchString: function() {
                var searchString = [];
                var index = this.model.input.get('ui.index');
                var source = this.model.input.get('ui.name');
                var sourceOverride = this.model.input.get('ui.source');
                var host = this.model.input.get('ui.host');
                var sourcetype = this.model.input.get('ui.sourcetype');
                var inputType = this.model.wizard.get('inputType');
                var isDirectory = this.model.wizard.isDirectory();
                var isArchive = this.model.wizard.get('isArchive');

                if (source) {
                    if (inputType === 'tcp' || inputType === 'udp') {
                        if (sourceOverride) {
                            source = sourceOverride;
                        } else {
                            source = inputType + ':' + source;
                        }
                    } else if (inputType === 'scripts') {
                        if (sourceOverride) {
                            source = sourceOverride;
                        }
                    } else if (inputType === 'file_monitor' || inputType === 'file_oneshot' || inputType === 'file_upload') {
                        source = source.replace(/\\/g, "\\\\");
                        if (isDirectory) {
                            source = source + this.getOsSep() + '*';
                        } else if (isArchive) {
                            source = source + ':*';
                        }
                    } else if (inputType === 'evt_logs_local' || inputType === 'evt_logs_remote') {
                        source = "WinEventLog:*";
                    } else if (inputType === 'perfmon_local') {
                        source = "Perfmon:" + source;
                    } else if (inputType === 'perfmon_remote') {
                        source = "WMI:" + source;
                    } else if (inputType === 'hostmon') {
                        sourcetype = 'WinHostMon';
                    } else if (inputType === 'printmon') {
                        sourcetype = 'WinPrintMon';
                    } else if (inputType === 'netmon') {
                        sourcetype = 'WinNetMon';
                    } else if (inputType === 'regmon') {
                        source = 'WinRegistry';
                        sourcetype = 'WinRegistry';
                    }
                    searchString.push('source="'+source+'"');

                    if (host && !this.model.wizard.isForwardMode()) {
                        if ( ((inputType === 'file_monitor' || inputType === 'file_oneshot' || inputType === 'file_upload') && this.model.input.get('hostSwitch') !== 'constant') ||
                             ((inputType === 'tcp' || inputType === 'udp') && this.model.input.get('ui.connection_host') !== 'none') ) {
                            // skip adding host value for non-deterministic overrides
                        } else {
                            searchString.push('host="'+host+'"');
                        }
                    }
                } else {
                    return '';
                }

                if (index && index !== 'default') {
                    searchString.push('index="'+index+'"');
                }
                if (sourcetype && sourcetype !== 'default') {
                    searchString.push('sourcetype="'+sourcetype+'"');
                }

                searchString = searchString.join(' ');
                return searchString;
            },

            generateSuccessMessage: function() {
                var inputTypeLabel = {
                    'file_upload':     _("File").t(),
                    'file_monitor':    _("File").t(),
                    'file_oneshot':    _("File").t(),
                    'tcp':             this.model.input.get('sourceSwitchPort') === 'tcp' ? _("TCP").t() : _("UDP").t(),
                    'udp':             _("UDP").t(),
                    'scripts':         _("Script").t(),
                    'evt_logs_local':  _("Local event logs").t(),
                    'evt_logs_remote': _("Remote event logs").t(),
                    'perfmon_local':   _("Local performance monitoring").t(),
                    'perfmon_remote':  _("Remote performance monitoring").t(),
                    'regmon':          _("Registry monitoring").t(),
                    'admon':           _("Active directory monitoring").t(),
                    'hostmon':         _("Host monitoring").t(),
                    'netmon':          _("Network monitoring").t(),
                    'printmon':        _("Print monitoring").t()
                }[this.model.wizard.get('inputType')];
                if (this.model.wizard.get('isModularInput')) {
                    inputTypeLabel = _('Modular').t();
                }

                var ending = _(' input has been created successfully.').t();
                if (this.model.wizard.get('inputType') === 'file_upload') {
                    ending = _(' has been uploaded successfully.').t();
                }
                return inputTypeLabel + ending;
            },

            controlVisibility: function() {
                var sourcetype = this.model.input.get('ui.sourcetype');
                if (_.isUndefined(sourcetype) || sourcetype == 'default') {
                    this.$('.extract-fields').hide();
                } else {
                    this.$('.extract-fields').show();
                }
                if (this.model.serverInfo.isLite()) {
                    this.$('.download-apps').hide();
                }
            },

            render: function () {
                var template = this.compiledTemplate({
                    dataInputsLink: this.dataInputsLink,
                    addDataLink: this.addDataLink,
                    addDataHelpLink: this.addDataHelpLink,
                    searchLink: this.searchLink,
                    searchHelpLink: this.searchHelpLink,
                    downloadAppsLink: this.downloadAppsLink,
                    downloadAppsHelpLink: this.downloadAppsHelpLink,
                    buildDashboardsLink: this.buildDashboardsLink,
                    buildDashboardsHelpLink: this.buildDashboardsHelpLink,
                    extractFieldsLink: this.extractFieldsLink,
                    fieldsHelpLink: this.fieldsHelpLink,
                    inputTypeSuccessMessage: this.generateSuccessMessage()
                });
                this.$el.html(template);
                this.controlVisibility();
                return this;
            }
        });
    }
);