define(function(require, exports, module) {

    var $ = require("jquery");
    var _ = require("underscore");
    var BaseView = require("views/Base");
    var EmptyTile = require("views/authentication_users/EmptyTile");
    var UserTile = require("views/authentication_users/UserTile");

    return BaseView.extend({

        moduleId: module.id,

        initialize: function() {
            BaseView.prototype.initialize.apply(this, arguments);

            this.children.tiles = [];

            this.activate();
        },

        startListening: function() {
            this.listenTo(this.collection.users, "reset add remove", this.debouncedRender);
        },

        render: function() {
            // destroy old tiles
            _(this.children.tiles).each(function(tile) {
                tile.remove();
            }, this);

            // create tiles for each user
            this.children.tiles = this.collection.users.map(function(user) {
                return new UserTile({
                    model: {
                        user: user
                    }
                });
            }, this);

            // create empty tile if admin and less than maxUsers
            var maxUsers = this.model.serverInfo.entry.content.get("max_users");
            if (this.model.user.hasRole("admin") && (this.children.tiles.length < maxUsers)) {
                this.children.tiles.push(new EmptyTile({}));
            }

            // render tiles
            _(this.children.tiles).each(function(tile) {
                tile.render().appendTo(this.$el);
            }, this);

            // render highlighter if needed
            if (this._highlighter) {
                this.showHighlighter(this._highlighterUsername);
            }

            return this;
        },

        showHighlighter: function(username) {
            this.hideHighlighter();

            // the highlighter design doesn't work well with more than five tiles, so disable in that case
            if (this.children.tiles.length > 5) {
                return;
            }

            var highlightIndex = -1;
            if (username) {
                this.collection.users.find(function(user, index) {
                    if (user.entry.get("name") === username) {
                        highlightIndex = index;
                        return true;
                    }
                });
            } else {
                highlightIndex = this.children.tiles.length - 1;
            }

            if (highlightIndex < 0) {
                return;
            }

            this._highlighterUsername = username;
            this._highlighter = $("<div></div>")
                .addClass("highlighter")
                .css({ left: (60 + 180 * highlightIndex) + "px" })
                .appendTo(this.$el);
        },

        hideHighlighter: function() {
            if (this._highlighter) {
                this._highlighter.remove();
                this._highlighter = null;
                this._highlighterUsername = null;
            }
        }

    });

});
