define(
    [
        'module',
        'views/Base',
        'views/clustering/config/Master',
        'views/clustering/push/Master',
        'views/clustering/disable/Master',
        'views/shared/delegates/Popdown',
        'contrib/text!views/clustering/EditMenu.html'
    ],
    function(
        module,
        BaseView,
        ConfigDialog,
        PushConfig,
        DisableClustering,
        PopdownView,
        EditMenuTemplate
        )
    {
        return BaseView.extend({
            moduleId: module.id,
            className: 'btn-combo',
            template: EditMenuTemplate,
            /**
             * @param {Object} options {
             *     model: <models.Application>
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                this.children.popdown = new PopdownView({
                    el: this.el
                });
            },
            events: {
                'click a#nodeType': function(e) {
                    this.children.configDialog = new ConfigDialog({
                        model: this.model,
                        onHiddenRemove: true
                    });
                    $('body').append(this.children.configDialog.render().el);
                    this.children.configDialog.show();
                    e.preventDefault();
                },
                'click a#masterConfig': function(e) {
                    this.children.configDialog = new ConfigDialog({
                        model: this.model,
                        startView: 'master',
                        onHiddenRemove: true
                    });
                    $('body').append(this.children.configDialog.render().el);
                    this.children.configDialog.show();
                    e.preventDefault();
                },
                'click a#peerConfig': function(e) {
                    this.children.configDialog = new ConfigDialog({
                        model: this.model,
                        startView: 'peer',
                        onHiddenRemove: true
                    });
                    $('body').append(this.children.configDialog.render().el);
                    this.children.configDialog.show();
                    e.preventDefault();
                },
                'click a#disableCluster': function(e) {
                    this.children.disableClustering = new DisableClustering({
                        model: this.model,
                        onHiddenRemove: true
                    });
                    $('body').append(this.children.disableClustering.render().el);
                    this.children.disableClustering.show();
                    e.preventDefault();
                }
            },
            render: function() {
                var html = this.compiledTemplate({
                    mode: this.model.clusterConfig.entry.content.get('mode'),

                    // on MASTER NODE, 'multisite = true' means multisite mode is enabled.
                    // on PEER NODE or SEARCHHEAD NODE, 'site' presents AND 'site != default' means multisite mode is enabled.

                    // we use this attribute to control whether enable or disable 'edit' menu on MASTER NODE.
                    // This is just for temporary purpose, because the 'edit' menu is not ready for multisite mode.
                    multisite: this.model.clusterConfig.entry.content.get('multisite')

                    // we use this attribute to control whether enable or disable 'edit' menu on PEER NODE.
                    // This is just for temporary purpose, because the 'edit' munu is not ready for multisite mode.
                    // site: this.model.clusterConfig.entry.content.get('site')
                });
                this.$el.html(html);
                return this;
            }
        });
    }
);
