define([
    'jquery',
    'underscore',
    'backbone',
    'module',
    'collections/services/Messages',
    'views/shared/Modal',
    'views/clustering/config/NodeType',
    'views/clustering/config/MasterSetup',
    'views/clustering/config/PeerSetup',
    'views/clustering/config/SearchHeadSetup',
    'views/clustering/config/Restart' ,
    'uri/route',
    'util/console'
],
    function(
        $,
        _,
        Backbone,
        module,
        MessageCollection,
        Modal,
        NodeType,
        MasterSetup,
        PeerSetup,
        SearchSetup,
        Restart,
        route,
        console
        ) {
        return Modal.extend({
            moduleId: module.id,
            className: Modal.CLASS_NAME,
            /**
             * @param {Object} options {
        *       model: <models.>,
        *       collection: <collections.services.>
        * }
             */
            initialize: function(options) {
                Modal.prototype.initialize.apply(this, arguments);
                this.wizardModel = new Backbone.Model();
                this.messages = new MessageCollection();
                /*
                 WizardModel is the bus for all the child pages (memebers of this.children).
                  startPage: the page shown on the first load of the dialog
                  currentPage: currently displayed child page
                 Events:
                  next: go to the next page
                  back: go one page back
                 */

                this.model = $.extend({}, this.model, {wizard: this.wizardModel});
                this.children.start = new NodeType({
                    model: this.model
                });
                this.children.master = new MasterSetup({
                    model: this.model
                });
                this.children.peer = new PeerSetup({
                    model: this.model
                });
                this.children.search = new SearchSetup({
                    model: this.model
                });
                this.children.restart = new Restart({
                    model: this.model
                });

                if (this.options && this.options.startView) {
                    this.wizardModel.set('startPage', this.options.startView);
                } else {
                    this.wizardModel.set('startPage', 'start');
                }

                this.wizardModel.set('currentPage', this.wizardModel.get('startPage'));
                this.wizardModel.on('next', function() {
                    var pageId = this.wizardModel.get('currentPage'),
                        page = this.children.hasOwnProperty(pageId)? this.children[pageId] : null;
                    if (pageId === 'start') {
                        var nextPageId = this.wizardModel.get('mode'),
                            nextPage = this.children.hasOwnProperty(nextPageId)? this.children[nextPageId] : null;
                        if (nextPage) {
                            page.$el.hide();
                            nextPage.$el.show();
                            this.wizardModel.set('currentPage', nextPageId);
                        } else {
                            console.error('Cannot navigate to '+nextPageId );
                        }
                    } else if (_.contains(['master','peer','search'], pageId)) {
                        this.messages.fetch().done(function() {
                            var restartRequired = this.messages.find(function(i) {
                                return i.entry.get('name')=='restart_required';
                            });
                            if (restartRequired) {
                                this.wizardModel.set('currentPage', 'restart');
                                page.$el.hide();
                                this.children.restart.$el.show();
                            } else {
                                this.hide();
                            }
                        }.bind(this));
                    }
                }, this);

                this.wizardModel.on('back', function() {
                    this.refreshConfig();
                    var pageId = this.wizardModel.get('currentPage'),
                        page = this.children.hasOwnProperty(pageId)? this.children[pageId] : null;
                    this.wizardModel.set('currentPage', 'start');
                    page.$el.hide();
                    this.children.start.$el.show();
                }, this);

                this.wizardModel.on('restart', function() {
                    var return_to = route.manager(
                        this.model.application.get('root'),
                        this.model.application.get('locale'),
                        this.model.application.get('app'),
                        'clustering');

                    window.location.href = route.manager(
                        this.model.application.get('root'),
                        this.model.application.get('locale'),
                        this.model.application.get('app'),
                        'control', {
                            data: {
                                return_to: return_to,
                                auto_restart: 1
                            }
                        });
                }, this);

            },
            refreshConfig: function() {
                this.model.clusterConfig.fetch().done(function() {
                    this.model.clusterConfig.transposeFromRest();
                }.bind(this));
            },
            render: function() {
                this.refreshConfig();
                this.$el.append(this.children.start.render().el);
                this.$el.append(this.children.master.render().el);
                this.$el.append(this.children.peer.render().el);
                this.$el.append(this.children.search.render().el);
                this.$el.append(this.children.restart.render().el);
                var v;
                for (v in this.children) {
                    if (this.children.hasOwnProperty(v)) {
                        if (v === this.wizardModel.get('startPage')) {
                            this.children[v].$el.show();
                        } else {
                            this.children[v].$el.hide();
                        }
                    }
                }
                return this;
            }
        });
    });
