define([
    'underscore',
    'module',
    'backbone',
    'views/shared/Modal',
    'views/clustering/disable/DisableClustering',
    'views/clustering/disable/Restart',
    'uri/route'
],
    function(
        _,
        module,
        Backbone,
        Modal,
        DisableClustering,
        Restart,
        route
        ) {
        return Modal.extend({
            moduleId: module.id,
            className: Modal.CLASS_NAME,
            /**
             * @param {Object} options {
        *       model: <models.>,
        *       collection: <collections.services.>
        * }
             */
            initialize: function(options) {
                Modal.prototype.initialize.apply(this, arguments);
                var that = this;
                this.wizardModel = new Backbone.Model();
                this.wizardModel.on('disable', function() {
                    this.model.clusterConfig.entry.content.set('mode', 'disabled');
                    this.model.clusterConfig.save({
                        wait: true
                    }).done(function(){
                        that.children.disableClustering.$el.hide();
                        that.children.restart.$el.show();
                    });
                }, this);

                this.wizardModel.on('restart', function() {
                    var return_to = route.manager(
                        this.model.application.get('root'),
                        this.model.application.get('locale'),
                        this.model.application.get('app'),
                        'clustering');
                    window.location.href = route.manager(
                        this.model.application.get('root'),
                        this.model.application.get('locale'),
                        this.model.application.get('app'),
                        'control', {
                            data: {
                                return_to: return_to,
                                auto_restart: 1
                            }
                        });
                }, this);

                this.model.wizardModel = this.wizardModel;
                this.children.disableClustering = new DisableClustering({
                    model: this.model
                });
                this.children.restart = new Restart({
                    model: this.model
                });
            },

            render: function() {
                this.$el.append(this.children.disableClustering.render().el);
                this.$el.append(this.children.restart.render().el);
                this.children.disableClustering.$el.show();
                this.children.restart.$el.hide();
                return this;
            }
        });
    });
