/**
 * @author sfishel
 * 
 * The master view for the "data_model_explorer" page.
 *
 * Child Views:
 * 
 * objectList <views/data_model_explorer/ObjectGrid> the grid display of objects in the data model
 *
 * Custom Events:
 *
 * action:goBack - triggered when the user should be navigated back to the previous interface
 * action:selectObject - triggered when an object has been selected from the list
 *     @param objectName {String} the name of the selected object
 */

define([
            'underscore',
            'module',
            'collections/shared/FlashMessages',
            'views/Base',
            'views/shared/FlashMessages',
            'views/data_model_explorer/ObjectGrid',
            'uri/route',
            'models/classicurl'
        ],
        function(
            _,
            module,
            FlashMessagesCollection,
            BaseView,
            FlashMessagesView,
            ObjectGrid,
            route,
            classicUrl
        ) {
    
    return BaseView.extend({
        
        moduleId: module.id,

        /**
         * @constructor
         * @param options {
         *    model: {
         *        dataModel: <models/services/datamodel/DataModel> the data model being explored
         *        application: <models/shared/Application> the application state model
         *    }
         *    collection: {
         *        dataModels <collections/services/datamodel/DataModels> the list of available data models
         *    }
         */
        
        initialize: function(options) {
            BaseView.prototype.initialize.call(this, options);
            this.children.flashMessages = new FlashMessagesView({ 
                model: {
                    dataModel: this.model.dataModel
                }     
            });
            
            this.children.objectList = new ObjectGrid({
                model: {
                    dataModel: this.model.dataModel,
                    application: this.model.application
                }
            });

            var that = this;

            this.model.dataModel.on('sync', function() {
                var editButton = this.$('.edit-objects-button');
                this.model.dataModel.entry.acl.canWrite() ? editButton.show() : editButton.hide();
            }, this);
        },
        
        render: function() {
            var pageRouter = route.getContextualPageRouter(this.model.application),
                editObjectsHref = pageRouter.data_model_editor({ data: { model: this.model.dataModel.id } }),
                backButtonHref = pageRouter.pivot(),
                html = _(this.template).template({
                    editObjectsHref: editObjectsHref,
                    backButtonHref: backButtonHref
                });

            this.$el.html(html);
            this.$('.flash-messages-placeholder').replaceWith(this.children.flashMessages.render().el);
            this.$('.object-list-placeholder').replaceWith(this.children.objectList.render().el);
            if(!this.model.dataModel.entry.acl.canWrite()) {
                this.$('.edit-objects-button').hide();
            }

            this.$("a.data-models-back-button").click(function(e) {
                e.preventDefault();
                classicUrl.unset("model").save({}, { trigger: true });
            });

            return this;
        },
        
        template: '\
            <div class="section-padded section-header">\
            	<h2 class="section-title">\
                    <%- _("Select a Data Object").t() %>\
                </h2>\
                <a href="<%- editObjectsHref %>" class="edit-objects-button btn">\
                    <%- _("Edit Objects").t() %>\
                </a>\
                <a class="data-models-back-button" href="<%- backButtonHref %>">\
                    <i class="icon-chevron-left icon-no-underline"></i><span><%- _("Back").t() %></span>\
                </a>\
                <div class="flash-messages-placeholder"></div>\
            </div>\
            <div class="object-list-placeholder"></div>\
        '
        
    });
    
});
