/**
 * @author sfishel
 *
 * Master view for the "data_models" page
 *
 * Child Views:
 *
 * dmList <views/data_model/DataModelGrid> a grid view that displays this list of available model
 *
 * Custom Events:
 *
 * action:selectModel - triggered when a data model has been selected from the list
 *     @param modelName {String} the name of the selected data model
 */

define(
        [
            'underscore',
            'module',
            'collections/shared/FlashMessages',
            'views/Base',
            'views/shared/FlashMessages',
            'views/data_models/DataModelGrid',
            'uri/route',
            'views/shared/CollectionPaginator'
        ],
        function(
            _,
            module,
            FlashMessagesCollection,
            BaseView,
            FlashMessagesView,
            DataModelGrid,
            route,
            CollectionPaginator
        ) {

    return BaseView.extend({

        moduleId: module.id,

        /**
         * @constructor
         * @param options {Object} {
         *     collection: <collections/services/datamodel/DataModels> the collection of data models
         *     model: {
         *         application: <models/shared/Application> the application state model
         *     }
         * }
         */

        initialize: function(options) {
            BaseView.prototype.initialize.call(this, options);

            this.children.flashMessages = new FlashMessagesView({
                collection: this.collection
            });
            this.children.dmList = new DataModelGrid({
                collection: this.collection,
                model: {
                    application: this.model.application
                }
            });
            this.children.collectionPaginator = new CollectionPaginator({
                collection: this.collection,
                model: this.collection.fetchData
            });
        },

        render: function() {
            var pageRouter = route.getContextualPageRouter(this.model.application),
                manageModelsHref = pageRouter.data_model_manager(),
                html = _(this.template).template({ manageModelsHref: manageModelsHref });

            this.$el.html(html);
            this.$('.flash-messages-placeholder').replaceWith(this.children.flashMessages.render().el);
            this.$('.dm-list-placeholder').replaceWith(this.children.dmList.render().el);
            this.$('.collection-paginator-placeholder').append(this.children.collectionPaginator.render().el);
            return this;
        },

        template: '\
            <div class="section-padded section-header">\
                <h2 class="section-title"><%- _("Select a Data Model").t() %></h2>\
                <a href="<%- manageModelsHref %>" class="manage-models-button btn">\
                    <%- _("Manage Data Models").t() %>\
                </a>\
                <div class="flash-messages-placeholder"></div>\
            </div>\
            <div class="pull-right collection-paginator-placeholder"></div>\
            <div class="dm-list-placeholder"></div>\
        '

    });

});
