define(
[
    'jquery',
    'underscore',
    'backbone',
    'module',
    'views/Base',
    'views/shared/DynamicFields',
    'views/datapreview/settings/CopyToClipboardDialog',
    'contrib/text!views/datapreview/settings/Advanced.html'
],
function(
    $,
    _,
    Backbone,
    module,
    BaseView,
    DynamicFieldsView,
    CopyToClipboardDialog,
    advancedTemplate
){
    return BaseView.extend({
        moduleId: module.id,
        template: advancedTemplate,
        initialize: function(){
            BaseView.prototype.initialize.apply(this, arguments);
            var self = this;
            this.intermediateModel = new Backbone.Model();

            this.model.sourcetypeModel.entry.content.on('change', function(sourcetypeModel, options){
                if(options && options.setby === 'intermediateevent'){
                    return;
                }
                self.setIntermediateModel.call(self);
            });

        },
        setSourcetypeModel: function(){
            this.model.sourcetypeModel.entry.content.set(this.intermediateModel.attributes, {setby: 'intermediateevent'});
        },
        setIntermediateModel: function(){
            this.intermediateModel.clear({silent: true});
            this.intermediateModel.set(this.model.sourcetypeModel.getExplicitProps(), {setby: 'intermediateevent'});
        },
        events: {
            //If user clicks "copy to clipboard", initialize copyToClipboardDialog
            //to display a modal with the sourcetype name stanza and props.conf name/value pairs in a text field
            'click .copyToClipboard' : function(e) {
                e.preventDefault();
                this.children.clipboardDialog = new CopyToClipboardDialog({
                        model: {
                            intermediateModel: this.intermediateModel,
                            sourcetypeModel: this.model.sourcetypeModel
                        },
                        onHiddenRemove: true
                    });
                this.children.clipboardDialog.render().appendTo($("body"));
                this.children.clipboardDialog.show();
            },
            'click .applySettings' : function(e) {
                e.preventDefault();
                this.setSourcetypeModel();
            }
        },
        render: function() {
            this.$el.html(this.compiledTemplate({_:_}));

            if (this.children.dynamicFields)
                this.children.dynamicFields.detach();

            this.children.dynamicFields = new DynamicFieldsView({
                model: this.intermediateModel,
                hideEmpty: true
            });
            this.$('.advanced').append(this.children.dynamicFields.render().el);

            return this;
        }
    });
});
