define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/datapreview/settings/Timestamp',
        'views/datapreview/settings/EventBreaks',
        'views/datapreview/settings/Fields',
        'views/datapreview/settings/Advanced',
        'contrib/text!views/datapreview/settings/AccordionGroup.html',
        'bootstrap.collapse' //NO IMPORT
    ],
    function(
        _,
        module,
        BaseView,
        TimestampView,
        EventBreaksView,
        FieldsView,
        AdvancedView,
        AccordionGroupTemplate
    ){
        return BaseView.extend({
            moduleId: module.id,
            className: 'accordion datapreview-settings',
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                var self = this;
                self.children.settingsPanels = {};

                this.children.settingsPanels.eventBreaksView = new EventBreaksView({
                    heading: 'Event Breaks',
                    model: this.model,
                    collection: this.collection
                });

                this.children.settingsPanels.timestamp = new TimestampView({
                    heading: 'Timestamp',
                    model: this.model,
                    collection: this.collection
                    //mode: this.model.sourcetypeModel.constructor.TABULAR
                });

                this.children.settingsPanels.fields = new FieldsView({
                    heading: 'Delimited settings',
                    model: this.model,
                    collection: this.collection
                });

                this.children.settingsPanels.advanced = new AdvancedView({
                    heading: 'Advanced',
                    model: this.model,
                    collection: this.collection
                    //mode: type
                });

                this.model.sourcetypeModel.on('sync', function(){
                    self.setPanels.call(self);
                });
            },
            setPanels: function(){
                var type = this.model.sourcetypeModel.getDataFormat();
                switch(type){
                    case this.model.sourcetypeModel.constructor.UNSTRUCTURED:
                       this.prepareUnstructured();
                    break;
                    case this.model.sourcetypeModel.constructor.HIERARCHICAL:
                       this.prepareHierarchical();
                    break;
                    case this.model.sourcetypeModel.constructor.TABULAR:
                       this.prepareTabular();
                    break;
                    default:
                        this.prepareUnstructured();
                }
            },

            events: {
                'click .accordion-heading a': function(e) {
                    if ( $(e.target).closest(".accordion-heading").next().hasClass("in") )  {
                        $(e.target).find(".icon-accordion-toggle").removeClass("icon-triangle-down-small").addClass("icon-triangle-right-small").closest('.accordion-group').removeClass('active');
                    } else {
                        $(e.target).find(".icon-accordion-toggle").removeClass("icon-triangle-right-small").addClass("icon-triangle-down-small").closest('.accordion-group').addClass('active');
                    }
                    e.preventDefault();
                }
            },

            render: function() {
                _.each(this.children.settingsPanels, function(panel) {
                    panel.detach();
                }, this);
                this.$el.html('');
                var i = 0;
                _.each(this.children.settingsPanels, function(panel) {
                    var accordionGroup = $(_.template(AccordionGroupTemplate, {
                        heading: panel.options.heading || '',
                        index: i++
                    }));
                    accordionGroup.find('.accordion-inner').append(panel.render().el);
                    accordionGroup.appendTo(this.$el);
                }, this);

                this.setPanels.call(this);

                return this;
            },
            prepareUnstructured: function(){
                this.children.settingsPanels.eventBreaksView.$el.closest('.accordion-group').show();
                this.children.settingsPanels.timestamp.$el.closest('.accordion-group').show();
                this.children.settingsPanels.fields.$el.closest('.accordion-group').hide();
                this.children.settingsPanels.advanced.$el.closest('.accordion-group').show();
            },
            prepareHierarchical: function(){
                this.children.settingsPanels.eventBreaksView.$el.closest('.accordion-group').hide();
                this.children.settingsPanels.timestamp.$el.closest('.accordion-group').show();
                this.children.settingsPanels.fields.$el.closest('.accordion-group').hide();
                this.children.settingsPanels.advanced.$el.closest('.accordion-group').show();
            },
            prepareTabular: function(){
                this.children.settingsPanels.eventBreaksView.$el.closest('.accordion-group').hide();
                this.children.settingsPanels.timestamp.$el.closest('.accordion-group').show();
                this.children.settingsPanels.fields.$el.closest('.accordion-group').show();
                this.children.settingsPanels.advanced.$el.closest('.accordion-group').show();
            }
        });
    }
);
