define(
    [
        'module',
        'views/Base',
        'views/shared/JSChart',
        'views/shared/SingleValue',
        'views/shared/results_table/LazyResultsTable',
        'views/shared/eventsviewer/LazyEventsViewer',
        'views/shared/LazyMap',
        'models/Base',
        'collections/Base',
        'splunk.config',
        'splunk.util'
    ],
    function(
        module,
        BaseView,
        JSChartView,
        SingleValueView,
        LazyResultsTable,
        LazyEventsViewerView,
        LazyMapView,
        BaseModel,
        BaseCollection,
        splunkConfig,
        splunkUtils) {
        return BaseView.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     {
             *     model: {
             *         application: <models.shared.Application>,
             *         job: <models.search.Job>,
             *         report: <models.search.Report>,
             *         result: <models.services.search.jobs.Result>,
             *         resultJSONCols: <models.services.search.jobs.ResultJsonCols>,
             *         resultJSONRows: <models.services.search.jobs.ResultJsonRows>,
             *         summary: <models.services.search.jobs.Summary>,
             *         mapState: <models.Base>
             *     },
             *     collection: {
             *         selectedFields: <collections.search.SelectedFields>,
             *         eventRenderers: <collections.services.configs.EventRenderers>,
             *         workflowActions: <collections.services.data.ui.WorkflowActions>
             *     }
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                this.activate();
            },
            startListening: function() {
            	this.listenTo(this.model.job, 'prepared', this.render);
            },
            updateMapState: function() {
            	var mapView = this.children.display.wrappedView,
            		attributes = {
	            		count: mapView.getMaxClusters(),
	            		search: mapView.getPostProcessSearch()
            		};
            	this.model.mapState.set(attributes);
            },
            render: function() {
                this.removeChildren();
                if (this.model.job.isNew() || this.model.job.isPreparing()) {
                    return this;
                }
                var displayVisualizationsType = this.model.report.entry.content.get('display.visualizations.type');
                if (this.model.job.isReportSearch()) {
                    if (splunkUtils.normalizeBoolean(this.model.report.entry.content.get('display.visualizations.show'))) {
                    	if (displayVisualizationsType === 'charting') {
                            //jschart
                            this.children.display = new JSChartView({
                                model: {
                                    searchResultsColumn: this.model.resultJSONCols,
                                    state: this.model.report.entry.content,
                                    application: this.model.application
                                },
                                width: '100%',
                                height: this.model.report.entry.content.get('display.visualizations.chartHeight')
                            });
                        } else if (displayVisualizationsType === 'mapping') {
                            //maps
                        	this.children.display = new LazyMapView({
                                model: {
                                    searchResultsColumn: this.model.resultJSONCols,
                                    state: this.model.report.entry.content
                                },
                                width: '100%',
                                height: this.model.report.entry.content.get('display.visualizations.mapHeight')
                            });
                            this.children.display.load();
                        	this.children.display.on('boundsChanged', this.updateMapState, this);
                        } else {
                            //singlevalue
                            this.children.display = new SingleValueView({
                                model: {
                                    searchResultsColumn: this.model.resultJSONCols,
                                    state: this.model.report.entry.content
                                },
                                width: 800,
                                height: 300
                            });
                        }
                    } else {
                        //table
                        this.children.display = new LazyResultsTable({
                            model: {
                                results: this.model.resultJSONRows,
                                metadata: this.model.report.entry.content,
                                format: this.model.report.entry.content
                            },
                            sortKeyAttr: 'display.statistics.sortColumn',
                            sortDirAttr: 'display.statistics.sortDirection',
                            sortableFields: false,
                            countAttr: 'display.prefs.statistics.count',
                            offsetAttr: 'display.prefs.statistics.offset',
                            tableDockOffset: -1
                        });
                        this.children.display.load();
                    }
                } else {
                    this.children.display = new LazyEventsViewerView({
                        model: {
                            result: this.model.result,
                            summary: this.model.summary,
                            searchJob: this.model.job,
                            report: this.model.report,
                            application: this.model.application
                        },
                        collection: {
                            eventRenderers: this.collection.eventRenderers,
                            selectedFields: this.collection.selectedFields,
                            workflowActions: this.collection.workflowActions
                        },
                        selectableFields: false,
                        sortableFields: false,
                        headerMode: 'dock',
                        headerOffset: -1,
                        allowRowExpand: false,
                        allowModalize: false,
                        defaultDrilldown: false
                    });
                    this.children.display.load();
                }
                this.$el.append(this.children.display.activate({deep: true}).render().el);
                var html = this.compiledTemplate({splunkConfig: splunkConfig});
                this.$el.append(html);
                return this;
            },
            template: '\
                <% if (splunkConfig.EMBED_FOOTER) { %>\
                    <footer>\
                        <div>\
                        <% if (splunkConfig.EMBED_FOOTER.toLowerCase() === "splunk>") { %>\
                            <span class="icon-splunk"></span><span class="icon-greater"></span>\
                        <% } else { %>\
                            <%- splunkConfig.EMBED_FOOTER %>\
                        <% } %>\
                        </div>\
                    </footer>\
                <% } %>\
            '
        });
    }
);