define([
            'underscore',
            'jquery',
            'module',
            'views/Base',
            'util/field_extractor_utils',
            'util/keyboard',
            'uri/route'
        ],
        function(
            _,
            $,
            module,
            BaseView,
            fieldExtractorUtils,
            keyboardUtils,
            route
        ) {

    return BaseView.extend({

        moduleId: module.id,

        className: 'existing-extractions-list',

        /**
         * @constructor
         *
         * @param options {Object} {
         *     model: {
         *         application <models.shared.Application>
         *         sourcetypeModel <models.knowledgeobjects.Sourcetype>
         *     }
         *     collection: {
         *         extractions <collections.services.data.props.Extractions>
         *     }
         * }
         */

        initialize: function() {
            BaseView.prototype.initialize.apply(this, arguments);

            this.listenTo(this.collection.extractions, 'request', this.renderLoading);
            this.listenTo(this.collection.extractions, 'sync reset', this.render);
        },

        events: {
            'click .fields-settings-page-button': function(e) {
                e.preventDefault();
                this._handleViewFieldSettings();
            },
            'keydown': function(e) {
                if(e.which === keyboardUtils.KEYS.TAB) {
                    keyboardUtils.handleCircularTabbing(this.$el, e);
                }
            }
        },

        renderLoading: function() {
            this.$el.html(_(this.loadingTemplate).template({}));
        },

        _handleViewFieldSettings: function() {
            var url = route.manager(
                this.model.application.get('root'),
                this.model.application.get('locale'),
                this.model.application.get('app'),
                ['fields']
            );
            window.open(url, '_blank');
        },

        render: function() {
            this.$el.html(this.compiledTemplate({
                extractions: this.getExtractionData(),
                sourcetype: this.model.state.get('sourcetype'),
                generateEditHref: _(function(extraction) {
                    return route.field_extractor(
                        this.model.application.get('root'),
                        this.model.application.get('locale'),
                        this.model.application.get('app'),
                        { data: { extractionId: extraction.id } }
                    );
                }).bind(this)
            }));
            this.focus();
            return this;
        },

        focus: function() {
            this.$('.fields-settings-page-button').focus();
        },

        getExtractionData: function() {
            return this.collection.extractions
                .chain()
                .map(function(extraction) {
                    var extractionContent = extraction.entry.content,
                        extractedFields = fieldExtractorUtils.getCaptureGroupNames(extractionContent.get('value'));

                    return _(extractedFields).map(function(field) {
                        return ({
                            id: extraction.id,
                            field: field,
                            attribute: extractionContent.get('attribute')
                        });
                    });
                })
                .flatten()
                .value();
        },

        loadingTemplate: '\
            <div class="alert alert-info">\
                <i class="icon-alert"></i>\
                <%- _("Loading Fields...").t() %>\
            </div>\
        ',

        template: '\
            <h3><%- _("Fields").t() %></h3>\
            <div>\
                <%- _("Sourcetype:").t() %>\
                <span class="sourcetype-name"><%- sourcetype %></span>\
            </div>\
            <div class="extractions-sidebar-instructions">\
                <%- _("The field extractions below have been previously defined for this sourcetype. For a complete list of field objects, please see the ").t() %>\
                <a href="#" class="fields-settings-page-button"> <%- _("Fields page in settings").t() %> <i class="icon-external"></i></a>.\
            </div>\
            <table class="table table-striped">\
                <thead>\
                    <tr>\
                        <th><%- _("Field Name").t() %></th>\
                        <th><%- _("Pattern Name").t() %></th>\
                        <th><%- _("Action").t() %></th>\
                    </tr>\
                </thead>\
                <tbody>\
                    <% _(extractions).each(function(extraction) { %>\
                        <tr data-extraction-id="<%- extraction.id %>">\
                            <td><%- extraction.field %></td>\
                            <td><%- extraction.attribute %></td>\
                            <td>\
                                <a href="<%- generateEditHref(extraction) %>" class="edit-button" target="_blank">\
                                    <%- _("open").t() %>\
                                    <i class="icon-external"></i>\
                                </a>\
                            </td>\
                        </tr>\
                    <% }) %>\
                </tbody>\
            </table>\
        '

    });

});