define([
            'underscore',
            'module',
            'models/Base',
            'models/services/data/props/Extraction',
            'models/knowledgeobjects/Sourcetype',
            'models/search/Job',
            'collections/Base',
            'collections/services/authorization/Roles',
            'views/Base',
            './ManualExtractionEditor',
            './ExistingExtractionsList',
            './MasterEventViewer',
            './MasterEventEditor',
            './ExtractionViewer',
            './SaveExtractionsView',
            './CounterExampleEditor',
            './ConfirmationView',
            'views/shared/knowledgeobjects/SourcetypeMenu',
            'views/shared/dataenrichment/preview/RexPreviewViewStack',
            'views/shared/dataenrichment/preview/PreviewJobController',
            'views/shared/dataenrichment/preview/RegexGeneratorController',
            'views/shared/controls/SyntheticSelectControl',
            'views/shared/controls/StepWizardControl',
            'views/shared/Sidebar',
            'views/shared/FlashMessages',
            'splunk.util',
            'util/field_extractor_utils',
            'util/splunkd_utils',
            'uri/route'
        ],
        function(
            _,
            module,
            BaseModel,
            Extraction,
            Sourcetype,
            Job,
            BaseCollection,
            RolesCollection,
            BaseView,
            ManualExtractionEditor,
            ExistingExtractionsList,
            MasterEventViewer,
            MasterEventEditor,
            ExtractionViewer,
            SaveExtractionsView,
            CounterExampleEditor,
            ConfirmationView,
            SourcetypeMenu,
            RexPreviewViewStack,
            PreviewJobController,
            RegexGeneratorController,
            SyntheticSelectControl,
            StepWizardControl,
            Sidebar,
            FlashMessages,
            splunkUtils,
            fieldExtractorUtils,
            splunkdUtils,
            route
        ) {

    return BaseView.extend({

        moduleId: module.id,

        className: 'field-extractor-master',

        SIDEBAR_WIDTH: 350,
        SAMPLE_EVENT_LIMIT: 4,

        events: {
            'click .view-extractions-button' : function(e) {
                e.preventDefault();
                this._createSidebar();
            },
            'click .content-header .manual-mode-button': function(e) {
                e.preventDefault();
                this.setManualMode();
            },
            'click .return-automatic-mode': function(e) {
                e.preventDefault();
                this.cleanupManualMode();
            },
            'click .return-manual-page': function(e) {
                e.preventDefault();
                this.model.state.set({ mode: this.lastAutomaticMode });
                this.setupManualView();
                this.$('.return-manual-page').hide();
            },
            'click .content-header .preview-in-search-link': function(e) {
                e.preventDefault();
                if(!$(e.currentTarget).hasClass('disabled')) {
                    this._handlePreviewInSearch();
                }
            },
            'click .page-footer .next-step-button': function(e) {
                e.preventDefault();
                if(!$(e.currentTarget).hasClass('disabled')){
                    if(this.model.state.get('mode') === fieldExtractorUtils.SAVE_FIELDS_MODE) {
                        this._validateAndSaveExtraction();
                    }else{
                       this.model.state.set({ mode: this._getNextMode().get('value') }); 
                    }
                }
            }
        },
        setManualMode: function() {
            this.lastAutomaticMode = this.model.state.get('mode');
            this.model.state.set({ interactiveMode: fieldExtractorUtils.NO_INTERACTION_MODE });
            this.children.stepWizardControl.$el.hide();
            this.setupManualView();
        },
        setupManualView: function() {
            this.cleanUpOldMode(this.model.state.get('mode'));
            this.$('.page-footer').find('.next-step-button').hide();
            this._refreshExtractionViews();
            if(this.model.extraction.isNew() && (!this.regexManuallyModified() || !this.model.state.get('regex'))) {
                this.$('.return-automatic-mode').show();
                this.children.manualExtractionEditor.$('.edit-regex-warning').show();
                this.$('.regex-editor-wrapper').find('textarea').focus();
            }
        },
        cleanupManualMode: function(){
            this.model.state.set('interactiveMode', fieldExtractorUtils.INTERACTION_MODE);
            this.children.stepWizardControl.$el.show();
            this.$('.page-footer').find('.next-step-button').show();
            this.children.manualExtractionEditor.cleanupState();
            this.cleanupManualView();
            this._refreshExtractionViews();
        },
        cleanupManualView: function() {
            this.$('.select-sourcetype-header').detach();
            this.children.previewView.$el.detach().hide();
            this.children.manualExtractionEditor.$el.detach().hide();
            this.$('.return-automatic-mode').hide();
            this.children.manualExtractionEditor.$('.edit-regex-warning').hide();
        },
        cleanUpOldMode: function(mode) {
            if(mode === fieldExtractorUtils.SELECT_SOURCETYPE_MODE) {
                this.$('.select-sourcetype-header').detach();
                this.children.sourcetypeDropDown.$el.detach().hide();
                this.$('.content-body').show();
            }
            if(mode === fieldExtractorUtils.SELECT_SAMPLE_MODE) {
                this.$('.select-sample-header').detach();
                this.children.previewView.$el.detach().hide();
                this.children.masterEventViewer.detach();
            }
            if(mode === fieldExtractorUtils.SELECT_FIELDS_MODE) {
                this.$('.select-fields-header').detach();
                this.children.extractionViewer.detach();
                this.children.masterEventEditor.$el.detach().hide();
                this.children.previewView.$el.detach().hide();
                this.$('.body-instructions').detach();
                if (this.$('.content-body').is(':hidden')) {
                    this.$('.content-body').css("display", "block");
                }
            }
            if(mode === fieldExtractorUtils.VALIDATE_FIELDS_MODE) {
                this.$('.validate-fields-header').detach();
                this.children.previewView.$el.detach().hide();
                this.children.extractionViewer.detach();
                this.children.counterExampleEditor.$el.detach().hide();
            }
            if(mode === fieldExtractorUtils.SAVE_FIELDS_MODE) {
                this.$('.save-extractions-header').detach();
                this.children.saveView.remove();
                this.$('.content-body').show();
                // When leaving the save mode, clear any errors that ocurred trying to save the extraction (SPL-89627).
                this.model.extraction.trigger('serverValidated', false, this.model.extraction);
                this.model.extraction.acl.trigger('serverValidated', false, this.model.extraction.acl);
            }
            if(mode === fieldExtractorUtils.CONFIRMATION_MODE) {
                this.$('.confirmation-header').detach();
                this.children.confirmationView.$el.detach().hide();
            }
        },
        _createSidebar: function() {
            if(this.children.extractionsSidebar) {
                this.children.extractionsSidebar.remove();
            }
            this.children.extractionsSidebar = new Sidebar({ modalize : true });
            this.children.extractionsSidebar.render().$el.appendTo($('body'));
            this.children.extractionsSidebar.addSidebar(this.children.existingExtractions.render().$el.css({'width' : this.SIDEBAR_WIDTH + 'px'}));
            // The sidebar clobbers DOM event listeners when it gets removed, so refresh them (SPL-88877).
            this.children.existingExtractions.delegateEvents();
            var extractionSearch = 'type=inline AND stanza=' + this.model.state.get('sourcetype');
            if(this.collection.extractions.fetchData.get('search') !== extractionSearch) {
                this.collection.extractions.reset();
                // Update the fetch data silently and then force a synchronous fetch so observers will see the collection
                // in a loading state.
                this.collection.extractions.fetchData.set({
                    search: extractionSearch,
                    count: 0
                }, { silent: true });
                this.collection.extractions.fetch({ data: this.model.application.pick('app', 'owner') }).done(_(function() {
                    // Filter out extractions that are using the <regex> in <field> syntax, we only want extractsion from _raw.
                    var filteredModels = this.collection.extractions.filter(function(model) {
                        return !/ in [a-zA-Z0-9_]+$/.test(model.entry.content.get('value'));
                    });
                    this.collection.extractions.reset(filteredModels);
                }).bind(this));
            }
        },
        /**
         * @constructor
         *
         * @param options {Object} {
         *     sourcetype {String} the name of the current sourcetype
         *     model: {
         *         application <models.Application>
         *         user <models.services.authentication.Users>
         *         extraction <models.services.data.props.Extraction>
         *     }
         *     collection: {
         *         extractions <collections.services.data.props.Extractions>
         *         sourcetypes <collections.knowledgeobjects.Sourcetypes>
         *         roles <collections.services.authorization.Roles>
         *     }
         * }
         */

        initialize: function() {
            BaseView.prototype.initialize.apply(this, arguments);

            this.model.state = new BaseModel({
                sourcetype: this.options.sourcetype,
                sampleSize: { head: 1000 },
                inputField: '_raw',
                regex: this.model.extraction.isNew() ? '' : this.model.extraction.entry.content.get('value'),
                interactiveMode: fieldExtractorUtils.INTERACTION_MODE,
                filter: '',
                clustering: fieldExtractorUtils.CLUSTERING_NONE,
                eventsView: fieldExtractorUtils.VIEW_ALL_EVENTS,
                mode: fieldExtractorUtils.SELECT_SOURCETYPE_MODE,
                masterEvent: this.options.masterEvent,
                examples: [],
                counterExamples: [],
                sampleEvents: []
            });
            this.model.searchJob = new Job({}, { delay: 1000, processKeepAlive: true });

            this.collection.wizardSteps = new BaseCollection([
                {
                    value: fieldExtractorUtils.SELECT_SOURCETYPE_MODE,
                    label: _('Select sourcetype').t(),
                    nextLabel: _('Next').t()
                },
                {
                    value: fieldExtractorUtils.SELECT_SAMPLE_MODE,
                    label: _('Select sample').t(),
                    enabled: !!this.model.state.get('sourcetype'),
                    nextLabel: _('Next').t()
                },
                {
                    value: fieldExtractorUtils.SELECT_FIELDS_MODE,
                    label: _('Select fields').t(),
                    enabled: !!this.model.state.get('masterEvent'),
                    nextLabel: _('Next').t()
                },
                {
                    value: fieldExtractorUtils.VALIDATE_FIELDS_MODE,
                    label: _('Validate fields').t(),
                    enabled: (this.model.state.get('examples') || []).length > 0,
                    nextLabel: _('Next').t()
                },
                {   
                    value: fieldExtractorUtils.SAVE_FIELDS_MODE,
                    label: _('Save').t(),
                    enabled: true,
                    nextLabel: _('Finish').t()
                },
                {
                    value: fieldExtractorUtils.CONFIRMATION_MODE,
                    label: '',
                    showPreviousButton: false,
                    showNextButton: false
                }
            ]);
            // If certain values are pre-populated in the state model, make sure to remove the skipped steps from
            // the wizard and set the correct mode in the state model.
            if(this.model.state.has('sourcetype')) {
                this.collection.wizardSteps.remove(
                    this.collection.wizardSteps.findWhere({ value: fieldExtractorUtils.SELECT_SOURCETYPE_MODE })
                );
                this.model.state.set({ mode: fieldExtractorUtils.SELECT_SAMPLE_MODE });
            }
            if(this.model.state.has('masterEvent')) {
                this.collection.wizardSteps.remove(
                    this.collection.wizardSteps.findWhere({ value: fieldExtractorUtils.SELECT_SAMPLE_MODE })
                );
                this.model.state.set({ mode: fieldExtractorUtils.SELECT_FIELDS_MODE });
            }

            this._createComponents();
            this.listenTo(this.model.state, 'change:sourcetype', function() {
                // Clear any previous extraction information when the sourcetype changes (SPL-89136)
                this.children.regexGenerationController.reset();
                this._syncStateModelFromController();
                this.collection.wizardSteps
                    .findWhere({ value: fieldExtractorUtils.SELECT_SAMPLE_MODE })
                    .set({ enabled: !!this.model.state.get('sourcetype') });
            });
            this.listenTo(this.model.state, 'change:masterEvent', function() {
                this.collection.wizardSteps
                    .findWhere({ value: fieldExtractorUtils.SELECT_FIELDS_MODE })
                    .set({ enabled: !!this.model.state.get('masterEvent') });
            });
            this.listenTo(this.model.state, 'change:examples', function() {
                this.collection.wizardSteps
                    .findWhere({ value: fieldExtractorUtils.VALIDATE_FIELDS_MODE })
                    .set({ enabled: (this.model.state.get('examples') || []).length > 0 });
            });
            this.listenTo(this.model.state, 'change:mode', function() {
                var currentMode = this.model.state.get('mode'),
                    previousMode = this.model.state.previous('mode');
                if(currentMode === fieldExtractorUtils.SELECT_SAMPLE_MODE
                    && previousMode === fieldExtractorUtils.SELECT_FIELDS_MODE){
                    this.tempClearState();
                }else if(currentMode === fieldExtractorUtils.SELECT_FIELDS_MODE
                    && previousMode === fieldExtractorUtils.SELECT_SAMPLE_MODE) {
                    // we are returning to this step after having gone backwards
                    if(this.cachedStateModel) {
                        var sourcetypeHasChanged = this.cachedStateModel.sourcetype !== this.model.state.get('sourcetype'),
                            masterEventHasChanged = this.cachedStateModel.masterEvent !== this.model.state.get('masterEvent'),
                            stateHasChanged = sourcetypeHasChanged || masterEventHasChanged;
                        if(stateHasChanged) {
                            this.cachedStateModel = null; // proceed as usual. state is up to date.
                        }else{
                            // user has gone back but not made any changes. restore previous state.
                            this.restoreClearedState();
                            // When user entered select sample mode and established enabled state of the next mode (validate fields),
                            // the examples array was empty so validate fields mode's next button is disabled. 
                            // Must update enabled state of next mode upon repopulating examples array.
                            this.children.stepWizardControl.updateNavButtons(); 
                        }
                    }
                }else if(currentMode === fieldExtractorUtils.VALIDATE_FIELDS_MODE) {
                    // When entering validate fields mode, set the view to all events so the user can find false
                    // positives (SPL-88792).
                    this.model.state.set({ eventsView: fieldExtractorUtils.VIEW_ALL_EVENTS });
                }
                this.cleanUpOldMode(this.model.state.previous('mode'));
                this._refreshExtractionViews();
                this._updateFooterButtonState();
            });
            this.listenTo(this.model.state, 'change:sampleEvents', function() {
                this.children.previewView.setAddSamplesEnabled(this.model.state.get('sampleEvents').length < this.SAMPLE_EVENT_LIMIT);
            });
            
            this.listenTo(this.model.state, 'change:regex', function() {
                if (this.model.state.get('regex') && this.$('.content-body').is(':hidden')) {
                    this.children.previewView.detach();
                    this.children.previewView.appendTo(this.$('.content-body')).$el.show();
                    this.$('.content-body').show();
                }
                if(this.model.state.get('interactiveMode') === fieldExtractorUtils.NO_INTERACTION_MODE){
                    if(this.regexManuallyModified() || this.model.state.get('mode') === fieldExtractorUtils.SELECT_SAMPLE_MODE) {
                        this.$('.return-automatic-mode').hide();
                        this.children.manualExtractionEditor.$('.edit-regex-warning').hide();
                    }
                }
            });

            this.listenTo(this.collection.wizardSteps, 'change', this._updateFooterButtonState);
            
            this.listenTo(this.model.state, 'change', function() {
                var whitelist = ['filter', 'regex', 'clustering', 'eventsView', 'sampleSize'],
                    shouldRefreshPreview = _(whitelist).any(function(attrName) {
                        return this.model.state.hasChanged(attrName);
                    }, this);

                if(shouldRefreshPreview) {
                    this._refreshPreview();
                }
            });
        },

        tempClearState: function() {
            // Create deep copy of state model
            this.cachedStateModel = $.extend(true, {}, this.model.state.attributes);
            // Set cached mode to the mode at which the cached state model would be restored
            this.cachedStateModel.mode = fieldExtractorUtils.SELECT_FIELDS_MODE;
            // Clear state model so user sees clean state when returning to previous step.
            // Selectively set only attributes that will affect how preview table is rendered
            this.model.state.set({
                sampleSize: { head: 1000 },
                regex: '',
                filter: '',
                clustering: fieldExtractorUtils.CLUSTERING_NONE,
                eventsView: fieldExtractorUtils.VIEW_ALL_EVENTS,
                examples: [],
                counterExamples: [],
                sampleEvents: [],
                requiredText: ''
            });
        },

        restoreClearedState: function() {
            // Restore state model to cached state as user did not make any changes
            this.model.state.set(this.cachedStateModel);
            // Delete cached state model
            this.cachedStateModel = null;
        },

        _createComponents: function() {
            this._setupControllers();
            this._setupStepWizard();
            this._setupFlashMessages();
            this._setupSelectSourcetypeMode();
            this._setupSelectMasterMode();
            this._setupCounterExampleEditor();
            this._setupMasterEventEditor();
            this._setupManualExtractionEditor();
            this._setupExtractionViewer();
            this._setupExistingExtractionsList();
            this._setupResultsTable();
            this._initializeHide();
            this._setupConfirmationView();
        },

        _setupControllers: function() {
            this.children.previewJobController = new PreviewJobController({
                model: {
                    application: this.model.application,
                    state: this.model.state,
                    searchJob: this.model.searchJob
                }
            });

            this.children.regexGenerationController = new RegexGeneratorController({
                model: {
                    application: this.model.application,
                    state: this.model.state,
                    searchJob: this.model.searchJob
                }
            });
            if(this.model.state.has('masterEvent')) {
                this.children.regexGenerationController.setMasterEvent(this.model.state.get('masterEvent'));
                // Since in this entry flow the user did not choose a sample event, we do not yet have a running job to
                // pass to the regex generator, so we dispatch one here and store the resulting promise.
                this.jobReadyDfd = this.model.searchJob.save({}, {
                    data: this.children.previewJobController.createJobAttributes(this._generatePreviewBasesearch(false))
                });
            }
            else {
                // If we reach this branch we are in an entry flow where the user selects a sample event.  This means
                // we are guaranteed to have a running job for the regex generator.
                this.jobReadyDfd = $.Deferred().resolve();
            }
        },

        _setupStepWizard: function() {
            this.children.stepWizardControl = new StepWizardControl({
                model: this.model.state,
                modelAttribute: 'mode',
                collection: this.collection.wizardSteps,
                validateNext: _(function() {
                    if(this.model.state.get('mode') === fieldExtractorUtils.SAVE_FIELDS_MODE) {
                        this._validateAndSaveExtraction();
                        // Return false because the validation will advance to the next step only on a successful save.
                        return false;
                    }
                    return true;
                }).bind(this)
            });
        },

        _setupFlashMessages: function() {
            this.children.flashMessages = new FlashMessages({
                model: {
                    searchJob: this.model.searchJob,
                    extraction: this.model.extraction,
                    extractionAcl: this.model.extraction.acl
                },
                whitelist: [splunkdUtils.FATAL, splunkdUtils.ERROR, splunkdUtils.WARNING]
            });
            this.regexErrorMessageId = _.uniqueId('regex-error-');
        },

        _setupSelectSourcetypeMode: function() {
            var currentSourcetype = new Sourcetype();
            currentSourcetype.entry.set({ name: this.model.state.get('sourcetype') });
            // If we're bootstrapping from an sid, show a simple drop-down control since it will only
            // contain the few sourcetypes from that job.
            var sourcetypePrompt = _('-- Select Sourcetype --').t();
            if(this.options.sid) {
                this.children.sourcetypeDropDown = new SyntheticSelectControl({
                    model: this.model.state,
                    modelAttribute: 'sourcetype',
                    toggleClassName: 'btn',
                    prompt: sourcetypePrompt,
                    items: this.collection.sourcetypes.map(function(sourcetype) {
                        return ({
                            description: sourcetype.entry.content.get('description') || '',
                            value: sourcetype.entry.get('name')
                        });
                    })
                });
            }
            else {
                this.children.sourcetypeDropDown = new SourcetypeMenu({
                    addLabel: false,
                    prompt: sourcetypePrompt,
                    model: {
                        sourcetypeModel: currentSourcetype
                    },
                    collection: {
                        sourcetypesCollection: this.collection.sourcetypes
                    }
                });
            }
            this.listenTo(currentSourcetype.entry, 'change:name', function() {
                this.model.state.set({ sourcetype: currentSourcetype.entry.get('name') });
            });
        },

        _setupSelectMasterMode: function() {
            this.children.masterEventViewer = new MasterEventViewer({
                model: {
                    state: this.model.state
                }
            });
        },

        _setupCounterExampleEditor: function() {
            this.children.counterExampleEditor = new CounterExampleEditor({
                model: {
                    state: this.model.state
                }
            });
            this.listenTo(this.children.counterExampleEditor, 'action:removeCounterExample', function(index) {
                var promise = this.children.regexGenerationController.removeCounterExample(index);
                this._handleRegexGeneratorPromise(promise);
            });
        },

        _setupMasterEventEditor: function() {
            this.children.masterEventEditor = new MasterEventEditor({
                model: {
                    state: this.model.state
                }
            });
            this.listenTo(this.children.masterEventEditor, 'action:addExtraction', function(selection) {
                this.showLoading();
                $.when(this.jobReadyDfd).then(_(function() {
                    var promise = this.children.regexGenerationController.addExample(
                        _(selection).pick('fieldName', 'startIndex', 'endIndex')
                    );
                    this._handleRegexGeneratorPromise(promise);
                }).bind(this));
            });
            this.listenTo(this.children.masterEventEditor, 'action:addSampleExtraction', function(selection, index) {
                var promise = this.children.regexGenerationController.addSampleExtraction(
                    _(selection).pick('fieldName', 'startIndex', 'endIndex'),
                    index
                );
                this._handleRegexGeneratorPromise(promise);
            });
            this.listenTo(this.children.masterEventEditor, 'action:addRequiredText', function(selection) {
                var promise = this.children.regexGenerationController.addRequiredText(selection.selectedText);
                this._handleRegexGeneratorPromise(promise);
            });
            this.listenTo(this.children.masterEventEditor, 'action:removeRequiredText', function() {
                var promise = this.children.regexGenerationController.removeRequiredText();
                this._handleRegexGeneratorPromise(promise);
            });
            this.listenTo(this.children.masterEventEditor, 'action:selectManualMode', function() {
                this.setManualMode();
            });
            this.listenTo(this.children.masterEventEditor, 'action:renameExistingExample', function(oldFieldName, newFieldName) {
                var promise = this.children.regexGenerationController.renameExample(oldFieldName, newFieldName);
                this._handleRegexGeneratorPromise(promise);
            });
            this.listenTo(this.children.masterEventEditor, 'action:removeExistingExample', function(fieldName) {
                var promise = this.children.regexGenerationController.removeExample(fieldName);
                this._handleRegexGeneratorPromise(promise);
            });
            this.listenTo(this.children.masterEventEditor, 'action:removeExistingSampleExtraction', function(fieldName, index) {
                var promise = this.children.regexGenerationController.removeSampleExtraction(fieldName, index);
                this._handleRegexGeneratorPromise(promise);
            });
            this.listenTo(this.children.masterEventEditor, 'action:removeExistingSampleEvent', function(index) {
                var promise = this.children.regexGenerationController.removeSampleEvent(index);
                this._handleRegexGeneratorPromise(promise);
            });
        },

        _setupManualExtractionEditor: function() {
            this.children.manualExtractionEditor = new ManualExtractionEditor({
                model: {
                    state: this.model.state,
                    application: this.model.application
                }
            });
            this.listenTo(this.children.manualExtractionEditor, 'action:selectAutomaticMode', function() {
                this.model.state.set('interactiveMode', fieldExtractorUtils.INTERACTION_MODE);
            });
            this.listenTo(this.children.manualExtractionEditor, 'action:save', this._handleManualExtractionSave);
            this.listenTo(this.children.manualExtractionEditor, 'action:previewInSearch', this._handlePreviewInSearch);
        },

        // The save action can have two different meanings to upstream handlers.
        // If the active extraction is already saved, then it's an in-place update.  If not, it's a creation.
        // In either case, the contract with upstream logic is that the model passed will not already be part of
        // the extractions collection, and the event handler is responsible for saving that model to the back end
        // and updating the extractions collection accordingly or displaying an error message.
        _setupSaveView: function() {
            if(this.model.extraction.isNew()) {
                this.model.extraction.entry.content.set({
                    name: splunkUtils.fieldListToString(
                        fieldExtractorUtils.getCaptureGroupNames(this.model.state.get('regex'))
                    ),
                    value: this.model.state.get('regex'),
                    stanza: this.model.state.get('sourcetype'),
                    type: 'Inline'
                });
            }
            else {
                this.model.extraction.entry.content.set({
                    value: this.model.state.get('regex')
                });
            }
            if(this.children.saveView) {
                this.children.saveView.detach();
                this.children.saveView.remove();
            }
            this.children.saveView = new SaveExtractionsView({
                model: {
                    extraction: this.model.extraction,
                    user: this.model.user,
                    application: this.model.application,
                    state: this.model.state,
                    serverInfo: this.model.serverInfo
                },
                collection: {
                    roles: this.collection.roles
                }
            });
            this.listenTo(this.children.saveView, 'action:finish', function() {
                this._validateAndSaveExtraction();
            });
        },

        _setupExtractionViewer: function() {
            this.children.extractionViewer = new ExtractionViewer({
                model: {
                    state: this.model.state
                }
            });
            this.listenTo(this.children.extractionViewer, 'action:previewInSearch', this._handlePreviewInSearch);
            this.listenTo(this.children.extractionViewer, 'action:selectManualMode', function() {
                this.setManualMode();
            });
        },

        _setupExistingExtractionsList: function() {
            this.children.existingExtractions = new ExistingExtractionsList({
                collection: {
                    extractions: this.collection.extractions
                },
                model: {
                    state: this.model.state,
                    application: this.model.application
                }
            });
        },

        _setupResultsTable: function() {
            this.children.previewView = new RexPreviewViewStack({
                model: {
                    application: this.model.application,
                    searchJob: this.model.searchJob,
                    state: this.model.state
                },
                className: 'preview-view-stack',
                autoDrilldownEnabled: false
            });

            this.listenTo(this.children.previewView, 'action:selectEvent', function(rawText) {
                if(this.model.state.get('mode') === fieldExtractorUtils.SELECT_SAMPLE_MODE) {
                    this.children.regexGenerationController.setMasterEvent(rawText);
                    $('html,body').animate({scrollTop: "0px"}, 300);
                }
                else {
                    this.children.regexGenerationController.addSampleEvent(rawText);
                }
                this._syncStateModelFromController();
            });
            this.listenTo(this.children.previewView, 'action:removeExtraction', function(selection) {
                var promise = this.children.regexGenerationController.addCounterExample(
                    _(selection).pick('fieldName', 'rawText', 'startIndex', 'endIndex')
                );
                this._handleRegexGeneratorPromise(promise);
            });
            this.listenTo(this.children.previewView, 'action:valueDrilldown', function(fieldName, value) {
                this.model.state.set({ filter: fieldName + '=' + value });
            });
            this.listenTo(this.children.previewView, 'action:nextStep', function() {
                this.model.state.set({ mode: this._getNextMode().get('value') });
            });
        },

        regexManuallyModified: function() {
            var regex = this.model.state.get('regex'),
                regexManuallyModified = regex !== this.children.regexGenerationController.getCurrentRegex();
            return regexManuallyModified;
        },

        _initializeHide: function() {
            this.children.masterEventEditor.$el.hide();
            this.children.manualExtractionEditor.$el.hide();
            this.children.previewView.$el.hide();
        },

        _setupConfirmationView: function() {
            this.children.confirmationView = new ConfirmationView({
                model: {
                    state: this.model.state,
                    application: this.model.application
                }
            });
        },

        _syncStateModelFromController: function() {
            this.model.state.set({
                masterEvent: this.children.regexGenerationController.getCurrentMasterEvent(),
                examples: this.children.regexGenerationController.getCurrentExamples(),
                requiredText: this.children.regexGenerationController.getCurrentRequiredText(),
                counterExamples: this.children.regexGenerationController.getCurrentCounterExamples(),
                sampleEvents: this.children.regexGenerationController.getCurrentSampleEvents(),
                regex: this.children.regexGenerationController.getCurrentRegex()
            });
        },

        _handleRegexGeneratorPromise: function(promise) {
            this._syncStateModelFromController();
            this.showLoading();
            promise.done(_(function(regexes) {
                var newRegex = regexes[0] || '';
                this.model.state.set({ regex: newRegex });
                this.hideLoading();
                if(newRegex || this.model.state.get('examples').length === 0) {
                    this.children.flashMessages.flashMsgHelper.removeGeneralMessage(this.regexErrorMessageId);
                }
                else {
                    this.showError();
                }
            }).bind(this));
            promise.fail(_(function() {
                var errorMessage,
                    mode = this.model.state.get('mode');
                if(mode === fieldExtractorUtils.SELECT_FIELDS_MODE){
                    errorMessage = _('The extraction failed. If you are extracting multiple fields, try removing one or more fields. Start with extractions that are embedded within longer text strings.').t();
                }else if(mode === fieldExtractorUtils.VALIDATE_FIELDS_MODE){
                    errorMessage = _('This counterexample cannot update the extraction. Remove it and try another counterexample. If you are extracting multiple fields, you may need to step back and remove one or more of them.').t();
                }else{
                    errorMessage = _('Splunk Enterprise cannot generate a regular expression based on the current event and field selections.').t();
                }
                this.children.flashMessages.flashMsgHelper.addGeneralMessage(this.regexErrorMessageId, {
                    type: splunkdUtils.ERROR,
                    html: errorMessage
                });
                this.model.state.set({ regex: '' });
                this.hideLoading();
                this.showError();
            }).bind(this));
        },

        _refreshPreview: function() {
            this.children.previewJobController.preview(this._generatePreviewBasesearch());
        },

        _generatePreviewBasesearch: function(forPreview) {
            var prefix = this.options.sid ? '| loadjob ' + this.options.sid + ' events=t ignore_running=f require_finished=f | search ' : '',
                previewBaseSearch = prefix + 'index=* sourcetype=' + splunkUtils.searchEscape(this.model.state.get('sourcetype')),
                regex = this.model.state.get('regex');

            if(regex && fieldExtractorUtils.getCaptureGroupNames(regex).length > 0) {
                var pipeToRex = splunkUtils.sprintf(
                    ' | rex field=%s %s' + (forPreview ? '' : ' offset_field=%s'),
                    splunkUtils.searchEscape(this.model.state.get('inputField')),
                    splunkUtils.searchEscape(regex, { forceQuotes: true }),
                    fieldExtractorUtils.OFFSET_FIELD_NAME
                );
                previewBaseSearch += pipeToRex;
            }
            return previewBaseSearch;
        },

        _showSourcetypeDropdown: function() {
            this.$('.select-sourcetype-header').detach();
            $(_(this.selectSourcetypeTemplate).template({})).appendTo(this.$('.content-header'));
            this.children.sourcetypeDropDown.detach();
            this.children.sourcetypeDropDown.appendTo(this.$('.sourcetype-dropdown-wrapper')).$el.show();
        },

        _refreshExtractionViews: function() {
            var mode = this.model.state.get('mode'),
                interactiveMode = this.model.state.get('interactiveMode'),
                isManualMode = interactiveMode === fieldExtractorUtils.NO_INTERACTION_MODE;

            if(isManualMode) {
                if(fieldExtractorUtils.isManualEditorMode(mode, interactiveMode)) {
                   this.children.manualExtractionEditor.appendTo(this.$('.content-header')).$el.show();
                   this.$('.regex-editor-wrapper').find('textarea').focus();
                }
            }else{
                if(mode === fieldExtractorUtils.SELECT_SOURCETYPE_MODE){
                    this._showSourcetypeDropdown();
                    this.$('.content-body').hide();
                }
            
                if(mode === fieldExtractorUtils.SELECT_SAMPLE_MODE){
                    this.$('.select-sample-header').detach();
                    $(_(this.selectMasterEventTemplate).template({
                        sourcetype: this.model.state.get('sourcetype'),
                        fieldsHelpHref: route.docHelp(
                            this.model.application.get('root'),
                            this.model.application.get('locale'),
                            'manager.fields'
                        )
                    })).appendTo(this.$('.content-header'));
                    this.children.masterEventViewer.render().appendTo(this.$('.content-header'));
                }

                if(mode === fieldExtractorUtils.SELECT_FIELDS_MODE){
                    this.$('.select-fields-header').detach();
                    $(_(this.selectFieldsTemplate).template(
                        {
                            ifxHelpHref: route.docHelp(
                                this.model.application.get('root'),
                                this.model.application.get('locale'),
                                'learnmore.field.extraction.automatic'
                            )
                        }
                    )).appendTo(this.$('.content-header'));
                    this.children.masterEventEditor.detach();
                    this.children.masterEventEditor.appendTo(this.$('.content-header')).$el.show();
                    this.children.extractionViewer.appendTo(this.$('.content-header'));
                    this.$('.content-body').prepend(_(this.previewInstructionsTemplate).template({
                        text: _('If you see incorrect results below, click an additional event to add it to the set of sample events. Highlight its values to improve the extraction. You can remove incorrect values in the next step.').t()
                    }));
                }

                if(mode === fieldExtractorUtils.VALIDATE_FIELDS_MODE){
                    //Render the instructions
                    this.$('.validate-fields-header').detach();
                    $(_(this.validateFieldsTemplate).template({
                        instructionsText: _('Validate your field extractions and remove values that are incorrectly highlighted in the Events tab. In the field tabs, inspect the extracted values for each field, and optionally click a value to apply it as a search filter to the Events tab event list.').t()
                    })).appendTo(this.$('.content-header'));

                    this.children.counterExampleEditor.detach();
                    this.children.counterExampleEditor.appendTo(this.$('.content-header')).$el.show();
                    this.children.extractionViewer.appendTo(this.$('.content-header'));
                }
            }

            if(mode === fieldExtractorUtils.SAVE_FIELDS_MODE){
                this.$('.save-extractions-header').detach();
                $(_(this.saveExtractionsTemplate).template({
                    extractionIsNew: this.model.extraction.isNew()
                })).appendTo(this.$('.content-header'));
                this._setupSaveView();
                this.children.saveView.render().appendTo(this.$('.content-header')).$el.show();
                this.$('.content-body').hide();
                if(this.model.state.get('interactiveMode') === fieldExtractorUtils.NO_INTERACTION_MODE){
                    this.$('.page-footer').find('.next-step-button').show();
                    this.$('.return-manual-page').show();
                }
            }

            if(mode === fieldExtractorUtils.CONFIRMATION_MODE) {
                this.$('.confirmation-header').detach();
                this.$('.content-body').hide();
                if(this.model.state.get('interactiveMode') === fieldExtractorUtils.NO_INTERACTION_MODE){
                    this.$('.return-manual-page').hide();
                }
                var editExtractionsHref = route.manager(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    this.model.application.get('app'),
                    ['data', 'props', 'extractions']
                );
                $(_(this.confirmationTemplate).template({
                    successMessage: splunkUtils.sprintf(
                        _('You have extracted additional fields from your data (sourcetype=%s).').t(),
                        this.model.state.get('sourcetype')
                    ),
                    editExtractionsMessage: splunkUtils.sprintf(
                        _('Edit your field extractions at any time by going to %s.').t(),
                        '<a href="' + editExtractionsHref + '" class="edit-extractions-link">' + _('Field Extractions').t() + '</a>'
                    )
                })).appendTo(this.$('.content-header'));
                this.children.confirmationView.render().appendTo(this.$('.content-header')).$el.show();
            }
            
            if(fieldExtractorUtils.isEditFieldsMode(mode, interactiveMode)){
                this.$('.view-all-extractions-button-container').html(_(this.viewAllExtractionsTemplate).template({}));
            }
            else {
                this.$('.view-all-extractions-button-container').empty();
            }

            if(fieldExtractorUtils.isEventsTableMode(mode, interactiveMode)){
                if (mode === fieldExtractorUtils.SELECT_FIELDS_MODE && !this.model.state.get('regex')) {
                    this.$('.content-body').hide();
                }
                else {
                    if (this.model.state.get('mode') === fieldExtractorUtils.SELECT_SAMPLE_MODE && this.model.state.previous('interactiveMode') !== fieldExtractorUtils.NO_INTERACTION_MODE && this.model.state.get('interactiveMode') !== fieldExtractorUtils.NO_INTERACTION_MODE) {
                        this._refreshPreview();
                    }
                    this.children.previewView.detach();
                    this.children.previewView.appendTo(this.$('.content-body')).$el.show();
                }
            }
        },

        _handlePreviewInSearch: function() {
            var url = route.search(
                this.model.application.get('root'),
                this.model.application.get('locale'),
                this.model.application.get('app'),
                { data: { q: this._generatePreviewBasesearch(true) } }
            );
            window.open(url, '_blank');
        },

        _handleManualExtractionSave: function() {
            this.model.state.set({ mode: fieldExtractorUtils.SAVE_FIELDS_MODE });
            this.cleanupManualView();
        },

        _validateAndSaveExtraction: function() {
            this.children.saveView.saveExtractions().done(_(function() {
                this.model.state.set({ mode: fieldExtractorUtils.CONFIRMATION_MODE });
            }).bind(this));
        },

        _getNextMode: function() {
            var activeMode = this._getCurrentMode(),
                activeModeIndex = this.collection.wizardSteps.indexOf(activeMode);

            return this.collection.wizardSteps.at(activeModeIndex + 1);
        },

        _getCurrentMode: function() {
            return this.collection.wizardSteps.findWhere({ value: this.model.state.get('mode') });
        },

        _updateFooterButtonState: function() {
            var currentMode = this._getCurrentMode(),
                nextMode = this._getNextMode(),
                $nextButton = this.$('.page-footer .next-step-button');
            if(!nextMode) {
                $nextButton.hide();
            }
            else {
                if(currentMode.get('nextLabel')){
                    $nextButton.html(currentMode.get('nextLabel') + ' <i class="icon-chevron-right"></i>');
                }
                $nextButton.show();
                if(nextMode.get('enabled') === false) {
                    $nextButton.addClass('disabled');
                }
                else {
                    $nextButton.removeClass('disabled');
                }
            }
        },

        render: function() {
            this.$el.html(this.compiledTemplate({
                sourcetype: this.model.state.get('sourcetype')
            }));
            this.children.stepWizardControl.render().appendTo(this.$('.step-buttons-container'));
            this.children.flashMessages.render().appendTo(this.$('.flash-messages-container'));
            this.children.sourcetypeDropDown.render();
            this.children.extractionViewer.render();
            this.children.previewView.render();
            this.children.masterEventEditor.render();
            this.children.manualExtractionEditor.render();
            this._refreshExtractionViews();
            this._updateFooterButtonState();
            if(!this.model.extraction.isNew()) {
                this.setManualMode();
            }
            return this;
        },

        showLoading: function() {
            this.children.stepWizardControl.disable();
            this.$('.instruction a').addClass('disabled');
            this.children.masterEventEditor.disable();
            this.children.counterExampleEditor.disable();
            this.children.previewView.$el.hide();
            this.$('.regex-loading-message').show();
            this.children.extractionViewer.disable();
            this.$('.footer-next-step-button').addClass('disabled');
        },

        hideLoading: function() {
            this.children.stepWizardControl.enable();
            this.$('.instruction a').removeClass('disabled');
            this.children.masterEventEditor.enable();
            this.children.counterExampleEditor.enable();
            this.children.previewView.$el.show();
            this.$('.regex-loading-message').hide();
            this.children.extractionViewer.enable();
            this._updateFooterButtonState();
        },

        showError: function() {
            this.children.stepWizardControl.disable();
            this.$('.instruction a').addClass('disabled');
            this.children.extractionViewer.disable();
            this.$('.footer-next-step-button').addClass('disabled');

        },

        viewAllExtractionsTemplate: '\
            <button class="view-extractions-button btn"><%- _("Existing fields").t() %> <i class="icon-chevron-right"></i></button>\
        ',

        selectSourcetypeTemplate: '\
            <div class="select-sourcetype-header">\
                <div class="instruction">\
                    <h3><%- _("Select a sourcetype").t() %></h3>\
                </div>\
                <div class="sourcetype-wrapper"><span class="sourcetype-label"><%- _("Source type").t() %></span>\
                <div class="sourcetype-dropdown-wrapper form-horizontal align-left"></div></div>\
            </div>\
        ',

        selectMasterEventTemplate: '\
            <div class="select-sample-header">\
                <div class="instruction">\
                    <h3 class="instruction-title"><%- _("Select Sample Event").t() %></h3>\
                    <span class="instruction-text">\
                        <%- _("Select a sample event and click Next. Splunk will use the sample event to extract fields using a regular expression. ").t() %>\
                        <a href="<%- fieldsHelpHref %>" class="external" target="_blank"><%- _("Read our documentation on fields").t() %></a>\
                        <%- _(" for help with this page.").t() %>\
                        <div class="manual-mode-button-container"><a href="#" class="manual-mode-button"><%- _("I prefer to write the regular expression myself").t() %> <i class="icon-chevron-right"></i></a></div>\
                    </span>\
                </div>\
                <div class="sourcetype-wrapper"><span class="sourcetype-label"><%- _("Sourcetype:").t() %></span>\
                <span class="sourcetype-name"><%- sourcetype %></span></div>\
            </div>\
        ',

        selectFieldsTemplate: '\
            <div class="select-fields-header">\
                <div class="instruction">\
                    <h3 class="instruction-title"><%- _("Select Fields").t() %></h3>\
                    <span class="instruction-text">\
                        <%- _("Highlight one or more values in the sample event to create fields. You can indicate one value is required, meaning it must exist in an event for the regular expression to match. Click on highlighted values in the sample event to modify them. ").t() %>\
                        <a href="<%- ifxHelpHref %>" class="external" target="_blank"><%- _("Learn more").t() %></a>\
                    </span>\
                </div>\
            </div>\
        ',

        validateFieldsTemplate: '\
            <div class="validate-fields-header">\
                <div class="instruction">\
                    <h3 class="instruction-title"><%- _("Validate Fields").t() %></h3>\
                    <span class="instruction-text"><%= instructionsText %></span>\
                </div>\
            </div>\
        ',

        saveExtractionsTemplate: '\
            <div class="save-extractions-header">\
                <div class="instruction"> \
                    <h3 class="instruction-title"><%- _("Save").t() %></h3>\
                    <span class="instruction-text">\
                        <% if(extractionIsNew) { %>\
                            <%- _("Name the extraction and set permissions.").t() %> \
                        <% } else { %>\
                            <%- _("Verify permissions").t() %> \
                        <% } %>\
                    </span>\
                </div>\
            </div>\
        ',

        confirmationTemplate: '\
            <div class="confirmation-header">\
                <div class=" instructions">\
                	<h3 class="instruction-title"><%- _("Success!").t() %></h3>\
                </div>\
                <p class="success-message"><%- successMessage %></p>\
                <p class="edit-extractions-message">\
                    <%= editExtractionsMessage %>\
                </p>\
            </div>\
        ',

        previewInstructionsTemplate: '\
            <div class="body-instructions">\
                <h3 class="instructions-title"><%- _("Preview").t() %></h3>\
                <div class="instructions-text">\
                    <%= text %>\
                </div>\
            </div>\
        ',

        template: '\
            <div class="page-header">\
                <div class="extract-fields-page-title"><%- _("Extract Fields").t() %></div>\
                <a href="#" class="btn previous-button return-automatic-mode" style="display:none"><i class="icon-chevron-left"></i> <%- _("Back").t() %></a>\
                <a href="#" class="btn previous-button return-manual-page" style="display:none"><i class="icon-chevron-left"></i> <%- _("Back").t() %></a>\
                <div class="step-buttons-container"></div>\
                <div class="view-all-extractions-button-container"></div>\
            </div>\
            <div class="page-content">\
                <div class="content-header">\
                    <div class="flash-messages-container"></div>\
                </div>\
                <div class="content-body preview-container">\
                    <div class="regex-loading-message" style="display: none;">\
                        <div class="alert alert-info">\
                            <i class="icon-alert"></i>\
                            <%- _("Generating a Regular Expression...").t() %>\
                        </div>\
                    </div>\
                </div>\
                <div class="content-footer"></div>\
            </div>\
            <div class="page-footer">\
                <button class="next-step-button footer-next-step-button btn btn-large btn-primary"><%- _("Next").t() %> <i class="icon-chevron-right"></i></button>\
            </div>\
        '

    });

});