/**
 * WARNING: the "copy and paste" software design pattern was used to create this file from
 * views/shared/documentcontrols/dialogs/permissions_dialog/Master.js.  If you fix a bug here, please check the other
 * file to see if the fix should be applied there as well.  Sorry for the inconvenience, Simon will buy you a beer.
 */

define([
    'jquery',
    'underscore',
    'backbone',
    'module',
    'models/Base',
    'models/ACLReadOnly',
    'views/Base',
    'views/shared/controls/ControlGroup',
    'views/shared/documentcontrols/dialogs/permissions_dialog/ACL',
    'util/splunkd_utils',
    'util/field_extractor_utils'
    ],
    function(
        $,
        _,
        Backbone,
        module,
        BaseModel,
        ACLReadOnlyModel,
        BaseView,
        ControlGroup,
        ACL,
        splunkd_utils,
        fieldExtractorUtils
    ) {

    return BaseView.extend({
        moduleId: module.id,
        /**
         * @constructor
         * @param options {
         *     model: {
         *         extraction <models.services.data.props.Extraction>
         *         user <models.services.authentication.User>
         *         application <models.shared.Application>
         *     }
         *     collection: {
         *         roles <collections.services.authorization.Roles>
         *     }
         * }
         */
        initialize: function(options) {
            var DEFAULT_ACL_SETTINGS = {
                app: this.model.application.get('app'),
                can_change_perms: true,
                can_list: true,
                can_share_app: true,
                can_share_global: true,
                can_share_user: true,
                can_write: true,
                modifiable: true,
                owner: this.model.application.get('owner'),
                perms: {
                   read: [
                      "*"
                   ],
                   write: [
                      "admin"
                   ]
                },
                removable: true,
                sharing: 'user'
            };

            var defaults = {
                nameKey: 'name'
            };
            _.defaults(this.options, defaults);

            BaseView.prototype.initialize.call(this, this.options);

            if(this.model.extraction.isNew()) {
                this.model.extraction.entry.acl.set(DEFAULT_ACL_SETTINGS);
            }

            this.model.perms = new BaseModel();
            this.model.inmem = new ACLReadOnlyModel($.extend(true, {}, this.model.extraction.entry.acl.toJSON()));

            this.translateToPermsModel();

            this.isManualMode = this.model.state.get('interactiveMode') === fieldExtractorUtils.NO_INTERACTION_MODE;
            if(this.model.extraction.isNew()) {
                this.children.name = new ControlGroup({
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: this.options.nameKey,
                        model: this.model.extraction.entry.content
                    },
                    label: _('Extractions Name').t()
                });
            }
            else {
                this.children.name = new ControlGroup({
                    controlType: 'Label',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: this.options.nameKey,
                        model: this.model.extraction.entry
                    },
                    label: _('Extractions Name').t()
                });
            }

            this.children.owner = new ControlGroup({
                controlType: 'Label',
                controlOptions: {
                    modelAttribute: 'owner',
                    model: this.model.inmem
                },
                label: _('Owner').t()
            });

            this.children.sourcetype = new ControlGroup({
                controlType: 'Label',
                controlOptions: {
                    modelAttribute: 'stanza',
                    model: this.model.extraction.entry.content
                },
                label: _('Sourcetype').t()
            });

            var fieldNamesModel = new BaseModel({ fieldNames:
                fieldExtractorUtils.getCaptureGroupNames(this.model.state.get('regex'))
            });
            this.children.fieldNames = new ControlGroup({
                controlType: 'Label',
                controlOptions: {
                    modelAttribute: 'fieldNames',
                    model: fieldNamesModel
                },
                label: _('Fields').t()
            });

            if(!this.isManualMode && this.model.extraction.isNew() && this.model.state.get('requiredText')){
                this.children.requiredText = new ControlGroup({
                    controlType: 'Label',
                    controlOptions: {
                        modelAttribute: 'requiredText',
                        model: this.model.state
                    },
                    label: _('RequiredText').t()
                });
            }
            
            this.children.regex = new ControlGroup({
                controlType: 'Label',
                controlOptions: {
                    modelAttribute: 'value',
                    model: this.model.extraction.entry.content
                },
                label: _('Regular Expression').t()
            });

            var exposeAppContext = this.model.user.canUseApps();
            var permission_tabs;
            if (!exposeAppContext) {
                permission_tabs = [
                    { label: _('Owner').t(), value: splunkd_utils.USER, className: 'user' },
                    { label: _('Global').t(), value: splunkd_utils.GLOBAL, className: 'global' }
                ];
            } else {
                permission_tabs = [
                    { label: _('Owner').t(), value: splunkd_utils.USER, className: 'user' },
                    { label: _('App').t(), value: splunkd_utils.APP, className: 'app' },
                    { label: _('All apps').t(), value: splunkd_utils.GLOBAL, className: 'global' }
                ];
            }

            this.children.display_for = new ControlGroup({
                controlType: 'SyntheticRadio',
                controlOptions: {
                    modelAttribute: 'sharing',
                    model: this.model.inmem,
                    items: permission_tabs,
                    save: false
                },
                label: _('Permissions').t()
            });

            this.children.acl = new ACL({
                model : {
                    perms: this.model.perms
                },
                collection: this.collection.roles
            });

            this.model.inmem.on('change:sharing', function() {
                if (this.model.inmem.get("sharing") === splunkd_utils.USER) {
                    this.children.acl.$el.hide();
                } else {
                    this.children.acl.$el.show();
                }
            }, this);

            if(exposeAppContext) {
                this.children.app = new ControlGroup({
                    controlType: 'Label',
                    controlOptions: {
                        modelAttribute: 'app',
                        model: this.model.inmem
                    },
                    label: _('App').t()
                });
            }
        },
        saveExtractions: function() {
            var data = { app: this.model.inmem.get("app") };
                data.owner = (this.model.inmem.get('sharing') !== splunkd_utils.USER) ?
                    splunkd_utils.NOBODY : this.model.application.get("owner");

            if(this.model.extraction.isNew()) {
                return this.model.extraction.save({}, { data: data }).then(_(this.updateACL).bind(this));
            }
            return this.model.extraction.save().then(_(this.updateACL).bind(this));
        },
        updateACL: function() {
            this.translateFromPermsModel();
            var data = this.model.inmem.toDataPayload();
            data.owner = this.model.application.get('owner');
            return this.model.extraction.acl.save({}, { data: data });
        },
        translateToPermsModel: function() {
            var perms = this.model.inmem.permsToObj();

            // add 'Everyone' attributes
            this.model.perms.set('Everyone.name', _('Everyone').t());
            this.model.perms.set('Everyone.read', _.indexOf(perms.read, '*')!=-1);
            this.model.perms.set('Everyone.write', _.indexOf(perms.write, '*')!=-1);

            // add 'name', 'read' and 'write' attributes for each role
            this.collection.roles.each(function(roleModel, i){
                var roleName = roleModel.entry.get("name"), j = i + 1;
                this.model.perms.set('role_' + j + '.name', roleName);
                this.model.perms.set('role_' + j + '.read', _.indexOf(perms.read, roleName)!=-1);
                this.model.perms.set('role_' + j + '.write', _.indexOf(perms.write, roleName)!=-1);
            }, this);
        },
        translateFromPermsModel: function() {
            var perms = {
                read: [],
                write: []
            };

            _(this.model.perms.toJSON()).each(function(value, key){
                var splitKey = key.split('.'),
                    role = splitKey[0],
                    type = splitKey[1];
                if (type === 'read' || type === 'write') {
                    if (value) {
                        // can read or write
                        if(role === 'Everyone') {
                            perms[type].push('*');
                        } else {
                            perms[type].push(this.model.perms.get(role + '.name'));
                        }
                    }
                }
            }.bind(this));

            this.model.inmem.set('perms', perms);

        },
        setView: function() {
            if (!this.model.inmem.get("can_share_user")) {
                this.children.display_for.$('.user').attr('disabled', true);
            }
            if (!this.model.inmem.get("can_share_app")) {
                this.children.display_for.$('.app').attr('disabled', true);
            }
            if (!this.model.inmem.get("can_share_global")) {
                this.children.display_for.$('.global').attr('disabled', true);
            }
            if(!this.model.inmem.get("modifiable")) {
                this.children.display_for.$('.btn').attr('disabled', true);
            }
            if (this.model.inmem.get("sharing") === 'user'){
                this.children.acl.$el.hide();
            } else {
                this.children.acl.$el.show();
            }
        },
        renderEvents: function() {
            var noRequiredTextObject = {text: ''},
                noStartIndex = -1;
            if(this.model.state.get('masterEvent') && this.model.state.get('masterEvent').length > 0) {
                $(_(this.sampleEventTemplate).template({})).appendTo(this.$('.review-extractions-form'));
                var requiredTextObject = {text: this.model.state.get('requiredText')};
                this.$('.master-event-wrapper').html($('<div class="event-text"></div>').html(
                    fieldExtractorUtils.replaceBoundingGroupsWithTemplate(
                        this.model.state.get('masterEvent'),
                        this.model.state.get('examples'),
                        requiredTextObject,
                        noStartIndex,
                        fieldExtractorUtils.highlightedContentTemplate
                    )
                ));
            }
            _(this.model.state.get('sampleEvents')).each((function(sample, i) {
                this.$('.sample-events-wrapper').append($('<div class="event-text"></div>').html(fieldExtractorUtils.replaceBoundingGroupsWithTemplate(
                    sample.rawText,
                    sample.extractions,
                    noRequiredTextObject,
                    noStartIndex,
                    fieldExtractorUtils.highlightedContentTemplate,
                    this.model.state.get('examples')
                )));
            }).bind(this));
        },
        render: function() {
            this.$el.html(this.compiledTemplate({
                extractionIsNew: this.model.extraction.isNew()
            }));

            this.children.name.render().appendTo(this.$('.save-extractions-form'));
            
            if(this.model.extraction.isNew()){
                this.children.name.$('.control').prepend($("<span class='extract-prelabel'>EXTRACT-</span>"));
            }

            this.children.owner.render().appendTo(this.$('.save-extractions-form'));

            if (this.children.app) {
                this.children.app.render().appendTo(this.$('.save-extractions-form'));
            }
            
            this.children.display_for.render().appendTo(this.$('.save-extractions-form'));

            if (this.children.acl) {
                this.children.acl.render().appendTo(this.$('.save-extractions-form'));
            }

            this.children.sourcetype.render().appendTo(this.$('.review-extractions-form'));

            if(!this.isManualMode) {
                this.renderEvents();
            }

            this.children.fieldNames.render().appendTo(this.$('.review-extractions-form'));
            if(this.children.requiredText) {
                this.children.requiredText && this.children.requiredText.render().appendTo(this.$('.review-extractions-form'));
            }
            this.children.regex.render().appendTo(this.$('.review-extractions-form'));

            this.setView();

            return this;
        },
        template: '\
            <div class="save-extractions-container">\
                <div class="save-extractions-form form-horizontal"></div>\
                <div class="review-extractions-form form-horizontal"></div>\
            </div>\
        ',
        sampleEventTemplate: '\
            <div class="sample-event-container">\
                <span class="sample-event-label control-label" for="sample-events-wrapper"><%- _("Sample event").t() %></span>\
                <div class="events-wrapper">\
                    <div class="master-event-wrapper"></div>\
                    <div class="sample-events-wrapper"></div>\
                </div>\
            </div>\
        '
    });
});
