define([
    'jquery',
    'underscore',
    'module',
    'uri/route',
    'views/Base'
],
function (
    $,
    _,
    module,
    route,
    BaseView
) {
    return BaseView.extend({
        tagName: 'iframe',
        attributes: {
            height: "100%",
            width: "100%",
            scrolling: "no", 
            frameborder: "0",
            src: 'about:blank'
        },
        events: {
            'load': function(e){
                // SPL-88064: overwriting the "redirect" function used by drilldown.js so that the parent window redirects instead of the iframe intself.  
                // Redirects inside of drilldown.js should cover the vast majority of use cases 
                var iframeWindow = $(document).find('iframe')[0].contentWindow; 
                if (iframeWindow && iframeWindow.require) {
                    iframeWindow.require(['splunkjs/mvc/utils'], function(utils){
                        utils.redirect = _.wrap(utils.redirect, function(oldRedirect, url, newWindow, target) {
                            if (!newWindow && !target) {
                                window.parent.location = url;
                            } else { 
                                oldRedirect.call(utils, url, newWindow, target); 
                            }
                        }); 
                    });  
                }
              
                if (this.isLoadingDashboard) {
                    // Ignore 'load' events triggered by the user choosing a new dashboard
                    this.isLoadingDashboard = false; 
                    return; 
                }

                // catch-all for all other 'load' events triggered outside of drilldown.js.  This is a shittier user experience, because the redirect will happen after all of the 
                // assets in the iframe are loaded, but this will be a rare case.     
                window.location.href = this.$el[0].contentWindow.location.href;  
            }
        }, 
        moduleId: module.id,
        initialize: function() {
            BaseView.prototype.initialize.apply(this, arguments);
            this.isLoadingDashboard = true;
            this.listenTo(this.model.dashboard, 'change:id', function() {
                this.render(); 
                this.isLoadingDashboard = true;
            });
        },
        render: function() {
            var src = 'about:blank';
            if (this.model.dashboard.isSimpleXML()) {
                src = route.dashboardFromID(this.model.application.get('root'), this.model.application.get('locale'), this.model.dashboard.get('id'), {data: {hideEdit: true, hideTitle: true, hideSplunkBar: true, hideAppBar: true, hideFooter: true, targetTop: true}});
                this.$el.attr('src', src);
            }
            return this;
        }
    });
});
