define([
    'jquery',
    'underscore',
    'module',
    'views/Base',
    'views/homeclassic/FindAndManageApps',
    'contrib/text!views/homeclassic/AppList.html',
    'contrib/text!views/homeclassic/AppBox.html',
    'contrib/text!views/shared/AppNav-SlideNavTemplate.html',
    'splunk.util',
    'uri/route',
    'util/color_utils',
    'splunk.widget.slidenav', //no import
    'jquery.ui.sortable'//no import
],
function(
    $,
    _,
    module,
    BaseView,
    FindAndManagerAppsView,
    templateAppList,
    templateAppBox,
    templateNavItem,
    splunk_util,
    route,
    color_utils
){
    return BaseView.extend({
        moduleId: module.id,
        className: 'column column-apps',
        events: {
            'sortstop .appBoxes': 'onSortableStop'
        },
        initialize: function() {
            this.templates = {};
            this.templates.templateAppList = _.template(templateAppList);
            this.templates.templateAppBox = _.template(templateAppBox);
            this.templates.templateNavItem = _.template(templateNavItem);
            this.children = {};
            this.children.findAndManageAppsView = new FindAndManagerAppsView({
                collection:this.collection,
                model:this.model,
                canManageRemoteAppsDfd: this.options.canManageRemoteAppsDfd
            });
            this.render();
            this.collection.navs.on('add', this.renderAppBox, this);
            BaseView.prototype.initialize.apply(this, arguments);
        },
        render: function() {
            this.$el.html(this.templates.templateAppList({
                makeManagerLink: this.makeManagerLink.bind(this)
            }));
            this.$el.append(this.children.findAndManageAppsView.render().el);
            this.$appBoxes = this.$(".appBoxes");
            this.bindDragAndDrop();
            return this;
        },
        renderAppBox: function(appModel) {
            this.hideLoadingLabel();
            var self = this,
                appName = appModel.get('appName'),
                appLabel = appModel.get('appLabel'),
                appBox;

            //dont show the home app
            if(appName && appName.toLowerCase() === 'launcher'){
                return;
            }

            appBox = this.renderAppBoxPlaceHolder(appName, appLabel);
            appModel.on('change:navData', function(){
                var navData = appModel.get('navData'),
                    appColor = navData.color,
                    nav = navData.nav;

                this.setGradientStyles(appColor, appBox, appName);

                if(nav && nav.length > 0){
                    appBox.find('.slideNavPlaceHolder').slidenav({
                        navData: {'submenu':nav},
                        levelTemplate: self.templates.templateNavItem,
                        childPropertyName: 'submenu',
                        backText: _('Back').t()
                    }).show();
                    appBox.find(".app-nav").addClass("app-nav-with-slidenav");
                }
            }, this);
        },
        setGradientStyles: function(color, appBox, appName){
            color = color_utils.normalizeHexString(color||'');
            var isHexColor = /(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(color);
            if (!isHexColor){
                return;
            }

            var styles='',
                hoverStyles='',
                luminosity = color_utils.hslColorFromRgbColor(color_utils.rgbColorFromHexString(color))[2],
                modifier =  1 - luminosity/400,
                appColorNorm = color,
                gradientTop = color_utils.modifyLuminosityOfHexString(color, 1.2 * modifier),
                gradientBottom = color_utils.modifyLuminosityOfHexString(color, 0.85 ),
                gradientTopHover = color_utils.modifyLuminosityOfHexString(color, 1.45 * modifier),
                gradientBottomHover = color_utils.modifyLuminosityOfHexString(color, 0.95),
                gradients = color_utils.generateGradientStyles(gradientTop, gradientBottom),
                gradientsHover = color_utils.generateGradientStyles(gradientTopHover, gradientBottomHover),
                borderColor = color_utils.modifyLuminosityOfHexString(color, 0.80 ),
                borderTopColor = color_utils.modifyLuminosityOfHexString(color, 0.95),
                borderBottomColor = color_utils.modifyLuminosityOfHexString(color, 0.65),
                borderColorHover = color_utils.modifyLuminosityOfHexString(color, 0.90),
                borderTopColorHover = color_utils.modifyLuminosityOfHexString(color, 0.95 ),
                borderBottomColorHover = color_utils.modifyLuminosityOfHexString(color, 0.80),
                highlightColor = color_utils.modifyLuminosityOfHexString(color, 1.3 * modifier),
                highlightColorHover = color_utils.modifyLuminosityOfHexString(color, 1.4 * modifier);

            styles += 'background-image: none;';
            hoverStyles += 'background-image: none;';
            for (var i=0;i<gradients.length-1;i++){
                styles += 'background-image:' + gradients[i] + ';';
                hoverStyles += 'background-image:' + gradientsHover[i] + ';';
            }
            styles += 'background-color: ' + appColorNorm + ';';
            styles += 'border: 1px solid ' + borderColor + ';';
            styles += 'border-top-color: 1px solid ' + borderTopColor + ';';
            styles += 'border-bottom-color: 1px solid ' + borderBottomColor + ';';
            styles += 'box-shadow: inset 0 1px 0 ' + highlightColor + ';';
            // NOTE: IE < 9 uses filter instead of bg-image
            styles += "filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='" + gradientTop + "', endColorstr='" + gradientBottom + "', GradientType=0) ; "; // IE9 and down

            hoverStyles += 'background-color: ' + appColorNorm + ';';
            hoverStyles += 'border: 1px solid ' + borderColorHover + ';';
            hoverStyles += 'border-top-color: 1px solid ' + borderTopColorHover + ';';
            hoverStyles += 'border-bottom-color: 1px solid ' + borderBottomColorHover + ';';
            styles += 'box-shadow: inset 0 1px 0 ' + highlightColorHover + ';';
            // NOTE: IE < 9 uses filter instead of bg-image
            hoverStyles += "filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='" + gradientTopHover + "', endColorstr='" + gradientBottomHover + "', GradientType=0) ; "; // IE9 and down

            $('head').append('<style>.appName-' + appName + ' .app-title{' + styles + '} .appName-' + appName + ' .app-title:hover{' + hoverStyles + '}</style>');
        },
        renderAppBoxPlaceHolder: function(appName, appLabel) {
            var appIcon = route.appIcon(
                this.model.application.get('root'),
                this.model.application.get('locale'),
                this.model.application.get('owner'),
                appName
            );

            var routeToApp = route.page(
                this.model.application.get('root'),
                this.model.application.get('locale'),
                appName,
                ''
            );

            var box = $(this.templates.templateAppBox({
                appName: appName,
                appLabel: appLabel,
                appLink: routeToApp,
                appIcon: appIcon
            }));
            this.$appBoxes.append(box);
            return box;
        },
        bindDragAndDrop: function() {
            var self = this;
            this.$appBoxes.sortable({
                revert: false,
                distance: 5,
                appendTo: 'body',
                forcePlaceholderSize: true,
                opacity: 0.7,
                tolerance: 'pointer',
                cursor: 'move',
                handle: '.draghandle',
                placeholder: "app-wrapper app-placeholder-wrapper",
                start: function(event, ui){
                    ui.placeholder.html("<div class='app-placeholder'></div>");
                    ui.helper.appendTo(self.$appBoxes);
                }
            });
        },
        onSortableStop: function(){
            var apps = this.$('.app-wrapper'),
                appList = [],
                appId;
            apps.each(function(index, item){
                appId = String($(item).data('appid'));
                appList.push(appId);
            });
            this.model.userPref.entry.content.set('appOrder', appList.join(','));
            this.model.userPref.save();
        },
        makeManagerLink: function(managerPage) {
            return route.manager(
                this.model.application.get('root'),
                this.model.application.get('locale'),
                this.model.application.get('app'),
                managerPage
            );
        },
        showFindMoreApps: function() {
            this.$('#findMoreAppsBtn').show();
        },
        hideLoadingLabel: function() {
            this.$('.loading').hide();
        }
    });
});
