define([
    'jquery',
    'underscore',
    'module',
    'views/Base',
    'views/shared/searchbar/Master',
    'views/shared/whatsnewdialog/Master',
    'views/shared/FlashMessages',
    './InfoBar',
    './AppList',
    'models/services/data/ui/Pref',
    'contrib/text!views/homeclassic/Home.html',
    'util/splunkd_utils',
    'splunk.util',
    'uri/route'
],
function(
    $,
    _,
    module,
    BaseView,
    SearchBar,
    WhatsNewDialogView,
    FlashMessage,
    InfoBar,
    AppList,
    UIPrefsModel,
    TemplateHome,
    splunkd_utils,
    splunk_util,
    route
){
    return BaseView.extend({
        moduleId: module.id,
        initialize: function() {
            BaseView.prototype.initialize.apply(this, arguments);
            this.isRenderedDfd = $.Deferred();
            this.logoFlavors = {
                'hunk':       '&#xF000;',
                'enterprise': '&#xF001;',
                'cloud':      '&#xF002;'
            };
            this.errorTypes = [splunkd_utils.FATAL, splunkd_utils.ERROR, splunkd_utils.NOT_FOUND];

            $.when(this.options.uiPrefsDfd, this.options.userDfd).done(function() {
                this.children.searchBar = new SearchBar({
                    collection: {
                        apps: this.collection.navs,
                        times: this.collection.times
                    },
                    model: {
                        state: this.model.setting,
                        user: this.model.user,
                        timeRangeWorking: this.model.timeRangeWorking,
                        timeRange: this.model.timeRange,
                        application: this.model.application,
                        appLocal: this.model.appLocal,
                        serverInfo: this.model.serverInfo
                    }
                });
            }.bind(this));

            this.children.appList = new AppList({
                collection: this.collection,
                model: this.model,
                canManageRemoteAppsDfd: this.options.canManageRemoteAppsDfd
            });

            this.children.infoBar = new InfoBar({
                model: {
                    serverInfo: this.model.serverInfo,
                    application: this.model.application,
                    metaDataResult: this.model.metaDataResult

                },
                canEditMonitorDfd: this.options.canEditMonitorDfd
            });

            this.children.flashMessage = new FlashMessage({
                model: {
                    searchJob: this.model.searchJob,
                    report: this.model.report
                },
                whitelist: this.errorTypes
            });

            this.model.serverInfo.on('change reset', function() {
                this.updateLogo();
            }, this);

        },
        template: TemplateHome,
        updateLogo: function() {
            $.when(this.isRenderedDfd).done(function() {
                var logoFlavor = this.getLogoFlavorIcon();
                if (logoFlavor) {
                    this.$('#productLogo').html('splunk<span class="prompt">&gt;&#x00AE;'+logoFlavor+'</span>');
                }
            }.bind(this));
        },
 
        getLogoFlavorIcon: function() {
            var branding = this.model.serverInfo.getBranding(),
                icon = '';
            if (this.logoFlavors.hasOwnProperty(branding)) {
                icon = this.logoFlavors[branding];
            }
            return icon;
        },

        render: function(){
            this.$el.append(this.compiledTemplate());
            $.when(this.options.uiPrefsDfd, this.options.userDfd).done(function() {
                this.$('.searchBarPlaceHolder').append(this.children.searchBar.render().el);
                this.$('.searchBarPlaceHolder').append(this.children.flashMessage.render().el);
            }.bind(this));
            this.$('.appListPlaceHolder').append(this.children.appList.el);
            this.$('.infoBarPlaceHolder').append(this.children.infoBar.render().el);

            this.isRenderedDfd.resolve();
            return this;
        }
    });
});
