define([
    'jquery',
    'underscore',
    'module',
    'views/Base',
    'views/shared/controls/TextControl',
    'contrib/text!views/homeclassic/InfoBar.html',
    'views/homeclassic/Stats',
    'splunk.util',
    'uri/route'
],
function(
    $,
    _,
    module,
    BaseView,
    TextControl,
    InfoBarTemplate,
    Stats,
    splunk_util,
    route
) {
    return BaseView.extend({
        moduleId: module.id,
        initialize: function() {
            BaseView.prototype.initialize.apply(this, arguments);
            this.showDataPanel = false;
            this.options.canEditMonitorDfd.done(function() {
                this.showDataPanel = true;
                this.children.stats = new Stats({
                    model: {
                        metaDataResult: this.model.metaDataResult
                    }
                });
                this.render();
            }.bind(this));

            this.serverInfoDfd = $.Deferred();
            this.model.serverInfo.on('change', function(){
                this.serverInfoDfd.resolve();
            }.bind(this));

            this.children.searchInput = new TextControl({
                placeholder: _('Search Documentation').t(),
                inputClassName: 'input-medium search-query'
            });
        },

        template: InfoBarTemplate,

        events: {
            'keypress input.search-query': "onDocsSearch"
        },
        
        render: function() {
            var inputsLink = route.manager(
                this.model.application.get('root'),
                this.model.application.get('locale'),
                this.model.application.get('app'),
                'datainputstats'
            );

            var addDataLink = route.manager(
                this.model.application.get('root'),
                this.model.application.get('locale'),
                this.model.application.get('app'),
                'adddata'
            );

            var template = this.compiledTemplate({
                showDataPanel: this.showDataPanel,
                inputsLink: inputsLink,
                addDataLink: addDataLink
            });
            var $html = $(template);
            $html.find('.form-search').append(this.children.searchInput.render().el);
            this.$el.html($html);

            if (this.showDataPanel) {
                this.$('#stats-placeholder').append(this.children.stats.render().el);
            }
            return this;
        },

        onDocsSearch: function(evt) {
            if (evt.keyCode === 13) { //ENTER
                var s = $(evt.target).val();
                evt.preventDefault();
                evt.stopPropagation();
                $.when(this.serverInfoDfd).then(function() {
                    var url = route.docSearch(
                        this.model.application.get('locale'),
                        this.model.serverInfo.entry.content.get('version'),
                        splunk_util.normalizeBoolean(this.model.serverInfo.entry.content.get('isFree')),
                        splunk_util.normalizeBoolean(this.model.serverInfo.entry.content.get('isTrial')),
                        s);
                    window.open(url);
                }.bind(this));
                $(evt.target).val('').blur();
            }
        }
    });
});
