define([
    'underscore',
    'module',
    'views/Base',
    'contrib/text!views/homeclassic/Stats.html',
    'splunk.i18n',
    'util/time'
],
function(
    _,
    module,
    BaseView,
    StatsTemplate,
    i18n,
    time_utils
) {
    return BaseView.extend({
        /**
             Displays brief statistical information about the events indexed by Splunk
         */
        moduleId: module.id,
        template: StatsTemplate,
        initialize: function() {
            BaseView.prototype.initialize.apply(this, arguments);
            this.model.metaDataResult.on('sync', this.render, this);
        },

        render: function() {
            var template,
                result = this.model.metaDataResult.results.at(0),
                hasResult = !!result,
                cntEvents = '0', earliestRel = _('N/A').t(), latestRel = _('N/A').t(), earliestTime = _('N/A').t(), latestTime = _('N/A').t(), latestTimeRaw= 0, now;

            if (hasResult) {
                cntEvents = i18n.format_decimal(parseInt(result.get('cnt')[0], 10));
                earliestRel = time_utils.convertToRelativeTime(parseInt(result.get('min')[0], 10)) || _('N/A').t();
                earliestTime = time_utils.convertToLocalTime(parseInt(result.get('min')[0], 10)) || _('N/A').t();

                latestTimeRaw = parseInt(result.get('max')[0], 10);
                now = (new Date()).getTime()/1000;
                if (latestTimeRaw >= now) {
                    latestRel = latestTime = _('Now').t();
                } else {
                    latestRel = time_utils.convertToRelativeTime(latestTimeRaw) || _('N/A').t();
                    latestTime = time_utils.convertToLocalTime(latestTimeRaw) || _('N/A').t();
                }
            }

            template = this.compiledTemplate({
                hasResults: hasResult,
                cntEvents: cntEvents,
                earliestRel: earliestRel,
                earliestTime: earliestTime,
                latestRel: latestRel,
                latestTime: latestTime
            });

            this.$el.html(template);

            return this;
        }
    });

});