define(
    [
        'jquery',
        'underscore',
        'backbone',
        'module',
        'collections/shared/FlashMessages',
        'views/Base',
        'views/shared/FlashMessagesLegacy'
    ],
    function(
        $,
        _,
        Backbone,
        module,
        FlashMessagesCollection,
        BaseView,
        FlashMessagesView
    ){
    
        return BaseView.extend({
            moduleId: module.id,
            
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);

                // working collection of all each model
                this.collection = this.options.collection || {};
                this.collection.flashMessages = new FlashMessagesCollection();
                
                // array of keys within model to remove
                this.hide = this.options.hide || [];

                // div element to insert table into
                this.table_div = this.options.table_div;
                if(!this.table_div) throw new Error("Table div not specified");
               
                this.collection.each(function(model) {
                   for(var i=0; i<this.hide.length; i++) {
                       model.unset(this.hide[i], {silent: true});
                   }
                }, this);

                // constructing table header row based on example model at index 0
                this.children.table_header = "<thead><tr>";
                var custom_id_count = 1;
                _.each(_.keys(this.collection.at(0).toJSON()), function(key) {
                    this.children.table_header += "<th class='col-header' id='col-"+custom_id_count+"'>"+key+"</th>";
                    custom_id_count++;
                }, this);
                this.children.table_header += "</thead></tr>";

                // iterates through rest of collection to construct the rest of the table
                this.children.table_row = "<tbody>";
                this.collection.each(function(model) {
                    var current_row = "<tr>";
                    var modelJSON = model.toJSON();
                    _.each(_.keys(modelJSON), function(key) {
                        current_row += "<td>"+modelJSON[key]+"</td>";
                    }, this);
                    current_row += "</tr>";
                    this.children.table_row += current_row;
                }, this);
                this.children.table_row += "</tbody>";
            },
            render : function() {
                var that = this;
                $('.'+that.table_div).append(that.template);
                $('.table').append(that.children.table_header);  
                $('.table').append(that.children.table_row);
            },
            template: '<div class="table-wrapper"><table class="table table-chrome table-striped table-row-expanding"></table></div>'
        });
    }
);
