define(
    [
        'module',
        'underscore',
        'jquery',
        'backbone',
        'views/Base',
        'views/shared/TableHead',
        'views/managementconsole/table/TableRow',
        'views/managementconsole/table/MoreInfo',
        'views/shared/delegates/TableDock',
        'views/shared/delegates/TableRowToggle',
        'views/shared/tablecaption/Master'
    ],
    function(
        module,
        _,
        $,
        Backbone,
        BaseView,
        TableHeadView,
        TableRowView,
        MoreInfoView,
        TableDock,
        TableRowToggleView,
        TableCaptionView
    ) {
        return BaseView.extend({
            moduleId: module.id,
           
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);

                this.collection = this.collection || {};
                this.collection.clientSidePeers = new Backbone.Collection();

                this.children.tableRowToggle = new TableRowToggleView({el: this.el, collapseOthers: true });

                var columns = [
                        { label: _('i').t(), className: 'col-info', html: '<i class="icon-info"></i>'},
                        { label: _('Instance (host)').t(), sortKey: 'host', className: 'col-name'},
                        { label: _('Instance (serverName)').t(), sortKey: 'peerName', className: 'col-serv-name' },
                        { label: _('Machine').t(), sortKey: 'host_fqdn', className: 'col-mach-name' }, 
                        { label: _('Actions').t(), className: 'col-actions' },
                        { label: _('Server roles').t(), className: 'col-server-roles' },
                        { label: _('Custom groups').t(), className: 'col-tags' },
                        { label: _('Status').t(), sortKey: 'status-toggle', className: 'col-status' },
                        { label: _('State').t(), sortKey: 'state', className: 'col-state' },
                        { label: _('Problems').t(), sortKey: 'problems', className: 'col-problems' }
                ];

                // In single instance mode, remove sortkeys.
                if (this.model.localInstance) {
                    columns = _.map(columns, function(column) {
                        return {
                            label: column.label,
                            className: column.className,
                            //sortKey: column.sortKey,
                            html: column.html
                        };
                    });
                }

                this.children.head = new TableHeadView({
                    model: this.model.state,
                    columns: columns
                });

                if (!this.model.localInstance) {
                    this.children.caption = new TableCaptionView({
                        countLabel: _("Instances").t(),
                        model: {
                            state: this.model.state,
                            serverInfo: this.model.serverInfo,
                            rawSearch: this.model.state
                        },
                        collection: this.collection.clientSidePeers,
                        noFilterButtons: true,
                        filterKey: ['peerName']
                    });
                }

                this.children.rows = this.rowsFromCollection();

                this.children.tableDock = new TableDock({ 
                    el: this.el, 
                    offset: this.model.localInstance ? 0 : 36, 
                    dockScrollBar: false, 
                    defaultLayout: 'fixed', 
                    flexWidthColumn: 1  
                });
                this.activate();
            },

            startListening: function () {
                this.model.state.on('updateRows', _.debounce(this.renderRows), this);
                if (this.model.localInstance) {
                    this.model.localInstance.on('sync', _.debounce(this.renderRows), this);
                } else {
                    this.collection.peers.on('sync', _.debounce(this.renderRows), this);
                    this.model.state.on(
                        'change:sortKey change:sortDirection change:offset change:search',
                        _.debounce(this.renderRows),
                        this
                    );
                }
            },
            rowsFromCollection: function() {
                if (this.model.localInstance) {
                    return [
                        new TableRowView({
                            model: {
                                appLocal: this.model.appLocal,
                                application: this.model.application,
                                peer: this.model.localInstance,
                                state: this.model.state
                            },
                            collection: {
                                peers: this.collection.peers
                            },
                            index: 0
                        }),
                        new MoreInfoView({
                            model: {
                                peer: this.model.localInstance
                            },
                            index: 0
                        })
                    ];
                }

                var peers = this.collection.peers;
                // Sort properly
                peers = peers.sortBy(function(peer) {
                    var sortKey = this.model.state.get('sortKey');
                    var value = null;
                    if (sortKey === 'name') {
                        value = peer.entry.get('name');
                    } else if (sortKey === 'state') {
                        var state = peer.entry.content.get('state');
                        if(state === "Configured") {
                            return "1";
                        } else if (state === "New") {
                            return "2";
                        } else {
                            return "-1";
                        }
                    } else if (sortKey === 'status-toggle') {
                        var toggle = peer.entry.content.get('status-toggle');
                        if(toggle === "Enabled") {
                            return "1";
                        } else if (toggle === "Disabled") {
                            return "2";
                        } else {
                            return "-1";
                        }
                    } else if (sortKey === 'problems') {
                        if(peer.entry.content.get('errorMessages').length > 0) {
                            return "1";
                        } else if (peer.entry.content.get('warningMessages').length > 0) {
                            return "2";
                        } else {
                            return "-1";
                        }
                    } else {
                        value = peer.entry.content.get(sortKey);
                    }
                   
                    return value;
                }, this);

                if (this.model.state.get('sortDirection') !== 'asc') {
                    peers = peers.reverse();
                }

                var extractValue = function(string) {
                    var s = string.split('=')[1];
                    return s.replace(/[\"|\'|\s]/g, "");
                };
                
                if (this.model.state.get('search')) {
                    peers = peers.filter(function(peer) {
                        // function in the DistributedSearchGroup mixin

                        if(peer.containsRole(this.model.state.get('rawSearch'))) {
                            return true;
                        }
                        
                        var searchables = [
                            peer.entry.content.get('peerName'),
                            peer.entry.content.get('host'),
                            peer.entry.content.get('host_fqdn'),
                            peer.entry.content.get('state'),
                            peer.entry.content.get('status-toggle')
                        ];
                        return _.reduce(searchables, function(memo, searchable) {
                            return memo || (searchable.toLowerCase()).indexOf(this.model.state.get('rawSearch').toLowerCase()) !== -1;
                        }, false, this);
                    }, this);
                }
                
                var totalPeers = peers.length || 0;
                
                // Client side pagination, if necessary:
                if (this.model.state.get('count') > 0 &&
                    this.model.state.get('count') < this.collection.peers.length) {

                    peers = peers.slice(
                        this.model.state.get('offset'),
                        this.model.state.get('offset') + this.model.state.get('count')
                    );
                }

                if (peers[0]) {
                    peers[0].paging.set('total', totalPeers);
                }
                this.collection.clientSidePeers.reset(peers);

                return _.flatten(
                    peers.map(function(model, i) {
                        return [
                            new TableRowView({
                                model: {
                                    appLocal: this.model.appLocal,
                                    application: this.model.application,
                                    peer: model,
                                    state: this.model.state
                                },
                                collection: {
                                    peers: this.collection.peers
                                },
                                index: i
                            }),
                            new MoreInfoView({
                                model: {
                                    peer: model
                                },
                                index: i
                            })
                        ];
                    }, this)
                );
            },
            _render: function() {
                _(this.children.rows).each(function(row){
                    row.render().appendTo(this.$('.managementconsole-listings'));
                }, this);
                this.children.tableDock.update();
            },
            renderRows: function() {
                 _(this.children.rows).each(function(row){ row.remove(); }, this);
                 this.children.rows = this.rowsFromCollection();
                 this._render();
            },
            render: function() {
                if (!this.el.innerHTML) {
                    if (this.children.caption) {
                        this.$el.append(this.children.caption.render().el);
                    }
                    this.$el.append(this.compiledTemplate({}));
                    this.children.head.render().prependTo(this.$('> .table-chrome'));
                }
                this._render();
                return this;
            },

            template: '\
                <table class="table table-chrome table-striped table-row-expanding table-listing">\
                <tbody class="managementconsole-listings"></tbody>\
                </table>\
            '
        });
    }
);
