define(
    [
        'jquery',
        'underscore',
        'module',
        'backbone',
        'collections/shared/FlashMessages',
        'views/shared/Modal',
        'views/shared/controls/ControlGroup',
        'views/managementconsole/table/controls/ConfirmationDialog',
        'views/shared/FlashMessagesLegacy',
        'splunk.config',
        'uri/route'
    ],
    function(
        $,
        _,
        module,
        Backbone,
        FlashMessagesCollection,
        Modal,
        ControlGroup,
        ConfirmationDialogView,
        FlashMessages,
        config,
        route
    ) {
    var BUTTON_DONE = '<a href="#" class="btn cancel modal-btn-primary btn-primary pull-right" data-dismiss="modal">' + _('Done').t() + '</a>',
    BUTTON_CANCEL = '<a href="#" class="btn cancel modal-btn-continue pull-right" data-dismiss="modal">' + _('Cancel').t() + '</a>';
        return Modal.extend({
            id: "edit-alerts-dialog",
            moduleId: module.id,
            initialize: function(options) {
                Modal.prototype.initialize.apply(this, arguments);
               
                this.model = this.model || {};
                
                // keeps track of current state of enabled alerts
                this.model.working = new Backbone.Model();
                
                // map to keep track of alert name to its cid
                this.model.cidMap = new Backbone.Model();
                this.collection = this.collection || {};

                this.collection.alerts = this.collection.alerts;
                this.collection.flashMessages = new FlashMessagesCollection();
               
                this.collection.alerts.each(function(model) {
                    var name = model.entry.get("name");
                    var cid = model.cid;
                    
                    var disabled = !model.entry.content.get("disabled");
                    this.model.working.set(name, disabled);
                    this.model.cidMap.set(name, cid);

                    // dnyamically create control group for alert name and its checkbox
                    this.children[name + '-alert-name-label'] = new ControlGroup({
                        controlType: "SyntheticCheckbox",
                        controlOptions: {
                            modelAttribute: name,
                            model: this.model.working
                        },
                        label: _(name).t()
                    });

                    // description label for each alert
                    this.children[name+'-alert-description-label'] = "<p class='alert-description'>" + model.entry.content.get("description") + "</p>";
                
                }, this);

                // error management
                this.children.flashMessage = new  FlashMessages({
                    collection: this.collection.flashMessages
                });
            },
            events: $.extend({}, Modal.prototype.events, {
                'click .btn-primary': function(e) {
                    e.preventDefault();
                    var dialog = this;

                    this.collection.flashMessages.reset();
                    
                    var enabledAlertsMap = this.model.working.toJSON();
                    var cids = this.model.cidMap.toJSON();
                    var deferredArr = [];

                    // iterate through alerts
                    _.each(this.model.working.keys(), function(name) {
                        var model = this.collection.alerts.get(cids[name]);
                        if(enabledAlertsMap[name]) {
                            model.entry.content.set("disabled", false);
                        } else {
                            model.entry.content.set("disabled", true);
                        }

                        deferredArr.push(model.save());
                    }, this);

                    // if any element failed to save, spit out error
                    $.when.apply($, deferredArr).done(function() {
                        dialog.remove();
                        dialog = new ConfirmationDialogView({
                            message: _("Your alerts have updated successfully.").t()
                        });
                        $('body').append(dialog.render().el);
                        dialog.show();
                    }.bind(dialog)).fail(function() {
                        this.collection.flashMessages.reset([{
                            type: 'error',
                            html: _("Sorry but your request could not be completed. Please try again later").t()
                        }]);
                    }.bind(dialog));
                }
            }),
            render: function() {
                var root = (config.MRSPARKLE_ROOT_PATH.indexOf("/") === 0 ? 
                    config.MRSPARKLE_ROOT_PATH.substring(1) : 
                    config.MRSPARKLE_ROOT_PATH
                );

                this.$el.html(Modal.TEMPLATE);
                this.$(Modal.HEADER_TITLE_SELECTOR).append(_("Enable/Disable Alerts").t());
                this.$(Modal.HEADER_SELECTOR).append("<div><p style='font-size: 10px;'>"+_("Your preconfigured alerts are listed here. ").t()+"<a style='font-weight:bold;' class='external' href='"+route.docHelp(root, config.LOCALE, "app.splunk_management_console.enable_alerts")+"' target='_blank'>"+_("Learn More").t()+"</a></p></div>");
                this.$(Modal.BODY_SELECTOR).prepend(this.children.flashMessage.render().el);
                this.$(Modal.BODY_SELECTOR).append(Modal.FORM_HORIZONTAL);

                // attach modal titles and fix positions
                this.$(Modal.BODY_FORM_SELECTOR).append(
                        "<div class='edit-alerts-modal-title'>" +
                            "<span class='alert-name-title'>"+_("Alert Name").t()+"</span>" +
                            "<span class='enabled-title'>"+_("Enabled").t()+"</span>" +
                        "</div>");
                
                // attach each alert label, option, and description
                _.each(_.keys(this.model.working.toJSON()), function(name) {
                    var elem = this.children[name+'-alert-name-label'].render().el;
                    this.$(Modal.BODY_FORM_SELECTOR).append(elem);
                    this.$(Modal.BODY_FORM_SELECTOR).children().last().append(elem);
                    var url_for_alert_edit = route.manager(root, config.LOCALE, 'splunk_management_console', ['saved', 'searches', 'DMC Alert - Critical System Physical Memory Usage'], {'data': {'ns': 'splunk_management_console', 'action': 'edit', 'uri': '/servicesNS/nobody/splunk_management_console/saved/searches/'+encodeURI(name)}});

                    this.$(Modal.BODY_FORM_SELECTOR).children().last().children().first().append("<a id='"+name+"-edit-link' href='"+url_for_alert_edit+"'' target='_blank'> [" + _("edit").t() + "]</a>");
                    this.$(Modal.BODY_FORM_SELECTOR).children().last().append(this.children[name+'-alert-description-label']);
                }, this);
                this.$(Modal.FOOTER_SELECTOR).append(Modal.BUTTON_CANCEL);
                this.$(Modal.FOOTER_SELECTOR).append(Modal.BUTTON_SAVE);
                
                return this;
            }
        });
    }
);
