define(
	[
		'jquery',
		'underscore',
		'backbone',
		'module',
		'collections/shared/FlashMessages',
		'views/shared/Modal',
		'views/shared/controls/ControlGroup',
		'views/managementconsole/table/controls/ConfirmationDialog',
		'views/managementconsole/table/controls/FailureDialog',
		'views/shared/FlashMessagesLegacy'
	],
	function(
		$,
		_,
		Backbone,
		module,
		FlashMessagesCollection,
		Modal,
		ControlGroup,
		ConfirmationDialog,
		FailureDialog,
		FlashMessagesView
	) {

		return Modal.extend({
			moduleId: module.id,
			initialize: function() {
				Modal.prototype.initialize.apply(this, arguments);

				this.model.working = new Backbone.Model({
					host: this.model.peer.entry.content.get('host') || '',
					hostFqdn: this.model.peer.entry.content.get('host_fqdn') || ''
				});
				this.collection = this.collectin || {};
				this.collection.flashMessages = new FlashMessagesCollection();

				this.children.hostField = new ControlGroup({
					label: _("Instance (host) name").t(),
					controlType: 'Text',
					controlOptions: {
						modelAttribute: 'host',
						model: this.model.working
					}
				});

				this.children.hostFqdnField = new ControlGroup({
					label: _("Machine name").t(),
					controlType: 'Text',
					controlOptions: {
						modelAttribute: 'hostFqdn',
						model: this.model.working
					}
				});

				this.children.flashMessage = new FlashMessagesView({
					collection: this.collection.flashMessages
				});
			},

			events: $.extend({}, Modal.prototype.events, {
				'click .btn-primary': function(e) {
					e.preventDefault();
					var that = this;

					that.collection.flashMessages.reset();

					var error = that.model.peer.entry.content.validate({
						'host': that.model.working.get('host'),
						'host_fqdn': that.model.working.get('hostFqdn')
					});

					if (error) {
						that.collection.flashMessages.reset([{
							type: 'error',
							html: error
						}]);
					} else {
						var oldHost = that.model.peer.entry.content.get('host');
						var oldHostFqdn = that.model.peer.entry.content.get('host_fqdn');
						that.model.peer.entry.content.set({
							'host': that.model.working.get('host'),
							'host_fqdn': that.model.working.get('hostFqdn')
						});


						that.model.peer.save().done(function() {
							that.model.state.set('changesMade', true);
							that.model.peer._hasOverrides = true;
							that.hide();
		            		var confirmationDialog = new ConfirmationDialog({
		            			message: _("Your instance name has updated successfully.").t()
		            		}).render();
		            		$('body').append(confirmationDialog.el);
		            		confirmationDialog.show();
						}.bind(that)).fail(function() {
							this.model.peer.entry.content.set('host', oldHost);
							this.model.peer.entry.content.set('host_fqdn', oldHostFqdn);
							
							that.hide();
	                    	var dialog = new FailureDialog().render();
	                    	$('body').append(dialog.el);
	                        dialog.show();
                   		}.bind(this));
					}


				}
			}),
			render: function() {
				this.$el.html(Modal.TEMPLATE);
	            this.$(Modal.HEADER_TITLE_SELECTOR).html(_("Edit Instance").t());
	            this.$(Modal.BODY_SELECTOR).prepend(this.children.flashMessage.render().el);
	            this.$(Modal.BODY_SELECTOR).append(Modal.FORM_HORIZONTAL);
	            this.$(Modal.BODY_FORM_SELECTOR).append(this.children.hostField.render().el);
	            this.$(Modal.BODY_FORM_SELECTOR).append(this.children.hostFqdnField.render().el);
	            this.$(Modal.FOOTER_SELECTOR).append(Modal.BUTTON_CANCEL);
	            this.$(Modal.FOOTER_SELECTOR).append(Modal.BUTTON_SAVE);
	            return this;
			}
		});

	}
);