/**
 * @author sfishel
 *
 * The view for editing a number cell pivot config form.
 */

define([
            'module',
            '../ConfigFormBase',
            '../../custom_controls/LabelControlGroup',
            '../../custom_controls/CellValueControlGroup',
            '../../custom_controls/CellDisplayControlGroup'
        ],
        function(
            module,
            ConfigFormBase,
            LabelControlGroup,
            CellValueControlGroup,
            CellDisplayControlGroup
        ) {

    return ConfigFormBase.extend({

        moduleId: module.id,

        /**
         * @constructor
         * @param options {Object} {
         *     model <models/pivot/elements/cells/NumberCell> the model to edit
         * }
         *
         * CellDisplayControlGroup is currently commented out because sparklines are not supported by the back end
         */

        initialize: function(options) {
            ConfigFormBase.prototype.initialize.call(this, options);
            this.children.labelControlGroup = new LabelControlGroup({ model: this.model });
            this.children.valueControlGroup = new CellValueControlGroup({ model: this.model });
//            this.children.displayControlGroup = new CellDisplayControlGroup({ model: this.model });

            this.model.on('change:value', function() {
                this.render();
                this.trigger('changeContents');
            }, this);
        },

        render: function() {
            // have to detach the control groups so their event listeners aren't destroyed
            this.children.labelControlGroup.$el.detach();
            this.children.valueControlGroup.$el.detach();
//            this.children.displayControlGroup.$el.detach();

            this.$el.append(this.children.labelControlGroup.render().el);
            this.$el.append(this.children.valueControlGroup.render().el);
//            if(!(this.model.get('value') in { 'list': true, 'values': true })) {
//                this.$el.append(this.children.displayControlGroup.render().el);
//            }
            return this;
        }

    });

});