/**
 * @author sfishel
 *
 * The view for a number column-split pivot config form.
 */

define([
            'module',
            '../ConfigFormBase',
            '../../custom_controls/CreateRangesControlGroup',
            '../../custom_controls/MaxRangesControlGroup',
            '../../custom_controls/RangeSizeControlGroup',
            '../../custom_controls/RangeStartControlGroup',
            '../../custom_controls/RangeEndControlGroup'
        ],
        function(
            module,
            ConfigFormBase,
            CreateRangesControlGroup,
            MaxRangesControlGroup,
            RangeSizeControlGroup,
            RangeStartControlGroup,
            RangeEndControlGroup
        ) {

    return ConfigFormBase.extend({

        moduleId: module.id,

        /**
         * @constructor
         * @param options {Object} {
                 *     model <models/pivot/elements/columns/NumberColumn> the model to edit
                 * }
         */

        initialize: function(options) {
            ConfigFormBase.prototype.initialize.call(this, options);
            this.children.splitControlGroup = new CreateRangesControlGroup({ model: this.model });
            this.children.rangeMaxNumControlGroup = new MaxRangesControlGroup({ model: this.model });
            this.children.rangeSizeControlGroup = new RangeSizeControlGroup({ model: this.model });
            this.children.rangeStartControlGroup = new RangeStartControlGroup({ model: this.model });
            this.children.rangeEndControlGroup = new RangeEndControlGroup({ model: this.model });

            this.model.on('change:display', function() {
                this.render();
                this.trigger('changeContents');
            }, this);
        },

        render: function() {
            // have to detach the control groups so their event listeners aren't destroyed
            this.children.rangeMaxNumControlGroup.$el.detach();
            this.children.rangeSizeControlGroup.$el.detach();
            this.children.rangeStartControlGroup.$el.detach();
            this.children.rangeEndControlGroup.$el.detach();

            this.$el.append(this.children.splitControlGroup.render().el);
            if(this.model.get('display') === 'ranges') {
                this.$el.append(this.children.rangeMaxNumControlGroup.render().el);
                this.$el.append(this.children.rangeSizeControlGroup.render().el);
                this.$el.append(this.children.rangeStartControlGroup.render().el);
                this.$el.append(this.children.rangeEndControlGroup.render().el);
            }
            return this;
        }

    });

});