/**
 * @author sfishel
 *
 * The view for editing a number filter pivot config form.
 */

define([
            'underscore',
            'module',
            '../ConfigFormBase',
            '../../custom_controls/FilterTypeControlGroup',
            '../../custom_controls/FilterMatchControlGroup',
            '../../custom_controls/FilterLimitByControlGroup',
            '../../custom_controls/FilterLimitControlGroup'
        ],
        function(
            _,
            module,
            ConfigFormBase,
            FilterTypeControlGroup,
            FilterMatchControlGroup,
            FilterLimitByControlGroup,
            FilterLimitControlGroup
        ) {

    return ConfigFormBase.extend({

        moduleId: module.id,

        /**
         * @constructor
         * @param options {Object} {
         *     model {
         *         element <models/pivot/elements/filters/StringFilter> the model to edit
         *         report: <models/pivot/PivotReport> the pivot report model
         *         dataModel: <models/services/datamodel/DataModel> the data model being reported on
         *         application: <models/shared/Application> the application state model
         *     }
         * }
         */

        initialize: function(options) {
            ConfigFormBase.prototype.initialize.call(this, options);
            this.children.typeControlGroup = new FilterTypeControlGroup({ model: this.model });
            this.children.matchControlGroup = new FilterMatchControlGroup({
                model: this.model,
                report: this.options.report,
                dataModel: this.options.dataModel,
                application: this.options.application
            });
            this.children.limitByControlGroup = new FilterLimitByControlGroup({
                model: this.model,
                report: this.options.report,
                dataModel: this.options.dataModel
            });
            this.children.limitControlGroup = new FilterLimitControlGroup({ model: this.model });

            this.model.on('change:filterType change:limitBy', this.debouncedRender, this);
        },

        render: function() {
            // have to detach the control groups so their event listeners aren't destroyed
            this.children.typeControlGroup.$el.detach();
            this.children.matchControlGroup.$el.detach();
            this.children.limitByControlGroup.$el.detach();
            this.children.limitControlGroup.$el.detach();

            this.$el.append(this.children.typeControlGroup.render().el);
            if(this.model.get('filterType') === 'match') {
                this.$el.append(this.children.matchControlGroup.render().el);
                this.children.matchControlGroup.$('.control').addClass('inline-control');
            }
            else {
                this.$el.append(this.children.limitByControlGroup.render().el);
                this.$el.append(this.children.limitControlGroup.render().el);
            }
            return this;
        }

    });

});