/**
 * @author sfishel
 */

define([
            './filters/StringFilterForm',
            './filters/NumberFilterForm',
            './filters/BooleanFilterForm',
            './filters/TimestampFilterForm',

            './cells/StringCellForm',
            './cells/NumberCellForm',
            './cells/TimestampCellForm',
            './cells/ObjectCountCellForm',

            './rows/StringRowForm',
            './rows/NumberRowForm',
            './rows/BooleanRowForm',
            './rows/TimestampRowForm',

            './columns/StringColumnForm',
            './columns/NumberColumnForm',
            './columns/BooleanColumnForm',
            './columns/TimestampColumnForm',

            'models/pivot/elements/BaseElement'
        ],
        function(
            StringFilter,
            NumberFilter,
            BooleanFilter,
            TimestampFilter,

            StringCell,
            NumberCell,
            TimestampCell,
            ObjectCountCell,

            StringRow,
            NumberRow,
            BooleanRow,
            TimestampRow,

            StringColumn,
            NumberColumn,
            BooleanColumn,
            TimestampColumn,

            BaseElementModel
        ) {

    var FACTORY_SCHEMA = {

        'filter': {
            'string': StringFilter,
            'number': NumberFilter,
            'boolean': BooleanFilter,
            'timestamp': TimestampFilter,
            'ipv4': StringFilter
        },

        'cell': {
            'string': StringCell,
            'number': NumberCell,
            'timestamp': TimestampCell,
            'objectCount': ObjectCountCell,
            'childCount': ObjectCountCell,
            'ipv4': StringCell
        },

        'row': {
            'string': StringRow,
            'number': NumberRow,
            'boolean': BooleanRow,
            'timestamp': TimestampRow,
            'ipv4': StringRow
        },

        'column': {
            'string': StringColumn,
            'number': NumberColumn,
            'boolean': BooleanColumn,
            'timestamp': TimestampColumn,
            'ipv4': StringColumn
        }

    };

    return ({

        /**
         * A factory for constructing the corresponding pivot form view for the given options.
         *
         * @param options the options to pass to the view constructor, consists of:
         *
         *      model {required} the pivot element model, its elementType and type attributes will be used to
         *                          select the correct view
         *
         * @return a pivot form view instance of the corresponding type
         */

        create: function(options) {
            options = options || {};
            var model = options.model;
            if(!(model instanceof BaseElementModel)) {
                throw 'options.model must an instance of models/pivot/elements/BaseElement';
            }

            var elementType = model.get('elementType');
            if(!FACTORY_SCHEMA.hasOwnProperty(elementType)) {
                throw ('un-supported element type ' + elementType);
            }

            var dataType = model.get('type');
            if(!FACTORY_SCHEMA[elementType].hasOwnProperty(dataType)) {
                throw ('un-supported elementType - dataType combination ' + elementType + ', ' + dataType);
            }

            var viewConstructor = FACTORY_SCHEMA[elementType][dataType];
            return new viewConstructor(options);
        }

    });

});