define([
            'jquery',
            'backbone',
            'module',
            'views/pivot/content/ContentBase',
            'views/shared/JSChart'
        ],
        function(
            $,
            Backbone,
            module,
            ContentBase,
            JSChart
        ) {

    var GAUGE_MAX_HEIGHT = 500;

    return ContentBase.extend({

        moduleId: module.id,

        initialize: function() {
            ContentBase.prototype.initialize.call(this, this.options);
            this.children.jsChart = new JSChart({
                model: {
                    searchResultsColumn: this.model.results,
                    state: this.model.report.entry.content,
                    application: this.model.application
                }
            });

             this.children.jsChart.on('chartClick', function(clickInfo) {
                this.model.results.trigger('drilldown', clickInfo);
            }, this);
        },

        renderData: function() {
            this.children.jsChart.render().appendTo(this.$el);
            this.reflow();
        },

        clearData: function() {
            this.children.jsChart.remove();
        },

        showDataDisplay: function() {
            this.children.jsChart.wake().$el.show();
        },

        hideDataDisplay: function() {
            this.children.jsChart.sleep().$el.hide();
        },

        reflow: function() {
            var flashMessagesHeight = this.flashMessages ? this.flashMessages.$el.outerHeight() : 0,
                chartHeight = this.$el.height() - flashMessagesHeight,
                chartWidth = $('body').width() - this.$el.offset().left,
                vizType = this.model.report.getVisualizationType();

            if(vizType === 'radialGauge' || vizType === 'markerGauge' || vizType === 'fillerGauge') {
                if(chartHeight > GAUGE_MAX_HEIGHT) {
                    chartHeight = GAUGE_MAX_HEIGHT;
                }
            }
            this.children.jsChart.$el.css({
                width: chartWidth,
                height: chartHeight,
                top: flashMessagesHeight
            });
            ContentBase.prototype.reflow.call(this);
        }

    });

});