define([
            'jquery',
            'underscore',
            'module',
            'views/Base',
            'helpers/user_agent'
        ],
        function(
            $,
            _,
            module,
            Base,
            userAgent
        ) {

    return Base.extend({

        moduleId: module.id,

        initialize: function() {
            Base.prototype.initialize.apply(this, arguments);
            this.flashMessages = this.options.flashMessages;
            if(this.flashMessages) {
                this.flashMessages.flashMsgCollection.on('add remove reset', this.debouncedReflow, this);
            }

            this.model.job.entry.content.on('change', this.updateData, this);
        },

        render: function() {
            this.$el.empty();
            this.renderContainer();
            this.renderData();
            this.updateData();
            return this;
        },

        renderContainer: function() {
            if(this.flashMessages) {
                this.flashMessages.render().prependTo(this.$el);
            }
        },

        updateData: function() {
            this.$('.data-display-alert').remove();
            var hasResults = !this.model.job.isPreparing() && this.model.job.resultCountSafe() > 0,
                jobContent = this.model.job.entry.content;

            if(hasResults) {
                this.showDataDisplay();
            }
            else if(!jobContent.get('isFailed') && !jobContent.get('isDone')) {
                this.hideDataDisplay();
                if(_.isEmpty(jobContent.toJSON())) {
                    return;
                }
                if(this.model.job.isQueued()) {
                    this.renderMessage('info', _('Waiting for queued job to start.').t());
                }
                else if(this.model.job.isParsing()) {
                    this.renderMessage('info', _('Parsing search.').t());
                }
                else if(this.model.job.isFinalizing()) {
                    this.renderMessage('info', _('Finalizing results.').t());
                }
                // the job is running
                else {
                    if(this.model.job.isRealtime()) {
                        this.renderMessage('info', _('No results yet found.').t());
                    }
                    else {
                        this.renderMessage('info', _('Waiting for results...').t());
                    }
                }
            }
            else {
                this.hideDataDisplay();
                this.renderMessage('error', _('Your search returned no results.').t());
            }
            this.trigger('updateData');
        },

        renderMessage: function(type, text) {
            $(_(this.messageTemplate).template({ type: type, text: text })).appendTo(this.$el);
        },

        reflow: function() {
            if(userAgent.isIE7()) {
                this.el.className = this.el.className;
            }
            Base.prototype.reflow.apply(this, arguments);
        },

        disable: function() {},

        messageTemplate: '\
            <div class="alert alert-<%- type %> data-display-alert">\
                <i class="icon-alert"></i>\
                <%- text %>\
            </div>\
        '

    });

});