define([
            'jquery',
            'underscore',
            'module',
            'views/pivot/content/ContentBase',
            'views/shared/statscontrols/StatsControlsMaster',
            'views/pivot/PivotElementsDisplay',
            'views/shared/results_table/LazyResultsTable'
        ],
        function(
            $,
            _,
            module,
            ContentBase,
            StatsControlsMaster,
            PivotElementsDisplay,
            LazyResultsTable
        ) {

    return ContentBase.extend({

        moduleId: module.id,

        initialize: function(options) {
            ContentBase.prototype.initialize.call(this, options);
            options = options || {};

            this.children.tableFormatBar = new StatsControlsMaster({
                model: {
                    report: this.model.report.entry.content,
                    metadata: this.model.results.fetchData,
                    job: this.model.job
                }
            });

            this.children.elementsDisplay = new PivotElementsDisplay({
                model: {
                    report: this.model.report,
                    dataModel: this.model.dataModel,
                    application: this.model.application,
                    appLocal: this.model.appLocal,
                    user: this.model.user
                },
                collection: {
                    timePresets: this.collection.timePresets
                }
            });

            this.children.resultsTable = new LazyResultsTable({
                model: {
                    results: this.model.results,
                    metadata: this.model.results.fetchData,
                    format: this.model.report.entry.content
                },
                enableTableDock: false,
                enableStaticHeader: true,
                numRowSplits: this.model.report.getNumRows()
            });

            this.children.resultsTable.on('cellClick', function(pointInfo) {
                this.model.results.trigger('drilldown', pointInfo);
            }, this);
        },

        renderContainer: function() {
            ContentBase.prototype.renderContainer.apply(this, arguments);
            this.children.elementsDisplay.render().appendTo(this.$el);
            this.children.tableFormatBar.render().appendTo(this.$el);
        },

        renderData: function() {
            this.children.resultsTable.load().activate().render().insertAfter(this.children.elementsDisplay.$el);
            $.when(this.children.resultsTable.wrappedViewLoaded()).done(_.bind(this.reflow, this));
        },

        clearData: function() {
            this.children.resultsTable.remove();
        },

        disable: function() {
            this.children.elementsDisplay.disable();
        },

        showDataDisplay: function() {
            this.children.resultsTable.wake().$el.show();
        },

        hideDataDisplay: function() {
            this.children.resultsTable.sleep().$el.hide();
        },

        renderMessage: function(type, text) {
            var messageHtml = _(this.messageTemplate).template({ type: type, text: text });
            this.children.elementsDisplay.$el.after(messageHtml);
        },

        reflow: function() {
            var flashMessagesHeight = this.flashMessages ? this.flashMessages.$el.outerHeight() : 0,
                elementsDisplayHeight = this.children.elementsDisplay.$el.outerHeight();

            this.children.resultsTable.$el.css({
                top: flashMessagesHeight + elementsDisplayHeight
            });
            ContentBase.prototype.reflow.call(this);
        }

    });

});