/**
 * @author sfishel
 *
 * A custom sub-class of ControlGroup for pivot filter config forms.
 *
 * Can create a drop-down list of comparator options and a either a text box or drop-down list for the compare to value.
 */

define([
            'jquery',
            'underscore',
            'module',
            'views/shared/controls/ControlGroup',
            'views/shared/controls/Control',
            'views/shared/controls/TextControl',
            'views/shared/delegates/Popdown',
            'util/pivot/config_form_utils'
        ],
        function(
            $,
            _,
            module,
            ControlGroup,
            Control,
            TextControl,
            Popdown,
            configFormUtils
        ) {

    var COMPARATOR_ITEMS = {

        string: [
            {
                value: 'is',
                label: configFormUtils.filterRuleToDisplay('is')
            },
            {
                value: 'contains',
                label: configFormUtils.filterRuleToDisplay('contains')
            },
            {
                value: 'isNot',
                label: configFormUtils.filterRuleToDisplay('isNot')
            },
            {
                value: 'doesNotContain',
                label: configFormUtils.filterRuleToDisplay('doesNotContain')
            },
            {
                value: 'startsWith',
                label: configFormUtils.filterRuleToDisplay('startsWith')
            },
            {
                value: 'endsWith',
                label: configFormUtils.filterRuleToDisplay('endsWith')
            },
            {
                value: 'isNull',
                label: configFormUtils.filterRuleToDisplay('isNull')
            },
            {
                value: 'isNotNull',
                label: configFormUtils.filterRuleToDisplay('isNotNull')
            }
            // regex is not supported by the back end
//            {
//                value: 'regex',
//                label: configFormUtils.filterRuleToDisplay('regex')
//            }
        ],

        ipv4: [
            {
                value: 'is',
                label: configFormUtils.filterRuleToDisplay('is')
            },
            {
                value: 'contains',
                label: configFormUtils.filterRuleToDisplay('contains')
            },
            {
                value: 'isNot',
                label: configFormUtils.filterRuleToDisplay('isNot')
            },
            {
                value: 'doesNotContain',
                label: configFormUtils.filterRuleToDisplay('doesNotContain')
            },
            {
                value: 'startsWith',
                label: configFormUtils.filterRuleToDisplay('startsWith')
            },
            {
                value: 'isNull',
                label: configFormUtils.filterRuleToDisplay('isNull')
            },
            {
                value: 'isNotNull',
                label: configFormUtils.filterRuleToDisplay('isNotNull')
            }
            // regex is not supported by the back end
//            {
//                value: 'regex',
//                label: configFormUtils.filterRuleToDisplay('regex')
//            }
        ],

        number: [
            {
                value: '=',
                label: configFormUtils.filterRuleToDisplay('=')
            },
            {
                value: '!=',
                label: configFormUtils.filterRuleToDisplay('!=')
            },
            {
                value: '<=',
                label: configFormUtils.filterRuleToDisplay('<=')
            },
            {
                value: '<',
                label: configFormUtils.filterRuleToDisplay('<')
            },
            {
                value: '>=',
                label: configFormUtils.filterRuleToDisplay('>=')
            },
            {
                value: '>',
                label: configFormUtils.filterRuleToDisplay('>')
            },
            {
                value: 'isNull',
                label: configFormUtils.filterRuleToDisplay('isNull')
            },
            {
                value: 'isNotNull',
                label: configFormUtils.filterRuleToDisplay('isNotNull')
            }
        ]
    };

    var BOOLEAN_COMPARE_TO_ITEMS = [
        {
            value: 'true',
            label: configFormUtils.filterRuleToDisplay('true')
        },
        {
            value: 'false',
            label: configFormUtils.filterRuleToDisplay('false')
        },
        {
            value: 'isNull',
            label: configFormUtils.filterRuleToDisplay('isNull')
        },
        {
            value: 'isNotNull',
            label: configFormUtils.filterRuleToDisplay('isNotNull')
        }
    ];

    var CompareToTextControl = TextControl.extend({

        initialize: function(options) {
            this.options.append = this.sampleValuesToggleTemplate;
            TextControl.prototype.initialize.call(this, this.options);
            this.children.sampleValues = new Popdown({ el: this.el, attachDialogTo: 'body' });
            this.children.sampleValues.on('show', this.onSampleValuesMenuShow, this);
            this.children.sampleValues.on('hide', this.onSampleValuesMenuHide, this);
        },

        render: function() {
            TextControl.prototype.render.apply(this, arguments);
            this.$el.append(this.sampleValuesMenuTemplate);
            this.$sampleValuesMenu = this.$('.sample-values-menu');
            return this;
        },

        onSampleValuesMenuShow: function() {
            var objectName = this.options.report.getPivotObjectName(),
                object = this.options.dataModel.objectByName(objectName),
                sampleValues = object.getSampleValuesModel(
                    this.model.get('fieldName'),
                    this.model.get('owner'),
                    {
                        data: {
                            app: this.options.application.get('app'),
                            owner: this.options.application.get('owner')
                        }
                    }
                );

            if(sampleValues.has('values')) {
                this.renderSampleValues(sampleValues);
            }
            else {
                this.$sampleValuesMenu.find(':not(.arrow)').remove();
                this.$sampleValuesMenu.append(_(this.loadingSampleValuesTemplate).template({}));
                sampleValues.once('change:values', this.renderSampleValues, this);
            }

            var that = this;
            this.$sampleValuesMenu.on('click', '.sample-value', function(e) {
                e.preventDefault();
                var value = $(e.currentTarget).attr('data-value');
                that.setValue(value, true);
            });
        },

        onSampleValuesMenuHide: function() {
            this.$sampleValuesMenu.off();
        },

        renderSampleValues: function(sampleValues) {
            this.$sampleValuesMenu.find(':not(.arrow)').remove();
            this.$sampleValuesMenu.append(
                _(this.sampleValuesContentTemplate).template({ limit: sampleValues.get('limit'), values: sampleValues.get('values') })
            );
        },

        sampleValuesToggleTemplate: '\
            <a href="#" class="add-on btn dropdown-toggle">\
                <i class="icon-triangle-down-small"></i>\
            </a>\
        ',

        sampleValuesMenuTemplate: '\
            <div class="sample-values-menu dropdown-menu">\
                <div class="arrow"></div>\
            </div>\
        ',

        loadingSampleValuesTemplate: '\
            <ul>\
                <li><a href="#" class="disabled"><%- _("Loading...").t() %></a></li>\
            </ul>\
        ',

        sampleValuesContentTemplate: '\
            <ul>\
                <% if(values.length > 0) { %>\
                    <% _(values).each(function(value) { %>\
                        <li class="sample-value" data-value="<%- value %>">\
                            <a href="#"><%- value %></a>\
                        </li>\
                    <% }); %>\
                    <% if(values.length == limit) { %>\
                        <li class="sample-value-limit">\
                            Displaying top <%- limit %> values\
                        </li>\
                    <% } %>\
                <% } else { %>\
                    <ul>\
                        <li><a href="#" class="disabled"><%- _("No Sample Values found.").t() %></a></li>\
                    </ul>\
                <% } %>\
            </ul>\
        '

    });

    return ControlGroup.extend({

        moduleId: module.id,

        /**
         * @constructor
         * @param options {Object} {
         *     model {Model} the pivot element model to operate on
         *     report <models/pivot/PivotReport> the current pivot report
         *     dataModel <models/services/datamodel/DataModel> the current data model
         *     application: <models/shared/Application> the application state model
         * }
         */

        initialize: function() {
            var controls = [],
                dataType = this.model.get('type');

            if(dataType === 'string' || dataType === 'ipv4' || dataType === 'number') {
                controls.push({
                    type: 'SyntheticSelect',
                    options: {
                        className: Control.prototype.className + ' input-prepend',
                        model: this.model,
                        modelAttribute: 'ruleComparator',
                        toggleClassName: 'btn',
                        menuWidth: 'narrow',
                        items: COMPARATOR_ITEMS[dataType],
                        popdownOptions: {
                            attachDialogTo: 'body'
                        }
                    }
                });
            }
            if(dataType === 'boolean') {
                controls.push({
                    type: 'SyntheticSelect',
                    options: {
                        model: this.model,
                        modelAttribute: 'ruleCompareTo',
                        toggleClassName: 'btn',
                        menuWidth: 'narrow',
                        items: BOOLEAN_COMPARE_TO_ITEMS,
                        popdownOptions: {
                            attachDialogTo: 'body'
                        }
                    }
                });
            }
            else {
                controls.push(new CompareToTextControl({
                    className: Control.prototype.className + ' input-prepend',
                    model: this.model,
                    modelAttribute: 'ruleCompareTo',
                    inputClassName: 'input-small',
                    report: this.options.report,
                    dataModel: this.options.dataModel,
                    application: this.options.application
                }));
            }

            this.options.label = _('Match').t();
            this.options.controls = controls;
            ControlGroup.prototype.initialize.call(this, this.options);

            // set up references to each control
            this.ruleComparatorControl = this.childList[0];
            if(this.childList.length > 1) {
                this.ruleCompareToControl = this.childList[1];
            }
            this.model.on('change:ruleComparator', this.handleComparatorChange, this);
        },

        render: function() {
            ControlGroup.prototype.render.apply(this, arguments);
            this.handleComparatorChange();
            return this;
        },

        handleComparatorChange: function() {
            if(!this.ruleCompareToControl) {
                return;
            }
            if(this.model.get('ruleComparator') in { isNull: true, isNotNull: true }) {
                this.ruleCompareToControl.detach();
                this.ruleComparatorControl.$el.removeClass('input-prepend');
            }
            else {
                this.ruleCompareToControl.insertAfter(this.ruleComparatorControl.el);
                this.ruleComparatorControl.$el.addClass('input-prepend');
            }
        }

    });

});