define(
    [
         'underscore',
         'jquery',
         'backbone',
         'module',
         'views/Base',
         'views/shared/searchbar/Master',
         'views/search/title/Master',
         'views/search/actions/openinpivotmenu/Master',
         'views/shared/FlashMessages',
         'views/search/results/Master',
         'views/search/initialhelp/Master',
         'views/search/initialhelplite/Master',
         'util/splunkd_utils'
    ],
    function(
        _,
        $,
        Backbone,
        module,
        Base,
        SearchBar,
        Title,
        OpenInPivotMenu,
        FlashMessage,
        ResultsContainer,
        InitialHelp,
        InitialHelpLite,
        splunkd_utils
    ){
        return Base.extend({
            moduleId: module.id,
            initialize: function(){
                Base.prototype.initialize.apply(this, arguments);

                this.errorTypes = [splunkd_utils.FATAL, splunkd_utils.ERROR, splunkd_utils.NOT_FOUND];
                                
                //views
                this.children.title = new Title({
                    model: {
                        report: this.model.report,
                        reportPristine: this.model.reportPristine,
                        application: this.model.application,
                        searchJob: this.model.searchJob,
                        appLocal: this.model.appLocal,
                        user: this.model.user,
                        serverInfo: this.model.serverInfo
                    },
                    collection:  {
                        eventRenderers: this.collection.eventRenderers
                    }
                });
                
                this.children.searchBar = new SearchBar({
                    model: {
                        state: this.model.report.entry.content,
                        timeRange: this.model.timeRange,
                        user: this.model.user,
                        appLocal: this.model.appLocal,
                        application: this.model.application,
                        serverInfo: this.model.serverInfo
                    },
                    collection: {
                        times: this.collection.times
                    },
                    disableOnSubmit: true,
                    primaryAction: false
                });
                
                this.children.flashMessage = new FlashMessage({
                    model: {
                        searchJob: this.model.searchJob,
                        searchJobControl: this.model.searchJob.control,
                        report: this.model.report
                    },
                    whitelist: this.errorTypes
                });

                if (this.model.serverInfo.isLite()){
                    this.children.initialHelp = new InitialHelpLite({
                        model: {
                            report: this.model.report,
                            metaDataResult: this.model.metaDataResult,
                            application: this.model.application,
                            appLocal: this.model.appLocal,
                            serverInfo: this.model.serverInfo,
                            user: this.model.user,
                            userPrefs: this.model.userPrefs
                        }
                    });
                } else {
                    this.children.initialHelp = new InitialHelp({
                        model: {
                            report: this.model.report,
                            metaDataResult: this.model.metaDataResult,
                            application: this.model.application,
                            appLocal: this.model.appLocal,
                            serverInfo: this.model.serverInfo
                        }
                    });
                }

                this.children.resultsContainer = new ResultsContainer({
                    model: {
                        result: this.model.result,
                        summary: this.model.summary,
                        timeline: this.model.timeline,
                        searchJob: this.model.searchJob,
                        report: this.model.report,
                        reportPristine: this.model.reportPristine,
                        resultJsonCols: this.model.resultJsonCols,
                        resultJsonRows: this.model.resultJsonRows,
                        application: this.model.application,
                        appLocal: this.model.appLocal,
                        user: this.model.user,
                        serverInfo: this.model.serverInfo
                    },
                    collection:  {
                        selectedFields: this.collection.selectedFields,
                        eventRenderers: this.collection.eventRenderers,
                        workflowActions: this.collection.workflowActions
                    },
                    showAutoPause: this.options.showAutoPause
                });
                                
                //TODO: when everyone moves to activate/deactivate and do not have activate in initialize
                //we will not have to do this.
                _.each(this.children, function(child, key){
                    child.deactivate({deep: true});
                });
            },
            startListening: function() {
                this.listenTo(this.model.searchJob, 'change:id', function() {
                    if (this.active) {
                        this.hasResultsContainer = !!this.model.searchJob.id;
                        this.manageStateOfChildren();
                    }
                });
            },
            activate: function(options) {
                var clonedOptions = _.extend({}, (options || {}));
                delete clonedOptions.deep;
                
                this.ensureDeactivated({deep: true});
                this.options.showAutoPause = clonedOptions.showAutoPause;
                
                this.isError = splunkd_utils.messagesContainsOneOfTypes(this.model.report.error.get("messages"), this.errorTypes) ||
                    splunkd_utils.messagesContainsOneOfTypes(this.model.searchJob.error.get("messages"), this.errorTypes);
                
                // Activate/Deactivate views
                this.hasResultsContainer = this.model.report.entry.content.get('search') || this.isError;
                
                this.children.title.activate();
                this.children.searchBar.activate({deep: true});
                this.children.flashMessage.activate({deep: true});
                
                this.manageStateOfChildren();
                this.listenTo(this.model.report, 'openInPivot', this.showOpenInPivotMenu);

                return Base.prototype.activate.call(this, clonedOptions);
            },
            deactivate: function(options) {
                if (!this.active) {
                    return Base.prototype.deactivate.apply(this, arguments);
                }
                Base.prototype.deactivate.apply(this, arguments);
                this.isError = false;
                return this;
            },
            manageStateOfChildren: function() {
                if (this.hasResultsContainer){
                    this.children.initialHelp.deactivate({deep: true}).$el.hide();
                    this.children.resultsContainer.options.showAutoPause = this.options.showAutoPause;
                    this.children.resultsContainer.activate().$el.show();
                } else {
                    this.children.resultsContainer.deactivate({deep: true}).$el.hide();
                    this.children.initialHelp.activate({deep: true}).$el.show();
                }                
            },
            showOpenInPivotMenu: function() {
                this.children.openInPivotMenu = new OpenInPivotMenu({
                    model: {
                        summary: this.model.summary,
                        searchJob: this.model.searchJob,
                        application: this.model.application
                    },
                    collection: {
                        selectedFields: this.collection.selectedFields
                    },
                    onHiddenRemove: true
                });
                this.children.openInPivotMenu.render().appendTo($('body')).show();
            },
            render: function() {
                var html = this.$el.html();
                if (!html) {
                    this.$el.html(_.template(this.template));
                    var header = this.$(".section-header");
                    this.children.title.render().appendTo(header);
                    this.children.searchBar.render().appendTo(header);
                    this.children.flashMessage.render().appendTo(header);
                    this.children.initialHelp.render().appendTo(this.$el);
                    this.children.resultsContainer.render().appendTo(this.$el);
                    
                    if (this.hasResultsContainer){
                        this.children.initialHelp.$el.hide();
                    } else {
                        this.children.resultsContainer.$el.hide();
                    }
                }

                return this;
            },
            template: '\
                <div class="section-padded section-header">\
                </div>\
            '
        });
    }
);