define(
    [
        'underscore',
        'module',
        'views/shared/Modal',
        'models/Base',
        'models/services/saved/EventType',
        'models/services/configs/EventRenderer',
        'views/shared/controls/ControlGroup',
        'views/search/actions/eventtype/Create',
        'views/search/actions/eventtype/Success'
    ],
    function(_, module, Modal, BaseModel, EventType, EventRenderer, ControlGroup, EventTypeCreate, EventTypeSuccess){
        return Modal.extend({
            moduleId: module.id,
            className: 'modal',
            initialize: function() {
                Modal.prototype.initialize.apply(this, arguments);
                this.model.eventType = new EventType();
                this.model.eventRenderer = new EventRenderer();
                this.model.state = new BaseModel();

                //set defaults
                this.model.eventType.entry.content.set('priority', 5);
                this.model.eventRenderer.entry.content.set('css_class', '');
                
                this.children.create  = new EventTypeCreate({
                    model: {
                        eventType: this.model.eventType,
                        eventRenderer: this.model.eventRenderer,
                        application: this.model.application,
                        state: this.model.state
                    },
                    collection: {
                        eventRenderers: this.collection.eventRenderers
                    },
                    searchString: this.options.searchString
                });
                
                this.children.success = new EventTypeSuccess({
                    model: {
                        eventType: this.model.eventType,
                        state: this.model.state,
                        application: this.model.application,
                        user: this.model.user
                    }
                });

                this.listenTo(this.model.eventType, 'success', function(){
                    this.children.create.$el.hide();
                    this.children.success.$el.show();                    
                });
                this.listenTo(this.model.eventType, 'close', function(){
                    this.$el.modal('hide');
                });
            },
            render: function() {
                this.children.create.render().appendTo(this.$el);
                this.children.success.render().appendTo(this.$el);
                this.children.success.$el.hide();
            }
        }
    );
});
