define(
    [
        'underscore',
        'jquery',
        'module',
        'views/Base',
        'views/search/initialhelp/What',
        'uri/route',
        'splunk.util'
    ],
    function(_, $, module, Base, What, route, splunkUtil) {
        return Base.extend({
            moduleId: module.id,
            className:'section-content',
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);

                this.children.what = new What({
                    model: {
                        report: this.model.report,
                        application: this.model.application,
                        metaDataResult: this.model.metaDataResult,
                        serverInfo: this.model.serverInfo
                    }
                });
            },
            activate: function() {
                if (this.active) {
                    return Base.prototype.activate.apply(this, arguments);
                }

                var enableMetaData = splunkUtil.normalizeBoolean(this.model.report.entry.content.get('display.prefs.enableMetaData')),
                    showDataSummary = splunkUtil.normalizeBoolean(this.model.report.entry.content.get('display.prefs.showDataSummary'));

                if (enableMetaData || showDataSummary) {
                    this.children.what.activate().$el.show();
                } else {
                    this.children.what.deactivate().$el.hide();
                }

                return Base.prototype.activate.call(this, arguments);
            },
            render: function() {
                var template = _.template(this.template),
                    root = this.model.application.get("root"),
                    locale = this.model.application.get("locale"),
                    app = this.model.application.get("app"),
                    version = this.model.appLocal.entry.content.get('version'),
                    appAllowsDisable = this.model.appLocal.appAllowsDisable();

                this.$el.html(this.compiledTemplate({
                    _: _,
                    docRoute: route.docHelp(root, locale, 'splunkcore.homepage.docs'),
                    tutorialDocRoute: route.docHelp(root, locale, 'search_app.tutorial')
                }));

                this.children.what.render().appendTo(this.$el);

                return this;
            },
            template: '\
                <div class="column column-how">\
                    <h3><%- _("How to Search").t() %></h3>\
                    <p><%- _("If you aren\'t familiar with searching in Splunk, or want to learn more, checkout one of the following resources.").t() %></p>\
                    <a href="<%- docRoute %>" target="_blank" title="<%- _("Splunk help").t() %>" class="btn btn-documentation" style="margin-bottom: 10px"><%- _("Documentation").t() %> <i class="icon-external"></i></a>\
                    <a href="<%- tutorialDocRoute %>" target="_blank" title="<%- _("Splunk help").t() %>" class="btn btn-documentation" style="margin-bottom: 10px"><%- _("Tutorial").t() %> <i class="icon-external"></i></a>\
                </div>\
            '
        });
    }
);
