define(
    [
        'underscore',
        'jquery',
        'module',
        'jquery.ui.draggable',
        'models/services/search/IntentionsParser',
        'models/services/search/jobs/ResultJsonCols',
        'models/search/Job',
        'views/Base',
        'views/shared/vizcontrols/Master',
        'views/shared/JSChartDrilldown',
        'views/shared/SingleValue',
        'views/shared/LazyMap',
        'views/shared/results_table/LazyResultsTable',
        'views/search/results/shared/NoStatistics',
        'views/shared/JobDispatchState',
        'util/drilldown_utils'
    ],
    function(_, $, module, jqueryDraggable, IntentionsParser, ResultJsonCols, Job, Base, VizEditor, JSChartDrilldown, SingleValue, LazyMap, LazyResultsTable, NoStatistics, JobDispatchState, drilldownUtils) {
        return Base.extend({
            moduleId: module.id,
            className: 'tab-pane',
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);
                
                this.model.intentionsParser = new IntentionsParser();
                
                this.children.noStatistics = new NoStatistics({
                    model: {
                        report: this.model.report,
                        appLocal: this.model.appLocal,
                        application: this.model.application,
                        user: this.model.user,
                        summary: this.model.summary
                    }
                });
                
                this.children.jobDispatchState = new JobDispatchState({
                    model: this.model.searchJob,
                    mode: this.model.searchJob.entry.content.get('isPreviewEnabled') ? 'results_preview' : 'results'
                });
                
                this.children.vizEditor = new VizEditor({
                    model: {
                        report: this.model.report,
                        application: this.model.application
                    },
                    vizTypes: ['visualizations'],
                    bindToChangeOfSearch: false
                });

                this.children.singleValue = new SingleValue({
                    model: {
                        searchResultsColumn: this.model.resultJsonCols,
                        state: this.model.report.entry.content
                    },
                    width: 800,
                    height: 300
                });

                this.$vizControls = $('<div class="visualization-controls"></div>');
                this.$noStatsContainer = $('<div class="viz-nostats-container"></div>');
                this.$chartContainer = $('<div class="viz-chart-container"></div>');
                this.$chartResize = $('<div class="chart-resize"></div>');
                this.$mapResize = $('<div class="map-resize"></div>');
            },
            startListening: function() {
                this.listenTo(this.model.intentionsParser, 'change', function() {
                    var attrs = $.extend({}, {
                        'search': this.model.intentionsParser.get('fullSearch'),
                        'dispatch.earliest_time': this.model.intentionsParser.get('dispatch.earliest_time'),
                        'dispatch.latest_time': this.model.intentionsParser.get('dispatch.latest_time')
                    });
                    
                    // in the case where there is no events search (i.e. | inputcsv), make sure we drill down to the statistics table
                    if (!this.model.searchJob.entry.content.get('eventSearch')) {
                        attrs['display.page.search.tab'] = 'statistics';
                    }
                    this.model.report.entry.content.set(attrs);
                });
                this.listenTo(this.model.searchJob.entry.content, 'change:dispatchState change:resultCount change:resultPreviewCount', _.debounce(function() {
                    if (this.active) {
                        this.visibility();
                    }
                }, 0));
                this.listenTo(this.model.report.entry.content, 'change:display.visualizations.type', function() {
                    if (this.active) {
                        this.visibility();
                    }
                });
            },
            initializeResultsTable: function() {
                this.children.resultsTable = new LazyResultsTable({
                    model: {
                        results: this.model.resultJsonRows,
                        metadata: this.model.report.entry.content,
                        format: this.model.report.entry.content
                    },
                    sortKeyAttr: 'display.statistics.sortColumn',
                    sortDirAttr: 'display.statistics.sortDirection',
                    countAttr: 'display.prefs.statistics.count',
                    offsetAttr: 'display.prefs.statistics.offset',
                    disable: true
                });
                
                //TODO: remove when all the views don't activate on init
                this.children.resultsTable.deactivate({deep: true});
            },
            initializeChart: function() {
                this.children.jsChartDrilldown = new JSChartDrilldown({
                    model: {
                        searchResultsColumn: this.model.resultJsonCols,
                        state: this.model.report.entry.content,
                        application: this.model.application,
                        intentionsParser: this.model.intentionsParser,
                        searchJob: this.model.searchJob
                    },
                    width: '100%',
                    height: this.model.report.entry.content.get('display.visualizations.chartHeight')
                });

                this.listenTo(this.children.jsChartDrilldown, 'chartUpdate', function(info) {
                    this.model.report.entry.content.set({
                        currentChartFields: info.fields
                    });
                });
                //TODO: remove when all the views don't activate on init
                this.children.jsChartDrilldown.deactivate({deep: true});
            },
            initializeMap: function() {
                this.model.mapResults = new ResultJsonCols();
                this.model.mapResults.fetchData.set({
                    offset: 0,
                    show_metadata: false
                }, { silent: true });

                this.children.map = new LazyMap({
                    model: {
                        searchResultsColumn: this.model.mapResults,
                        state: this.model.report.entry.content
                    },
                    width: '100%',
                    height: this.model.report.entry.content.get('display.visualizations.mapHeight')
                });
            },
            activate: function(options) {
                var clonedOptions = _.extend({}, (options || {}));
                delete clonedOptions.deep;
                this.ensureDeactivated({deep: true});
                
                if (!this.children.resultsTable) {
                    this.initializeResultsTable();
                }
                // the chart needs to be initialized in activate because it relies on heights from the report
                if (!this.children.jsChartDrilldown) {
                    this.initializeChart();
                }
                this.debouncedWindowResize =  _(_.debounce(this.reflow, 50)).bind(this);
                $(window).on('resize.' + this.uniqueNS(), this.debouncedWindowResize);

                Base.prototype.activate.call(this, clonedOptions);
                this.visibility();
                
                return this;
            },
            deactivate: function(options) {
                // there is a significant overhead in destroying/creating a map view, so we don't want deactivate to do that,
                // but we want to set a flag indicating that the map's listeners have been un-bound, see showMapView()
                this.mapIsActive = false;
                if (!this.active) {
                    return Base.prototype.deactivate.apply(this, arguments);
                }
                
                this.children.resultsTable.remove();
                delete this.children.resultsTable;
                this.children.jsChartDrilldown.remove();
                this.$chartResize.remove();
                delete this.children.jsChartDrilldown;

                Base.prototype.deactivate.apply(this, arguments);
                
                this.model.intentionsParser.clear();
                $(window).off('resize.' + this.uniqueNS());
                return this;
            },
            updateChartHeight: function() {
                var heightDelta = parseInt(this.$chartResize.css('top').replace('px',''), 10),
                    currentHeight = parseInt(this.model.report.entry.content.get('display.visualizations.chartHeight'), 10) ||
                                    this.children.jsChartDrilldown.$el.height(),
                    height = currentHeight + heightDelta;
                
                if(height < 26) {
                    height = 26;
                }
                                    
                this.children.jsChartDrilldown.setHeight(height);
                this.model.report.entry.content.set('display.visualizations.chartHeight', height);
                this.children.jsChartDrilldown.reflow();
                this.$chartResize.css('top', 0);
            },
            updateMapHeight: function() {
                var heightDelta = parseInt(this.$mapResize.css('top').replace('px',''), 10),
                    currentHeight = parseInt(this.model.report.entry.content.get('display.visualizations.mapHeight'), 10) ||
                                    this.children.map.$el.height(),
                    height = currentHeight + heightDelta;

                this.children.map.$el.height(height);
                this.model.report.entry.content.set('display.visualizations.mapHeight', height);
                this.$mapResize.css('top', 0);
            },
            remove: function() {
                Base.prototype.remove.apply(this, arguments);
                $(window).off('resize.' + this.uniqueNS());
            },
            visibility: function() {
                var resultCount = this.model.searchJob.entry.content.get('isPreviewEnabled') ? this.model.searchJob.entry.content.get('resultPreviewCount') : this.model.searchJob.entry.content.get('resultCount'),
                    vizType = this.model.report.entry.content.get('display.visualizations.type'),
                    isSingleValue = (vizType === 'singlevalue'),
                    isMap = (vizType === 'mapping');

                if (!this.model.searchJob.isReportSearch()) {
                    this.children.vizEditor.deactivate({deep: true}).$el.hide();
                    this.children.jobDispatchState.deactivate({deep: true}).$el.hide();
                    this.children.noStatistics.activate({deep: true}).$el.show();
                    this.hideChart();
                    this.hideSingleValue();
                    this.hideMap();
                    this.hideResultsTable();
                    return;
                }
                this.children.vizEditor.activate({deep: true}).$el.show();
                if (resultCount === 0) {
                    this.children.noStatistics.deactivate({deep: true}).$el.hide();
                    this.children.jobDispatchState.activate({deep: true}).$el.show();
                    if (this.model.searchJob.entry.content.get('dispatchState') == 'DONE' || isSingleValue || isMap) {
                        this.hideChart();
                    } else {
                        this.children.jobDispatchState.deactivate({deep: true}).$el.hide();
                        this.showChart();
                    }
                    this.hideSingleValue();
                    this.hideMap();
                    this.hideResultsTable();
                } else {
                    this.children.jobDispatchState.deactivate({deep: true}).$el.hide();
                    this.children.noStatistics.deactivate({deep: true}).$el.hide();
                    if (isSingleValue) {
                        this.hideChart();
                        this.showSingleValue();
                        this.hideMap();
                    } else if(isMap) {
                        this.hideChart();
                        this.hideSingleValue();
                        this.showMap();
                    } else {
                        this.showChart();
                        this.hideSingleValue();
                        this.hideMap();
                    }
                    this.showResultsTable();
                }
            },
            showChart: function() {
                this.showVizChild(this.children.jsChartDrilldown, this.$chartContainer);
                if(!$.contains(this.el, this.$chartResize[0])) {
                    this.$chartResize.insertAfter(this.children.jsChartDrilldown.el);
                    var that = this;
                    this.$chartResize.draggable({
                        axis: 'y',
                        stop: function() {
                            that.updateChartHeight();
                        }
                    });
                }
                this.$chartResize.show();
            },
            showSingleValue: function() {
                this.showVizChild(this.children.singleValue);
            },
            showMap: function() {
                // lazily initialize the map, just the init routine will fetch tiles from the server,
                // so it should only be done on demand
                if(!this.children.map) {
                    this.initializeMap();
                    $.when(this.children.map.wrappedViewLoaded()).done(_.bind(this.readMapCenterAndZoom, this));
                }
                if(!this.mapIsActive) {
                    this.listenTo(this.children.map, 'boundsChanged', function() {
                        this.readMapCenterAndZoom();
                        this.fetchMapResults();
                    });
                    this.listenTo(this.children.map, 'mapClicked', function(e) {
                        if(this.model.report.entry.content.get('display.visualizations.mapping.drilldown') === 'none') {
                            return;
                        }
                        this.model.intentionsParser.clear({silent: true});
                        var intentionsData = $.extend(drilldownUtils.convertMapClickToIntentions(e), {
                            q: this.model.report.entry.content.get('search'),
                            app: this.model.application.get('app'),
                            owner: this.model.application.get('owner')
                        });
                        this.model.intentionsParser.fetch({ data: intentionsData });
                    });
                    this.mapIsActive = true;
                }
                this.children.map.load();
                this.showVizChild(this.children.map);
                if(!$.contains(this.el, this.$mapResize[0])) {
                    this.$mapResize.insertAfter(this.children.map.el);
                    var that = this;
                    this.$mapResize.draggable({
                        axis: 'y',
                        stop: function() {
                            that.updateMapHeight();
                        }
                    });
                }
                this.$mapResize.show();
                this.listenTo(this.model.searchJob, 'jobProgress:' + Job.RESULTS_PREVIEW, this.fetchMapResults);
                if(!this.model.searchJob.isPreparing()) {
                    this.fetchMapResults();
                }
            },
            readMapCenterAndZoom: function() {
                this.model.report.entry.content.set({
                    currentMapZoom: this.children.map.wrappedView.getZoom(),
                    currentMapCenter: this.children.map.wrappedView.getCenter().clone()
                });
            },
            fetchMapResults: function() {
                $.when(this.children.map.wrappedViewLoaded()).done(_.bind(function() {
                    this.model.mapResults.set(this.model.mapResults.idAttribute, this.model.searchJob.entry.links.get(Job.RESULTS_PREVIEW));
                    this.model.mapResults.safeFetch({
                        data: {
                            search: this.children.map.wrappedView.getPostProcessSearch(),
                            count: this.children.map.wrappedView.getMaxClusters()
                        }
                    });
                }, this));
            },
            // helper method to manage showing one of the visualization child views
            // the child will be inserted (above the results table) and rendered only if needed, and activated only if needed
            // $wrapper is optional and should only be specified if the child requires a wrapper element
            showVizChild: function(child, $wrapper) {
                var insertBeforeResultsTable = _(function($toInsert) {
                    if(this.children.resultsTable.isDescendantOf(this)) {
                        $toInsert.insertBefore(this.children.resultsTable.el);
                    }
                    else {
                        $toInsert.appendTo(this.el);
                    }
                }).bind(this);
                
                var needsRender = false;
                if($wrapper) {
                    if(!$.contains(this.el, $wrapper[0])) {
                        insertBeforeResultsTable($wrapper);
                    }
                    if(!$.contains($wrapper[0], child.el)) {
                        child.appendTo($wrapper);
                        needsRender = true;
                    }
                }
                else {
                    if(!$.contains(this.el, child.el)) {
                        insertBeforeResultsTable(child.$el);
                        needsRender = true;
                    }
                }
                
                if(!child.active) {
                    child.activate({ deep: true });
                }
                if(needsRender) {
                    child.render();
                }
                child.$el.show();
            },
            showResultsTable: function() {
                var needsRender = false;
                if(!this.children.resultsTable.isDescendantOf(this)) {
                    this.children.resultsTable.appendTo(this.el);
                    needsRender = true;
                }
                // show before activate/render to make sure sparklines work
                this.children.resultsTable.$el.show();
                if(!this.children.resultsTable.active) {
                    // Use the skipUpdate flag to avoid activate leading to a render.  We will re-render if needed below.
                    // (SPL-82081)
                    this.children.resultsTable.activate({deep: true, skipUpdate: true});
                }
                if(needsRender) {
                    this.children.resultsTable.load().render();
                }
            },
            hideChart: function() {
                this.children.jsChartDrilldown.deactivate({deep: true}).$el.hide();
                this.$chartResize.hide();
            },
            hideSingleValue: function() {
                this.children.singleValue.deactivate({deep: true}).$el.hide();
            },
            hideMap: function() {
                if(!this.children.map) {
                    return;
                }
                this.stopListening(this.model.searchJob, 'jobProgress:' + Job.RESULTS_PREVIEW, this.fetchMapResults);
                this.children.map.deactivate({deep: true}).$el.hide();
                this.$mapResize.hide();
            },
            hideResultsTable: function() {
                this.children.resultsTable.deactivate({deep: true}).$el.hide();
            },
            render: function() {
                // some children may be appended in activate(), so we have to non-destructive here...
                this.$noStatsContainer.prependTo(this.el);
                this.children.noStatistics.render().appendTo(this.$noStatsContainer);

                this.$vizControls.insertAfter(this.$noStatsContainer);
                this.children.vizEditor.render().appendTo(this.$vizControls);

                this.children.jobDispatchState.render().appendTo(this.el);
                return this;
            }
        });
    }
);
