define(
    [
        'jquery',
        'underscore',
        'module',
        'models/services/search/IntentionsParser',
        'models/Base',
        'views/Base',
        'views/search/results/statisticspane/StatisticsControls',
        'views/shared/LazyResultsTableDrilldown',
        'views/shared/JobDispatchState',
        'views/search/results/shared/NoStatistics',
        'uri/route'
    ],
    function($, _, module, IntentionsParser, BaseModel, Base, StatisticsControls, LazyResultsTableDrilldown, JobDispatchState, NoStatistics, route){
        return Base.extend({
            moduleId: module.id,
            className: 'tab-pane',
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);

                this.model.intentionsParser = new IntentionsParser();
                this.model.state = new BaseModel();

                //child views
                this.children.jobDispatchState = new JobDispatchState({
                    model: this.model.searchJob,
                    mode: this.model.searchJob.entry.content.get('isPreviewEnabled') ? 'results_preview' : 'results'
                });
                
                this.children.noStatistics = new NoStatistics({
                    model: {
                        report: this.model.report,
                        appLocal: this.model.appLocal,
                        application: this.model.application,
                        user: this.model.user,
                        summary: this.model.summary
                    }
                });
                
                this.children.statisticsControls = new StatisticsControls({
                    model: {
                        report: this.model.report,
                        timeline: this.model.timeline,
                        searchJob: this.model.searchJob,
                        application: this.model.application,
                        state: this.model.state
                    }
                });
                
                this.children.resultsTableDrilldown = new LazyResultsTableDrilldown({
                    model: {
                        results: this.model.resultJsonRows,
                        metadata: this.model.report.entry.content,
                        format: this.model.report.entry.content,
                        intentionsParser: this.model.intentionsParser,
                        searchJob: this.model.searchJob,
                        application: this.model.application,
                        state: this.model.state
                    },
                    sortKeyAttr: 'display.statistics.sortColumn',
                    sortDirAttr: 'display.statistics.sortDirection',
                    countAttr: 'display.prefs.statistics.count',
                    offsetAttr: 'display.prefs.statistics.offset',
                    tableDockOffset: 36
                });
            },
            startListening: function() {
                this.listenTo(this.model.intentionsParser, 'change', function() {
                    var attrs = $.extend({}, {
                            'search': this.model.intentionsParser.get('fullSearch'),
                            'dispatch.earliest_time': this.model.intentionsParser.get('dispatch.earliest_time'),
                            'dispatch.latest_time': this.model.intentionsParser.get('dispatch.latest_time')
                        }),
                        newTab = this.model.intentionsParser.get('drilldownNewTab');
                    if (newTab) {
                        var searchRoute = route.search(this.model.application.get('root'), this.model.application.get('locale'), this.model.application.get('app'), {
                            data: {
                                q: attrs.search,
                                earliest: attrs['dispatch.earliest_time'],
                                latest: attrs['dispatch.latest_time']
                            },
                            absolute: true
                        });
                        newTab.location = searchRoute;
                        this.model.intentionsParser.clear({silent: true});
                    } else {
                        this.model.report.entry.content.set(attrs);
                    }
                });
                
                this.listenTo(this.model.report.entry.content, 'change:display.visualizations.type', function() {
                    if (this.active) {
                        this.visibility();
                    }
                });
                this.listenTo(this.model.searchJob.entry.content, 'change:resultPreviewCount change:resultCount change:dispatchState change:isPreviewEnabled',  _.debounce(function() {
                    if (this.active) {
                        this.visibility();
                    }
                }, 0));
            },
            activate: function(options) {
                var clonedOptions = _.extend({}, (options || {}));
                delete clonedOptions.deep;
                this.ensureDeactivated({deep:true});
                Base.prototype.activate.call(this, clonedOptions);
                this.visibility();
                return this;
            },
            deactivate: function(options) {
                if (!this.active) {
                    return Base.prototype.deactivate.apply(this, arguments);
                }
                Base.prototype.deactivate.apply(this, arguments);
                this.model.intentionsParser.clear();
                this.model.state.clear();
                return this;
            },
            visibility: function() {
                var resultCount = this.model.searchJob.entry.content.get('isPreviewEnabled') ? this.model.searchJob.entry.content.get('resultPreviewCount') : this.model.searchJob.entry.content.get('resultCount');
                this.children.jobDispatchState.options.mode = this.model.searchJob.entry.content.get('isPreviewEnabled') ? 'results_preview' : 'results';

                if (!this.model.searchJob.isReportSearch()) {
                    this.children.jobDispatchState.deactivate({deep: true}).$el.hide();
                    this.children.noStatistics.activate({deep: true}).$el.show();
                    this.children.statisticsControls.deactivate({deep: true}).$el.hide();
                    this.children.resultsTableDrilldown.deactivate({deep: true}).$el.hide();
                    return;
                }
                if (resultCount === 0) {
                    this.children.jobDispatchState.activate({deep: true}).$el.show();
                    this.children.noStatistics.deactivate({deep: true}).$el.hide();
                    this.children.statisticsControls.activate({deep: true}).$el.show();
                    this.children.resultsTableDrilldown.deactivate({deep: true}).$el.hide();
                } else {
                    this.children.jobDispatchState.deactivate({deep: true}).$el.hide();
                    this.children.noStatistics.deactivate({deep: true}).$el.hide();
                    this.children.statisticsControls.activate({deep: true}).$el.show();
                    this.children.resultsTableDrilldown.$el.show();
                    this.children.resultsTableDrilldown.load().activate({deep: true});
                }
            },
            render: function() {
                if (!this.el.innerHTML) {
                    this.children.noStatistics.render().appendTo(this.$el);
                    this.children.statisticsControls.render().appendTo(this.$el);
                    this.children.jobDispatchState.render().appendTo(this.$el);
                    this.children.resultsTableDrilldown.render().appendTo(this.$el);
                }
                return this;
            }
    });
});
