define(
    [
        'underscore',
        'module',
        'uri/route',
        'views/Base',
        'views/shared/Modal',
        'contrib/text!views/shared/AboutDialog.html',
        'splunk.time'
    ],
    function(
        _,
        module,
        route,
        Base,
        Modal,
        Template,
        Time
    ){
        return Modal.extend({
            moduleId: module.id,
            template: Template,
            initialize: function() {
                Modal.prototype.initialize.apply(this, arguments);
                var that = this,
                    currentAppName = that.model.applicationModel.get('app'),
                    currentApp,
                    date = new Time.DateTime();

                this.year = date.getYear();
                this.logoFlavors = {
                    'hunk':       '&#xF000;',
                    'enterprise': '&#xF001;',
                    'cloud':      '&#xF002;',
                    'lite':       '&#xF003;',
                    'lite_free':  '&#xF003;'
                };
                if (currentAppName === 'launcher') {
                    that.currentAppLabel = _('Home').t();
                } else {
                    if (that.collection) {
                        currentApp = that.collection.find(function(app) {
                            return app.entry.get('name') === currentAppName;
                        });
                    }
                    that.currentAppLabel = currentApp ? currentApp.entry.content.get('label') : _('N/A').t();
                }

                this.model.serverInfo.on('change reset', function() {
                    this.render();
                }, this);
            },
            getListOfProducts: function() {
                var addOns = this.model.serverInfo.entry.content.get('addOns'),
                    result;
                if (addOns && !$.isEmptyObject(addOns)) {
                    result = _(addOns).keys().join(', ');
                }
                return result;
            },
            getLogoFlavorIcon: function() {
                var branding = this.model.serverInfo.getBranding(),
                    icon = '';
                if (this.logoFlavors.hasOwnProperty(branding)) {
                    icon = this.logoFlavors[branding];
                }
                return icon;
            },
            render: function() {
                this.$el.html(Modal.TEMPLATE);
                this.$(Modal.HEADER_TITLE_SELECTOR).html('splunk<span class="prompt">&gt;&#x00AE;'+this.getLogoFlavorIcon()+'</span>');
                var isLite = this.model.serverInfo.isLite(),
                    isLiteFree = this.model.serverInfo.isLiteFree(),
                    versionNumberText = this.model.serverInfo.entry.content.get('version') || _('N/A').t(),
                    liteVersionText = (isLiteFree) ? _('Splunk Light Free Version ' + versionNumberText).t() : _('Splunk Light Version ' + versionNumberText).t(),
                    versionText = (isLite) ? liteVersionText : versionNumberText,
                    template = this.compiledTemplate({
                        serverName: this.model.serverInfo.entry.content.get('serverName') || _('N/A').t(),
                        version: versionText,
                        isEnterprise: this.model.serverInfo.isEnterprise(),
                        build: this.model.serverInfo.entry.content.get('build') || _('N/A').t(),
                        appVersion: this.model.appLocal.entry.content.get('version') || null,
                        appBuild: this.model.appLocal.entry.content.get('build') || null,
                        listOfProducts: this.getListOfProducts(),
                        currentApp: this.currentAppLabel,
                        isLite: this.isLite,
                        thirdPartyCredits: route.docHelp(this.model.applicationModel.get('root'), this.model.applicationModel.get('locale'), 'ReleaseNotes.Credits'),
                        copyrightYear: this.year
                    });
                this.$(Modal.BODY_SELECTOR).html(template);

                return this;
            }
        });
    });
