define(
    [
        'underscore',
        'contrib/text!views/shared/Paginator.html',
        'splunk.paginator',
        'module',
        'views/Base'
    ],
    function(
        _,
        template,
        splunkPaginator,
        module,
        BaseView
    ) {
    /**
     * Paginate twice it's a long way to the bay!
     *
     */
        return BaseView.extend({
            className: 'pagination pull-right',
            template: template,
            moduleId: module.id,
            /**
             * @param {Object} options {
             *      collection: <SplunkDsBaseV2>
             *      model: <Backbone.Model> (Optional), model with offset and count. If not passed the collections fetchData is used.
             * }
             */
            initialize: function(options) {
                BaseView.prototype.initialize.call(this, options);
                if(!this.model) {
                    this.model = this.collection.fetchData;
                }
                this.collection.on('add remove reset', this.debouncedRender, this);
                this.model.on('change:count', function(){
                    this.model.set('offset', 0);
                }, this);
            },
            events: {
                'click li:not(.disabled) a': function(e) {
                    this.model.set('offset', parseInt($(e.currentTarget).closest('a').attr('data-offset'), 10));
                    e.preventDefault();
                }
            },
            render: function() {
                var length = 0;

                if(this.collection.models[0] && this.collection.models[0].paging){
                    length = this.collection.models[0].paging.get('total');
                }

                var options = {
                        max_items_page: this.model.get('count') || 10,
                        max_pages: 10,
                        item_offset: this.model.get('offset') || 0
                    },
                    paginator = new splunkPaginator.Google(length || 0, options),
                    template = this.compiledTemplate({
                        paginator: paginator,
                        _: _
                    });
                this.$el.html(template);
                return this;
            }
        });
    }
);