define(
    [
        'underscore',
        'module',
        'views/shared/JSChart'
    ],
    function(
        _,
        module,
        JSChart
    )
    {
        return JSChart.extend({
            moduleId: module.id,
            /**
             * Extends the JSChart view and adds a listenter to
             * chartClick to perform drilldown logic.
             *
             * @constructor
             * @param options {
             * model: {
             *     searchResultsColumn <models.services.jobs.ResultJsonCols>: the data
             *     state: <content from models.search.report>: the charting properties
             *     application: <models.Application>: the application state,
             *     intentionsParser: <models.services.search.IntentionsParser>,
             *     searchJob: <models.services.search.Job>
             * },
             * width: initial width,
             * height: initial height
             * }
             */
            initialize: function() {
                JSChart.prototype.initialize.apply(this, arguments);
            },
            startListening: function () {
                JSChart.prototype.startListening.apply(this, arguments);
                this.listenTo(this, 'chartClick', this.onChartClick);
            },
            onChartClick: function(eventInfo) {
                var field, value;
                this.model.intentionsParser.clear({silent: true});

                if(eventInfo.type === 'legendClick') {
                    // if the legend click has a name and value (this happens for scatter charts), use them
                    if(eventInfo.hasOwnProperty('name') && eventInfo.hasOwnProperty('value')) {
                        this.model.intentionsParser.fetch({
                            data: {
                                q: this.model.state.get('search'),
                                action: 'fieldvalue',
                                field: eventInfo.name,
                                value: eventInfo.value,
                                fieldMetaData: JSON.stringify(this.model.searchJob.entry.content.get('fieldMetadataResults')),
                                app: this.model.application.get('app'),
                                owner: this.model.application.get('owner')
                            }
                        });
                    }
                    // otherwise do a keyword search based on name2
                    else {
                        this.model.intentionsParser.fetch({
                            data: {
                                q: this.model.state.get('search'),
                                action: 'keyword',
                                field: eventInfo.name2,
                                fieldMetaData: JSON.stringify(this.model.searchJob.entry.content.get('fieldMetadataResults')),
                                app: this.model.application.get('app'),
                                owner: this.model.application.get('owner')
                            }
                        });
                    }
                    return;
                }
                
                if (eventInfo._span) {
                    this.model.intentionsParser.set({
                        'dispatch.earliest_time': eventInfo.value,
                        'dispatch.latest_time': JSON.stringify(parseFloat(eventInfo.value) + eventInfo._span)
                    }, {silent: true});
                }
                if (eventInfo.name === '_time') {
                    field = eventInfo.name2;
                    value = eventInfo.value2;
                } else {
                    field = [eventInfo.name, eventInfo.name2];
                    value = [eventInfo.value, eventInfo.value2];
                }

                this.model.intentionsParser.fetch({
                    data: {
                        q: this.model.state.get('search'),
                        action: 'fieldvalue',
                        field: field,
                        value: value,
                        fieldMetaData: JSON.stringify(this.model.searchJob.entry.content.get('fieldMetadataResults')),
                        app: this.model.application.get('app'),
                        owner: this.model.application.get('owner')
                    }
                });
            }
        });
    }
);
