define(
    [
        'jquery',
        'underscore',
        'module',
        'views/Base',
        'util/math_utils',
        'bootstrap.tooltip'
    ],
    function($, _, module, BaseView, math_utils) {
        return BaseView.extend({
            moduleId: module.id,
            className: "json-tree",
            /**
             * @param {Object} options {
             *     json: <json stringified object>
             *     isValidJSON: <a flag allowing opt-out of safe set json routine>
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                
                var defaults = {
                        isValidJSON: false,
                        json: '{}'
                };
                this.options = $.extend(true, defaults, this.options);

                if(this.options.isValidJSON) {
                    this._json = this.options.json;
                } else {
                    this.setJSON(this.options.json);
                }
            },
            setJSON: function(json) {
                var parsed;

                if(typeof json !== 'string') {
                    json = JSON.stringify(json);
                } 

                try{
                    parsed = JSON.parse(json);
                } catch(e) {}
                
                this._json = parsed;
            }, 
            isValidJSON: function() {
                return !!this._json;
            },
            events: {
                'click .jsexpands': function(e) {
                    var $target = $(e.currentTarget);
                    this.trigger('interaction');
                    this.interacted = true;
                    $target.removeClass('jsexpands').addClass('jscollapse').text('[-]').next().show();
                    e.preventDefault();
                },
                'click .jscollapse': function(e) {
                    var $target = $(e.currentTarget);
                    this.trigger('interaction');
                    this.interacted = true;
                    $target.removeClass('jscollapse').addClass('jsexpands').text('[+]').next().hide();
                    e.preventDefault();
                }
            },
            render: function() {
                if(!this.interacted) {
                    this.$el.html(this.compiledTemplate({ 
                        obj: this._json,
                        indent: 0,
                        template: this.compiledTemplate,
                        path: '',
                        _: _,
                        math_utils: math_utils
                    }));

                    this.$el.find('.isLessThanMinSafeInt').tooltip({title:_('This value may have been rounded because it exceeds the minimum allowed int value.').t()});
                    this.$el.find('.isGreaterThanMaxSafeInt').tooltip({title:_('This value may have been rounded because it exceeds the maximum allowed int value.').t()});
                }
                return this;
            },
            template: '\
                <% var indent = indent + 2; %>\
                <% var level  = indent/2; %>\
                <% var type   = (obj === null) ? "null": typeof obj; %>\
                \
                <% if(!_.isObject(obj)) { %>\
                    <span class="t <%- type %>"><%- obj === null ? "null" : obj %></span>\
                <% } else { %>\
                    <% var isObj = !(obj instanceof Array) %>\
                    <% var list  = (isObj) ? _.keys(obj).sort() : obj.sort(); %>\
                    <span><%if(isObj) { %>{<% } else { %>[<% } %></span>\
                    \
                    <% if (list.length > 0) { %>\
                        <% if(!(level < 2)) { %>\
                            <a href="#" class="jsexpands">[+]</a>\
                        <% } else { %>\
                            <a href="#" class="jscollapse">[-]</a>\
                        <% } %>\
                    <% } %>\
                    \
                    <% if (level != 1) { %>\
                        <span style="display: none;">\
                    <% } else { %>\
                        <span>\
                    <% } %>\
                    \
                    <% var i = 0; %>\
                    <% for(; i<list.length; i++){ %>\
                        <br><% for(var j=0; j<indent; j++){ %>&nbsp;<% } %>\
                        <% var key = list[i]; %>\
                        <% var newPath = path + key + "."; %>\
                        <% if(isObj) { %>\
                            <% if(math_utils.isLessThanMinSafeInt(obj[list[i]])) { %>\
                                <span class="key level-<%-level%> isLessThanMinSafeInt">\
                            <% } else if(math_utils.isGreaterThanMaxSafeInt(obj[list[i]])) { %>\
                                <span class="key level-<%-level%> isGreaterThanMaxSafeInt">\
                            <% } else { %>\
                                <span class="key level-<%-level%>">\
                            <% } %>\
                                <span class="key-name" data-path="<%- path + key %>"><%-key%></span>:\
                                <%= template({ obj:obj[key], indent: indent, template: template, path: newPath, _:_, math_utils:math_utils}) %>\
                            </span>\
                        <% } else { %>\
                            <%= template({ obj:key, indent: indent, template: template, path: newPath, _:_, math_utils:math_utils}) %>\
                        <% } %>\
                    <% } %>\
                    </span><br>\
                    <span>\
                        <% for(var j=0; j<indent-2; j++){ %>&nbsp;<% } %>\
                        <% if(isObj){ %><span>}</span><% } else { %><span>]</span><%}%>\
                    </span>\
                \
                <% } %>\
                \
            '
        });
    }
 );
