define(
    [
        'underscore',
        'module',
        'collections/shared/FlashMessages',
        'views/shared/FlashMessagesLegacy'
    ],
    function(_, module, FlashMessagesCollection, FlashMessagesLegacyView) {
        return FlashMessagesLegacyView.extend({
            moduleId: module.id,
            className: 'message-single',
            /**
             * @param {Object} options {
             *     model: <models.Job>
             *     mode: <events|results|results_preview|auto> Defaults to events.
             * }
             */
            initialize: function() {
                this.collection = new FlashMessagesCollection();
                FlashMessagesLegacyView.prototype.initialize.apply(this, arguments);
            },
            startListening: function() {
                FlashMessagesLegacyView.prototype.startListening.apply(this, arguments);
                if (this.resultCountAttribute) {
                    this.listenTo(this.model.entry.content, 'change:' + this.resultCountAttribute, this.update);
                }
                this.listenTo(this.model.entry.content, 'change:dispatchState', this.update);
            },
            activate: function(options) {
                this.ensureDeactivated({deep: true});
            
                switch (this.options.mode) {
                    case 'results_preview':
                        this.resultCountAttribute = 'resultPreviewCount';
                        break;
                    case 'results':
                        this.resultCountAttribute = 'resultCount';
                        break;
                    case 'auto':
                        break;
                    default:
                        this.resultCountAttribute = 'eventAvailableCount';
                        break;
                }
                
                FlashMessagesLegacyView.prototype.activate.apply(this, arguments);
                this.update();
                return this;
            },
            update: function() {
                if (this.model.isNew()) {
                    return;
                }
                if (this.model.isPreparing()) {
                    this.updatePreparingStates();
                    return;
                }
                
                if ((this.options.mode === 'auto') && (!this.resultCountAttribute)) {
                    if (this.model.isReportSearch()) {
                        if (this.model.entry.content.get('isPreviewEnabled')) {
                            this.resultCountAttribute = 'resultPreviewCount';
                        } else {
                            this.resultCountAttribute = 'resultCount';
                        }   
                    } else {
                        this.resultCountAttribute = 'eventAvailableCount';
                    }
                }
                
                this.updateRanStates();
            },
            updatePreparingStates: function() {
                var dispatchState = this.model.entry.content.get('dispatchState');
                
                if (dispatchState === 'PARSING') {
                    this.collection.reset([{
                        key: 'waiting',
                        type: 'info',
                        html: _('Parsing search.').t()
                    }]);
                } else if (dispatchState === 'QUEUED') {
                    this.collection.reset([{
                        key: 'waiting',
                        type: 'info',
                        html: _('Waiting for queued job to start.').t()
                    }]);
                } else {
                    this.collection.reset([]);
                }        
            },
            updateRanStates: function() {
                var dispatchState = this.model.entry.content.get('dispatchState'), 
                    resultCount = this.model.entry.content.get(this.resultCountAttribute),
                    html;
                
                if (dispatchState === 'RUNNING' && resultCount === 0) {
                    if (this.model.isRealtime()) {
                        this.collection.reset([{
                            key: 'waiting',
                            type: 'info',
                            html: _('No results in current time range.').t()
                        }]);
                    } else {
                        html = _('No results yet found.').t();
                        
                        if (this.model.isReportSearch()) {
                            if (!this.model.entry.content.get('isPreviewEnabled')) {
                                html = _('Waiting for search to complete. Display results while the search is running by enabling Preview.').t();
                            }  
                        }
                        
                        this.collection.reset([{
                            key: 'waiting',
                            type: 'info',
                            html: html
                        }]);
                    }
                } else if (dispatchState === 'FINALIZING' && resultCount === 0) {
                    this.collection.reset([{
                        key: 'waiting',
                        type: 'info',
                        html: _('Finalizing results.').t()
                    }]);
                } else if (dispatchState === 'DONE' && resultCount === 0) {
                    this.collection.reset([{
                        key: 'zero',
                        type: 'error',
                        html: _('No results found.').t()
                    }]);
                } else {
                    this.collection.reset([]);
                }
            },
            render: function() {
                var template = this.compiledTemplate({
                    flashMessages: this.collection
                });
                this.$el.html(template);
                return this;
            }
        });
    }
);