define([
            'jquery',
            'underscore',
            'module',
            'views/Base',
            'util/deferred_loader'
        ],
        function(
            $,
            _,
            module,
            Base,
            deferredLoader
        ) {

    return Base.extend({

        _pendingRender: false,
        className: 'lazy-view-container',
        loadingMessage: _('Loading...').t(),
        deps: [],

        initialize: function() {
            Base.prototype.initialize.apply(this, arguments);
            this._wrappedViewLoadedDfd = $.Deferred();
        },

        load: function() {
            if(!this.wrappedModuleName) {
                throw new Error('subclasses must define a wrapped module name');
            }
            if(!this._pendingLoad && !this.children.wrappedView) {
                var that = this;
                this._pendingLoad = true;
                this.$el.addClass('loading');
                this._showLoading();
                this.trigger('loadStart');
                deferredLoader.load([this.wrappedModuleName].concat(this.deps)).then(function(WrappedViewConstructor) {
                    if(that._removed) {
                        return;
                    }
                    that.children.wrappedView = that.wrappedView = new WrappedViewConstructor(that._getWrappedViewOptions());
                    if(that._rendered && that.active && that.awake) {
                        that.children.wrappedView.render();
                    }
                    if(that.active) {
                        that.children.wrappedView.activate({ deep: true });
                        that._bindWrappedViewEvents();
                    } else {
                        that.children.wrappedView.deactivate({ deep: true });
                    }
                    that.$el.removeClass('loading');
                    that._hideLoading();
                    that.children.wrappedView.appendTo(that.el);
                    that._onWrappedViewLoaded();
                    that._pendingLoad = false;
                    that.trigger('loadComplete');
                    that._wrappedViewLoadedDfd.resolve();
                });
            }
            return this;
        },

        render: function() {
            if(this.children.wrappedView) {
                this.children.wrappedView.render();
            } else {
                this._rendered = true;
            }
            return this;
        },

        remove: function() {
            this._removed = true;
            return Base.prototype.remove.apply(this, arguments);
        },

        startListening: function() {
            Base.prototype.startListening.apply(this, arguments);
            this._bindWrappedViewEvents();
        },
        
        wrappedViewLoaded: function() {
            return this._wrappedViewLoadedDfd.promise();
        },

        _bindWrappedViewEvents: function() {
            if(this.children.wrappedView) {
                this.listenTo(this.children.wrappedView, 'all', this.trigger);
            }
        },

        _showLoading: function() {
            this.$el.html(this.compiledTemplate({ message: this.loadingMessage }));
        },

        _hideLoading: function() {
            this.$el.empty();
        },

        _onWrappedViewLoaded: function() {},

        _getWrappedViewOptions: function() {
            return _.omit(this.options, 'el');
        },

        template: '\
            <div class="alert alert-info">\
                <i class="icon-alert"></i>\
                <%- message %>\
            </div>\
        '

    });

});