define(
    [
        'module',
        'views/Base',
        'views/shared/splunkbar/Master',
        'views/shared/litebar/Master',
        'views/shared/appbar/Master',
        'views/shared/footer/Master',
        'views/shared/tourbar/TourBar'
    ],
    function(
        module,
        BaseView,
        SplunkBarView,
        SplunkBarSideView,
        AppBarView,
        FooterView,
        TourBarView
    ) {
        return BaseView.extend({
            moduleId: module.id,
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                this.useSideNav = this.model.user.canUseSidenav();

                if (this.useSideNav) {
                    this.children.sideNav = SplunkBarSideView.create({
                        model: {
                            application: this.model.application,
                            appNav: this.model.appNav,
                            appLocal: this.model.appLocal,
                            user: this.model.user,
                            serverInfo: this.model.serverInfo,
                            config: this.model.config
                        },
                        collection: this.collection 
                    });
                } else {
                    if (this.options.splunkBar) {
                        this.children.splunkBar = SplunkBarView.create({
                            model: {
                                application: this.model.application,
                                appNav: this.model.appNav,
                                appLocal: this.model.appLocal,
                                user: this.model.user,
                                serverInfo: this.model.serverInfo,
                                config: this.model.config
                            },
                            collection: this.collection, 
                            showAppsList: this.options.hasOwnProperty("showAppsList") ? this.options.showAppsList : true  
                        });
                    }
                    if (this.model.appNav) {
                        this.children.appBar = AppBarView.create({
                            section: this.options.section || '',
                            model: {
                                application: this.model.application,
                                appNav: this.model.appNav,
                                user: this.model.user,
                                serverInfo: this.model.serverInfo
                            },
                            collection: this.collection
                        });
                    }
                }
                
                if (this.options.footer) {
                    this.children.footer = FooterView.create({
                        model: this.model,
                        collection: this.collection
                    });
                }
                if (this.model.tour) {
                    this.children.tourBar = new TourBarView({
                        model: this.model.tour
                    });
                }
            },
            resetTourModel: function(model) {
                this.model.tour = model;
                if (this.children.tourBar) {
                    this.children.tourBar.resetModel(this.model.tour);
                } 
            },
            render: function() {
                this.renderLoadingMessage();
                this.renderHeader();
                if (this.model.tour) {
                    $.when(this.options.deferreds.tour).then(function() {
                        this.renderTour();
                    }.bind(this));
                }
                this.renderFooter();
                return this;
            },
            renderHeader: function() {
                if (this.useSideNav) {
                    this.$('header').append(this.children.sideNav.el);
                } else {        
                    if (this.options.splunkBar) {
                        this.$('header').append(this.children.splunkBar.render().el);
                    }
                    if (this.model.appNav) {
                        this.$('header').append(this.children.appBar.el);
                    }
                }
            },
            renderFooter: function() {
                if (this.options.footer) {
                    this.$('footer').append(this.children.footer.render().el);
                }
            },
            renderLoadingMessage: function() {
                var html = this.compiledTemplate({loadingMessage: this.options.loadingMessage || ''});
                this.$el.html(html);
            },
            renderTour: function() {
                this.$('header').append(this.children.tourBar.render().el);
            },
            template: '\
                <a class="navSkip" href="#navSkip" tabIndex="1">Screen reader users, click here to skip the navigation bar</a>\
                <header role="banner"></header>\
                <a id="navSkip"></a> \
                <div class="main-section-body" role="main">\
                    <% if (loadingMessage) { %>\
                        <div class="loading-message"><%- loadingMessage %></div>\
                    <% } %>\
                </div>\
                <footer role="contentinfo"></footer>\
            '
        });
    }
);
