define(
    [
        'underscore',
        'backbone',
        'module',
        'views/Base'
    ],
    function(
        _,
        Backbone,
        module,
        BaseView
    )
    {
        return BaseView.extend({
            moduleId: module.id,
            tagName: 'thead',
            /**
             * @param {Object} options {
             *     model: <Backbone.Model>
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                var defaults = {
                    columns: [] // hash: label, sortKey (optional)
                };
                _.defaults(this.options, defaults);
                this.activate();
            },
            events: {
                'click th': function(e) {
                    var $target = $(e.currentTarget),
                        sortKey = $target.attr('data-key'),
                        sortDirection = $target.hasClass('asc') ? 'desc': 'asc';
                    if (!sortKey) {
                        return true;
                    }
                    this.model.set({sortKey: sortKey, sortDirection: sortDirection});
                    e.preventDefault();
                },
                'click th > a': function(e) {
                    e.preventDefault();
                }
            },
            startListening: function() {
                this.listenTo(this.model, 'change:sortKey change:sortDirection', this.debouncedRender);
            },
            render: function() {
                var html = this.compiledTemplate({
                    _: _,
                    columns: this.options.columns,
                    model: this.model
                });
                this.$el.html(html);
                return this;
            },
            template: '\
                <tr class="">\
                    <% _.each(columns, function(value, sortKey) { %>\
                        <% var sortableClassName = (value.sortKey) ? "sorts" : "" %>\
                        <% var activeClassName = model.get("sortKey") && value.sortKey==model.get("sortKey") ? "active " + model.get("sortDirection") : "" %>\
                        <th data-key="<%- value.sortKey || "" %>" class="<%- sortableClassName %> <%- activeClassName %> <%- value.className || "" %>" <%- value.colSpan ? "colspan=" + value.colSpan : "" %> >\
                        <% if (value.html) { %>\
                            <%= value.html %>\
                        <% } else if (value.sortKey) { %>\
                            <a href="#"><%- value.label %><i class="icon-sorts <%- activeClassName %>"></i></a>\
                        <% } else { %>\
                            <%- value.label %>\
                        <% } %>\
                        </th>\
                    <% }) %>\
                </tr>\
            '
        });
    }
);
