define(
    [
        'underscore',
        'module',
        'jquery',
        'models/search/Report',
        'views/Base',
        'views/shared/delegates/Popdown',
        'views/shared/documentcontrols/dialogs/TitleDescriptionDialog',
        'views/shared/documentcontrols/dialogs/permissions_dialog/Master',
        'views/shared/alertcontrols/dialogs/edit_dialog/Master',
        'views/shared/alertcontrols/dialogs/ActionsDialog',
        'views/shared/alertcontrols/dialogs/EnableDisableDialog',
        'views/shared/alertcontrols/dialogs/clone_dialog/Master',
        'views/shared/documentcontrols/dialogs/DeleteDialog',
        'uri/route'
    ],
    function(
        _,
        module,
        $,
        ReportModel,
        BaseView,
        PopdownView,
        TitleDescriptionDialog,
        PermissionsDialog,
        EditTypeDialog,
        EditActionsDialog,
        EnableDisableDialog,
        CloneDialog,
        DeleteDialog,
        route
    )
    {
        return BaseView.extend({
            moduleId: module.id,
             /**
             * @param {Object} options {
             *     model: {
             *         savedAlert: <models.Report> type Alert,
             *         application: <models.Application>,
             *         appLocal: <models.services.AppLocal>
             *         user: <models.services.admin.User>
             *     },
             *     collection: <collections.services.authorization.Roles>,
             *     {Boolean} button: (Optional) Whether or not the Edit dropdown has class btn-pill. Default is false and class is btn.
             *     {Boolean} showOpenActions: (Optional) Whether or not the open actions links are visable. Default is true.
             *     {Boolean} deleteRedirect: (Optional) Whether or not to redirect to reports page after delete. Default is false.
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);

                var defaults = {
                    button: true,
                    gear: false,
                    showOpenActions: true,
                    deleteRedirect: false
                };

                _.defaults(this.options, defaults);

                this.model.savedAlert.entry.content.on('change:disabled', this.render, this);
            },
            events: {
                'click a.edit-description': function (e) {
                    this.children.titleDescriptionDialog = new TitleDescriptionDialog({
                        model: this.model.savedAlert,
                        onHiddenRemove: true
                    });

                    this.children.titleDescriptionDialog.render().appendTo($("body"));
                    this.children.titleDescriptionDialog.show();

                    e.preventDefault();
                },
                'click a.edit-permissions': function(e) {
                    this.children.permissionsDialog = new PermissionsDialog({
                        model: {
                            document: this.model.savedAlert,
                            nameModel: this.model.savedAlert.entry,
                            user: this.model.user,
                            serverInfo: this.model.serverInfo
                        },
                        collection: this.collection,
                        onHiddenRemove: true,
                        nameLabel: _('Alert').t()
                    });

                    this.children.permissionsDialog.render().appendTo($("body"));
                    this.children.permissionsDialog.show();

                    e.preventDefault();
                },
                'click a.edit-type': function(e) {
                    this.children.editTypeDialog = new EditTypeDialog({
                        model:  {
                            report: this.model.savedAlert,
                            application: this.model.application,
                            appLocal: this.model.appLocal,
                            user: this.model.user,
                            serverInfo: this.model.serverInfo
                        },
                        dialogType: 'edit',
                        onHiddenRemove: true
                    });

                    this.children.editTypeDialog.render().appendTo($("body"));
                    this.children.editTypeDialog.show();

                    e.preventDefault();
                },
                'click a.edit-actions': function(e) {
                    this.children.editActionsDialog = new EditActionsDialog({
                        model:  {
                            report: this.model.savedAlert,
                            application: this.model.application,
                            appLocal: this.model.appLocal,
                            user: this.model.user,
                            serverInfo: this.model.serverInfo
                        },
                        onHiddenRemove: true
                    });

                    this.children.editActionsDialog.render().appendTo($("body"));
                    this.children.editActionsDialog.show();

                    e.preventDefault();
                },
                'click a.enable-disable': function(e) {
                    this.children.enableDisableDialog = new EnableDisableDialog({
                        model: this.model.savedAlert,
                        onHiddenRemove: true
                    });

                    this.children.enableDisableDialog.render().appendTo($("body"));
                    this.children.enableDisableDialog.show();

                    e.preventDefault();
                },
                'click a.clone': function(e) {
                    this.children.cloneDialog = new CloneDialog({
                        model: {
                            savedAlert: this.model.savedAlert,
                            application: this.model.application
                        },
                        onHiddenRemove: true
                    });

                    this.children.cloneDialog.render().appendTo($("body"));
                    this.children.cloneDialog.show();

                    e.preventDefault();
                },
                'click a.delete': function(e){
                    this.children.deleteDialog = new DeleteDialog({
                        model: {
                            report: this.model.savedAlert,
                            application: this.model.application
                        },
                        deleteRedirect: this.options.deleteRedirect,
                        onHiddenRemove: true
                    });

                    this.children.deleteDialog.render().appendTo($("body"));
                    this.children.deleteDialog.show();

                    e.preventDefault();
                }
            },
            render: function() {
                var openInSearch = route.search(
                        this.model.application.get("root"),
                        this.model.application.get("locale"),
                        this.model.application.get("app"),
                        { data: { s: this.model.savedAlert.id }}),
                    canScheduleSearch = this.model.user.canScheduleSearch(),
                    canWrite = this.model.savedAlert.canWrite(canScheduleSearch, this.model.user.canRTSearch()),
                    canClone = this.model.savedAlert.canClone(canScheduleSearch, this.model.user.canRTSearch()),
                    canDelete = this.model.savedAlert.canDelete();

                if (canWrite || (1 < (0 + this.options.showOpenActions + canDelete + canClone))) {
                    this.$el.html(this.compiledTemplate({
                        _: _,
                        route: route,
                        button: this.options.button,
                        gear: this.options.gear,
                        searchLink: openInSearch,
                        enableDiable: this.model.savedAlert.entry.content.get('disabled') ? _('Enable').t() : _('Disable').t(),
                        canChangePerms: this.model.savedAlert.entry.acl.get('can_change_perms'),
                        customAlertActions: this.model.savedAlert.getCustomAlertActions(),
                        applicationModel: this.model.application,
                        alertId: this.model.savedAlert.id
                    }));

                    if (this.options.showOpenActions) {
                        this.$('.open_actions').show();
                    } else {
                        this.$('.open_actions').remove();
                    }

                    if (!canWrite) {
                        this.$('.second-group').remove();
                    }

                    if (canClone) {
                        this.$('.third-group').append('<li><a href="#" class="clone">' + _("Clone").t() + '</a></li>');
                    }

                    if (canDelete) {
                        this.$('.third-group').append('<li><a href="#" class="delete">' + _("Delete").t() + '</a></li>');
                    }
                    this.children.popdown = new PopdownView({ el: this.$el });
                } else {

                    if (this.options.showOpenActions) {
                        this.$el.append('<a class="' + (this.options.button ? "btn" : "") + '" href="' + openInSearch + '">' + _("Open in Search").t() +'</a>');
                    }
                    if (canClone) {
                        this.$el.append('<a class="clone' + (this.options.button ? " btn" : "") + '" href="#">' + _("Clone").t() +'</a>');
                    }
                    if (canDelete) {
                        this.$el.append('<a class="delete' + (this.options.button ? " btn" : "") + '" href="#">' + _("Delete").t() +'</a>');
                    }
                }

                return this;
            },
            template: '\
                <% if (gear) { %>\
                    <a class="dropdown-toggle <%- button ? "btn" : "" %>" href="#"><i class="icon-large icon-gear"></i> <span class="caret"></span></a>\
                <% } else { %>\
                    <a class="dropdown-toggle <%- button ? "btn" : "" %>" href="#"><%- _("Edit").t() %><span class="caret"></span></a>\
                <% } %>\
                <div class="dropdown-menu dropdown-menu-narrow">\
                    <div class="arrow"></div>\
                    <ul class="open_actions">\
                        <li><a href="<%= searchLink %>"><%- _("Open in Search").t() %></a></li>\
                    </ul>\
                    <ul class="second-group">\
                        <li><a class="edit-description" href="#"><%- _("Edit Description").t() %></a></li>\
                        <% if (canChangePerms) { %>\
                            <li><a class="edit-permissions" href="#"><%- _("Edit Permissions").t() %></a></li>\
                        <% } %>\
                        <li><a class="edit-type" href="#"><%- _("Edit Alert Type and Trigger").t() %></a></li>\
                        <li><a class="edit-actions" href="#"><%- _("Edit Actions").t() %></a></li>\
                        <li><a class="enable-disable" href="#"><%= enableDiable %></a></li>\
                    </ul>\
                    <ul class="third-group">\
                    </ul>\
                    <% if (!_.isEmpty(customAlertActions)) { %>\
                        <ul>\
                            <li class="title"><%- _("Integrations").t() %></li>\
                        <% _.each(customAlertActions, function(customAlertAction) { %>\
                            <li>\
                                <a href="<%= route.alertIntegration(applicationModel.get("root"), applicationModel.get("locale"), applicationModel.get("app"), customAlertAction.app, customAlertAction.view, {data: {s: alertId}}) %>">\
                                    <%- customAlertAction.label %>\
                                </a>\
                            </li>\
                        <% }) %>\
                        </ul>\
                    <% } %>\
                </div>\
            '
        });
    }
);
