define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/shared/alertcontrols/dialogs/EnableDisableDialog',
        'views/shared/alertcontrols/dialogs/edit_dialog/Master',
        'views/shared/alertcontrols/details/Type',
        'views/shared/alertcontrols/details/Trigger',
        'views/shared/alertcontrols/dialogs/ActionsDialog',
        'views/shared/alertcontrols/details/Actions',
        'views/shared/documentcontrols/details/App',
        'views/shared/documentcontrols/details/Permissions',
        'views/shared/documentcontrols/dialogs/permissions_dialog/Master',
        'uri/route'
    ],
    function(
        _,
        module,
        BaseView,
        EnableDisableDialog,
        EditTypeAndTriggerDialog,
        TypeTextView,
        TriggerTextView,
        EditActionsDialog,
        ActionsTextView,
        AppText,
        PermissionsTextView,
        PermissionsDialog,
        route
    )
    {
        return BaseView.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model: {
             *         savedAlert: <models.Report>,
             *         application: <models.Application>,
             *         appLocal: <models.services.AppLocal>,
             *         user: <models.services.admin.User>
             *     },
             *     collection: roles: <collections.services.authorization.Roles>
             *     twoColumn: {Boolean} Whether definitions are shown in two columns otherwise one column. Default if false.
             *     displayApp: {Boolean} Whether to display App. Defalut is false
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);

                this.children.triggerTextView = new TriggerTextView({model: this.model.savedAlert});
                this.children.typeTextView = new TypeTextView({model: this.model.savedAlert});
                this.children.actionsTextView = new ActionsTextView({model: this.model.savedAlert});
                this.children.appText = new AppText({model: this.model.savedAlert});
                this.children.permissionsTextView = new PermissionsTextView({model: {report: this.model.savedAlert, user: this.model.user}});

                this.model.savedAlert.entry.content.on('change:disabled', this.render, this);
            },
            events: {
                'click a.edit-permissions': function(e) {
                    this.children.permissionsDialog = new PermissionsDialog({
                        model: {
                            document: this.model.savedAlert,
                            nameModel: this.model.savedAlert.entry,
                            user: this.model.user,
                            serverInfo: this.model.serverInfo
                        },
                        collection: this.collection.roles,
                        onHiddenRemove: true,
                        nameLabel: _('Alert').t()
                    });

                    this.children.permissionsDialog.render().appendTo($("body"));
                    this.children.permissionsDialog.show();

                    e.preventDefault();
                },
                'click a.edit-alert-type-or-trigger': function(e) {
                    this.children.editTypeAndTriggerDialog = new EditTypeAndTriggerDialog({
                        model:  {
                            report: this.model.savedAlert,
                            application: this.model.application,
                            appLocal: this.model.appLocal,
                            user: this.model.user,
                            serverInfo: this.model.serverInfo
                        },
                        dialogType: 'edit',
                        onHiddenRemove: true
                    });

                    this.children.editTypeAndTriggerDialog.render().appendTo($("body"));
                    this.children.editTypeAndTriggerDialog.show();

                    e.preventDefault();
                },
                'click a.edit-alert-actions': function(e) {
                    this.children.editActionsDialog = new EditActionsDialog({
                        model:  {
                            report: this.model.savedAlert,
                            application: this.model.application,
                            appLocal: this.model.appLocal,
                            user: this.model.user,
                            serverInfo: this.model.serverInfo
                        },
                        onHiddenRemove: true
                    });

                    this.children.editActionsDialog.render().appendTo($("body"));
                    this.children.editActionsDialog.show();

                    e.preventDefault();
                },
                'click a.enable-disable': function(e) {
                    this.children.enableDisableDialog = new EnableDisableDialog({
                        model: this.model.savedAlert,
                        onHiddenRemove: true
                    });

                    this.children.enableDisableDialog.render().appendTo($("body"));
                    this.children.enableDisableDialog.show();

                    e.preventDefault();
                }
            },
            render: function() {
                var isDisabled = this.model.savedAlert.entry.content.get('disabled');

                this.$el.html(_.template(this.options.twoColumn ? this.template2Col : this.template, {
                    _: _,
                    route: route,
                    enableDisable: isDisabled ? _('Enable').t() : _('Disable').t(),
                    enableDisableText: isDisabled ? _('No. ').t(): _('Yes. ').t(),
                    displayApp: this.options.displayApp && this.model.user.canUseApps(),
                    customAlertActions: this.model.savedAlert.getCustomAlertActions(),
                    alertId: this.model.savedAlert.id,
                    applicationModel: this.model.application
                }));

                this.children.typeTextView.render().prependTo(this.$('dd.alert-type'));
                this.children.triggerTextView.render().prependTo(this.$('dd.alert-trigger'));
                this.children.actionsTextView.render().prependTo(this.$('dd.alert-actions'));
                this.children.appText.render().prependTo(this.$('dd.app'));
                this.children.permissionsTextView.render().prependTo(this.$('dd.permissions'));

                if (!this.model.savedAlert.canWrite(this.model.user.canScheduleSearch(), this.model.user.canRTSearch())) {
                    this.$('a.edit-link').remove();
                } else {
                    if (!this.model.savedAlert.entry.acl.get("can_change_perms")) {
                        this.$('a.edit-permissions').remove();
                    }
                }

                return this;
            },
            template: '\
                <dl class="list-dotted">\
                <dt><%- _("Enabled").t() %>:</dt>\
                <dd><%= enableDisableText %><a href="#" class="enable-disable edit-link"><%= enableDisable %></a></dd>\
                <% if(displayApp) { %>\
                    <dt><%- _("App").t() %>:</dt>\
                    <dd class="app"></dd>\
                <% } %>\
                <dt><%- _("Alert Type").t() %>:</dt>\
                <dd class="alert-type"> <a href="#" class="edit-alert-type-or-trigger edit-link"><%- _("Edit").t() %></a></dd>\
                <dt><%- _("Trigger Condition").t() %>:</dt>\
                <dd class="alert-trigger"> <a href="#" class="edit-alert-type-or-trigger edit-link"><%- _("Edit").t() %></a></dd>\
                <dt><%- _("Actions").t() %>:</dt>\
                <dd class="alert-actions"> <a href="#" class="edit-alert-actions edit-link"><%- _("Edit").t() %></a></dd>\
                <dt><%- _("Permissions").t() %>:</dt>\
                <dd class="permissions"><a href="#" class="edit-permissions edit-link"><%- _("Edit").t() %></a></dd>\
                <% if (!_.isEmpty(customAlertActions)) { %>\
                <dt><%- _("Integrations").t() %>:</dt>\
                    <% _.each(customAlertActions, function(customAlertAction) { %>\
                        <dd>\
                            <a target="_blank" href="<%= route.alertIntegration(applicationModel.get("root"), applicationModel.get("locale"), applicationModel.get("app"), customAlertAction.app, customAlertAction.view, {data: {s: alertId}}) %>">\
                                <%- customAlertAction.label %>\
                            </a>\
                        </dd>\
                    <% }) %>\
                <% } %>\
                </dl>\
                </td>\
            ',
            template2Col: '\
                <dl class="list-dotted">\
                <dt><%- _("Enabled").t() %>:</dt>\
                <dd><%= enableDisableText %><a href="#" class="enable-disable edit-link"><%= enableDisable %></a></dd>\
                <dt><%- _("Alert Type").t() %>:</dt>\
                <dd class="alert-type"> <a href="#" class="edit-alert-type-or-trigger edit-link"><%- _("Edit").t() %></a></dd>\
                <dt><%- _("Trigger Condition").t() %>:</dt>\
                <dd class="alert-trigger"> <a href="#" class="edit-alert-type-or-trigger edit-link"><%- _("Edit").t() %></a></dd>\
                <% if (!_.isEmpty(customAlertActions)) { %>\
                <dt><%- _("Integrations").t() %>:</dt>\
                    <% _.each(customAlertActions, function(customAlertAction) { %>\
                        <dd>\
                            <a target="_blank" href="<%= route.alertIntegration(applicationModel.get("root"), applicationModel.get("locale"), applicationModel.get("app"), customAlertAction.app, customAlertAction.view, {data: {s: alertId}}) %>">\
                                <%- customAlertAction.label %>\
                            </a>\
                        </dd>\
                    <% }) %>\
                <% } %>\
                </dl>\
                <dl class="list-dotted">\
                 <dt><%- _("Actions").t() %>:</dt>\
                <dd class="alert-actions"> <a href="#" class="edit-alert-actions edit-link"><%- _("Edit").t() %></a></dd>\
                <% if(displayApp) { %>\
                    <dt><%- _("App").t() %>:</dt>\
                    <dd class="app"></dd>\
                <% } %>\
                <dt><%- _("Permissions").t() %>:</dt>\
                <dd class="permissions"><a href="#" class="edit-permissions edit-link"><%- _("Edit").t() %></a></dd>\
                </dl>\
            '
        });
    }
);
