define(
    [
        'underscore',
        'backbone',
        'module',
        'models/search/Alert',
        'views/shared/Modal',
        'views/shared/alertcontrols/dialogs/shared/actions/Master',
        'views/shared/alertcontrols/dialogs/shared/SuccessEdit',
        'views/shared/alertcontrols/dialogs/edit_dialog/CanNotEdit'
     ],
     function(
        _,
        Backbone,
        module,
        AlertModel,
        Modal,
        EditActionsView,
        SuccessView,
        CanNotEditView
    ){
        return Modal.extend({
            moduleId: module.id,
            className: Modal.CLASS_NAME + " " + Modal.CLASS_MODAL_WIDE,
             /**
             * @param {Object} options {
             *     model: {
             *         report: <models.Report> type Alert,
             *         application: <models.Application>,
             *         appLocal: <models.services.AppLocal>
             *     }
             * }
             */
            initialize: function() {
                Modal.prototype.initialize.apply(this, arguments);

                // model
                this.model.inmem = this.model.report.clone();
                this.canNotEdit = (this.model.inmem.entry.content.get('dispatch.earliest_time')==='rt' || this.model.inmem.entry.content.get('dispatch.latest_time') ==='rt') && this.model.inmem.entry.content.get('alert_type') !== 'always';

                // views
                if (this.canNotEdit) {
                    this.children.canNotEditView = new CanNotEditView({
                        model: {
                            application: this.model.application,
                            report: this.model.report
                        }
                    }); 
                } else {
                    this.model.alert = new AlertModel();
                    this.model.alert.transposeFromSavedsearchAlert(this.model.report);
                    
                    this.children.editActionsView = new EditActionsView({
                        model: {
                            alert: this.model.alert,
                            inmem: this.model.inmem,
                            application: this.model.application,
                            appLocal: this.model.appLocal,
                            user: this.model.user,
                            serverInfo: this.model.serverInfo
                        },
                        dialogType: 'edit',
                        dialogTitle: _('Edit Alert').t(),
                        mainScreen: true
                    });
    
                    this.children.successView = new SuccessView({
                        model: {
                            application: this.model.application
                        }
                    });
    
                    this.model.alert.entry.content.on('save', function() {
                        this.model.alert.validate();
                    },this);
    
                    this.model.alert.on('validated', function(isValid, model, payload) {
                        if(isValid){
                            this.saveAlert();
                        }
                    },this);
    
                    this.on('hidden', function(){
                        if (!this.model.alert.isNew()) {
                            this.model.report.fetch();
                        }
                    });
                }
            },
            saveAlert: function() {
                var uiAttrs = this.model.alert.toJSON();
                this.model.alert.transposeToSavedsearch();
                var attrs = this.model.alert.entry.content.toJSON();
                this.model.alert.setFromSplunkD(this.model.report.toSplunkD());
                this.model.alert.entry.content.set(attrs);

                this.model.alert.save({},
                    {
                        validate: false,
                        success: function(){
                            this.children.editActionsView.$el.hide();
                            this.children.successView.$el.show();
                        }.bind(this),
                        error: function() {
                            this.model.alert.clear();
                            this.model.alert.set(uiAttrs);
                        }.bind(this)
                    }
                );
            },
            render: function() {
                if (this.canNotEdit) {
                    this.children.canNotEditView.render().appendTo(this.$el);
                } else {
                    this.children.editActionsView.render().appendTo(this.$el);
                    this.model.alert.trigger('toggleExecuteActions');
                    this.children.successView.render().appendTo(this.$el);
                    this.children.successView.$el.hide();
                }
                return this;
            }
        }
    );
});
