define(
    [
        'underscore',
        'backbone',
        'module',
        'models/search/Alert',
        'models/shared/Cron',
        'models/ACLReadOnly',
        'collections/shared/FlashMessages',
        'views/shared/Modal',
        'views/shared/FlashMessages',
        'helpers/FlashMessagesHelper',
        'views/shared/alertcontrols/dialogs/edit_dialog/CanNotEdit',
        'views/shared/alertcontrols/dialogs/edit_dialog/step1/Master',
        'views/shared/alertcontrols/dialogs/shared/actions/Master',
        'views/shared/alertcontrols/dialogs/shared/SuccessEdit',
        'views/shared/alertcontrols/dialogs/shared/SuccessWithAdditionalSettings',
        'util/splunkd_utils'
    ],
    function(
        _,
        Backbone,
        module,
        AlertModel,
        CronModel,
        ACLReadOnlyModel,
        FlashMessagesCollection,
        Modal,
        FlashMessagesView,
        FlashMessagesHelper,
        CanNotEditView,
        Step1,
        Step2,
        SuccessEdit,
        SuccessCreate,
        splunkd_utils
    ){
        return Modal.extend({
            moduleId: module.id,
            className: Modal.CLASS_NAME + " " + Modal.CLASS_MODAL_WIDE,
            /**
             * @param {Object} options {
             *     model: {
             *         report: <models.Report>,
             *         application: <models.Application>,
             *         appLocal: <models.services.AppLocal>,
             *         user: <models.services.admin.User>
             *     },
             *     {string} dialogType (Optional): edit|new  and indicates if it is a create new or edit dialog. Default is new.
             * }
             */
            initialize: function() {
                Modal.prototype.initialize.apply(this, arguments);

                var defaults = {
                    dialogType: 'new'
                };

                _.defaults(this.options, defaults);

                var dialogTitle = this.options.dialogType === "edit" ? _('Edit Alert').t(): _('Save As Alert').t(),
                    validCron = false;

                //model
                this.model.inmem = this.model.report.clone();
                // for backwards compatibility
                if (this.model.inmem.isNew() || this.options.dialogType === 'new') {
                    this.model.inmem.entry.content.set('action.email.useNSSubject', '1');
                }
                this.model.alert = new AlertModel();
                // sets step and required defaults for a new alert
                this.model.alert.set({
                    'ui.step': 'triggers',
                    'action.email.message.alert': this.model.inmem.entry.content.get('action.email.message.alert'),
                    'action.email.subject.alert': this.model.inmem.entry.content.get('action.email.subject.alert'),
                    'action.email.priority': this.model.inmem.entry.content.get('action.email.priority'),
                    'action.email.useNSSubject': this.model.inmem.entry.content.get('action.email.useNSSubject')
                });

                this.canNotEdit = (this.model.inmem.entry.content.get('dispatch.earliest_time')==='rt' || this.model.inmem.entry.content.get('dispatch.latest_time') ==='rt') && this.model.inmem.entry.content.get('alert_type') !== 'always';

                if (this.canNotEdit) {
                    this.children.canNotEditView = new CanNotEditView({
                        model: {
                            application: this.model.application,
                            report: this.model.report
                        }
                    }); 

                } else {
                    // if edit dialog or new alert from previously saved alert populate alert model from saved alert
                    if(this.model.inmem.isAlert() && (this.options.dialogType === 'edit' || !this.model.inmem.isNew())) {

                        //SPL-68947 reset earliest time latest time incase timerange picker was change
                        if (this.model.reportPristine && !this.model.reportPristine.isNew()) {
                            this.model.inmem.entry.content.set({
                                'dispatch.earliest_time': this.model.reportPristine.entry.content.get('dispatch.earliest_time'),
                                'dispatch.latest_time': this.model.reportPristine.entry.content.get('dispatch.latest_time')
                            });
                        }

                        this.model.alert.transposeFromSavedsearchAlert(this.model.inmem);
                        if (this.options.dialogType === 'new') {
                            this.model.alert.set('ui.name', '');
                        }
                    }

                    // if new alert from a saved report not of type alert
                    if (this.options.dialogType === 'new' && !this.model.inmem.isNew() && !this.model.inmem.isAlert()) {
                        this.model.inmem.stripReportAttributesToSaveAsAlert();
                        this.model.alert.set('ui.description', this.model.inmem.entry.content.get('description'));
                    }

                    //views
                    this.children.step1 = new Step1({
                        model:  {
                            alert: this.model.alert,
                            user: this.model.user,
                            inmem: this.model.inmem,
                            application: this.model.application,
                            serverInfo: this.model.serverInfo
                        },
                        dialogType: this.options.dialogType,
                        dialogTitle: dialogTitle
                    });

                    this.children.step2 = new Step2({
                        model: {
                            alert: this.model.alert,
                            inmem: this.model.inmem,
                            application: this.model.application,
                            appLocal: this.model.appLocal,
                            user: this.model.user,
                            serverInfo: this.model.serverInfo
                        },
                        dialogType: this.options.dialogType,
                        dialogTitle: dialogTitle
                    });

                    if(this.options.dialogType === 'edit') {
                        this.children.success = new SuccessEdit({
                            model: {
                                application: this.model.application
                            }
                        });
                    } else {
                        this.children.success = new SuccessCreate({
                            model: {
                                alert: this.model.alert,
                                application: this.model.application
                            }
                        });
                    }

                    this.model.alert.cron.on('validated', function(isValid, model, payload) {
                        validCron = isValid;
                        this.model.alert.validate();
                    },this);

                    this.model.alert.on('validated', function(isValid, model, payload) {
                        if(validCron && isValid){
                            if (this.model.alert.get('ui.step') === 'triggers') {
                                this.model.alert.set('ui.step', 'actions');
                                this.children.step1.$el.hide();
                                this.children.step2.$el.show();
                                this.model.alert.trigger('toggleExecuteActions');
                            } else {
                                this.saveAlert();
                            }
                        }
                    },this);

                    this.model.alert.entry.content.on('next', function(){
                        this.model.alert.cron.set({'ui.type': this.model.alert.get('ui.type')});
                        this.model.alert.cron.validate();
                    },this);

                    this.model.alert.entry.content.on('save', function(){
                        this.model.alert.validate();
                    },this);

                    this.model.alert.entry.content.on('back', function(){
                        this.model.alert.set('ui.step', 'triggers');
                        this.children.step2.$el.hide();
                        this.children.step1.$el.show();
                    },this);
                    
                    this.model.alert.on('saveSuccess', function() {
                        this.children.step2.$el.hide();
                        this.children.success.render().appendTo(this.$el);
                    }, this);
                    
                    this.on('hidden', function(){
                        if (!this.model.alert.isNew()) {
                            this.model.report.fetch({url: splunkd_utils.fullpath(this.model.alert.id)});
                        }
                    });
                }
            },
            saveAlert: function() {
                var uiAttrs = this.model.alert.toJSON(),
                    uiCronAttrs = this.model.alert.cron.toJSON(),
                    uiWorkingTimeRange = this.model.alert.workingTimeRange.toJSON();
                this.model.alert.transposeToSavedsearch();
                var permissions = this.model.alert.get('ui.permissions'),
                    attrs = this.model.alert.entry.content.toJSON(),
                    data = {};

                this.model.alert.setFromSplunkD(this.model.inmem.toSplunkD());
                this.model.alert.entry.content.set(attrs);

                //ensure that summary_index is always set to false
                this.model.alert.entry.content.set("action.summary_index", "0");

                // check if saving from an already saved report
                if (this.options.dialogType === 'new' && !this.model.alert.isNew()) {
                    this.model.alert.entry.content.set('disabled', false);
                    this.model.alert.unset(this.model.alert.idAttribute);
                }

                if (this.model.alert.isNew()) {
                    this.model.alert.entry.content.set({
                        'request.ui_dispatch_app': this.model.application.get('app'),
                        'request.ui_dispatch_view': this.model.application.get('page')
                    });

                    data = {
                        app: this.model.application.get('app'),
                        owner: ((permissions === splunkd_utils.USER) ? this.model.application.get('owner') : splunkd_utils.NOBODY)
                    };
                }
                this.model.alert.save({},
                    {
                        validate: false,
                        data: data,
                        success: function(model){
                            if (this.options.dialogType === 'new' && model.entry.acl.get('sharing') !== permissions) {
                                this.model.aclReadOnly = new ACLReadOnlyModel($.extend(true, {}, model.entry.acl.toJSON()));
                                this.model.aclReadOnly.set('sharing', permissions);
                                var data = this.model.aclReadOnly.toDataPayload();
                                this.model.alert.acl.save({}, {
                                    data: data,
                                    success: function() {
                                        this.model.alert.trigger('saveSuccess');
                                    }.bind(this)
                                });
                            } else {
                                this.model.alert.trigger('saveSuccess');
                            }
                        }.bind(this),
                        error: function() {
                            this.model.alert.clear();
                            this.model.alert.set(uiAttrs);
                            this.model.alert.cron.set(uiCronAttrs);
                            this.model.alert.workingTimeRange.set(uiWorkingTimeRange);
                            this.model.alert.set('ui.step', 'triggers');
                            this.children.step2.$el.hide();
                            this.children.step1.$el.show();
                        }.bind(this)
                    }
                );
            },
            render: function() {
                if (this.canNotEdit) {
                    this.children.canNotEditView.render().appendTo(this.$el);
                } else {
                    this.children.step1.render().appendTo(this.$el);
                    this.children.step2.render().appendTo(this.$el);
                    this.children.step2.$el.hide();
                }
                return this;
            }
        }
    );
});
