define(
    [
        'underscore',
        'backbone',
        'module',
        'views/Base',
        'bootstrap.modal',
        'views/shared/controls/ControlGroup',
        'views/shared/controls/SyntheticSelectControl',
        'views/shared/controls/TextControl',
        'views/shared/alertcontrols/dialogs/edit_dialog/TriggerCondition'
    ],
    function(
        _,
        Backbone,
        module,
        Base,
        undefined,
        ControlGroup,
        SyntheticSelectControl,
        TextControl,
        TriggerCondition
    )
    {
        return Base.extend({
            moduleId: module.id,
            className: ' realtime-master-view',
            /**
             * @param {Object} options {
             *     model: {
             *         alert: <models.Alert>
             *     }
             * }
             */
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);

                var triggerConditionOptions = {
                    items: [
                        {
                            value: 'per_result',
                            label: _('Per-Result').t(),
                            description: _('Triggers whenever search returns a result.').t()
                        },
                        {
                            value: 'events',
                            label: _('Number of Results').t(),
                            description: _('Triggers based on a number of search results during a rolling-window of time.').t()
                        },
                        {
                            value: 'hosts',
                            label: _('Number of Hosts').t(),
                            description: _('Triggers based on a number of hosts during a rolling-window of time.').t()
                        },
                        {
                            value: 'sources',
                            label: _('Number of Sources').t(),
                            description: _('Triggers based on a number of sources during a rolling-window of time.').t()
                        },
                        {
                            value: 'custom',
                            label: _('Custom').t(),
                            description: _('Triggers based on a custom condition during a rolling-window time.').t()
                        }
                    ],
                    model: this.model,
                    modelAttribute: 'ui.realtime.triggercondition',
                    defaultValue: 'per_result',
                    className: 'controls',
                    popdownOptions: {
                        attachDialogTo: '.modal:visible',
                        scrollContainer: '.modal:visible .modal-body:visible'
                    }
                };

                //views
                this.children.triggerCondition = new TriggerCondition(triggerConditionOptions);

                this.children.numResultsTriggerView = new ControlGroup({
                    label: _('Number of results is').t(),
                    controlClass: 'controls-split input-prepend',
                    controls: [
                        new SyntheticSelectControl ({
                            modelAttribute: 'ui.realtime.resultscomparator',
                            model: this.model,
                            items: [
                                { label: _('Greater than').t(), value: 'greater than' },
                                { label: _('Less than').t(), value: 'less than' },
                                { label: _('Equal to').t(), value: 'equal to' },
                                { label: _('Not equal to').t(), value: 'not equal to' }
                            ],
                            save: false,
                            toggleClassName: 'btn',
                            labelPosition: 'outside',
                            elastic: true,
                            popdownOptions: {
                                attachDialogTo: '.modal:visible',
                                scrollContainer: '.modal:visible .modal-body:visible'
                            }
                        }),
                        new TextControl({
                            modelAttribute: 'ui.realtime.resultscomparatorinput',
                            model: this.model,
                            save: false
                        })
                    ]
                });

                this.children.customTriggerView = new ControlGroup({
                    className: 'alert-name control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    help: _('e.g. "search count > 10". Evaluted against the results of the base search.').t(),
                    controlOptions: {
                        modelAttribute: 'ui.realtime.customsearch',
                        model: this.model,
                        save: false
                    },
                    label: _('Custom Condition').t()
                });

                this.children.rollingTimeWindow = new ControlGroup({
                    label: _('in').t(),
                    controlClass: 'controls-split input-append',
                    controls: [
                        new TextControl({
                            modelAttribute: 'ui.realtime.resultstime',
                            model: this.model,
                            save: false
                        }),
                        new SyntheticSelectControl({
                            modelAttribute: 'ui.realtime.resultstimeunit',
                            model: this.model,
                            items: [
                                { label: _('minute(s)').t(), value: 'm' },
                                { label: _('hour(s)').t(), value: 'h' },
                                { label: _('day(s)').t(), value: 'd' }
                            ],
                            save: false,
                            toggleClassName: 'btn',
                            labelPosition: 'outside',
                            elastic: true,
                            popdownOptions: {
                                attachDialogTo: '.modal:visible',
                                scrollContainer: '.modal:visible .modal-body:visible'
                            }
                        })
                    ]
                });

                // listeners
                this.model.on('change:ui.realtime.triggercondition', function() {
                    this.toggleChildren();
                }, this);
            },

            toggleChildren: function() {
                switch(this.model.get('ui.realtime.triggercondition')){
                    case 'events':
                    case 'hosts':
                    case 'sources':
                        this.children.rollingTimeWindow.$el.show();
                        this.children.numResultsTriggerView.$el.show();
                        this.children.customTriggerView.$el.hide();
                        break;
                    case 'custom':
                        this.children.rollingTimeWindow.$el.show();
                        this.children.customTriggerView.$el.show();
                        this.children.numResultsTriggerView.$el.hide();
                        break;
                    default:
                        this.children.customTriggerView.$el.hide();
                        this.children.numResultsTriggerView.$el.hide();
                        this.children.rollingTimeWindow.$el.hide();
                        break;
                }
            },

        render: function() {
                this.$el.html(this.compiledTemplate({
                    _: _
                }));
                this.children.triggerCondition.render().appendTo(this.$el.find('.realtime-controls'));
                this.children.numResultsTriggerView.render().appendTo(this.$el);
                this.children.customTriggerView.render().appendTo(this.$el);
                this.children.rollingTimeWindow.render().appendTo(this.$el);

                this.toggleChildren();

                return this;
            },

        template: '\
            <div class="realtime-controls control-group controls-block">\
                <label class="control-label"><%- _("Trigger condition").t() %></label>\
            </div>\
        '
        }
    );
});
