define(
    [
        'underscore',
        'backbone',
        'module',
        'views/Base',
        'bootstrap.modal',
        'views/shared/controls/ControlGroup',
        'views/shared/controls/SyntheticSelectControl',
        'views/shared/controls/TextControl',
        'views/shared/ScheduleSentence',
        'views/shared/alertcontrols/dialogs/edit_dialog/TriggerCondition',
        'models/services/search/TimeParser',
        'views/shared/timerangepicker/dialog/advanced/timeinput/Master',
        'uri/route',
        'splunk.util'
    ],
    function(
        _,
        Backbone,
        module,
        Base,
        boostrapModal,
        ControlGroup,
        SyntheticSelectControl,
        TextControl,
        ScheduleSentenceView,
        TriggerCondition,
        TimeParserModel,
        TimeInput,
        route,
        splunkUtil

    ){
        return Base.extend({
            moduleId: module.id,
            className: 'controls-wrapper',
             /**
             * @param {Object} options {
             *     model: {
             *         alert: <models.Alert>,
             *         application: <models.Application>
             *     }
             * }
             */
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);

                var triggerConditionOptions = {
                    items: [
                        {
                            value: 'events',
                            label: _('Number of Results').t(),
                            description: _('Triggers based on a number of search results during a scheduled search.').t()
                        },
                        {
                            value: 'hosts',
                            label: _('Number of Hosts').t(),
                            description: _('Triggers based on a number of hosts during a scheduled search.').t()
                        },
                        {
                            value: 'sources',
                            label: _('Number of Sources').t(),
                            description: _('Triggers based on a number of sources during a scheduled search.').t()
                        },
                        {
                            value: 'custom',
                            label: _('Custom').t(),
                            description: _('Triggers based on a custom condition during a scheduled search.').t()
                        }
                    ],
                    model: this.model.alert,
                    modelAttribute: 'ui.scheduled.triggercondition',
                    popdownOptions: {
                        attachDialogTo: '.modal:visible',
                        scrollContainer: '.modal:visible .modal-body:visible'
                    }
                };

                // views
                this.children.triggerCondition   = new TriggerCondition(triggerConditionOptions);

                this.children.scheduleSentenceView = new ScheduleSentenceView({
                    model: {
                        cron: this.model.alert.cron,
                        application: this.model.application
                    },
                    lineOneLabel: _('Time Range').t(),
                    lineTwoLabel: _('Schedule').t(),
                    popdownOptions: {
                        attachDialogTo: '.modal:visible',
                        scrollContainer: '.modal:visible .modal-body:visible'
                    }

                });

                this.model.earliestTimeParser = new TimeParserModel();
                this.model.latestTimeParser = new TimeParserModel();

                this.children.earliestTimeInput = new TimeInput({
                    model: {
                        working: this.model.alert.workingTimeRange,
                        timeParser: this.model.earliestTimeParser
                    },
                    label: _("Earliest").t(),
                    blankValue: '0',
                    modelAttribute: 'earliest',
                    className:'earliest-timepicker control-group',
                    help: _('this is some help text ....').t() // timeinput doesn't support help text
                });

                this.children.latestTimeInput = new TimeInput({
                    model: {
                        working: this.model.alert.workingTimeRange,
                        timeParser: this.model.latestTimeParser
                    },
                    label: _("Latest").t(),
                    blankValue: 'now',
                    modelAttribute: 'latest',
                    className:'latest-timepicker control-group'
                });

                this.children.resultsComparativeGroup = new ControlGroup ({
                    label: _('Trigger if number of results').t(),
                    controlClass: 'controls-split input-prepend',
                    controls: [
                        new SyntheticSelectControl ({
                            modelAttribute: 'ui.scheduled.resultscomparator',
                            model: this.model.alert,
                            items: [
                                { label: _('is Greater than').t(), value: 'greater than' },
                                { label: _('is Less than').t(), value: 'less than' },
                                { label: _('is Equal to').t(), value: 'equal to' },
                                { label: _('is Not equal to').t(), value: 'not equal to' },
                                { label: _('Drops by').t(), value: 'drops by' },
                                { label: _('Rises by').t(), value: 'rises by' }
                            ],
                            save: false,
                            toggleClassName: 'btn',
                            labelPosition: 'outside',
                            elastic: true,
                            popdownOptions: {
                                attachDialogTo: '.modal:visible',
                                scrollContainer: '.modal:visible .modal-body:visible'
                            }
                        }),
                        new TextControl ({
                            modelAttribute: 'ui.scheduled.resultsinput',
                            model: this.model.alert,
                            save: false
                        })
                    ]
                });

                this.children.customInput = new ControlGroup({
                    className: 'alert-name custom-condition control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    help: _('e.g. "search count > 10". Evaluted against the results of the base search.').t(),
                    controlOptions: {
                        modelAttribute: 'ui.scheduled.customsearch',
                        model: this.model.alert,
                        save: false
                    },
                    label: _('Custom Condition').t()
                });

                // listeners
                this.model.alert.on('change:ui.scheduled.triggercondition', function() {
                    this.toggleTriggerCondition();
                }, this);
            },
            toggleTriggerCondition: function() {
                switch(this.model.alert.get('ui.scheduled.triggercondition')){
                    case 'events':
                    case 'hosts':
                    case 'sources':
                        this.children.resultsComparativeGroup.$el.show();
                        this.children.customInput.$el.hide();
                        break;
                    case 'custom':
                        this.children.resultsComparativeGroup.$el.hide();
                        this.children.customInput.$el.show();
                        break;
                }
            },
            render: function() {
                this.$el.html(this.compiledTemplate({
                    _: _
                }));
                this.children.scheduleSentenceView.render().prependTo(this.$el);
                var $customTime = this.children.scheduleSentenceView.$el.find('.custom_time');
                this.children.latestTimeInput.render().prependTo($customTime);
                this.children.earliestTimeInput.render().prependTo($customTime);
                this.children.triggerCondition.render().appendTo(this.$el.find('.scheduled-controls'));
                this.children.resultsComparativeGroup.render().appendTo(this.$el);
                this.children.customInput.render().appendTo(this.$el);

                // inject help text
                var docRoute = route.docHelp(
                    this.model.application.get("root"),
                    this.model.application.get("locale"),
                    'learnmore.timerange.picker'
                );
                var helpBlock = '<span class="help-block">' +
                    splunkUtil.sprintf(_("e.g. -1h@h (1 hour ago, to the hour). %s").t(),
                        '<a href="'+ docRoute +'" class="help" target="_blank" title="' + 
                        _("Splunk help").t() +'">' + _("Learn More").t()) + '</a></span>';
                this.$el.find('.earliest-timepicker .controls').append(helpBlock);
                this.$el.find('.latest-timepicker .controls').append(helpBlock);
                // ghetto form hacking
                this.$el.find('.earliest-timepicker .controls').addClass('controls-block');
                this.$el.find('.latest-timepicker .controls').addClass('controls-block');

                this.toggleTriggerCondition();

                return this;
            },
            template: '\
                <div class="scheduled-controls control-group controls-block">\
                    <label class="control-label"><%- _("Trigger condition").t() %></label>\
                </div>\
            '
        }
    );
});
