define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/shared/Modal',
        'views/shared/FlashMessages',
        'uri/route'
     ],
     function(_, module, Base, Modal, FlashMessages, route){
        return Base.extend({
            moduleId: module.id,
             /**
             * @param {Object} options {
             *     model: {
             *         alert: <models.Report>,
             *         application: <models.Application>
             *     }
             * }
             */
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);
                this.children.flashMessage = new FlashMessages({ model: this.model.alert });
            },
            render: function() {
                //render is only called after the alert has been successfully saved and id is set
                var routeToAlert = route.alert(this.model.application.get('root'),
                                     this.model.application.get('locale'),
                                     this.model.application.get("app"),
                                     {data: {s: this.model.alert.id}}),
                    customAlertActions = this.model.alert.getCustomAlertActions();
                
                this.$el.html(Modal.TEMPLATE);

                this.$(Modal.HEADER_TITLE_SELECTOR).html(_("Alert has been saved").t());
                this.$(Modal.BODY_SELECTOR).html(this.compiledTemplate({
                    _: _,
                    route: route,
                    applicationModel: this.model.application,
                    alertId: this.model.alert.id,
                    customAlertActions: customAlertActions,
                    hasCustomAlertActions: !_.isEmpty(customAlertActions),
                    canChangePerms: this.model.alert.entry.acl.get('can_change_perms'),
                    routeToPermissions: route.alert(this.model.application.get('root'),
                                           this.model.application.get('locale'),
                                           this.model.application.get("app"),
                                           {data: {s: this.model.alert.id, dialog: 'permissions'}})
                }));
                this.children.flashMessage.render().prependTo(this.$(Modal.BODY_SELECTOR));

                this.$(Modal.FOOTER_SELECTOR).append(Modal.BUTTON_CONTINUE);
                this.$(Modal.FOOTER_SELECTOR).append('<a href="' + routeToAlert + '" class="btn btn-primary modal-btn-primary">' + _("View Alert").t() + '</a>');

                return this;
            },
            focus: function() {
                this.$('.btn-primary').focus();
            },
            template: '\
                <p>\
                    <span class="alert-success-message">\
                    <% if (hasCustomAlertActions && canChangePerms) { %>\
                        <%- _("You can view your alert, change additional settings, configure integrations, or continue editing it.").t() %>\
                    <% } else if (canChangePerms) { %>\
                        <%- _("You can view your alert, change additional settings, or continue editing it.").t() %>\
                    <% } else if (hasCustomAlertActions) { %>\
                        <%- _("You can view your alert, configure integrations, or continue editing it.").t() %>\
                    <% } else { %>\
                        <%- _("You can view your alert, or continue editing it.").t() %>\
                    <% } %>\
                    </span>\
                </p>\
                <% if (canChangePerms) { %>\
                    <p class="can-change-perms">\
                        <%- _("Additional Settings:").t() %>\
                        <ul class="can-change-perms">\
                            <li><a href="<%- routeToPermissions %>"><%- _("Permissions").t() %></a></li>\
                        </ul>\
                    </p>\
                <% } %>\
                <% if (hasCustomAlertActions) { %>\
                    <p class="integrations">\
                        <%- _("Integrations:").t() %>\
                        <ul>\
                        <% _.each(customAlertActions, function(customAlertAction) { %>\
                            <li>\
                                <a target="_blank" href="<%= route.alertIntegration(applicationModel.get("root"), applicationModel.get("locale"), applicationModel.get("app"), customAlertAction.app, customAlertAction.view, {data: {s: alertId}}) %>">\
                                    <%- customAlertAction.label %> <i class="icon-external"></i>\
                                </a>\
                            </li>\
                        <% }) %>\
                        </ul>\
                    </p>\
                <% } %>\
            '
        });
    }
);