define(
    [
        'underscore',
        'backbone',
        'module',
        'views/Base',
        'views/shared/controls/ControlGroup',
        'views/shared/controls/SyntheticSelectControl',
        'views/shared/controls/TextControl'
    ],
    function(
        _,
        Backbone,
        module,
        Base,
        ControlGroup,
        SyntheticSelectControl,
        TextControl
    ) {
        return Base.extend({
            moduleId: module.id,
            className: 'action-options-view',
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);

                this.children.executeActions = new ControlGroup({
                    className: 'alert-type control-group',
                    controlType: 'SyntheticRadio',
                    controlClass: 'controls-halfblock',
                    controlOptions: {
                        modelAttribute: 'ui.executeactions',
                        model: this.model,
                        items: [
                            { label: _('Once').t(), value: true },
                            { label: _('For each result').t(), value: false }
                        ],
                        save: false
                    },
                    label: _('When triggered, execute actions').t()
                });
                this.children.throttle = new ControlGroup({
                    className: 'control-group',
                    controlType: 'SyntheticCheckbox',
                    controlOptions: {
                        modelAttribute: 'ui.throttle',
                        model: this.model,
                        save: false
                    },
                    tooltip: _('After an alert is triggered, subsequent alerts will not be triggered until after the throttle period.').t(),
                    label: _('Throttle').t()
                });
                this.children.fieldValue = new ControlGroup({
                    className: 'alert-name control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'ui.fieldvalue',
                        model: this.model,
                        save: false
                    },
                    label: _('Suppress results containing field value').t()
                });

                this.children.supressTime = new ControlGroup({
                    label:_('Suppress triggering for').t(),
                    controlClass: 'controls-split input-append',
                    controls: [
                        new TextControl({
                            modelAttribute: 'ui.supresstime',
                            model: this.model,
                            save: false
                        }),
                        new SyntheticSelectControl({
                            modelAttribute: 'ui.supresstimeunit',
                            model: this.model,
                            items: [
                                { label: _('second(s)').t(), value: 's' },
                                { label: _('minute(s)').t(), value: 'm' },
                                { label: _('hour(s)').t(), value: 'h' },
                                { label: _('day(s)').t(), value: 'd' }
                            ],
                            save: false,
                            toggleClassName: 'btn',
                            labelPosition: 'outside',
                            popdownOptions: {
                                attachDialogTo: '.modal:visible',
                                scrollContainer: '.modal:visible .modal-body:visible'
                            }
                        })

                    ]
                });

                this.model.on('change:ui.throttle change:ui.executeactions change:ui.realtime.triggercondition change:ui.type', _.debounce(function() {
                    this.throttleToggle();
                },0),this);

                this.model.on('toggleExecuteActions', function(){
                    if (this.model.get('ui.type') === 'realtime' && this.model.get('ui.realtime.triggercondition') === 'per_result') {
                        this.children.executeActions.$el.hide();
                    } else {
                        this.children.executeActions.$el.show();
                    }
                }, this);
            },
            throttleToggle: function() {
                if(this.model.get('ui.throttle')){
                    if(!this.model.get('ui.executeactions') || this.model.get('ui.type') === 'realtime'  && this.model.get('ui.realtime.triggercondition') === 'per_result'){
                        this.children.fieldValue.$el.show();
                    } else {
                        this.children.fieldValue.$el.hide();
                    }
                    this.children.supressTime.$el.show();
                    this.$el.find('.throttle-options').addClass('outline');
                } else{
                    this.children.fieldValue.$el.hide();
                    this.children.supressTime.$el.hide();
                    this.$el.find('.throttle-options').removeClass('outline');
                }
            },
            render: function()  {
                this.children.executeActions.render().appendTo(this.$el);
                this.$el.append('<fieldset class="throttle-options"></fieldset>');
                var $throttleOptions = this.$el.find('.throttle-options');
                this.children.throttle.render().appendTo($throttleOptions);
                this.children.fieldValue.render().appendTo($throttleOptions);
                this.children.supressTime.render().appendTo($throttleOptions);

                this.throttleToggle();

                return this;
            }
        });
});
