define(
    [
        'underscore',
        'backbone',
        'module',
        'views/Base',
        'views/shared/EmailOptions',
        'views/shared/controls/ControlGroup',
        'views/shared/controls/SyntheticCheckboxControl',
        'views/shared/controls/SyntheticSelectControl',
        'splunk.util',
        'uri/route'
    ],
    function(_, Backbone, module, Base, EmailOptions, ControlGroup, SyntheticCheckboxControl, SyntheticSelectControl, splunkUtil, route) {
        return Base.extend({
            moduleId: module.id,
            className: 'enable-actions-view',
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);

                this.children.showTriggeredAlerts = new ControlGroup({
                    className: 'control-group',
                    controlType: 'SyntheticCheckbox',
                    controlOptions: {
                        modelAttribute: 'ui.showtriggeredalerts',
                        model: this.model.alert
                    },
                    label: _('List in Triggered Alerts').t(),
                    help: _('Triggered Alerts is available in the activity menu.').t()
                });

                this.children.severity = new ControlGroup({
                    className: 'control-group',
                    controlType: 'SyntheticSelect',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'ui.severity',
                        model: this.model.alert,
                        items: [
                            { label: _('Info').t(), value: 1 },
                            { label: _('Low').t(), value: 2 },
                            { label: _('Medium').t(), value: 3 },
                            { label: _('High').t(), value: 4 },
                            { label: _('Critical').t(), value: 5 }
                        ],
                        toggleClassName: 'btn',
                        labelPosition: 'outside',
                        elastic: true,
                        popdownOptions: {
                            attachDialogTo: '.modal:visible',
                            scrollContainer: '.modal:visible .modal-body:visible'
                        }
                    },
                    label: _('Severity').t()
                });
                var configEmailHelpLink = route.docHelp(
                        this.model.application.get("root"),
                        this.model.application.get("locale"),
                        'learnmore.alert.email'
                );

                var emailHelpText;
                if (this.model.serverInfo && this.model.serverInfo.isLite()) {
                    emailHelpText = _('Email must be configured in Server&nbsp;Settings > Email&nbsp;Settings. %s').t();
                } else {
                    emailHelpText = _('Email must be configured in System&nbsp;Settings > Alert&nbsp;Email&nbsp;Settings. %s').t();
                }

                this.children.sendEmail = new ControlGroup({
                    className: 'control-group',
                    controlType: 'SyntheticCheckbox',
                    controlOptions: {
                        modelAttribute: 'ui.email',
                        model: this.model.alert
                    },
                    label: _('Send Email').t(),
                    help: splunkUtil.sprintf(emailHelpText, ' <a href="' + configEmailHelpLink + '" target="_blank" title="' + _("Splunk help").t() +'">' + _("Learn More").t() + ' <i class="icon-external"></i></a>')
                });

                this.children.emailOptions = new EmailOptions({
                    model: {
                        state: this.model.alert,
                        application: this.model.application
                    },
                    suffix: 'alert',
                    includeSubjectDefaultPlaceholder: true
                });

                this.children.includeResultsLink = new SyntheticCheckboxControl({
                    modelAttribute: 'ui.include_results_link',
                    model: this.model.alert,
                    label: _('Link to Results').t()
                });
                
                this.children.includeInline = new SyntheticCheckboxControl({
                    additionalClassNames: 'include-inline',
                    modelAttribute: 'ui.include_results_inline',
                    model: this.model.alert,
                    label: _('Inline').t()
                });

                this.children.includeInlineFormat = new SyntheticSelectControl({
                    additionalClassNames: 'include-inline-format',
                    modelAttribute: 'ui.include_results_inline_format',
                    menuWidth: 'narrow',
                    model: this.model.alert,
                    items: [
                        { label: _('Table').t(), value: 'table' },
                        { label: _('Raw').t(), value: 'raw' },
                        { label: _('CSV').t(), value: 'csv' }
                    ],
                    labelPosition: 'outside',
                    popdownOptions: {
                        attachDialogTo: '.modal:visible',
                        scrollContainer: '.modal:visible .modal-body:visible'
                    }
                });
                
                this.children.includeCSV = new SyntheticCheckboxControl({
                    modelAttribute: 'ui.include_results_csv',
                    model: this.model.alert,
                    label: _('Attach CSV').t()
                });

                this.children.includePDF = new SyntheticCheckboxControl({
                    modelAttribute: 'ui.include_results_pdf',
                    model: this.model.alert,
                    label: _('Attach PDF').t()
                });

                this.children.includeSearchString = new SyntheticCheckboxControl({
                    modelAttribute: 'ui.include_search',
                    model: this.model.alert,
                    label: _('Search String').t()
                });
                this.children.includeViewLink = new SyntheticCheckboxControl({
                    modelAttribute: 'ui.include_view_link',
                    model: this.model.alert,
                    label: _('Link to Alert').t()
                });

                this.children.includeTriggerCondition = new SyntheticCheckboxControl({
                    modelAttribute: 'ui.include_trigger',
                    model: this.model.alert,
                    label: _('Trigger Condition').t()
                });

                this.children.includeTriggerTime = new SyntheticCheckboxControl({
                    modelAttribute: 'ui.include_trigger_time',
                    model: this.model.alert,
                    label: _('Trigger Time').t()
                });

                var includeCheckboxes = [
                    this.children.includeViewLink,
                    this.children.includeResultsLink,
                    this.children.includeSearchString,
                    this.children.includeInline,
                    this.children.includeInlineFormat,
                    this.children.includeTriggerCondition,
                    this.children.includeCSV,
                    this.children.includeTriggerTime
                ];
                
                if (this.options.pdfAvailable) {
                    includeCheckboxes.push(this.children.includePDF);
                }

                this.children.emailInclude = new ControlGroup({
                    controlClass: 'email-include',
                    controls: includeCheckboxes,
                    label: _('Include').t()
                });

                this.children.runScript = new ControlGroup({
                    className: 'control-group',
                    controlType: 'SyntheticCheckbox',
                    controlOptions: {
                        modelAttribute: 'ui.script',
                        model: this.model.alert
                    },
                    label: _('Run a Script').t()
                });
                this.children.scriptName = new ControlGroup({
                    className: 'alert-name control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'ui.scriptfilename',
                        model: this.model.alert
                    },
                    label: _('Filename').t(),
                    help: _('Located in $SPLUNK_HOME/bin/scripts').t()
                });

                this.model.alert.on('change:ui.showtriggeredalerts', function(){
                    this.toggleShowTriggeredAlerts();
                },this);

                this.model.alert.on('change:ui.email', function(){
                    this.toggleEmail();
                },this);

                this.model.alert.on('change:ui.script', function(){
                    this.toggleScript();
                },this);

                this.model.alert.on('change:ui.include_results_inline_format', function(){
                    if (!this.model.alert.get('ui.include_results_inline')) {
                        this.model.alert.set('ui.include_results_inline', 1);
                    }
                }, this);
            },
            toggleShowTriggeredAlerts: function() {
                if (this.model.alert.get('ui.showtriggeredalerts')) {
                    this.children.severity.$el.show();
                    this.$el.find('.alert-options').addClass('outline');
                } else {
                    this.children.severity.$el.hide();
                    this.$el.find('.alert-options').removeClass('outline');
                }
            },
            toggleEmail: function() {
                if (this.model.alert.get('ui.email')){
                    this.children.emailOptions.$el.show();
                    this.children.emailInclude.$el.show();
                    this.$el.find('.email-options').addClass('outline');

                } else {
                    this.children.emailOptions.$el.hide();
                    this.children.emailInclude.$el.hide();
                    this.$el.find('.email-options').removeClass('outline');
                }
            },
            toggleScript: function() {
                if (this.model.alert.get('ui.script')) {
                    this.children.scriptName.$el.show();
                    this.$el.find('.script-options').addClass('outline');
                } else {
                    this.children.scriptName.$el.hide();
                    this.$el.find('.script-options').removeClass('outline');
                }
            },
            toggleAll: function() {
                this.toggleShowTriggeredAlerts();
                this.toggleEmail();
                this.toggleScript();
            },
            render: function()  {
                // add some structure
                this.$el.append('<fieldset class="alert-options"></fieldset>');
                var $alertOptions = this.$el.find('.alert-options');
                this.children.showTriggeredAlerts.render().appendTo($alertOptions);
                this.children.severity.render().appendTo($alertOptions);

                this.$el.append('<fieldset class="email-options"></fieldset>');
                var $emailOptions = this.$el.find('.email-options');
                this.children.sendEmail.render().appendTo($emailOptions);
                this.children.emailOptions.render().appendTo($emailOptions);
                this.children.emailInclude.render().appendTo($emailOptions);

                this.$el.append('<fieldset class="script-options"></fieldset>');
                var $scriptOptions = this.$el.find('.script-options');
                this.children.runScript.render().appendTo($scriptOptions);
                this.children.scriptName.render().appendTo($scriptOptions);

                this.children.severity.$el.hide();
                this.children.emailOptions.$el.hide();
                this.children.emailInclude.$el.hide();
                this.children.scriptName.$el.hide();

                this.toggleAll();
                return this;
            }
        });
});
