define([
            'underscore',
            'module',
            'views/shared/controls/Control',
            'splunk.util',
            'select2/select2'
        ],
        function(
            _,
            module,
            Control,
            splunkUtils
            /* remaining modules do not export */
        ) {

    var DELIMITER = '::::';

    return Control.extend({

        moduleId: module.id,

        events: {

            'change input': function(e) {
                var values = e.val || [];
                this.setValue(splunkUtils.fieldListToString(values), false);
            }

        },

        /**
         * @constructor
         * @param {Object} options {
         *     {String, required} modelAttribute The attribute on the model to observe and update on selection
         *     {Model, required} model The model to operate on
         *     {String, optional} placeholder The placeholder text for an empty input
         *     {Array<String>, optional} autoCompleteFields A list of fields to use for auto-complete
         *     {String, optional} inputClassName A class name to apply to the input element
         * }
         */

        render: function() {
            if(this.el.innerHTML) {
                return;
            }
            this.$el.html(this.compiledTemplate({ options: this.options }));
            var $input = this.$('input');
            $input.select2({
                placeholder: this.options.placeholder,
                tags: this.options.autoCompleteFields || [],
                formatNoMatches: function() { return '&nbsp;'; },
                dropdownCssClass: 'empty-results-allowed',
                separator: DELIMITER,
                // SPL-77050, this needs to be false for use inside popdowns/modals
                openOnEnter: false
            })
            .select2('val', splunkUtils.stringToFieldList(this._value || ''));
            return this;
        },

        remove: function() {
            this.$('input').select2('close').select2('destroy');
            return Control.prototype.remove.apply(this, arguments);
        },

        template: '\
            <input class="<%= options.inputClassName %>" />\
        '

    });

});