define(
    [
        'jquery',
        'underscore',
        'backbone',
        'module',
        'views/Base',
        'views/shared/eventsviewer/list/body/row/SelectedFields',
        'views/shared/eventsviewer/shared/EventFields',
        'views/shared/eventsviewer/shared/RawField',
        'splunk.util',
        'util/console',
        'util/keyboard'
    ],
    function(
        $,
        _,
        Backbone,
        module,
        BaseView,
        SelectedFieldsView,
        EventFieldsView,
        RawField,
        splunkUtil,
        console,
        keyboard
    ){
        return BaseView.extend({
            moduleId: module.id,
            tagName: 'tr',
            className: 'tabbable-list-row',
            attributes: {
                tabindex: -1
            },
            /**
             * @param {Object} options {
             *      model: {
             *         event: <models.services.search.job.ResultsV2.results[i]>,
             *         result: <models.services.search.job.ResultsV2>,
             *         state: <models.Base>,
             *         summary: <models.services.searchjob.SummaryV2>,
             *         report: <models.services.SavedSearch>,
             *         searchJob: <models.Job>,
             *         application: <models.Application>
             *     },
             *     collection: {
             *         selectedFields: <collections.SelectedFields>,
             *         eventRenderers: <collections.services.configs.EventRenderers>,
             *         workflowActions: <collections.services.data.ui.WorkflowActions>
             *     },
             *     selectableFields: true|false,
             *     idx: <integer>,
             *     lineNum: <integer>,
             *     allowRowExpand: true|false,
             *     showWarnings: true|false,
             *     highlightExtractedTime: true|false (caution: will disable segmentation/drilldown)
             */
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);
                
                this.interaction = 'i' + this.options.idx;
                this.rowExpanded = 'r' + this.options.idx;
                this.showAllLines = 's' + this.options.idx;
                
                this.model.state.unset(this.showAllLines);

                this.model.renderer = this.collection.eventRenderers.getRenderer(this.model.event.get('eventtype'));

                this.children.raw = new RawField({
                    model: {
                        event: this.model.event,
                        state: this.model.state,
                        result: this.model.result,
                        report: this.model.report,
                        searchJob: this.model.searchJob,
                        application: this.model.application
                    },
                    collection: {
                        eventRenderers: this.collection.eventRenderers
                    },
                    idx: this.options.idx,
                    highlightExtractedTime: this.options.highlightExtractedTime,
                    clickFocus: this.options.clickFocus
                });

                this.children.selectedFields = new SelectedFieldsView({
                    model: {
                        event: this.model.event,
                        state: this.model.state,
                        result: this.model.result,
                        summary: this.model.summary,
                        application: this.model.application,
                        report: this.model.report,
                        searchJob: this.model.searchJob
                    },
                    collection: {
                        selectedFields: this.collection.selectedFields,
                        workflowActions: this.collection.workflowActions
                    },
                    rowExpanded: this.rowExpanded,
                    selectableFields: this.options.selectableFields,
                    idx: this.options.idx,
                    clickFocus: this.options.clickFocus
                });
            },
            startListening: function() {
                this.listenTo(this.model.state, this.interaction, function() {
                    this.model.state.set('modalizedRow', this.options.idx);
                });
                
                /*
                 * Our bus for communication from our grandparent regarding clicks on the
                 * modalization mask.
                 */
                this.listenTo(this.model.state, 'change:' + this.rowExpanded, function(model, value, options){
                    if(!value) {
                        this.collapseState();
                    }
                });
                
                this.listenTo(this.model.state, 'change:' + this.showAllLines, function(model, value, options){
                    this.eventFetch();
                    this.model.state.set('modalizedRow', this.options.idx);
                });
                
                //on change of the search string we should unmodalize
                this.listenTo(this.model.state, 'intentions-fetch', this.collapseState);
                
                this.listenTo(this.model.report.entry.content, 'change:display.events.type', this.manageListRawState);
            },
            manageListRawState: function() {
                if (this.isList()) {
                    if (this.model.event.get('_icon')) {
                        this.$('td.col-icon').show();
                    }
                    this.$('td._time').show();
                    if (!this.children.selectedFields.active) {
                        this.children.selectedFields.activate().$el.show(); 
                    } else {
                        this.children.selectedFields.$el.show(); 
                    }
                } else {
                    this.children.selectedFields.deactivate().$el.hide(); 
                    this.$('td._time').hide();
                    this.$('td.col-icon').hide();
                }
            },
            events: {
                'click td.expands': function(e) {
                    this.expand();
                    e.preventDefault();
                },
                'click td._time': function(e) {
                    this.drilldown($(e.currentTarget), e);
                    e.preventDefault();
                },
                'keydown td._time': function(e) {
                    if (e.which === keyboard.KEYS.ENTER) {
                        this.drilldown($(e.currentTarget), e);
                        /**
                         * SPL-87936
                         * ---------
                         * On keydown, the _time accelerator PopTart opens,
                         * and simultaneously, focus is given to the close
                         * button, which is an anchor tag. Then, on keyup,
                         * FF/IE interperets it as a click event on the
                         * currently active element (now the close button),
                         * and therefore immediately closes the PopTart.
                         * This e.preventDefault() ensures that FF/IE does
                         * not interperet a click event on the PopTart's
                         * close button as soon as it's opened.
                         */
                        e.preventDefault();
                    }
                },
                'keydown': function(e) {
                    var keyCode = e.which;

                    if (keyCode === keyboard.KEYS.TAB && this.model.state.get('modalizedRow') === this.options.idx) {
                        keyboard.handleCircularTabbing(this.$el, e);
                    }
                }
            },
            drilldown: function($target, e) {
                var $anchor = $target.find('span.formated-time'),
                    data = $anchor.data(),
                    timeIso, epoch;
                
                if (data.timeIso) {
                    timeIso = data.timeIso;
                    epoch = splunkUtil.getEpochTimeFromISO(timeIso);
                    this.model.state.trigger('drilldown', {
                        noFetch: true, 
                        data: {
                            'dispatch.earliest_time': epoch,
                            'dispatch.latest_time': '' + (parseFloat(epoch) + 1)
                        },
                        event: e,
                        _time: timeIso,
                        idx: this.options.idx,
                        type: "time",
                        $target: $target,
                        $anchor: $anchor,
                        stateModel: this.model.state
                    });                    
                }                
            },
            eventFieldsFactory: function() {
                this.children.eventFields = new EventFieldsView({
                    model: {
                        event: this.model.event,
                        state: this.model.state,
                        result: this.model.result,
                        summary: this.model.summary,
                        application: this.model.application,
                        report: this.model.report,
                        searchJob: this.model.searchJob
                    },
                    collection: {
                        selectedFields: this.collection.selectedFields,
                        workflowActions: this.collection.workflowActions
                    },
                    selectableFields: this.options.selectableFields,
                    allowRowExpand: this.options.allowRowExpand,
                    idx: this.options.idx,
                    swappingKey: this.swappingKey,
                    showAllLines: this.showAllLines,
                    clickFocus: this.options.clickFocus
                });
                //remove and put into another function
                this.children.eventFields.render().insertAfter(this.$('.event').find(this.children.raw.el));
            },
            expand: function(options) {
                if (!this.options.allowRowExpand) {
                    console.warn('Cannot expand row with allowRowExpand=' + this.options.allowRowExpand);
                    return;
                }
                if (!this.children.eventFields) {
                    this.eventFieldsFactory();
                }
                (this.isExpanded()) ? this.collapseState(): this.expandState();
            },
            eventFetch: function(showAll) {
                this.model.event.set(this.model.event.idAttribute, this.model.searchJob.entry.links.get('events'));
                this.model.event.fetch({
                    data: $.extend(true, this.model.application.toJSON(), {
                        isRt: this.model.searchJob.isRealtime(),
                        search: this.model.report.getSortingSearch(),
                        earliest_time: this.model.report.entry.content.get('display.events.timelineEarliestTime'),
                        latest_time: this.model.report.entry.content.get('display.events.timelineLatestTime'),
                        segmentation:  this.model.report.entry.content.get('display.events.list.drilldown'),
                        max_lines: this.model.state.get(this.showAllLines) ? 0: this.model.report.getNearestMaxlines(),
                        offset: this.model.result.offset(this.options.idx) || 0,
                        oid: this.model.application.get('oid')
                    })     
                });
            },
            remove: function() {
                // destroy element's tooltip if any
                if (this.tooltip && this.tooltip.length > 0) { this.tooltip.tooltip('destroy'); }
                return BaseView.prototype.remove.apply(this, arguments);
            },
            isExpanded: function() {
                return this.model.state.get(this.rowExpanded);
            },
            expandState: function() {
                // Ensure no rows are expanded
                var currentlyExpandedRow = this.model.state.get('currentlyExpandedRow');
                if (currentlyExpandedRow) {
                    currentlyExpandedRow.collapseState();
                }
                this.model.state.set('currentlyExpandedRow', this);

                this.eventFetch();
                this.model.state.set(this.rowExpanded, true);
                this.model.state.set('modalizedRow', this.options.idx);
                this.children.eventFields && this.children.eventFields.activate().$el.show();
                this.isList() && this.children.selectedFields.deactivate().$el.hide();
                this.toggleArrow(true);
            },
            collapseState: function() {
                // Notify that no rows are expanded
                this.model.state.set('currentlyExpandedRow', null);

                this.model.state.set(this.rowExpanded, false);
                this.model.state.set('modalizedRow', false);
                this.children.eventFields && this.children.eventFields.deactivate().$el.hide();
                this.isList() && this.children.selectedFields.activate().$el.show();
                this.toggleArrow(false);
            },
            isList: function() {
                return (this.model.report.entry.content.get('display.events.type') === 'list');
            },
            toggleArrow: function(open) {
                var $arrow =  this.$('td.expands > a > i').removeClass('icon-triangle-right-small icon-triangle-down-small');
                $arrow.addClass((open) ? 'icon-triangle-down-small':  'icon-triangle-right-small');
            },
            render: function() {
                var isList = this.isList();
                                
                this.$el.html(this.compiledTemplate({
                    $: $,
                    event: this.model.event,
                    iconPath: this.model.event.getIconPath(this.model.application.get('root'), this.model.application.get('locale')),
                    lineNum: this.options.lineNum,
                    application: this.model.application, //ghetto and inconsistent 
                    expanded: this.isExpanded(),
                    isList: isList,
                    formattedTime: this.model.event.formattedTime(),
                    colorClass: (this.model.renderer) ? this.model.renderer.entry.content.get('css_class'): '',
                    allowRowExpand: this.options.allowRowExpand, //misuse by data model
                    showWarnings: this.options.showWarnings,
                    warningTexts: this.model.event.getWarningTexts()
                }));

                // add warnings tooltip only if enabled and there are warnings associated with the event
                if (this.options.showWarnings && !_.isEmpty(this.model.event.getWarningTexts())) {
                    this.tooltip = this.$el.find('.tooltip-right').tooltip({
                        animation: false,
                        html: true,
                        container: 'body',
                        placement: 'right'
                    });
                }

                this.children.raw.render().appendTo(this.$('.event'));
                this.children.selectedFields.render().appendTo(this.$('.event'));

                if (!isList) {
                    this.children.selectedFields.deactivate().$el.hide();
                }

                return this;
            },
            template: '\
                <% if (allowRowExpand) { %>\
                <td class="expands <%- colorClass %>">\
                    <a href="#"><i class="icon-triangle-<%- expanded ? "down" : "right" %>-small"></i></a>\
                </td>\
                <% } %>\
                <td class="line-num"><span><%- lineNum %></span></td>\
                <% if (showWarnings) { %>\
                    <td class="col-warnings" tabindex="0">\
                    <% if (warningTexts && warningTexts.length > 0) {\
                        var warningTextsHTML = _(warningTexts).map(function(warningText) { return "<li>" + _.escape(_(warningText).t()) + "</li>"; }).join("");\
                        warningTextsHTML = "<ul class=\'tooltip-warnings events-viewer-tooltip\'>" + warningTextsHTML + "</ul>"; %>\
                        <i class="icon-alert tooltip-right" data-title="<%= warningTextsHTML%>"></i>\
                        <span><%= warningTextsHTML%></span>\
                    <% } %>\
                    </td>\
                <% } %>\
                <td class="_time" <% if (!isList) { %> style="display:none" <%}%> tabindex="0">\
                    <span class="formated-time" data-time-iso="<%- event.get("_time") %>">\
                    <% if(application.get("locale").indexOf("en") > -1){ %>\
                         <span><%- $.trim(formattedTime.slice(0, formattedTime.indexOf(" "))) %></span>\
                         <br>\
                         <span><%- $.trim(formattedTime.slice(formattedTime.indexOf(" "))) %></span>\
                    <% } else { %>\
                         <%- formattedTime %>\
                    <% } %>\
                    </span>\
                </td>\
                <td class="col-icon" <% if (!isList || !iconPath) { %> style="display:none" <%}%> tabindex="0">\
                   <em class="icon"><img src="<%= iconPath %>"/></em>\
                </td>\
                <td class="event"></td>\
            '
        });
    }
);
