define([
	'underscore',
    'jquery',
    'module',
    'splunk.util',
    'models/config',
    'collections/shared/FlashMessages',
    'views/Base',
    'views/shared/FlashMessagesLegacy',
    'contrib/text!views/shared/interstitialdialog/AddFile.html',
    'jquery.ui.effect-slide'
],
    function(
    	_,
        $,
        module,
        splunkUtil,
        ConfigModel,
        FlashMessagesCollection,
        BaseView,
        FlashMessagesView,
        Template,
        Slide
        ){        
    		var SLIDESHOW_INTERVAL_MS = 3500;

		    var cancelEvent = function(e) {
		        e.preventDefault();
		        e.stopPropagation();
		    };
		    var canDragFiles = function() {
		    	return 'draggable' in document.createElement('span');
		    };
		    var bytesToMb = function(bytes) {
		    	return Math.floor(bytes/1024/1024);
		    };
		            
	        return BaseView.extend({
            moduleId: module.id,
            template: Template,
            initialize: function(options) {
                BaseView.prototype.initialize.call(this, options);
                this.wizardModel = this.options.model.wizard;
                this.userPrefModel = this.options.model.userPref;
                this.collection = {
                	flashMessages: new FlashMessagesCollection()
                };
                
                this.children.flashMessages = new FlashMessagesView({
                	collection: this.collection.flashMessages
                });
               
            },

            events: {
	            'drop .drop-container': 'handleFileAdded',
	            'change .drop-container input': 'handleFileAdded',
	            // Stops browser from loading the file we're trying to drag
	            'dragenter .drop-container': cancelEvent,
	            'dragover .drop-container': cancelEvent,
	            'dragleave .drop-container': cancelEvent,

	            'click .arrow-container': function(e) {
	            	if (this._intervalId) {
	            		clearInterval(this._intervalId);
	            		this._intervalId = null;
	            	}
	            	this._progressSlideshow(!!$(e.currentTarget).hasClass('right-arrow'));

	            }
            },		
            
	        handleFileAdded: function(e) {
	            cancelEvent(e);
	            var file = null;
	            var files = null;
	
	            if (e.originalEvent && e.originalEvent.dataTransfer) { // means it got dragged
	                files = e.originalEvent.dataTransfer.files;

	                if (files.length > 1) {
	                	this.collection.flashMessages.reset([{
	                		key: 'waiting', // TODO don't know
	                		type: 'error',
	                		html: _('Please select only one file.').t()
	                	}]);
	                	return;
	                }
	            } else if (e.target.files) {
	                files = e.target.files;
	            } else { // IE9
	            	var pathBreakdown = $(e.target).val().split('\\');
	            	files = [{
	            		name: pathBreakdown[pathBreakdown.length - 1]
	            	}];
	            }

	            file = files[0];

	            if (file && !_.isUndefined(file.size)) {

	            	if (file.size > ConfigModel.get('MAX_UPLOAD_SIZE')) {
		            	this.collection.flashMessages.reset([{
		            		key: 'waiting',
		            		type: 'error',
		            		html: _('Your file size exceeds the maximum allowable (' + bytesToMb(ConfigModel.get('MAX_UPLOAD_SIZE')) + 'MB).').t()
		            	}]);
		            	return;
		            } else if (file.size === 0) {
		            	this.collection.flashMessages.reset([{
		            		key: 'waiting',
		            		type: 'error',
		            		html: _('Please add a file with size is greater than zero bytes.').t()
		            	}]);
		            	return;
		            }
	            }


	            this.wizardModel.set('file', file);
	            this.wizardModel.set('currentScreen', this.wizardModel.get('screens').INDEX_SETTINGS);
	        },
	        
            render: function() {
            	var maxUploadSizeMB = bytesToMb(ConfigModel.get('MAX_UPLOAD_SIZE'));        	
                var html = this.compiledTemplate({
                	maxUploadSizeMB: maxUploadSizeMB,
                	examplesImgBaseUrl: splunkUtil.make_url('/static/img/skins/default/interstitialdialog')
                });
                this.$el.html(html);
				
				this.$('.error-message').append(this.children.flashMessages.render().el);

                if (!canDragFiles()) {
                	this.$('.drop-instruction').remove();
                	// switch image on .drop-container
                	var img = 'url(' + splunkUtil.make_url('/static/skins/default/interstitialdialog/drop-zone-ie9.png') + ')';
                	this.$('.drop-container').css('background-image', img);
                	this.$('.drop-container').on('drop', cancelEvent);
                }
				
				this.$('.examples-container .selected').show();
				
                if (this._intervalId) {
                	clearInterval(this._intervalId);
                }

                this._intervalId = setInterval(function() {
                	this._progressSlideshow(true);
                }.bind(this), SLIDESHOW_INTERVAL_MS);
            },

            resetModel: function() {
            	this.collection.flashMessages.reset();
            	this.wizardModel.set({'file': ''}, {silent: true});
            },

            _progressSlideshow: function(goRight) {
            	var $selected = this.$('.examples-container .selected');
            	var $showEl = $selected[goRight ? 'next' : 'prev']();

            	// Wrap around if there is no next sibling
            	if (!$showEl.length) {
            		$showEl = this.$('.examples div')[goRight ? 'first' : 'last']();
            	}

            	$showEl.addClass('selected');
            	$selected.removeClass('selected');
            	
            	$selected.hide("slide", { direction: goRight ? "left" : "right" }, 500, function() {
            		$showEl.show("slide", { direction: goRight ? "right" : "left" }, 500);
            	});
            },

            _intervalId: null
        });

    });
