define([
    'jquery',
    'underscore',
    'backbone',
    'module',
    'collections/shared/FlashMessages',
    'views/Base',
    'views/shared/controls/ControlGroup',
    'views/shared/FlashMessagesLegacy',
    'contrib/text!views/shared/interstitialdialog/IndexSettings.html',
    'jquery.fileupload',
    'jquery.iframe-transport',
    'splunk.util'
],
    function(
        $,
        _,
        Backbone,
        module,
        FlashMessagesCollection,
        BaseView,
        ControlGroup,
        FlashMessagesView,
        Template,
        FileUpload,
        IframeTransport,
        splunkUtil
        ) {
        return BaseView.extend({
            moduleId: module.id,
            template: Template,
            initialize: function(options) {
                BaseView.prototype.initialize.call(this, options);
                this.wizardModel = this.options.model.wizard;

                this.collection.sourcetypeFlashMessages = new FlashMessagesCollection();
                this.collection.hostFlashMessages = new FlashMessagesCollection();

                this.children.sourcetypeFlashMessages = new FlashMessagesView({
                    collection: this.collection.sourcetypeFlashMessages
                });
                this.children.hostFlashMessages = new FlashMessagesView({
                    collection: this.collection.hostFlashMessages
                });

 				this.wizardModel.on('change:file', function(model) {
                    this._setFile(model.get('file'));
                	// Reset index to "main"
                	this.wizardModel.set('ui.index', 'main');                    
                }, this);
                this.wizardModel.on('change:ui.sourcetype', function(model) {
                    this._setManualSourceType(model.get('ui.sourcetype'));
                }, this);
				              
                // Sourcetype
                this.sourcetypeNames = [
                    {
                        label: 'Automatic',
                        value: ''
                    }/*, // Removing manual option per PM's request
                    {
                        label: 'Manual',
                        value: 'manual'
                    }*/
                ];
                this.collection.sourcetypes.each(function(model) {
                    this.sourcetypeNames.push({
                        label: model.entry.get('name'), 
                        value: model.entry.get('name')
                    });
                }, this);
                
                this.children.sourcetype = new ControlGroup({
                    className: 'control-group sourcetype-field',
                    controlType: 'SyntheticSelect',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'ui.sourcetype',
                        model: this.wizardModel,
                        items: this.sourcetypeNames,
                        className: 'btn-group view-count',
                        menuWidth: 'medium',
                        toggleClassName: 'btn'
                    },
                    required: true,
                    label: _("Source Type").t(),
                    help: _('The source type field describes the format of your data. Choose how to set the source type value.').t(),
                    helpClass: 'hint'
                }); 

                this.children.sourcetypeManual = new ControlGroup({
                    className: 'control-group sourcetype-manual-field',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'ui.sourcetype.manual',
                        model: this.wizardModel,
                        save: false
                    },
                    required: true,
                    label: _("Source Type (manual)").t(),
                    help: _("Enter a source type value for your data.").t(),
                    helpClass: 'hint'
                });

                // Host
                this.children.host = new ControlGroup({
                    className: 'control-group host-field',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'ui.host',
                        model: this.wizardModel,
                        save: false
                    },
                    required: true,
                    label: _('Host').t(),
                    help: _('The host field describes the machine that generates your data. Enter a host value.').t(),
                    helpClass: 'hint'                    
                });
                
                // Index
                this.indexNames = [];
                this.collection.indexes.map(function(model) {
                   this.indexNames.push({label:model.entry.get('name'), value:model.entry.get('name')});
                }, this);
                
                this.children.index = new ControlGroup({
                    className: 'control-group index-field',
                    controlType: 'SyntheticSelect',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'ui.index',
                        model: this.wizardModel,
                        items: this.indexNames,
                        className: 'btn-group view-count',
                        menuWidth: 'medium',
                        toggleClassName: 'btn'
                    },  
                    required: true,
                    label: _("Index").t(),
                    help: _('Choose where to store your data. The default index is "main".').t(),
                    helpClass: 'hint'
                });
                this.wizardModel.set('ui.index', 'main');
            },

            render: function() {
                this.$el.html(this.compiledTemplate({}));
                this.$('#interstitialdialog-index-settings-page').append(this.children.sourcetype.render().el);
                 this.$('#interstitialdialog-index-settings-page').append(this.children.sourcetypeManual.render().$el.hide());
                this.$('#interstitialdialog-index-settings-page').append(this.children.host.render().el);
				this.$('#interstitialdialog-index-settings-page').append(this.children.index.render().el);
                this.$('.sourcetype-error-message').append(this.children.sourcetypeFlashMessages.render().el);
                this.$('.host-error-message').append(this.children.hostFlashMessages.render().el);
                this._setFile(this.wizardModel.get('file'));
                this._setManualSourceType(this.wizardModel.get('ui.sourcetype'));
            },

            _setFile: function(file) {               
                var fileUnits = "MB";
                var fileSize = Math.floor((file.size || 0)/1024/1024);
                if (fileSize === 0) {
                    fileSize = Math.floor((file.size || 0)/1024);
                    fileUnits = "KB";
                }
                if (fileSize === 0) {
                    fileSize = file.size;
                    fileUnits = "bytes";
                }


                this.$('.filename').text(' ' + (file.name || ''));
                this.$('.file-size').text(file.size ? fileSize + ' ' + fileUnits : '');
            },

            _setManualSourceType: function(sourcetype) {
                if (sourcetype === 'manual') {
                    this.children.sourcetypeManual.$el.show();
                } else {
                    this.children.sourcetypeManual.$el.hide();
                }
            },

            resetModel: function() {
                this.collection.sourcetypeFlashMessages.reset();
                this.collection.hostFlashMessages.reset();
                this.wizardModel.set({
                    'uploading.percentage': '',
                    'uploading.status': ''
                }, {silent: true});
            },

            getBackButton: function() {
                return {
                    text: "Go back",
                    click: function(e) {
                        e.preventDefault();
                        e.stopPropagation();
                        this.wizardModel.set('currentScreen', this.wizardModel.get('screens').ADD_FILE);
                    }.bind(this)
                };
            },

            getNextButton: function() {
                return {
                    text: "Add data",
                    click: function(e) {
                        e.preventDefault();
                        e.stopPropagation();

                        var that = this;

                        // TODO the regexes here are probably not correct
                        var validateSourceType = function() {
                            if (this.wizardModel.get('ui.sourcetype') === 'manual') {
                                if (!$.trim(this.wizardModel.get('ui.sourcetype.manual'))) {
                                    return _("Source type is required and must contain at least one character.").t();
                                }
                            }
                        }.bind(this);
                        var validateHost = function() {
                            if (!$.trim(this.wizardModel.get('ui.host'))) {
                                return _("Host is required and must contain at least one character.").t();
                            }
                        }.bind(this);

                        var sourcetypeErrorMessage = validateSourceType();
                        var hostErrorMessage = validateHost();
                        this.collection.sourcetypeFlashMessages.reset();
                        this.collection.hostFlashMessages.reset();

                        if (sourcetypeErrorMessage) {
                            this.collection.sourcetypeFlashMessages.reset([{
                                key: 'waiting',
                                type: 'error',
                                html: sourcetypeErrorMessage
                            }]);
                        }
                        if (hostErrorMessage) {
                            this.collection.hostFlashMessages.reset([{
                                key: 'waiting',
                                type: 'error',
                                html: hostErrorMessage
                            }]);
                        }
                        if (hostErrorMessage || sourcetypeErrorMessage) {
                            return;
                        }

                        this.wizardModel.set('uploading.status', 'uploading');
                        this.wizardModel.set('uploading.percentage', 0);
                        						
		                // Initialize file upload plugin
		                $('#spl-ctrl_remotefile').fileupload({
		                	url: this.wizardModel.get('endpoint'),
		                	dataType: 'json',
		                	formData: [
		                		{
		                			name: 'splunk_form_key',
		                			value: splunkUtil.getFormKey()
		                		},
		                		{
		                			name: 'host',
		                			value: that.wizardModel.get('ui.host')
		                		},
		                		{
		                			name: 'sourcetype',
		                			value: that.wizardModel.get('ui.sourcetype') == 'manual' ? this.wizardModel.get('ui.sourcetype.manual') : this.wizardModel.get('ui.sourcetype')
		                		},
		                 		{
		                			name: 'index',
		                			value: that.wizardModel.get('ui.index')
		                		}                       		                        		                        		                        		
		                	],        	
		                    done: function (e, data) {
		                        if (data.jqXHR.status === 200) {
		                            var responseJson = data.jqXHR.responseJSON;
		                            if (responseJson.status === "OK") {
		                                that.wizardModel.set('uploading.status', 'success');
		                                that.wizardModel.set('uploading.percentage', 100);
		                                return;
		                            }
		                        }
		                        that.wizardModel.set('uploading.status', 'error');
		                    },   	
						    progressall: function (e, data) {
						    	//NOTE: No 100% progress is received in Firefox due to the Firefox bug.  Therefore,
						    	// we'll never switch to a striped upload indicator in Uploading.js.
						        var progress = parseInt(data.loaded / data.total * 100, 10);
								that.wizardModel.set('uploading.percentage', progress);
						    },
						    always : function (e, data) {
		                        // Destroy file upload widget functionality.  This is needed so our own even listeners on file input field continue to work.
		                        $('#spl-ctrl_remotefile').fileupload('destroy');						    	
						    }                       	
		                });     
                
						// Upload using fileupload plugin
						var ieVersion = splunkUtil.getIEVersion();
						if (ieVersion == -1 || ieVersion >=10) { 
							$('#spl-ctrl_remotefile').fileupload('add', {
								files: [this.wizardModel.get('file')]
							});
						}
						else { // IE9 and below
							$('#spl-ctrl_remotefile').fileupload('add', {
								files: [this.wizardModel.get('file')],
							    fileInput: $('#spl-ctrl_remotefile'),
				            	formData: [
			                		{
			                			name: 'splunk_form_key',
			                			value: splunkUtil.getFormKey()
			                		},
			                		{
			                			name: 'host',
			                			value: that.wizardModel.get('ui.host')
			                		},
			                		{
			                			name: 'sourcetype',
			                			value: that.wizardModel.get('ui.sourcetype') == 'manual' ? this.wizardModel.get('ui.sourcetype.manual') : this.wizardModel.get('ui.sourcetype')
			                		},
			                 		{
			                			name: 'index',
			                			value: that.wizardModel.get('ui.index')
			                		}                       		                        		                        		                        		
			                	]        						    						    
							});						
						}
                        this.wizardModel.set('currentScreen', this.wizardModel.get('screens').UPLOADING);
                    }.bind(this) 
                };
            }
        });

    });
