define(
[
    'jquery',
    'underscore',
    'backbone',
    'module',
    'models/services/data/UserPref',
    'views/Base',
    'views/shared/Modal',
    'uri/route',
    'splunk.util',
    'collections/services/data/Indexes',
    'collections/services/saved/Sourcetypes',
    'views/shared/interstitialdialog/AddFile',
    'views/shared/interstitialdialog/IndexSettings',
    'views/shared/interstitialdialog/Uploading',
    'views/shared/controls/SyntheticCheckboxControl',
    'contrib/text!views/shared/interstitialdialog/Master.html'
],
function(
    $,
    _,
    Backbone,
    module,
    UserPrefModel,
    Base,
    Modal,
    route,
    splunk_util,
    IndexesConfCollection,
    SourcetypesCollection,
    AddFileView,
    IndexSettingsView,
    UploadingView,
    SyntheticCheckboxControl,
    Template
){
    var cancelEvent = function(e) {
        e.preventDefault();
        e.stopPropagation();
    };

    var ADDDATA_ENDPOINT = splunk_util.make_url('/manager/system/adddata_oneshot');
	var SCREENS = {
		ADD_FILE: 1,
		INDEX_SETTINGS: 2,
		UPLOADING: 3
   	};
    var FOOTER_TYPES = {
        BUTTONS: 1,
        DONT_SHOW_AGAIN: 2,
        NONE: 3
    };
            	
    return Modal.extend({
        moduleId: module.id,
        template: Template,

        initialize: function() {
            Modal.prototype.initialize.apply(this, arguments);
            var self = this;
            this.on('hidden', function(){
                self.$el.remove();
                self.modalWrapper.remove();
            });

            this.resetModel();
            
            this.owner = this.options.owner;
            this.userPrefModel = new UserPrefModel();
            
            // Collections
			this.indexesConfCollection = new IndexesConfCollection();
			this.sourcetypesCollection = new SourcetypesCollection();
				
            // Render and go to the Add File view
            this.render();
            this.wizardModel.on('change:modal.title', this._updateTitle, this);
            this.wizardModel.on('change:modal.footer', this._updateFooter, this);
            this.wizardModel.set('currentScreen', this.wizardModel.get('screens').ADD_FILE);
        },

        notShowAgainChangeHandler: function(value) {
            if(this.userPrefModel && this.userPrefModel.entry && this.userPrefModel.entry.content){
                this.userPrefModel.entry.content.set('hideInterstitialDialog', value);
                this.userPrefModel.save();
            }
        },

		changeScreen: function(screen) {
			if (screen === SCREENS.ADD_FILE) {
				this.showAddFile();
			} else if (screen === SCREENS.INDEX_SETTINGS) {
				this.showIndexSettings();
            } else if (screen === SCREENS.UPLOADING) {
                this.showUploading();
            }   
		},
		
		showAddFile: function() {
			this.hideAllViews();
			
			if (!this.children.addFileView) {
				var userPrefFetch = this.userPrefModel.fetch({data: {app:'user-prefs', owner: this.owner, count:-1}});  
                $.when(userPrefFetch).done(_(function() {
					this.children.addFileView = new AddFileView({
						model: {
							wizard: this.wizardModel
						}
					});

                    this.children.notShowAgain = new SyntheticCheckboxControl({
                        modelAttribute: 'ui.not_show_again',
                        model: this.wizardModel,
                        label: _("Do not show this page again on startup").t()
                    });
                    if (this.userPrefModel && this.userPrefModel.entry && 
                    	this.userPrefModel.entry.content && 
                    	this.userPrefModel.entry.content.get('hideInterstitialDialog'))
                        	this.children.notShowAgain.setValue(this.userPrefModel.entry.content.get('hideInterstitialDialog'));
                    this.children.notShowAgain.on("change", this.notShowAgainChangeHandler, this);
					$('#master-form').append(this.children.addFileView.render().el);

					this.children.addFileView.$el.show();
                    this.wizardModel.set("modal.footer", { type: FOOTER_TYPES.DONT_SHOW_AGAIN });

                }).bind(this));
			}
			else {
                // Whenever we navigate back to this view, reset state
                this.reset();
				this.children.addFileView.$el.show();
                this.wizardModel.set("modal.footer", { type: FOOTER_TYPES.DONT_SHOW_AGAIN });

			}

            this.wizardModel.set("modal.title", _("Welcome to the Splunk Online Sandbox!").t());
		},

		showIndexSettings: function() {
			this.hideAllViews();
			if (!this.children.indexSettingsView) {
				// Fetch a list of sourcetypes and indexes
				var indexesFetch = this.indexesConfCollection.fetch({
                    data:{
                        f: 'name',
                        search: 'isInternal=0'
                    }
                });
				var sourcetypesFetch = this.sourcetypesCollection.fetch({
                    data:{
                        f: 'name',
                        search: 'pulldown_type=1',
                        count: 0
                    }
                });
                               
                // TODO: do we need to show "loading..." spinner here for slow connections
                $.when(indexesFetch, sourcetypesFetch).done(_(function() {
					this.children.indexSettingsView = new IndexSettingsView({
						model: {
							wizard: this.wizardModel
						},
						collection: {
							indexes: this.indexesConfCollection,
							sourcetypes: this.sourcetypesCollection
						}
					});
					$('#master-form').append(this.children.indexSettingsView.render().el);                	
                	this.children.indexSettingsView.$el.show();
                    this.wizardModel.set("modal.footer", {
                        type: FOOTER_TYPES.BUTTONS,
                        backButton: this.children.indexSettingsView.getBackButton(),
                        nextButton: this.children.indexSettingsView.getNextButton()
                    });
                }).bind(this));			
			}
			else {
				this.children.indexSettingsView.$el.show();
                this.wizardModel.set("modal.footer", {
                    type: FOOTER_TYPES.BUTTONS,
                    backButton: this.children.indexSettingsView.getBackButton(),
                    nextButton: this.children.indexSettingsView.getNextButton()
                });
			}

            this.wizardModel.set("modal.title", _("Settings").t());
           
		},

        showUploading: function() {
            this.hideAllViews();

            if (!this.children.uploadingView) {
                this.children.uploadingView = new UploadingView({ wizardModel: this.wizardModel});
                $('#master-form').append(this.children.uploadingView.render().el);
            }

            this.wizardModel.set('modal.title', _("Please wait...").t());
            this.wizardModel.set("modal.footer", { type: FOOTER_TYPES.NONE });
            this.children.uploadingView.$el.show();
        },

        		
		hideAllViews: function() {
			if (this.children.addFileView) {
				this.children.addFileView.$el.hide();
            }
				
			if (this.children.indexSettingsView) {
				this.children.indexSettingsView.$el.hide();
            }

            if (this.children.uploadingView) {
                this.children.uploadingView.$el.hide();
            }
            
		},
				
        render: function() {
            var template = this.compiledTemplate({});
            this.$el.html(Modal.TEMPLATE);
            this.$(Modal.BODY_SELECTOR).html(template);
            this.modalWrapper = $('<div class="splunk-components"></div>');
            $(this.el).appendTo(this.modalWrapper);
            this.modalWrapper.appendTo('body');
            this.show();
            return this;
        },

        reset: function() {
            this.resetModel();
            this.$('form')[0].reset();
        },

        resetModel: function() {
            if (_.isUndefined(this.wizardModel)) {
                this.wizardModel = new Backbone.Model();

                    
                this.wizardModel.on('change:currentScreen', function (model) {
                   var screen = model.get('currentScreen');
                   var that = this;

                   that.changeScreen(screen);
                }, this);   
            }

            this.wizardModel.set({
                screens: SCREENS,
                footerTypes: FOOTER_TYPES,
                endpoint: ADDDATA_ENDPOINT,
                'ui.index': 'default',
                'ui.sourcetype': '',
                'ui.sourcetype.manual': '',
                'ui.host': 'splunk_cloud_trial',
                'modal.title': "",
                'modal.footer': {}
            });

            _.each(['addFileView', 'indexSettingsView', 'uploadingView'], function(viewName) {
                var view = this.children[viewName];
                if (!_.isUndefined(view) && !_.isUndefined(view.resetModel)) {
                    view.resetModel();
                }
            }, this);
        },

        _updateTitle: function() {
            this.$(Modal.HEADER_TITLE_SELECTOR).text(
                _(this.wizardModel.get('modal.title')).t()
            );
        },

        _updateFooter: function() {
            var footerInfo = this.wizardModel.get('modal.footer');
            var $footer = this.$(Modal.FOOTER_SELECTOR);

            $footer.empty();
            $footer.css('border-width','1px');
            switch (footerInfo.type) {
                case FOOTER_TYPES.BUTTONS:

                    if (footerInfo.backButton) {
                        var backButton = $(Modal.BUTTON_BACK).appendTo($footer);

                        if (footerInfo.backButton.click) {
                            backButton.click(footerInfo.backButton.click);
                        }
                        if (footerInfo.backButton.href) {
                            backButton.attr('href', footerInfo.backButton.href);
                        }
                        if (footerInfo.backButton.text) {
                            backButton.text(footerInfo.backButton.text);
                        }

                    }
                    if (footerInfo.nextButton) {
                        var nextButton = $(Modal.BUTTON_NEXT).appendTo($footer);

                        if (footerInfo.nextButton.click) {
                            nextButton.click(footerInfo.nextButton.click);
                        }
                        if (footerInfo.nextButton.href) {
                            nextButton.attr('href', footerInfo.nextButton.href);
                        }
                        if (footerInfo.nextButton.text) {
                            nextButton.text(footerInfo.nextButton.text);
                        }

                    }

                    break;
                case FOOTER_TYPES.DONT_SHOW_AGAIN:
                    $footer.append(
                        $('<a class="btn btn-primary see-sample-data"><h3>' + _("Take a tour first").t() + ' <span class="icon-chevron-right"></h3></span></a>').attr('href', splunk_util.make_url('/app/cloud-tour'))
                    );
                    $footer.append($('<div class="chkbox-not-show-again"></div>').append(this.children.notShowAgain.render().el));
                    break;
                case FOOTER_TYPES.NONE:
                    $footer.css('border-width','0');
                    break;
                default:
                    break;
            }
        }
    });
});
