define([
    'jquery',
    'underscore',
    'backbone',
    'module',
    'views/Base',
    'contrib/text!views/shared/interstitialdialog/Uploading.html',
    'splunk.util'
],
    function(
        $,
        _,
        Backbone,
        module,
        BaseView,
        Template,
        splunkUtil
        ) {

        var INDEXING_TEXT = _('Starting to index...').t();
        var UPLOADING_TEXT = _('Uploading...').t();

        return BaseView.extend({
            moduleId: module.id,
            template: Template,
            initialize: function(options) {
                BaseView.prototype.initialize.call(this, options);
                this.wizardModel = this.options.wizardModel;

                this.wizardModel.on('change:uploading.percentage', this._setPercentage, this);
                this.wizardModel.on('change:uploading.status', this._updateStatus, this);
            },


            render: function() {
                this.$el.html(this.compiledTemplate({
                    searchUrl: this._getSearchUrl(),
                    uploadingText: UPLOADING_TEXT,
                    dataInputsUrl: splunkUtil.make_url('/manager/launcher/datainputstats')
                }));
                this._setPercentage();
            },

            _setPercentage: function() {
                var ieVersion = splunkUtil.getIEVersion();
                if (ieVersion == -1 || ieVersion > 9) {                   	
	                var percentage = this.wizardModel.get('uploading.percentage') + '%';
	                var progressText = this.$('.progress-text').text();
	                var updatedText = percentage === '100%' ? INDEXING_TEXT : UPLOADING_TEXT;
	
	                this.$('.progress-bar').css('width', percentage);
													
	                if (percentage === '100%') {
	                    // Since we are using CSS animations, wait some time for the width to get there
	                    setTimeout(function() {
	                        if (this.wizardModel.get('uploading.status') === 'uploading') {
	                            this.$('.progress-text').text(INDEXING_TEXT);
	                            this.$('.progress').addClass('progress-striped active');
	                        }
	                    }.bind(this), 600);
	                } 
	                else {
	                    this.$('.progress').removeClass('progress-striped active');
	                    if (progressText !== UPLOADING_TEXT) {
	                        this.$('.progress-text').text(UPLOADING_TEXT);
	                    }
	                }
                }
                else {
                    // For IE9 the progress is not working with fileupload plugin.  So, just
                    // set the progress to the striped indicator
                    this.$('.progress-bar').css('width', '100%');
	                this.$('.progress-text').text(UPLOADING_TEXT);
	                this.$('.progress').addClass('progress-striped active');                                         	           	
                }
            },

            _updateStatus: function() {
                switch(this.wizardModel.get('uploading.status')) {
                    case 'success':
                    	this.$('.dontclose').hide();
                        this.$('.progress').hide();
                        this.$('.progress-text').hide();
                        this.$('.progress-text').text('Success!');
                        this.$('.progress').removeClass('progress-striped active');
                        this.$('.success').show();
                        this.wizardModel.set('modal.footer', {
                            type: this.wizardModel.get('footerTypes').BUTTONS,
                            nextButton: {
                                text: _("Start searching").t(),
                                href: this._getSearchUrl()
                            },
                            backButton: {
                                text: _("Add more data").t(),
                                click: function(e) {
                                     this.wizardModel.set('currentScreen', this.wizardModel.get('screens').ADD_FILE);
                                }.bind(this)
                            }
                        });
                        this.wizardModel.set('modal.title', _('What is next?').t());
                        break;
                    case 'uploading':
                        this.$('#interstitialdialog-uploading-page').children().hide();                       
                        this.$('.dontclose').show();                        
                        this.$('.progress').show();
                        this.$('.progress-text').show();
                        break;
                    // case 'error':
                    // case 'timeout':
                    default:
                        this.$('.progress').hide();
                        this.$('.progress-text').hide();
                        this.$('.dontclose').hide();
                        this.$('.error').show();
                        this.wizardModel.set('modal.footer', {
                            type: this.wizardModel.get('footerTypes').BUTTONS,
                            backButton: {
                                text: "Try again",
                                click: function(e) {
                                     this.wizardModel.set('currentScreen', this.wizardModel.get('screens').ADD_FILE);
                                }
                            }
                        });
                        break;
                }
            },

            _getSearchUrl: function() {
                var index = this.wizardModel.get('ui.index');
                var host = this.wizardModel.get('ui.host');
                var sourcetype = this.wizardModel.get('ui.sourcetype');
                var source = this.wizardModel.get('file').name;

                // Unconditionally use these as part of the search
                var kvStrings = [
                    'host="' + host + '"',
                    'source="' + source + '"'
                ];

                // Only include sourcetype if it is not auto (in which case we don't know what Splunk decides)
                if (sourcetype) {
                    if (sourcetype === 'manual') {
                        sourcetype = this.wizardModel.get('ui.sourcetype.manual');
                    }
                    kvStrings.push('sourcetype="' + sourcetype + '"');
                }

                // If index is "default", we don't know which index the data ends up in
                if (index !== "default") {
                    kvStrings.push('index="' + index + '"');
                }


                return splunkUtil.make_url('/app/search/search?q=' + 
                    encodeURIComponent(kvStrings.join(' ')));
            }

        });

    });