define(
    [
        'underscore',
        'module',
        'models/classicurl',
        'views/Base',
        'views/shared/jobstatus/Count',
        'views/shared/jobstatus/controls/Master',
        'views/shared/jobstatus/buttons/Master',
        'views/shared/jobstatus/SearchMode',
        'views/shared/jobstatus/AutoPause',
        'views/shared/ProgressBar',
        'uri/route',
        'splunk.window',
        'splunk.util',
        'util/splunkd_utils'
    ],
    function(_, module, classicurlModel, Base, Count, Controls, Buttons, SearchMode, AutoPause, ProgressBar, route, splunkwindow, splunkUtil, splunkd_utils) {
        return Base.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model: {
             *         state: <models.BaseModel>,
             *         searchJob: <helpers.ModelProxy>,
             *         application: <models.Application>,
             *         appLocal: <models.services.AppLocal>,
             *         report: <models.Report> (Optional.)
             *     },
             *     enableSearchMode: <Boolean> Controls the display of adhoc search mode via bunny button.
             *     enableReload: <Boolean> Controls if the reload button will be shown when the job is done, defaults to false
             *     allowDelete: <Boolean> Controls if delete job link is displayed.
             * }
             */
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);
                
                this.fatalErrorTypes = [splunkd_utils.FATAL, splunkd_utils.NOT_FOUND];
                this.renderableErrorTypes = [splunkd_utils.ERROR, splunkd_utils.FATAL, splunkd_utils.NOT_FOUND];

                var defaults = {
                    allowDelete: true
                };

                _.defaults(this.options, defaults);
                // searchMode
                if (this.options.enableSearchMode) { 
                    this.children.searchMode = new SearchMode({
                        model: this.model.state,
                        btnClass: 'btn-mini',
                        rightAlign: true
                    });
                }

                if (this.options.showJobButtons !== false) {
                    this.children.buttons = new Buttons({
                        model: {
                            searchJob: this.model.searchJob,
                            application: this.model.application,
                            report: this.model.report,
                            reportPristine: this.model.reportPristine,
                            serverInfo: this.model.serverInfo
                        }
                    });
                }

                //controls
                this.children.controls = new Controls({
                    showJobMenu: this.options.showJobMenu,
                    allowDelete: this.options.allowDelete,
                    enableReload: this.options.enableReload,
                    model: {
                        searchJob: this.model.searchJob,
                        application: this.model.application,
                        appLocal: this.model.appLocal,
                        report: this.model.report,
                        serverInfo: this.model.serverInfo
                    }
                });
                
                //count
                this.children.count = new Count({model: this.model.searchJob});
                //AutoPause
               
                if (this.options.showAutoPause) {
                    this.children.autoPause = new AutoPause({
                        model: {
                            searchJob: this.model.searchJob
                        }
                    });
                }
                
                
                //Progress
                this.children.progress = new ProgressBar({model: this.model.searchJob});
                
                this.activate();
            },
            startListening: function() {
                this.listenTo(this.model.searchJob, "sync error", function(model, response) {
                    var messages = this.model.searchJob.error.get("messages");
                    if (splunkd_utils.messagesContainsOneOfTypes(messages, this.renderableErrorTypes)) {
                        this.renderError(messages, response);
                    }
                });
                
                this.listenTo(this.model.searchJob.control, "error", function(model, response) {
                    var messages = this.model.searchJob.control.error.get("messages");
                    if (splunkd_utils.messagesContainsOneOfTypes(messages, this.renderableErrorTypes)) {
                        this.renderError(messages, response);
                    }
                });
            },
            events: {
                'click a.job_inspector': function(e) {
                    splunkwindow.open(
                        route.jobInspector(this.model.application.get('root'), this.model.application.get('locale'), this.model.application.get('app'), this.model.searchJob.id),
                        'splunk_job_inspector',
                        {
                            width: 870, 
                            height: 560,
                            menubar: false
                        }
                    );
                    e.preventDefault();
                }
            },
            render: function() {
                if (splunkd_utils.messagesContainsOneOfTypes(this.model.searchJob.error.get("messages"), this.fatalErrorTypes) || this.model.searchJob.entry.content.get("isFailed")) {
                    return this.renderError();
                }
                
                if (this.options.showAutoPause) {
                    this.children.autoPause.render().appendTo(this.$el);
                }

                if(this.$('.jobstatus-control-grouping').length === 0) {
                    var rightSideButtons = this.compiledTemplate({});
                    this.$el.append(rightSideButtons);
                }
                
                this.children.controls.render().appendTo(this.$('.jobstatus-control-grouping'));

                if(this.options.showJobButtons !== false) {
                    this.children.buttons.render().appendTo(this.$('.jobstatus-control-grouping'));
                }

                if (this.options.enableSearchMode) {
                    this.children.searchMode.render().appendTo(this.$('.jobstatus-control-grouping'));
                }

                this.children.count.render().appendTo(this.$el);
                this.children.progress.render().appendTo(this.$el);
                
                return this;
            },
            renderError: function(messages, response) {
                var link = '<a class="job_inspector" href="#">' + _('Job Inspector').t() + '</a>',
                    id = this.model.searchJob.id || _('unknown').t(),
                    error, template;
                
                if (splunkd_utils.messagesContainsOneOfTypes(messages, [splunkd_utils.NOT_FOUND]) || (response && response.hasOwnProperty('status') && response.status == 404)) {
                    error = splunkUtil.sprintf(_('The search job "%s" was canceled remotely or expired.').t(), id);
                } else if (this.model.searchJob.entry.content.get("isFailed")) {
                    error = splunkUtil.sprintf(_('The search job has failed due to an error. You may be able view the job in the %s.').t(), link);
                }
                
                if (error) {
                    template = _.template(this.errorTemplate, {
                        _: _,
                        link: link,
                        splunkUtil: splunkUtil,
                        error: error
                    });
                    
                    this.el.innerHTML = template;
                }

                return this;
            },
            template: '\
                <div class="pull-right jobstatus-control-grouping"></div>\
            ',
            errorTemplate: '\
                <div class="alert alert-error">\
                    <i class="icon-alert"></i>\
                    <%= error %>\
                </div>\
            ',
            notFoundTemplate: '\
                <div class="alert alert-error">\
                    <i class="icon-alert"></i>\
                    <%= splunkUtil.sprintf(_("The job has failed due to an error. You may be able view the job in the %s.").t(), link) %>\
                </div>\
            '
        });
    }
);
