define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/shared/jobstatus/buttons/ShareDialog',
        'models/search/Job',
        'bootstrap.tooltip'
    ],
    function(_, module, Base, ShareDialog, SearchJobModel) {
        return Base.extend({
            moduleId: module.id,
            className: 'share btn-pill btn-square disabled',
            tagName: 'a',
            attributes: {
                "href": "#"
            },
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);
                this.$el.html('<i class="icon-share"></i><span class="hide-text">' + _("Share").t() + '</span>');
                this.$el.tooltip({animation:false, title:_('Share').t(), container: this.$el});
                this.activate();
            },
            startListening: function() {
                this.listenTo(this.model.searchJob.entry.acl, "change", this.render);
                this.listenTo(this.model.searchJob.entry.content, "change:ttl", this.render);
            },
            activate: function(options) {
                if (this.active) {
                    return Base.prototype.activate.apply(this, arguments);
                }
                this.defaultSaveTTL = parseInt(this.model.searchJob.entry.content.get("defaultSaveTTL"), 10);

                return Base.prototype.activate.apply(this, arguments);
            },
            events: {
                'click': function(e) {
                    var $target = $(e.currentTarget),
                        isRealTime = this.model.searchJob.entry.content.get("isRealTimeSearch"),
                        saveDeferred;

                    e.preventDefault();

                    if ($target.hasClass("disabled")) {
                        return;
                    }

                    //do the work of sharing the job
                    if (isRealTime) {
                        saveDeferred = this.model.searchJob.saveJob({
                            auto_cancel: SearchJobModel.DEFAULT_AUTO_CANCEL
                        });
                    } else {
                        //if the job is not realtime then we use the job endpoint's inherent clear of auto_pause and auto_cancel
                        saveDeferred = this.model.searchJob.saveJob();
                    }
                    $.when(saveDeferred).then(function() {
                        this.model.searchJob.makeWorldReadable({success: function() {
                            var fetch = this.model.searchJob.fetch();

                            $.when(fetch).then(function() {
                                this.children.shareDialog = new ShareDialog({
                                    model: {
                                        searchJob: this.model.searchJob,
                                        application: this.model.application,
                                        report: this.model.report,
                                        serverInfo: this.model.serverInfo
                                    },
                                    onHiddenRemove: true
                                });
                                
                                this.children.shareDialog.render().appendTo($("body"));
                                this.children.shareDialog.show();
                            }.bind(this));
                        }.bind(this)});
                    }.bind(this));
                }
            },
            render: function() {
                var canWrite = this.model.searchJob.entry.acl.canWrite(),
                    isShared = this.model.searchJob.isSharedAccordingToTTL(this.defaultSaveTTL);

                if (canWrite){
                    this.$el.removeClass("disabled");
                } else {
                    this.$el.addClass("disabled");
                }
                return this;
            }
        });
    }
);
