define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/shared/jobstatus/controls/Cancel',
        'views/shared/jobstatus/controls/Stop',
        'views/shared/jobstatus/controls/PlayPause',
        'views/shared/jobstatus/controls/Reload',
        'views/shared/jobstatus/controls/menu/Master',
        'util/console'
    ],
    function(_, module, Base, Cancel, Stop, PlayPause, Reload, Menu, console) {
        return Base.extend({
            moduleId: module.id,
            className: 'pull-left includes-job-menu',
            initialize: function(){
                Base.prototype.initialize.apply(this, arguments);

                if (this.options.showJobMenu !== false) {
                    this.children.menu = new Menu({
                        model: {
                            searchJob: this.model.searchJob,
                            application: this.model.application,
                            appLocal: this.model.appLocal,
                            report: this.model.report,
                            serverInfo: this.model.serverInfo
                        },
                        allowDelete: this.options.allowDelete
                    });
                } else {
                    this.$el.removeClass('includes-job-menu');
                }

                this.children.playPause = new PlayPause({model: this.model.searchJob});
                //TODO: hiding cancel for now because it doesn't show up in the prototype
                //this.children.cancel = new Cancel({model: this.model});
                this.children.stop = new Stop({model: this.model.searchJob});
                if(this.options.enableReload) {
                    this.children.reload = new Reload({model: this.model.searchJob});
                }
                
                this.activate({skipRender: true});
            },
            startListening: function() {
                this.listenTo(this.model.searchJob, 'sync', this.render);
            },
            activate: function(options) {
                options = options || {};  
                if (this.active) {
                    return Base.prototype.activate.apply(this, arguments);
                }
                if (!options.skipRender) {
                    this.render();
                }
                return Base.prototype.activate.apply(this, arguments);
            },
            render: function() {
                if (!this.el.innerHTML) {
                    _.each(this.children, function(child) {
                        child.render().appendTo(this.$el);
                    }, this);
                }

                var dynamicChildren = [
                    this.children.playPause,
                    //this.children.cancel,
                    this.children.stop
                ];
                if(this.options.enableReload) {
                    dynamicChildren.push(this.children.reload);
                }
                _.each(dynamicChildren, function(child) {
                        child.$el[child.isActive() ? "removeClass" : "addClass"]("disabled");
                        // Only append the child if it's not already there, this avoids a flickering button. (SPL-80413)
                        if(!$.contains(this.el, child.el)) {
                            child.appendTo(this.$el);
                        }
                        var wasFocused = child.$el.is(':focus');
                        if(wasFocused) {
                            child.$el.focus();
                        }
                        
                }, this);

                return this;
            }
        });
    }
);
