define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/shared/controls/SyntheticSelectControl'
    ],
    function(
        _,
        module,
        BaseView,
        SyntheticSelectControl
    ) {
        return BaseView.extend({
            moduleId: module.id,
            className: 'sourcetypeMenu',
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                var self = this;

                // Should we add "New Sourcetype" item?
                this.addNewSourcetypeLink = this.options.addNewSourcetypeLink;

                var opts = {
                    label: this.options.addLabel ? _('Sourcetype: ').t() : '',  //IB
                    defaultValue: 'default',
                    toggleClassName: 'btn',
                    menuClassName: 'dropdown-menu-sourcetype',
                    items: this.buildItems.call(this),
                    prompt: _('-- Select Sourcetype --').t(),
                    popdownOptions: { attachDialogTo: 'body' }
                };
                if(this.model.sourcetypeModel){
                    opts.model = this.model.sourcetypeModel.entry;
                    opts.modelAttribute = 'name';
                } else if (this.model && this.options.modelAttribute) {
                    opts.model = this.model;
                    opts.modelAttribute = this.options.modelAttribute;
                }

                this.children.sourcetypes = new SyntheticSelectControl(opts);

                this.collection.sourcetypesCollection.on('sync add', function() {
                    self.children.sourcetypes.setItems(self.buildItems.call(self));
                });
            },
            buildItems: function(){
                var items = [],
                    staticItems = [],
                    dynamicItems = [],
                    categoryToIndex = {};

                if (this.addNewSourcetypeLink) { //IB
                    staticItems.push({
                        value: _('default').t(),
                        label: _('System Defaults').t(),
                        description: _('Select to use default settings with no custom configurations').t()
                    });
                }

                this.collection.sourcetypesCollection.each(function(sourcetype){
                    var name = sourcetype.entry.get('name'),
                        category = sourcetype.entry.content.get('category') || _('Uncategorized').t(),
                        description = sourcetype.entry.content.get('description') || '',
                        item;

                    // item must have a name value to be considered
                    if (!name) { return; }

                    //Add client side manufactured sourcetype to static items
                    if(name === '__auto__learned__'){
                        staticItems.push({
                            value: name,
                            label: _('Suggested Settings').t(),
                            description: _('Select to use settings based on your data input').t()
                        });
                        return;
                    }

                    item = {
                        value: name,
                        category: category,
                        description: description
                    };

                    if (category in categoryToIndex) {
                        // append item to corresponding category
                        dynamicItems[categoryToIndex[category]].children.push(item);
                    } else {
                        // add new category, and append item to it
                        dynamicItems.push({value: category, children: []});
                        categoryToIndex[category] = dynamicItems.length-1;
                        dynamicItems[dynamicItems.length-1].children.push(item);
                    }
                });

                // Sort categories
                dynamicItems.sort(function(a,b) {
                    return a.value.localeCompare(b.value);
                });

                items = staticItems.length > 0 ? [staticItems, dynamicItems] : [dynamicItems];
                return items;
            },
            render: function() {
                if (this.children.sourcetypes) {
                    this.children.sourcetypes.detach();
                }
                this.$el.append(this.children.sourcetypes.render().$el);
                return this;
            }
        });
    }
);