// splunk bar
define([
    'jquery',
    'underscore',
    'backbone',
    'module',
    'views/Base',
    'views/shared/litebar/SystemSection',
    'views/shared/litebar/Activity',
    'views/shared/litebar/HelpMenu',
    'views/shared/litebar/RestartDialog',
    'views/shared/onboarding/Master',
    'models/services/data/UserPref',
    'models/services/AppLocal',
    'models/services/authentication/User',
    'models/services/data/ui/Pref',
    'models/shared/Application',
    'collections/services/data/ui/Managers',
    'collections/shared/splunkbar/SystemMenuSections',
    'contrib/text!views/shared/litebar/SideNav.html',
    'views/shared/delegates/Accordion',
    'uri/route',
    'splunk.util',
    'splunk.messenger',
    'splunk.logger',
    'util/splunkd_utils'
],
function(
    $,
     _,
     Backbone,
     module,
     BaseView,
     SystemSection,
     ActivityMenu,
     HelpMenu,
     RestartDialog,
     OnboardingModal,
     UserPrefModel,
     AppLocalModel,
     UserModel,
     uiPref,
     ApplicationModel,
     ManagersCollection,
     SystemMenuSectionsCollection,
     sideTemplate,
     Accordion,
     route,
     splunkUtil,
     splunkMessenger,
     splunkLogger,
     splunkDUtils
){
    var View = BaseView.extend({
        moduleId: module.id,
        template: sideTemplate,
        className: 'sidenav',
        initialize: function() {
            BaseView.prototype.initialize.apply(this, arguments);
            
            this.children.accordion = new Accordion({
                el: this.el,
                group: ".collapsible-group",
                toggle: ".collapsible-toggle",
                body: ".collapsible-body",
                icon: ".icon-collapsible-toggle",
                inactiveIconClass: "icon-chevron-right",
                activeIconClass: "icon-chevron-down",
                collapsible: true
            });
            this.children.activityMenu = new ActivityMenu({
                model: {
                    user: this.model.user,
                    application: this.model.application
                }
            });
            this.children.helpMenu = new HelpMenu({
                model: this.model,
                collection: {
                    apps: this.collection.apps
                }
            });

            this.collection.sections.on('ready', function() {
                this.debouncedRender();
            }, this);

            this.model.userPref.entry.content.on('ready', function() {
                this.debouncedRender();
            }, this);

            this.children.accordion.on('toggled', function() {
                this.setPrefs();
            }, this);

            //listen for restart events
            $(document).on('restart_failed', function() {
                this.hideRestartModal();
            }.bind(this));
        },
        events: {
            'click #control' : function(e) {    
                e.preventDefault();
                if (confirm(_("Are you sure you want to restart Splunk?").t())) {
                    this.showRestartModal();
                    splunkUtil.restart_server();
                }
            },
            'click .splunk-tour' : function(e) {    
                e.preventDefault();
                this.children.onboardingModal = new OnboardingModal({
                    model: {
                        application: this.model.application                        
                    },
                    backdrop: false,
                    keyboard: false
                });
                
                // Necessary for modal to render correctly in manager.
                $('body').append('<div class="splunk-components onboarding-modal"></div>');
                $('.onboarding-modal').append(this.children.onboardingModal.render().el);
                this.children.onboardingModal.show();
            }
        },
        showRestartModal: function() {
                this.children.restartDialog = new RestartDialog({backdrop: 'static'});
                // Necessary for modal to render correctly in manager.
                $("body").append('<div class="splunk-components restartModal"></div>');
                $(".restartModal").append(this.children.restartDialog.render().el);
                this.children.restartDialog.show();
        },
        hideRestartModal: function() {
            if (this.children.restartDialog) {
                this.children.restartDialog.hide();
            }
        },
        _setInitials: function() {
            var name = this.model.user.entry.content.get("realname") || this.model.user.entry.get("name") || "",
                names = name.split(/\s/),
                first = (names.length > 0) ? names[0].charAt(0) : "",
                last = (names.length > 1) ? names[names.length - 1].charAt(0) : "";
            
            this.$('.user-name').html(first + last);
        },
        setPrefs: function(e) {
            var activeNavs = '|',
                elements = this.$('.collapsible-group');
            for (var i = 0; i < elements.length; i++) {
                if ($(elements[i]).hasClass('active')) {
                    activeNavs = activeNavs.concat($(elements[i]).attr('id')).concat('|');
                }
            }
            this.model.userPref.entry.content.set({
                'display.prefs.activeSideNav': activeNavs
            });
            this.model.userPref.save({});
        },
        render: function() {
            var rootUrl = this.model.application.get('root'),
                locale = this.model.application.get('locale'),
                userName = this.model.user.entry.get('name'),
                logoutLink = this.SSOModeOn ? null : route.logout(rootUrl, locale),
                accountLink = route.manager(
                    rootUrl,
                    locale,
                    this.model.application.get('app'),
                    [
                        'authentication',
                        'users'
                    ]
                ),
                that = this;
            
            var html = this.compiledTemplate({
                makeUrl: splunkUtil.make_url,
                options: this.options,
                user: this.model.user.entry.content.get('realname'),
                logoutLink: logoutLink,
                accountLink : accountLink,
                splunkUtil: splunkUtil
            });

            this.$el.html(html);
            
            this._setInitials();

            this.collection.sections.each(function(section){
                if (section.get('items') && section.get('items').length === 0) {
                    return;
                }
                var sectionView = new SystemSection({
                    model: section
                });
                
                that.$('.sidenav-body').append(sectionView.render().el);
            });

            this.$('.sidenav-body').prepend(this.children.activityMenu.render().el);
            this.$('.sidenav-body').append(this.children.helpMenu.render().el);

            var activeSideNavs = this.model.userPref.entry.content.get('display.prefs.activeSideNav') || [];
            _.each(this.$('.collapsible-group'), function(node) {
                if (activeSideNavs.indexOf(node.id) > -1) {
                    this.children.accordion.initialToggleOn($(node));
                }
            }.bind(this));

            return this;
        }
    },
    {
        create: function(options){
            options = options || {};
            options.collection = options.collection || {};
            options.model = options.model || {};

            //the APPLICATION model is REQUIRED argument from the consumer. If its not passed, make up an empty one, to keep things rendering, and assure continuance
            //TODO should log this
            var applicationDfd = $.Deferred();
            if (!options.model.application){
                options.model.application = new ApplicationModel();
            }
            // handle both when the application model is already filled and when it has yet to complete fetching
            if (options.model.application.get('app')) {
                applicationDfd.resolve();
            } else {
                options.model.application.on('change', applicationDfd.resolve);
            }

            var currentUserIdDfd = $.Deferred();
            currentUserIdDfd.resolve(options.model.application.get('owner'));

            if (!options.model.user) {
                options.model.user = new UserModel();
                $.when(currentUserIdDfd).done(function(currentUserId) {
                    options.model.user.set('id', encodeURIComponent(currentUserId));
                    options.model.user.fetch();
                });
            }

            var managersDfd = $.Deferred();
            if (!options.collection.managers) {
                options.collection.managers = new ManagersCollection();
                $.when(currentUserIdDfd).done(function(currentUsername) {
                    options.collection.managers.fetch({
                        data: {
                            app: '-',
                            owner: currentUsername,
                            count: 0,
                            digest: 1
                        },
                        success: function() {
                            managersDfd.resolve();
                        }
                    });
                });
            } else {
                managersDfd.resolve();
            }

            var sections = options.collection.sections = new SystemMenuSectionsCollection();

            sections.add({
                id: 'knowledge_configurations',
                label: _('Knowledge').t(),
                icon: 'bookmark',
                order: 2
            });

            if (options.model.user.hasRole("admin")) {
                sections.add({
                    id: 'data_configurations',
                    label: _('Data').t(),
                    icon: 'data',
                    order: 1
                });
                sections.add({
                    id: 'system_configurations',
                    label: _('System').t(),
                    icon: 'settings',
                    order: 3
                });
            }

            $.when(managersDfd).done(function() {
                options.collection.managers.each(function(manager) {
                    var menuUrl = manager.entry.content.get('menu.url') || '',
                        sectionName = manager.entry.content.get('menu.name'),
                        disabledByLicense = splunkUtil.normalizeBoolean(manager.entry.content.get('disabled_by_license') || false),
                        order = manager.entry.content.get('menu.order') || 1000,
                        pageStart = route.encodeRoot(options.model.application.get('root'), options.model.application.get('locale')),
                        url = pageStart + splunkUtil.sprintf(menuUrl, {namespace: options.model.application.get('app') || 'NOTHING'});

                    if (!disabledByLicense && sectionName) {
                        var section = sections.get(sectionName);
                        if (section) {
                            var sectionItems = section.get('items');
                            if (sectionItems) {
                                sectionItems.push(manager);
                            }
                        }
                    }
                    manager.set({
                        url: url,
                        order: order
                    });
                });
                sections.trigger('ready');
            });

            return new View(options);
        }
    });
    return View;
});
