define(
    [
        'jquery',
        'underscore',
        'backbone',
        'module',
        'views/shared/Modal',
        'contrib/text!views/shared/onboarding/Master.html',
        'splunk.util',
        'uri/route',
        'bootstrap.carousel'
    ],
    function(
        $, 
        _, 
        backbone, 
        module, 
        Modal,
        TourTemplate,
        splunkUtil,
        route,
        undefined
    ){
        return Modal.extend({
            moduleId: module.id,
            className: Modal.prototype.className + ' super-modal onboarding',
            template: TourTemplate,
            initialize: function() {
                Modal.prototype.initialize.apply(this, arguments);

                this.imgCheck = $.Deferred();
                this.root = this.model.application.get('root');
                this.locale = this.model.application.get('locale');
                this.imgUrl = splunkUtil.make_url('/static/img/skins/default/onboarding');
                this.imgUrlLocalized = this.imgUrl + '/' + this.locale;
                this.testImg = this.imgUrlLocalized + '/tour0.png';
                this.panels = 7;
                this.welcomeSlide = this.options.welcomeSlide || false;

                // Check for localized version of Onboarding experience.
                // Defaults to en-US if not.
                $('<img src="' + this.testImg + '" />')
                .load(function() {
                    this.imgCheck.resolve();
                }.bind(this))
                .error(function() {
                    this.imgUrlLocalized = this.imgUrl + '/en-US';
                    this.imgCheck.resolve();
                }.bind(this));
            },

            events: {
                'click .close-tour': function(e) {
                    this.hide();
                },
                'click .exit-tour': function(e) {
                    this.hide();
                },
                'click .start-tour': function(e) {
                    $('.welcome-slide').hide();
                },
                'mouseout .carousel': function(e) {
                    $('carousel').carousel('pause');
                }
            },

            hide: function() {
                this.$el.modal('hide');
                this.closeTour();
            },

            setFirstTimePref: function() {
                if (this.model.userPrefs) {
                    this.model.userPrefs.entry.content.set('first.time.lite', 1);
                    this.model.userPrefs.save();
                }
            },

            closeTour: function() {
                this.setFirstTimePref();
                this.remove();
                $('.super-backdrop').fadeOut();
            },

            render: function() {
                var docLink = route.docHelp(this.root, this.locale, 'splunkcore.homepage.docs'),
                    salesLink = 'http://www.splunk.com/view/contact-us/SP-CAAAAH7',
                    supportLink = 'http://www.splunk.com/r/support',
                    educationLink = 'http://www.splunk.com/view/education/SP-CAAAAH9',
                    answersLink = 'http://answers.splunk.com/';

                // The 'splunk-components' wrapper is necessary  
                // for modal to render correctly in manager.
                $('body').append('<div class="splunk-components"><div class="super-backdrop modal-backdrop in"></div></div>');
                
                $.when(this.imgCheck).then(function() {
                    this.$el.html(this.compiledTemplate({
                        _: _,
                        panels: this.panels,
                        imgUrl: this.imgUrl,
                        imgUrlLocalized: this.imgUrlLocalized,
                        docLink: docLink,
                        salesLink: salesLink,
                        supportLink: supportLink,
                        educationLink: educationLink,
                        answersLink: answersLink,
                        welcomeSlide: this.welcomeSlide
                    }));

                    this.$('.carousel').carousel({
                        interval: false,
                        pause: true
                    }).on('slid', '', function() {
                        $('.carousel-control').show();
                        if($('.item:first').hasClass('active')) {
                            $('.left.carousel-control').hide();
                        } else if ($('.item:last').hasClass('active')) {
                            $('.right.carousel-control').hide();
                        }
                    });
                }.bind(this));
            }
        });
    }
);
