define(
    [
         'jquery',
         'underscore',
         'module',
         'views/Base',
         'views/shared/controls/ControlGroup',
         'views/shared/FlashMessages',
         'views/shared/Modal',
         'helpers/user_agent'
     ],
     function(
         $,
         _,
         module,
         Base,
         ControlGroup,
         FlashMessage,
         Modal,
         userAgent
     )
     {
        return Base.extend({
            moduleId: module.id,
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);

                var viz_type = this.model.inmem.getVizType();


                this.model.inmem.setTimeRangeWarnings();
                this.children.flashMessage = new FlashMessage({ model: this.model.inmem });

                //name
                this.children.title = new ControlGroup({
                    label: _("Title").t(),
                    controlType:'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        model: this.model.inmem.entry.content,
                        modelAttribute: 'name'
                    }
                });

                //description
                this.children.description = new ControlGroup({
                    label: _("Description").t(),
                    controlType:'Textarea',
                    controlClass: 'controls-block',
                    controlOptions: {
                        model: this.model.inmem.entry.content,
                        modelAttribute: 'description',
                        placeholder: _('optional').t()
                    }
                });

                //viz toggle
                this.children.visualization = new ControlGroup({
                    label: _("Visualization").t(),
                    controlType:'SyntheticRadio',
                    controlClass: 'controls-halfblock',
                    controlOptions: {
                        className: "btn-group btn-group-2",
                        items: [
                            { value: '1', label: viz_type.label, icon: viz_type.icon, iconSize: '' },
                            { value: '0', label: _('None').t()}
                        ],
                        model: this.model.inmem.entry.content,
                        modelAttribute: 'display.visualizations.show'
                    }
                });

                //timeRangePicker toggle
                this.children.timeRangePickerToggle = new ControlGroup({
                    label: _("Time Range Picker").t(),
                    controlType:'SyntheticRadio',
                    controlClass: 'controls-halfblock',
                    controlOptions: {
                        className: "btn-group btn-group-2",
                        items: [
                            { value: '1', label: _('Yes').t() },
                            { value: '0', label: _('No').t() }
                        ],
                        model: this.model.inmem.entry.content,
                        modelAttribute: 'display.general.timeRangePicker.show'
                    }
                });
            },
            events: {
                "click .modal-btn-primary": function(e) {
                    this.submit();
                    e.preventDefault();
                }
            },
            render: function() {
                this.$el.html(Modal.TEMPLATE);

                this.$(Modal.HEADER_TITLE_SELECTOR).html(_("Save As Report").t());

                this.children.flashMessage.render().prependTo(this.$(Modal.BODY_SELECTOR));

                this.$(Modal.BODY_SELECTOR).append(Modal.FORM_HORIZONTAL);

                this.children.title.render().appendTo(this.$(Modal.BODY_FORM_SELECTOR));
                this.children.description.render().appendTo(this.$(Modal.BODY_FORM_SELECTOR));

                if (this.options.chooseVisualizationType) {

                    // SPL-77544 - IE by default selects the first button on enter
                    if(userAgent.isIE8()) {
                        this.$(Modal.BODY_FORM_SELECTOR).append('<button style="display:none" />');
                    } else if(userAgent.isIE()) {
                        this.$(Modal.BODY_FORM_SELECTOR).append('<button style="position: absolute; right: 10000px;" />');
                    }

                    if (this.model.searchJob.isReportSearch()) {
                        var type = this.model.inmem.entry.content.get('display.general.type');
                        
                        if (type === 'statistics') {
                            this.model.inmem.entry.content.set('display.visualizations.show', '0');
                        } else if (type === 'visualizations') {
                            this.model.inmem.entry.content.set('display.visualizations.show', '1');
                        }
                        
                        this.children.visualization.render().appendTo(this.$(Modal.BODY_FORM_SELECTOR));
                    } else {
                        this.model.inmem.entry.content.set('display.visualizations.show', '0');
                    }
                }

                if (!this.model.inmem.isAlert()) {
                    this.children.timeRangePickerToggle.render().appendTo(this.$(Modal.BODY_FORM_SELECTOR));
                }

                this.$(Modal.FOOTER_SELECTOR).append(Modal.BUTTON_CANCEL);
                this.$(Modal.FOOTER_SELECTOR).append(Modal.BUTTON_SAVE);

                return this;
            },
            submit: function() {
                this.model.inmem.save({}, {
                    data: this.model.application.getPermissions("private"),
                    success: function(model, response) {
                        this.model.inmem.trigger('createSuccess');
                    }.bind(this)
                });
            }
        });
     }
);
