define(
    [
        'module',
        'jquery',
        'underscore',
        'models/classicurl',
        'views/shared/PopTart',
        'views/shared/documentcontrols/dialogs/TitleDescriptionDialog',
        'views/shared/documentcontrols/dialogs/permissions_dialog/Master',
        'views/shared/reportcontrols/dialogs/schedule_dialog/Master',
        'views/shared/reportcontrols/dialogs/AccelerationDialog',
        'views/shared/documentcontrols/dialogs/DeleteDialog',
        'views/shared/reportcontrols/dialogs/clone_dialog/Master',
        'views/shared/reportcontrols/dialogs/embed_dialog/Master',
        'uri/route',
        'util/general_utils',
        'bootstrap.modal'
    ],
    function(
        module,
        $,
        _,
        classicUrlModel,
        PopTartView,
        TitleDescriptionDialog,
        PermissionsDialog,
        ScheduleDialog,
        AccelerationDialog,
        DeleteDialog,
        CloneDialog,
        EmbedDialog,
        route,
        util,
        undefined
    )
    {
        return PopTartView.extend({
            moduleId: module.id,
            className: 'dropdown-menu dropdown-menu-narrow',
            initialize: function() {
                PopTartView.prototype.initialize.apply(this, arguments);
                var defaults = {
                        button: true,
                        showOpenActions: true,
                        deleteRedirect: false
                    };

                _.defaults(this.options, defaults);

                if (this.model.searchJob){
                    this.model.searchJob.on("prepared", function() {
                        this.$('a.clone').removeClass('disabled');
                        this.$('a.edit-acceleration').removeClass('disabled');
                    }, this);
                }
            },
            events: {
                'click a.edit-description': function(e) {
                    this.hide();
                    this.children.titleDescriptionDialog = new TitleDescriptionDialog({
                        model: this.model.report,
                        onHiddenRemove: true
                    });

                    this.children.titleDescriptionDialog.render().appendTo($("body"));
                    this.children.titleDescriptionDialog.show();

                    e.preventDefault();
                },
                'click a.edit-permissions': function(e) {
                    this.hide();
                    this.children.permissionsDialog = new PermissionsDialog({
                        model: {
                            document: this.model.report,
                            nameModel: this.model.report.entry,
                            user: this.model.user,
                            serverInfo: this.model.serverInfo
                        },
                        collection: this.collection,
                        onHiddenRemove: true,
                        nameLabel: _('Report').t()
                    });

                    this.children.permissionsDialog.render().appendTo($("body"));
                    this.children.permissionsDialog.show();

                    e.preventDefault();
                },
                'click a.edit-schedule': function(e) {
                    this.hide();
                    this.children.scheduleDialog = new ScheduleDialog({
                        model: {
                            report: this.model.report,
                            application: this.model.application,
                            user: this.model.user,
                            appLocal: this.model.appLocal,
                            serverInfo: this.model.serverInfo
                        },
                        onHiddenRemove: true
                    });

                    this.children.scheduleDialog.render().appendTo($("body"));
                    this.children.scheduleDialog.show();

                    e.preventDefault();
                },
                'click a.edit-acceleration': function(e) {
                    this.hide();
                    this.children.accelerationDialog = new AccelerationDialog({
                        model: {
                            report: this.model.report,
                            searchJob: this.model.searchJob,
                            application: this.model.application
                        },
                        onHiddenRemove: true
                    });

                    this.children.accelerationDialog.render().appendTo($("body"));
                    this.children.accelerationDialog.show();

                    e.preventDefault();
                },
                'click a.clone': function(e) {
                    this.hide();
                    this.children.cloneDialog = new CloneDialog({
                        model: {
                            report: this.model.report,
                            searchJob: this.model.searchJob,
                            application: this.model.application,
                            user: this.model.user,
                            serverInfo: this.model.serverInfo
                        },
                        onHiddenRemove: true
                    });

                    this.children.cloneDialog.render().appendTo($("body"));
                    this.children.cloneDialog.show();

                    e.preventDefault();
                },
                'click a.embed': function(e) {
                    this.hide();
                    this.children.embedDialog = new EmbedDialog({
                        model: this.model,
                        onHiddenRemove: true
                    });

                    this.children.embedDialog.render().appendTo($("body"));
                    this.children.embedDialog.show();

                    e.preventDefault();
                },
                'click a.delete': function(e){
                    this.hide();
                    this.children.deleteDialog = new DeleteDialog({
                        model: {
                            report: this.model.report,
                            application: this.model.application
                        },
                        deleteRedirect: this.options.deleteRedirect,
                        onHiddenRemove: true
                    });

                    this.children.deleteDialog.render().appendTo($("body"));
                    this.children.deleteDialog.show();

                    e.preventDefault();
                },
                'click a.open-in-search': function(e) {
                    var data = {};
                    if (this.model.searchJob && !this.model.searchJob.isNew()) {
                        data = classicUrlModel.toJSON();
                    } else {
                        data = {s: this.model.report.id};
                    }
                    var routeString = route.search(
                            this.model.application.get('root'),
                            this.model.application.get('locale'),
                            this.model.application.get('app'),
                            {data: data}
                        );
                    window.location = routeString;

                    e.preventDefault();
                }
            },
            render: function() {
                var openInPivot = route.pivot(
                        this.model.application.get("root"),
                        this.model.application.get("locale"),
                        this.model.application.get("app"),
                        {data: {s: this.model.report.id}}
                     ),
                    canWrite = this.model.report.canWrite(this.model.user.canScheduleSearch(), this.model.user.canRTSearch()),
                    canClone = this.model.report.canClone(this.model.user.canScheduleSearch(), this.model.user.canRTSearch()),
                    canEmbed = this.model.report.canEmbed(this.model.user.canScheduleSearch(), this.model.user.canEmbed()),
                    canDelete = this.model.report.canDelete(),
                    isEmbedded = util.normalizeBoolean(this.model.report.entry.content.get('embed.enabled'));
                var html = this.compiledTemplate({
                    _: _,
                    button: this.options.button,
                    report: this.model.report,
                    user: this.model.user,
                    openInPivot: openInPivot
                });
                this.$el.html(PopTartView.prototype.template_menu);
                this.$el.append(html);
                
                if (isEmbedded) {
                    this.$('.edit_messages').append('<li class="message">' + _("Disable embedding to edit report.").t() + '</li>');
                } else {
                    this.$('.edit_messages').remove();
                }

                if (this.options.showOpenActions) {
                    this.$('.open_actions').show();
                } else {
                    this.$('.open_actions').remove();
                }

                if (canWrite && !isEmbedded) {
                    this.$('.edit_actions').append('<li><a class="edit-description" href="#">' + _("Edit Description").t() + '</a></li>');

                    // Only show if user has perm to change perms
                    if (this.model.report.entry.acl.get('can_change_perms')) {
                        this.$('.edit_actions').append('<li><a class="edit-permissions" href="#">' + _("Edit Permissions").t() + '</a></li>');
                    }
                    if (this.model.user.canScheduleSearch() && !this.model.report.isRealTime()) {
                    // Check if real-time. User can not schedule a real-time search
                        this.$('.edit_actions').append('<li><a class="edit-schedule" href="#">' + _("Edit Schedule").t() + '</a></li>');
                    }
                    if (!this.model.report.isPivotReport() && this.model.user.canAccelerateReport()) {
                        this.$('.edit_actions').append('<li><a class="edit-acceleration" href="#">' + _("Edit Acceleration").t() + '</a></li>');
                    }
                } else {
                    this.$('.edit_actions').remove();
                }

                if (canClone) {
                    this.$('.other_actions').append('<li><a href="#" class="clone">' + _("Clone").t() + '</a></li>');
                }

                if (canEmbed) {
                    this.$('.other_actions').append('<li><a href="#" class="embed">' + _("Embed").t() + '</a></li>');
                }

                if (canDelete && !isEmbedded) {
                    this.$('.other_actions').append('<li><a href="#" class="delete">' + _("Delete").t() + '</a></li>');
                }

                if (this.model.searchJob && this.model.searchJob.isPreparing()) {
                    this.$('a.clone').addClass('disabled');
                    this.$('a.edit-acceleration').addClass('disabled');
                }
                return this;
            },
            template: '\
                <ul class="edit_messages ">\
                </ul>\
                <ul class="open_actions">\
                    <% if (report.openInView(user) === "pivot") { %>\
                        <li><a href="<%- openInPivot %>"><%- _("Open in Pivot").t() %></a></li>\
                    <% } else { %>\
                        <li><a class="open-in-search" href="#"><%- _("Open in Search").t() %></a></li>\
                    <% } %>\
                </ul>\
                <ul class="edit_actions">\
                </ul>\
                <ul class="other_actions">\
                </ul>\
            '
        });
    }
);
