/**
 * @author jszeto
 * @date 11/4/13
 *
 * ResultTableMaster subclass that adds special logic for showing a checkmark or x in the left column
 */
define([
    'jquery',
    'underscore',
    'backbone',
    'module',
    './ResultsTableMaster',
    './MatchResultsTableHeader',
    './renderers/CheckmarkCellRenderer',
    'util/field_extractor_utils'
],
    function (
        $,
        _,
        Backbone,
        module,
        ResultsTableMaster,
        MatchResultsTableHeader,
        CheckmarkCellRenderer,
        fieldExtractorUtils) {
        
        var MatchResultsTableMaster = ResultsTableMaster.extend({

            moduleId: module.id,

            events: _.extend({}, ResultsTableMaster.prototype.events, {
                'click .remove-button': function(e) {
                    e.preventDefault();
                    var $target = $(e.currentTarget),
                        $highlighted = $target.closest('.' + fieldExtractorUtils.HIGHLIGHTED_MATCH_CLASS),
                        $td = $target.closest('td');

                    this.trigger(
                        'action:removeExtraction',
                        {
                            rawText: $td.text(),
                            fieldName: $highlighted.data('fieldName'),
                            selectedText: $highlighted.text(),
                            startIndex: $highlighted.data('startIndex'),
                            endIndex: $highlighted.data('endIndex')
                        }
                    );
                },
                'mouseenter tr.shared-resultstable-resultstablerow': function(e) {
                    var $target = $(e.target),
                        $tableRow = $target.closest('tr'),
                        $firstCell = $tableRow.children().first();
                    if(this._getAddSampleEnabled()){
                        // Clean up existing add sample event buttons if mouseleave event is not fired because
                        // the user immediately moused into another row's button location
                        _.invoke($('.results-table').find('.add-sample-event-button'), 'remove');
                        $firstCell.append(_(this.addSampleEventButton).template({}));

                        var windowWidth = $(window).width(),
                            buttonMarginLeft = windowWidth - 200;
                        $target.find('.add-sample-event-button').css({ 'margin-left': buttonMarginLeft + 'px' });

                        // Have to dynamically add styling to keep data cell contents aligned correctly
                        $tableRow.find('td:last').css({ 'padding-right':'20px' });
                    }
                },
                'mouseleave tr.shared-resultstable-resultstablerow': function(e) {
                    var $target = $(e.target);
                    if(this._getAddSampleEnabled()){
                        $target.closest('tr').find('.add-sample-event-button').remove();
                    }
                },
                'dblclick tbody td': function(e) {
                    this.handleCellClick($(e.target), e);
                    if(this.model.state.get('mode') === fieldExtractorUtils.SELECT_SAMPLE_MODE && this.model.state.get('interactiveMode') === fieldExtractorUtils.INTERACTION_MODE){
                        this.trigger('action:nextStep');
                    }
                },
                'mousemove tbody td': function(e) {
                    if(this.model.state.get('mode') === fieldExtractorUtils.SELECT_SAMPLE_MODE){
                        e.preventDefault();
                    }
                }
            }),

            initialize: function (options) {
                this.options = $.extend(true, {
                    headerClass: MatchResultsTableHeader
                }, this.options);

                this.addSamplesEnabled = true;
                this.updateTableInteractivity();
                ResultsTableMaster.prototype.initialize.call(this, this.options);
                this.addCellRenderer(_(this.highlightedFieldsCellRenderer).bind(this));
                this.addCellRenderer(new CheckmarkCellRenderer());
                this.listenTo(this.model.state, 'change:mode change:interactiveMode', function() {
                    var mode = this.model.state.get('mode'),
                        previousMode = this.model.state.previous('mode'),
                        interactiveMode = this.model.state.get('interactiveMode'),
                        prevInteractiveMode = this.model.state.previous('interactiveMode');
                    this.updateTableInteractivity();
                    if ((mode === fieldExtractorUtils.VALIDATE_FIELDS_MODE && previousMode === fieldExtractorUtils.SELECT_FIELDS_MODE) ||
                        (previousMode === fieldExtractorUtils.VALIDATE_FIELDS_MODE && mode === fieldExtractorUtils.SELECT_FIELDS_MODE) ||
                        (interactiveMode === fieldExtractorUtils.NO_INTERACTION_MODE && mode === fieldExtractorUtils.VALIDATE_FIELDS_MODE) ||
                        (prevInteractiveMode === fieldExtractorUtils.NO_INTERACTION_MODE && mode === fieldExtractorUtils.VALIDATE_FIELDS_MODE) ||
                        (prevInteractiveMode !== interactiveMode && this.options.requiredText))  {
                        this.debouncedRender();
                    }
                });
            },

            setAddSamplesEnabled: function(isEnabled) {
                this.addSamplesEnabled = isEnabled;
                this.updateTableInteractivity();
            },

            updateTableInteractivity: function() {
                if(this._getCellClickEnabled()) {
                    this.model.format.set({ 'display.statistics.drilldown': 'row' });
                }
                else {
                    this.model.format.set({ 'display.statistics.drilldown': 'none' });
                    this.$('tr').removeClass('highlighted');
                }
            },

            handleCellClick: function($target, event) {
                // Keyboard events can sometimes result in an empty target here, ignore these events (SPL-88809).
                if($target.length === 0) {
                    return;
                }
                if(this._getCellClickEnabled()) {
                    event.preventDefault();
                    $target.blur();
                    $target.closest('tr').blur();
                    if(this.model.state.get('mode') === fieldExtractorUtils.SELECT_SAMPLE_MODE){
                        this.trigger('action:selectEvent', $target.text());
                    }else{
                        var rowTarget = $target.closest('tr').find('.interactive-extraction-cell'); //preserve closest tr in case $target is the add-sample-event-button that is removed
                        $('.add-sample-event-button').remove();
                        this.trigger('action:selectEvent', rowTarget.text());
                        $('html,body').animate({scrollTop: "0px"}, 300);
                    }
                }
            },

            // handleCellMouseover: function($target) {
            //     ResultsTableMaster.prototype.handleCellMouseover.apply(this, arguments);
            //     //TODO - when user hovers over the currently selected sample event, show the htmlBlue
            //     // $('.highlighted').children().css("background-color", "#b5d6fd");
            // },

            generateHeaderViewOptions: function() {
                return _.extend(
                    { matchFieldName: fieldExtractorUtils.MATCH_FIELD_NAME },
                    ResultsTableMaster.prototype.generateHeaderViewOptions.apply(this, arguments)
                );
            },

            generateCellObjects: function(row, fields, dataOverlay, rowNumber) {
                var extractedFieldBoundsIndex = _(this.rawFields).indexOf(fieldExtractorUtils.OFFSET_FIELD_NAME),
                    cellObjects = _(ResultsTableMaster.prototype.generateCellObjects.apply(this, arguments)).map(function(cellObject) {
                    return _.extend({
                            matchFieldName: fieldExtractorUtils.MATCH_FIELD_NAME,
                            inputField: this.options.inputField,
                            fieldBounds: this.generateFieldBounds(rowNumber, extractedFieldBoundsIndex)
                        },
                        cellObject
                    );
                }, this);
                var captureGroupField = _(cellObjects).find(
                    function(cellObject){
                        return cellObject.field === fieldExtractorUtils.OFFSET_FIELD_NAME;
                    }
                );
                _(cellObjects).each(function(cellObject){
                    if(captureGroupField && !cellObject.fieldBounds){
                        cellObject.fieldBounds = captureGroupField.fieldBounds;
                    }
                });
                return cellObjects;
            },

            generateFieldBounds: function(rowIndex, extractedFieldBoundsIndex) {
                if(extractedFieldBoundsIndex !== -1){
                    // This cell is an extracted field
                    var boundingGroupString = this.rawRows[rowIndex][extractedFieldBoundsIndex];
                    return boundingGroupString ? fieldExtractorUtils.parseBoundingGroupString(boundingGroupString) : [];
                }
            },

            isDataField: function(fieldName) {
                return fieldName === fieldExtractorUtils.MATCH_FIELD_NAME
                            || ResultsTableMaster.prototype.isDataField.apply(this, arguments);
            },

            highlightedFieldsCellRenderer: function($td, cellData) {
                if(cellData.field !== cellData.inputField) {
                    return false;
                }
                var selectionStartIndex = -1,
                    requiredTextObject = {text: this.options.requiredText},
                    html;
                if (!this.options.regex) {
                    requiredTextObject.text = "";
                }
                if(this.model.state.get('interactiveMode') === fieldExtractorUtils.NO_INTERACTION_MODE){
                    // Do not display required text highlighting when in manual mode
                    html = fieldExtractorUtils.replaceBoundingGroupsWithTemplate(
                        cellData.value,
                        cellData.fieldBounds
                    );
                }else{
                    html = fieldExtractorUtils.replaceBoundingGroupsWithTemplate(
                        cellData.value,
                        cellData.fieldBounds,
                        requiredTextObject,
                        selectionStartIndex,
                        fieldExtractorUtils.highlightedContentTemplate
                    );
                }
                
                $td.addClass(fieldExtractorUtils.INTERACTIVE_CELL_CLASS).addClass('field-extraction').html(html);
                if(this.model.state.get('mode') === fieldExtractorUtils.VALIDATE_FIELDS_MODE && this.model.state.get('interactiveMode') === fieldExtractorUtils.INTERACTION_MODE) {
                    $td.find('.' + fieldExtractorUtils.HIGHLIGHTED_MATCH_CLASS).append('<a class="remove-button" href="#"><i class="icon-x-circle icon-large"></i></a>');
                }
                return true;
            },

            _getCellClickEnabled: function() {
                return (
                    (this._getAddSampleEnabled() || this.model.state.get('mode') === fieldExtractorUtils.SELECT_SAMPLE_MODE) &&
                    this.model.state.get('interactiveMode') === fieldExtractorUtils.INTERACTION_MODE
                );
            },

            _getAddSampleEnabled: function() {
                return (this.model.state.get('mode') === fieldExtractorUtils.SELECT_FIELDS_MODE && this.model.state.get('interactiveMode') === fieldExtractorUtils.INTERACTION_MODE && this.addSamplesEnabled);
            },

            addSampleEventButton: '<div class="add-sample-event-button"><i class="icon-plus" style="color:white"></i><%- _(" Add sample event").t() %></div>'

        });

        return MatchResultsTableMaster;

    });

