define(function(require, exports, module) {

    var _ = require("underscore");
    var ResultsTableRow = require("../ResultsTableRow");
    var BaseCellRenderer = require("./BaseCellRenderer");

    return BaseCellRenderer.extend({

        canRender: function(cellData) {
            return (cellData.columnType === "number");
        },

        setup: function($td, cellData) {
            $td.addClass("numeric");
        },

        teardown: function($td, cellData) {
            $td.removeClass("numeric with-overlay");
        },

        render: function($td, cellData) {
            cellData = _.extend({
                floatValue: parseFloat(cellData.value),
                heatValue: (cellData.dataOverlay === "heatmap") ?
                    (cellData.heatRange === 0 ? 0 : (Math.min(Math.max(Math.ceil(((cellData.value - cellData.heatOffset) / cellData.heatRange) * 1000) / 1000, 0), 1))) :
                    null,
                MV_SUBCELL_CLASSNAME: ResultsTableRow.MV_SUBCELL_CLASSNAME,
                MV_INDEX_ATTR: ResultsTableRow.MV_INDEX_ATTR
            }, cellData);

            if ((cellData.dataOverlay === "highlow") || (cellData.dataOverlay === "heatmap")) {
                $td.addClass("with-overlay");
            } else {
                $td.removeClass("with-overlay");
            }

            $td.html(_.template(this.template, cellData));
        },

        template: '\
            <% if (dataOverlay === "heatmap") { %>\
                <div class="overlay-cell">\
                    <div class="overlay-cell-value heat-value" style="opacity: <%- heatValue %>; filter: alpha(opacity=<%- heatValue * 100 %>);"></div>\
                </div>\
            <% } else if (dataOverlay === "highlow") { %>\
                <% if (floatValue === extremes.min) { %>\
                    <div class="overlay-cell">\
                        <div class="overlay-cell-value min-value"></div>\
                    </div>\
                <% } else if (floatValue === extremes.max) { %>\
                    <div class="overlay-cell">\
                        <div class="overlay-cell-value max-value"></div>\
                    </div>\
                <% } else { %>\
                    <div class="overlay-cell">\
                        <div class="overlay-cell-value no-value"></div>\
                    </div>\
                <% } %>\
            <% } %>\
            <% if (_.isArray(value)) { %>\
                <% _(value).each(function(subValue, i) { %>\
                    <div class="<%- MV_SUBCELL_CLASSNAME %>" <%- MV_INDEX_ATTR %>="<%- i %>">\
                        <%- subValue %>\
                    </div>\
                <% }) %>\
            <% } else { %>\
                <%- value %>\
            <% } %>\
        '

    });

});
