define(
    [
        'jquery',
        'underscore',
        'backbone',
        'module',
        'views/Base',
        'views/shared/searchbar/Apps',
        'views/shared/searchbar/input/Master',
        'views/shared/timerangepicker/Master',
        'views/shared/searchbar/Submit'
    ],
    function($, _, Backbone, module, BaseView, Apps, Input, TimeRangePicker, Submit) {
        return BaseView.extend({
            moduleId: module.id,
            className: 'search-bar-wrapper',
            /**
             * @param {Object} options {
             *     model: {
             *         state: <models.Report.entry.content>,
             *         timeRange: <models.TimeRange>,
             *         appLocal: <models.services.AppLocal>,
             *         user: <models.services.authentication.User>,
             *         application: <models.Application>
             *         serverInfo: <models.services.server.ServerInfo>
             *     },
             *     collection: {
             *         times (Optional): <collections.services.data.ui.TimesV2>,
             *         apps (Optional): <collections.services.AppsLocals>
             *     }
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);

                _.defaults(this.options, {
                    showTimeRangePicker:true,
                    disableOnSubmit: false,
                    primaryAction: false
                });
                this.showTimeRangePicker = this.options.showTimeRangePicker;

                if (this.collection && this.collection.apps) {
                    this.children.apps = new Apps({
                        collection: this.collection.apps,
                        model: this.model.state,
                        applicationModel: this.model.application
                    });
                }

                this.children.searchInput = new Input($.extend(true, {}, this.options, {
                    model: {
                        content: this.model.state,
                        application: this.model.application
                    }
                }));

                this.children.submit = new Submit({
                    model: this.model.state,
                    serverInfo: this.model.serverInfo
                });

            },
            render: function() {
                if (!this.$el.html()) {
                    var template = _.template(this.template, {showTimeRangePicker: this.showTimeRangePicker, primary: this.options.primaryAction});
                    this.$el.html(template);
    
                    if (this.children.apps) {
                        this.children.apps.render().appendTo(this.$('.search-apps'));
                    }
                    this.children.searchInput.render().appendTo(this.$('.search-input'));
                    if (this.showTimeRangePicker) {
                        this.children.timeRangePicker = new TimeRangePicker({
                            model: {
                                state: this.model.state,
                                timeRange: this.model.timeRange,
                                appLocal: this.model.appLocal,
                                user: this.model.user,
                                application: this.model.application
                            },
                            collection: this.collection.times,
                            className: 'btn-group',
                            timerangeClassName: 'btn',
                            forceTimerangeChange: true
                        });
                        this.children.timeRangePicker.render().appendTo(this.$('.search-timerange'));
                    }
                    this.children.submit.render().appendTo(this.$('.search-button'));
                }
                return this;
            },
            template: '\
                <form method="get" action="" class="search-form">\
                    <table class="search-bar <%- primary ? "search-bar-primary" : "" %>">\
                        <tbody>\
                            <tr>\
                                <td class="search-input" width="100%"></td>\
                                <td class="search-apps"></td>\
                                <% if (showTimeRangePicker) { %>\
                                    <td class="search-timerange"></td>\
                                <% } %>\
                                <td class="search-button"></td>\
                            </tr>\
                        </tbody>\
                    </table>\
                </form>\
            '
        });
    }
);
