define(
    [
        'jquery',
        'underscore',
        'models/search/SearchBar',
        'module',
        'views/Base',
        'views/shared/searchbar/input/SearchField',
        'views/shared/searchbar/input/SearchAssistant',
        'splunk.util'
    ],
    function($, _, SearchBarModel, module, Base, SearchField, SearchAssistant, splunkUtils) {
        return Base.extend({
            moduleId: module.id,
            initialize: function(options) {
                Base.prototype.initialize.apply(this, arguments);

                this.options = $.extend(true, {}, this.options, (options || {}));

                var defaults = {
                    useTypeahead: true,
                    useAssistant: true,
                    useAutoFocus: true,
                    autoOpenAssistant: splunkUtils.normalizeBoolean(this.model.content.get('display.prefs.autoOpenSearchAssistant')),
                    showCommandHelp: true,
                    showCommandHistory: true,
                    showFieldInfo: false,
                    maxSearchBarLines: 80,
                    minWithForTwoColumns: 560,
                    disableOnSubmit: false
                };
                _.defaults(this.options, defaults);
                
                this.model.searchBar = new SearchBarModel();
                
                this.children.searchField = new SearchField($.extend(true, {}, this.options, {
                    model: {
                        content: this.model.content,
                        searchBar: this.model.searchBar
                    }
                }));
                
                if (this.options.useAssistant) {
                    this.children.searchAssistant = new SearchAssistant($.extend(true, {}, this.options, {
                        model: {
                            searchBar: this.model.searchBar,
                            content: this.model.content,
                            application: this.model.application
                        }
                    }));
                }
                
                this.activate();
            },
            startListening: function() {
                this.listenTo(this.model.searchBar, 'change:assistantOpen', function() {
                    if (this.model.searchBar.get('assistantOpen')) {
                        this.$el.addClass('search-assistant-open');
                    } else {
                        this.$el.removeClass('search-assistant-open');
                    }
                });
            },
            activate: function(options) {
                if (this.active) {
                    return Base.prototype.activate.apply(this, arguments);
                }
                this.model.searchBar.set({search: this.model.content.get("search")});
                return Base.prototype.activate.apply(this, arguments);
            },
            deactivate: function(options) {
                if (!this.active) {
                    return Base.prototype.deactivate.apply(this, arguments);
                }
                Base.prototype.deactivate.apply(this, arguments);
                this.model.searchBar.clear({setDefaults: true});
                return this;
            },
            render: function() {
                if (this.$el.html()) {
                    return this;
                }
                
                var template = _.template(this.template, {});
                this.$el.html(template);
                
                this.children.searchField.render().appendTo(this.$el);

                if (this.options.useAssistant) {
                    this.children.searchAssistant.render().appendTo(this.$el);
                }

                return this;
            },
            template: '\
                <div class="search-field-background"></div>\
            '
        });
    }
);
