define(
    [
        'jquery',
        'underscore',
        'module',
        'views/Base',
        'models/search/SHelper',
        'util/keyboard',
        'util/dom_utils',
        'splunk.util',
        'jquery.bgiframe'
    ],
    function($, _, module, Base, SHelperModel, keyboard_utils, dom_utils, splunkUtils) {
        return Base.extend({
            moduleId: module.id,
            className: 'search-assistant-wrapper',
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);
                this.debouncedFillAssistant = _.debounce(this.fillAssistant, 250).bind(this);
                this.nameSpace = this.uniqueNS();
                this.model.sHelper = new SHelperModel();
                this.windowListenersActive = false;
                
                this.activate({skipRender: true});
            },
            startListening: function() {
                this.listenTo(this.model.searchBar, 'change:assistantCursor', this._highlightKeyword);
                this.listenTo(this.model.searchBar, 'openAssistant', function() {
                    if (!this.model.searchBar.get('assistantOpen')) {
                        if (this.options.autoOpenAssistant) {
                            this.openAssistant();
                        }
                    } else {
                        this.debouncedFillAssistant();
                    }
                });
                this.listenTo(this.model.searchBar, 'closeAssistant', this.closeAssistant);
                this.listenTo(this.model.searchBar, 'downArrow', this.handleDownArrow);
            },
            activate: function(options) {
                options = options || {};
                this.ensureDeactivated();
                
                if (!this.windowListenersActive) {
                    $(window).on('resize.' + this.nameSpace, this.setAssistantWidth.bind(this));
            
                    $(document).on('keyup.' + this.nameSpace, function(e) {
                        if (e.keyCode == keyboard_utils.KEYS['ESCAPE']) {
                            this.closeAssistant();
                        }
                    }.bind(this));
                   
                    $(document).on('click.' + this.nameSpace, function(e) {
                        if ((e.target === this.$el[0]) ||($.contains(this.$el[0], e.target))) {
                            return;
                        }
                        this.closeAssistant();
                    }.bind(this));
                    
                    this.windowListenersActive = true;
                }
                
                if (!options.skipRender) {
                    this.options.autoOpenAssistant = splunkUtils.normalizeBoolean(
                        this.model.content.get('display.prefs.autoOpenSearchAssistant'));
                    
                    this.render();
                }

                return Base.prototype.activate.apply(this, arguments);
            },
            deactivate: function(options) {
                if (!this.active) {
                   return Base.prototype.deactivate.apply(this, arguments);
                }
                Base.prototype.deactivate.apply(this, arguments);
                this.model.sHelper.fetchAbort();
                this.resetFillAssistantFlags();
                this.closeAssistant();
                $(document).off(".assistantResizeActive");
                return this;
            },
            events: {
                'click .search-assistant-autoopen-toggle': function(e) {
                    this.setAutoOpen(!this.options.autoOpenAssistant);
                    e.preventDefault();                    
                },
                'click .search-assistant-activator': function(e) {
                    this.model.searchBar.trigger('focusSearchField');
                    if (this.model.searchBar.get('assistantOpen')) {
                        this.closeAssistant();
                    } else {
                        this.openAssistant();
                    }
                    e.preventDefault();                    
                },
                'keydown a.sakeyword': function(e) {
                    if (!e.metaKey && !e.ctrlKey) {
                        switch (e.keyCode) {
                            case keyboard_utils.KEYS['DOWN_ARROW']:
                                // Left bracket and down arrow register as 40. If the shift key is down, then it must be a bracket.
                                if (!e.shiftKey) {
                                    return this.handleDownArrow(e);
                                }
                                break;
                            case keyboard_utils.KEYS['UP_ARROW']:
                                if (this.model.searchBar.get('assistantOpen') && (this.model.searchBar.get('assistantKeywordCount')>0)) {
                                    this.selectPreviousKeyword();
                                    e.preventDefault();
                                }
                                return;
                            case keyboard_utils.KEYS['TAB']:
                            case keyboard_utils.KEYS['ENTER']:
                            case keyboard_utils.KEYS['RIGHT_ARROW']:
                            case keyboard_utils.KEYS['SPACE_BAR']:
                                return this.onSuggestionSelect(e);
                            default:
                                break;
                        }
                    }
                },
                'keyup a.sakeyword': 'onKeywordKeyUp',
                'keyup .salink': 'onKeywordKeyUp',
                'click .search-assistant-container a': 'onSuggestionSelect',
                'mousedown .search-assistant-resize': 'resizeAssistant',
                //this is a port of the generated javascript from within the shelper mako template
                'click .saMoreLink': function(e) {
                    var $element = $(e.target);
                    if ($element.hasClass('saMoreLinkOpen')) {
                        $element.removeClass('saMoreLinkOpen')
                               .html(_.t('More &raquo;'));
                        $($element.attr('divToShow')).css('display','none');   
                    } else {
                        $element.addClass('saMoreLinkOpen')
                               .html(_.t('&laquo; Less'));
                        $($element.attr('divToShow')).css('display', 'block');
                    }
                }                
            },
            setAutoOpen: function(isEnabled) {
                this.options.autoOpenAssistant = isEnabled;

                if (splunkUtils.normalizeBoolean(this.model.content.get('display.prefs.autoOpenSearchAssistant')) !== isEnabled) {
                    this.model.content.set({'display.prefs.autoOpenSearchAssistant': isEnabled});
                }

                this.$assistantAutoOpenToggle.find("i").toggleClass('icon-check', this.options.autoOpenAssistant);
            },
            handleDownArrow: function(e) {
                var assistantOpen = this.model.searchBar.get('assistantOpen'),
                    assistantKeywordCount = this.model.searchBar.get('assistantKeywordCount'),
                    keywordPosition = this.model.searchBar.get("assistantCursor");
                
                if (e) {
                    e.preventDefault();
                }

                if (assistantOpen && assistantKeywordCount > 0) {
                    if (keywordPosition >= (assistantKeywordCount-1)) {
                        this.model.searchBar.set("assistantCursor", assistantKeywordCount - 1);
                    } else {
                        this.model.searchBar.set("assistantCursor", ++keywordPosition);
                    }
                } else if (!assistantOpen && !this.model.searchBar.get('search')) {
                    this.openAssistant();
                }
            },
            onKeywordKeyUp: function(e) {
                if (!e.metaKey && !e.ctrlKey && !e.shiftKey) {
                    switch (e.keyCode) {
                        case keyboard_utils.KEYS['DOWN_ARROW']:
                        case keyboard_utils.KEYS['UP_ARROW']:
                        case keyboard_utils.KEYS['TAB']:
                        case keyboard_utils.KEYS['ENTER']:
                        case keyboard_utils.KEYS['LEFT_ARROW']:
                        case keyboard_utils.KEYS['RIGHT_ARROW']:
                        case keyboard_utils.KEYS['SPACE_BAR']:
                            e.preventDefault();
                            return;
                        default:
                            break;
                    }
                }
            },
            resizeAssistant: function(e) {
                var startY = e.pageY;
                var startHeight = this.$assistantContainer.height();
                e.preventDefault();
                e.stopPropagation();

                this.$assistantResize.on("click.assistantResizeActive",
                    function(e){
                        e.preventDefault();
                        e.stopPropagation();
                    }.bind(this)
                );

                $(document).on("mousemove.assistantResizeActive",
                    function(e){
                        var newHeight = startHeight - (startY - e.pageY);
                        newHeight = newHeight < 75 ? 0 : newHeight;
                        newHeight = Math.min(newHeight, 500);
                        this.setAssistantHeight(newHeight);
                        e.preventDefault();
                        e.stopPropagation();
                    }.bind(this)
                );

                $(document).on("mouseup.assistantResizeActive",
                    function(e){
                        var newHeight = startHeight - (startY - e.pageY);
                        if (newHeight < 75) {
                            this.closeAssistant();
                            this.setAssistantHeight(startHeight);
                        }
                        $(document).off(".assistantResizeActive");
                    }.bind(this)
                );
            },
            closeAssistant: function() {
                //try to kill any fetches to the search helper
                this.model.sHelper.fetchAbort();
                this.resetFillAssistantFlags();

                // Exit early if closeAssistant has been called before render has created all of the views
                if (!this.$assistantContainer || !this.model.searchBar.get("assistantOpen")) {
                    return;
                }
                
                this.$assistantContainer.hide();
                this.$assistantAutoOpenToggle.hide();
                this.$assistantActivator.addClass("icon-triangle-down-small").removeClass("icon-triangle-up-small");
                this.$assistantResize.removeClass("search-assistant-resize-active");
                this.$el.removeClass('open');
                
                this.model.searchBar.set('assistantOpen', false);
            },
            openAssistant: function() {
                if (!this.model.searchBar.get("assistantOpen")) {                
                    this.$assistantActivator.addClass("icon-triangle-up-small").removeClass("icon-triangle-down-small");
                    this.$assistantResize.addClass("search-assistant-resize-active");
                    this.$el.addClass('open');
                    this.model.searchBar.set("assistantOpen", true);
                    this.fillAssistant();
                }
            },
            resetFillAssistantFlags: function() {
                this.assistantFillPending = false;
                this.assistantNeedsUpdate = false;
            },
            fillAssistant: function() {
                if (this.model.searchBar.get("assistantOpen")){
                    if (!this.assistantFillPending) {
                        var searchString = splunkUtils.addLeadingSearchCommand(this.model.searchBar.get('search') || '*', true);

                        this.assistantFillPending = true;
                        this.model.sHelper.safeFetch({
                            data: {
                                'snippet': 'true',
                                'snippetEmbedJS': 'false',
                                'namespace': this.model.application.get('app') || 'search',
                                'search': searchString,
                                'useTypeahead': this.options.useTypeahead,
                                'useAssistant': this.options.useAssistant,
                                'showCommandHelp': this.options.showCommandHelp,
                                'showCommandHistory': this.options.showCommandHistory,
                                'showFieldInfo': this.options.showFieldInfo
                            },
                            success: function() {
                                this.$assistantContainer.html(this.model.sHelper.get('raw') || '');
                                this.assistantFillPending = false;
                                this.fillAssistantCompleteCallback();
                            }.bind(this)
                        });
                    } else {
                        this.assistantNeedsUpdate = true;
                    }
                }
            },
            fillAssistantCompleteCallback: function() {
                if (this.model.searchBar.get('assistantOpen')) {
                    if (this.assistantNeedsUpdate) {
                        this.assistantNeedsUpdate = false;
                        this.fillAssistant();
                    }

                    this.setAssistantWidth();

                    this.$assistantContainer.show().bgiframe().scrollTop(0);
                    this.$assistantAutoOpenToggle.show();
                    this.setAssistantHeight(this.$assistantContainer.height() || 250);

                    this.model.searchBar.set({
                        assistantKeywordCount: this.$('.sakeyword').length,
                        assistantCursor: -1
                    });

                    this.model.searchBar.trigger('searchFieldfocus');
                }
            },
            setAssistantHeight: function(newHeight) {
                if (newHeight > 500) {
                    newHeight = 500; // make sure we don't go over 500px
                }
                this.$assistantContainer.height(newHeight);
                this.$('.saHelpWrapper').css('min-height', newHeight);
            },
            setAssistantWidth: function() {
                var assistantInner = this.$('.assistantInner');
                if (assistantInner.length && (this.$el.width() < this.options.minWithForTwoColumns)) {
                    assistantInner.addClass('assistant-inner-narrow');
                } else {
                    assistantInner.removeClass('assistant-inner-narrow');
                }
           },
            _highlightKeyword: function() {
                if (this.model.searchBar.get('assistantOpen')) {
                    var keywordPosition = this.model.searchBar.get("assistantCursor");
                
                    // set the CSS style for selected
                    var el = $('.sakeyword', this.$assistantContainer
                        ).removeClass('saKeywordSelected'
                        ).slice(keywordPosition, keywordPosition + 1
                        ).addClass('saKeywordSelected');

                    if (el.length) {
                        // keep selected item in view
                        var win = this.$assistantContainer;
                        var visibleWindowTop = win.scrollTop();
                        var visibleWindowBottom = win.scrollTop() + win.height();
                        var elementTop = el.position().top + visibleWindowTop;

                        var elementHeight = el.outerHeight();
                        if (elementTop < visibleWindowTop) {
                            win.scrollTop(elementTop);
                        } else if (elementTop + elementHeight > visibleWindowBottom) {
                            win.scrollTop(elementTop + elementHeight - win.height());
                        }

                        el.focus();
                    }
                }
            },
            selectPreviousKeyword: function() {
                var keywordPosition = this.model.searchBar.get("assistantCursor");
                if (keywordPosition <= 0) {
                    this.model.searchBar.set("assistantCursor", -1);
                    this.model.searchBar.trigger('searchFieldfocus');
                } else {
                    this.model.searchBar.set("assistantCursor", --keywordPosition);
                }
            },
            onSuggestionSelect: function(e) {
                var newval = $.trim($(e.currentTarget).attr('replacement'));

                if (this.model.searchBar.get('assistantOpen') && newval) {

                    if (newval.substr(-1) != '=') {
                           newval += ' '; // don't add space after =
                    }
                    this.model.searchBar.set('search', newval);
                    this.fillAssistant();
                    this.model.searchBar.trigger('setCaretPositionToEnd');

                    e.preventDefault();
                }
            },
            remove: function() {
                $(window).off('resize.' + this.nameSpace);
                $(document).off('keyup.' + this.nameSpace);
                $(document).off('click.' + this.nameSpace);
                $(document).off(".assistantResizeActive");
                return Base.prototype.remove.apply(this, arguments);
            },
            render: function() {
                if (this.$el.html()) {
                    return this;
                }
                
                var template = _.template(this.template, {});
                this.$el.html(template);
                
                // Setup shortcuts
                this.$assistantWrapper = this.$el;
                this.$assistantContainer = this.$('.search-assistant-container');
                this.$assistantAutoOpenToggle = this.$('.search-assistant-autoopen-toggle');
                this.$assistantActivator = this.$('.search-assistant-activator');
                this.$assistantResize = this.$('.search-assistant-resize');

                this.setAutoOpen(this.options.autoOpenAssistant);
                
                return this;
            },
            template: '\
                <div class="search-assistant-container-wrapper"><div class="search-assistant-container"></div></div>\
                <a class="search-assistant-autoopen-toggle" href="" style="display:none;"><i></i><%= _("Auto Open").t() %></a>\
                   <div class="search-assistant-resize"></div>\
                <a href="#" class="search-assistant-activator icon-triangle-down-small"></a>\
            '
        });
    }
);
