define([
    'jquery',
    'underscore',
    'backbone',
    'module',
    'splunk.util',
    'util/string_utils',
    'views/Base',
    'views/shared/delegates/Popdown',
    'views/shared/splunkbar/SystemMenuSection',
    'views/shared/splunkbar/SystemMenuModal',
    'uri/route',
    'contrib/text!views/shared/splunkbar/SystemMenu.html'
],
function(
    $,
    _,
    Backbone,
    module,
    splunk_util,
    string_utils,
    BaseView,
    Popdown,
    SystemMenuSection,
    SystemMenuModal,
    route,
    systemMenuTemplate
){
    return BaseView.extend({
        moduleId: module.id,
        template: systemMenuTemplate,
        tagName: 'li',
        className: 'dropdown menu-system',
        initialize: function() {
            var self = this;
            BaseView.prototype.initialize.apply(this, arguments);
            self.debouncedRender();
            this.collection.sections.on('ready', function(){
                self.debouncedRender();
            }, this);
        },
        events: {
            'click .show-all-sections': 'showAllSections'
        },
        render: function() {
            var root = this.model.application.get('root');
            var locale = this.model.application.get('locale');
            var app = this.model.application.get('app');
            var managementConsoleApp = this.collection.apps.findByEntryName('splunk_management_console');
            var managementConsoleAvailable = (managementConsoleApp && !managementConsoleApp.entry.content.get("disabled"));
            var html = this.compiledTemplate({
                userCanAddData:
                    this.model.user.canAddData() &&
                    this.collection.managers.findByEntryName('adddata'),
                addDataURL: route.addData(root, locale, app),
                userCanExploreData: this.model.user.canExploreData(),
                exploreDataURL: route.exploreData(root, locale, app),
                managementConsoleAvailable: managementConsoleAvailable,
                managementConsoleURL: route.managementConsole(root, locale)
            });
            var $html = $(html);
            var $menu = $html.find('.popdown-dialog-body');
                
            this.addSections($menu);
            if(this.sectionCount < 2) {
                $menu.parent().addClass('mega-menu-narrow');
            }

            var $footer = $html.find('.popdown-dialog-footer');
            if (this.collection.managers.canShowMore()) {
                $footer.show();
            } else {
                $footer.hide();
            }

            this.$el.html($html);
            
            var popup = this.$el.find('#global-system-menu');
            this.children.popdown = new Popdown({el:popup.parent(), mode: 'dialog'});

            return this;
        },
        addSections: function($menu){
            var self = this;
            this.sectionCount = 0;
            this.collection.sections.each(function(section){
                if (section.get('items') && section.get('items').length === 0) {
                    return;
                }
                var sectionView = self.children[section.get('id')] = new SystemMenuSection({
                    model: section
                });
                
                self.sectionCount++;
                $menu.append(sectionView.render().el);
            });
        },
        showAllSections: function(evt) {
            this.children.dialog = new SystemMenuModal({
                collection: {
                    managers: this.collection.managers,
                    sections: this.collection.sections
                },
                model: {
                    application: this.model.application
                },
                onHiddenRemove: true
            });
            this.children.popdown.hide();
            this.modalWrapper = $('<div class="splunk-components"></div>').appendTo($("body"));
            this.modalWrapper.append(this.children.dialog.render().el);
            this.children.dialog.show();
        }
    });
});
