define([
    'jquery',
    'underscore',
    'module',
    'models/Base',
    'views/shared/splunkbar/find/results/Master',
    'views/shared/splunkbar/find/FindProgress',
    'views/shared/tablecaption/Input',
    'collections/shared/Dashboards',
    'collections/search/Reports',
    'collections/services/datamodel/DataModels',
    'views/Base',
    'util/splunkd_utils'
],
function(
    $,
    _,
    module,
    BaseModel,
    FindBarResults,
    FindProgress,
    InputView,
    DashboardsCollection,
    ReportsCollection,
    DataModelsCollection,
    BaseView,
    splunkdUtils
) {
    return BaseView.extend({
        moduleId: module.id,
        tagName: 'li',
        initialize: function() {
            BaseView.prototype.initialize.apply(this, arguments);
            //models
            this.stateModel = new BaseModel();
            this.rawSearch = new BaseModel();
            //collections
            this.collection.dashboards = new DashboardsCollection();
            this.collection.reports = new ReportsCollection();
            this.collection.alerts = new ReportsCollection();
            this.collection.datamodels = new DataModelsCollection();
            //deferreds
            this.deferreds = {};
            this.deferreds.dashboardDeferred = $.Deferred();
            this.deferreds.reportDeferred = $.Deferred();
            this.deferreds.alertDeferred = $.Deferred();
            this.deferreds.datamodelDeferred = $.Deferred();
            
            this.children.findProgress = new FindProgress({
                model: {
                    application: this.model.application,
                    state: this.stateModel
                },
                mode: 'menu',
                onHiddenRemove: true
            });

            this.stateModel.on('change:search',  
                _.debounce(function() {
                    var rawSearch = this.rawSearch.get('rawSearch');
                    if (!rawSearch || rawSearch === '') {
                        if(this.children.findbarResults) {
                            this.children.findbarResults.hide();   
                        }
                        this.collection.dashboards = new DashboardsCollection();
                        this.collection.reports = new ReportsCollection();
                        this.collection.alerts = new ReportsCollection();
                        this.collection.datamodels = new DataModelsCollection();
                       return;
                    }
                    var $target =  $('.findbar-container'),
                        $focus = $('.findbar-container .search-query');
                    
                    // Show find progress while waiting for results to return.
                    if(this.children.findbarResults) {
                        this.children.findbarResults.hide();   
                    }
                    $($target).append(this.children.findProgress.render().el);
                    this.children.findProgress.show($target, {$onOpenFocus: $focus});

                    this.deferreds.dashboardDeferred = $.Deferred();
                    this.deferreds.reportDeferred = $.Deferred();
                    this.deferreds.alertDeferred = $.Deferred();
                    this.deferreds.datamodelDeferred = $.Deferred();
                    this.fetchDashboardCollection();
                    this.fetchReportCollection();
                    this.fetchAlertCollection();
                    this.fetchDataModelCollection();
                    
                    this.collection.dashboards.on('sync', function() {
                            this.deferreds.dashboardDeferred.resolve();
                    }.bind(this));
                    
                    this.collection.reports.on('sync', function() {
                        this.deferreds.reportDeferred.resolve();
                    }.bind(this));
            
                    this.collection.alerts.on('sync', function() {
                        this.deferreds.alertDeferred.resolve();
                    }.bind(this));
            
                    this.collection.datamodels.on('sync', function() {
                        this.deferreds.datamodelDeferred.resolve();
                    }.bind(this));
                    
                    $.when(
                        this.deferreds.dashboardDeferred,
                        this.deferreds.reportDeferred,
                        this.deferreds.alertDeferred,
                        this.deferreds.datamodelDeferred
                    ).then(function() {
                        this.showFindBarResults($target);
                    }.bind(this));
                }.bind(this), 250),
                this
            );
        },
        events: {
            'click .findbar-container': function(e){
                var rawSearch = this.rawSearch.get('rawSearch');
                if (rawSearch && rawSearch !== '') {
                    var $target = $(e.currentTarget);
                    this.showFindBarResults($target);
                }
            }
        },
        showFindBarResults: function($target) {
            var $focus = $('.findbar-container .search-query');
            this.children.findProgress.hide();
            if (this.children.findbarResults && this.children.findbarResults.shown) {
                this.children.findbarResults.render();
                return;
            }
            this.children.findbarResults = new FindBarResults({
                collection: {
                    dashboards: this.collection.dashboards,
                    reports: this.collection.reports,
                    alerts: this.collection.alerts,
                    datamodels: this.collection.datamodels,
                    apps: this.collection.apps
                },
                model: {
                    application: this.model.application,
                    state: this.stateModel,
                    rawSearch: this.rawSearch
                },
                mode: 'menu',
                onHiddenRemove: true
            });
            $($target).append(this.children.findbarResults.render().el);
            this.children.findbarResults.show($target, {$onOpenFocus: $focus});
        },
        fetchDashboardCollection: function() { 
            var dashboards = this.collection.dashboards,
                search = '';
            if (this.rawSearch.get('rawSearch')) {
                search = splunkdUtils.createSearchFilterString(this.rawSearch.get('rawSearch'), ['label','eai:data']);
                
            }
            if (search) {
                search += ' AND ';
            }
            search += DashboardsCollection.availableWithUserWildCardSearchString(this.model.application.get('owner')); 

            dashboards.safeFetch({ 
                data : { 
                    app: '-', 
                    owner: '-', 
                    search: search, 
                    count: 3,
                    offset: 0
                }, 
                success: function(response) {
                }.bind(this) 
            }); 
        },
        fetchReportCollection: function() {
            var reports = this.collection.reports,
                search = this.stateModel.get('search') || '';
            if (search) {
                search += ' AND ';
            }
            search += ReportsCollection.availableWithUserWildCardSearchString(this.model.application.get('owner')) + ' AND is_visible=1 AND NOT ' + ReportsCollection.ALERT_SEARCH_STRING;

            reports.safeFetch({
                data : {
                    app: '-',
                    owner: '-',
                    search: search,
                    count: 3
                },
                success: function() {
                }.bind(this)
            });
        },
        fetchAlertCollection: function() {
            if (!this.model.serverInfo.entry.content.get("isFree")) {
                var alerts = this.collection.alerts,
                    search = this.stateModel.get('search') || '';
                if (search) {
                    search += ' AND ';
                }
                search += ReportsCollection.availableWithUserWildCardSearchString(this.model.application.get('owner')) + ' AND is_visible=1 AND ' + ReportsCollection.ALERT_SEARCH_STRING;

                alerts.safeFetch({
                    data:{
                        app: '-',
                        owner: '-',
                        search: search,
                        count: 3
                    },
                    success: function() {
                    }.bind(this)
                });
            } else {
                this.deferreds.alertDeferred.resolve();
            }
        },
        fetchDataModelCollection: function() { 
            if (!this.model.serverInfo.isLite()) {
                var datamodels = this.collection.datamodels, 
                    search = this.rawSearch.get('rawSearch') || ''; 
                if (search) {
                    search += ' AND ';
                }
                search += '(eai:acl.owner="*")';

                datamodels.safeFetch({
                    data : {
                        app: '-',
                        owner: '-',
                        sort_mode: ['auto', 'auto'],
                        search: search,
                        count: 3,
                        offset: 0,
                        concise: true
                    }, 
                    success: function(response) {
                    }.bind(this) 
                }); 
            } else {
                this.deferreds.datamodelDeferred.resolve();
            }
        },
        render: function() {
            var html = this.compiledTemplate();
            this.$el.append(html);
            this.children.findbar = new InputView({
                model: {
                    state: this.stateModel,
                    rawSearch: this.rawSearch
                },
                findBar: true
            });
            this.$('.findbar-container').append(this.children.findbar.render().el);
        },
        template: '\
        <div class="findbar-container"></div>\
        '
    });
});
