define(
[
    'underscore',
    'jquery',
    'module',
    'views/shared/PopTart',
    'views/shared/splunkbar/find/results/DashboardResults',
    'views/shared/splunkbar/find/results/ReportResults',
    'views/shared/splunkbar/find/results/AlertResults',
    'views/shared/splunkbar/find/results/DataModelResults',
    'uri/route',
    'splunk.util'
],
function(
    _,
    $,
    module,
    PopTartView,
    DashboardResults,
    ReportResults,
    AlertResults,
    DataModelResults,
    route,
    splunkUtils
){
    return PopTartView.extend({
        moduleId: module.id,
        className: 'dropdown-menu',
            /**
             * @param {Object} options {
             *     collection: {
             *         dashboards: <collections.shared.Dashboards>,
             *         reports: <collections.search.Reports>,
             *         alerts: <collections.search.Reports>,
             *         datamodels: <collections.services.datamodel.DataModels>,
             *         apps: <collections.services.appLocal>
             *     model: {
             *         state: <models.State>,
             *         application: <models.shared.Application>,
             *         rawSearch: <models.Base>
             *     },
             *     mode: <string>,
             *     onHiddenRemove: <Boolean>
             * }
             */
        initialize: function() {
            PopTartView.prototype.initialize.apply(this, arguments);
        },
        render: function() {
            this.el.innerHTML = PopTartView.prototype.template_menu;
            
            if (!this.collection.reports.size() &&
                !this.collection.dashboards.size() &&
                !this.collection.alerts.size() &&
                !this.collection.datamodels.size()) {
                this.$el.append('<div class="no-results">' + _("No Results for this search.").t() + '</div>');
            }
            var currentApp = this.model.application.get('app'),
                alternateApp = currentApp !== 'system' ? currentApp : 'search';
            var searchApp = _.find(this.collection.apps.models, function(app) {
                return app.entry.get('name') === 'search';
            });
            if (alternateApp === 'search' && searchApp && searchApp.entry.content.get("disabled")) {
                this.collection.apps.sortWithString(this.model.userPref.entry.content.get('appOrder'));
                alternateApp = this.collection.apps.models[0].entry.get('name');
            }
            
            // Report Results
            if (this.collection.reports.size()) {
                this.children.reportResults = new ReportResults({
                    collection: {
                        reports: this.collection.reports,
                        apps: this.collection.apps
                    },
                    model: {
                        application: this.model.application,
                        state: this.model.state,
                        rawSearch: this.model.rawSearch
                    },
                    alternateApp: alternateApp
                });
                this.$el.append(this.children.reportResults.render().el);
            }
            
            // Dashboard Results
            if ( this.collection.dashboards.size()) {
                this.children.dashboardResults = new DashboardResults({
                    collection: {
                        dashboards: this.collection.dashboards,
                        apps: this.collection.apps
                    },
                    model: {
                        application: this.model.application,
                        state: this.model.state,
                        rawSearch: this.model.rawSearch
                    },
                    alternateApp: alternateApp
                });
                this.$el.append(this.children.dashboardResults.render().el);
            }
                            
            // Alert Results
            if (this.collection.alerts.size()) {
                this.children.alertResults = new AlertResults({
                    collection: {
                        alerts: this.collection.alerts,
                        apps: this.collection.apps
                    },
                    model: {
                        application: this.model.application,
                        state: this.model.state,
                        rawSearch: this.model.rawSearch
                    },
                    alternateApp: alternateApp
                });
                this.$el.append(this.children.alertResults.render().el);
            }
            
            // Data Model Results
            if (this.collection.datamodels.size()) {
                this.children.dataModelResults = new DataModelResults({
                    collection: {
                        datamodels: this.collection.datamodels,
                        apps: this.collection.apps
                    },
                    model: {
                        application: this.model.application,
                        state: this.model.state,
                        rawSearch: this.model.rawSearch
                    },
                    alternateApp: alternateApp
                });
                this.$el.append(this.children.dataModelResults.render().el);
            }
            
            // Open "keyword" in new search
            var searchLink = route.search(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    'search',
                    {data: {q: this.model.rawSearch.get('rawSearch')}}
                );

            var html = this.compiledTemplate({
                _: _,
                splunkUtils: splunkUtils,
                searchLink: searchLink,
                rawSearch: this.model.rawSearch.get('rawSearch')
            });
            this.$el.append(html);

            return this;
        },
        template: '\
        <ul><li>\
            <a class="primary-link search-link" href="<%- searchLink %>">\
                <i class="icon-search-thin findbar-results-icon"></i>\
                <%- splunkUtils.sprintf(_("Open %s in search").t(), rawSearch) %>\
            </a>\
            <a class="secondary-link search-link" href="<%- searchLink %>" target="_blank">\
                <i class="icon-external"></i>\
            </a>\
        </li></ul>\
        '
    });
});