// splunk bar
define(
[
    'jquery',
    'underscore',
    'backbone',
    'module',
    'views/Base',
    'views/shared/litebar/RestartDialog',
    'splunk.util',
    'util/time'
],
function(
    $,
    _,
    Backbone,
    module,
    BaseView,
    RestartDialog,
    splunk_util,
    time_utils
){
    /**
     * View Hierarchy:
     *
     * Messages
     */
    return BaseView.extend({
        moduleId: module.id,

        messageMap: {
            'restart_required': _('Splunk must be restarted for changes to take effect. [[/manager/search/control|Click here to restart from the Manager]].').t()
        },

        messageMapForLite: {
            'restart_required': _('Restart for changes to take effect. [[/manager/search/control|Click here to restart]].').t()
        },

        tagName: 'li',

        isLiteRestart : false,

        initialize: function() {
            BaseView.prototype.initialize.apply(this, arguments);

            //listen for restart events
            $(document).on('restart_failed', function() {
                this.hideRestartModal();
            }.bind(this));

        },

        events: {
            'click .message-content a' : 'checkRestartForLite'     
        },

        checkRestartForLite: function(e) {
            if (this.isLiteRestart) {
                e.preventDefault();
                if (confirm(_("Are you sure you want to restart Splunk?").t())) {
                    this.showRestartModal();
                    splunk_util.restart_server();
                }
            }
        },

        showRestartModal: function() {
            this.children.restartDialog = new RestartDialog({backdrop: 'static'});
            // Necessary for modal to render correctly in manager.
            $("body").append('<div class="splunk-components restartModal"></div>');
            $(".restartModal").append(this.children.restartDialog.render().el);
            this.children.restartDialog.show();
        },
 
        hideRestartModal: function() {
            if (this.children.restartDialog) {
                this.children.restartDialog.hide();
            }
        },

        render: function() {
            var msgFullId = this.model.id,
                msgId = (this.model.entry.get('name')||''),
                msg,
                msgLevel = this.model.entry.content.get('severity') || 'warn',
                contactToken = '$CONTACT_SPLUNK_SALES_TEXT$',
                contactStr = '[http://www.splunk.com/r/getlicense Contact Splunk] (sales@splunk.com or +1.866.GET.SPLUNK) for details on how to renew.',
                contactStrLite = 'To manage your license or to learn how to renew your license go the licensing management page.';

            var isLite = false;
            if (this.options.serverInfo && this.options.serverInfo.isLite()) {
                isLite = true;
            }

            var messageMap = isLite ? this.messageMapForLite : this.messageMap;
            if (msgId && messageMap[msgId]) {
                msg = messageMap[msgId]; 
            } else {
                msg = this.model.entry.content.get("message") || "";
            }

            if (msg.indexOf(contactToken) != -1) {
                msg = msg.replace(contactToken, (isLite) ? contactStrLite : contactStr);
            }

            var msgTime = this.model.entry.content.get("timeCreated_iso");
            if(msgTime){
                var dateObj = time_utils.isoToDateObject(msgTime);
                msgTime = dateObj.toLocaleString();
            }

            var formattedMsg = splunk_util.getWikiTransform(msg);
            if ((msgId == 'restart_required') && isLite) {
                this.isLiteRestart = true;
            }                                                                                             

            var html = this.compiledTemplate({
                msg: formattedMsg,
                msgLevel: msgLevel,
                msgTime: msgTime || ''
            });

            this.$el.html(html);
            this.$el.addClass(msgLevel);
            this.$el.attr('data-id', msgFullId);

            return this;
        },
        template: '<i class="message-icon icon-<%- msgLevel %>"></i>\
                <span class="message-content"><%= msg %></span>\
                <span class="message-time"><%= msgTime %></span>\
                <a href="#" class="delete-message"><i class="icon-x-circle"></i></a>'
    });
});
