/**
 * @author sfishel
 *
 * A container view for statistics table formatting controls.
 *
 * Child Views:
 *
 * count <views/shared/SyntheticSelectControl> a drop-down control for the rows-per-page
 * drillDown <views/shared/controls/ControlGroup> a radio-style control group for specifying drilldown configuration
 * rowNumbers <views/shared/controls/ControlGroup> a radio-style control group for toggling row numbering on and off
 * wrapResults <views/shared/controls/ControlGroup> a radio-style control group for toggling wrapping on and off
 * dataOverlay <views/shared/controls/ControlGroup> a drop-down-style control group for specifying data overlay options
 */

define([
            'underscore',
            'module',
            'views/Base',
            'views/shared/controls/ControlGroup',
            'views/shared/controls/SyntheticSelectControl',
            'views/shared/delegates/Popdown'
        ],
        function(
            _,
            module,
            BaseView,
            ControlGroup,
            SyntheticSelectControl,
            Popdown
        ) {

    return BaseView.extend({

        className: 'btn-group popdown pull-left',
        moduleId: module.id,

        /**
         * @constructor
         * @param options {Object} {
         *     model <models/Setting> the setting model to hold the current formatting settings
         * }
         */

        initialize: function(options) {
            BaseView.prototype.initialize.call(this, options);

            //child views
            this.children.count = new SyntheticSelectControl({
                menuWidth: "narrow",
                className: "btn-group pull-left",
                items: [
                    {value: 10, label: _("10 per page").t()},
                    {value: 20, label: _("20 per page").t()},
                    {value: 50, label: _("50 per page").t()}
                ],
                model: this.model,
                modelAttribute: 'display.prefs.statistics.count',
                toggleClassName: 'btn-pill',
                popdownOptions: {
                    attachDialogTo: 'body'
                }
            });

            this.children.drillDown = new ControlGroup({
                label: _("Drilldown").t(),
                controlClass: 'controls-thirdblock',
                controlType: 'SyntheticRadio',
                controlOptions: {
                    className: "btn-group",
                    items: [
                        {value: "row", label: _("Row").t()},
                        {value: "cell", label: _("Cell").t()},
                        {value: "none", label: _("None").t()}
                    ],
                    model: this.model,
                    modelAttribute: 'display.statistics.drilldown'
                }
            });

            this.children.rowNumbers = new ControlGroup({
                label: _("Row Numbers").t(),
                controlType: 'SyntheticRadio',
                controlClass: 'controls-halfblock',
                controlOptions: {
                    className: "btn-group",
                    items: [
                        {value: "1", label: _("Yes").t()},
                        {value: "0", label: _("No").t()}
                    ],
                    model: this.model,
                    modelAttribute: 'display.statistics.rowNumbers'
                }
            });

            this.children.wrapResults = new ControlGroup({
                label: _("Wrap Results").t(),
                controlType:'SyntheticRadio',
                controlClass: 'controls-halfblock',
                controlOptions: {
                    className: "btn-group",
                    items: [
                        {value:"1", label: _("Yes").t()},
                        {value:"0", label: _("No").t()}
                    ],
                    model: this.model,
                    modelAttribute: 'display.statistics.wrap'
                }
            });

            this.children.dataOverlay = new ControlGroup({
                label: _("Data Overlay").t(),
                controlType:'SyntheticSelect',
                controlClass: 'controls-block',
                controlOptions: {
                    model: this.model,
                    menuWidth: "narrow",
                    items: [
                        {value: 'none', label: _('None').t()},
                        {value: 'heatmap', label: _('Heat map').t()},
                        {value: 'highlow', label: _('High and low values').t()}
                    ],
                    modelAttribute: 'display.statistics.overlay',
                    toggleClassName: "btn"
                }
            });

        },

        render: function() {
            this.$el.html(_(this.template).template({}));
            this.children.popdown = new Popdown({ el: this.el, mode: 'dialog', attachDialogTo: 'body' });

            this.$el.prepend(this.children.count.render().el);

            this.$('.popdown-dialog-body').append(this.children.wrapResults.render().el);
            this.$('.popdown-dialog-body').append(this.children.rowNumbers.render().el);
            this.$('.popdown-dialog-body').append(this.children.drillDown.render().el);
            this.$('.popdown-dialog-body').append(this.children.dataOverlay.render().el);

            return this;
        },

        template: '\
                <a class="dropdown-toggle btn-pill" href="#"><%- _("Format").t() %><span class="caret"></span></a>\
                <div class="popdown-dialog dialog-format dialog-format-statistics">\
                    <div class="arrow"></div>\
                    <div class="popdown-dialog-body popdown-dialog-padded form form-horizontal form-format"></div>\
                </div>\
        '

    });

});
