define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/shared/WaitSpinner',
        'splunk.util'
    ],
    function(
        _,
        module,
        BaseView,
        WaitSpinner,
        splunkUtil
    )
    {
        return BaseView.extend({
            moduleId: module.id,
            tagName: 'h3',
            /**
             * @param {Object} options {
             *     countLabel: (Optional) <String> The count vanity label
             *     collection: <collections.services.SavedSearches>
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                this.collection.on('reset destroy', this.debouncedRender, this);
                this.model.on('change:fetching', this.debouncedRender, this);
            },
            render: function() {
                !this.children.spinner || this.children.spinner.$el.detach();
                var collectionTotal = 0;
                if(this.collection.models[0] && this.collection.models[0].paging){
                    collectionTotal = this.collection.models[0].paging.get('total');
                }
                var html = this.compiledTemplate({
                    countLabel: this.options.countLabel,
                    collectionStatus: this.model.get("fetching") ? _("Loading").t() : collectionTotal
                });
                this.$el.html(html);

                this.children.spinner = this.children.spinner || new WaitSpinner();
                this.children.spinner.prependTo(this.$el);
                this.children.spinner.$el[this.model.get("fetching") ? 'show' : 'hide']();
                this.children.spinner[this.model.get("fetching") ? 'start' : 'stop']();

                return this;
            },
            template: '<%- collectionStatus %> <%- countLabel || "" %>'
        });
    }
);
