define(
        [
            "jquery",
            "underscore",
            "backbone",
            "module",
            "models/shared/DateInput",
            "models/shared/TimeRange",
            "views/Base",
            "views/shared/timerangepicker/dialog/dateandtimerange/timeinput/Master",
            "views/shared/FlashMessages",
            "util/time"
        ],
        function($, _, Backbone, module, DateInputModel, TimeRangeModel, Base, TimeInput, FlashMessages, time_utils) {
            return Base.extend({
                className: 'accordion-inner form dateandtime-container',
                moduleId: module.id,
                initialize: function() {
                    Base.prototype.initialize.apply(this, arguments);
            
                    this.model = {
                        timeRange: this.model,
                        workingRange: new TimeRangeModel(),
                        earliestDateInput: new DateInputModel(),
                        latestDateInput: new DateInputModel()
                    };
                    this.syncInternals();
                        
                    this.label = _("Date & Time Range").t();
                    this.earliest_errored = false;
                    this.latest_errored = false;
                    
                    //views
                    this.children.earliestTimeInput = new TimeInput({
                        label: _("Earliest").t(),
                        inputClassName: 'earliest',
                        model: {
                            dateTime: this.model.earliestDateInput
                        }
                    });
            
                    this.children.latestTimeInput = new TimeInput({
                        label: _("Latest").t(),
                        inputClassName: 'latest',
                        model: {
                            dateTime: this.model.latestDateInput
                        }
                    });
                    
                    this.children.flashMessages = new FlashMessages({
                        model: {
                            workingRange: this.model.workingRange,
                            earliestDateInput: this.model.earliestDateInput,
                            latestDateInput: this.model.latestDateInput
                        }
                    });
                    
                    this.activate();
                },
                startListening: function() {
                    this.listenTo(this.model.workingRange, "sync error", this.enableApply);
                    
                    this.listenTo(this.model.earliestDateInput, "validated:invalid", function(){
                        this.earliest_errored = true;
                        this.disableApply();
                    });

                    this.listenTo(this.model.earliestDateInput, "validated:valid", function(){
                        this.earliest_errored = false;
                        this.enableApply();
                    });

                    this.listenTo(this.model.latestDateInput, "validated:invalid", function(){
                        this.latest_errored = true;
                        this.disableApply();
                    });

                    this.listenTo(this.model.latestDateInput, "validated:valid", function(){
                        this.latest_errored = false;
                        this.enableApply();
                    });

                    this.listenTo(this.model.earliestDateInput, "change", function(){
                        this.model.workingRange.trigger("validated", true, this.model.workingRange, {});
                    });

                    this.listenTo(this.model.latestDateInput, "change", function(){
                        this.model.workingRange.trigger("validated", true, this.model.workingRange, {});
                    });

                    this.listenTo(this.model.timeRange, 'change:earliest_iso change:latest_iso prepopulate', _.debounce(this.prepopulate, 0));
                },
                syncInternals: function() {
                    this.yesterday = new Date();
                    this.yesterday.setDate(this.yesterday.getDate() - 1);
                    this.yesterday.setHours(12, 0, 0, 0);
                    this.model.earliestDateInput.setFromJSDate(this.yesterday, {silent:true});
                                        
                    this.today = new Date();
                    this.today.setHours(12, 0, 0, 0);
                    this.model.latestDateInput.setFromJSDate(this.today, {silent:true});
                },
                activate: function(options) {
                    if (this.active) {
                        return Base.prototype.activate.apply(this, arguments);
                    }
                    this.syncInternals();
                    return Base.prototype.activate.apply(this, arguments);
                },
                deactivate: function(options) {
                    if (!this.active) {
                        return Base.prototype.deactivate.apply(this, arguments);
                    }
                    Base.prototype.deactivate.apply(this, arguments);
                    
                    this.model.workingRange.fetchAbort();
                    this.model.workingRange.clear({setDefaults: true});
                    this.model.earliestDateInput.clear({setDefaults: true});
                    this.model.latestDateInput.clear({setDefaults: true});
                    
                    return this;
                },
                remove: function() {
                    Base.prototype.remove.apply(this, arguments);
                    this.model.workingRange.deepOff();
                    this.model.workingRange.fetchAbort();
                },
                prepopulate: function() {
                    var earliest = this.model.timeRange.get('earliest_iso'),
                        latest =  this.model.timeRange.get('latest_iso'),
                        earliest_date = this.model.timeRange.get('earliest_date'),
                        latest_date = this.model.timeRange.get('latest_date');
                                        
                    if (earliest_date) {
                        this.model.earliestDateInput.setFromJSDate(earliest_date, {validate: true});
                    }
                    if (latest_date) {
                        this.model.latestDateInput.setFromJSDate(latest_date, {validate: true});
                    }
                },
                supportsRange: function() {
                    return time_utils.generateDateTimeRangeLabel(this.model.timeRange.get('earliest'), this.model.timeRange.get('latest'));
                },
                events: {
                    "click .apply": function(event) {
                        if ($(event.currentTarget).hasClass("disabled")) {
                            event.preventDefault();
                            return;
                        }
                    
                        if (!this.earliest_errored && !this.latest_errored){
                            var that = this,
                            earliestIsoWithoutTZ = this.model.earliestDateInput.isoWithoutTZ(),
                            latestIsoWithoutTZ = this.model.latestDateInput.isoWithoutTZ();

                            this.disableApply();
                            this.model.workingRange.save(
                                    {
                                        earliest: earliestIsoWithoutTZ,
                                        latest: latestIsoWithoutTZ
                                    },
                                    {
                                        success: function(model) {
                                            that.enableApply();
                                            that.model.timeRange.set({
                                                'earliest': model.get('earliest_epoch'),
                                                'latest': model.get('latest_epoch'),
                                                'earliest_epoch': model.get('earliest_epoch'),
                                                'latest_epoch': model.get('latest_epoch'),
                                                'earliest_iso': model.get('earliest_iso'),
                                                'latest_iso': model.get('latest_iso'),
                                                'earliest_date': new Date(model.get('earliest_date').getTime()),
                                                'latest_date': new Date(model.get('latest_date').getTime())
                                            });
                                            that.model.timeRange.trigger("applied");
                                        },
                                        error: function(model, error) {
                                            that.enableApply();
                                        }
                                    }
                            );
                        }
                        event.preventDefault();
                    }
                },
                enableApply: function() {
                    this.$("#apply_" + this.cid).removeClass('disabled');
                },
                disableApply: function() {
                    this.$("#apply_" + this.cid).addClass('disabled');
                },
                render: function() {
                    var template = _.template(this.template, {
                        _: _,
                        cid: this.cid
                    });
                    this.$el.html(template);

                    this.children.flashMessages.render().insertBefore(this.$(".apply"));
                    this.children.earliestTimeInput.render().insertBefore(this.$(".apply"));
                    this.children.latestTimeInput.render().insertBefore(this.$(".apply"));

                    return this;
                },
                template: '\
                    <button class="apply btn pull-left" id="apply_<%- cid %>"><%- _("Apply").t() %></button>\
                '
        });
    }
);
