define([
        "jquery",
        "underscore",
        "backbone",
        "module",
        "models/shared/DateInput",
        "models/shared/TimeRange",
        'views/Base',
        "views/shared/controls/DateControl",
        "views/shared/FlashMessages"
      ],
     function($, _, Backbone, module, DateInputModel, TimeRangeModel, Base, DateControl, FlashMessages) {
        return Base.extend({
            // tagName: 'tbody',
            moduleId: module.id,
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);

                this.model = {
                    timeRange: this.model,
                    workingRange: new TimeRangeModel(),
                    latestDateInput: new DateInputModel()
                };
                this.syncInternals();

                this.children.latestTimeInput = new DateControl({
                    model:  this.model.latestDateInput,
                    validate: true
                });

                this.children.flashMessages = new FlashMessages({
                    model: {
                        workingRange: this.model.workingRange,
                        latestDateInput: this.model.latestDateInput
                    }
                });

                this.activate();
            },
            startListening: function() {
                this.listenTo(this.model.timeRange, 'change:latest_date prepopulate', _.debounce(this.prepopulate, 0));

                this.listenTo(this.model.latestDateInput, "validated:invalid", this.disableApply);
                this.listenTo(this.model.latestDateInput, "validated:valid", this.enableApply);

                this.listenTo(this.model.latestDateInput, "change", function(){
                    this.model.workingRange.trigger("validated", true, this.model.workingRange, {});
                    this.model.workingRange.trigger("serverValidated", true, this.model.workingRange, []);
                });
            },
            syncInternals: function() {
                this.today = new Date();
                this.today.setHours(0, 0, 0, 0);
                this.model.latestDateInput.setFromJSDate(this.today, {silent:true});
            },
            activate: function(options) {
                if (this.active) {
                    return Base.prototype.activate.apply(this, arguments);
                }
                this.syncInternals();
                return Base.prototype.activate.apply(this, arguments);
            },
            deactivate: function(options) {
                if (!this.active) {
                    return Base.prototype.deactivate.apply(this, arguments);
                }
                Base.prototype.deactivate.apply(this, arguments);

                this.model.workingRange.fetchAbort();
                this.model.workingRange.clear({setDefaults: true});
                this.model.latestDateInput.clear({setDefaults: true});

                return this;
            },
            prepopulate: function() {
                var latest = this.model.timeRange.get('latest_date');

                if (latest) {
                    this.model.latestDateInput.setFromJSDate(latest, {validate: true});
                    this.model.latestDateInput.setHoursMinSecFromStr("00:00:00.000", {silent:true});
                }
            },
            events: {
                "click .apply": function(event) {
                    if ($(event.currentTarget).hasClass("disabled")) {
                        event.preventDefault();
                        return;
                    }

                    this.model.latestDateInput.setHoursMinSecFromStr("00:00:00.000", {silent:true, validate:true});

                    var that = this,
                    latestIsoWithoutTZ = this.model.latestDateInput.isoWithoutTZ();

                    this.disableApply();

                    this.model.workingRange.save(
                            {
                                earliest: "",
                                latest: latestIsoWithoutTZ
                            },
                            {
                                success: function(model) {
                                    that.enableApply();
                                    that.model.timeRange.set({
                                        'earliest': "",
                                        'latest': model.get('latest_epoch'),
                                        'earliest_epoch': model.get('earliest_epoch'),
                                        'latest_epoch': model.get('latest_epoch'),
                                        'earliest_iso': model.get('earliest_iso'),
                                        'latest_iso': model.get('latest_iso'),
                                        'earliest_date': new Date(0),
                                        'latest_date': new Date(model.get('latest_date').getTime())
                                    });
                                    that.model.timeRange.trigger("applied");
                                },
                                error: function(model, error) {
                                    that.enableApply();
                                }
                            }
                    );
                    event.preventDefault();
                }
            },
            enableApply: function() {
                this.$("#apply_" + this.cid).removeClass('disabled');
            },
            disableApply: function() {
                this.$("#apply_" + this.cid).addClass('disabled');
            },
            render: function() {
                var template = _.template(this.template, {
                    _: _,
                    cid: this.cid
                });
                this.$el.html(template);
                this.children.flashMessages.render().appendTo(this.$(".flash_messages"));
                this.children.latestTimeInput.render().prependTo(this.$(".latest_time"));
                return this;
            },
            template: '\
                <div class="time-before-date">\
                    <div class="flash_messages"></div>\
                    <div class="latest_time col-1">\
                        <span class="help-block"><%- _("00:00:00").t() %></span>\
                    </div>\
                    <div class="col-2">\
                        <button class="apply btn" id="apply_<%- cid %>"><%- _("Apply").t() %></button>\
                    </div>\
                </div>\
            '
        });
    }
);
