define(
    [
        'jquery',
        'underscore',
        'module',
        'backbone',
        'splunk.util',
        'views/Base',
        'contrib/text!views/shared/tourbar/TourBar.html'
    ],
    function(
        $,
        _,
        module,
        Backbone,
        splunk_util,
        BaseView,
        template
    ) {
    
        return BaseView.extend({
            template: template,
            moduleId: module.id,
            initialize: function(options) {
                BaseView.prototype.initialize.call(this, options);
                this.fixedHeight = this.options.fixedHeight || 0;
                this.resetModel(this.options.model);
            },
            events: {
                "click .close-btn"  : "handleEndTour",
                "click .next"       : "handleNext",
                "click .previous"   : "handlePrevious"
            },
            setFixedHeight: function(height) {
                this.fixedHeight = height;
            },
            resetModel: function(model) {
                this.model = model;
                this.debouncedRender();
            },
            removeHighlight: function() {
                // if an element is currently highlighted, remove that highlight
                if (this._currentHighlightedSelector) {
                    var currentElement = $(this._currentHighlightedSelector);
                    currentElement.removeClass("highlightedElement");
                    this._currentHighlightedSelector = undefined;
                }
                this._highlightSelector = undefined;
            },
            highlightElement: function(selector) {
                if (selector) {
                    this._highlightSelector = selector;
                }
    
                if (this._highlightSelector) { 
                    var element = $(this._highlightSelector);
                    if (element) {
                        element.addClass("highlightedElement");
                        this._currentHighlightedSelector = this._highlightSelector;
                    } 

                    setTimeout(function() {
                        this.highlightElement(); 
                    }.bind(this), 250);
                }
            },
            render: function() {
                var html = undefined,
                    info = undefined,
                    nextLink = undefined,
                    previousLink = undefined;

                this.removeHighlight();

                if (this.model) {

                    info = this.model.entry.content.get("info");
                    var selector = this.model.entry.content.get("highlightSelector");
                    if (selector) {
                        this.highlightElement(selector);
                    }

                    nextLink = this.model.getNextLink();
                    previousLink = this.model.getPreviousLink();
                }
                html = this.compiledTemplate({
                    previousLink: previousLink,
                    nextLink: nextLink,
                    info: info,
                    fixedHeight: this.fixedHeight
                });
                this.$el.html(html);

                return this;
            },
            handleNext: function(event) {
                event.preventDefault(); 
                if (this.model) {
                    this.model.navigateToNext();
                }
            },
            handlePrevious: function(event) {
                event.preventDefault();
                if (this.model) {
                    this.model.navigateToPrevious();
                }
            },
            handleEndTour: function(event) {
                event.preventDefault();
                if (this.model) {
                    this.model.endTour();
                }
            }
        });
    }
);

