define(
    [
        'underscore',
        'jquery',
        'module',
        'views/Base',
        'views/shared/vizcontrols/custom_controls/OverlayFieldsControlGroup',
        'views/shared/vizcontrols/custom_controls/AxisEnabledControlGroup',
        'views/shared/vizcontrols/custom_controls/AxisTitleControlGroup',
        'views/shared/vizcontrols/custom_controls/AxisScaleControlGroup',
        'views/shared/vizcontrols/custom_controls/AxisIntervalControlGroup',
        'views/shared/vizcontrols/custom_controls/AxisMinValueControlGroup',
        'views/shared/vizcontrols/custom_controls/AxisMaxValueControlGroup',
        'splunk.util'
    ],
    function(
        _,
        $,
        module,
        Base,
        OverlayFieldsControlGroup,
        AxisEnabledControlGroup,
        AxisTitleControlGroup,
        AxisScaleControlGroup,
        AxisIntervalControlGroup,
        AxisMinValueControlGroup,
        AxisMaxValueControlGroup,
        splunkUtils
    ){
        return Base.extend({
            moduleId: module.id,
            className: 'form form-horizontal',
            vizToYAxisComponents: {
                line: ['overlayFields', 'enabled', 'title', 'scale', 'interval', 'min', 'max'],
                bar: ['overlayFields', 'enabled', 'title', 'scale', 'interval', 'min', 'max'],
                area: ['overlayFields', 'enabled', 'title', 'scale', 'interval', 'min', 'max'],
                column: ['overlayFields', 'enabled', 'title', 'scale', 'interval', 'min', 'max'],
                scatter: [],
                bubble: [],
                pie: [],
                radialGauge: [],
                fillerGauge: [],
                markerGauge: [],
                single: [],
                events: [],
                statistics: []
            },
            
            /**
             * @constructor
             * @param options {
             *     model: {
             *         visualization: <models.shared.Visualization>
             *         report: <models.search.Report>
             *     }
             * }
             */
            
            initialize: function(options) {
                Base.prototype.initialize.apply(this, arguments);
                var controls = this.vizToYAxisComponents[this.model.visualization.get('viz_type')];
                if(_.indexOf(controls, 'overlayFields')>-1){
                    this.children.overlayFields = new OverlayFieldsControlGroup({
                        model: {
                            visualization: this.model.visualization,
                            report: this.model.report
                        },
                        controlClass: 'controls-block'
                    });
                }
                if(_.indexOf(controls, 'enabled')>-1){
                    this.children.enabled = new AxisEnabledControlGroup({
                        label: _('View as Axis').t(),
                        model: this.model.visualization,
                        controlClass: 'controls-halfblock',
                        axisType: AxisEnabledControlGroup.Y_AXIS_2
                    });
                }
                if(_.indexOf(controls, 'title')>-1){
                    this.children.title = new AxisTitleControlGroup({
                        className: 'y-axis-title control-group',
                        model: this.model.visualization,
                        axisType: AxisTitleControlGroup.Y_AXIS_2
                    });
                }
                if(_.indexOf(controls, 'scale')>-1) {
                    this.children.scale = new AxisScaleControlGroup({
                        model: this.model.visualization,
                        className: 'scale control-group',
                        controlClass: 'controls-thirdblock',
                        supportInherit: true,
                        axisType: AxisScaleControlGroup.Y_AXIS_2
                    });
                }
                if(_.indexOf(controls, 'interval')>-1) {
                    this.children.interval = new  AxisIntervalControlGroup({
                        model: this.model.visualization,
                        controlClass: 'controls-block',
                        axisType: AxisIntervalControlGroup.Y_AXIS_2
                    });
                }
                if(_.indexOf(controls, 'min')>-1) {
                    this.children.min = new AxisMinValueControlGroup({
                        model: this.model.visualization,
                        controlClass: 'controls-block',
                        axisType: AxisMinValueControlGroup.Y_AXIS_2
                    });
                }
                if(_.indexOf(controls, 'max')>-1) {
                    this.children.max = new AxisMaxValueControlGroup({
                        model: this.model.visualization,
                        controlClass: 'controls-block',
                        axisType: AxisMaxValueControlGroup.Y_AXIS_2
                    });
                }
                if(this.model.visualization.get('display.visualizations.charting.axisY2.scale')=='log') {
                    this.children.interval.$el.hide();
                }
                this.model.visualization.on('change:display.visualizations.charting.axisY2.enabled', this.handleSecondAxisEnabled, this);
                this.model.visualization.on('change:display.visualizations.charting.chart.overlayFields', this.handleOverlayFields, this);
            },
            events: {
                'click .scale button': function(e){
                    this.intervalVal = this.intervalVal || this.model.visualization.get('display.visualizations.charting.axisLabelsY2.majorUnit');
                    if(($(e.currentTarget).data('value'))=='log'){
                        this.children.interval.$el.hide();
                        this.model.visualization.set('display.visualizations.charting.axisLabelsY2.majorUnit', '');
                    } else {
                        this.children.interval.$el.show();
                        this.model.visualization.set('display.visualizations.charting.axisLabelsY2.majorUnit', this.intervalVal);
                    }
                }
            },
            render: function() {
                this.children.overlayFields && this.children.overlayFields.render().appendTo(this.$el);
                this.children.enabled && this.children.enabled.render().appendTo(this.$el);
                this.children.title && this.children.title.render().appendTo(this.$el);
                this.children.scale && this.children.scale.render().appendTo(this.$el);
                this.children.interval && this.children.interval.render().appendTo(this.$el);
                this.children.min && this.children.min.render().appendTo(this.$el);
                this.children.max && this.children.max.render().appendTo(this.$el);
                this.handleSecondAxisEnabled();
                this.handleOverlayFields();
                return this;
            },
            handleSecondAxisEnabled: function() {
                var enabled = splunkUtils.normalizeBoolean(this.model.visualization.get('display.visualizations.charting.axisY2.enabled'));
                if(enabled) {
                    this.children.title.enable();
                    this.children.scale.enable();
                    this.children.interval.enable();
                    this.children.min.enable();
                    this.children.max.enable();
                }
                else {
                    this.children.title.disable();
                    this.children.scale.disable();
                    this.children.interval.disable();
                    this.children.min.disable();
                    this.children.max.disable();
                }
            },
            handleOverlayFields: function() {
                if(this.model.visualization.get('display.visualizations.charting.chart.overlayFields')) {
                    this.children.enabled.enable();
                }
                else {
                    this.children.enabled.disable();
                }
            }
        });
    }
);