define([
            'underscore',
            'module',
            'views/Base',
            'views/shared/vizcontrols/custom_controls/MapMarkerOpacityControlGroup',
            'views/shared/vizcontrols/custom_controls/MapMarkerMinSizeControlGroup',
            'views/shared/vizcontrols/custom_controls/MapMarkerMaxSizeControlGroup',
            'views/shared/vizcontrols/custom_controls/MapMarkerMaxClustersControlGroup'
        ],
        function(
            _,
            module,
            Base,
            MapMarkerOpacityControlGroup,
            MapMarkerMinSizeControlGroup,
            MapMarkerMaxSizeControlGroup,
            MapMarkerMaxClustersControlGroup
        ) {

    return Base.extend({

        moduleId: module.id,

        className: 'form form-horizontal',

        vizToGeneralComponents: {
            line: [],
            area: [],
            column: [],
            bar: [],
            pie: [],
            scatter: [],
            bubble: [],
            radialGauge: [],
            fillerGauge: [],
            markerGauge: [],
            single: [],
            mapping: ['opacity', 'minSize', 'maxSize', 'maxClusters']
        },

        initialize: function() {
            Base.prototype.initialize.apply(this, arguments);
            var controls = this.vizToGeneralComponents[this.model.get('viz_type')];
            if(_.contains(controls, 'opacity')) {
                this.children.opacity = new MapMarkerOpacityControlGroup({
                    model: this.model
                });
            }
            if(_.contains(controls, 'minSize')) {
                this.children.minSize = new MapMarkerMinSizeControlGroup({
                    model: this.model
                });
            }
            if(_.contains(controls, 'maxSize')) {
                this.children.maxSize = new MapMarkerMaxSizeControlGroup({
                    model: this.model
                });
            }
            if(_.contains(controls, 'maxClusters')) {
                this.children.maxClusters = new MapMarkerMaxClustersControlGroup({
                    model: this.model
                });
            }
        },

        render: function() {
            this.children.opacity && this.children.opacity.render().appendTo(this.$el);
            this.children.minSize && this.children.minSize.render().appendTo(this.$el);
            this.children.maxSize && this.children.maxSize.render().appendTo(this.$el);
            this.children.maxClusters && this.children.maxClusters.render().appendTo(this.$el);
            return this;
        }

    });

});