define(
    [
        'underscore',
        'jquery',
        'module',
        'views/Base',
        'views/shared/FlashMessages',
        'views/shared/vizcontrols/components/General',
        'views/shared/vizcontrols/components/XAxis',
        'views/shared/vizcontrols/components/YAxis',
        'views/shared/vizcontrols/components/ChartOverlay',
        'views/shared/vizcontrols/components/Legend',
        'views/shared/vizcontrols/components/color/Master',
        'views/shared/vizcontrols/components/Statistics',
        'views/shared/vizcontrols/components/Events',
        'views/shared/vizcontrols/components/Map',
        'views/shared/vizcontrols/components/Markers',
        'views/shared/vizcontrols/components/Tiles',
        'views/shared/vizcontrols/components/Bubble'

    ],
    function(_, $, module, Base, FlashMessages, General, XAxis, YAxis, ChartOverlay, Legend, Ranges, Statistics, Events, Map, Markers, Tiles, Bubble){
        return Base.extend({
            
            moduleId: module.id,
            
            className: 'tabbable ',
            
            events: {
                'click a[data-toggle]': function(e) {
                    var $target = $(e.currentTarget),
                        type = $target.data().type;
                    
                    _(this.children).each(function(child) { 
                        child.$el.hide(); 
                    },this);
                    this.children[type].$el.show(); 
                    this.$el.find('.nav > li').removeClass('active');
                    $target.parent().addClass('active');
                    
                    e.preventDefault();
                }
            },

            /**
             * @constructor
             * @param options {
             *     model: {
             *         visualization: <models.shared.Visualization>
             *         report: <models.search.Report>
             *     }
             * }
             */
            
            initialize: function(options) {
                Base.prototype.initialize.apply(this, arguments);
                this.children.flashMessages = new FlashMessages({ model: this.model.visualization });
                this.vizType = this.model.visualization.get('viz_type');
                if(this.hasTabType('gen')) {
                    this.children.general = new General({ model: this.model.visualization });
                }
                else if(this.hasTabType('statistics')) {
                    this.children.general = new Statistics({ model: this.model.visualization });
                }
                else if(this.hasTabType('events')) {
                    this.children.general = new Events({ model: this.model.visualization });
                }

                if(this.hasTabType('x')) {
                    this.children.xaxis = new XAxis({ model: this.model.visualization });
                }
                if(this.hasTabType('y')) {
                    this.children.yaxis = new YAxis({ model: this.model.visualization });
                }
                if(this.hasTabType('overlay')) {
                    this.children.overlay = new ChartOverlay({
                        model: {
                            visualization: this.model.visualization,
                            report: this.model.report
                        }
                    });
                }
                if(this.hasTabType('leg')) {
                    this.children.legend = new Legend({ model: this.model.visualization });
                }
                if(this.hasTabType('ranges')) {
                    this.children.ranges = new Ranges({ model: this.model.visualization });
                }
                if(this.hasTabType('map')) {
                    this.children.map = new Map({
                        model: {
                            visualization: this.model.visualization,
                            report: this.model.report
                        }
                    });
                }
                if(this.hasTabType('markers')) {
                    this.children.markers = new Markers({ model: this.model.visualization });
                }
                if(this.hasTabType('tiles')) {
                    this.children.tiles = new Tiles({ model: this.model.visualization });
                }
                if(this.hasTabType('mark')) {
                    this.children.mark = new Bubble({ model: this.model.visualization });
                }
            },
            
            render: function() {
                this.$el.html(_(this.template).template({
                    hasTabType: _(this.hasTabType).bind(this)
                }));
                this.children.flashMessages.render().prependTo(this.$('.tab-content'));
                _(this.children).each(function(child) { 
                    this.$('.tab-content').append(child.render().el);
                    child.$el.hide();
                },this);
                this.children.general.$el.show();
                return this;
            },
            
            hasTabType: function(tabType) {
                return _.contains(this.model.visualization.components[this.vizType], tabType);
            },
            
            template: '\
                <ul class="nav nav-tabs-left">\
                    <% if(hasTabType("gen") || hasTabType("statistics") || hasTabType("events")) { %>\
                        <li class="active">\
                            <a href="#" data-toggle="tab" data-type="general"><%- _("General").t() %></a>\
                        </li>\
                    <% } %>\
                    <% if(hasTabType("x")) { %>\
                        <li>\
                            <a href="#" data-toggle="tab" data-type="xaxis"><%- _("X-Axis").t() %></a>\
                        </li>\
                    <% } %>\
                    <% if(hasTabType("y")) { %>\
                        <li>\
                            <a href="#" data-toggle="tab" data-type="yaxis"><%- _("Y-Axis").t() %></a>\
                        </li>\
                    <% } %>\
                    <% if(hasTabType("overlay")) { %>\
                        <li>\
                            <a href="#" data-toggle="tab" data-type="overlay"><%- _("Chart Overlay").t() %></a>\
                        </li>\
                    <% } %>\
                    <% if(hasTabType("leg")) { %>\
                        <li>\
                            <a href="#" data-toggle="tab" data-type="legend"><%- _("Legend").t() %></a>\
                        </li>\
                    <% } %>\
                    <% if(hasTabType("ranges")) { %>\
                        <li>\
                            <a href="#" data-toggle="tab" data-type="ranges"><%- _("Color Ranges").t() %></a>\
                        </li>\
                    <% } %>\
                    <% if(hasTabType("map")) { %>\
                        <li>\
                            <a href="#" data-toggle="tab" data-type="map"><%- _("Map").t() %></a>\
                        </li>\
                    <% } %>\
                    <% if(hasTabType("markers")) { %>\
                        <li>\
                            <a href="#" data-toggle="tab" data-type="markers"><%- _("Markers").t() %></a>\
                        </li>\
                    <% } %>\
                    <% if(hasTabType("tiles")) { %>\
                        <li>\
                            <a href="#" data-toggle="tab" data-type="tiles"><%- _("Tiles").t() %></a>\
                        </li>\
                    <% } %>\
                    <% if(hasTabType("mark")) { %>\
                        <li>\
                            <a href="#" data-toggle="tab" data-type="mark"><%- _("Mark").t() %></a>\
                        </li>\
                    <% } %>\
                </ul>\
                <div class="tab-content"></div>\
            '
        });
    }
);
