define(
    [
        'underscore',
        'jquery',
        'module',
        'views/Base',
        'views/shared/vizcontrols/custom_controls/AxisTitleControlGroup',
        'views/shared/vizcontrols/custom_controls/AxisScaleControlGroup',
        'views/shared/vizcontrols/custom_controls/AxisIntervalControlGroup',
        'views/shared/vizcontrols/custom_controls/AxisMinValueControlGroup',
        'views/shared/vizcontrols/custom_controls/AxisMaxValueControlGroup', 
        'views/shared/vizcontrols/custom_controls/AxisLabelRotationControlGroup', 
        'views/shared/vizcontrols/custom_controls/AxisLabelElisionControlGroup'
    ],
    function(
        _, 
        $,
        module, 
        Base, 
        AxisTitleControlGroup,
        AxisScaleControlGroup,
        AxisIntervalControlGroup,
        AxisMinValueControlGroup,
        AxisMaxValueControlGroup, 
        AxisLabelRotationControlGroup, 
        AxisLabelElisionControlGroup
    ){
        return Base.extend({
            moduleId: module.id,
            className: 'form form-horizontal',
            vizToGeneralComponents: {
                line: ['title', 'elision', 'rotation'],
                area: ['title', 'elision', 'rotation'],
                column: ['title', 'elision', 'rotation'],
                bar:['title'],
                pie: [],
                scatter: ['title', 'elision', 'rotation', 'scale', 'interval', 'min', 'max'],
                bubble: ['title', 'elision', 'rotation', 'scale', 'interval', 'min', 'max'],
                radialGauge: [],
                fillerGauge: [],
                markerGauge: [],
                single: [],
                events: [],
                statistics: [] 
            },
            initialize: function(options) {
                Base.prototype.initialize.apply(this, arguments);
                var controls = this.vizToGeneralComponents[this.model.get('viz_type')];
                
                if (_.indexOf(controls, 'title')>-1) { 
                    this.children.title = new AxisTitleControlGroup({
                        model: this.model,
                        axisType: AxisTitleControlGroup.X_AXIS
                    });
                }
                if (_.indexOf(controls, 'elision')>-1) {
                    this.children.elision = new AxisLabelElisionControlGroup({
                        model: this.model,
                        axisType: AxisLabelElisionControlGroup.X_AXIS
                    });
                }
                if (_.indexOf(controls, 'rotation')>-1) {
                    this.children.rotation = new AxisLabelRotationControlGroup({
                        model: this.model,
                        axisType: AxisLabelRotationControlGroup.X_AXIS
                    });
                }
                if(_.indexOf(controls, 'scale')>-1) {
                    this.children.scale = new AxisScaleControlGroup({
                        model: this.model,
                        className: 'scale control-group',
                        controlClass: 'controls-halfblock',
                        axisType: AxisScaleControlGroup.X_AXIS
                    });
                }
                if(_.indexOf(controls, 'interval')>-1) {
                    this.children.interval = new AxisIntervalControlGroup({
                        model: this.model,
                        controlClass: 'controls-block',
                        axisType: AxisIntervalControlGroup.X_AXIS
                    });
                }    
                if(_.indexOf(controls, 'min')>-1) {
                    this.children.min = new AxisMinValueControlGroup({
                        model: this.model,
                        controlClass: 'controls-block',
                        axisType: AxisMinValueControlGroup.X_AXIS
                    });
                }
                if(_.indexOf(controls, 'max')>-1) {
                    this.children.max = new AxisMaxValueControlGroup({
                        model: this.model,
                        controlClass: 'controls-block',
                        axisType: AxisMaxValueControlGroup.X_AXIS
                    });
                }
                if(this.model.get('display.visualizations.charting.axisX.scale')=='log') {
                    if(this.children.interval) {
                        this.children.interval.$el.hide();
                    }
                }
                if(this.children.elision) {
                    this.model.on('change:display.visualizations.charting.axisLabelsX.majorLabelStyle.rotation', this.handleRotation, this);
                }
            },
            events: {
                'click .scale button': function(e){
                    this.intervalVal = this.intervalVal || this.model.get('display.visualizations.charting.axisLabelsX.majorUnit');
                    if(($(e.currentTarget).data('value'))=='log'){
                        if(this.children.interval) {
                            this.children.interval.$el.hide();
                        }
                        this.model.set('display.visualizations.charting.axisLabelsX.majorUnit', '');
                    } else {
                        if(this.children.interval) {
                            this.children.interval.$el.show();
                        }
                        this.model.set('display.visualizations.charting.axisLabelsX.majorUnit', this.intervalVal);
                    }
                }
            },
            render: function() {
                this.children.title && this.children.title.render().appendTo(this.$el);
                this.children.elision && this.children.elision.render().appendTo(this.$el);
                this.children.rotation && this.children.rotation.render().appendTo(this.$el);
                this.children.scale && this.children.scale.render().appendTo(this.$el);
                this.children.interval && this.children.interval.render().appendTo(this.$el);
                this.children.min && this.children.min.render().appendTo(this.$el);
                this.children.max && this.children.max.render().appendTo(this.$el);
                if(this.children.elision) {
                    this.handleRotation();
                }
                return this;
            }, 
            handleRotation: function() {
                if(this.model.get('display.visualizations.charting.axisLabelsX.majorLabelStyle.rotation') === '0'){
                    if(this.children.elision) {
                        this.children.elision.disable();
                    }
                }else{
                    if(this.children.elision) {
                        this.children.elision.enable();
                    }
                }
            }
        });
    }
);
